Package: LaplacesDemon
Version: 16.1.8
Title: Complete Environment for Bayesian Inference
Authors@R: c(person("Byron", "Hall", role = "aut"),
    person("Martina", "Hall", role = "aut"),
    person(family="Statisticat, LLC", role = c("aut", "cph")),
    person(given = "Charles J.", family="Geyer", role = c("ctb", "cph"),
    comment="for TR method in LaplaceApproximation, derived from trust::trust"),
    person(given="Eric", family="Brown", role = "ctb"),
    person(given="Richard", family="Hermanson", role = "ctb"),
    person(given="Emmanuel", family="Charpentier", role = "ctb"),
    person(given="Daniel", family="Heck", role = "ctb"),
    person(given="Stephane", family="Laurent", role = "ctb"),
    person(given="Quentin F.", family="Gronau", role = "ctb"),
    person(given="Henrik", family="Singmann", 
    email="singmann+LaplacesDemon@gmail.com", role="cre"))
Depends: R (>= 3.0.0)
Imports: parallel, grDevices, graphics, stats, utils
Suggests: KernSmooth
ByteCompile: TRUE
Description: Provides a complete environment for Bayesian inference using a variety of different samplers (see ?LaplacesDemon for an overview).
License: MIT + file LICENSE
URL: https://github.com/LaplacesDemonR/LaplacesDemon
BugReports: https://github.com/LaplacesDemonR/LaplacesDemon/issues
NeedsCompilation: no
Packaged: 2026-02-17 12:00:50 UTC; singmann
Author: Byron Hall [aut],
  Martina Hall [aut],
  Statisticat, LLC [aut, cph],
  Charles J. Geyer [ctb, cph] (for TR method in LaplaceApproximation,
    derived from trust::trust),
  Eric Brown [ctb],
  Richard Hermanson [ctb],
  Emmanuel Charpentier [ctb],
  Daniel Heck [ctb],
  Stephane Laurent [ctb],
  Quentin F. Gronau [ctb],
  Henrik Singmann [cre]
Maintainer: Henrik Singmann <singmann+LaplacesDemon@gmail.com>
Repository: CRAN
Date/Publication: 2026-02-17 15:20:07 UTC

import(parallel)
export(ABB,
       AcceptanceRate,
       as.covar,
       as.indicator.matrix,
       as.initial.values,
       as.inverse,
       as.parm.matrix,
       as.parm.names,
       as.positive.definite,
       as.positive.semidefinite,
       as.ppc,
       as.symmetric.matrix,
       BayesFactor,
       BayesianBootstrap,
       BayesTheorem,
       BigData,
       Blocks,
       BMK.Diagnostic,
       burnin,
       caterpillar.plot,
       CenterScale,
       cloglog,
       Combine,
       cond.plot,
       Consort,
       Cov2Cor,
       Cov2Prec,
       CovEstim,
       CSF,
       dalaplace,
       dallaplace,
       daml,
       dbern,
       dcat,
       dcrmrf,
       ddirichlet,
       deburn,
       delicit,
       de.Finetti.Game,
       dgpd,
       dgpois,
       dhalfcauchy,
       dhalfnorm,
       dhalft,
       dhs,
       dhuangwand,
       dhuangwandc,
       dhyperg,
       dinvbeta,
       dinvchisq,
       dinvgamma,
       dinvgaussian,
       dinvmatrixgamma,
       dinvwishart,
       dinvwishartc,
       dlaplace,
       dlaplacem,
       dlaplacep,
       dlasso,
       dllaplace,
       dlnormp,
       dmatrixgamma,
       dmatrixnorm,
       dmvc,
       dmvcc,
       dmvcp,
       dmvcpc,
       dmvl,
       dmvlc,
       dmvn,
       dmvnc,
       dmvnp,
       dmvnpc,
       dmvpe,
       dmvpec,
       dmvpolya,
       dmvt,
       dmvtc,
       dmvtp,
       dmvtpc,
       dnorminvwishart,
       dnormlaplace,
       dnormm,
       dnormp,
       dnormv,
       dnormwishart,
       dpareto,
       dpe,
       dsdlaplace,
       dsiw,
       dslaplace,
       dStick,
       dst,
       dstp,
       dtrunc,
       dwishart,
       dwishartc,
       dyangberger,
       dyangbergerc,
       dzellner,
       ESS,
       elicit,
       extrunc,
       GaussHermiteCubeRule,
       GaussHermiteQuadRule,
       Gelfand.Diagnostic,
       Gelman.Diagnostic,
       Geweke.Diagnostic,
       GIV,
       Hangartner.Diagnostic,
       Heidelberger.Diagnostic,
       Hermite,
       Hessian,
       IAT,
       Importance,
       interval,
       invcloglog,
       invlogit,
       invloglog,
       is.amodal,
       is.appeased,
       is.bayesfactor,
       is.bayesian,
       is.bimodal,
       is.blocks,
       is.bmk,
       is.constant,
       is.constrained,
       is.data,
       is.demonoid,
       is.demonoid.hpc,
       is.demonoid.ppc,
       is.demonoid.val,
       is.hangartner,
       is.heidelberger,
       is.importance,
       is.iterquad,
       is.iterquad.ppc,
       is.juxtapose,
       is.laplace,
       is.laplace.ppc,
       is.miss,
       is.model,
       is.multimodal,
       is.pmc,
       is.pmc.ppc,
       is.pmc.val,
       is.positive.definite,
       is.positive.semidefinite,
       is.posteriorchecks,
       is.proper,
       is.raftery,
       is.rejection,
       is.sensitivity,
       is.square.matrix,
       is.stationary,
       is.symmetric.matrix,
       is.trimodal,
       is.unimodal,
       is.vb,
       is.vb.ppc,
       IterativeQuadrature,
       Jacobian,
       joint.density.plot,
       joint.pr.plot,
       Juxtapose,
       KLD,
       KS.Diagnostic,
       LaplaceApproximation,
       LaplacesDemon,
       LaplacesDemon.hpc,
       LaplacesDemon.RAM,
       Levene.Test,
       LML,
       logadd,
       logdet,
       logit,
       loglog,
       LossMatrix,
       lower.triangle,
       LPL.interval,
       MCSE,
       MCSS,
       MinnesotaPrior,
       MISS,
       Mode,
       Model.Spec.Time,
       Modes,
       PMC,
       PMC.RAM,
       PosteriorChecks,
       Prec2Cov,
       palaplace,
       pallaplace,
       partial,
       ppareto,
       pbern,
       phalfcauchy,
       phalfnorm,
       phalft,
       plaplace,
       plaplacem,
       plaplacep,
       pllaplace,
       plotMatrix,
       plotSamples,
       plnormp,
       pnormm,
       pnormp,
       pnormv,
       ppe,
       prec2sd,
       prec2var,
       psdlaplace,
       pslaplace,
       pst,
       pstp,
       ptrunc,
       p.interval,
       qalaplace,
       qallaplace,
       qbern,
       qcat,
       qhalfcauchy,
       qhalfnorm,
       qhalft,
       qlaplace,
       qlaplacep,
       qllaplace,
       qlnormp,
       qnormp,
       qnormv,
       qpareto,
       qpe,
       qsdlaplace,
       qslaplace,
       qst,
       qstp,
       qtrunc,
       Raftery.Diagnostic,
       ralaplace,
       rallaplace,
       raml,
       rbern,
       rcat,
       rcrmrf,
       rdirichlet,
       read.matrix,
       RejectionSampling,
       rgpd,
       rhalfcauchy,
       rhalfnorm,
       rhalft,
       rhs,
       rhuangwand,
       rhuangwandc,
       rinvbeta,
       rinvchisq,
       rinvgamma,
       rinvgaussian,
       rinvwishart,
       rinvwishartc,
       rlaplace,
       rlaplacem,
       rlaplacep,
       rlasso,
       rllaplace,
       rlnormp,
       rmatrixnorm,
       rmvc,
       rmvcc,
       rmvcp,
       rmvcpc,
       rmvl,
       rmvlc,
       rmvn,
       rmvnc,
       rmvnp,
       rmvnpc,
       rmvpe,
       rmvpec,
       rmvpolya,
       rmvt,
       rmvtc,
       rmvtp,
       rmvtpc,
       rnorminvwishart,
       rnormlaplace,
       rnormm,
       rnormp,
       rnormv,
       rnormwishart,
       rpareto,
       rpe,
       rsdlaplace,
       rsiw,
       rslaplace,
       rst,
       rStick,
       rstp,
       rtrunc,
       rwishart,
       rwishartc,
       rzellner,
       sd2prec,
       sd2var,
       SensitivityAnalysis,
       server_Listening,
       SIR,
       SparseGrid,
       Stick,
       Thin,
       TransitionMatrix,
       tr,
       upper.triangle,
       Validate,
       VariationalBayes,
       vartrunc,
       var2prec,
       var2sd,
       WAIC,
       .colVars,
       .iqagh,
       .iqaghsg,
       .iqcagh,
       .laaga,
       .labfgs,
       .labhhh,
       .lacg,
       .ladfp,
       .lahar,
       .lahj,
       .lalbfgs,
       .lalm,
       .lanm,
       .lanr,
       .lapso,
       .larprop,
       .lasgd,
       .lasoma,
       .laspg,
       .lasr1,
       .latr,
       .mcmcadmg,
       .mcmcafss,
       .mcmcagg,
       .mcmcahmc,
       .mcmcaies,
       .mcmcam,
       .mcmcamm,
       .mcmcamm.b,
       .mcmcamwg,
       .mcmccharm,
       .mcmcdemc,
       .mcmcdram,
       .mcmcdrm,
       .mcmcess,
       .mcmcgibbs,
       .mcmcgg,
       .mcmcggcp,
       .mcmcggcpp,
       .mcmcggdp,
       .mcmcggdpp,
       .mcmcharm,
       .mcmchmc,
       .mcmchmcda,
       .mcmcim,
       .mcmcinca,
       .mcmcmala,
       .mcmcmcmcmc,
       .mcmcmtm,
       .mcmcmwg,
       .mcmcnuts,
       .mcmcohss,
       .mcmcram,
       .mcmcrdmh,
       .mcmcrefractive,
       .mcmcrj,
       .mcmcrss,
       .mcmcrwm,
       .mcmcsamwg,
       .mcmcsgld,
       .mcmcslice,
       .mcmcsmwg,
       .mcmcthmc,
       .mcmctwalk,
       .mcmcuess,
       .mcmcusamwg,
       .mcmcusmwg,
       .rowVars,
       .vbsalimans2)

S3method("plot","bmk")
S3method("plot","demonoid")
S3method("plot","demonoid.hpc")
S3method("plot","demonoid.ppc")
S3method("plot","importance")
S3method("plot","iterquad")
S3method("plot","iterquad.ppc")
S3method("plot","juxtapose")
S3method("plot","laplace")
S3method("plot","laplace.ppc")
S3method("plot","miss")
S3method("plot","pmc")
S3method("plot","pmc.ppc")
S3method("plot","vb")
S3method("plot","vb.ppc")
S3method("predict","demonoid")
S3method("predict","iterquad")
S3method("predict","laplace")
S3method("predict","pmc")
S3method("predict","vb")
S3method("print","demonoid")
S3method("print","heidelberger")
S3method("print","iterquad")
S3method("print","laplace")
S3method("print","miss")
S3method("print","pmc")
S3method("print","raftery")
S3method("print","vb")
S3method("summary","demonoid.ppc")
S3method("summary","iterquad.ppc")
S3method("summary","laplace.ppc")
S3method("summary","miss")
S3method("summary","pmc.ppc")
S3method("summary","vb.ppc")
importFrom("grDevices", "col2rgb", "colorRampPalette", "dev.off",
             "heat.colors", "palette", "pdf", "rgb")
  importFrom("graphics", "abline", "axis", "barplot", "boxplot",
             "dotchart", "hist", "image", "layout", "legend", "lines",
             "panel.smooth", "par", "plot", "plot.new", "plot.window",
             "points", "polygon", "rect", "segments", "smoothScatter",
             "strwidth", "symbols", "text")
  importFrom("stats", "ARMAacf", "Gamma", "acf", "aggregate", "approx",
             "ar", "ar.yw", "as.dist", "chisq.test", "complete.cases",
             "cor", "cov", "cov.wt", "cutree", "dbeta", "dbinom",
             "density", "dexp", "dgamma", "dist", "dnorm", "ecdf", "fft",
             "glm", "hclust", "integrate", "ks.test", "lm", "lowess",
             "median", "optim", "optimize", "pbinom", "pgamma", "pnorm",
             "predict", "pt", "qbinom", "qf", "qgamma", "qnorm", "qt",
             "quantile", "rbeta", "rbinom", "rchisq", "residuals",
             "rexp", "rgamma", "rmultinom", "rnorm", "rt", "runif", "sd",
             "spline", "start", "ts", "uniroot", "var", "rlnorm")
  importFrom("utils", "capture.output", "count.fields", "head",
             "object.size", "tail")

LaplacesDemon was initially developed and uploaded to CRAN by Byron Hall, the owner of Statisticat, LLC. Later on, the maintainer of the package changed to Martina Hall.

The last version available on CRAN from the original authors and maintainers was version 13.03.04, which was removed from CRAN on 2013-07-16 at the request of the maintainer. 

After removal from CRAN, the development of LaplacesDemon continued for some time on GitHub under the name of Statisticat LLC (presumably still run by Byron Hall). The last commit by Statisticat for LaplacesDemon on GitHub was performed on 25. Mar 2015. After that Statisticat deleted their account on GitHub and ceased further development of the package. 

As Statisticat could not be reached, neither by e-mail nor by snail-mail (the latter was attempted by Rasmus Bååth), Henrik Singmann took over as maintainer of LaplacesDemon in July 2016 with the goal to resubmit the package to CRAN (as version 16.0.x). Henrik Singmann does not actively continue the development of LaplacesDemon but only retains it on CRAN in its current state.

Note that in order to resubmit the package to CRAN all links to the now defunct website of Statisticat (formerly: http://www.bayesian-inference.com) were replaced with links to versions of this website on the web archive (https://web.archive.org/web/20141224051720/http://www.bayesian-inference.com/index).

To contribute to the development of LaplacesDemon or discuss the development please visit its new repository: https://github.com/LaplacesDemonR/LaplacesDemon

###########################################################################
# plot.bmk                                                                #
#                                                                         #
# The purpose of the plot.bmk function is to plot an object of class bmk. #
###########################################################################

plot.bmk <- function(x, col=colorRampPalette(c("black","red"))(100),
     title="", PDF=FALSE, Parms=NULL, ...)
     {
     ### Initial Checks
     if(missing(x)) stop("x is a required argument.")
     if(!identical(class(x), "bmk"))
          stop("x must be an object of class bmk.")
     ### Selecting Parms
     if(!is.null(Parms)) {
          Parms <- sub("\\[","\\\\[",Parms)
          Parms <- sub("\\]","\\\\]",Parms)
          Parms <- sub("\\.","\\\\.",Parms)
          if(length(grep(Parms[1], rownames(x))) == 0)
               stop("Parameter in Parms does not exist.")
          keeprows <- grep(Parms[1], rownames(x))
          if(length(Parms) > 1) {
               for (i in 2:length(Parms)) {
                    if(length(grep(Parms[i], rownames(x))) == 0)
                         stop("Parameter in Parms does not exist.")
                    keeprows <- c(keeprows,
                         grep(Parms[i], rownames(x)))}}
          x.temp <- as.matrix(x[keeprows,])
          rownames(x.temp) <- rownames(x)[keeprows]
          x <- x.temp
          rm(x.temp)}
     ### Initial Settings
     min <- 0
     max <- 1
     if(is.null(rownames(x))) xLabels <- 1:nrow(x)
     else xLabels <- colnames(x)
     if(is.null(colnames(x))) yLabels <- 1:ncol(x)
     else yLabels <- rownames(x)
     ### plot.bmk
     if(PDF == TRUE) pdf("plot.bmk.pdf")
     ### Layout and Colors
     layout(matrix(data=c(1,2), nrow=1, ncol=2), widths=c(4,1),
          heights=c(1,1))
     ColorRamp <- col
     ColorLevels <- seq(min, max, length=length(ColorRamp))
     ### Reverse y-axis
     reverse <- nrow(x):1
     yLabels <- yLabels[reverse]
     x <- x[reverse,]
     ### Data Map
     par(mar = c(3,5,2.5,2))
     image(1:length(xLabels), 1:length(yLabels), t(x), col=ColorRamp,
          xlab="", ylab="", axes=FALSE, zlim=c(min,max))
     if(!is.null(title)) title(main=title)
     axis(BELOW<-1, at=1:length(xLabels), labels=xLabels, cex.axis=0.7)
     axis(LEFT <-2, at=1:length(yLabels), labels=yLabels,
          las=HORIZONTAL<-1, cex.axis=0.7)
     par(mar=c(3,2.5,2.5,2))
     image(1, ColorLevels,
          matrix(data=ColorLevels, ncol=length(ColorLevels), nrow=1),
          col=ColorRamp, xlab="", ylab="", xaxt="n")
     layout(1)
     if(PDF == TRUE) dev.off()
     }

#End

###########################################################################
# Model.Spec.Time                                                         #
#                                                                         #
# The purpose of the Model.Spec.Time function is to return three things:  #
# the amount of time in minutes that it took to evaluate a model          #
# specification a number of times, the evaluations per minute, and the    #
# componentwise iterations per minute.                                    #
###########################################################################

Model.Spec.Time <- function(Model, Initial.Values, Data, n=1000)
     {
     if(missing(Model)) stop("The Model argument is required.")
     if(missing(Initial.Values))
          stop("The Initial.Values argument is required.")
     if(missing(Data)) stop("The Data argument is required.")
     t <- as.vector(system.time(for (i in 1:n) {Model(Initial.Values, Data)})[3])
     out <- list(Time=round(t/60,3),
          Evals.per.Minute=round(n/(t/60),3),
          Componentwise.Iters.per.Minute=round(n/(t/60)/length(Initial.Values),3))
     return(out)
     }

#End

###########################################################################
# LML (Logarithm of the Marginal Likelihood)                              #
#                                                                         #
# This function provides a few different methods of approximating the     #
# logarithm of the marginal likelihood (LML), and usually also            #
# approximates the Hessian matrix (returning a variance-covariance        #
# matrix, its negative inverse).                                          #
###########################################################################

LML <- function(Model=NULL, Data=NULL, Modes=NULL, theta=NULL, LL=NULL,
     Covar=NULL, method="NSIS")
     {
     LML.out <- list(LML=NA, VarCov=NA)
     if(!is.null(Modes) & !is.null(theta) & !is.null(LL) &
          !is.null(Covar) & method == "GD") {
          log.g.theta <- dmvn(theta, Modes, Covar, log=TRUE)
          LML <- log(1 / mean(exp(log.g.theta - LL)))
          ### Output
          LML.out <- list(LML=LML, VarCov=NA)
          }
     if(!is.null(LL) & method == "HME") {
          med <- median(LL)
          LML <- med - log(mean(exp(-LL + med)))
          ### Output
          LML.out <- list(LML=LML, VarCov=NA)
          }
     if(!is.null(Model) & !is.null(Data) & !is.null(Modes) &
          (method == "LME")) {
          Interval <- 1.0e-6
          parm.len <- length(Modes)
          if(is.null(Covar)) {
               eps <- Interval * Modes
               Approx.Hessian <- Hessian(Model, Modes, Data)
               VarCov <- try(-as.inverse(Approx.Hessian), silent=TRUE)
               if(!inherits(VarCov, "try-error"))
                    diag(VarCov)[which(diag(VarCov) <= 0)] <- .Machine$double.eps
               else {
                    cat("\nWARNING: Failure to solve matrix inversion of ",
                         "Approx. Hessian in LML.\n", sep="")
                    cat("NOTE: Identity matrix is supplied instead.\n")
                    VarCov <- diag(parm.len)}
               }
          else {
               VarCov <- Covar
               rm(Covar)}
          ### Logarithm of the Marginal Likelihood
          LML <- NA
          options(warn=-1)
          LML.test <- try(parm.len/2 * log(2*pi) + 0.5*logdet(VarCov) +
               as.vector(Model(Modes, Data)[["LP"]]), silent=TRUE)
          if(!inherits(LML.test, "try-error")) LML <- LML.test[1]
          options(warn=0)
          ### Output
          LML.out <- list(LML=LML, VarCov=VarCov)
          }
     if(!is.null(theta) & !is.null(LL) & (method =="NSIS")) {
          if(!is.matrix(theta)) stop("theta must be a matrix.")
          thetacol <- ncol(theta)
          LL <- as.vector(LL)
          LLlen <- length(LL)
          if(nrow(theta) != LLlen)
               stop("The number of rows in theta differs from the ",
                    "length of LL.")
          if(LLlen < 301) {
               cat("\nWARNING: At least 301 samples are required for NSIS in LML.\n")
               return(list(LML=NA, VarCov=NA))}
          if(thetacol > round(LLlen / 2)) {
               cat("\nWARNING: The number of parameters,", thetacol,
                    ",\n exceeds half the number of stationary samples,",
                    round(LLlen / 2), ",\n required for NSIS.\n")
               return(list(LML=NA, VarCov=NA))}
          cov.prob <- 0.5
          bounds <- matrix(c(-Inf, Inf), 2, ncol(theta))
          .GetID <- function(point, center, width)
               {return(paste(floor((point - center) / width),
                    collapse=" "))}
          .PopulateHist <- function(points, heights, width)
               {
               max.point <- points[heights == max(heights), ,
                    drop=FALSE][1, ]
               center <- max.point - width / 2
               hist.env <- new.env(hash=TRUE, parent=emptyenv())
               for (i in seq(along=heights)) {
                    id <- .GetID(points[i, ], center, width)
                    if(exists(id, envir=hist.env))
                         cur.tuple <- get(id, envir=hist.env)
                    else cur.tuple <- c(Inf, 0)
                    assign(id, c(min(heights[i], cur.tuple[1]),
                         cur.tuple[2] + 1), envir=hist.env)
                    }
               return(list(env=hist.env, center=center, width=width,
                    dim=length(center)))
               }
          .Profile <- function(hist)
               {
               ids <- ls(hist$env)
               counts <- sapply(ids, function(id) get(id,
                    envir=hist$env)[2])
               names(counts) <- ids
               return(sort(counts, decreasing=TRUE))
               }
          .Lookup <- function(point, hist)
               {
               id <- .GetID(point, hist$center, hist$width)
               return(ifelse(exists(id, envir=hist$env),
                    get(id, envir=hist$env)[1], -Inf))
               }
          .SetHistNorm <- function(hist)
               {
               ids <- ls(hist$env)
               heights <- sapply(ids, function(id) get(id,
                    envir=hist$env)[1])
               max.height <- max(heights)
               hist$norm <- (log(hist$width ^ hist$dim *
                    sum(exp(heights - max.height))) + max.height)
               return(hist)
               }
          .Coverage <- function(points, hist)
               {
               heights <- apply(points, 1, function(point) .Lookup(point,
                    hist))
               return(length(heights[heights > -Inf]) / length(heights))
               }
          .Dist <- function(p1, p2)
               {
               return(max(abs(p1 - p2)))
               }
          .GetWidth <- function(theta.hist, theta.width, LL.hist,
               opt.prob=0.5)
               {
               low.point <- theta.hist[which(LL.hist == min(LL.hist))[1], ,
                    drop=FALSE]
               high.point <- theta.hist[which(LL.hist == max(LL.hist))[1], ,
                    drop=FALSE]
               minLL.dists <- apply(theta.hist, 1, function(theta) {
                    .Dist(theta, low.point)})
               maxLL.dists <- apply(theta.hist, 1, function(theta) {
                    .Dist(theta, high.point)})
               small.dist <- 0.5 * min(maxLL.dists[maxLL.dists > 0])
               big.dist <- 2 * max(minLL.dists)
               F <- function(width) {
                    hist <- .PopulateHist(theta.hist, LL.hist, width)
                    return(.Coverage(theta.width, hist) - opt.prob)
                    }
               options(warn=-1)
               solve.test <- try(uniroot(F, lower=small.dist,
                    upper=big.dist, tol=min(.05, 2/nrow(theta.width))),
                    silent=TRUE)
               options(warn=0)
               if(!inherits(solve.test, "try-error")) return(solve.test$root)
               else return(1)
               }
          .GetLimits <- function(hist)
               {
               split.ids <- strsplit(ls(hist$env), " ")
               coords <- matrix(NA, ncol=hist$dim, nrow=length(split.ids))
               for (i in seq(along=split.ids))
                    coords[i, ] <- as.integer(split.ids[[i]])
               max.coords <- apply(coords, 2, max)
               min.coords <- apply(coords, 2, min)
               result <- rbind(apply(coords, 2, min) * hist$width +
                    hist$center, (apply(coords, 2, max) + 1) * hist$width +
                    hist$center)
               rownames(result) <- c("min", "max")
               return(result)
               }
          .MargLL <- function(hist, theta.imp, LL.imp)
               {
               n <- length(LL.imp)
               samples <- rep(0, n)
               for (i in seq(length=n))
                    samples[i] <- exp(.Lookup(theta.imp[i, ],
                         hist) - LL.imp[i])
               return(list(samples=samples, mll=-log(mean(samples))))
               }
          .SplitComponents <- function(theta, LL)
               {
               tot.count <- length(LL)
               hist.count <- min(0.2 * tot.count,
                    round(sqrt(tot.count) * 2))
               width.count <- 40
               imp.count <- tot.count - hist.count - width.count
               return(list(theta.hist=theta[1:hist.count, , drop=FALSE],
                    theta.bw=theta[(hist.count + 1):(hist.count +
                         width.count), , drop=FALSE],
                    theta.imp=theta[(tot.count - imp.count + 1):tot.count, ,
                         drop=FALSE],
                    LL.hist=LL[1:hist.count],
                    LL.imp=LL[(tot.count - imp.count + 1):tot.count]))
               }
          .CheckBounds <- function(hist, bounds)
               {
               limits <- hist$limits <- .GetLimits(hist)
               if(!all((bounds[1, ] == -Inf) |
                    (bounds[1, ] < limits[1, ])))
                    stop(paste("Bounds error: histogram lower limits (",
                         paste(limits[1, ], collapse=" "),
                         ") lower than specified bounds (",
                         paste(bounds[1, ], collapse=" "), ").", sep=""))
               if(!all((bounds[2, ] == Inf) | (bounds[2, ] > limits[2, ])))
                    stop(paste("Bounds error: histogram upper limits (",
                         paste(limits[2, ], collapse=" "),
                         ") greater than specified bounds (",
                         paste(bounds[2, ], collapse=" "), ").", sep=""))
               return(hist)
               }
          options(warn=-1)
          comps <- .SplitComponents(theta, LL)
          width <- .GetWidth(comps$theta.hist, comps$theta.bw,
               comps$LL.hist, cov.prob)
          hist <- .PopulateHist(comps$theta.hist, comps$LL.hist, width)
          hist <- .SetHistNorm(hist)
          hist <- .CheckBounds(hist, bounds)
          ml.out <- .MargLL(hist, comps$theta.imp, comps$LL.imp)
          ml.out$samples <- ml.out$samples[is.finite(ml.out$samples)]
          sd.samples <- sd(ml.out$samples) / sqrt(length(ml.out$samples))
          conf.interval <- qnorm(c(0.975, 0.025)) * sd.samples +
               mean(ml.out$samples)
          conf.interval[which(conf.interval < .Machine$double.eps)] <- .Machine$double.eps
          conf.interval <- -log(conf.interval)
          options(warn=0)
          LML <- ml.out$mll + hist$norm
          LML[which(!is.finite(LML))] <- NA
          ### Output
          LML.out <- list(LML=LML, VarCov=NA)
          }
     return(LML.out)
     }

#End

###########################################################################
# interval                                                                #
#                                                                         #
# The purpose of the interval function is to constrain the element(s) of  #
# a scalar, vector, matrix, or array to the interval [a,b].               #
###########################################################################

interval <- function(x, a=-Inf, b=Inf, reflect=TRUE)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     if(a > b) stop("a > b.")
     if(reflect & is.finite(a) & is.finite(b) & any(!is.finite(x))) {
               if(is.array(x)) {
                    d <- dim(x)
                    x <- as.vector(x)}
               x.inf.pos <- !is.finite(x);
               x[x.inf.pos] <- interval(x[x.inf.pos], a, b, reflect=FALSE)
               if(is.array(x)) x <- array(x, dim=d)
          }
     ### Scalar
     if(is.vector(x) & {length(x) == 1}) {
          if(reflect == FALSE) x <- max(a, min(b, x))
          else if(x < a | x > b) {
               out <- TRUE
               while(out) {
                    if(x < a) x <- a + a - x
                    if(x > b) x <- b + b - x
                    if(x >= a & x <= b) out <- FALSE
                    }}}
     ### Vector
     else if(is.vector(x) & {length(x) > 1}) {
          if(reflect == FALSE) {
               x.num <- which(x < a)
               x[x.num] <- a
               x.num <- which(x > b)
               x[x.num] <- b}
          else if(any(x < a) | any(x > b)) {
               out <- TRUE
               while(out) {
                    x.num <- which(x < a)
                    x[x.num] <- a + a - x[x.num]
                    x.num <- which(x > b)
                    x[x.num] <- b + b - x[x.num]
                    if(all(x >= a) & all(x <= b)) out <- FALSE
                    }}}
     ### Matrix or Array
     else if(is.array(x)) {
          d <- dim(x)
          x <- as.vector(x)
          if(reflect == FALSE) {
               x.num <- which(x < a)
               x[x.num] <- a
               x.num <- which(x > b)
               x[x.num] <- b}
          else if(any(x < a) | any(x > b)) {
               out <- TRUE
               while(out) {
                    x.num <- which(x < a)
                    x[x.num] <- a + a - x[x.num]
                    x.num <- which(x > b)
                    x[x.num] <- b + b - x[x.num]
                    if(all(x >= a) & all(x <= b)) out <- FALSE
               }}
          x <- array(x, dim=d)}
     return(x)
     }

#End

###########################################################################
# deburn                                                                  #
#                                                                         #
# The purpose of deburn() is to remove the user-specified burn-in from an #
# object of class demonoid.                                               #
###########################################################################

deburn <- function(x, BurnIn=0)
     {
     ### Initial Checks
     if(!identical(class(x), "demonoid"))
          stop("x is not an object of class demonoid.")
     S <- nrow(x$Posterior1)
     if(S < 22) stop("x has too few posterior samples.")
     BurnIn <- abs(round(BurnIn))
     if(BurnIn >= S) BurnIn <- S - 2
     LIV <- x$Parameters
     ### Remove Burn-in
     x$Posterior1 <- x$Posterior2 <- x$Posterior1[(BurnIn+1):S,]
     x$Deviance <- x$Deviance[(BurnIn+1):S]
     x$Monitor <- x$Monitor[(BurnIn+1):S,,drop=FALSE]
     x$Rec.BurnIn.Thinned <- 0
     x$Rec.BurnIn.UnThinned <- 0
     x$Thinned.Samples <- x$Thinned.Samples - BurnIn
     ### Summary1
     x$Summary1[1:LIV,1] <- colMeans(x$Posterior1)
     x$Summary1[1:LIV,2] <- sqrt(.colVars(x$Posterior1))
     x$Summary1[1:LIV,4] <- ESS(x$Posterior1)
     x$Summary1[1:LIV,5] <- apply(x$Posterior1, 2, quantile, c(0.025),
          na.rm=TRUE)
     x$Summary1[1:LIV,6] <- apply(x$Posterior1, 2, quantile, c(0.500),
          na.rm=TRUE)
     x$Summary1[1:LIV,7] <- apply(x$Posterior1, 2, quantile, c(0.975),
          na.rm=TRUE)
     for (i in 1:LIV) {
          temp <- try(MCSE(x$Posterior1[,i]), silent=TRUE)
          if(!inherits(temp, "try-error")) x$Summary1[i,3] <- temp
          else x$Summary1[i,3] <- MCSE(x$Posterior1[,i],
               method="sample.variance")}
     ### Deviance
     x$Summary1[LIV+1,1] <- mean(x$Deviance)
     x$Summary1[LIV+1,2] <- sd(x$Deviance)
     temp <- try(MCSE(x$Deviance), silent=TRUE)
     if(inherits(temp, "try-error"))
          temp <- MCSE(x$Deviance, method="sample.variance")
     x$Summary1[LIV+1,3] <- temp
     x$Summary1[LIV+1,4] <- ESS(x$Deviance)
     x$Summary1[LIV+1,5] <- as.numeric(quantile(x$Deviance, probs=0.025,
          na.rm=TRUE))
     x$Summary1[LIV+1,6] <- as.numeric(quantile(x$Deviance, probs=0.500,
          na.rm=TRUE))
     x$Summary1[LIV+1,7] <- as.numeric(quantile(x$Deviance, probs=0.975,
          na.rm=TRUE))
     ### Monitor
     Num.Mon <- ncol(x$Monitor)
     x$Summary1[LIV+1+1:Num.Mon,1] <- colMeans(x$Monitor)
     x$Summary1[LIV+1+1:Num.Mon,2] <- sqrt(.colVars(x$Monitor))
     x$Summary1[LIV+1+1:Num.Mon,4] <- ESS(x$Monitor)
     x$Summary1[LIV+1+1:Num.Mon,5] <- apply(x$Monitor, 2, quantile,
          c(0.025), na.rm=TRUE)
     x$Summary1[LIV+1+1:Num.Mon,6] <- apply(x$Monitor, 2, quantile,
          c(0.500), na.rm=TRUE)
     x$Summary1[LIV+1+1:Num.Mon,7] <- apply(x$Monitor, 2, quantile,
          c(0.975), na.rm=TRUE)
     for (i in 1:Num.Mon) {
          temp <- try(MCSE(x$Monitor[,i]), silent=TRUE)
          if(!inherits(temp, "try-error")) x$Summary1[LIV+1+i,3] <- temp
          else x$Summary1[LIV+1+i,3] <- MCSE(x$Monitor[,i],
               method="sample.variance")}
     ### Summary2
     x$Summary2 <- x$Summary1
     ### DIC
     x$DIC1 <- x$DIC2 <- c(mean(x$Deviance), var(x$Deviance)/2,
          mean(x$Deviance) + var(x$Deviance)/2)
     ### Output
     return(x)
     }

#End

###########################################################################
# print.heidelberger                                                      #
#                                                                         #
# The purpose of the print.heidelberger function is to print the contents #
# of an object of class raftery to the screen.                            #
###########################################################################

print.heidelberger <- function(x, digits=3, ...) 
     {
     HW.title <- matrix(c("Stationarity", "test", "start", "iteration",
          "p-value", "", "Halfwidth", "test", "Mean", "", "Halfwidth", ""),
          nrow=2)
     y <- matrix("", nrow=nrow(x), ncol=6)
     for (j in 1:ncol(y)) y[,j] <- format(x[,j], digits=digits)
     y[,c(1,4)] <- ifelse(x[,c(1,4)], "passed", "failed")
     y <- rbind(HW.title, y)
     vnames <- if(is.null(rownames(x))) paste("[,", 1:nrow(x), "]", sep="")
     else rownames(x)
     dimnames(y) <- list(c("", "", vnames), rep("", 6))
     print.default(y[, 1:3], quote=FALSE, ...)
     print.default(y[, 4:6], quote=FALSE, ...)
     invisible(x)
     }

#End

###########################################################################
# Elicitation                                                             #
#                                                                         #
# The purpose of these functions is to facilitate prior elicitation and   #
# its use in model specification.                                         #
###########################################################################

delicit <- function(theta, x, a=-Inf, b=Inf, log=FALSE)
     {
     ### Initial Checks
     if(missing(theta)) stop("The theta argument is required.")
     if(missing(x)) stop("The x argument is required.")
     if(a >= b) stop("Lower bound a is not less than upper bound b.")
     if(is.finite(a) & is.infinite(b)) {
          if(a > 0) {theta <- log(theta); x <- log(x)}
          else if(a == 0) {
               theta <- log(theta + 1e-04); x <- log(x + 1e-04)
               }
          else { #(a < 0)
               theta <- log(theta - a + 1e-04)
               x <- log(x - a + 1e-04)
               }
          }
     if(is.infinite(a) & is.finite(b)) {
          a <- .Machine$double.xmin
          theta[which(theta <= a)] <- a + 1e-04
          theta[which(theta >= b)] <- b - 1e-04
          x[which(x <= a)] <- a + 1e-04
          x[which(x >= b)] <- b - 1e-04
          theta <- log((theta-a) / (b-theta))
          x <- log((x-a) / (b-x))
          }
     if(is.finite(a) & is.finite(b)) {
          theta[which(theta <= a)] <- a + 1e-04
          theta[which(theta >= b)] <- b - 1e-04
          x[which(x <= a)] <- a + 1e-04
          x[which(x >= b)] <- b - 1e-04
          theta <- log((theta-a) / (b-theta))
          x <- log((x-a) / (b-x))
          }
     ### Estimate Density
     kde <- density(x)
     dens <- approx(kde$x, kde$y, theta)$y
     if(log == TRUE) dens <- log(dens)
     return(dens)
     }
elicit <- function(n, cats, cat.names, show.plot=FALSE)
     {
     ### Initial Checks
     if(missing(n)) stop("The n argument is required.")
     if(missing(cats)) stop("The cats argument is required.")
     if(missing(cat.names)) stop("The cat.names argument is required.")
     if(!identical(length(cats),length(cat.names)))
          stop("Different lengths found for cats and cat.names.")
     cat.labels <- letters[1:length(cats)]
     ### Introduction
     cat("\nYou have", n, "chips.")
     cat("\nEach chip must be allocated to a category.")
     cat("\nThe categories are:")
     cat("\n\n", cat.names)
     cat("\n\nYou will be asked two questions until all chips are allocated:")
     cat("\n\n1. How many chips would you like to allocate now?")
     cat("\n2. To which category do you allocate these chips?\n")
     cat("\nCategories:", cat.names)
     cat("\nCategory Entry:", cat.labels, "\n\n")
     readline("Press Enter or Return when ready to begin: ")
     ### Elicitation
     while(n > 0) {
          cat("\n\nYou have", n ,"chips remaining.\n")
          N <- 0
          while ((N <= 0) | (N > n))
               N <- readline("How many chips would you like to allocate now? ")
          N <- as.numeric(N)
          cat("\nTo which category do you allocate these chips?\n")
          cat("\nCategories:", cat.names)
           cat("\nCategory Entry:", cat.labels, "\n\n")
          answer <- "LaplacesDemon"
          while (all(cat.labels != answer))
               answer <- readline("Category: ")
          pos <- which(cat.labels == answer)
          if(!exists("out")) out <- rep(cats[pos], N)
          else out <- c(out, rep(cats[pos], N))
          n <- n - N
          ### Barplot
          if(exists("out")) {if(show.plot == TRUE) {
               out.table <- table(out)
               count <- rep(0,length(cats))
               count[as.numeric(names(out.table))] <- as.vector(out.table)
               barplot(count, names.arg=cat.names, xlab="Category",
                    ylab="Chips", col="red")}}
          }
     cat("\n\nThank you for participating.\n")
     #Output
     return(out)
     }


#End

###########################################################################
# Importance                                                              #
#                                                                         #
# The purpose of the Importance function is to compare the impact of      #
# design matrix X on replicates when each column vector (predictor) is    #
# sequentially removed.                                                   #
###########################################################################

Importance <- function(object, Model, Data, Categorical=FALSE, Discrep,
     d=0, CPUs=1, Type="PSOCK")
     {
     if(missing(object)) stop("The object argument is required.")
     if(missing(Model)) stop("The Model arguement is required.")
     if(missing(Data)) stop("The Data argument is required.")
     if(is.null(Data[["X"]])) stop("Data must have X.")
     if(missing(Discrep)) Discrep <- NULL
     X.orig <- Data[["X"]]
     cat("\nX has", ncol(X.orig), "variables")
     cat("\nEstimating the full model...")
     Pred <- predict(object, Model, Data)
     Summ <- summary(Pred, Categorical=Categorical, Discrep=Discrep, d=d,
          Quiet=TRUE)
     out <- matrix(0, ncol(X.orig) + 1, 4)
     out[1,1] <- Summ$BPIC[1,3]
     if(Categorical == FALSE) out[1,2] <- round(Summ$Concordance, 3)
     else out[1,2] <- round(Summ$Mean.Lift, 3)
     out[1,3] <- Summ$Discrepancy.Statistic
     if(Categorical == FALSE) {
          out[1,4] <- Summ$L.criterion
          S.L <- Summ$S.L}
     else S.L <- NA
     for (i in 1:ncol(X.orig)) {
          cat("\nEstimating without X[,", i, "]...", sep="")
          X.temp <- X.orig
          X.temp[,i] <- 0
          Data[["X"]] <- X.temp
          Pred <- predict(object, Model, Data, CPUs, Type)
          Summ <- summary(Pred, Categorical=Categorical,
               Discrep=Discrep, d=d, Quiet=TRUE)
          out[i+1,1] <- Summ$BPIC[1,3]
          if(Categorical == FALSE) out[i+1,2] <- round(Summ$Concordance, 3)
          else out[i+1,2] <- round(Summ$Mean.Lift, 3)
          out[i+1,3] <- Summ$Discrepancy.Statistic
          if(Categorical == FALSE) {
               out[i+1,4] <- Summ$L.criterion
               S.L <- c(S.L, Summ$S.L)}}
     if(Categorical == FALSE) cat("\n\nS.L:", S.L)
     colnames(out) <- c("BPIC","Concordance", "Discrep", "L-criterion")
     rownames(out) <- c("Full", paste("X[,-", 1:ncol(X.orig), "]", sep=""))
     attr(out, "S.L") <- S.L
     class(out) <- "importance"
     cat("\n\n")
     return(out)
     }

#End

###########################################################################
# de.Finetti.Game                                                         #
#                                                                         #
# The purpose of the de.Finetti.Game function is to elicit the interval   #
# of a subjective probability about a possible event in the near future.  #
###########################################################################

de.Finetti.Game <- function(width)
     {
     if(missing(width)) stop("The width argument is required.")
     if((width <= 0) | (width > 1))
          stop("The width argument is incoherent.")
     ques <- paste("\nDescribe a possible event in the near",
          "future (such as ``rain tomorrow''): ")
     event <- readline(ques)
     region <- c(0,1)
     while((region[2] - region[1]) > width) {
          x <- round(mean(region) * 100)
          y <- 100 - x
          cat("\nYou have two options:")
          cat("\n\n1.Wait and receive $1 if the event happens.")
          cat("\n2.Draw a marble from an urn with", x,
               "black marbles and", y, "white marbles. Drawing a black",
               "marble results in receiving $1.")
          ans <- readline("\n\nChoose 1 or 2: ")
          if(ans == 1) region[1] <- x / 100
          else if(ans == 2) region[2] <- x / 100
          else region <- c(0,0)
          }
     if(sum(region) == 0) cat("\nTry again. Valid answers are 1 or 2.")
     else {cat("\n\nYour subjective probability is in the interval [",
          region[1], ",", region[2], "] regarding ", event, ".\n\n",
          sep="")}
     return(region)
     }

#End

###########################################################################
# BayesianBootstrap                                                       #
#                                                                         #
# The purpose of the BayesianBootstrap is to allow the user to produce    #
# either bootstrapped weights or statistics.                              #
###########################################################################

BayesianBootstrap <- function(X, n=1000, Method="weights", Status=NULL)
     {
     ### Initial Checks
     if(missing(X)) stop("X is a required argument.")
     if(!is.matrix(X)) X <- as.matrix(X)
     if(any(!is.finite(X))) stop("Non-finite values found in X.")
     S <- round(abs(n))
     if(S < 1) S <- 1
     if(!(is.numeric(Status) & (length(Status) == 1))) Status <- S + 1
     else {
          Status <- round(abs(Status))
          if(Status < 1 | Status > S) Status <- S + 1}
     N <- nrow(X)
     J <- ncol(X)
     if(identical(Method, "weights")) {
          BB <- replicate(S, diff(c(0, sort(runif(N-1)), 1)))
          return(BB)}
     ### Bayesian Bootstrap: Statistics
     BB <- vector("list", S)
     for (s in 1:S) {
          if(s %% Status == 0) cat("\nBootstrapped Samples:", s)
          u <- c(0, sort(runif(N - 1)), 1)
          g <- diff(u)
          BB[[s]] <- Method(X, g)}
     if(Status < S) cat("\n\nThe Bayesian Bootstrap has finished.\n\n")
     ### Output
     BB <- lapply(BB, identity)
     if(is.vector(BB[[1]])) 
          if(length(BB[[1]]) == 1) BB <- as.matrix(BB)
          else {
               B <- matrix(unlist(BB), S, length(BB[[1]]), byrow=TRUE)
               colnames(B) <- names(BB[[1]])
               BB <- B
               }
     else {
          if(is.null(dim(BB[[1]])))
               stop("Method must return a vector, matrix or array")
          B <- array(NA, dim=c(S, dim(BB[[1]])))
          for (s in 1:S) {B[s,,] <- BB[[s]]}
          BB <- B
          }
     return(BB)
     }

#End

###########################################################################
# logit                                                                   #
#                                                                         #
# The logit function is the inverse of the sigmoid or logistic function,  #
# and transforms a continuous value (usually probability p) in the        #
# interval [0,1] to the real line (where it usually is the logarithm of   #
# the odds). The invlogit function (called either the inverse logit or    #
# the logistic function) transforms a real number (usually the logarithm  #
# of the odds) to a value (usually probability p) in the interval [0,1].  #
# If p is a probability, then p/(1-p) is the corresponding odds, while    #
# logit of p is the logarithm of the odds. The difference between the     #
# logits of two probabilities is the logarithm of the odds ratio. The     #
# derivative of probability p in a logistic function is:                  #
# (d / dx) = p * (1 - p).                                                 #
###########################################################################

invlogit <- function(x)
     {
     InvLogit <- 1 / {1 + exp(-x)}
     return(InvLogit)
     }
logit <- function(p)
     {
     if({any(p < 0)} || {any(p > 1)}) stop("p must be in [0,1].")
     Logit <- log(p / {1 - p})
     return(Logit)
     }

#End

###########################################################################
# print.iterquad                                                          #
#                                                                         #
# The purpose of the print.iterquad function is to print the contents of  #
# an object of class iterquad to the screen.                              #
###########################################################################

print.iterquad <- function(x, ...)
     {
     if(missing(x)) stop("The x argument is required.")
     cat("\nAlgorithm: ", x$Algorithm, sep="")
     cat("\nCall:\n")
     print(x$Call)
     cat("\nConverged: ", x$Converged, "\n", sep="")
     cat("Covariance Matrix: (NOT SHOWN HERE; diagonal shown instead)\n")
     print(diag(x$Covar))
     cat("\nDeviance (Final): ", x$Deviance[length(x$Deviance)], "\n")
     cat("History: (NOT SHOWN HERE)\n")
     cat("Initial Values:\n")
     print(x$Initial.Values)
     cat("\nIterations: ", x$Iterations, "\n", sep="")
     cat("Log(Marginal Likelihood): ", x$LML, "\n", sep="")
     cat("Log-Posterior (Final): ", x$LP.Final, "\n", sep="")
     cat("Log-Posterior (Initial): ", x$LP.Initial, "\n", sep="")
     cat("Log-Posterior (Weights): (NOT SHOWN HERE)\n")
     cat("M: (NOT SHOWN HERE)\n")
     cat("Minutes of run-time: ", x$Minutes, "\n", sep="")
     cat("Monitor: (NOT SHOWN HERE)\n")
     cat("Nodes: ", x$N, "\n", sep="")
     cat("Posterior: (NOT SHOWN HERE)\n")
     cat("Summary1: (SHOWN BELOW)\n")
     cat("Summary2: (SHOWN BELOW)\n")
     cat("Tolerance (Final):\n")
     print(x$Tolerance.Final)
     cat("Tolerance (Stop):\n")
     print(x$Tolerance.Stop)
     cat("Z: (NOT SHOWN HERE)\n")
     
     cat("\nSummary1:\n")
     print(x$Summary1)
     if({x$Converged == TRUE} && !any(is.na(x$Posterior))) {
          cat("\nSummary2:\n")
          print(x$Summary2)}
     
     invisible(x)
     }

#End

###########################################################################
# Asymmetric Laplace Distribution                                         #
#                                                                         #
# These functions are similar to those in the VGAM package.               #
###########################################################################

dalaplace <- function(x, location=0, scale=1, kappa=1, log=FALSE)
     {
     x <- as.vector(x); location <- as.vector(location)
     scale <- as.vector(scale); kappa <- as.vector(kappa)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     if(any(kappa <= 0)) stop("The kappa parameter must be positive.")
     NN <- max(length(x), length(location), length(scale), length(kappa))
     x <- rep(x, len=NN); location <- rep(location, len=NN)
     scale <- rep(scale, len=NN); kappa <- rep(kappa, len=NN)
     logconst <- 0.5 * log(2) - log(scale) + log(kappa) - log1p(kappa^2)
     temp <- which(x < location); kappa[temp] <- 1/kappa[temp]
     exponent <- -(sqrt(2) / scale) * abs(x - location) * kappa
     dens <- logconst + exponent
     if(log == FALSE) dens <- exp(logconst + exponent)
     return(dens)
     }
palaplace <- function(q, location=0, scale=1, kappa=1)
     {
     q <- as.vector(q); location <- as.vector(location)
     scale <- as.vector(scale); kappa <- as.vector(kappa)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     if((kappa <= 0)) stop("The kappa parameter must be positive.")
     NN <- max(length(q), length(location), length(scale), length(kappa))
     q <- rep(q, len=NN); location <- rep(location, len=NN)
     scale <- rep(scale, len=NN); kappa <- k2 <- rep(kappa, len=NN)
     temp <- which(q < location); k2[temp] <- 1/kappa[temp]
     exponent <- -(sqrt(2) / scale) * abs(q - location) * k2
     temp <- exp(exponent) / (1 + kappa^2)
     p <- 1 - temp
     index1 <- (q < location)
     p[index1] <- (kappa[index1])^2 * temp[index1]
     return(p)
     }
qalaplace <- function(p, location=0, scale=1, kappa=1)
     {
     p <- as.vector(p); location <- as.vector(location)
     scale <- as.vector(scale); kappa <- as.vector(kappa)
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     if(any(kappa <= 0)) stop("The kappa parameter must be positive.")
     NN <- max(length(p), length(location), length(scale), length(kappa))
     p <- rep(p, len=NN); location <- rep(location, len=NN)
     scale <- rep(scale, len=NN); kappa <- rep(kappa, len=NN)
     q <- p
     temp <- kappa^2 / (1 + kappa^2)
     index1 <- (p <= temp)
     exponent <- p[index1] / temp[index1]
     q[index1] <- location[index1] + (scale[index1] * kappa[index1]) *
          log(exponent) / sqrt(2)
     q[!index1] <- location[!index1] - (scale[!index1] / kappa[!index1]) *
          (log1p((kappa[!index1])^2) + log1p(-p[!index1])) / sqrt(2)
     q[p == 0] = -Inf
     q[p == 1] = Inf
     return(q)
     }
ralaplace <- function(n, location=0, scale=1, kappa=1)
     {
     location <- rep(location, len=n)
     scale <- rep(scale, len=n)
     kappa <- rep(kappa, len=n)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     if(any(kappa <= 0)) stop("The kappa parameter must be positive.")
     x <- location + scale *
          log(runif(n)^kappa / runif(n)^(1/kappa)) / sqrt(2)
     return(x)
     }

###########################################################################
# Asymmetric Log-Laplace Distribution                                     #
#                                                                         #
# These functions are similar to those in the VGAM package.               #
###########################################################################

dallaplace <- function(x, location=0, scale=1, kappa=1, log=FALSE)
     {
     x <- as.vector(x); location <- as.vector(location)
     scale <- as.vector(scale); kappa <- as.vector(kappa)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     if(any(kappa <= 0)) stop("The kappa parameter must be positive.")
     NN <- max(length(x), length(location), length(scale), length(kappa))
     x <- rep(x, len=NN); location <- rep(location, len=NN)
     scale <- rep(scale, len=NN); kappa <- rep(kappa, len=NN)
     Alpha <- sqrt(2) * kappa / scale
     Beta  <- sqrt(2) / (scale * kappa)
     Delta <- exp(location)
     exponent <- -(Alpha + 1)
     temp <- which(x >= Delta); exponent[temp] <- Beta[temp] - 1
     exponent <- exponent * (log(x) - location)
     dens <- -location + log(Alpha) + log(Beta) -
          log(Alpha + Beta) + exponent
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
pallaplace <- function(q, location=0, scale=1, kappa=1)
     {
     q <- as.vector(q); location <- as.vector(location)
     scale <- as.vector(scale); kappa <- as.vector(kappa)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     if(any(kappa <= 0)) stop("The kappa parameter must be positive.")
     NN <- max(length(q), length(location), length(scale), length(kappa))
     q <- rep(q, len=NN); location <- rep(location, len=NN)
     scale <- rep(scale, len=NN); kappa <- rep(kappa, len=NN)
     Alpha <- sqrt(2) * kappa / scale
     Beta  <- sqrt(2) / (scale * kappa)
     Delta <- exp(location)
     temp <- Alpha + Beta
     p <- (Alpha / temp) * (q / Delta)^(Beta)
     p[q <= 0] <- 0
     index1 <- (q >= Delta)
     p[index1] <- (1 - (Beta/temp) * (Delta/q)^(Alpha))[index1]
     return(p)
     }
qallaplace <- function(p, location=0, scale=1, kappa=1)
     {
     p <- as.vector(p); location <- as.vector(location)
     scale <- as.vector(scale); kappa <- as.vector(kappa)
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     if(any(kappa <= 0)) stop("The kappa parameter must be positive.")
     NN <- max(length(p), length(location), length(scale), length(kappa))
     p <- rep(p, len=NN); location <- rep(location, len=NN)
     scale <- rep(scale, len=NN); kappa <- rep(kappa, len=NN)
     Alpha <- sqrt(2) * kappa / scale
     Beta  <- sqrt(2) / (scale * kappa)
     Delta <- exp(location)
     temp <- Alpha + Beta
     q <- Delta * (p * temp / Alpha)^(1/Beta)
     index1 <- (p > Alpha / temp)
     q[index1] <- (Delta * ((1-p) * temp / Beta)^(-1/Alpha))[index1]
     q[p == 0] <- 0
     q[p == 1] <- Inf
     return(q)
     }
rallaplace <- function(n, location=0, scale=1, kappa=1)
     {
     location <- rep(location, len=n); scale <- rep(scale, len=n)
     kappa <- rep(kappa, len=n)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     if(any(kappa <= 0)) stop("The kappa parameter must be positive.")
     x <- exp(location) *
          (runif(n)^kappa / runif(n)^(1/kappa))^(scale / sqrt(2))
     return(x)
     }

###########################################################################
# Asymmetric Multivariate Laplace Distribution                            #
###########################################################################

daml <- function (x, mu, Sigma, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(Sigma)) Sigma <- diag(ncol(x))
     if(!is.matrix(Sigma)) Sigma <- matrix(Sigma)
     Sigma <- as.symmetric.matrix(Sigma)
     if(!is.positive.definite(Sigma))
          stop("Matrix Sigma is not positive-definite.")
     k <- nrow(Sigma)
     Omega <- as.inverse(Sigma)
     x.Omega.mu <- rowSums(x %*% Omega * mu)
     x.Omega.x <- rowSums(x %*% Omega * x)
     mu.Omega.mu <- rowSums(mu %*% Omega * mu)
     dens <- as.vector(log(2) + x.Omega.mu -
          (log(2*pi)*(k/2) + logdet(Sigma)*0.5) +
          (log(x.Omega.x) - (log(2 + mu.Omega.mu)))*((2-k)/4) +
          log(besselK(sqrt((2 + mu.Omega.mu) * x.Omega.x), (2-k)/2)))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
raml <- function(n, mu, Sigma)
     {
     mu <- rbind(mu)
     if(missing(Sigma)) Sigma <- diag(ncol(mu))
     if(!is.matrix(Sigma)) Sigma <- matrix(Sigma)
     if(!is.positive.definite(Sigma))
          stop("Matrix Sigma is not positive-definite.")
     k <- ncol(Sigma)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     e <- matrix(rexp(n, 1), n, k)
     z <- rmvn(n, rep(0, k), Sigma)
     x <- mu*e + sqrt(e)*z
     return(x)
     }

###########################################################################
# Bernoulli Distribution                                                  #
#                                                                         #
# These functions are similar to those in the Rlab package.               #
###########################################################################

dbern <- function(x, prob, log=FALSE)
     {return(dbinom(x, 1, prob, log))}
pbern <- function(q, prob, lower.tail=TRUE, log.p=FALSE)
     {return(pbinom(q, 1, prob, lower.tail, log.p))}
qbern <- function(p, prob, lower.tail=TRUE, log.p=FALSE)
     {return(qbinom(p, 1, prob, lower.tail, log.p))}
rbern <- function(n, prob)
     {return(rbinom(n, 1, prob))}

###########################################################################
# Categorical Distribution                                                #
###########################################################################

dcat <- function (x, p, log=FALSE)
     {
     if(!is.matrix(p)) {
          if(is.vector(x)) p <- matrix(p, length(x), length(p), byrow=TRUE)
          else p <- matrix(p, nrow(x), length(p), byrow=TRUE)}
     if(is.vector(x)) {
          if(length(x) == 1) {
               temp <- rbind(rep(0, ncol(p)))
               temp[1,x] <- 1
               x <- temp
               }
          else x <- as.indicator.matrix(x)}
     if(!identical(nrow(x), nrow(p)))
          stop("The number of rows of x and p differ.")
     if(!identical(ncol(x), ncol(p))) {
          x.temp <- matrix(0, nrow(p), ncol(p))
          x.temp[, as.numeric(colnames(x))] <- x
          x <- x.temp}
     dens <- x * log(p)
     if(log == FALSE) dens <- x * p
     dens <- as.vector(rowSums(dens))
     return(dens)
     }
qcat <- function(pr, p, lower.tail=TRUE, log.pr=FALSE)
     {
     if(!is.vector(pr)) pr <- as.vector(pr)
     if(!is.vector(p)) p <- as.vector(p)
     if(log.pr == FALSE) {
          if(any(pr < 0) | any(pr > 1))
               stop("pr must be in [0,1].")}
     else if(any(!is.finite(pr)) | any(pr > 0))
          stop("pr, as a log, must be in (-Inf,0].")
     if(sum(p) != 1) stop("sum(p) must be 1.")
     if(lower.tail == FALSE) pr <- 1 - pr
     breaks <- c(0, cumsum(p))
     if(log.pr == TRUE) breaks <- log(breaks)
     breaks <- matrix(breaks, length(pr), length(breaks), byrow=TRUE)
     x <- rowSums(pr > breaks)
     return(x)
     }
rcat <- function(n, p)
     {
     if(is.vector(p)) {
          x <- as.vector(which(rmultinom(n, size=1, prob=p) ==
               1, arr.ind=TRUE)[, "row"])
          }
     else {
          d <- dim(p)
          n <- d[1]
          k <- d[2]
          lev <- dimnames(p)[[2]]
          if(!length(lev)) lev <- 1:k
          z <- colSums(p)
          U <- apply(p, 1, cumsum)
          U[,k] <- 1
          un <- rep(runif(n), rep(k,n))
          x <- lev[1 + colSums(un > U)]}
     return(x)
     }

###########################################################################
# Continuous Relaxation of a Markov Random Field Distribution             #
###########################################################################

dcrmrf <- function(x, alpha, Omega, log=FALSE)
     {
     alpha <- as.vector(alpha)
     if(missing(Omega)) Omega <- diag(length(alpha))
     if(!is.matrix(Omega)) Omega <- matrix(Omega)
     Omega <- as.symmetric.matrix(Omega)
     if(!is.positive.definite(Omega))
          stop("Matrix Omega is not positive-definite.")
     dens <- as.vector(-0.5*t(x) %*% as.inverse(Omega) %*% x) +
          log(prod(1 + exp(x + alpha)))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rcrmrf <- function(n=1, alpha, Omega)
     {
     alpha <- as.vector(alpha)
     J <- length(alpha)
     if(missing(Omega)) Omega <- diag(J)
     if(!is.matrix(Omega)) Omega <- matrix(Omega)
     if(!is.positive.definite(Omega))
          stop("Matrix Omega is not positive-definite.")
     dens <- rep(0,J)
     x <- rep(0,n)
     for (i in 1:n) {
          for (j in 1:J) {
               z <- as.vector(rmvn(1,
                    as.vector(Omega %*% diag(J)[j,]), Omega))
               dens[j] <- dcrmrf(z, alpha, Omega, log=FALSE)}
          x[i] <- sample(1:J, size=1, replace=TRUE, prob=dens)}
     return(x)
     }

###########################################################################
# Dirichlet Distribution                                                  #
#                                                                         #
# These functions are similar to those in the MCMCpack package.           #
###########################################################################

ddirichlet <- function(x, alpha, log=FALSE)
     {
     if(missing(x)) stop("x is a required argument.")
     if(missing(alpha)) stop("alpha is a required argument.")
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(alpha))
          alpha <- matrix(alpha, nrow(x), length(alpha), byrow=TRUE)
     if(any(rowSums(x) != 1)) x / rowSums(x)
     if(any(x < 0)) stop("x must be non-negative.")
     if(any(alpha <= 0)) stop("alpha must be positive.")
     dens <- as.vector(lgamma(rowSums(alpha)) - rowSums(lgamma(alpha)) +
          rowSums((alpha-1)*log(x)))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rdirichlet <- function (n, alpha)
     {
     alpha <- rbind(alpha)
     alpha.dim <- dim(alpha)
     if(n > alpha.dim[1])
          alpha <- matrix(alpha, n, alpha.dim[2], byrow=TRUE)
     x <- matrix(rgamma(alpha.dim[2]*n, alpha), ncol=alpha.dim[2])
     sm <- x %*% rep(1, alpha.dim[2])
     return(x/as.vector(sm))
     }

###########################################################################
# Generalized Pareto Distribution                                         #
###########################################################################

dgpd <- function(x, mu, sigma, xi, log=FALSE)
     {
     x <- as.vector(x)
     mu <- as.vector(mu)
     sigma <- as.vector(sigma)
     xi <- as.vector(xi)
     if(any(sigma <= 0)) stop("The sigma parameter must be positive.")
     NN <- max(length(x), length(mu), length(sigma), length(xi))
     x <- rep(x, len=NN); mu <- rep(mu, len=NN)
     sigma <- rep(sigma, len=NN); xi <- rep(xi, len=NN)
     xi.ge.0 <- which(xi >= 0)
     xi.lt.0 <- which(xi < 0)
     if(any(mu > x) |
     any(mu[xi.lt.0] < x[xi.lt.0] + sigma[xi.lt.0] / xi[xi.lt.0]))
     stop("x is outside of support.")
     z <- (x - mu) / sigma
     xi0 <- which(xi == 0)
     xi1 <- which(xi != 0)
     dens <- rep(NA, NN)
     dens[xi0] <- -z[xi0] - log(sigma[xi0])
     dens[xi1] <- log(1/sigma[xi1]) + log(1 + xi[xi1] *
     z[xi1]) * (-1/xi[xi1] - 1)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rgpd <- function(n, mu, sigma, xi)
     {
     mu <- as.vector(mu)
     sigma <- as.vector(sigma)
     xi <- as.vector(xi)
     if(any(sigma <= 0)) stop("The sigma parameter must be non-negative.")
     mu <- rep(mu, len=n); sigma <- rep(sigma, len=n)
     xi <- rep(xi, len=n)
     x <- rep(NA,n)
     u <- runif(n, min=0.001)
     xi0 <- which(xi == 0)
     xi1 <- which(xi != 0)
     x[xi0] <- mu[xi0] - sigma[xi0] * log(u[xi0])
     x[xi1] <- mu[xi1] + sigma[xi1] *
     (u[xi1]^(-xi[xi1]) - 1) / xi[xi1]
     return(x)
     }

###########################################################################
# Generalized Poisson                                                     #
###########################################################################

dgpois <- function(x, lambda=0, omega=0, log=FALSE)
     {
     x <- as.vector(x); lambda <- as.vector(lambda)
     omega <- as.vector(omega)
     lambda.star <- (1 - omega)*lambda + omega*x
     dens <- log(1 - omega) + log(lambda) + (x - 1)*log(lambda.star) -
          lgamma(x + 1) - lambda.star
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }

###########################################################################
# Half-Cauchy Distribution                                                #
###########################################################################

dhalfcauchy <- function(x, scale=25, log=FALSE)
     {
     x <- as.vector(x); scale <- as.vector(scale)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     NN <- max(length(x), length(scale))
     x <- rep(x, len=NN); scale <- rep(scale, len=NN)
     dens <- log(2*scale) - log(pi*{x*x + scale*scale})
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
phalfcauchy <- function(q, scale=25)
     {
     q <- as.vector(q); scale <- as.vector(scale)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     NN <- max(length(q), length(scale))
     q <- rep(q, len=NN); scale <- rep(scale, len=NN)
     z <- {2/pi}*atan(q/scale)
     return(z)
     }
qhalfcauchy <- function(p, scale=25)
     {
     p <- as.vector(p); scale <- as.vector(scale)
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     NN <- max(length(p), length(scale))
     p <- rep(p, len=NN); scale <- rep(scale, len=NN)
     q <- scale*tan({pi*p}/2)
     return(q)
     }
rhalfcauchy <- function(n, scale=25)
     {
     scale <- rep(scale, len=n)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     p <- runif(n, 0, 1)
     x <- scale*tan({pi*p}/2)
     return(x)
     }

###########################################################################
# Half-Normal Distribution                                                #
#                                                                         #
# This half-normal distribution has mean=0 and is similar to the halfnorm #
# functions in package fdrtool.                                           #
###########################################################################

dhalfnorm <- function(x, scale=sqrt(pi/2), log=FALSE)
     {
     dens <- log(2) + dnorm(x, mean=0, sd=sqrt(pi/2) / scale, log=TRUE)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
phalfnorm <- function(q, scale=sqrt(pi/2), lower.tail=TRUE, log.p=FALSE)
     {
     q <- as.vector(q); scale <- as.vector(scale)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     NN <- max(length(q), length(scale))
     q <- rep(q, len=NN); scale <- rep(scale, len=NN)
     p <- 2*pnorm(q, mean=0, sd=sqrt(pi/2) / scale) - 1
     if(lower.tail == FALSE) p <- 1-p
     if(log.p == TRUE) p <- log.p(p)
     return(p)
     }
qhalfnorm <- function(p, scale=sqrt(pi/2), lower.tail=TRUE, log.p=FALSE)
     {
     p <- as.vector(p); scale <- as.vector(scale)
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     NN <- max(length(p), length(scale))
     p <- rep(p, len=NN); scale <- rep(scale, len=NN)
     if(log.p == TRUE) p <- exp(p)
     if(lower.tail == FALSE) p <- 1-p
     q <- qnorm((p+1)/2, mean=0, sd=sqrt(pi/2) / scale)
     return(q)
     }
rhalfnorm <- function(n, scale=sqrt(pi/2))
     {
     scale <- rep(scale, len=n)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     x <- abs(rnorm(n, mean=0, sd=sqrt(pi/2) / scale))
     return(x)
     }

###########################################################################
# Half-t Distribution                                                     #
###########################################################################

dhalft <- function(x, scale=25, nu=1, log=FALSE)
     {
     x <- as.vector(x); scale <- as.vector(scale); nu <- as.vector(nu)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     NN <- max(length(x), length(scale), length(nu))
     x <- rep(x, len=NN); scale <- rep(scale, len=NN)
     nu <- rep(nu, len=NN)
     dens <- log(2) -log(scale) + lgamma((nu + 1)/2) - lgamma(nu/2) -
       .5*log(pi*nu) - (nu + 1)/2 * log(1 + (1/nu) * (x/scale) * (x/scale))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
phalft <- function(q, scale=25, nu=1)
     {
     q <- as.vector(q); scale <- as.vector(scale); nu <- as.vector(nu)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     NN <- max(length(q), length(scale), length(nu))
     q <- rep(q, len=NN); scale <- rep(scale, len=NN)
     p <- ptrunc(q, "st", a=0, b=Inf, mu=0, sigma=scale, nu=nu)
     return(p)
     }
qhalft <- function(p, scale=25, nu=1)
     {
     p <- as.vector(p); scale <- as.vector(scale); nu <- as.vector(nu)
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     NN <- max(length(p), length(scale), length(nu))
     p <- rep(p, len=NN); scale <- rep(scale, len=NN)
     q <- qtrunc(p, "st", a=0, b=Inf, mu=0, sigma=scale, nu=nu)
     return(q)
     }
rhalft <- function(n, scale=25, nu=1)
     {
     scale <- rep(scale, len=n); nu <- rep(nu, len=n)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     x <- rtrunc(n, "st", a=0, b=Inf, mu=0, sigma=scale, nu=nu)
     return(x)
     }

###########################################################################
# Horseshoe Distribution                                                  #
###########################################################################

dhs <- function(x, lambda, tau, log=FALSE)
     {
     dens <- dnorm(x, 0, lambda*tau, log=log)
     return(dens)
     }
rhs <- function(n, lambda, tau)
     {
     x <- rnorm(n, 0, lambda*tau)
     return(x)
     }

###########################################################################
# Huang-Wand Distribution                                                 #
###########################################################################

dhuangwand <- function(x, nu=2, a, A, log=FALSE)
     {
     if(!is.matrix(x)) x <- matrix(x)
     if(!is.positive.definite(x))
          stop("Matrix x is not positive-definite.")
     a <- as.vector(a)
     A <- as.vector(A)
     k <- nrow(x)
     if(!identical(length(a), length(A), k, ncol(x)))
          stop("Dimensions of x, a, and A do not agree.")
     dens <- sum(dinvgamma(a, 0.5, 1/A^2, log=TRUE)) +
          dinvwishart(x, nu + k - 1, 2*nu*diag(1/a), log=TRUE)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rhuangwand <- function(nu=2, a, A)
     {
     if(missing(A)) k <- length(a)
     else {
          k <- length(A)
          a <- rinvgamma(k, 0.5, 1/A^2)}
     x <- rinvwishart(nu + k - 1, 2*nu*diag(1/a))
     return(x)
     }

###########################################################################
# Huang-Wand Distribution (Cholesky Parameterization)                     #
###########################################################################

dhuangwandc <- function(x, nu=2, a, A, log=FALSE)
     {
     if(!is.matrix(x)) x <- matrix(x)
     a <- as.vector(a)
     A <- as.vector(A)
     k <- nrow(x)
     if(!identical(length(a), length(A), k, ncol(x)))
          stop("Dimensions of x, a, and A do not agree.")
     dens <- sum(dinvgamma(a, 0.5, 1/A^2, log=TRUE)) +
          dinvwishartc(x, nu + k - 1, 2*nu*diag(1/a), log=TRUE)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rhuangwandc <- function(nu=2, a, A)
     {
     if(missing(A)) k <- length(a)
     else {
          k <- length(A)
          a <- rinvgamma(k, 0.5, 1/A^2)}
     x <- rinvwishartc(nu + k - 1, 2*nu*diag(1/a))
     return(x)
     }

###########################################################################
# Inverse Beta Distribution                                               #
###########################################################################

dinvbeta <- function(x, a, b, log=FALSE)
     {
     const <- lgamma(a + b) - lgamma(a) - lgamma(b)
     dens <- const + (a - 1) * log(x) - (a + b) * log(1 + x)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rinvbeta <- function(n, a, b)
     {
     x <- rbeta(n, a, b)
     x <- x / (1-x)
     return(x)
     }

###########################################################################
# Inverse Chi-Squared Distribution                                        #
#                                                                         #
# These functions are similar to those in the GeoR package.               #
###########################################################################

dinvchisq <- function(x, df, scale=1/df, log=FALSE)
     {
     x <- as.vector(x); df <- as.vector(df); scale <- as.vector(scale)
     if(any(x <= 0)) stop("x must be positive.")
     if(any(df <= 0)) stop("The df parameter must be positive.")
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     NN <- max(length(x), length(df), length(scale))
     x <- rep(x, len=NN); df <- rep(df, len=NN);
     scale <- rep(scale, len=NN)
     nu <- df / 2
     dens <- nu*log(nu) - lgamma(nu) + nu*log(scale) -
          (nu+1)*log(x) - (nu*scale/x)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }

rinvchisq <- function(n, df, scale=1/df)
     {
     df <- rep(df, len=n); scale <- rep(scale, len=n)
     if(any(df <= 0)) stop("The df parameter must be positive.")
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     z <- rchisq(n, df=df)
     z[which(z == 0)] <- 1e-100
     x <- (df*scale) / z
     return(x)
     }

###########################################################################
# Inverse Gamma Distribution                                              #
#                                                                         #
# These functions are similar to those in the MCMCpack package.           #
###########################################################################

dinvgamma <- function(x, shape=1, scale=1, log=FALSE)
     {
     x <- as.vector(x); shape <- as.vector(shape)
     scale <- as.vector(scale)
     if(any(shape <= 0) | any(scale <=0))
          stop("The shape and scale parameters must be positive.")
     NN <- max(length(x), length(shape), length(scale))
     x <- rep(x, len=NN); shape <- rep(shape, len=NN)
     scale <- rep(scale, len=NN)
     alpha <- shape; beta <- scale
     dens <- alpha * log(beta) - lgamma(alpha) -
          {alpha + 1} * log(x) - {beta/x}
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rinvgamma <- function(n, shape=1, scale=1)
     {
     x <- rgamma(n=n, shape=shape, rate=scale)
     x[which(x < 1e-300)] <- 1e-300
     return(1 / x)
     }

###########################################################################
# Inverse Gaussian Distribution                                           #
###########################################################################

dinvgaussian <- function(x, mu, lambda, log=FALSE)
     {
     x <- as.vector(x); mu <- as.vector(mu); lambda <- as.vector(lambda)
     if(any(x <= 0)) stop("x must be positive.")
     if(any(mu <= 0)) stop("The mu parameter must be positive.")
     if(any(lambda <= 0)) stop("The lambda parameter must be positive.")
     NN <- max(length(x), length(mu), length(lambda))
     x <- rep(x, len=NN); mu <- rep(mu, len=NN)
     lambda <- rep(lambda, len=NN)
     dens <- log(lambda^0.5/(2 * pi * x^3)^0.5) - ((lambda * (x - mu)^2)/(2 * mu^2 * x))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rinvgaussian <- function(n, mu, lambda)
     {
     mu <- rep(mu, len=n); lambda <- rep(lambda, len=n)
     if(any(mu <= 0)) stop("The mu parameter must be positive.")
     if(any(lambda <= 0)) stop("The lambda parameter must be positive.")
     nu <- rnorm(n)
     y <- nu^2
     x <- mu + ((mu^2*y)/(2*lambda)) - (mu/(2*lambda)) *
          sqrt(4*mu*lambda*y + mu^2*y^2)
     z <- runif(n)
     temp <- which(z > {mu / (mu + x)})
     x[temp] <- mu[temp] * mu[temp] / x[temp]
     return(x)
     }

###########################################################################
# Inverse Matrix Gamma Distribution                                       #
###########################################################################

dinvmatrixgamma <- function(X, alpha, beta, Psi, log=FALSE)
     {
     if(!is.matrix(X)) stop("X is not a matrix.")
     if(alpha <= 2) stop("The alpha parameter must be greater than 2.")
     if(beta <= 0) stop("The beta parameter must be positive.")
     if(missing(Psi)) Psi <- diag(ncol(X))
     if(!is.positive.definite(Psi))
          stop("Matrix Psi is not positive-definite.")
     k <- nrow(Psi)
     gamsum <- 0
     for (i in 1:k) gamsum <- gamsum + lgamma(alpha - 0.5*(i-1))
     gamsum <- gamsum + log(pi)*(k*(k-1)/4)
     Omega <- as.inverse(Psi)
     dens <- logdet(Psi)*alpha - (log(beta)*(k*alpha) + gamsum) +
          logdet(X)*(-alpha-(k+1)/2) +
          tr(-(1/beta)*(Psi %*% as.inverse(X)))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }

rinvmatrixgamma <- function(alpha, beta, Psi){
  rinvwishart(2*alpha, 2/beta*Psi)
}

###########################################################################
# Inverse Wishart Distribution                                            #
###########################################################################

dinvwishart <- function(Sigma, nu, S, log=FALSE)
     {
     if(!is.matrix(Sigma)) Sigma <- matrix(Sigma)
     if(!is.positive.definite(Sigma))
          stop("Matrix Sigma is not positive-definite.")
     if(!is.matrix(S)) S <- matrix(S)
     if(!is.positive.semidefinite(S))
          stop("Matrix S is not positive-semidefinite.")
     if(!identical(dim(S), dim(Sigma)))
          stop("The dimensions of Sigma and S differ.")
     if(nu < nrow(S))
          stop("The nu parameter is less than the dimension of S.")
     k <- nrow(Sigma)
     gamsum <- 0
     for (i in 1:k) {gamsum <- gamsum + lgamma((nu + 1 - i)/2)}
     dens <- -(nu*k/2)*log(2) - ((k*(k - 1))/4)*log(pi) - gamsum +
          (nu/2)*log(det(S)) - ((nu + k + 1)/2)*logdet(Sigma) -
          0.5*tr(S %*% as.inverse(Sigma))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rinvwishart <- function(nu, S)
     {return(chol2inv(rwishartc(nu, chol2inv(chol(S)))))}

###########################################################################
# Inverse Wishart Distribution (Cholesky Parameterization)                #
###########################################################################

dinvwishartc <- function(U, nu, S, log=FALSE)
     {
     if(missing(U)) stop("Upper triangular U is required.")
     Sigma <- t(U) %*% U
     if(!is.matrix(S)) S <- matrix(S)
     if(!is.positive.semidefinite(S))
          stop("Matrix S is not positive-semidefinite.")
     if(!identical(dim(S), dim(Sigma)))
          stop("The dimensions of Sigma and S differ.")
     if(nu < nrow(S))
          stop("The nu parameter is less than the dimension of S.")
     k <- nrow(Sigma)
     gamsum <- 0
     for (i in 1:k) {gamsum <- gamsum + lgamma((nu + 1 - i)/2)}
     dens <- -(nu*k/2)*log(2) - ((k*(k - 1))/4)*log(pi) - gamsum +
          (nu/2)*log(det(S)) - ((nu + k + 1)/2)*logdet(Sigma) -
          0.5*tr(S %*% as.inverse(Sigma))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rinvwishartc <- function(nu, S)
     {return(chol(as.inverse(rwishart(nu, as.inverse(S)))))}

###########################################################################
# Laplace Distribution                                                    #
#                                                                         #
# These functions are similar to those in the VGAM package.               #
###########################################################################

dlaplace <- function(x, location=0, scale=1, log=FALSE)
     {
     x <- as.vector(x); location <- as.vector(location)
     scale <- as.vector(scale)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     NN <- max(length(x), length(location), length(scale))
     x <- rep(x, len=NN); location <- rep(location, len=NN)
     scale <- rep(scale, len=NN)
     dens <- (-abs(x - location) / scale) - log(2 * scale)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
plaplace <- function(q, location=0, scale=1)
     {
     q <- as.vector(q); location <- as.vector(location)
     scale <- as.vector(scale)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     z <- {q - location} / scale
     NN <- max(length(q), length(location), length(scale))
     q <- rep(q, len=NN); location <- rep(location, len=NN)
     p <- q
     temp <- which(q < location); p[temp] <- 0.5 * exp(z[temp])
     temp <- which(q >= location); p[temp] <- 1 - 0.5 * exp(-z[temp])
     return(p)
     }
qlaplace <- function(p, location=0, scale=1)
     {
     p <- as.vector(p); location <- as.vector(location)
     scale <- as.vector(scale)
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     NN <- max(length(p), length(location), length(scale))
     p <- p2 <- rep(p, len=NN); location <- rep(location, len=NN)
     temp <- which(p > 0.5); p2[temp] <- 1 - p[temp]
     q <- location - sign(p - 0.5) * scale * log(2 * p2)
     return(q)
     }
rlaplace <- function(n, location=0, scale=1)
     {
     location <- rep(location, len=n); scale <- rep(scale, len=n)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     r <- r2 <- runif(n)
     temp <- which(r > 0.5); r2[temp] <- 1 - r[temp]
     x <- location - sign(r - 0.5) * scale * log(2 * r2)
     return(x)
     }

###########################################################################
# Laplace Distribution (Precision Parameterization)                       #
###########################################################################

dlaplacep <- function(x, mu=0, tau=1, log=FALSE)
     {
     x <- as.vector(x); mu <- as.vector(mu); tau <- as.vector(tau)
     if(any(tau <= 0)) stop("The tau parameter must be positive.")
     NN <- max(length(x), length(mu), length(tau))
     x <- rep(x, len=NN); mu <- rep(mu, len=NN); tau <- rep(tau, len=NN)
     dens <- log(tau/2) + (-tau*abs(x-mu))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
plaplacep <- function(q, mu=0, tau=1)
     {
     q <- as.vector(q); mu <- as.vector(mu)
     if(any(tau <= 0)) stop("The tau parameter must be positive.")
     NN <- max(length(q), length(mu), length(tau))
     q <- rep(q, len=NN); mu <- rep(mu, len=NN); tau <- rep(tau, len=NN)
     p <- plaplace(q, mu, scale=1/tau)
     return(p)
     }
qlaplacep <- function(p, mu=0, tau=1)
     {
     p <- as.vector(p); mu <- as.vector(mu)
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(tau <= 0)) stop("The tau parameter must be positive.")
     NN <- max(length(p), length(mu), length(tau))
     p <- rep(p, len=NN); mu <- rep(mu, len=NN); tau <- rep(tau, len=NN)
     q <- qlaplace(p, mu, scale=1/tau)
     return(q)
     }
rlaplacep <- function(n, mu=0, tau=1)
     {
     mu <- rep(mu, len=n); tau <- rep(tau, len=n)
     if(any(tau <= 0)) stop("The tau parameter must be positive.")
     x <- rlaplace(n, mu, scale=1/tau)
     return(x)
     }

###########################################################################
# Laplace Distribution Mixture                                            #
###########################################################################

dlaplacem <- function(x, p, location, scale, log=FALSE)
     {
     if(missing(x)) stop("x is a required argument.")
     x <- as.vector(x)
     n <- length(x)
     if(missing(p)) stop("p is a required argument.")
     p <- as.vector(p)
     if(any(p <= 0) | any(p > 1)) stop("p must be in (0,1].")
     if(sum(p) != 1) stop("p must sum to 1 for all components.")
     m <- length(p)
     p <- matrix(p, n, m, byrow=TRUE)
     if(missing(location)) stop("location is a required argument.")
     location <- as.vector(location)
     if(!identical(m, length(location)))
          stop("p and location differ in length.")
     location <- matrix(location, n, m, byrow=TRUE)
     if(missing(scale)) stop("scale is a required argument.")
     scale <- as.vector(scale)
     if(!identical(m, length(scale)))
          stop("p and scale differ in length.")
     scale <- matrix(scale, n, m, byrow=TRUE)
     dens <- matrix(dlaplace(x, location, scale, log=TRUE), n, m)
     dens <- dens + log(p)
     if(log == TRUE) dens <- apply(dens, 1, logadd)
     else dens <- rowSums(exp(dens))
     return(dens)
     }
plaplacem <- function(q, p, location, scale)
     {
     n <- length(q)
     m <- length(p)
     q <- matrix(q, n, m)
     p <- matrix(p, n, m, byrow=TRUE)
     location <- matrix(location, n, m, byrow=TRUE)
     scale <- matrix(scale, n, m, byrow=TRUE)
     cdf <- matrix(plaplace(q, location, scale), n, m)
     cdf <- rowSums(cdf * p)
     return(cdf)
     }
rlaplacem <- function(n, p, location, scale)
     {
     if(missing(p)) stop("p is a required argument.")
     p <- as.vector(p)
     if(any(p <= 0) | any(p > 1)) stop("p must be in (0,1].")
     if(sum(p) != 1) stop("p must sum to 1 for all components.")
     m <- length(p)
     p <- matrix(p, n, m, byrow=TRUE)
     if(missing(location)) stop("location is a required argument.")
     location <- as.vector(location)
     if(!identical(m, length(location)))
          stop("p and location differ in length.")
     if(missing(scale)) stop("scale is a required argument.")
     scale <- as.vector(scale)
     if(!identical(m, length(scale)))
          stop("p and scale differ in length.")
     if(any(scale <= 0)) stop("scale must be positive.")
     z <- rcat(n, p)
     x <- rlaplace(n, location=location[z], scale=scale[z])
     return(x)
     }

###########################################################################
# LASSO Distribution                                                      #
###########################################################################

dlasso <- function(x, sigma, tau, lambda, a=1, b=1, log=FALSE)
     {
     if(any(c(sigma, tau, lambda, a, b) <= 0))
          stop("Scale parameters must be positive.")
     dens <- sum(dmvn(x, 0, sigma^2*diag(tau^2), log=TRUE)) +
          log(1/sigma^2) + sum(dexp(tau^2, lambda^2/2, log=TRUE)) +
          dgamma(lambda^2, a, b, log=TRUE)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rlasso <- function(n, sigma, tau, lambda, a=1, b=1)
     {
     a <- as.vector(a)[1]
     b <- as.vector(b)[1]
     if(missing(sigma)) sigma <- runif(1, 1e-100, 1000)
     if(missing(lambda)) lambda <- sqrt(rgamma(1, a, b))
     if(missing(tau)) stop("The tau parameter is a required argument.")
     sigma <- as.vector(sigma)[1]
     lambda <- as.vector(lambda)[1]
     x <- rnorm(n, 0, sigma^2*diag(tau^2))
     return(x)
     }

###########################################################################
# Log-Laplace Distribution                                                #
###########################################################################

dllaplace <- function(x, location=0, scale=1, log=FALSE)
     {
     x <- as.vector(x); location <- as.vector(location)
     scale <- as.vector(scale)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     NN <- max(length(x), length(location), length(scale))
     x <- rep(x, len=NN); location <- rep(location, len=NN)
     scale <- rep(scale, len=NN)
     Alpha <- sqrt(2) * scale
     Beta  <- sqrt(2) / scale
     Delta <- exp(location)
     exponent <- -(Alpha + 1)
     temp <- which(x < Delta); exponent[temp] <- Beta[temp] - 1
     exponent <- exponent * (log(x) - location)
     dens <- -location + log(Alpha) + log(Beta) -
          log(Alpha + Beta) + exponent
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
pllaplace <- function(q, location=0, scale=1)
     {
     q <- as.vector(q); location <- as.vector(location)
     scale <- as.vector(scale)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     NN <- max(length(q), length(location), length(scale))
     q <- rep(q, len=NN); location <- rep(location, len=NN)
     scale <- rep(scale, len=NN)
     Alpha <- sqrt(2) * scale
     Beta  <- sqrt(2) / scale
     Delta <- exp(location)
     temp <- Alpha + Beta
     p <- (Alpha / temp) * (q / Delta)^(Beta)
     p[q <= 0] <- 0
     index1 <- (q >= Delta)
     p[index1] <- (1 - (Beta/temp) * (Delta/q)^(Alpha))[index1]
     return(p)
     }
qllaplace <- function(p, location=0, scale=1)
     {
     p <- as.vector(p); location <- as.vector(location)
     scale <- as.vector(scale)
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     NN <- max(length(p), length(location), length(scale))
     p <- rep(p, len=NN); location <- rep(location, len=NN)
     scale <- rep(scale, len=NN)
     Alpha <- sqrt(2) * scale
     Beta  <- sqrt(2) / scale
     Delta <- exp(location)
     temp <- Alpha + Beta
     q <- Delta * (p * temp / Alpha)^(1/Beta)
     index1 <- (p > Alpha / temp)
     q[index1] <- (Delta * ((1-p) * temp / Beta)^(-1/Alpha))[index1]
     q[p == 0] <- 0
     q[p == 1] <- Inf
     return(q)
     }
rllaplace <- function(n, location=0, scale=1)
     {
     location <- rep(location, len=n); scale <- rep(scale, len=n)
     if(any(scale <= 0)) stop("The scale parameter must be positive.")
     x <- exp(location) * (runif(n) / runif(n))^(scale / sqrt(2))
     return(x)
     }

###########################################################################
# Log-Normal Distribution (Precision Parameterization)                    #
###########################################################################

dlnormp <- function(x, mu, tau = NULL, var = NULL, log = FALSE)
{
	if (!is.null(tau) & !is.null(var))
		stop("use either tau or var, but not both")

	x <- as.vector(x); mu <- as.vector(mu);
	if (!is.null(tau))
	{
		tau <- as.vector(tau)
		if(any(tau <= 0))
			stop("The tau parameter must be positive.")

		NN <- max(length(x), length(mu), length(tau))
		x <- rep(x, len=NN); mu <- rep(mu, len=NN); tau <- rep(tau, len=NN)
		dens <- 0.5*(log(tau) - log(2*pi)) - log(x) - tau/2*(log(x) - mu)^2
	}

	if (!is.null(var))
	{
		var <- as.vector(var)
		if(any(var <= 0))
			stop("The var parameter must be positive.")

		NN <- max(length(x), length(mu), length(var))
		x <- rep(x, len=NN); mu <- rep(mu, len=NN); var <- rep(var, len=NN)
		dens <- -0.5*log(2*pi*var) - log(x) - (log(x) - mu)^2/(2*var)
	}

	if(log == FALSE)
		dens <- exp(dens)

	return(dens)
}
plnormp <- function(q, mu, tau, lower.tail=TRUE, log.p=FALSE)
     {
     q <- as.vector(q); mu <- as.vector(mu); tau <- as.vector(tau)
     if(any(tau <= 0)) stop("The tau parameter must be positive.")
     NN <- max(length(q), length(mu), length(tau))
     q <- rep(q, len=NN); mu <- rep(mu, len=NN); tau <- rep(tau, len=NN)
     p <- pnorm(q, mu, sqrt(1/tau), lower.tail, log.p)
     return(p)
     }
qlnormp <- function(p, mu, tau, lower.tail=TRUE, log.p=FALSE)
     {
     p <- as.vector(p); mu <- as.vector(mu); tau <- as.vector(tau)
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(tau <= 0)) stop("The tau parameter must be positive.")
     NN <- max(length(p), length(mu), length(tau))
     p <- rep(p, len=NN); mu <- rep(mu, len=NN); tau <- rep(tau, len=NN)
     q <- qnorm(p, mu, sqrt(1/tau), lower.tail, log.p)
     return(q)
}

rlnormp <- function(n, mu, tau = NULL, var = NULL)
{
	if (!is.null(tau) & !is.null(var))
		stop("use either tau or var, but not both")

	mu <- rep(mu, len=n);
	if (!is.null(tau))
	{
		if(any(tau <= 0))
			stop("The tau parameter must be positive.")

		tau <- rep(tau, len=n)
		x <- rlnorm(n, log(mu^2 / sqrt(1/tau^2 + mu^2)), sqrt(log(1 + 1/(mu^2*tau^2))))
	}
	if (!is.null(var))
	{
		var = rep(var, len=n)
		if(any(var <= 0))
			stop("The var parameter must be positive.")

		var <- rep(var, len=n)
		x <- rlnorm(n, log(mu^2 / sqrt(var^2 + mu^2)), sqrt(log(1 + (var^2/mu^2))))
	}
	return(x)
}

###########################################################################
# Matrix Gamma Distribution                                               #
###########################################################################

dmatrixgamma <- function(X, alpha, beta, Sigma, log=FALSE)
     {
     if(!is.matrix(X)) stop("X is not a matrix.")
     if(alpha <= 2) stop("The alpha parameter must be greater than 2.")
     if(beta <= 0) stop("The beta parameter must be positive.")
     if(missing(Sigma)) Sigma <- diag(ncol(X))
     if(!is.positive.definite(Sigma))
          stop("Matrix Sigma is not positive-definite.")
     k <- nrow(Sigma)
     gamsum <- 0
     for (i in 1:k) gamsum <- gamsum + lgamma(alpha - 0.5*(i-1))
     gamsum <- gamsum + log(pi)*(k*(k-1)/4)
     Omega <- as.inverse(Sigma)
     dens <- alpha*logdet(Omega) + logdet(X)*(alpha - 0.5*(k+1)) -
          (log(beta)*(k*alpha) + gamsum) + (-1/beta)*tr(Omega %*% X)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }

rmatrixgamma <- function(alpha, beta, Sigma){
  rwishart(2*alpha, beta/2*Sigma)
}
###########################################################################
# Matrix Normal Distribution                                              #
###########################################################################

dmatrixnorm <- function(X, M, U, V, log=FALSE)
     {
     if(!is.matrix(X)) X <- rbind(X)
     if(!is.matrix(M)) M <- matrix(M, nrow(X), ncol(X), byrow=TRUE)
     if(missing(U)) U <- diag(nrow(X))
     if(!is.matrix(U)) U <- matrix(U)
     if(!is.positive.definite(U))
          stop("Matrix U is not positive-definite.")
     if(missing(V)) V <- diag(ncol(X))
     if(!is.matrix(V)) V <- matrix(V)
     if(!is.positive.definite(V))
          stop("Matrix V is not positive-definite.")
     n <- nrow(X)
     k <- ncol(X)
     ss <- X - M
     dens <- -0.5*tr(as.inverse(V) %*% t(ss) %*%
          as.inverse(U) %*% ss) - 
       (log(2*pi)*(n*k/2) - logdet(V) * (n/2) - logdet(U) * (k/2))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmatrixnorm <- function(M, U, V)
     {
     if(missing(M)) stop("Matrix M is missing.")
     if(!is.matrix(M)) M <- matrix(M)
     if(missing(U)) stop("Matrix U is missing.")
     if(!is.matrix(U)) U <- matrix(U)
     if(!is.positive.definite(U))
          stop("Matrix U is not positive-definite.")
     if(missing(V)) stop("Matrix V is missing.")
     if(!is.matrix(V)) V <- matrix(V)
     if(!is.positive.definite(V))
          stop("Matrix V is not positive-definite.")
     if(nrow(M) != nrow(U))
          stop("Dimensions of M and U are incorrect.")
     if(ncol(M) != ncol(V))
          stop("Dimensions of M and V are incorrect.")
     n <- nrow(U)
     k <- ncol(V)
     Z <- matrix(rnorm(n * k), n, k)
     X <- M + t(chol(U)) %*% Z %*% chol(V)
     return(X)
     }

###########################################################################
# Multivariate Cauchy Distribution                                        #
###########################################################################

dmvc <- function(x, mu, S, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(S)) S <- diag(ncol(x))
     if(!is.matrix(S)) S <- matrix(S)
     if(!is.positive.definite(S))
          stop("Matrix S is not positive-definite.")
     k <- nrow(S)
     ss <- x - mu
     Omega <- as.inverse(S)
     z <- rowSums({ss %*% Omega} * ss)
     dens <- as.vector(lgamma((1 + k)/2) - (lgamma(0.5) +
          (k/2)*log(pi) + 0.5*logdet(S) + ((1+k)/2)*log(1+z)))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvc <- function(n=1, mu=rep(0,k), S)
     {
     mu <- rbind(mu)
     if(missing(S)) S <- diag(ncol(mu))
     if(!is.matrix(S)) S <- matrix(S)
     if(!is.positive.definite(S))
          stop("Matrix S is not positive-definite.")
     k <- ncol(S)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     x <- rchisq(n,1)
     x[which(x == 0)] <- 1e-100
     z <- rmvn(n, rep(0,k), S)
     x <- mu + z/sqrt(x)
     return(x)
     }

###########################################################################
# Multivariate Cauchy Distribution (Cholesky Parameterization)            #
###########################################################################

dmvcc <- function(x, mu, U, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(U)) stop("Upper triangular U is required.")
     k <- nrow(U)
     S <- t(U) %*% U
     ss <- x - mu
     Omega <- as.inverse(S)
     z <- rowSums({ss %*% Omega} * ss)
     dens <- as.vector(lgamma(k/2) - (lgamma(0.5) + log(1^(k/2)) +
          (k/2)*log(pi) + 0.5*logdet(S) + ((1+k)/2)*log(1+z)))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvcc <- function(n=1, mu=rep(0,k), U)
     {
     mu <- rbind(mu)
     if(missing(U)) stop("Upper triangular U is required.")
     k <- ncol(U)
     S <- t(U) %*% U
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     x <- rchisq(n,1)
     x[which(x == 0)] <- 1e-100
     z <- rmvnc(n, rep(0,k), U)
     x <- mu + z/sqrt(x)
     return(x)
     }

###########################################################################
# Multivariate Cauchy Distribution (Precision Parameterization)           #
###########################################################################

dmvcp <- function(x, mu, Omega, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(Omega)) Omega <- diag(ncol(x))
     if(!is.matrix(Omega)) Omega <- matrix(Omega)
     if(!is.positive.definite(Omega))
          stop("Matrix Omega is not positive-definite.")
     k <- nrow(Omega)
     logdetOmega <- logdet(Omega)
     ss <- x - mu
     z <- rowSums({ss %*% Omega} * ss)
     dens <- as.vector(lgamma((1+k)/2) - (lgamma(0.5) + log(1^(k/2)) +
          (k/2)*log(pi)) + 0.5*logdetOmega + (-(1+k)/2)*log(1 + z))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvcp <- function(n=1, mu, Omega)
     {
     mu <- rbind(mu)
     if(missing(Omega)) Omega <- diag(ncol(mu))
     if(!is.matrix(Omega)) Omega <- matrix(Omega)
     if(!is.positive.definite(Omega))
          stop("Matrix Omega is not positive-definite.")
     Sigma <- as.inverse(Omega)
     k <- ncol(Sigma)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     x <- rchisq(n,1)
     x[which(x == 0)] <- 1e-100
     z <- rmvn(n, rep(0,k), Sigma)
     x <- mu + z/sqrt(x)
     return(x)
     }

###########################################################################
# Multivariate Cauchy Distribution (Precision-Cholesky Parameterization)  #
###########################################################################

dmvcpc <- function(x, mu, U, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(U)) stop("Upper triangular U is required.")
     k <- nrow(U)
     Omega <- t(U) %*% U
     logdetOmega <- logdet(Omega)
     ss <- x - mu
     z <- rowSums({ss %*% Omega} * ss)
     dens <- as.vector(lgamma((1+k)/2) - (lgamma(0.5) + log(1^(k/2)) +
          (k/2)*log(pi)) + 0.5*logdetOmega + (-(1+k)/2)*log(1 + z))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvcpc <- function(n=1, mu, U)
     {
     mu <- rbind(mu)
     if(missing(U)) stop("Upper triangular U is required.")
     k <- ncol(U)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     x <- rchisq(n,1)
     x[which(x == 0)] <- 1e-100
     z <- rmvnc(n, rep(0,k), U)
     x <- mu + z/sqrt(x)
     return(x)
     }

###########################################################################
# Multivariate Laplace Distribution                                       #
###########################################################################

dmvl <- function(x, mu, Sigma, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(Sigma)) Sigma <- diag(ncol(x))
     if(!is.matrix(Sigma)) Sigma <- matrix(Sigma)
     Sigma <- as.symmetric.matrix(Sigma)
     if(!is.positive.definite(Sigma))
         stop("Matrix Sigma is not positive-definite.")
     k <- nrow(Sigma)
     Omega <- as.inverse(Sigma)
     ss <- x - mu
     z <- rowSums({ss %*% Omega} * ss)
     z[which(z == 0)] <- 1e-300
     dens <- as.vector(log(2) - log(2 * pi) * (k/2) - logdet(Sigma) * 0.5 + 
                      (log(pi) - log(2) - log(2 * z) * 0.5) * 0.5 - 
                      sqrt(2 * z) - log(z/2) * 0.5 * (k/2 - 1))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvl <- function(n, mu, Sigma)
     {
     mu <- rbind(mu)
     if(missing(Sigma)) Sigma <- diag(ncol(mu))
     if(!is.matrix(Sigma)) Sigma <- matrix(Sigma)
     if(!is.positive.definite(Sigma))
          stop("Matrix Sigma is not positive-definite.")
     k <- ncol(Sigma)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     e <- matrix(rexp(n, 1), n, k)
     z <- rmvn(n, rep(0, k), Sigma)
     x <- mu + sqrt(e)*z
     return(x)
     }

###########################################################################
# Multivariate Laplace Distribution (Cholesky Parameterization)           #
###########################################################################

dmvlc <- function(x, mu, U, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(U)) stop("Upper triangular U is required.")
     k <- ncol(U)
     Sigma <- t(U) %*% U
     Omega <- as.inverse(Sigma)
     ss <- x - mu
     z <- rowSums({ss %*% Omega} * ss)
     z[which(z == 0)] <- 1e-300
     dens <- as.vector(log(2) - log(2*pi)*(k/2) + logdet(Sigma)*0.5 +
          (log(pi) - log(2) + log(2*z)*0.5)*0.5 - log(2*z)*0.5 -
          log(z/2)*0.5*(k/2 - 1))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvlc <- function(n, mu, U)
     {
     mu <- rbind(mu)
     if(missing(U)) stop("Upper triangular U is required.")
     k <- ncol(U)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     e <- matrix(rexp(n, 1), n, k)
     z <- rmvnc(n, rep(0, k), U)
     x <- mu + sqrt(e)*z
     return(x)
     }

###########################################################################
# Multivariate Normal Distribution                                        #
###########################################################################

dmvn <- function(x, mu, Sigma, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(Sigma)) Sigma <- diag(ncol(x))
     if(!is.matrix(Sigma)) Sigma <- matrix(Sigma)
     Sigma <- as.symmetric.matrix(Sigma)
     if(!is.positive.definite(Sigma))
          stop("Matrix Sigma is not positive-definite.")
     k <- nrow(Sigma)
     Omega <- as.inverse(Sigma)
     ss <- x - mu
     z <- rowSums({ss %*% Omega} * ss)
     dens <- as.vector(-0.5 * (k * log(2 * pi) + logdet(Sigma)) - (0.5 * z))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvn <- function(n=1, mu=rep(0,k), Sigma)
     {
     mu <- rbind(mu)
     if(missing(Sigma)) Sigma <- diag(ncol(mu))
     if(!is.matrix(Sigma)) Sigma <- matrix(Sigma)
     if(!is.positive.definite(Sigma))
          stop("Matrix Sigma is not positive-definite.")
     k <- ncol(Sigma)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     z <- matrix(rnorm(n*k),n,k) %*% chol(Sigma)
     x <- mu + z
     return(x)
     }

###########################################################################
# Multivariate Normal Distribution (Cholesky Parameterization)            #
###########################################################################

dmvnc <- function(x, mu, U, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(U)) stop("Upper triangular U is required.")
     k <- ncol(U)
     Sigma <- t(U) %*% U
     k <- nrow(Sigma)
     Omega <- as.inverse(Sigma)
     ss <- x - mu
     z <- rowSums({ss %*% Omega} * ss)
     dens <- as.vector(-0.5 * (k * log(2 * pi) + logdet(Sigma)) - (0.5 * z))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvnc <- function(n=1, mu=rep(0,k), U)
     {
     mu <- rbind(mu)
     if(missing(U)) stop("Upper triangular U is required.")
     k <- ncol(U)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     z <- matrix(rnorm(n*k),n,k) %*% U
     x <- mu + z
     return(x)
     }

###########################################################################
# Multivariate Normal Distribution (Precision Parameterization)           #
###########################################################################

dmvnp <- function(x, mu, Omega, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(Omega)) Omega <- diag(ncol(x))
     if(!is.matrix(Omega)) Omega <- matrix(Omega)
     if(!is.positive.definite(Omega))
          stop("Matrix Omega is not positive-definite.")
     k <- nrow(Omega)
     logdetOmega <- logdet(Omega)
     ss <- x - mu
     z <- rowSums({ss %*% Omega} * ss)
     dens <- as.vector((-k/2)*log(2*pi) + 0.5*logdetOmega - 0.5*z)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvnp <- function(n=1, mu=rep(0, k), Omega)
     {
     mu <- rbind(mu)
     if(missing(Omega)) Omega <- diag(ncol(mu))
     if(!is.matrix(Omega)) Omega <- matrix(Omega)
     if(!is.positive.definite(Omega))
          stop("Matrix Omega is not positive-definite.")
     k <- ncol(Omega)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     z <- matrix(rnorm(n*k),n,k) %*% solve(t(chol(Omega)))
     x <- mu + z
     return(x)
     }

###########################################################################
# Multivariate Normal Distribution (Precision-Cholesky Parameterization)  #
###########################################################################

dmvnpc <- function(x, mu, U, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(U)) stop("Upper triangular U is required.")
     k <- ncol(U)
     Omega <- t(U) %*% U
     logdetOmega <- logdet(Omega)
     ss <- x - mu
     z <- rowSums({ss %*% Omega} * ss)
     dens <- as.vector((-k/2)*log(2*pi) + 0.5*logdetOmega - 0.5*z)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvnpc <- function(n=1, mu=rep(0,k), U)
     {
     mu <- rbind(mu)
     if(missing(U)) stop("Upper triangular U is required.")
     Sigma <- as.inverse(t(U) %*% U)
     k <- ncol(Sigma)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     z <- matrix(rnorm(n*k),n,k) %*% chol(Sigma)
     x <- mu + z
     return(x)
     }

###########################################################################
# Multivariate Polya Distribution                                         #
###########################################################################

dmvpolya <- function(x, alpha, log=FALSE)
     {
     x <- as.vector(x)
     alpha <- as.vector(alpha)
     if(!identical(length(x), length(alpha)))
          stop("x and alpha differ in length.")
     dens <- (log(factorial(sum(x))) - sum(log(factorial(x)))) +
          (log(factorial(sum(alpha)-1)) -
          log(factorial(sum(x) + sum(alpha)-1))) +
          (sum(log(factorial(x + alpha - 1)) - log(factorial(alpha - 1))))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvpolya <- function(n=1, alpha)
     {
     p <- rdirichlet(n, alpha)
     x <- rcat(n,p)
     return(x)
     }

###########################################################################
# Multivariate Power Exponential Distribution                             #
###########################################################################

dmvpe <- function(x=c(0,0), mu=c(0,0), Sigma=diag(2), kappa=1, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(Sigma)) Sigma <- diag(ncol(x))
     if(!is.matrix(Sigma)) Sigma <- matrix(Sigma)
     if(!is.positive.definite(Sigma))
          stop("Matrix Sigma is not positive-definite.")
     if(any(kappa <= 0)) stop("The kappa parameter must be positive.")
     k <- nrow(Sigma)
     Omega <- as.inverse(Sigma)
     ss <- x - mu
     temp <- rowSums({ss %*% Omega} * ss)
     dens <- as.vector(((log(k)+lgamma(k/2)) - ((k/2)*log(pi) +
          0.5*logdet(Sigma) + lgamma(1 + k/(2*kappa)) +
          (1 + k/(2*kappa))*log(2))) + kappa*(-0.5*temp))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvpe <- function(n, mu=c(0,0), Sigma=diag(2), kappa=1)
     {
     mu <- rbind(mu)
     k <- ncol(mu)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     if(k != nrow(Sigma)) {
          stop("mu and Sigma have non-conforming size.")}
     ev <- eigen(Sigma, symmetric=TRUE)
     if(!all(ev$values >= -sqrt(.Machine$double.eps) *
          abs(ev$values[1]))) {
          stop("Sigma must be positive-definite.")}
     SigmaSqrt <- ev$vectors %*% diag(sqrt(ev$values),
          length(ev$values)) %*% t(ev$vectors)
     radius <- (rgamma(n, shape=k/(2*kappa), scale=2))^(1/(2*kappa))
     runifsphere <- function(n, k)
          {
          p <- as.integer(k)
          if(!is.integer(k))
               stop("k must be an integer in [2,Inf)")
          if(k < 2) stop("k must be an integer in [2,Inf).")
          Mnormal <- matrix(rnorm(n*k,0,1), nrow=n)
          rownorms <- sqrt(rowSums(Mnormal^2))
          unifsphere <- sweep(Mnormal,1,rownorms, "/")
          return(unifsphere)
          }
     un <- runifsphere(n=n, k=k)
     x <- mu + radius * un %*% SigmaSqrt
     return(x)
     }

###########################################################################
# Multivariate Power Exponential Distribution (Cholesky Parameterization) #
###########################################################################

dmvpec <- function(x=c(0,0), mu=c(0,0), U, kappa=1, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(U)) stop("Upper triangular U is required.")
     if(any(kappa <= 0)) stop("The kappa parameter must be positive.")
     Sigma <- t(U) %*% U
     k <- nrow(Sigma)
     Omega <- as.inverse(Sigma)
     ss <- x - mu
     temp <- rowSums({ss %*% Omega} * ss)
     dens <- as.vector(((log(k)+lgamma(k/2)) - ((k/2)*log(pi) +
          0.5*logdet(Sigma) + lgamma(1 + k/(2*kappa)) +
          (1 + k/(2*kappa))*log(2))) + kappa*(-0.5*temp))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvpec <- function(n, mu=c(0,0), U, kappa=1)
     {
     mu <- rbind(mu)
     k <- ncol(mu)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     if(k != nrow(U)) {
          stop("mu and U have non-conforming size.")}
     Sigma <- t(U) %*% U
     ev <- eigen(Sigma, symmetric=TRUE)
     if(!all(ev$values >= -sqrt(.Machine$double.eps) *
          abs(ev$values[1]))) {
          stop("Sigma must be positive-definite.")}
     SigmaSqrt <- ev$vectors %*% diag(sqrt(ev$values),
          length(ev$values)) %*% t(ev$vectors)
     radius <- (rgamma(n, shape=k/(2*kappa), scale=1/2))^(1/(2*kappa))
     runifsphere <- function(n, k)
          {
          p <- as.integer(k)
          if(!is.integer(k))
               stop("k must be an integer in [2,Inf)")
          if(k < 2) stop("k must be an integer in [2,Inf).")
          Mnormal <- matrix(rnorm(n*k,0,1), nrow=n)
          rownorms <- sqrt(rowSums(Mnormal^2))
          unifsphere <- sweep(Mnormal,1,rownorms, "/")
          return(unifsphere)
          }
     un <- runifsphere(n=n, k=k)
     x <- mu + radius * un %*% SigmaSqrt
     return(x)
     }

###########################################################################
# Multivariate t Distribution                                             #
###########################################################################

dmvt <- function(x, mu, S, df=Inf, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(S)) S <- diag(ncol(x))
     if(!is.matrix(S)) S <- matrix(S)
     if(!is.positive.definite(S))
          stop("Matrix S is not positive-definite.")
     if(any(df <= 0)) stop("The df parameter must be positive.")
     if(any(df > 10000)) return(dmvn(x, mu, S, log))
     k <- nrow(S)
     ss <- x - mu
     Omega <- as.inverse(S)
     z <- rowSums({ss %*% Omega} * ss)
     dens <- as.vector(lgamma((df+k)/2) - lgamma(df/2) - (k/2)*log(df) -
          (k/2)*log(pi) - 0.5*logdet(S) - ((df+k)/2)*log(1 + (1/df) * z))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvt <- function(n=1, mu=rep(0,k), S, df=Inf)
     {
     mu <- rbind(mu)
     if(missing(S)) S <- diag(ncol(mu))
     if(!is.matrix(S)) S <- matrix(S)
     if(!is.positive.definite(S))
          stop("Matrix S is not positive-definite.")
     if(any(df <= 0)) stop("The df parameter must be positive.")
     k <- ncol(S)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     if(df==Inf) x <- 1 else x <- rchisq(n,df) / df
     x[which(x == 0)] <- 1e-100
     z <- rmvn(n, rep(0,k), S)
     x <- mu + z/sqrt(x)
     return(x)
     }

###########################################################################
# Multivariate t Distribution (Cholesky Parameterization)                 #
###########################################################################

dmvtc <- function(x, mu, U, df=Inf, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(U)) stop("Upper triangular U is required.")
     if(any(df <= 0)) stop("The df parameter must be positive.")
     if(any(df > 10000)) return(dmvnc(x, mu, U, log))
     k <- nrow(U)
     ss <- x - mu
     S <- t(U) %*% U
     Omega <- as.inverse(S)
     z <- rowSums({ss %*% Omega} * ss)
     dens <- as.vector(lgamma((df+k)/2) - lgamma(df/2) + (k/2)*df +
          (k/2)*log(pi) + 0.5*logdet(S) + ((df+k)/2)*log(1 + (1/df) * z))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvtc <- function(n=1, mu=rep(0,k), U, df=Inf)
     {
     mu <- rbind(mu)
     if(missing(U)) stop("Upper triangular U is required.")
     if(any(df <= 0)) stop("The df parameter must be positive.")
     k <- ncol(U)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     if(df==Inf) x <- 1 else x <- rchisq(n,df) / df
     x[which(x == 0)] <- 1e-100
     z <- rmvnc(n, rep(0,k), U)
     x <- mu + z/sqrt(x)
     return(x)
     }

###########################################################################
# Multivariate t Distribution (Precision Parameterization)                #
###########################################################################

dmvtp <- function(x, mu, Omega, nu=Inf, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(Omega)) Omega <- diag(ncol(x))
     if(!is.matrix(Omega)) Omega <- matrix(Omega)
     if(!is.positive.definite(Omega))
          stop("Matrix Omega is not positive-definite.")
     if(any(nu <= 0)) stop("The nu parameter must be positive.")
     if(any(nu > 10000)) return(dmvnp(x, mu, Omega, log))
     k <- ncol(Omega)
     logdetOmega <- logdet(Omega)
     ss <- x - mu
     z <- rowSums({ss %*% Omega} * ss)
     dens <- as.vector(lgamma((nu+k)/2) - (lgamma(nu/2) + (k/2)*log(nu) +
          (k/2)*log(pi)) + 0.5*logdetOmega +
          (-(nu+k)/2)*log(1 + (1/nu) * z))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvtp <- function(n=1, mu, Omega, nu=Inf)
     {
     mu <- rbind(mu)
     if(missing(Omega)) Omega <- diag(ncol(mu))
     if(!is.matrix(Omega)) Omega <- matrix(Omega)
     if(!is.positive.definite(Omega))
          stop("Matrix Omega is not positive-definite.")
     if(any(nu <= 0)) stop("The nu parameter must be positive.")
     Sigma <- as.inverse(Omega)
     k <- ncol(Sigma)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     if(nu == Inf) x <- 1 else x <- rchisq(n,nu) / nu
     x[which(x == 0)] <- 1e-100
     z <- rmvn(n, rep(0,k), Sigma)
     x <- mu + z/sqrt(x)
     return(x)
     }

###########################################################################
# Multivariate t Distribution (Precision-Cholesky Parameterization)       #
###########################################################################

dmvtpc <- function(x, mu, U, nu=Inf, log=FALSE)
     {
     if(!is.matrix(x)) x <- rbind(x)
     if(!is.matrix(mu)) mu <- matrix(mu, nrow(x), ncol(x), byrow=TRUE)
     if(missing(U)) stop("Upper triangular U is required.")
     if(any(nu <= 0)) stop("The nu parameter must be positive.")
     if(any(nu > 10000)) return(dmvnpc(x, mu, U, log))
     k <- ncol(U)
     Omega <- t(U) %*% U
     logdetOmega <- logdet(Omega)
     ss <- x - mu
     z <- rowSums({ss %*% Omega} * ss)
     dens <- as.vector(lgamma((nu+k)/2) - (lgamma(nu/2) + (k/2)*log(nu) +
          (k/2)*log(pi)) + 0.5*logdetOmega +
          (-(nu+k)/2)*log(1 + (1/nu) * z))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rmvtpc <- function(n=1, mu, U, nu=Inf)
     {
     mu <- rbind(mu)
     if(missing(U)) stop("Upper triangular U is required.")
     if(any(nu <= 0)) stop("The nu parameter must be positive.")
     k <- ncol(U)
     if(n > nrow(mu)) mu <- matrix(mu, n, k, byrow=TRUE)
     if(nu == Inf) x <- 1 else x <- rchisq(n,nu) / nu
     x[which(x == 0)] <- 1e-100
     z <- rmvnpc(n, rep(0,k), U)
     x <- mu + z/sqrt(x)
     return(x)
     }

###########################################################################
# Normal Distribution Mixture                                             #
###########################################################################

dnormm <- function(x, p, mu, sigma, log=FALSE)
     {
     if(missing(x)) stop("x is a required argument.")
     x <- as.vector(x)
     n <- length(x)
     if(missing(p)) stop("p is a required argument.")
     p <- as.vector(p)
     if(any(p <= 0) | any(p > 1)) stop("p must be in (0,1].")
     if(sum(p) != 1) stop("p must sum to 1 for all components.")
     m <- length(p)
     p <- matrix(p, n, m, byrow=TRUE)
     if(missing(mu)) stop("mu is a required argument.")
     mu <- as.vector(mu)
     if(!identical(m, length(mu)))
          stop("p and mu differ in length.")
     mu <- matrix(mu, n, m, byrow=TRUE)
     if(missing(sigma)) stop("sigma is a required argument.")
     sigma <- as.vector(sigma)
     if(!identical(m, length(sigma)))
          stop("p and sigma differ in length.")
     sigma <- matrix(sigma, n, m, byrow=TRUE)
     dens <- matrix(dnorm(x, mu, sigma, log=TRUE), n, m)
     dens <- dens + log(p)
     if(log == TRUE) dens <- apply(dens, 1, logadd)
     else dens <- rowSums(exp(dens))
     return(dens)
     }
pnormm <- function(q, p, mu, sigma, lower.tail=TRUE, log.p=FALSE)
     {
     n <- length(q)
     m <- length(p)
     q <- matrix(q, n, m)
     p <- matrix(p, n, m, byrow=TRUE)
     mu <- matrix(mu, n, m, byrow=TRUE)
     sigma <- matrix(sigma, n, m, byrow=TRUE)
     cdf <- matrix(pnorm(q, mu, sigma, lower.tail=lower.tail,
          log.p=log.p), n, m)
     if(log.p == FALSE) cdf <- rowSums(cdf * p)
     else stop("The log.p argument does not work yet.")
     return(cdf)
     }
rnormm <- function(n, p, mu, sigma)
     {
     if(missing(p)) stop("p is a required argument.")
     p <- as.vector(p)
     if(any(p <= 0) | any(p > 1)) stop("p must be in (0,1].")
     if(sum(p) != 1) stop("p must sum to 1 for all components.")
     m <- length(p)
     p <- matrix(p, n, m, byrow=TRUE)
     if(missing(mu)) stop("mu is a required argument.")
     mu <- as.vector(mu)
     if(!identical(m, length(mu)))
          stop("p and mu differ in length.")
     if(missing(sigma)) stop("sigma is a required argument.")
     sigma <- as.vector(sigma)
     if(!identical(m, length(sigma)))
          stop("p and sigma differ in length.")
     if(any(sigma <= 0)) stop("sigma must be positive.")
     z <- rcat(n, p)
     x <- rnorm(n, mean=mu[z], sd=sigma[z])
     return(x)
     }

###########################################################################
# Normal Distribution (Precision Parameterization)                        #
###########################################################################

dnormp <- function(x, mean=0, prec=1, log=FALSE)
     {
     #dens <- sqrt(prec/(2*pi)) * exp(-(prec/2)*(x-mu)^2)
     dens <- dnorm(x, mean, sqrt(1/prec), log)
     return(dens)
     }
pnormp <- function(q, mean=0, prec=1, lower.tail=TRUE, log.p=FALSE)
     {return(pnorm(q, mean=mean, sd=sqrt(1/prec), lower.tail, log.p))}
qnormp <- function(p, mean=0, prec=1, lower.tail=TRUE, log.p=FALSE)
     {return(qnorm(p, mean=mean, sd=sqrt(1/prec), lower.tail, log.p))}
rnormp <- function(n, mean=0, prec=1)
     {return(rnorm(n, mean=mean, sd=sqrt(1/prec)))}

###########################################################################
# Normal Distribution (Variance Parameterization)                         #
###########################################################################

dnormv <- function(x, mean=0, var=1, log=FALSE)
     {
     #dens <- (1/(sqrt(2*pi*var))) * exp(-((x-mu)^2/(2*var)))
     dens <- dnorm(x, mean, sqrt(var), log)
     return(dens)
     }
pnormv <- function(q, mean=0, var=1, lower.tail=TRUE, log.p=FALSE)
     {return(pnorm(q, mean=mean, sd=sqrt(var), lower.tail, log.p))}
qnormv <- function(p, mean=0, var=1, lower.tail=TRUE, log.p=FALSE)
     {return(qnorm(p, mean=mean, sd=sqrt(var), lower.tail, log.p))}
rnormv <- function(n, mean=0, var=1)
     {return(rnorm(n, mean=mean, sd=sqrt(var)))}

###########################################################################
# Normal-Inverse-Wishart Distribution                                     #
###########################################################################

dnorminvwishart <- function(mu, mu0, lambda, Sigma, S, nu, log=FALSE)
     {
     dens <- dinvwishart(Sigma, nu, S, log=TRUE) +
          dmvn(mu, mu0, 1/lambda*Sigma, log=TRUE)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rnorminvwishart <- function(n=1, mu0, lambda, S, nu)
     {
     Sigma <- rinvwishart(nu, S)
     mu <- rmvn(n, mu0, 1/lambda*Sigma)
     return(list(mu=mu, Sigma=Sigma))
     }

###########################################################################
# Normal-Laplace Distribution                                             #
###########################################################################

dnormlaplace <- function(x, mu=0, sigma=1, alpha=1, beta=1, log=FALSE)
     {
     x <- as.vector(x)
     mu <- as.vector(mu)
     sigma <- as.vector(sigma)
     alpha <- as.vector(alpha)
     beta <- as.vector(beta)
     if(any(sigma <= 0))
          stop("The sigma parameter must be positive.")
     if(any(alpha <= 0))
          stop("The alpha parameter must be positive.")
     if(any(beta <= 0))
          stop("The beta parameter must be positive.")
    NN <- max(length(x), length(mu), length(sigma), length(alpha),
          length(beta))
    x <- rep(x, len=NN)
    mu <- rep(mu, len=NN)
    sigma <- rep(sigma, len=NN)
    alpha <- rep(alpha, len=NN)
    beta <- rep(beta, len=NN)
     a <- dnorm((x - mu) / sigma, log=TRUE)
     z <- alpha*sigma - (x - mu) / sigma
     b <- pnorm(z, lower.tail=FALSE, log.p=TRUE) -
          dnorm(z, log=TRUE)
     z <- beta*sigma + (x - mu) / sigma
     cc <- pnorm(z, lower.tail=FALSE, log.p=TRUE) -
          dnorm(z, log=TRUE)
     z <- log(alpha*beta / (alpha + beta))
     d <- pnorm(z, lower.tail=FALSE, log.p=TRUE) -
          dnorm(z, log=TRUE)
     if(log == FALSE) dens <- exp(d + a + b) + exp(d + a + cc)
     else {
          dens <- rep(NA, NN)
          e <- d + a + b
          f <- d + a + cc
          for (i in 1:NN) dens[i] <- logadd(c(e[i], f[i]))
      }
     return(dens)
     }
rnormlaplace <- function(n, mu=0, sigma=1, alpha=1, beta=1)
     {
     z <- rnorm(n, 0, sigma^2)
     w <- rslaplace(n, mu, 1/beta, 1/alpha)
     return(z + w)
     }

###########################################################################
# Normal-Wishart Distribution                                             #
###########################################################################

dnormwishart <- function(mu, mu0, lambda, Omega, S, nu, log=FALSE)
     {
     dens <- dwishart(Omega, nu, S, log=TRUE) +
          dmvnp(mu, mu0, lambda*Omega, log=TRUE)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rnormwishart <- function(n=1, mu0, lambda, S, nu)
     {
     Omega <- rwishart(nu, S)
     mu <- rmvn(n, mu0, as.inverse(lambda*Omega))
     return(list(mu=mu, Omega=Omega))
     }

###########################################################################
# Pareto Distribution                                                     #
###########################################################################

dpareto <- function(x, alpha, log=FALSE)
     {
     x <- as.vector(x); alpha <- as.vector(alpha)
     if(any(alpha <= 0)) stop("The alpha parameter must be positive.")
     NN <- max(length(x), length(alpha))
     x <- dens <- rep(x, len=NN); alpha <- rep(alpha, len=NN)
     dens[which(x < 1)] <- -Inf
     temp <- which(x >= 1)
     dens[temp] <- log(alpha[temp]) - (alpha[temp] + 1)*log(x[temp])
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
ppareto <- function(q, alpha)
     {
     q <- as.vector(q); alpha <- as.vector(alpha)
     if(any(alpha <= 0)) stop("The alpha parameter must be positive.")
     NN <- max(length(q), length(alpha))
     p <- q <- rep(q, len=NN); alpha <- rep(alpha, len=NN)
     p[which(q < 1)] <- 0
     temp <- which(q >= 1)
     p[temp] <- 1 - 1 / q[temp]^alpha[temp]
     return(p)
     }
qpareto <- function(p, alpha)
     {
     p <- as.vector(p); alpha <- as.vector(alpha)
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(alpha <= 0)) stop("The alpha parameter must be positive.")
     NN <- max(length(p), length(alpha))
     p <- rep(p, len=NN); alpha <- rep(alpha, len=NN)
     q <- (1-p)^(-1/alpha)
     return(q)
     }
rpareto <- function(n, alpha)
     {
     alpha <- rep(alpha, len=n)
     if(any(alpha <= 0)) stop("The alpha parameter must be positive.")
     x <- runif(n)^(-1/alpha)
     return(x)
     }

###########################################################################
# Power Exponential Distribution                                          #
#                                                                         #
# These functions are similar to those in the normalp package.            #
###########################################################################

dpe <- function(x, mu=0, sigma=1, kappa=2, log=FALSE)
     {
     x <- as.vector(x); mu <- as.vector(mu); sigma <- as.vector(sigma)
     kappa <- as.vector(kappa)
     if(any(sigma <= 0)) stop("The sigma parameter must be positive.")
     if(any(kappa <= 0)) stop("The kappa parameter must be positive.")
     NN <- max(length(x), length(mu), length(sigma), length(kappa))
     x <- rep(x, len=NN); mu <- rep(mu, len=NN)
     sigma <- rep(sigma, len=NN); kappa <- rep(kappa, len=NN)
     cost <- 2 * kappa^(1/kappa) * gamma(1 + 1/kappa) * sigma
     expon1 <- (abs(x - mu))^kappa
     expon2 <- kappa * sigma^kappa
     dens <- log(1/cost) + (-expon1 / expon2)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
ppe <- function(q, mu=0, sigma=1, kappa=2, lower.tail=TRUE, log.p=FALSE)
     {
     q <- as.vector(q); mu <- as.vector(mu); sigma <- as.vector(sigma)
     kappa <- as.vector(kappa)
     if(any(sigma <= 0)) stop("The sigma parameter must be positive.")
     if(any(kappa <= 0)) stop("The kappa parameter must be positive.")
     NN <- max(length(q), length(mu), length(sigma), length(kappa))
     q <- rep(q, len=NN); mu <- rep(mu, len=NN)
     sigma <- rep(sigma, len=NN); kappa <- rep(kappa, len=NN)
     z <- (q - mu) / sigma
     zz <- abs(z)^kappa
     p <- pgamma(zz, shape=1/kappa, scale=kappa)
     p <- p / 2
     temp <- which(z < 0); p[temp] <- 0.5 - p[temp]
     temp <- which(z >= 0); p[temp] <- 0.5 + p[temp]
     if(lower.tail == FALSE) p <- 1 - p
     if(log.p == TRUE) p <- log(p)
     return(p)
     }
qpe <- function(p, mu=0, sigma=1, kappa=2, lower.tail=TRUE, log.p=FALSE)
     {
     p <- as.vector(p); mu <- as.vector(mu); sigma <- as.vector(sigma)
     kappa <- as.vector(kappa)
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(sigma <= 0)) stop("The sigma parameter must be positive.")
     if(any(kappa <= 0)) stop("The kappa parameter must be positive.")
     NN <- max(length(p), length(mu), length(sigma), length(kappa))
     p <- rep(p, len=NN); mu <- rep(mu, len=NN)
     sigma <- rep(sigma, len=NN); kappa <- rep(kappa, len=NN)
     if(log.p == TRUE) p <- log(p)
     if(lower.tail == FALSE) p <- 1 - p
     zp <- 0.5 - p
     temp <- which(p >= 0.5); zp[temp] <- p[temp] - 0.5
     zp <- 2 * zp
     qg <- qgamma(zp, shape=1/kappa, scale=kappa)
     z <- qg^(1/kappa)
     temp <- which(p < 0.5); z[temp] <- -z[temp]
     q <- mu + z * sigma
     return(q)
     }
rpe <- function(n, mu=0, sigma=1, kappa=2)
     {
     mu <- rep(mu, len=n); sigma <- rep(sigma, len=n)
     kappa <- rep(kappa, len=n)
     if(any(sigma <= 0)) stop("The sigma parameter must be positive.")
     if(any(kappa <= 0)) stop("The kappa parameter must be positive.")
     qg <- rgamma(n, shape=1/kappa, scale=kappa)
     z <- qg^(1/kappa)
     u <- runif(n)
     temp <- which(u < 0.5); z[temp] <- -z[temp]
     x <- mu + z * sigma
     return(x)
     }

###########################################################################
# Scaled Inverse Wishart Distribution                                     #
###########################################################################

dsiw <- function(Q, nu, S, zeta, mu, delta, log=FALSE)
     {
     dens <- dinvwishart(Q, nu, S, log=TRUE) +
          sum(dmvn(log(zeta), mu, diag(delta), log=TRUE))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rsiw <- function(nu, S, mu, delta)
     {
     Q <- rinvwishart(nu, S)
     Zeta <- diag(as.vector(exp(rmvn(1, mu, diag(delta)))))
     x <- Zeta %*% Q %*% Zeta
     return(x)
     }

###########################################################################
# Skew Discrete Laplace Distribution                                      #
###########################################################################

dsdlaplace <- function(x, p, q, log=FALSE)
     {
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(q < 0) || any(q > 1)) stop("q must be in [0,1].")
     NN <- max(length(x), length(p), length(q))
     x <- rep(x, len=NN); p <- rep(p, len=NN); q <- rep(q, len=NN)
     dens <- log(1-p) + log(1-q) - (log(1-p*q) + x*log(p))
     temp <- which(x < 0)
     dens[temp] <- log(1-p[temp]) + log(1-q[temp]) -
          (log(1-p[temp]*q[temp]) + abs(x[temp])*log(q[temp]))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
psdlaplace <- function(x, p, q)
     {
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(q < 0) || any(q > 1)) stop("q must be in [0,1].")
     NN <- max(length(x), length(p), length(q))
     x <- rep(x, len=NN); p <- rep(p, len=NN); q <- rep(q, len=NN)
     pr <- 1-(1-q)*p^(floor(x)+1)/(1-p*q)
     temp <- which(x < 0)
     pr[temp] <- (1-p[temp])*q[temp]^(-floor(x[temp]))/(1-p[temp]*q[temp])
     return(pr)
     }
qsdlaplace <- function(prob, p, q)
     {
     if(any(prob < 0) || any(prob > 1)) stop("prob must be in [0,1].")
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(q < 0) || any(q > 1)) stop("q must be in [0,1].")
     NN <- max(length(prob), length(p), length(q))
     prob <- rep(prob, len=NN); p <- rep(p, len=NN); q <- rep(q, len=NN)
     x <- numeric(NN)
     for (i in 1:NN) {
          k <- 0
          if(prob[i] >= psdlaplace(k, p[i], q[i])) {
               while(prob[i] >= psdlaplace(k, p[i], q[i])) {
                    k <- k + 1}}
          else if(prob[i] < psdlaplace(k, p[i], q[i])) {
               while(prob[i] < psdlaplace(k, p[i], q[i])) {
                    k <- k - 1}
               k <- k + 1}
          x[i] <- k
          }
     return(x)
     }
rsdlaplace <- function(n, p, q)
     {
     if(length(p) > 1) stop("p must have a length of 1.")
     if(length(q) > 1) stop("q must have a length of 1.")
     if((p < 0) || (p > 1)) stop("p must be in [0,1].")
     if((q < 0) || (q > 1)) stop("q must be in [0,1].")
     u <- runif(n)
     return(qsdlaplace(u,p,q))
     }

###########################################################################
# Skew-Laplace Distribution                                               #
###########################################################################

dslaplace <- function(x, mu, alpha, beta, log=FALSE)
     {
     x <- as.vector(x); mu <- as.vector(mu)
     alpha <- as.vector(alpha); beta <- as.vector(beta)
     if(any(alpha <= 0)) stop("The alpha parameter must be positive.")
     if(any(beta <= 0)) stop("The beta parameter must be positive.")
     NN <- max(length(x), length(mu), length(alpha), length(beta))
     x <- rep(x, len=NN); mu <- rep(mu, len=NN)
     alpha <- rep(alpha, len=NN); beta <- rep(beta, len=NN)
     ab <- alpha + beta
     dens <- belowMu <- log(1/ab) + ((x - mu)/alpha)
     aboveMu <- log(1/ab) + ((mu - x)/beta)
     temp <- which(x > mu); dens[temp] <- aboveMu[temp]
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
pslaplace <- function(q, mu, alpha, beta)
     {
     q <- as.vector(q); mu <- as.vector(mu)
     alpha <- as.vector(alpha); beta <- as.vector(beta)
     if(any(alpha <= 0)) stop("The alpha parameter must be positive.")
     if(any(beta <= 0)) stop("The beta parameter must be positive.")
     NN <- max(length(q), length(mu), length(alpha), length(beta))
     q <- rep(q, len=NN); mu <- rep(mu, len=NN)
     alpha <- rep(alpha, len=NN); beta <- rep(beta, len=NN)
     ab <- alpha + beta
     p <- belowMu <- (alpha/ab) * exp((q - mu)/alpha)
     aboveMu <- 1 - (beta/ab) * exp((mu - q)/beta)
     temp <- which(q >= mu); p[temp] <- aboveMu[temp]
     return(p)
     }
qslaplace <- function(p, mu, alpha, beta)
     {
     p <- as.vector(p); mu <- as.vector(mu)
     alpha <- as.vector(alpha); beta <- as.vector(beta)
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(alpha <= 0)) stop("The alpha parameter must be positive.")
     if(any(beta <= 0)) stop("The beta parameter must be positive.")
     NN <- max(length(p), length(mu), length(alpha), length(beta))
     p <- rep(p, len=NN); mu <- rep(mu, len=NN)
     alpha <- rep(alpha, len=NN); beta <- rep(beta, len=NN)
     ab <- alpha + beta
     q <- belowMu <- alpha*log(p*ab/alpha) + mu
     aboveMu <- mu - beta*log(ab*(1 - p)/beta)
     temp <- which(p >= alpha/ab); q[temp] <- aboveMu[temp]
     return(q)
     }
rslaplace <- function(n, mu, alpha, beta)
     {
     if(any(alpha <= 0)) stop("The alpha parameter must be positive.")
     if(any(beta <= 0)) stop("The beta parameter must be positive.")
     mu <- rep(mu, len=n)
     alpha <- rep(alpha, len=n)
     beta <- rep(beta, len=n)
     y <- rexp(n, 1)
     probs <- alpha / (alpha + beta)
     signs <- runif(n)
     temp <- which(signs <= probs)
     signs[temp] <- -1
     signs[-temp] <- 1
     mult <- signs * beta
     temp <- which(signs < 0)
     mult[temp] <- signs[temp] * alpha[temp]
     x <- mult * y + mu
     return(x)
     }

###########################################################################
# Stick-Breaking Prior Distribution                                       #
###########################################################################

dStick <- function(theta, gamma, log=FALSE)
     {
     if(log == FALSE) dens <- prod(dbeta(theta, 1, gamma, log=FALSE))
     else dens <- sum(dbeta(theta, 1, gamma, log=TRUE))
     return(dens)
     }
rStick <- function(M, gamma)
     {
     betas <- rbeta(M+1, 1, gamma)
     remaining <- c(1, cumprod(1 - betas))[1:(M+1)]
     w <- remaining * betas
     w <- w / sum(w)
     return(w)
     #return(Stick(rbeta(M,1,gamma))) #Deprecated
     }

###########################################################################
# Student t Distribution (3-parameter)                                    #
#                                                                         #
# The pst and qst functions are similar to the TF functions in the        #
# gamlss.dist package, but dst and rst have been refined.                 #
###########################################################################

dst <- function(x, mu=0, sigma=1, nu=10, log=FALSE)
     {
     x <- as.vector(x); mu <- as.vector(mu)
     sigma <- as.vector(sigma); nu <- as.vector(nu)
     if(any(sigma <= 0)) stop("The sigma parameter must be positive.")
     else if(any(nu <= 0)) stop("The nu parameter must be positive.")
     NN <- max(length(x), length(mu), length(sigma), length(nu))
     x <- rep(x, len=NN); mu <- rep(mu, len=NN)
     sigma <- rep(sigma, len=NN); nu <- rep(nu, len=NN)
     const <- lgamma((nu+1)/2) - lgamma(nu/2) - log(sqrt(pi*nu) * sigma)
     dens <- const + log((1 + (1/nu)*((x-mu)/sigma)^2)^(-(nu+1)/2))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     #return({1/sigma} * dt({x-mu}/sigma, df=nu, log)) #Deprecated
     }
pst <- function(q, mu=0, sigma=1, nu=10, lower.tail=TRUE, log.p=FALSE)
     {
     q <- as.vector(q); mu <- as.vector(mu)
     sigma <- as.vector(sigma); nu <- as.vector(nu)
     if(any(sigma <= 0)) stop("The sigma parameter must be positive.")
     if(any(nu <= 0)) stop("The nu parameter must be positive.")
     NN <- max(length(q), length(mu), length(sigma), length(nu))
     q <- rep(q, len=NN); mu <- rep(mu, len=NN)
     sigma <- rep(sigma, len=NN); nu <- rep(nu, len=NN)
     p <- pt({q-mu}/sigma, df=nu, lower.tail=lower.tail, log.p=log.p)
     temp <- which(nu > 1e6)
     p[temp] <- pnorm(q[temp], mu[temp], sigma[temp],
          lower.tail=lower.tail, log.p=log.p)
     return(p)
     }
qst <- function(p, mu=0, sigma=1, nu=10, lower.tail=TRUE, log.p=FALSE)
     {
     p <- as.vector(p); mu <- as.vector(mu)
     sigma <- as.vector(sigma); nu <- as.vector(nu)
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(sigma <= 0)) stop("The sigma parameter must be positive.")
     if(any(nu <= 0)) stop("The nu parameter must be positive.")
     NN <- max(length(p), length(mu), length(sigma), length(nu))
     p <- rep(p, len=NN); mu <- rep(mu, len=NN)
     sigma <- rep(sigma, len=NN); nu <- rep(nu, len=NN)
     q <- mu + sigma * qt(p, df=nu, lower.tail=lower.tail)
     temp <- which(nu > 1e6)
     q[temp] <- qnorm(p[temp], mu[temp], sigma[temp],
          lower.tail=lower.tail, log.p=log.p)
     return(q)
     }
rst <- function(n, mu=0, sigma=1, nu=10)
     {
     mu <- rep(mu, len=n); sigma <- rep(sigma, len=n); nu <- rep(nu, len=n)
     if(any(sigma <= 0)) stop("The sigma parameter must be positive.")
     if(any(nu <= 0)) stop("The nu parameter must be positive.")
     n <- ceiling(n)
     y <- rnorm(n)
     z <- rchisq(n, nu)
     x <- mu + sigma*y*sqrt(nu/z)
     return(x)
     }

###########################################################################
# Student t Distribution (Precision Parameterization)                     #
###########################################################################

dstp <- function(x, mu=0, tau=1, nu=10, log=FALSE)
     {
     x <- as.vector(x); mu <- as.vector(mu)
     tau <- as.vector(tau); nu <- as.vector(nu)
     if(any(tau <= 0)) stop("The tau parameter must be positive.")
     if(any(nu <= 0)) stop("The nu parameter must be positive.")
     NN <- max(length(x), length(mu), length(tau), length(nu))
     x <- rep(x, len=NN); mu <- rep(mu, len=NN)
     tau <- rep(tau, len=NN); nu <- rep(nu, len=NN)
     dens <- (lgamma((nu+1)/2) - lgamma(nu/2)) + 0.5*log(tau/(nu*pi)) +
          (-(nu+1)/2)*log(1 + (tau/nu)*(x-mu)^2)
     temp <- which(nu > 1e6)
     dens[temp] <- dnorm(x[temp], mu[temp], sqrt(1/tau[temp]), log=TRUE)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
pstp <- function(q, mu=0, tau=1, nu=10, lower.tail=TRUE, log.p=FALSE)
     {
     q <- as.vector(q); mu <- as.vector(mu)
     tau <- as.vector(tau); nu <- as.vector(nu)
     if(any(tau <= 0)) stop("The tau parameter must be positive.")
     if(any(nu <= 0)) stop("The nu parameter must be positive.")
     NN <- max(length(q), length(mu), length(tau), length(nu))
     q <- rep(q, len=NN); mu <- rep(mu, len=NN)
     tau <- rep(tau, len=NN); nu <- rep(nu, len=NN)
     p <- pst(q, mu, sqrt(1/tau), nu, lower.tail, log.p)
     return(p)
     }
qstp <- function(p, mu=0, tau=1, nu=10, lower.tail=TRUE, log.p=FALSE)
     {
     p <- as.vector(p); mu <- as.vector(mu)
     tau <- as.vector(tau); nu <- as.vector(nu)
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(any(tau <= 0)) stop("The tau parameter must be positive.")
     if(any(nu <= 0)) stop("The nu parameter must be positive.")
     NN <- max(length(p), length(mu), length(tau), length(nu))
     p <- rep(p, len=NN); mu <- rep(mu, len=NN)
     tau <- rep(tau, len=NN); nu <- rep(nu, len=NN)
     q <- qst(p, mu, sqrt(1/tau), nu, lower.tail, log.p)
     return(q)
     }
rstp <- function(n, mu=0, tau=1, nu=10)
     {
     mu <- rep(mu, len=n); tau <- rep(tau, len=n); nu <- rep(nu, len=n)
     if(any(tau <= 0)) stop("The tau parameter must be positive.")
     if(any(nu <= 0)) stop("The nu parameter must be positive.")
     x <- rst(n, mu, sqrt(1/tau), nu)
     n <- ceiling(n)
     p <- runif(n)
     x <- qst(p, mu=mu, sqrt(1/tau), nu=nu)
     return(x)
     }

###########################################################################
# Truncated Distribution                                                  #
#                                                                         #
# These functions are similar to those from Nadarajah, S. and Kotz, S.    #
# (2006). ``R Programs for Computing Truncated Distributions''. Journal   #
# of Statistical Software, 16, Code Snippet 2, 1-8. These functions have  #
# been corrected to work with log-densities.                              #
###########################################################################

dtrunc <- function(x, spec, a=-Inf, b=Inf, log=FALSE, ...)
     {
     if(a >= b) stop("Lower bound a is not less than upper bound b.")
     if(any(x < a) | any(x > b))
          stop("At least one instance of (x < a) or (x > b) found.")
     dens <- rep(0, length(x))
     g <- get(paste("d", spec, sep=""), mode="function")
     G <- get(paste("p", spec, sep=""), mode="function")
     if(log == TRUE) {
          dens <- g(x, log=TRUE, ...) - log(G(b, ...) - G(a, ...))
          }
     else {
          dens <- g(x, ...) / (G(b, ...) - G(a, ...))}
     return(dens)
     }
extrunc <- function(spec, a=-Inf, b=Inf, ...)
     {
     f <- function(x) x * dtrunc(x, spec, a=a, b=b, log=FALSE, ...)
     return(integrate(f, lower=a, upper=b)$value)
     }
ptrunc <- function(x, spec, a=-Inf, b=Inf, ...)
     {
     if(a >= b) stop("Lower bound a is not less than upper bound b.")
     if(any(x < a) | any(x > b))
          stop("At least one instance of (x < a) or (x > b) found.")
     p <- x
     aa <- rep(a, length(x))
     bb <- rep(b, length(x))
     G <- get(paste("p", spec, sep=""), mode="function")
     p <- G(apply(cbind(apply(cbind(x, bb), 1, min), aa), 1, max), ...)
     p <- p - G(aa, ...)
     p <- p / {G(bb, ...) - G(aa, ...)}
     return(p)
     }
qtrunc <- function(p, spec, a=-Inf, b=Inf, ...)
     {
     if(any(p < 0) || any(p > 1)) stop("p must be in [0,1].")
     if(a >= b) stop("Lower bound a is not less than upper bound b.")
     q <- p
     G <- get(paste("p", spec, sep=""), mode="function")
     Gin <- get(paste("q", spec, sep=""), mode="function")
     q <- Gin(G(a, ...) + p*{G(b, ...) - G(a, ...)}, ...)
     return(q)
     }
rtrunc <- function(n, spec, a=-Inf, b=Inf, ...)
     {
     if(a >= b) stop("Lower bound a is not less than upper bound b.")
     x <- u <- runif(n)
     x <- qtrunc(u, spec, a=a, b=b,...)
     return(x)
     }
vartrunc <- function(spec, a=-Inf, b=Inf, ...)
     {
     ex <- extrunc(spec, a=a, b=b, ...)
     f <- function(x) {
          {x - ex}^2 * dtrunc(x, spec, a=a, b=b, log=FALSE, ...)}
     sigma2 <- integrate(f, lower=a, upper=b)$value
     return(sigma2)
     }

###########################################################################
# Wishart Distribution                                                    #
###########################################################################

dwishart <- function(Omega, nu, S, log=FALSE)
     {
     if(!is.matrix(Omega)) Omega <- matrix(Omega)
     if(!is.positive.definite(Omega))
          stop("Matrix Omega is not positive-definite.")
     if(!is.matrix(S)) S <- matrix(S)
     if(!is.positive.semidefinite(S))
          stop("Matrix S is not positive-semidefinite.")
     if(!identical(dim(Omega), dim(S)))
          stop("The dimensions of Omega and S differ.")
     if(nu < nrow(S))
          stop("The nu parameter is less than the dimension of S.")
     k <- nrow(Omega)
     gamsum <- 0
     for (i in 1:k) {gamsum <- gamsum + lgamma((nu + 1 - i)/2)}
     dens <- -((nu*k)/2) * log(2) - ((k*(k - 1))/4) * log(pi) - gamsum -
          (nu/2) * log(det(S)) + ((nu - k - 1)/2) * logdet(Omega) -
          (tr(as.inverse(S) %*% Omega)/2)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rwishart <- function(nu, S)
     {
     if(!is.matrix(S)) S <- matrix(S)
     if(!is.positive.semidefinite(S))
          stop("Matrix S is not positive-semidefinite.")
     if(nu < nrow(S))
          stop("The nu parameter is less than the dimension of S.")
     k <- nrow(S)
     Z <- matrix(0, k, k)
     x <- rchisq(k, nu:{nu - k + 1})
     x[which(x == 0)] <- 1e-100
     diag(Z) <- sqrt(x)
     if(k > 1) {
          kseq <- 1:(k-1)
          Z[rep(k*kseq, kseq) +
               unlist(lapply(kseq, seq))] <- rnorm(k*{k - 1}/2)}
     return(crossprod(Z %*% chol(S)))
     }

###########################################################################
# Wishart Distribution (Cholesky Parameterization)                        #
###########################################################################

dwishartc <- function(U, nu, S, log=FALSE)
     {
     if(missing(U)) stop("Upper triangular U is required.")
     Omega <- t(U) %*% U
     if(!is.matrix(S)) S <- matrix(S)
     if(!is.positive.semidefinite(S))
          stop("Matrix S is not positive-semidefinite.")
     if(!identical(dim(Omega), dim(S)))
          stop("The dimensions of Omega and S differ.")
     if(nu < nrow(S))
          stop("The nu parameter is less than the dimension of S.")
     k <- nrow(Omega)
     gamsum <- 0
     for (i in 1:k) {gamsum <- gamsum + lgamma((nu + 1 - i)/2)}
     dens <- -((nu*k)/2) * log(2) - ((k*(k - 1))/4) * log(pi) - gamsum -
          (nu/2) * log(det(S)) + ((nu - k - 1)/2) * logdet(Omega) -
          (tr(as.inverse(S) %*% Omega)/2)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
rwishartc <- function(nu, S)
     {
     if(!is.matrix(S)) S <- matrix(S)
     if(!is.positive.semidefinite(S))
          stop("Matrix S is not positive-semidefinite.")
     if(nu < nrow(S)) {
          stop("The nu parameter is less than the dimension of S.")}
     k <- nrow(S)
     Z <- matrix(0, k, k)
     x <- rchisq(k, nu:{nu - k + 1})
     x[which(x == 0)] <- 1e-100
     diag(Z) <- sqrt(x)
     if(k > 1) {
          kseq <- 1:(k-1)
          Z[rep(k*kseq, kseq) +
               unlist(lapply(kseq, seq))] <- rnorm(k*{k - 1}/2)}
     return(Z %*% chol(S))
     }

###########################################################################
# Yang-Berger Distribution                                                #
###########################################################################

dyangberger <- function(x, log=FALSE)
     {
     if(missing(x)) stop("Matrix x is a required argument.")
     if(!is.matrix(x)) x <- as.matrix(x)
     x <- as.symmetric.matrix(x)
     if(!is.positive.definite(x))
          stop("Matrix x is not positive-definite.")
     d <- sort(eigen(x)$values)
     dens <- log(1) - logdet(x) * prod(diff(d))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
dyangbergerc <- function(x, log=FALSE)
     {
     if(missing(x))
          stop("Upper triangular matrix x is a required argument.")
     if(!is.matrix(x)) x <- as.matrix(x)
     x <- t(x) %*% x
     d <- sort(eigen(x)$values)
     dens <- log(1) - logdet(x) * prod(diff(d))
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }

###########################################################################
# Zellner's g-Prior                                                       #
###########################################################################

dhyperg <- function(g, alpha=3, log=FALSE)
     {
     if(g <= 0) stop("The g parameter must be positive.")
     if(alpha <= 0) stop("The alpha parameter must be positive.")
     dens <- log((alpha - 2)/2) -(alpha/2)*log(1 + g)
     if(log == FALSE) dens <- exp(dens)
     return(dens)
     }
dzellner <- function(beta, g, sigma, X, log=FALSE)
     {
     if(g <= 0) stop("The g parameter must be positive.")
     if(sigma <= 0) stop("The sigma parameter must be positive.")
  ## HS (01/2020): The original formulation was:
  ### g*sigma*sigma* as.inverse(t(X) %*% X)
  ### the latter bit is equivalent to: solve(crossprod(X))
  ### which should be equivalent to the more stable and accurate
  ### chol2inv(qr.R(qr(X))) 
  ## see: strucchange::solveCrossprod
     dens <- dmvn(beta, rep(0, length(beta)),
          g*sigma*sigma*chol2inv(qr.R(qr(X))), log=log)
     return(dens)
     }
rzellner <- function(n, g, sigma, X)
     {
     x <- rmvn(n, rep(0, ncol(X)), g*sigma*sigma*chol2inv(qr.R(qr(X))))
     return(x)
     }

#End

###########################################################################
# p.interval                                                              #
#                                                                         #
# The purpose of the p.interval function is to estimate either a          #
# quantile-based probability interval, uni-modal highest posterior        #
# density (HPD) interval, or multi-modal HPD intervals of posterior       #
# samples. The code for the uni-modal HPD interval is similar to code in  #
# the coda package, but this is designed to work with matrices or objects #
# of class demonoid, iterquad, laplace, or pmc.                           #
###########################################################################

p.interval <- function(obj, HPD=TRUE, MM=TRUE, prob=0.95, plot=FALSE,
     PDF=FALSE, ...)
     {
     ### Initial Checks
     if(missing(obj)) stop("The obj argument is required.")
     if(length(prob) > 1)
          stop("The prob argument must be a scalar.")
     if((prob <= 0) | (prob >= 1))
          stop("The prob argument must be in the interval (0,1).")
     if(identical(class(obj), "demonoid")) {
          thin <- obj$Rec.BurnIn.Thinned
          n <- nrow(obj$Posterior1)
          if(thin < nrow(obj$Posterior1))
               x <- cbind(obj$Posterior1[thin:n,], obj$Monitor[thin:n,])
          if(thin >= nrow(obj$Posterior1))
               x <- cbind(obj$Posterior1, obj$Monitor)
          colnames(x) <- c(colnames(obj$Posterior1),
               colnames(obj$Monitor))
          obj <- x
          }
     else if(identical(class(obj), "iterquad")) {
          if(any(is.na(obj$Posterior)))
               stop("Posterior samples do not exist in obj.")
          obj <- obj$Posterior
          }
     else if(identical(class(obj), "laplace")) {
          if(any(is.na(obj$Posterior)))
               stop("Posterior samples do not exist in obj.")
          obj <- obj$Posterior
          }
     else if(identical(class(obj), "pmc")) {
          x <- cbind(obj$Posterior2, obj$Monitor)
          colnames(x) <- c(colnames(obj$Posterior2), colnames(obj$Monitor))
          obj <- x
          }
     else {
          x <- as.matrix(obj)
          oname <- deparse(substitute(obj))
          colnames(x) <- colnames(obj)#oname
          obj <- x
          }
     if(any(!is.finite(obj)))
          stop("The obj argument must contain finite values.")
     if(any(apply(obj, 2, is.constant)))
          stop("The obj argument has insufficient length.")
     ### Probability Intervals
     if(HPD == TRUE) {
          vals <- apply(obj, 2, sort)
          if(!is.matrix(vals)) stop("obj must have nsamp > 1.")
          nsamp <- nrow(vals)
          npar <- ncol(vals)
          gap <- max(1, min(nsamp - 1, round(nsamp * prob)))
          init <- 1:(nsamp - gap)
          inds <- apply(vals[init + gap, , drop=FALSE] -
               vals[init, , drop=FALSE], 2, which.min)
          ans <- cbind(vals[cbind(inds, 1:npar)],
               vals[cbind(inds + gap, 1:npar)])
          dimnames(ans) <- list(colnames(obj), c("Lower", "Upper"))
          attr(ans, "Probability.Interval") <- gap / nsamp
          }
     if(HPD == FALSE) {
          ans <- apply(obj, 2, quantile,
               probs=c((1-prob)/2,1-((1-prob)/2)))
          ans <- as.matrix(t(ans))
          colnames(ans) <- c("Lower","Upper")
          attr(ans, "Probability.Interval") <- prob}
     if(MM == TRUE) {
          ansmm <- ans
          mm <- apply(obj, 2, is.multimodal)
          if(any(mm)) {
               cat("\n\nPotentially multimodal column vectors:\n",
                    which(mm),"\n")
               vals <- apply(obj, 2, sort)
               if(!is.matrix(vals)) stop("obj must have nsamp > 1.")
               for (m in which(mm)) {
                    kde <- density(vals[,m])
                    dens <- approx(kde$x, kde$y, vals[,m])$y
                    dens.ind <- dens >= as.vector(quantile(dens,
                         probs=1-prob)) * 1
                    ints <- ""
                    count <- 1
                    for (i in 1:nrow(vals)) {
                         if((i == 1) & (dens.ind[i] == 1)) {
                              ints <- paste("(",round(vals[i,m],3),",",sep="")
                              if(count > ncol(ansmm)) ansmm <- cbind(ansmm,NA)
                              ansmm[m,count] <- vals[i,m]
                              count <- count + 1
                              }
                         if(i > 1) {
                              if((dens.ind[i] == 0) & (dens.ind[i-1] == 1)) {
                                   ints <- paste(ints,round(vals[i-1,m],3),")",sep="")
                                   if(count > ncol(ansmm)) ansmm <- cbind(ansmm,NA)
                                   ansmm[m,count] <- vals[i-1,m]
                                   count <- count + 1
                                   }
                              if((dens.ind[i] == 1) & (dens.ind[i-1] == 0))  {
                                   ints <- paste(ints," (",round(vals[i,m],3),",",sep="")
                                   if(count > ncol(ansmm)) ansmm <- cbind(ansmm,NA)
                                   ansmm[m,count] <- vals[i,m]
                                   count <- count + 1
                                   }
                              }
                         }
                    if((dens.ind[i] == 1) & (dens.ind[i-1] == 1)) {
                         ints <- paste(ints,round(vals[i,m],3),")",sep="")
                         if(count > ncol(ansmm)) ansmm <- cbind(ansmm,NA)
                              ansmm[m,count] <- vals[i,m]
                              count <- count + 1
                         }
                    cat("\nColumn", m, "multimodal intervals:", ints, "\n")}
               }
          }
     if(plot == TRUE) {
          if(PDF == TRUE) {
               pdf("P.Interval.Plots.pdf")
               par(mfrow=c(1,1))}
          else par(mfrow=c(1,1), ask=TRUE)
          if(MM == FALSE) {
               for (i in 1:nrow(ans)) {
                    kde <- kde.low <- kde.high <- density(obj[,i])
                    kde.low$x <- kde$x[kde$x < ans[i,1]]
                    kde.low$y <- kde$y[which(kde$x < ans[i,1])]
                    kde.high$x <- kde$x[kde$x > ans[i,2]]
                    kde.high$y <- kde$y[which(kde$x > ans[i,2])]
                    plot(kde, xlab="Value", main=colnames(obj)[i])
                    polygon(kde, col="black", border="black")
                    polygon(c(min(kde.low$x), kde.low$x, max(kde.low$x)),
                         c(min(kde.low$y), kde.low$y, min(kde.low$y)),
                         col="gray", border="gray")
                    polygon(c(min(kde.high$x), kde.high$x, max(kde.high$x)),
                         c(min(kde.high$y), kde.high$y, min(kde.high$y)),
                         col="gray", border="gray")
                    abline(v=0, col="red", lty=2)
                    }
               }
          if(MM == TRUE) {
               for (i in 1:nrow(ansmm)) {
                    ### Mode 1
                    kde <- density(obj[,i])
                    x1 <- min(which(kde$x >= ansmm[i,1]))
                    x2 <- max(which(kde$x <= ansmm[i,2]))
                    plot(kde, xlab="Value", main=colnames(obj)[i])
                    polygon(kde, col="gray", border="gray")
                    with(kde, polygon(x=c(x[c(x1,x1:x2,x2)]),
                         y=c(0,y[x1:x2],0), col="black"))
                    ### Mode 2
                    if((ncol(ansmm) > 2) && !is.na(ansmm[i,3])) {
                         x1 <- min(which(kde$x >= ansmm[i,3]))
                         x2 <- max(which(kde$x <= ansmm[i,4]))
                         with(kde, polygon(x=c(x[c(x1,x1:x2,x2)]),
                              y=c(0,y[x1:x2],0), col="black"))
                         }
                    if((ncol(ansmm) > 4) && !is.na(ansmm[i,5])) {
                         x1 <- min(which(kde$x >= ansmm[i,5]))
                         x2 <- max(which(kde$x <= ansmm[i,6]))
                         with(kde, polygon(x=c(x[c(x1,x1:x2,x2)]),
                              y=c(0,y[x1:x2],0), col="black"))
                         }
                    abline(v=0, col="red", lty=2)
                    }
               }
          if(PDF == TRUE) dev.off()
          }
     return(ans)
     }

#End

###########################################################################
# print.demonoid                                                          #
#                                                                         #
# The purpose of the print.demonoid function is to print the contents of  #
# an object of class demonoid to the screen.                              #
###########################################################################

print.demonoid <- function(x, ...)
     {
     if(missing(x)) stop("The x argument is required.")
     cat("Call:\n")
     print(x$Call)
     cat("\nAcceptance Rate: ", round(x$Acceptance.Rate,5),
          "\n", sep="")
     cat("Algorithm: ", x$Algorithm, "\n", sep="")
     cat("Covariance Matrix: (NOT SHOWN HERE; diagonal shown instead)\n")
     if(is.matrix(x$Covar)) {
          print(diag(x$Covar))
          }
     else if(!is.list(x$Covar) & is.vector(x$Covar)) {
          print(x$Covar)
          }
     else for (i in 1:length(x$Covar)) {
          cat("Block:", i, "\n")
          print(diag(x$Covar[[i]]))
          cat("\n")}
     cat("\nCovariance (Diagonal) History: (NOT SHOWN HERE)\n")
     cat("Deviance Information Criterion (DIC):\n")
     DIC <- matrix(c(round(x$DIC1[1],3), round(x$DIC1[2],3),
          round(x$DIC1[3],3), round(x$DIC2[1],3), round(x$DIC2[2],3),
          round(x$DIC2[3],3)), 3, 2,
          dimnames=list(c("Dbar","pD","DIC"),c("All","Stationary")))
     print(DIC)
     cat("Initial Values:\n")
     print(x$Initial.Values)
     cat("\nIterations: ", x$Iterations, "\n", sep="")
     cat("Log(Marginal Likelihood): ", x$LML, "\n", sep="")
     cat("Minutes of run-time: ", round(x$Minutes,2), "\n",
          sep="")
     cat("Model: (NOT SHOWN HERE)\n")
     cat("Monitor: (NOT SHOWN HERE)\n")
     cat("Parameters (Number of): ", x$Parameters, "\n",
          sep="")
     cat("Posterior1: (NOT SHOWN HERE)\n")
     cat("Posterior2: (NOT SHOWN HERE)\n")
     cat("Recommended Burn-In of Thinned Samples: ",
          x$Rec.BurnIn.Thinned, "\n", sep="")
     cat("Recommended Burn-In of Un-thinned Samples: ",
          x$Rec.BurnIn.UnThinned, "\n", sep="")
     cat("Recommended Thinning: ", x$Rec.Thinning, "\n", sep="")
     cat("Specs: (NOT SHOWN HERE)\n")
     cat("Status is displayed every ", x$Status,
          " iterations\n", sep="")
     cat("Summary1: (SHOWN BELOW)\n")
     cat("Summary2: (SHOWN BELOW)\n")
     cat("Thinned Samples: ", x$Thinned.Samples, "\n",
          sep="")
     cat("Thinning: ", x$Thinning, "\n", sep="")
     cat("\n\nSummary of All Samples\n")
     print(x$Summary1)
     cat("\n\nSummary of Stationary Samples\n")
     print(x$Summary2)
     invisible(x)
     }

#End

###########################################################################
# is.appeased                                                             #
#                                                                         #
# The purpose of the is.appeased function is to perform a logical test of #
# whether or not Laplace's Demon is appeased with an object of class      #
# demonoid.                                                               #
###########################################################################

is.appeased <- function(x)
     {
     appeased <- FALSE
     if(!identical(class(x), "demonoid"))
          stop("x must be of class demonoid.")
     captive <- capture.output(Consort(x))
     z <- grep("has been appeased", captive)
     if(length(z) > 0) appeased <- TRUE
     return(appeased)
     }

#End

###########################################################################
# is.class                                                                #
#                                                                         #
# The purpose of the is.class functions is to provide logical tests       #
# regarding the classes of objects.                                       #
###########################################################################

is.bayesfactor <- function(x)
     {
     bayesfactor <- FALSE
     if(identical(class(x), "bayesfactor")) bayesfactor <- TRUE
     return(bayesfactor)
     }
is.blocks <- function(x)
     {
     if(identical(class(x), "blocks")) blocks <- TRUE
     return(blocks)
     }
is.bmk <- function(x)
     {
     bmk <- FALSE
     if(identical(class(x), "bmk")) bmk <- TRUE
     return(bmk)
     }
is.demonoid <- function(x)
     {
     demonoid <- FALSE
     if(identical(class(x), "demonoid")) demonoid <- TRUE
     return(demonoid)
     }
is.demonoid.hpc <- function(x)
     {
     demonoid.hpc <- FALSE
     if(identical(class(x), "demonoid.hpc")) demonoid.hpc <- TRUE
     return(demonoid.hpc)
     }
is.demonoid.ppc <- function(x)
     {
     demonoid.ppc <- FALSE
     if(identical(class(x), "demonoid.ppc")) demonoid.ppc <- TRUE
     return(demonoid.ppc)
     }
is.demonoid.val <- function(x)
     {
     demonoid.val <- FALSE
     if(identical(class(x), "demonoid.val")) demonoid.val <- TRUE
     return(demonoid.val)
     }
is.hangartner <- function(x)
     {
     hangartner <- FALSE
     if(identical(class(x), "hangartner")) hangartner <- TRUE
     return(hangartner)
     }
is.heidelberger <- function(x)
     {
     heidelberger <- FALSE
     if(identical(class(x), "heidelberger")) heidelberger <- TRUE
     return(heidelberger)
     }
is.importance <- function(x)
     {
     importance <- FALSE
     if(identical(class(x), "importance")) importance <- TRUE
     return(importance)
     }
is.iterquad <- function(x)
     {
     iterquad <- FALSE
     if(identical(class(x), "iterquad")) iterquad <- TRUE
     return(iterquad)
     }
is.iterquad.ppc <- function(x)
     {
     iterquad.ppc <- FALSE
     if(identical(class(x), "iterquad.ppc")) iterquad.ppc <- TRUE
     return(iterquad.ppc)
     }
is.juxtapose <- function(x)
     {
     juxtapose <- FALSE
     if(identical(class(x), "juxtapose")) juxtapose <- TRUE
     return(juxtapose)
     }
is.laplace <- function(x)
     {
     laplace <- FALSE
     if(identical(class(x), "laplace")) laplace <- TRUE
     return(laplace)
     }
is.laplace.ppc <- function(x)
     {
     laplace.ppc <- FALSE
     if(identical(class(x), "laplace.ppc")) laplace.ppc <- TRUE
     return(laplace.ppc)
     }
is.miss <- function(x)
     {
     miss <- FALSE
     if(identical(class(x), "miss")) miss <- TRUE
     return(miss)
     }
is.pmc <- function(x)
     {
     pmc <- FALSE
     if(identical(class(x), "pmc")) pmc <- TRUE
     return(pmc)
     }
is.pmc.ppc <- function(x)
     {
     pmc.ppc <- FALSE
     if(identical(class(x), "pmc.ppc")) pmc.ppc <- TRUE
     return(pmc.ppc)
     }

is.pmc.val <- function(x)
     {
     pmc.val <- FALSE
     if(identical(class(x), "pmc.val")) pmc.val <- TRUE
     return(pmc.val)
     }
is.posteriorchecks <- function(x)
     {
     posteriorchecks <- FALSE
     if(identical(class(x), "posteriorchecks")) posteriorchecks <- TRUE
     return(posteriorchecks)
     }
is.raftery <- function(x)
     {
     raftery <- FALSE
     if(identical(class(x), "raftery")) raftery <- TRUE
     return(raftery)
     }
is.rejection <- function(x)
     {
     rejection <- FALSE
     if(identical(class(x), "rejection")) rejection <- TRUE
     return(rejection)
     }
is.sensitivity <- function(x)
     {
     sensitivity <- FALSE
     if(identical(class(x), "sensitivity")) sensitivity <- TRUE
     return(sensitivity)
     }
is.vb <- function(x)
     {
     vb <- FALSE
     if(identical(class(x), "vb")) vb <- TRUE
     return(vb)
     }
is.vb.ppc <- function(x)
     {
     vb.ppc <- FALSE
     if(identical(class(x), "vb.ppc")) vb.ppc <- TRUE
     return(vb.ppc)
     }

#End

###########################################################################
# print.raftery                                                           #
#                                                                         #
# The purpose of the print.raftery function is to print the contents of   #
# an object of class raftery to the screen.                               #
###########################################################################

print.raftery <- function(x, digits=3, ...) 
     {
     cat("\nQuantile (q) =", x$params["q"])
     cat("\nAccuracy (r) = +/-", x$params["r"])
     cat("\nProbability (s) =", x$params["s"], "\n")
     if(x$resmatrix[1] == "Error") 
     cat("\nYou need a sample size of at least", x$resmatrix[2], 
          "with these values of q, r and s.\n")
     else {
          out <- x$resmatrix
          for (i in ncol(out)) out[, i] <- format(out[, i], digits=digits)
          out <- rbind(matrix(c("Burn-in ", "Total", "Lower bound ", 
               "Dependence", "(M)", "(N)", "(Nmin)", "factor (I)"), 
               byrow=TRUE, nrow=2), out)
          if(!is.null(rownames(x$resmatrix))) 
          out <- cbind(c("", "", rownames(x$resmatrix)), out)
          dimnames(out) <- list(rep("", nrow(out)), rep("", ncol(out)))
          print.default(out, quote=FALSE, ...)
          cat("\n")}
     invisible(x)
     }

#End

###########################################################################
# caterpillar.plot                                                        #
#                                                                         #
# The purpose of the caterpillar.plot function is to provide a            #
# caterpillar plot of the posterior summaries in an object of class       #
# demonoid, laplace, or pmc, or also of S x J matrix of S samples and J   #
# variables.                                                              #
###########################################################################

caterpillar.plot <- function(x, Parms=NULL, Title=NULL)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     par(mfrow=c(1,1))
     if(identical(class(x), "demonoid")) {
          if(any(is.na(x$Summary2))) {
               x <- x$Summary1
               x.lab <- "All Samples"}
          else {
               x <- x$Summary2
               x.lab <- "Stationary Samples"}
          if(!is.null(Parms)) {
               if(is.character(Parms)) {
                    Parms <- sub("\\[","\\\\[",Parms)
                    Parms <- sub("\\]","\\\\]",Parms)
                    Parms <- sub("\\.","\\\\.",Parms)
                    if(length(grep(Parms[1], rownames(x))) == 0)
                         stop("Parameter in Parms does not exist.")
                    keeprows <- grep(Parms[1], rownames(x))
                    if(length(Parms) > 1) {
                         for (i in 2:length(Parms)) {
                              if(length(grep(Parms[i], rownames(x))) == 0)
                                   stop("Parameter in Parms does not exist.")
                              keeprows <- c(keeprows,
                                   grep(Parms[i], rownames(x)))}
                         }
                    }
               if(is.numeric(Parms)) keeprows <- Parms
               temp <- x
               x <- matrix(x[keeprows,], length(keeprows), ncol(temp))
               rownames(x) <- rownames(temp)[keeprows]
               colnames(x) <- colnames(temp)
               }
          ### Setup
          x.rows <- nrow(x)
          x.lim <- c(min(x[,5]), max(x[,7]))
          y.lim <- c(0, x.rows+1)
          ### Basic Plot
          plot(0, 0, ylim=y.lim, xlim=x.lim, main=Title, sub="",
               xlab=x.lab, ylab="", type="n", ann=TRUE, yaxt="n")
          abline(v=0, col="gray")
          ### Add Medians
          points(x[,6], x.rows:1, pch=20)
          ### Add Horizontal Lines for 2.5%-97.5% Quantiles
          for (i in 1:x.rows) {
               lines(x[i,c(5,7)], c(x.rows-i+1, x.rows-i+1))}
          ### Add y-axis labels
          yy <- x.rows:1
          cex.labels <- 1 / {log(x.rows)/5 + 1}
          axis(2, labels=rownames(x), tick=FALSE, las=1, at=yy,
               cex.axis=cex.labels)
          }
     else if(identical(class(x), "demonoid.hpc")) {
          Chains <- length(x)
          x.temp <- list()
          for (i in 1:Chains) {x.temp[[i]] <- x[[i]][["Summary1"]]}
          x <- x.temp; remove(x.temp)
          x.lab <- "All Samples"
          if(!is.null(Parms)) {
               if(is.character(Parms)) {
                    Parms <- sub("\\[","\\\\[",Parms)
                    Parms <- sub("\\]","\\\\]",Parms)
                    Parms <- sub("\\.","\\\\.",Parms)
                    if(length(grep(Parms[1], rownames(x[[1]]))) == 0)
                         stop("Parameter in Parms does not exist.")
                    keeprows <- grep(Parms[1], rownames(x[[1]]))
                    if(length(Parms) > 1) {
                         for (i in 2:length(Parms)) {
                              if(length(grep(Parms[i], rownames(x[[1]]))) == 0)
                                   stop("Parameter in Parms does not exist.")
                              keeprows <- c(keeprows,
                                   grep(Parms[i], rownames(x[[1]])))}
                         }
                    }
               if(is.numeric(Parms)) keeprows <- Parms
               temp <- x
               for (i in 1:Chains) {
                    x[[i]] <- matrix(x[[i]][keeprows,], length(keeprows),
                         ncol(temp[[1]]))}
               rownames(x[[1]]) <- rownames(temp[[1]])[keeprows]
               colnames(x[[1]]) <- colnames(temp[[1]])
               }
          ### Setup
          x.rows <- nrow(x[[1]])
          x.lim <- c(min(x[[1]][,5]), max(x[[1]][,7]))
          y.lim <- c(0, x.rows+1)
          ### Basic Plot
          plot(0, 0, ylim=y.lim, xlim=x.lim, main=Title, sub="",
               xlab=x.lab, ylab="", type="n", ann=TRUE, yaxt="n")
          abline(v=0, col="gray")
          ### Add Medians
          points(x[[1]][,6], x.rows:1, pch=20)
          for (i in 2:Chains) {points(x[[i]][,6], (x.rows:1)-(i/10), col=i, pch=20)}
          ### Add Horizontal Lines for 2.5%-97.5% Quantiles
          for (i in 1:x.rows) {
               lines(x[[1]][i,c(5,7)], c(x.rows-i+1, x.rows-i+1))}
          for (j in 2:Chains) {for (i in 1:x.rows) {
               lines(x[[j]][i,c(5,7)], c(x.rows-i+1-(j/10), x.rows-i+1-(j/10)), col=j)}}
          ### Add y-axis labels
          yy <- x.rows:1
          cex.labels <- 1 / {log(x.rows)/5 + 1}
          axis(2, labels=rownames(x[[1]]), tick=FALSE, las=1, at=yy,
               cex.axis=cex.labels)
          }
     else if(identical(class(x), "iterquad")) {
          if(any(is.na(x$Posterior))) {
               x <- x$Summary1
               x.lab <- "Point-Estimates"}
          else {
               x <- x$Summary2[1:length(x$Initial.Values),]
               x.lab <- "SIR Samples"}
          if(is.null(Parms)) {
               keeprows <- Parms <- 1:length(x$Initial.Values)}
          else {
               if(is.numeric(Parms)) keeprows <- Parms
               if(is.character(Parms)) {
                    Parms <- sub("\\[","\\\\[",Parms)
                    Parms <- sub("\\]","\\\\]",Parms)
                    Parms <- sub("\\.","\\\\.",Parms)
                    if(length(grep(Parms[1], rownames(x))) == 0)
                         stop("Parameter in Parms does not exist.")
                    keeprows <- grep(Parms[1], rownames(x))
                    if(length(Parms) > 1) {
                         for (i in 2:length(Parms)) {
                              if(length(grep(Parms[i],
                                   rownames(x))) == 0)
                                   stop("Parameter in Parms does not exist.")
                              keeprows <- c(keeprows,
                                   grep(Parms[i], rownames(x)))}
                         }
                    }
               }
          temp <- x
          x <- matrix(x[keeprows,], length(keeprows),
               ncol(temp))
          rownames(x) <- rownames(temp)[keeprows]
          colnames(x) <- colnames(temp)
          if(x.lab != "SIR Samples") Modes <- x[,1]
          else Modes <- x[,6]
          if(x.lab != "SIR Samples") LB <- x[,3]
          else LB <- x[,5]
          if(x.lab != "SIR Samples") UB <- x[,4]
          else UB <- x[,7]
          ### Setup
          x.rows <- length(Modes)
          x.lim <- c(min(LB), max(UB))
          y.lim <- c(0, x.rows+1)
          ### Basic Plot
          plot(0, 0, ylim=y.lim, xlim=x.lim, main=Title, sub="",
               xlab=x.lab, ylab="", type="n", ann=TRUE, yaxt="n")
          abline(v=0, col="gray")
          ### Add Modes
          points(Modes, x.rows:1, pch=20)
          ### Add Horizontal Lines for 2.5%-97.5% Quantiles
          for (i in 1:x.rows) {
               lines(c(LB[i], UB[i]), c(x.rows-i+1, x.rows-i+1))}
          ### Add y-axis labels
          yy <- x.rows:1
          cex.labels <- 1/{log(x.rows)/5 + 1}
          axis(2, labels=rownames(x), tick=FALSE, las=1,
               at=yy, cex.axis=cex.labels)
          }
     else if(identical(class(x), "laplace")) {
          if(any(is.na(x$Posterior))) {
               x <- x$Summary1
               x.lab <- "Point-Estimates"}
          else {
               x <- x$Summary2[1:length(x$Initial.Values),]
               x.lab <- "SIR Samples"}
          if(is.null(Parms)) {
               keeprows <- Parms <- 1:length(x$Initial.Values)}
          else {
               if(is.numeric(Parms)) keeprows <- Parms
               if(is.character(Parms)) {
                    Parms <- sub("\\[","\\\\[",Parms)
                    Parms <- sub("\\]","\\\\]",Parms)
                    Parms <- sub("\\.","\\\\.",Parms)
                    if(length(grep(Parms[1], rownames(x))) == 0)
                         stop("Parameter in Parms does not exist.")
                    keeprows <- grep(Parms[1], rownames(x))
                    if(length(Parms) > 1) {
                         for (i in 2:length(Parms)) {
                              if(length(grep(Parms[i],
                                   rownames(x))) == 0)
                                   stop("Parameter in Parms does not exist.")
                              keeprows <- c(keeprows,
                                   grep(Parms[i], rownames(x)))}
                         }
                    }
               }
          temp <- x
          x <- matrix(x[keeprows,], length(keeprows),
               ncol(temp))
          rownames(x) <- rownames(temp)[keeprows]
          colnames(x) <- colnames(temp)
          if(x.lab != "SIR Samples") Modes <- x[,1]
          else Modes <- x[,6]
          if(x.lab != "SIR Samples") LB <- x[,3]
          else LB <- x[,5]
          if(x.lab != "SIR Samples") UB <- x[,4]
          else UB <- x[,7]
          ### Setup
          x.rows <- length(Modes)
          x.lim <- c(min(LB), max(UB))
          y.lim <- c(0, x.rows+1)
          ### Basic Plot
          plot(0, 0, ylim=y.lim, xlim=x.lim, main=Title, sub="",
               xlab=x.lab, ylab="", type="n", ann=TRUE, yaxt="n")
          abline(v=0, col="gray")
          ### Add Modes
          points(Modes, x.rows:1, pch=20)
          ### Add Horizontal Lines for 2.5%-97.5% Quantiles
          for (i in 1:x.rows) {
               lines(c(LB[i], UB[i]), c(x.rows-i+1, x.rows-i+1))}
          ### Add y-axis labels
          yy <- x.rows:1
          cex.labels <- 1/{log(x.rows)/5 + 1}
          axis(2, labels=rownames(x), tick=FALSE, las=1,
               at=yy, cex.axis=cex.labels)
          }
     else if(identical(class(x), "pmc")) {
          x <- x$Summary
          x.lab <- "All Samples"
          if(!is.null(Parms)) {
               if(is.character(Parms)) {
                    Parms <- sub("\\[","\\\\[",Parms)
                    Parms <- sub("\\]","\\\\]",Parms)
                    Parms <- sub("\\.","\\\\.",Parms)
                    if(length(grep(Parms[1], rownames(x))) == 0)
                         stop("Parameter in Parms does not exist.")
                    keeprows <- grep(Parms[1], rownames(x))
                    if(length(Parms) > 1) {
                         for (i in 2:length(Parms)) {
                              if(length(grep(Parms[i], rownames(x))) == 0)
                                   stop("Parameter in Parms does not exist.")
                              keeprows <- c(keeprows,
                                   grep(Parms[i], rownames(x)))}
                         }
                    }
               if(is.numeric(Parms)) keeprows <- Parms
               temp <- x
               x <- matrix(x[keeprows,], length(keeprows), ncol(temp))
               rownames(x) <- rownames(temp)[keeprows]
               colnames(x) <- colnames(temp)
               }
          ### Setup
          x.rows <- nrow(x)
          x.lim <- c(min(x[,5]), max(x[,7]))
          y.lim <- c(0, x.rows+1)
          ### Basic Plot
          plot(0, 0, ylim=y.lim, xlim=x.lim, main=Title, sub="",
               xlab=x.lab, ylab="", type="n", ann=TRUE, yaxt="n")
          abline(v=0, col="gray")
          ### Add Medians
          points(x[,6], x.rows:1, pch=20)
          ### Add Horizontal Lines for 2.5%-97.5% Quantiles
          for (i in 1:x.rows) {
               lines(x[i,c(5,7)], c(x.rows-i+1, x.rows-i+1))}
          ### Add y-axis labels
          yy <- x.rows:1
          cex.labels <- 1 / {log(x.rows)/5 + 1}
          axis(2, labels=rownames(x), tick=FALSE, las=1, at=yy,
               cex.axis=cex.labels)
          }
      else if(identical(class(x), "vb")) {
          if(any(is.na(x$Posterior))) {
               x <- x$Summary1
               x.lab <- "Point-Estimates"}
          else {
               x <- x$Summary2[1:length(x$Initial.Values),]
               x.lab <- "SIR Samples"}
          if(is.null(Parms)) {
               keeprows <- Parms <- 1:length(x$Initial.Values)}
          else {
               if(is.numeric(Parms)) keeprows <- Parms
               if(is.character(Parms)) {
                    Parms <- sub("\\[","\\\\[",Parms)
                    Parms <- sub("\\]","\\\\]",Parms)
                    Parms <- sub("\\.","\\\\.",Parms)
                    if(length(grep(Parms[1], rownames(x))) == 0)
                         stop("Parameter in Parms does not exist.")
                    keeprows <- grep(Parms[1], rownames(x))
                    if(length(Parms) > 1) {
                         for (i in 2:length(Parms)) {
                              if(length(grep(Parms[i],
                                   rownames(x))) == 0)
                                   stop("Parameter in Parms does not exist.")
                              keeprows <- c(keeprows,
                                   grep(Parms[i], rownames(x)))}
                         }
                    }
               }
          temp <- x
          x <- matrix(x[keeprows,], length(keeprows),
               ncol(temp))
          rownames(x) <- rownames(temp)[keeprows]
          colnames(x) <- colnames(temp)
          if(x.lab != "SIR Samples") Modes <- x[,1]
          else Modes <- x[,6]
          if(x.lab != "SIR Samples") LB <- x[,3]
          else LB <- x[,5]
          if(x.lab != "SIR Samples") UB <- x[,4]
          else UB <- x[,7]
          ### Setup
          x.rows <- length(Modes)
          x.lim <- c(min(LB), max(UB))
          y.lim <- c(0, x.rows+1)
          ### Basic Plot
          plot(0, 0, ylim=y.lim, xlim=x.lim, main=Title, sub="",
               xlab=x.lab, ylab="", type="n", ann=TRUE, yaxt="n")
          abline(v=0, col="gray")
          ### Add Modes
          points(Modes, x.rows:1, pch=20)
          ### Add Horizontal Lines for 2.5%-97.5% Quantiles
          for (i in 1:x.rows) {
               lines(c(LB[i], UB[i]), c(x.rows-i+1, x.rows-i+1))}
          ### Add y-axis labels
          yy <- x.rows:1
          cex.labels <- 1/{log(x.rows)/5 + 1}
          axis(2, labels=rownames(x), tick=FALSE, las=1,
               at=yy, cex.axis=cex.labels)
          }
     else {
          x <- as.matrix(x)
          x.hpd <- p.interval(x, HPD=TRUE, MM=FALSE, prob=0.95)
          x.median <- apply(x, 2, median)
          x <- cbind(colMeans(x), sqrt(.colVars(x)),
               apply(x, 2, MCSE), ESS(x), x.hpd[,1],
               apply(x, 2, median), x.hpd[,2])
          rownames(x) <- rownames(x.hpd)
          colnames(x) <- c("Mean","SD","MCSE","ESS","LB","Median","UB")
          x.lab <- "All Samples"
          ### Setup
          x.rows <- nrow(x)
          x.lim <- c(min(x[,5]), max(x[,7]))
          y.lim <- c(0, x.rows+1)
          ### Basic Plot
          plot(0, 0, ylim=y.lim, xlim=x.lim, main=Title, sub="",
               xlab=x.lab, ylab="", type="n", ann=TRUE, yaxt="n")
          abline(v=0, col="gray")
          ### Add Medians
          points(x[,6], x.rows:1, pch=20)
          ### Add Horizontal Lines for 2.5%-97.5% Quantiles
          for (i in 1:x.rows) {
               lines(x[i,c(5,7)], c(x.rows-i+1, x.rows-i+1))}
          ### Add y-axis labels
          yy <- x.rows:1
          cex.labels <- 1 / {log(x.rows)/5 + 1}
          axis(2, labels=rownames(x), tick=FALSE, las=1, at=yy,
               cex.axis=cex.labels)
          }
     }

#End

###########################################################################
# plot.iterquad                                                           #
#                                                                         #
# The purpose of the plot.iterquad function is to plot an object of class #
# iterquad.                                                               #
###########################################################################

plot.iterquad <- function(x, Data=NULL, PDF=FALSE, Parms=NULL, ...)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     if(!inherits(x, "iterquad"))
          stop("x must be of class iterquad.")
     if(is.null(Data)) stop("The Data argument is NULL.")
     if(any(is.na(x$History))) stop("There is no history to plot.")
     ### Selecting Parms
     if(is.null(Parms)) {
          History <- x$History
          Posterior <- x$Posterior}
     else {
          Parms <- sub("\\[","\\\\[",Parms)
          Parms <- sub("\\]","\\\\]",Parms)
          Parms <- sub("\\.","\\\\.",Parms)
          if(length(grep(Parms[1], colnames(x$History))) == 0)
               stop("Parameter in Parms does not exist.")
          keepcols <- grep(Parms[1], colnames(x$History))
          if(length(Parms) > 1) {
               for (i in 2:length(Parms)) {
                    if(length(grep(Parms[i],
                         colnames(x$History))) == 0)
                         stop("Parameter in Parms does not exist.")
                    keepcols <- c(keepcols, grep(Parms[i],
                         colnames(x$History)))}}
          History <- as.matrix(x$History[,keepcols])
          colnames(History) <- colnames(x$History)[keepcols]
          if(all(!is.na(x$Posterior))) {
               Posterior <- as.matrix(x$Posterior[,keepcols])
               colnames(Posterior) <- colnames(History)}
          else Posterior <- x$Posterior
          }
     if(PDF == TRUE)
          {
          pdf("IterativeQuadrature.Plots.pdf")
          par(mfrow=c(2,2))
          }
     else {par(mfrow=c(2,2), ask=TRUE)}
     ### Plot Parameter
     for (j in 1:ncol(History))
          {
          plot(1:nrow(History), History[,j],
               type="l", xlab="Iterations", ylab="Value",
               main=colnames(History)[j])
          LPw <- x$LPw[,j]
          Mw <- x$M[,j]
          if(sum(Mw) > 0) Mw <- Mw / sum(Mw)
          Z <- x$Z[,j]
          LPw <- as.vector(by(LPw, Z, max))
          Mw <- as.vector(by(Mw, Z, max))
          Z <- unique(Z)
          o <- order(Z)
          LPw <- LPw[o]
          Mw <- Mw[o]
          Z <- Z[o]
          covdens <- dnorm(Z, History[nrow(History),j],
               sqrt(diag(x$Covar))[j])
          if(sum(covdens) > 0) covdens <- covdens / sum(covdens)
          if({x$Converged == TRUE} & !any(is.na(Posterior))) {
               dens <- density(Posterior[,j])
               if(sum(dens$y) > 0) dens$y <- dens$y / sum(dens$y)
               x.lim <- range(c(Z,dens$x))
               y.lim <- c(0, max(Mw, LPw, covdens, dens$y))
               }
          else {
               x.lim <- range(Z)
               y.lim <- c(0, max(Mw, LPw, covdens))
               }
          plot(Z, Mw, type="h", xlim=x.lim, ylim=y.lim, col="white",
               main=colnames(History)[j], xlab="Value",
               ylab="Normalized Density")
          polygon(c(Z, rev(Z)), c(rep(0,length(LPw)), rev(covdens)),
               col=rgb(0,255,0,50,maxColorValue=255), border=NA)
          if({x$Converged == TRUE} & !any(is.na(Posterior)))
               polygon(dens,
                    col=rgb(0,0,255,50,maxColorValue=255), border=NA)
          polygon(c(Z, rev(Z)),
               c(rep(0,length(Mw)),rev(Mw)),
               col=rgb(0,0,0,50,maxColorValue=255), border=NA)
          polygon(c(Z, rev(Z)),
               c(rep(0,length(LPw)),rev(LPw)),
               col=rgb(255,0,0,50,maxColorValue=255), border=NA)
          abline(v=0, col="red", lty=2)
          lines(Z, Mw, type="h")
          lines(Z, LPw, type="h", col="red")
          }
     ### Plot Deviance History
     plot(1:length(x$Deviance), x$Deviance, type="l", xlab="Iterations",
          ylab="Value", main="Deviance")
     ### Plot Monitor
     if({x$Converged == TRUE} & !any(is.na(x$Monitor))) {
          for (j in 1:ncol(x$Monitor)) {
               plot(density(x$Monitor[,j]),
                    xlab="Value", main=Data[["mon.names"]][j])
               polygon(density(x$Monitor[,j]),
                    col="black", border="black")
               abline(v=0, col="red", lty=2)}
          }
     if(PDF == TRUE) dev.off()
     }

#End

###########################################################################
# MISS                                                                    #
#                                                                         #
# The MISS function performs multiple imputation via sequential sampling. #
###########################################################################

MISS <- function(X, Iterations=100, Algorithm="GS", Fit=NULL, verbose=TRUE)
     {
     ### Initial Checks
     if(missing(X)) stop("X is a required argument.")
     if(!is.matrix(X)) stop("X is not a matrix.")
     N <- nrow(X)
     J <- ncol(X)
     if(J < 2) stop("At least 2 columns in X are required.")
     Nmiss <- apply(X, 2, function(x) sum(is.na(x)))
     if(sum(Nmiss) == 0) stop("There are no missing values in X.")
     cat("\nNumber of Missing Values by Variable:\n")
     print(Nmiss)
     cat("\n")
     if(Algorithm != "GS") stop("Algorithm unknown.")
     for (i in 1:N)
          if(sum(is.na(X[i,])) == J) stop("All missing row found.")
     ### Parameters and Variable Type
     if(is.null(Fit)) {
          Type <- rep(1, J)
          parm <- list()
          for (j in 1:J) {
               uniq <- unique(X[complete.cases(X[,j]),j])
               if(length(uniq) == 1) stop("Constant found.")
               else if(length(uniq) == 2) {
                    if(all(c(0,1) == uniq[order(uniq)])) {
                         ### Binary Logit or Robit
                         Type[j] <- 2
                         parm[[j]] <- list(z=rep(0, sum(!is.na(X[,j]))),
                              beta=rep(0, J),
                              lambda=rep(1, sum(!is.na(X[,j]))))
                         }
                    else parm[[j]] <- list(beta=rep(0,J), sigma=0)
                    }
               else {
                    ### Linear Regression
                    if(Algorithm == "ESS")
                         parm[[j]] <- list(beta=rep(0,J), gamma=0, sigma=0)
                    else parm[[j]] <- list(beta=rep(0,J), sigma=0)
                    }
               }
          rm(uniq)
          }
     else {
          if(!identical(class(Fit), "miss"))
               stop("Fit is not an object of class miss.")
          Algorithm <- Fit$Algorithm
          parm <- Fit$parm
          Type <- Fit$Type}
     ### Observed indicator matrix O
     varnames <- colnames(X)
     O <- matrix(TRUE, N, J)
     O[which(is.na(X))] <- FALSE
     ### Initial Missing Values
     if(is.null(Fit)) {
          for (j in 1:J)
               if(Type[j] == 1)
                    X[which(is.na(X[,j])),j] <- mean(X[,j], na.rm=TRUE)
               else X[which(is.na(X[,j])),j] <- 1
          }
     else {
          if(sum(Nmiss) != length(Fit$Imp[,1]))
               stop("Length of Initial.Missings differs from number missing.")
          X[which(is.na(X))] <- Fit$Imp[,ncol(Fit$Imp)]}
     Imp <- matrix(0, sum(Nmiss), Iterations)
     ### Multiple Imputation Samplers
     GibbsLinReg <- function(y, obs, X) {
          X <- cbind(1, as.matrix(X))
          Xobs <- X[obs,]
          yobs <- y[obs]
          XtX <- t(Xobs) %*% Xobs
          ridge <- 1e-5
          penalty <- ridge * diag(XtX)
          if(length(penalty) == 1) penalty <- matrix(penalty)
          v <- as.inverse(as.symmetric.matrix(XtX + diag(penalty)))
          coef <- t(yobs %*% Xobs %*% v)
          resid <- yobs - Xobs %*% coef
          df <- max(sum(obs) - ncol(Xobs), 1)
          sigma <- sqrt(sum((resid)^2) / rchisq(1, df))
          beta <- coef + {t(chol({v + t(v)} / 2)) %*%
               rnorm(ncol(Xobs))} * sigma
          imp <- X[!obs,] %*% beta + rnorm(sum(!obs)) * sigma
          out <- list(imp=imp, beta=beta, sigma=sigma)
          return(out)}
     GibbsRobit <- function(y, obs, X, z, beta, lambda) {
          X <- cbind(1, as.matrix(X))
          Xobs <- X[obs,]
          yobs <- y[obs]
          n <- length(yobs)
          nu <- 8
          mu <- Xobs %*% beta
          z[yobs==0] <- qnorm(runif(n, 0, pnorm(0, mu, sqrt(1/lambda))),
               mu, sqrt(1/lambda))[yobs==0]
          z[yobs==1] <- qnorm(runif(n, pnorm(0, mu, sqrt(1/lambda)),1),
               mu, sqrt(1/lambda))[yobs==1]
          W <- diag(lambda)
          vbeta <- as.inverse(as.symmetric.matrix(t(Xobs) %*% W %*% Xobs))
          betahat <- vbeta %*% {t(Xobs) %*% W %*% z}
          beta <- c(rmvn(1, t(betahat), vbeta))
          lambda <- rgamma(n, {nu + 1} / 2,
               scale=2/(nu + {z - Xobs %*% beta}^2))
          mu <- X[!obs,] %*% beta
          eta <- invlogit(mu)
          imp <- rbern(length(eta), eta)
          out <- list(imp=imp, z=z, beta=beta, lambda=lambda)
          }
     ### Main Loop
     cat("\nImputation begins...\n")
     for (i in 1:Iterations) {
          cat("\nIteration:", i, "  ")
          imp <- NULL
          for (j in 1:J) {
               if(Nmiss[j] > 0) {
                    if(verbose == TRUE) cat(" V", j, " ", sep="")
                    if(Algorithm == "GS" & Type[j] == 1) {
                         out <- GibbsLinReg(y=X[,j], obs=O[,j], X=X[,-j])
                         parm[[j]]$beta <- out$beta
                         parm[[j]]$sigma <- out$sigma
                         }
                    else if(Algorithm == "GS" & Type[j] == 2) {
                         out <- GibbsRobit(y=X[,j], obs=O[,j],
                              X=X[,-j],
                              z=parm[[j]]$z, beta=parm[[j]]$beta,
                              lambda=parm[[j]]$lambda)
                         parm[[j]]$z <- out$z
                         parm[[j]]$beta <- out$beta
                         parm[[j]]$sigma <- out$sigma
                         }
                    X[,j][!O[,j]] <- out$imp
                    imp <- c(imp, out$imp)}
               if(j == J) Imp[,i] <- imp}}
     cat("\n\nEstimating Posterior Modes...")
     PostMode <- apply(Imp, 1, Mode)
     cat("\nFinished.\n")
     out <- list(Algorithm=Algorithm, Imp=Imp, parm=parm,
          PostMode=PostMode, Type=Type)
     class(out) <- "miss"
     return(out)
     }

#End

###########################################################################
# BMK.Diagnostic                                                          #
#                                                                         #
# The purpose of the BMK.Diagnostic function is to estimate Hellinger     #
# distances between consecutive batches of posterior samples, so that     #
# when Hellinger distances are below a given threshold the portion of the #
# chain may suggest MCMC convergence. Although the code is slightly       #
# different, it is similar to the bmkconverge function in the BMK         #
# package.                                                                #
###########################################################################

BMK.Diagnostic <- function(X, batches=10)
     {
     HD.Batch <- function(batch1, batch2)
          {
          n1 <- nrow(as.matrix(batch1))
          batches.combined <- c(batch1, batch2)
          batches.min <- min(batches.combined)
          batches.max <- max(batches.combined)
          P1 <- try(density(batch1, from=batches.min, to=batches.max,
               n=n1), silent=TRUE)
          Q1 <- try(density(batch2, from=batches.min, to=batches.max,
               n=n1), silent=TRUE)
          if(inherits(P1, "try-error")) P1 <- density(rnorm(n1,0,1))
          if(inherits(Q1, "try-error")) Q1 <- density(rnorm(n1,1,1))
          step1 <- P1$x[2] - P1$x[1]
          diver1 <- (sqrt(P1$y) - sqrt(Q1$y))^2 * step1
          out <- sqrt(sum(diver1) / 2)
          return(out)
          }
     HD.Diag <- function(x, batch.size, batch.list)
          {
          x <- as.vector(x)
          c1 <- 0
          for (i in 1:(length(batch.list)-1)) {
               batch.label1 <- batch.list[i]:(batch.list[i+1]-1)
               batch.label2 <- batch.list[i+1]:((i+1)*batch.size)
               HD <- try(HD.Batch(x[batch.label1], x[batch.label2]),
                    silent=TRUE)
               if(inherits(HD, "try-error")) HD <- 0
               c1 <- c(c1, HD)}
          c1 <- c1[-1]
          return(c1)
          }
     ### Initial Checks
     if(!is.matrix(X) & !identical(class(X), "demonoid"))
          stop("X must be a matrix or an object of class demonoid.")
     if(identical(class(X), "demonoid")) X <- X$Posterior1
     n.iter <- nrow(X)
     n.par <- ncol(X)
     batch.size <- floor(n.iter / batches)
     if(n.iter %% batch.size != 0)
          stop("Batches of even size are required.")
     batch.list <- seq(from=1, to=n.iter, by=batch.size)
     size <- floor(n.iter / batch.size) - 1
     out <- matrix(0, n.par, size)
     ### Hellinger Distance
     for (i in 1:n.par) {out[i,] <- HD.Diag(X[,i], batch.size, batch.list)}
     ### Constrain to the interval [0,1]
     d <- dim(out)
     out <- as.vector(out)
     out.num <- which(out < 0)
     out[out.num] <- 0
     out.num <- which(out > 1)
     out[out.num] <- 1
     out <- array(out, dim=d)
     ### Output
     if(is.null(colnames(X)))
          rownames(out) <- paste("V", 1:ncol(X), sep="")
     else rownames(out) <- colnames(X)
     colnames(out) <- (1:size)*batch.size
     class(out) <- "bmk"
     return(out)
     }

#End

###########################################################################
# LaplacesDemon.hpc                                                       #
#                                                                         #
# The purpose of the LaplacesDemon.hpc function is to extend              #
# LaplacesDemon to parallel processing on multiple cores.                 #
###########################################################################

#I don't know how to 'shut up' the makeCluster function...
#no luck with capture.output, sink, or invisible

LaplacesDemon.hpc <- function(Model, Data, Initial.Values, Covar=NULL,
     Iterations=10000, Status=100, Thinning=10, Algorithm="MWG",
     Specs=list(B=NULL), Debug=list(DB.chol=FALSE, DB.eigen=FALSE,
     DB.MCSE=FALSE, DB.Model=TRUE), LogFile="", Chains=2, CPUs=2,
     Type="PSOCK", Packages=NULL, Dyn.libs=NULL)
     {
     detectedCores <- max(detectCores(), as.integer(Sys.getenv("NSLOTS")),
          na.rm=TRUE)
     cat("\n\nCPUs Detected:", detectedCores, "\n", file=LogFile,
          append=TRUE)
     if(CPUs > detectedCores) {
          cat("\nOnly", detectedCores, "will be used.\n", file=LogFile,
               append=TRUE)
          CPUs <- detectedCores}
     if(is.vector(Initial.Values)) {
          Initial.Values <- matrix(Initial.Values, Chains,
               length(Initial.Values), byrow=TRUE)
          cat("\nWarning: initial values were a vector, and are now a",
               file=LogFile, append=TRUE)
          cat("\n", Chains, "x", length(Initial.Values), "matrix.\n",
               file=LogFile, append=TRUE)}
     if(Algorithm == "INCA" && Chains != CPUs) {
          Chains <- CPUs
          cat("\nINCA:", Chains, "chains will be used\n")}
     cat("\nLaplace's Demon is preparing environments for CPUs...",
          file=LogFile, append=TRUE)
     cat("\n##################################################\n",
          file=LogFile, append=TRUE)
     cl <- makeCluster(CPUs, Type)
     cat("\n##################################################\n",
          file=LogFile, append=TRUE)
     on.exit({stopCluster(cl); cat("\n\nLaplace's Demon has finished.\n", file=LogFile, append=TRUE)})
     Packages <- c(Packages, "LaplacesDemon")
     varlist <- unique(c(ls(), ls(envir=.GlobalEnv),
          ls(envir=parent.env(environment()))))
     clusterExport(cl, varlist=varlist, envir=environment())
     clusterSetRNGStream(cl)
     wd <- getwd()
     clusterExport(cl, varlist=c("Packages", "Dyn.libs", "wd"),
          envir=environment())
     demon.wrapper <- function(x, ...)
          {
          if(!is.null(Packages)) {
               sapply(Packages,
                    function(x) library(x, character.only=TRUE,
                         quietly=TRUE))}
          if(!is.null(Dyn.libs)) {
               sapply(Dyn.libs,
                    function(x) dyn.load(paste(wd, x, sep = "/")))
               on.exit(sapply(Dyn.libs,
                    function(x) dyn.unload(paste(wd, x, sep = "/"))))}
          LaplacesDemon(Model, Data, Initial.Values[x,],
               Covar, Iterations, Status, Thinning,
               Algorithm, Specs, Debug,
               LogFile=paste(LogFile, ".", x, sep=""))
          }
     cat("\nStatus messages are not displayed for parallel processing.",
          file=LogFile, append=TRUE)
     cat("\nLaplace's Demon is beginning parallelization...\n",
          file=LogFile, append=TRUE)
     if(Algorithm == "INCA") {
          ### Start hpc server
          system(paste("Rscript -e 'library(parallel);library(LaplacesDemon);server_Listening(n=",CPUs,")'", sep=""), wait=FALSE)
          cat("Start hpc server...\n", file=LogFile, append=TRUE)
          ### Export chain number
          clusterExport(cl, varlist="Chains", envir=environment())
          ### Connect each process to server_Listening with 0.5s time delay
          clusterEvalQ(cl, con <- NULL)
          doCon <- function(i) {
               Sys.sleep(i/2)
               con <<- socketConnection("localhost", 19009, blocking=TRUE,
                    open="r+")}
          clusterExport(cl, varlist="doCon", envir=environment())
          expr <- NULL
          for (i in 1:CPUs) {
               tmp <- parse(text=paste("doCon(", i,")", sep=""))
               expr <- c(expr, tmp)}
          clusterApply(cl, expr, eval, env=.GlobalEnv)
          cat("\nOpen connections to hpc server...", file=LogFile,
               append=TRUE)}
     LaplacesDemon.out <- clusterApply(cl, 1:Chains, demon.wrapper,
          Model, Data, Initial.Values, Covar, Iterations, Status,
          Thinning, Algorithm, Specs, Debug)
     class(LaplacesDemon.out) <- "demonoid.hpc"
     if(Algorithm == "INCA") {
          ### Stop server_Listening
          clusterEvalQ(cl, {close(con)})
          cat("\nClose connections to hpc server...", file=LogFile,
               append=TRUE)
          }
     return(LaplacesDemon.out)
     }

#End

###########################################################################
# Heidelberger.Diagnostic                                                 #
#                                                                         #
# The purpose of the Heidelberger.Diagnostic function is to perform the   #
# Heidelberger and Welch MCMC convergence diagnostic on Markov chains.    #
###########################################################################

Heidelberger.Diagnostic <- function(x, eps=0.1, pvalue=0.05) 
     {
     if(missing(x)) stop("x is a required argument")
     if(!identical(class(x), "demonoid"))
          stop("x must be an object of class demonoid.")
     if(all(is.na(x$Posterior2))) x <- x$Posterior1
     else x <- x$Posterior2
     HW.mat0 <- matrix(0, ncol=6, nrow=ncol(x))
     dimnames(HW.mat0) <- list(colnames(x),
          c("stest", "start", "pvalue", "htest", "mean", "halfwidth"))
     HW.mat <- HW.mat0
     spectrum0 <- function(x, max.freq=0.5, order=1, max.length=200)
          {
          x <- as.matrix(x)
          if(!is.null(max.length) && nrow(x) > max.length) {
               batch.size <- ceiling(nrow(x) / max.length)
               x <- aggregate(ts(x, frequency=batch.size), nfreq=1,
                    FUN=mean)
               }
          else batch.size <- 1
          out <- do.spectrum0(x, max.freq=max.freq, order=order)
          out$spec <- out$spec * batch.size
          return(out)
          }
     do.spectrum0 <- function(x, max.freq=0.5, order=1)
          {
          fmla <- switch(order+1, spec ~ one, spec ~ f1, spec ~ f1 + f2)
          if(is.null(fmla)) stop("invalid order")
          N <- nrow(x)
          Nfreq <- floor(N/2)
          freq <- seq(from=1/N, by=1/N, length=Nfreq)
          f1 <- sqrt(3) * (4 * freq - 1)
          f2 <- sqrt(5) * (24 * freq^2 - 12 * freq + 1)
          v0 <- numeric(ncol(x))
          for (i in 1:ncol(x)) {
               y <- x[,i]
               v <- var(y, na.rm=TRUE)
               if(!is.finite(v)) v <- 0
               if(v == 0) v0[i] <- 0
               else {
                    yfft <- fft(y)
                    spec <- Re(yfft * Conj(yfft)) / N
                    spec.data <- data.frame(one=rep(1, Nfreq), f1=f1,
                         f2=f2, spec=spec[1 + (1:Nfreq)],
                         inset=I(freq <= max.freq))
                    glm.out <- try(glm(fmla, family=Gamma(link="log"),
                         data=spec.data), silent=TRUE)
                    if(!inherits(glm.out, "try-error"))
                         v0[i] <- predict(glm.out, type="response",
                              newdata=data.frame(spec=0, one=1,
                              f1=-sqrt(3), f2=sqrt(5)))
                    else v0[i] <- 0}}
          return(list(spec=v0))
          }
     pcramer <- function(q, eps=1.0e-5)
          {
          log.eps <- log(eps)
          y <- matrix(0, nrow=4, ncol=length(q))
          for (k in 0:3) {
               z <- gamma(k + 0.5) * sqrt(4*k + 1) /
                    (gamma(k+1) * pi^(3/2) * sqrt(q))
               u <- (4*k + 1)^2/(16*q)
               y[k+1,] <- ifelse(u > -log.eps, 0,
                    z * exp(-u) * besselK(x=u, nu=1/4))}
          return(colSums(y))
          }
     ### Heidelberger and Welch Diagnostic
     for (j in 1:ncol(x)) {
          start.vec <- seq(from=1, to=nrow(x)/2, by=nrow(x)/10)
          Y <- x[, j, drop=TRUE]
          n1 <- length(Y)
          ### Schruben's test for convergence, applied sequentially
          S0 <- spectrum0(Y[(n1/2):n1])$spec
          converged <- FALSE
          for (i in seq(along=start.vec)) {
               Y <- Y[start.vec[i]:length(Y)]
               n <- length(Y)
               ybar <- mean(Y)
               B <- cumsum(Y) - ybar * (1:n)
               Bsq <- (B * B) / (n * S0)
               I <- sum(Bsq) / n
               if(converged <- !is.na(I) && pcramer(I) < 1 - pvalue)
                    break}
          ### Recalculate S0 using section of chain that passed convergence test
          S0ci <- spectrum0(Y)$spec
          halfwidth <- 1.96 * sqrt(S0ci/n)
          passed.hw <- !is.na(halfwidth) & (abs(halfwidth/ybar) <= eps)
          if(!converged || is.na(I) || is.na(halfwidth)) {
               nstart <- NA
               passed.hw <- NA
               halfwidth <- NA
               ybar <- NA
               }
          else nstart <- start(Y)[1]
          HW.mat[j, ] <- c(converged, nstart, 1 - pcramer(I), 
               passed.hw, ybar, halfwidth)}
     class(HW.mat) <- "heidelberger"
     return(HW.mat)
     }

#End

###########################################################################
# CSF                                                                     #
#                                                                         #
# The purpose of the CSF function is to provide a visual MCMC diagnostic  #
# based on the cumulative sample function (CSF).                          #
###########################################################################

CSF <- function(x, name, method="Quantiles", quantiles=c(.025,.5,.975),
     output=FALSE)
     {
     if(missing(x)) stop("The x argument is required.")
     if(missing(name)) name <- "x"
     if(is.constant(x)) stop("x must not be constant.")
     if(!is.vector(x)) x <- as.vector(x)
     if(method == "ESS") {
          y <- rep(0, length(x))
          for (i in 1:length(x)) {
               test <- try(ESS(x[1:i]), silent=TRUE)
               if(!inherits(test, "try-error")) y[i] <- test}
          plot(y, type="l", xlab="Cumulative Sample", ylab="ESS")
          if(output == TRUE) return(y)}
     if(method == "Geweke.Diagnostic") {
          y <- rep(0, length(x))
          for (i in 1:length(x)) {
               test <- try(Geweke.Diagnostic(x[1:i]), silent=TRUE)
               if(!inherits(test, "try-error")) y[i] <- test}
          par(mfrow=c(2,1))
          plot(1:length(x), x, type="l", xlab="Iterations", ylab=name)
          panel.smooth(1:length(x), x, pch="")
          plot(y, type="l", xlab="Cumulative Sample",
               ylab="Geweke Diagnostic")
          abline(h=2, lty=2, col="red"); abline(h=-2, lty=2, col="red")
          if(output == TRUE) return(y)}
     if(method == "HPD") {
          Y <- matrix(0, length(x), 2)
          for (i in 1:length(x)) {
               test <- try(as.vector(p.interval(x[1:i], HPD=TRUE,
                    MM=FALSE)[1,]), silent=TRUE)
               if(!inherits(test, "try-error")) Y[i,] <- test}
          plot(x, type="l", col="gray", xlab="Sample Size",
               ylab="HPD (95%)")
          for (i in 1:2) lines(Y[,i], col="black")
          if(output == TRUE) return(Y)}
     if(method == "is.stationary") {
          y <- rep(FALSE, length(x))
          for (i in 1:length(x)) {
               test <- try(is.stationary(x[1:i]), silent=TRUE)
               if(!inherits(test, "try-error")) y[i] <- test}
          par(mfrow=c(2,1))
          plot(1:length(x), x, type="l", xlab="Iterations", ylab=name)
          panel.smooth(1:length(x), x, pch="")
          plot(y, type="l", xlab="Cumulative Sample",
               ylab="Stationary Indicator")
          if(output == TRUE) return(y)}
     if(method == "Kurtosis") {
          kurtosis <- function(x) {  
               m4 <- mean((x-mean(x))^4) 
               kurt <- m4/(sd(x)^4)-3  
               return(kurt)}
          y <- rep(0, length(x))
          for (i in 1:length(x)) {
               test <- try(kurtosis(x[1:i]), silent=TRUE)
               if(!inherits(test, "try-error")) y[i] <- test}
          par(mfrow=c(2,1))
          plot(1:length(x), x, type="l", xlab="Iterations", ylab=name)
          panel.smooth(1:length(x), x, pch="")
          plot(y, type="l", xlab="Cumulative Sample", ylab="Kurtosis")
          if(output == TRUE) return(y)}
     if(method == "MCSE") {
          y <- rep(1, length(x))
          for (i in 1:length(x)) {
               test <- try(MCSE(x[1:i]), silent=TRUE)
               if(!inherits(test, "try-error")) y[i] <- test}
          y[which(!is.finite(y))] <- 1
          y[which(y > 1)] <- 1
          plot(y, type="l", xlab="Cumulative Sample", ylab="MCSE")
          if(output == TRUE) return(y)}
     if(method == "MCSE.bm") {
          y <- rep(1, length(x))
          for (i in 1:length(x)) {
               test <- try(MCSE(x[1:i], method="batch.means"), silent=TRUE)
               if(!inherits(test, "try-error")) y[i] <- test}
          y[which(!is.finite(y))] <- 1
          y[which(y > 1)] <- 1
          plot(y, type="l", xlab="Cumulative Sample", ylab="MCSE")
          if(output == TRUE) return(y)}
     if(method == "MCSE.sv") {
          y <- rep(1, length(x))
          for (i in 1:length(x)) {
               test <- try(MCSE(x[1:i], method="sample.variance"),
                    silent=TRUE)
               if(!inherits(test, "try-error")) y[i] <- test}
          y[which(!is.finite(y))] <- 1
          y[which(y > 1)] <- 1
          plot(y, type="l", xlab="Cumulative Sample", ylab="MCSE")
          if(output == TRUE) return(y)}
     if(method == "Mean") {
          y <- rep(0, length(x))
          for (i in 1:length(x)) {
               test <- try(mean(x[1:i]), silent=TRUE)
               if(!inherits(test, "try-error")) y[i] <- test}
          par(mfrow=c(2,1))
          plot(1:length(x), x, type="l", xlab="Iterations", ylab=name)
          panel.smooth(1:length(x), x, pch="")
          plot(y, type="l", xlab="Cumulative Sample", ylab="Mean")
          if(output == TRUE) return(y)}
     if(method == "Mode") {
          y <- rep(0, length(x))
          for (i in 1:length(x)) {
               test <- try(Mode(x[1:i]), silent=TRUE)
               if(!inherits(test, "try-error")) y[i] <- test[1]}
          par(mfrow=c(2,1))
          plot(1:length(x), x, type="l", xlab="Iterations", ylab=name)
          panel.smooth(1:length(x), x, pch="")
          plot(y, type="l", xlab="Cumulative Sample", ylab="Mode")
          if(output == TRUE) return(y)}
     if(method == "N.Modes") {
          y <- rep(1, length(x))
          for (i in 1:length(x)) {
               test <- try(Modes(x[1:i]), silent=TRUE)
               if(!inherits(test, "try-error")) y[i] <- length(test$modes)}
          par(mfrow=c(2,1))
          plot(1:length(x), x, type="l", xlab="Iterations", ylab=name)
          panel.smooth(1:length(x), x, pch="")
          plot(y, type="l", xlab="Cumulative Sample", ylab="Number of Modes")
          if(output == TRUE) return(y)}
     if(method == "Precision") {
          y <- rep(0, length(x))
          for (i in 1:length(x)) {
               test <- try(var(x[1:i]), silent=TRUE)
               if(!inherits(test, "try-error")) y[i] <- 1 / test}
          par(mfrow=c(2,1))
          plot(1:length(x), x, type="l", xlab="Iterations", ylab=name)
          panel.smooth(1:length(x), x, pch="")
          plot(y, type="l", xlab="Cumulative Sample", ylab="Precision")
          if(output == TRUE) return(y)}
     if(method == "Quantiles") {
          Y <- matrix(0, length(x), length(quantiles))
          for (i in 1:length(x)) {
               test <- try(quantile(x[1:i], probs=quantiles), silent=TRUE)
               if(!inherits(test, "try-error")) Y[i,] <- test}
          plot(x, type="l", col="gray", xlab="Sample Size", ylab="Quantiles")
          for (i in 1:ncol(Y)) lines(Y[,i], col="black")
          if(output == TRUE) return(Y)}
     if(method == "Skewness") {
          skewness <-  function(x) {
               m3 <- mean((x-mean(x))^3)
               skew <- m3/(sd(x)^3)
               return(skew)}
          y <- rep(0, length(x))
          for (i in 1:length(x)) {
               test <- try(skewness(x[1:i]), silent=TRUE)
               if(!inherits(test, "try-error")) y[i] <- test}
          par(mfrow=c(2,1))
          plot(1:length(x), x, type="l", xlab="Iterations", ylab=name)
          panel.smooth(1:length(x), x, pch="")
          plot(y, type="l", xlab="Cumulative Sample", ylab="Skewness")
          if(output == TRUE) return(y)}
     }

#End

###########################################################################
# Gelman.Diagnostic                                                       #
#                                                                         #
# The purpose of the Gelman.Diagnostic function is to perform the         #
# Gelman-Rubin MCMC diagnostic on a set of posterior samples. This        #
# function is similar to the gelman.diag function in the coda package,    #
# but has been modified to work with objects of class demonoid.           #
###########################################################################

Gelman.Diagnostic <- function(x, confidence=0.95, transform=FALSE)
     {
     Nchain <- length(x)
     if(Nchain < 2) stop("More than one chain is required.")
     if(!all(sapply(x, class) == "demonoid"))
        stop("At least one item in list x is not of class demonoid.")
     Burn <- Ntot <- Niter <- Nvar <- rep(0, Nchain)
     for (i in 1:Nchain) {
          Burn[i] <- x[[i]]$Rec.BurnIn.Thinned
          Ntot[i] <- nrow(x[[i]]$Posterior1)
          if(Burn[i] >= Ntot[i]) Niter[i] <- Ntot[i]
          if(Burn[i] < Ntot[i]) Niter[i] <- Ntot[i] - Burn[i]
          Nvar[i] <- x[[i]]$Parameters}
     if(length(unique(Ntot)) != 1)
          stop("Total number of iterations differs with demonoid objects.")
     Ntot <- Ntot[1]
     Burn <- max(Burn)
     Niter <- min(Niter)
     if(length(unique(Nvar)) != 1)
          stop("Total number of parameters differs with demonoid objects.")
     else Nvar <- Nvar[1]
     xnames <- colnames(x[[1]]$Posterior1)
     if(transform == TRUE) {
          Gelman.Transform <- function(x, Nvar, Nchain)
               {
               for (i in 1:Nchain) {
                    for (j in 1:Nvar) {
                         if(min(x[[i]][,j]) > 0) {
                              if(max(x[[i]][,j]) < 1) {
                                   x[[i]][,j] <- log(x[[i]][,j] /
                                   (1 - x[[i]][,j]))}
                              else x[[i]][,j] <- log(x[[i]][,j])}
                         }
                    }
               return(x)
               }
          }
     ## Multivariate (upper case)
     if(Burn < Ntot) {
          for (i in 1:Nchain) {
               x[[i]]$Posterior1 <- x[[i]]$Posterior1[Burn:Ntot,]}}
     else warning("Non-stationary samples were used.")
     temp <- unlist(x, recursive=FALSE)
     x <- temp[names(temp) == "Posterior1"]
     if(transform == TRUE) x <- Gelman.Transform(x, Nvar, Nchain)
     S2 <- array(sapply(x, var, simplify=TRUE), dim=c(Nvar,Nvar,Nchain))
     W <- apply(S2, c(1,2), mean)
     if(Nvar > 1){
          xbar <- matrix(sapply(x, apply, 2, mean, simplify=TRUE),
               nrow=Nvar, ncol=Nchain)
          }
     else {
          xbar <- matrix(sapply(x, mean, simplify=TRUE), nrow=Nvar,
               ncol=Nchain)}
     B <- Niter * var(t(xbar))
     if(Nvar > 1) {
          CW <- chol(W)
          emax <- eigen(backsolve(CW, t(backsolve(CW, B,
               transpose=TRUE)), transpose=TRUE),
               symmetric=TRUE, only.values=TRUE)$values[1]
          mpsrf <- sqrt((1 - 1/Niter) + (1 + 1/Nvar) * emax/Niter)
          }
     else mpsrf <- NULL
     ## Univariate (lower case)
     w <- diag(W)
     b <- diag(B)
     s2 <- matrix(apply(S2, 3, diag), nrow=Nvar, ncol=Nchain)
     muhat <- rowMeans(xbar)
     var.w <- .rowVars(s2) / Nchain
     var.b <- (2 * b^2) / (Nchain - 1)
     cov.wb <- (Niter / Nchain) * diag(var(t(s2), t(xbar^2)) -
          2 * muhat * var(t(s2), t(xbar)))
     V <- (Niter - 1) * w / Niter + (1 + 1/Nchain) * b / Niter
     var.V <- ((Niter - 1)^2 * var.w + (1 + 1/Nchain)^2 *
          var.b + 2 * (Niter - 1) * (1 + 1/Nchain) * cov.wb) / Niter^2
     df.V <- (2 * V^2) / var.V
     df.adj <- (df.V + 3) / (df.V + 1)
     B.df <- Nchain - 1
     W.df <- (2 * w^2) / var.w
     R2.fixed <- (Niter - 1) / Niter
     R2.random <- (1 + 1/Nchain) * (1/Niter) * (b/w)
     R2.estimate <- R2.fixed + R2.random
     R2.upper <- R2.fixed + qf((1 + confidence)/2, B.df, W.df) * R2.random
     psrf <- cbind(sqrt(df.adj * R2.estimate), sqrt(df.adj * R2.upper))
     dimnames(psrf) <- list(xnames, c("Point Est.", "Upper C.I."))
     out <- list(PSRF=psrf, MPSRF=mpsrf)
     return(out)
     }

#End

###########################################################################
# Log-Log                                                                 #
#                                                                         #
# The logit and probit links are symmetric, because the probabilities     #
# approach zero or one at the same rate. The log-log and complementary    #
# log-log links are asymmetric. Complementary log-log links approach zero #
# slowly and one quickly. Log-log links approach zero quickly and one     #
# slowly. Either the log-log or complementary log-log link will tend to   #
# fit better than logistic and probit, and are frequently used when the   #
# probability of an event is small or large. A mixture of the two links,  #
# the log-log and complementary log-log is often used, where each link is #
# weighted. The reason that logit is so prevalent is because logistic     #
# parameters can be interpreted as odds ratios.                           #
###########################################################################

loglog <- function(p)
     {
     if({any(p < 0)} || {any(p > 1)}) stop("p must be in [0,1].")
     x <- log(-log(p))
     return(x)
     }
invloglog <- function(x)
     {
     p <- exp(-exp(x))
     return(p)
     }
cloglog <- function(p)
     {
     if({any(p < 0)} || {any(p > 1)}) stop("p must be in [0,1].")
     x <- log(-log(1 - p))
     return(x)
     }
invcloglog <- function(x)
     {
     p <- 1 - exp(-exp(x))
     return(p)
     }

#End

###########################################################################
# is.model                                                                #
#                                                                         #
# The purpose of the is.model function is to estimate if a model          #
# specification function meets some minimum criteria.                     #
###########################################################################

is.model <- function(Model, Initial.Values, Data)
     {
     if(missing(Model)) stop("The Model argument is required.")
     ismodel <- TRUE
     if(!is.function(Model)) {
          cat("\nModel must be a function.\n")
          ismodel <- FALSE}
     if(missing(Initial.Values))
          stop("Initial.Values argument is required.")
     if(!is.vector(Initial.Values))
          stop("Initial.Values must be a vector.")
     if(missing(Data)) stop("The Data argument is required.")
     if(!is.data(Data)) stop("The Data argument is not Data.")
     if(!identical(length(Initial.Values), length(Data[["parm.names"]])))
          stop("Lengths of Initial.Values and parm.names differ.")
     Mo <- try(Model(Initial.Values, Data), silent=TRUE)
     if(inherits(Mo, "try-error")) stop("Error in executing the Model.")
     if(!is.list(Mo)) {
          cat("\nModel must return a list.\n")
          ismodel <- FALSE}
     else if(length(Mo) != 5) {
          cat("\nModel must return 5 list components.\n")
          ismodel <- FALSE}
     else if(!identical(Mo[[1]], Mo[["LP"]])) {
          cat("\nThe first output component must be named LP.\n")
          ismodel <- FALSE}
     else if(length(Mo[["LP"]]) != 1) {
          cat("\nThe length of LP must be 1.\n")
          ismodel <- FALSE}
     else if(!identical(Mo[[2]], Mo[["Dev"]])) {
          cat("\nThe second output component must be named Dev.\n")
          ismodel <- FALSE}
     else if(length(Mo[["Dev"]]) != 1) {
          cat("\nThe length of Dev must be 1.\n")
          ismodel <- FALSE}
     else if(!identical(Mo[[3]], Mo[["Monitor"]])) {
          cat("\nThe third output component must be named Monitor.\n")
          ismodel <- FALSE}
     else if(!identical(length(Mo[["Monitor"]]),
          length(Data[["mon.names"]]))) {
          cat("\nThe lengths of Monitor values and mon.names differ.\n")
          ismodel <- FALSE}
     else if(!identical(Mo[[4]], Mo[["yhat"]])) {
          cat("\nThe fourth output component must be named yhat.\n")
          ismodel <- FALSE}
     else if(!identical(Mo[[5]], Mo[["parm"]])) {
          cat("\nThe fifth output component must be named parm.\n")
          ismodel <- FALSE}
     else if(!identical(length(Mo[["parm"]]),
          length(Data[["parm.names"]]))) {
          cat("\nThe lengths of parm and parm.names differ.\n")
          ismodel <- FALSE}
     return(ismodel)
     }

#End

###########################################################################
# VariationalBayes                                                        #
#                                                                         #
# The purpose of the VariationalBayes function is to estimate a model     #
# with a Variational Bayes algorithm.                                     #
###########################################################################

VariationalBayes <- function(Model, parm, Data, Covar=NULL,
     Interval=1.0E-6, Iterations=1000, Method="Salimans2", Samples=1000,
     sir=TRUE, Stop.Tolerance=1.0E-5, CPUs=1, Type="PSOCK")
     {
     ##########################  Initial Checks  ##########################
     time1 <- proc.time()
     VB.call <- match.call()
     if(missing(Model)) stop("Model is a required argument.")
     if(!is.function(Model)) stop("Model must be a function.")
     if(missing(Data)) stop("Data is a required argument.")
     if(missing(parm)) {
          cat("Initial values were not supplied, and\n")
          cat("have been set to zero prior to VariationalBayes().\n")
          parm <- rep(0, length(Data[["parm.names"]]))}
     LIV <- length(as.vector(parm))
     for (i in 1:length(Data)) {
          if(is.matrix(Data[[i]])) {
               if(all(is.finite(Data[[i]]))) {
                    mat.rank <- qr(Data[[i]], tol=1e-10)$rank
                    if(mat.rank < ncol(Data[[i]])) {
                         cat("WARNING: Matrix", names(Data)[[i]],
                              "may be rank-deficient.\n")}}}}
     if({Interval <= 0} | {Interval > 1}) Interval <- 1.0E-6
     Iterations <- min(max(round(Iterations), 10), 1000000)
     "%!in%" <- function(x,table) return(match(x, table, nomatch=0) == 0)
     if(Method %!in% c("Salimans2"))
          stop("Method is unknown.")
     if(Stop.Tolerance <= 0) Stop.Tolerance <- 1.0E-5
     as.character.function <- function(x, ... )
          {
          fname <- deparse(substitute(x))
          f <- match.fun(x)
          out <- c(sprintf('"%s" <- ', fname), capture.output(f))
          if(grepl("^[<]", tail(out,1))) out <- head(out, -1)
          return(out)
          }
     acount <- length(grep("apply", as.character.function(Model)))
     if(acount > 0) {
          cat("Suggestion:", acount, " possible instance(s) of apply functions\n")
          cat(     "were found in the Model specification. Iteration speed will\n")
          cat("     increase if apply functions are vectorized in R or coded\n")
          cat("     in a faster language such as C++ via the Rcpp package.\n")}
     acount <- length(grep("for", as.character.function(Model)))
     if(acount > 0) {
          cat("Suggestion:", acount, " possible instance(s) of for loops\n")
          cat("     were found in the Model specification. Iteration speed will\n")
          cat("     increase if for loops are vectorized in R or coded in a\n")
          cat("     faster language such as C++ via the Rcpp package.\n")}
     ### Sample Size of Data
     if(!is.null(Data[["n"]])) if(length(Data[["n"]]) == 1) N <- Data[["n"]] 
     if(!is.null(Data[["N"]])) if(length(Data[["N"]]) == 1) N <- Data[["N"]] 
     if(!is.null(Data[["y"]])) N <- nrow(matrix(Data[["y"]]))
     if(!is.null(Data[["Y"]])) N <- nrow(matrix(Data[["Y"]]))
     if(!is.null(N)) cat("Sample Size: ", N, "\n")
     else stop("Sample size of Data not found in n, N, y, or Y.")
     ###########################  Preparation  ############################
     m.old <- Model(parm, Data)
     if(!is.list(m.old)) stop("Model must return a list.")
     if(length(m.old) != 5) stop("Model must return five components.")
     if(any(names(m.old) != c("LP","Dev","Monitor","yhat","parm")))
          stop("Name mismatch in returned list of Model function.")
     if(length(m.old[["LP"]]) > 1) stop("Multiple joint posteriors exist!")
     if(!identical(length(parm), length(m.old[["parm"]])))
          stop("The number of initial values and parameters differs.")
     if(!is.finite(m.old[["LP"]])) {
          cat("Generating initial values due to a non-finite posterior.\n")
          if(!is.null(Data[["PGF"]]))
               Initial.Values <- GIV(Model, Data, PGF=TRUE)
          else Initial.Values <- GIV(Model, Data, PGF=FALSE)
          m.old <- Model(Initial.Values, Data)
          }
     if(!is.finite(m.old[["LP"]])) stop("The posterior is non-finite.")
     if(!is.finite(m.old[["Dev"]])) stop("The deviance is non-finite.")
     parm <- m.old[["parm"]]
     if(!identical(Model(m.old[["parm"]], Data)[["LP"]], m.old[["LP"]])) {
          cat("WARNING: LP differs when initial values are held constant.\n")
          cat("     Derivatives may be problematic if used.\n")}
     #####################  Begin Variational Bayes  #####################
     cat("Variational Bayes begins...\n")
     if(Method == "Salimans2") {
          VB <- .vbsalimans2(Model, parm, Data, Covar, Iterations,
               Interval, Stop.Tolerance, m.old)
          }
     Dev <- as.vector(VB$Dev)
     iter <- VB$iter
     parm.len <- LIV
     parm.new <- VB$parm.new
     parm.old <- VB$parm.old
     post <- VB$post
     Step.Size <- VB$Step.Size
     tol.new <- VB$tol.new
     VarCov <- VB$VarCov
     rm(VB)
     if(iter == 1) stop("VariationalBayes stopped at iteration 1.")
     if(tol.new <= Stop.Tolerance) converged <- TRUE
     else converged <- FALSE
     ### Column names to samples
     if(dim(post)[2] == length(Data[["parm.names"]]))
          dimnames(post) <- list(1:dim(post)[1], Data[["parm.names"]], 1:2)
     #################  Sampling Importance Resampling  ##################
     if({sir == TRUE} & {converged == TRUE}) {
          cat("Sampling from Posterior with Sampling Importance Resampling\n")
          posterior <- SIR(Model, Data, mu=parm.new, Sigma=VarCov,
               n=Samples, CPUs=CPUs, Type=Type)
          Mon <- matrix(0, nrow(posterior), length(Data[["mon.names"]]))
          dev <- rep(0, nrow(posterior))
          for (i in 1:nrow(posterior)) {
               mod <- Model(posterior[i,], Data)
               dev[i] <- mod[["Dev"]]
               Mon[i,] <- mod[["Monitor"]]
               }
          colnames(Mon) <- Data[["mon.names"]]}
     else {
          if({sir == TRUE} & {converged == FALSE})
               cat("Posterior samples are not drawn due to Converge=FALSE\n")
          posterior <- NA; Mon <- NA}
     #####################  Summary, Point-Estimate  ######################
     cat("Creating Summary from Point-Estimates\n")
     Summ1 <- matrix(NA, parm.len, 4, dimnames=list(Data[["parm.names"]],
          c("Mean","SD","LB","UB")))
     Summ1[,1] <- parm.new
     Summ1[,2] <- sqrt(diag(VarCov))
     Summ1[,3] <- parm.new - 2*Summ1[,2]
     Summ1[,4] <- parm.new + 2*Summ1[,2]
     ###################  Summary, Posterior Samples  ####################
     Summ2 <- NA
     if({sir == TRUE} & {converged == TRUE}) {
          cat("Creating Summary from Posterior Samples\n")
          Summ2 <- matrix(NA, ncol(posterior), 7,
               dimnames=list(Data[["parm.names"]],
                    c("Mean","SD","MCSE","ESS","LB","Median","UB")))
          Summ2[,1] <- colMeans(posterior)
          Summ2[,2] <- sqrt(.colVars(posterior))
          Summ2[,3] <- Summ2[,2] / sqrt(nrow(posterior))
          Summ2[,4] <- rep(nrow(posterior), ncol(posterior))
          Summ2[,5] <- apply(posterior, 2, quantile, c(0.025))
          Summ2[,6] <- apply(posterior, 2, quantile, c(0.500))
          Summ2[,7] <- apply(posterior, 2, quantile, c(0.975))
          Deviance <- rep(0, 7)
          Deviance[1] <- mean(dev)
          Deviance[2] <- sd(dev)
          Deviance[3] <- sd(dev) / sqrt(nrow(posterior))
          Deviance[4] <- nrow(posterior)
          Deviance[5] <- as.numeric(quantile(dev, probs=0.025, na.rm=TRUE))
          Deviance[6] <- as.numeric(quantile(dev, probs=0.500, na.rm=TRUE))
          Deviance[7] <- as.numeric(quantile(dev, probs=0.975, na.rm=TRUE))
          Summ2 <- rbind(Summ2, Deviance)
          for (j in 1:ncol(Mon)) {
               Monitor <- rep(NA,7)
               Monitor[1] <- mean(Mon[,j])
               Monitor[2] <- sd(as.vector(Mon[,j]))
               Monitor[3] <- sd(as.vector(Mon[,j])) / sqrt(nrow(Mon))
               Monitor[4] <- nrow(Mon)
               Monitor[5] <- as.numeric(quantile(Mon[,j], probs=0.025,
                    na.rm=TRUE))
               Monitor[6] <- as.numeric(quantile(Mon[,j], probs=0.500,
                    na.rm=TRUE))
               Monitor[7] <- as.numeric(quantile(Mon[,j], probs=0.975,
                    na.rm=TRUE))
               Summ2 <- rbind(Summ2, Monitor)
               rownames(Summ2)[nrow(Summ2)] <- Data[["mon.names"]][j]
               }
          }
     ###############  Logarithm of the Marginal Likelihood  ###############
     LML <- list(LML=NA, VarCov=VarCov)
     if({sir == TRUE} & {converged == TRUE}) {
          cat("Estimating Log of the Marginal Likelihood\n")
          lml <- LML(theta=posterior, LL=(dev*(-1/2)), method="NSIS")
          LML[[1]] <- lml[[1]]}
     else if({sir == FALSE} & {converged == TRUE}) {
          cat("Estimating Log of the Marginal Likelihood\n")
          LML <- LML(Model, Data, Modes=parm.new, Covar=VarCov,
               method="LME")}
     colnames(VarCov) <- rownames(VarCov) <- Data[["parm.names"]]
     time2 <- proc.time()
     #############################  Output  ##############################
     VB <- list(Call=VB.call,
          Converged=converged,
          Covar=VarCov,
          Deviance=as.vector(Dev),
          History=post,
          Initial.Values=parm,
          Iterations=iter,
          LML=LML[[1]],
          LP.Final=as.vector(Model(parm.new, Data)[["LP"]]),
          LP.Initial=m.old[["LP"]],
          Minutes=round(as.vector(time2[3] - time1[3]) / 60, 2),
          Monitor=Mon,
          Posterior=posterior,
          Step.Size.Final=Step.Size,
          Step.Size.Initial=Step.Size,
          Summary1=Summ1,
          Summary2=Summ2,
          Tolerance.Final=tol.new,
          Tolerance.Stop=Stop.Tolerance)
     class(VB) <- "vb"
     cat("Variational Bayes is finished.\n\n")
     return(VB)
     }
.vbsalimans2 <- function(Model, parm, Data, Covar, Iterations, Interval,
     Stop.Tolerance, m.old)
     {
     m.new <- m.old
     LIV <- length(parm)
     m <- parm
     if(is.null(Covar)) V <- diag(LIV) #Variance
     else {
          V <- as.positive.definite(Covar)
          if(nrow(V) != ncol(V)) stop("Covar is not square.")
          if(nrow(V) != LIV) V <- diag(LIV)
          else {
               V <- Covar
               diag(V) <- abs(diag(V))}}
     rm(Covar)
     z <- m #Guess of the mean
     P <- as.inverse(V) #Precision
     a <- rep(0, LIV)
     zbar <- rep(0, LIV)
     Pbar <- matrix(0, LIV, LIV)
     abar <- rep(0, LIV)
     w <- 1 / sqrt(Iterations) #step-size
     half1 <- Iterations / 2
     half2 <- 2 / Iterations
     post <- array(0, dim=c(Iterations, LIV, 2))
     post[1,,1] <- m
     post[1,,2] <- diag(V)
     Dev <- matrix(m.old[["Dev"]],1,1)
     ### Stochastic Approximation
     for (iter in 1:Iterations) {
          mbar <- mbar.last <- m #mean
          Vbar <- Vbar.last <- V #variance
          m.old <- m.new #model
          ### Print Status
          if(iter %% round(Iterations / 10) == 0) {
               cat("Iteration: ", iter, " of ", Iterations, "\n")}
          ### Draw a sample from the approximating distribution q
          xstar <- m
          while(identical(xstar, m)) {
               xstar <- try(m +
                    as.vector(matrix(rnorm(LIV),1,LIV) %*% chol(V)),
                    silent=TRUE)
               if(inherits(xstar, "try-error"))
                    xstar <- rnorm(LIV,m,abs(diag(V)))
               m.temp <- try(Model(xstar, Data), silent=TRUE)
               if(inherits(xstar, "try-error")) xstar <- m
               else xstar <- m.temp[["parm"]]
               if(any(!is.finite(c(m.temp[["LP"]], m.temp[["Dev"]],
                    m.temp[["Monitor"]]))))
                    xstar <- m}
          ### Gradient and Hessian
          g <- partial(Model, xstar, Data, Interval=Interval)
          H <- Hessian(Model, xstar, Data, Interval=Interval)
          ### Stochastic Approx.
          a <- (1 - w)*a + w*g
          P <- (1 - w)*P - w*H
          z <- (1 - w)*z + w*xstar
          if(any(!is.finite(P))) P <- diag(LIV)
          if(!is.symmetric.matrix(P))
               P <- as.symmetric.matrix(P)
          diag(P) <- abs(diag(P))
          if(!is.positive.definite(P))
               P <- as.positive.definite(P)
          V <- as.inverse(P)
          m <- as.vector(V %*% a + z)
          ### Evaluate Proposal
          m.new <- try(Model(m, Data), silent=TRUE)
          if(inherits(m.new, "try-error")) m.new <- m.old
          else if(any(!is.finite(c(m.new[["LP"]], m.new[["Dev"]],
               m.new[["Monitor"]]))))
               m.new <- m.old
          else if(log(runif(1)) >= (m.new[["LP"]] - m.old[["LP"]]))
               m.new <- m.old
          m <- m.new[["parm"]]
          ### Storage
          post[iter,,1] <- m
          post[iter,,2] <- diag(V)
          Dev <- rbind(Dev, m.new[["Dev"]])
          ### Do averaging if over half-way
          if(iter > half1) {
               abar <- abar + half2*g
               Pbar <- Pbar - half2*H
               zbar <- zbar + half2*xstar
               if(any(!is.finite(Pbar))) Pbar <- diag(LIV)
               if(!is.symmetric.matrix(Pbar))
                    Pbar <- as.symmetric.matrix(Pbar)
               diag(Pbar) <- abs(diag(Pbar))
               if(!is.positive.definite(Pbar))
                    Pbar <- as.positive.definite(Pbar)
               Vbar <- as.inverse(Pbar)
               mbar <- as.vector(Vbar %*% abar + zbar)
               mbar <- Model(mbar, Data)[["parm"]]
               ### Tolerance
               tol.new <- sum(sqrt(sum({mbar - mbar.last} *
                    {mbar - mbar.last})),
                    sqrt(sum({diag(Vbar) - diag(Vbar.last)} *
                    {diag(Vbar) - diag(Vbar.last)})))
               if(tol.new <= Stop.Tolerance) {
                    post <- post[1:iter,,]
                    break}
               }
          }
     Dev <- Dev[-1,]
     #mbar and Vbar should be returned, but not if wildly different...
     LB <- mbar - 3*diag(Vbar)
     UB <- mbar + 3*diag(Vbar)
     if(any((m < LB) | (m > UB))) {
          mbar <- m
          Vbar <- diag(LIV) * diag(V)}
     ### Output
     VB <- list(Dev=Dev, iter=iter, parm.new=mbar, parm.old=parm,
          post=post, Step.Size=w, tol.new=tol.new, VarCov=Vbar)
     return(VB)
     }

#End

###########################################################################
# IterativeQuadrature                                                     #
#                                                                         #
# The purpose of the IterativeQuadrature function is to perform iterative #
# quadrature on a Bayesian model.                                         #
###########################################################################

IterativeQuadrature <- function(Model, parm, Data, Covar=NULL,
     Iterations=100, Algorithm="CAGH", Specs=NULL, Samples=1000, sir=TRUE,
     Stop.Tolerance=c(1e-5,1e-15), CPUs=1, Type="PSOCK")
     {
     cat("\nIterativeQuadrature was called on ", date(), "\n", sep="")
     time1 <- proc.time()
     IQ.call <- match.call()
     ##########################  Initial Checks  ##########################
     cat("\nPerforming initial checks...\n")
     if(missing(Model)) stop("Model is a required argument.")
     if(!is.function(Model)) stop("Model must be a function.")
     if(missing(Data)) stop("Data is a required argument.")
     if(missing(parm)) {
          cat("Initial values were not supplied, and\n")
          cat("have been set to zero prior to LaplaceApproximation().\n")
          parm <- rep(0, length(Data[["parm.names"]]))}
     for (i in 1:length(Data)) {
          if(is.matrix(Data[[i]])) {
               if(all(is.finite(Data[[i]]))) {
                    mat.rank <- qr(Data[[i]], tol=1e-10)$rank
                    if(mat.rank < ncol(Data[[i]])) {
                         cat("WARNING: Matrix", names(Data)[[i]],
                              "may be rank-deficient.\n")}}}}
     Iterations <- min(max(round(Iterations), 2), 1000000)
     "%!in%" <- function(x,table) return(match(x, table, nomatch=0) == 0)
     if(Algorithm %!in% c("AGH","AGHSG","CAGH"))
          stop("Algorithm is unknown.")
     if(Algorithm == "AGH") {
          Algorithm <- "Adaptive Gauss-Hermite"
          if(missing(Specs)) stop("The Specs argument is required.")
          if(length(Specs) != 4) stop("The Specs argument is incorrect.")
          if(Specs[["N"]] == Specs[[1]])
               N <- max(abs(round(Specs[[1]])), 2)
          if(Specs[["Nmax"]] == Specs[[2]])
               Nmax <- max(abs(round(Specs[[2]])), N)
          Packages <- NULL
          if(!is.null(Specs[["Packages"]]))
               Packages <- Specs[["Packages"]]
          Dyn.libs <- NULL
          if(!is.null(Specs[["Dyn.libs"]]))
               Dyn.libs <- Specs[["Dyn.libs"]]
          }
     else if(Algorithm == "AGHSG") {
          Algorithm <- "Adaptive Gauss-Hermite Sparse Grid"
          if(missing(Specs)) stop("The Specs argument is required.")
          if(length(Specs) != 4) stop("The Specs argument is incorrect.")
          if(Specs[["K"]] == Specs[[1]])
               K <- max(abs(round(Specs[[1]])), 2)
          if(Specs[["Kmax"]] == Specs[[2]])
               Kmax <- max(abs(round(Specs[[2]])), K)
          Packages <- NULL
          if(!is.null(Specs[["Packages"]]))
               Packages <- Specs[["Packages"]]
          Dyn.libs <- NULL
          if(!is.null(Specs[["Dyn.libs"]]))
               Dyn.libs <- Specs[["Dyn.libs"]]
          }
     else if(Algorithm == "CAGH") {
          Algorithm <- "Componentwise Adaptive Gauss-Hermite"
          if(missing(Specs)) stop("The Specs argument is required.")
          if(length(Specs) != 4) stop("The Specs argument is incorrect.")
          if(Specs[["N"]] == Specs[[1]])
               N <- max(abs(round(Specs[[1]])), 2)
          if(Specs[["Nmax"]] == Specs[[2]])
               Nmax <- max(abs(round(Specs[[2]])), N)
          Packages <- NULL
          if(!is.null(Specs[["Packages"]]))
               Packages <- Specs[["Packages"]]
          Dyn.libs <- NULL
          if(!is.null(Specs[["Dyn.libs"]]))
               Dyn.libs <- Specs[["Dyn.libs"]]
          }
     if(any(Stop.Tolerance <= 0)) Stop.Tolerance <- c(1e-5,1e-15)
     as.character.function <- function(x, ... )
          {
          fname <- deparse(substitute(x))
          f <- match.fun(x)
          out <- c(sprintf('"%s" <- ', fname), capture.output(f))
          if(grepl("^[<]", tail(out,1))) out <- head(out, -1)
          return(out)
          }
     acount <- length(grep("apply", as.character.function(Model)))
     if(acount > 0) {
          cat("Suggestion:", acount, " possible instance(s) of apply functions\n")
          cat(     "were found in the Model specification. Iteration speed will\n")
          cat("     increase if apply functions are vectorized in R or coded\n")
          cat("     in a faster language such as C++ via the Rcpp package.\n")}
     acount <- length(grep("for", as.character.function(Model)))
     if(acount > 0) {
          cat("Suggestion:", acount, " possible instance(s) of for loops\n")
          cat("     were found in the Model specification. Iteration speed will\n")
          cat("     increase if for loops are vectorized in R or coded in a\n")
          cat("     faster language such as C++ via the Rcpp package.\n")}
     ###########################  Preparation  ############################
     m.old <- Model(parm, Data)
     if(!is.list(m.old)) stop("Model must return a list.")
     if(length(m.old) != 5) stop("Model must return five components.")
     if(any(names(m.old) != c("LP","Dev","Monitor","yhat","parm")))
          stop("Name mismatch in returned list of Model function.")
     if(length(m.old[["LP"]]) > 1) stop("Multiple joint posteriors exist!")
     if(!identical(length(parm), length(m.old[["parm"]])))
          stop("The number of initial values and parameters differs.")
     if(!is.finite(m.old[["LP"]])) {
          cat("Generating initial values due to a non-finite posterior.\n")
          if(!is.null(Data[["PGF"]]))
               Initial.Values <- GIV(Model, Data, PGF=TRUE)
          else Initial.Values <- GIV(Model, Data, PGF=FALSE)
          m.old <- Model(Initial.Values, Data)
          }
     if(!is.finite(m.old[["LP"]])) stop("The posterior is non-finite.")
     if(!is.finite(m.old[["Dev"]])) stop("The deviance is non-finite.")
     parm <- m.old[["parm"]]
     LIV <- length(parm)
     ScaleF <- 2.381204 * 2.381204 / LIV
     if(is.null(Covar)) Covar <- diag(LIV) * ScaleF
     if(is.vector(Covar))
          if(identical(length(Covar), LIV)) Covar <- diag(LIV) * Covar
     if(!is.matrix(Covar)) stop("Covar must be a matrix.")
     if(!is.square.matrix(Covar)) stop("Covar must be a square matrix.")
     if(!is.symmetric.matrix(Covar)) Covar <- as.symmetric.matrix(Covar)
     ####################  Begin Iterative Quadrature  ####################
     cat("Algorithm:", Algorithm, "\n")
     cat("\nIterativeQuadrature is beginning to update...\n")
     if(Algorithm == "Adaptive Gauss-Hermite") {
          IQ <- .iqagh(Model, parm, Data, Covar, Iterations,
               Stop.Tolerance, CPUs, m.old, N, Nmax, Packages, Dyn.libs)
          }
     else if(Algorithm == "Adaptive Gauss-Hermite Sparse Grid") {
          IQ <- .iqaghsg(Model, parm, Data, Covar, Iterations,
               Stop.Tolerance, CPUs, m.old, K, Kmax, Packages, Dyn.libs)
          }
     else if(Algorithm == "Componentwise Adaptive Gauss-Hermite") {
          IQ <- .iqcagh(Model, parm, Data, Covar, Iterations,
               Stop.Tolerance, CPUs, m.old, N, Nmax, Packages, Dyn.libs)
          }
     VarCov <- IQ$Covar
     Dev <- as.vector(IQ$Dev)
     iter <- IQ$iter
     LPw <- IQ$LPw
     M <- IQ$M
     mu <- IQ$mu
     N <- IQ$N
     parm.len <- ncol(mu)
     parm.new <- mu[nrow(mu),]
     parm.old <- IQ$parm.old
     tol <- IQ$tol
     Z <- IQ$Z
     rm(IQ)
     if(iter == 1) stop("IterativeQuadrature stopped at iteration 1.")
     converged <- FALSE
     if({tol[1] <= Stop.Tolerance[1]} & {tol[2] <= Stop.Tolerance[2]})
          converged <- TRUE
     ### Column names to samples
     if(ncol(mu) == length(Data[["parm.names"]]))
          colnames(mu) <- Data[["parm.names"]]
     rownames(mu) <- 1:nrow(mu)
     cat("\n\n")
     #################  Sampling Importance Resampling  ##################
     if({sir == TRUE} & {converged == TRUE}) {
          cat("Sampling from Posterior with Sampling Importance Resampling\n")
          posterior <- SIR(Model, Data, mu=parm.new, Sigma=VarCov,
               n=Samples, CPUs=CPUs, Type=Type)
          Mon <- matrix(0, nrow(posterior), length(Data[["mon.names"]]))
          dev <- rep(0, nrow(posterior))
          for (i in 1:nrow(posterior)) {
               mod <- Model(posterior[i,], Data)
               dev[i] <- mod[["Dev"]]
               Mon[i,] <- mod[["Monitor"]]
               }
          colnames(Mon) <- Data[["mon.names"]]}
     else {
          if({sir == TRUE} & {converged == FALSE})
               cat("Posterior samples are not drawn due to Converge=FALSE\n")
          posterior <- NA; Mon <- NA}
     #####################  Summary, Point-Estimate  ######################
     cat("Creating Summary from Point-Estimates\n")
     Summ1 <- matrix(NA, parm.len, 4, dimnames=list(Data[["parm.names"]],
          c("Mean","SD","LB","UB")))
     Summ1[,1] <- parm.new
     Summ1[,2] <- sqrt(diag(VarCov))
     Summ1[,3] <- parm.new - 2*Summ1[,2]
     Summ1[,4] <- parm.new + 2*Summ1[,2]
     ###################  Summary, Posterior Samples  ####################
     Summ2 <- NA
     if({sir == TRUE} & {converged == TRUE}) {
          cat("Creating Summary from Posterior Samples\n")
          Summ2 <- matrix(NA, ncol(posterior), 7,
               dimnames=list(Data[["parm.names"]],
                    c("Mean","SD","MCSE","ESS","LB","Median","UB")))
          Summ2[,1] <- colMeans(posterior)
          Summ2[,2] <- sqrt(.colVars(posterior))
          Summ2[,3] <- Summ2[,2] / sqrt(nrow(posterior))
          Summ2[,4] <- rep(nrow(posterior), ncol(posterior))
          Summ2[,5] <- apply(posterior, 2, quantile, c(0.025))
          Summ2[,6] <- apply(posterior, 2, quantile, c(0.500))
          Summ2[,7] <- apply(posterior, 2, quantile, c(0.975))
          Deviance <- rep(0, 7)
          Deviance[1] <- mean(dev)
          Deviance[2] <- sd(dev)
          Deviance[3] <- sd(dev) / sqrt(nrow(posterior))
          Deviance[4] <- nrow(posterior)
          Deviance[5] <- as.numeric(quantile(dev, probs=0.025, na.rm=TRUE))
          Deviance[6] <- as.numeric(quantile(dev, probs=0.500, na.rm=TRUE))
          Deviance[7] <- as.numeric(quantile(dev, probs=0.975, na.rm=TRUE))
          Summ2 <- rbind(Summ2, Deviance)
          for (j in 1:ncol(Mon)) {
               Monitor <- rep(NA,7)
               Monitor[1] <- mean(Mon[,j])
               Monitor[2] <- sd(as.vector(Mon[,j]))
               Monitor[3] <- sd(as.vector(Mon[,j])) / sqrt(nrow(Mon))
               Monitor[4] <- nrow(Mon)
               Monitor[5] <- as.numeric(quantile(Mon[,j], probs=0.025,
                    na.rm=TRUE))
               Monitor[6] <- as.numeric(quantile(Mon[,j], probs=0.500,
                    na.rm=TRUE))
               Monitor[7] <- as.numeric(quantile(Mon[,j], probs=0.975,
                    na.rm=TRUE))
               Summ2 <- rbind(Summ2, Monitor)
               rownames(Summ2)[nrow(Summ2)] <- Data[["mon.names"]][j]
               }
          }
     ###############  Logarithm of the Marginal Likelihood  ###############
     LML <- list(LML=NA, VarCov=VarCov)
     if({sir == TRUE} & {converged == TRUE}) {
          cat("Estimating Log of the Marginal Likelihood\n")
          lml <- LML(theta=posterior, LL=(dev*(-1/2)), method="NSIS")
          LML[[1]] <- lml[[1]]}
     else if({sir == FALSE} & {converged == TRUE}) {
          cat("Estimating Log of the Marginal Likelihood\n")
          LML <- LML(Model, Data, Modes=parm.new, Covar=VarCov,
               method="LME")}
     colnames(VarCov) <- rownames(VarCov) <- Data[["parm.names"]]
     time2 <- proc.time()
     #############################  Output  ##############################
     cat("Creating Output\n")
     IQ <- list(Algorithm=Algorithm,
          Call=IQ.call,
          Converged=converged,
          Covar=VarCov,
          Deviance=as.vector(Dev),
          History=mu,
          Initial.Values=parm,
          Iterations=iter,
          LML=LML[[1]],
          LP.Final=as.vector(Model(parm.new, Data)[["LP"]]),
          LP.Initial=m.old[["LP"]],
          LPw=LPw,
          M=M,
          Minutes=round(as.vector(time2[3] - time1[3]) / 60, 2),
          Monitor=Mon,
          N=N,
          Posterior=posterior,
          Summary1=Summ1,
          Summary2=Summ2,
          Tolerance.Final=tol,
          Tolerance.Stop=Stop.Tolerance,
          Z=Z)
     class(IQ) <- "iterquad"
     cat("\nIterativeQuadrature is finished.\n")
     return(IQ)
     }
.iqagh <- function(Model, parm, Data, Covar, Iterations, Stop.Tolerance,
     CPUs, m.old, N, Nmax, Packages, Dyn.libs)
     {
     if(CPUs == 1) {
          Dev <- matrix(m.old[["Dev"]],1,1)
          LPworst <- m.old[["LP"]]
          parm.len <- length(parm)
          if(parm.len > 10)
               warning("The number of parameters may be too large.")
          mu <- rbind(parm)
          parm.old <- parm
          colnames(mu) <- Data[["parm.names"]]
          sigma <- sqrt(diag(Covar))
          rule <- GaussHermiteCubeRule(N=N, dims=parm.len)
          X <- rule$nodes
          W <- rule$weights
          n <- length(W)
          count <- 0
          expand <- FALSE
          IE <- 1
          tol <- c(1,1)
          ### Begin Iterative Quadrature
          options(warn=-1)
          cat("\nIteration: 1,   Univariate Nodes: ", N,
               ",   Multivariate Nodes: ", n, sep="")
          for (iter in 2:Iterations) {
               ### New Quadrature Rule
               if({expand == TRUE} & {N < Nmax}) {
                    N <- min(N+1, Nmax)
                    rule <- GaussHermiteCubeRule(N=N, dims=parm.len)
                    X <- rule$nodes
                    W <- rule$weights
                    n <- length(W)}
               adjust <- FALSE
               expand <- FALSE
               cat("\nIteration: ", iter, ",   Univariate Nodes: ", N,
                    ",   Multivariate Nodes: ", n, sep="")
               LP <- rep(LPworst, n)
               LPw <- M <- Z <- matrix(0, n, parm.len)
               mu <- rbind(mu, 0)
               ### Evaluate at the Nodes
               for (i in 1:n) {
                    Z[i,] <- mu[iter-1,] + sqrt(2)*sigma*X[i,]
                    mod <- Model(Z[i,], Data)
                    if(all(is.finite(c(mod[["LP"]], mod[["Dev"]],
                         mod[["Monitor"]]))))
                         LP[i] <- mod[["LP"]]
                    Z[i,] <- mod[["parm"]]}
               ### Weights
               LPw <- matrix(exp(LP - logadd(LP)), n, parm.len)
               LPw <- LPw / matrix(colSums(LPw), n, parm.len)
               LPw[which(!is.finite(LPw))] <- 0
               if(all(LPw == 0)) {
                    expand <- TRUE
                    LPw <- rep(.Machine$double.eps, n)}
               ### Adapt mu and Sigma
               mu[iter,] <- colSums(LPw * Z)
               temp <- cov.wt(Z, wt=LPw[,1])$cov
               if(all(is.finite(temp))) Covar <- as.symmetric.matrix(temp)
               else expand <- TRUE
               diag(Covar) <- abs(diag(Covar))
               diag(Covar)[which(diag(Covar) < .Machine$double.eps)] <- .Machine$double.eps
               M <- W * exp(X) * sqrt(2) *
                    matrix(sigma, n, parm.len, byrow=TRUE)
               sigma <- sqrt(diag(Covar))
               mod <- Model(mu[iter,], Data)
               Dev <- rbind(Dev, mod[["Dev"]])
               ### Accept Only Improved mu
               m.old <- Model(mu[iter-1,], Data)
               if(all(is.finite(c(mod[["LP"]], mod[["Dev"]],
                    mod[["Monitor"]])))) {
                    if(m.old[["LP"]] >= mod[["LP"]]) {
                         expand <- TRUE
                         mu[iter,] <- mu[iter-1,]}
                    }
               else {
                    expand <- TRUE
                    mu[iter,] <- mu[iter-1,]}
               ### Integration Error and Tolerance
               Mw <- M / matrix(colSums(M), n, parm.len, byrow=TRUE)
               IE <- c(IE, mean(abs(Mw - LPw)))
               tol[1] <- sqrt(sum({mu[iter,] - mu[iter-1,]}^2))
               tol[2] <- abs(IE[iter] - IE[iter-1])
               if(tol[1] <= Stop.Tolerance[1]) {
                    expand <- TRUE
                    if(tol[2] <= Stop.Tolerance[2]) {
                         if(count > 0) break
                         count <- count + 1}
                    }
               else count <- 0
               }
          }
     else {
          detectedCores <- detectCores()
          cat("\n\nCPUs Detected:", detectedCores, "\n")
          if(CPUs > detectedCores) {
               cat("\nOnly", detectedCores, "will be used.\n")
               CPUs <- detectedCores}
          cat("\nLaplace's Demon is preparing environments for CPUs...")
          cat("\n##################################################\n")
          cl <- makeCluster(CPUs)
          cat("\n##################################################\n")
          on.exit(stopCluster(cl))
          varlist <- unique(c(ls(), ls(envir=.GlobalEnv),
               ls(envir=parent.env(environment()))))
          clusterExport(cl, varlist=varlist, envir=environment())
          clusterSetRNGStream(cl)
          wd <- getwd()
          clusterExport(cl, varlist=c("Packages", "Dyn.libs", "wd"),
               envir=environment())
          Dev <- matrix(m.old[["Dev"]],1,1)
          LPworst <- m.old[["LP"]]
          parm.len <- length(parm)
          if(parm.len > 10)
               warning("The number of parameters may be too large.")
          mu <- rbind(parm)
          parm.old <- parm
          colnames(mu) <- Data[["parm.names"]]
          sigma <- sqrt(diag(Covar))
          rule <- GaussHermiteCubeRule(N=N, dims=parm.len)
          X <- rule$nodes
          W <- rule$weights
          n <- length(W)
          count <- 0
          expand <- FALSE
          IE <- 1
          tol <- c(1,1)
          ### Begin Iterative Quadrature
          options(warn=-1)
          cat("\nIteration: 1,   Univariate Nodes: ", N,
               ",   Multivariate Nodes: ", n, sep="")
          for (iter in 2:Iterations) {
               ### New Quadrature Rule
               if({expand == TRUE} & {N < Nmax}) {
                    N <- min(N+1, Nmax)
                    rule <- GaussHermiteCubeRule(N=N, dims=parm.len)
                    X <- rule$nodes
                    W <- rule$weights
                    n <- length(W)}
               adjust <- FALSE
               expand <- FALSE
               cat("\nIteration: ", iter, ",   Univariate Nodes: ", N,
                    ",   Multivariate Nodes: ", n, sep="")
               LP <- rep(LPworst, n)
               LPw <- M <- Z <- matrix(0, n, parm.len)
               mu <- rbind(mu, 0)
               ### Evaluate at the Nodes
               Z <- matrix(mu[iter-1,], n, parm.len, byrow=TRUE) +
                    sqrt(2) * matrix(sigma, n, parm.len, byrow=TRUE) * X
               mod <- parLapply(cl, 1:n, function(x) Model(Z[x,], Data))
               LP <- as.vector(unlist(lapply(mod, function(x) x[["LP"]])))
               Z <- matrix(as.vector(unlist(lapply(mod,
                    function(x) x[["parm"]]))), n, parm.len, byrow=TRUE)
               LP[which(!is.finite(LP))] <- LPworst
               ### Weights
               LPw <- matrix(exp(LP - logadd(LP)), n, parm.len)
               LPw <- LPw / matrix(colSums(LPw), n, parm.len)
               LPw[which(!is.finite(LPw))] <- 0
               if(all(LPw == 0)) {
                    expand <- TRUE
                    LPw <- rep(.Machine$double.eps, n)}
               ### Adapt mu and Sigma
               mu[iter,] <- colSums(LPw * Z)
               temp <- cov.wt(Z, wt=LPw[,1])$cov
               if(all(is.finite(temp))) Covar <- as.symmetric.matrix(temp)
               else expand <- TRUE
               diag(Covar) <- abs(diag(Covar))
               diag(Covar)[which(diag(Covar) < .Machine$double.eps)] <- .Machine$double.eps
               M <- W * exp(X) * sqrt(2) *
                    matrix(sigma, n, parm.len, byrow=TRUE)
               sigma <- sqrt(diag(Covar))
               mod <- Model(mu[iter,], Data)
               Dev <- rbind(Dev, mod[["Dev"]])
               ### Accept Only Improved mu
               m.old <- Model(mu[iter-1,], Data)
               if(all(is.finite(c(mod[["LP"]], mod[["Dev"]],
                    mod[["Monitor"]])))) {
                    if(m.old[["LP"]] >= mod[["LP"]]) {
                         expand <- TRUE
                         mu[iter,] <- mu[iter-1,]}
                    }
               else {
                    expand <- TRUE
                    mu[iter,] <- mu[iter-1,]}
               ### Integration Error and Tolerance
               Mw <- M / matrix(colSums(M), n, parm.len, byrow=TRUE)
               IE <- c(IE, mean(abs(Mw - LPw)))
               tol[1] <- sqrt(sum({mu[iter,] - mu[iter-1,]}^2))
               tol[2] <- abs(IE[iter] - IE[iter-1])
               if(tol[1] <= Stop.Tolerance[1]) {
                    expand <- TRUE
                    if(tol[2] <= Stop.Tolerance[2]) {
                         if(count > 0) break
                         count <- count + 1}
                    }
               else count <- 0
               }
          }
     options(warn=0)
     ### Output
     IQ <- list(Covar=Covar, Dev=Dev, iter=iter, LPw=LPw, M=M,
          mu=mu, N=n, parm.old=parm.old, tol=tol, Z=Z)
     return(IQ)
     }
.iqaghsg <- function(Model, parm, Data, Covar, Iterations, Stop.Tolerance,
     CPUs, m.old, K, Kmax, Packages, Dyn.libs)
     {
     if(CPUs == 1) {
          Dev <- matrix(m.old[["Dev"]],1,1)
          LPworst <- m.old[["LP"]]
          parm.len <- length(parm)
          if(parm.len > 10)
               warning("The number of parameters may be too large.")
          mu <- rbind(parm)
          parm.old <- parm
          colnames(mu) <- Data[["parm.names"]]
          sigma <- sqrt(diag(Covar))
          rule <- SparseGrid(J=parm.len, K=K)
          X <- rule$nodes
          W <- rule$weights
          N <- length(W)
          count <- 0
          expand <- FALSE
          IE <- 1
          tol <- c(1,1)
          ### Begin Iterative Quadrature
          options(warn=-1)
          cat("\nIteration: 1,   K: ", K, ",   Nodes: ", N, sep="")
          for (iter in 2:Iterations) {
               ### New Quadrature Rule
               if({expand == TRUE} & {K < Kmax}) {
                    K <- min(K+1, Kmax)
                    rule <- SparseGrid(J=parm.len, K=K)
                    X <- rule$nodes
                    W <- rule$weights
                    N <- length(W)}
               expand <- FALSE
               cat("\nIteration: ", iter, ",   K: ", K, ",   Nodes: ", N,
                    sep="")
               LP <- rep(LPworst, N)
               LPw <- M <- Z <- matrix(0, N, parm.len)
               mu <- rbind(mu, 0)
               ### Evaluate at the Nodes
               for (i in 1:N) {
                    Z[i,] <- mu[iter-1,] + sqrt(2)*sigma*X[i,]
                    mod <- Model(Z[i,], Data)
                    if(all(is.finite(c(mod[["LP"]], mod[["Dev"]],
                         mod[["Monitor"]]))))
                         LP[i] <- mod[["LP"]]
                    Z[i,] <- mod[["parm"]]}
               ### Weights
               LPw <- matrix(exp(LP - logadd(LP)), N, parm.len)
               LPw <- LPw / matrix(colSums(LPw), N, parm.len)
               LPw[which(!is.finite(LPw))] <- 0
               if(all(LPw == 0)) {
                    expand <- TRUE
                    LPw <- rep(.Machine$double.eps, N)}
               ### Adapt mu and Sigma
               mu[iter,] <- colSums(LPw * Z)
               temp <- cov.wt(Z, wt=LPw[,1])$cov
               if(all(is.finite(temp))) Covar <- as.symmetric.matrix(temp)
               else expand <- TRUE
               diag(Covar) <- abs(diag(Covar))
               diag(Covar)[which(diag(Covar) < .Machine$double.eps)] <- .Machine$double.eps
               M <- W * exp(X) * sqrt(2) *
                    matrix(sigma, N, parm.len, byrow=TRUE)
               sigma <- sqrt(diag(Covar))
               mod <- Model(mu[iter,], Data)
               Dev <- rbind(Dev, mod[["Dev"]])
               ### Accept Only Improved mu
               m.old <- Model(mu[iter-1,], Data)
               if(all(is.finite(c(mod[["LP"]], mod[["Dev"]],
                    mod[["Monitor"]])))) {
                    if(m.old[["LP"]] >= mod[["LP"]]) {
                         expand <- TRUE
                         mu[iter,] <- mu[iter-1,]}
                    }
               else {
                    expand <- TRUE
                    mu[iter,] <- mu[iter-1,]}
               ### Integration Error and Tolerance
               Mw <- M / matrix(colSums(M), N, parm.len, byrow=TRUE)
               IE <- c(IE, mean(abs(Mw - LPw)))
               tol[1] <- sqrt(sum({mu[iter,] - mu[iter-1,]}^2))
               tol[2] <- abs(IE[iter] - IE[iter-1])
               if(tol[1] <= Stop.Tolerance[1]) {
                    expand <- TRUE
                    if(tol[2] <= Stop.Tolerance[2]) {
                         if(count > 0) break
                         count <- count + 1}
                    }
               else count <- 0
               }
          }
     else {
          detectedCores <- detectCores()
          cat("\n\nCPUs Detected:", detectedCores, "\n")
          if(CPUs > detectedCores) {
               cat("\nOnly", detectedCores, "will be used.\n")
               CPUs <- detectedCores}
          cat("\nLaplace's Demon is preparing environments for CPUs...")
          cat("\n##################################################\n")
          cl <- makeCluster(CPUs)
          cat("\n##################################################\n")
          on.exit(stopCluster(cl))
          varlist <- unique(c(ls(), ls(envir=.GlobalEnv),
               ls(envir=parent.env(environment()))))
          clusterExport(cl, varlist=varlist, envir=environment())
          clusterSetRNGStream(cl)
          wd <- getwd()
          clusterExport(cl, varlist=c("Packages", "Dyn.libs", "wd"),
               envir=environment())
          Dev <- matrix(m.old[["Dev"]],1,1)
          LPworst <- m.old[["LP"]]
          parm.len <- length(parm)
          if(parm.len > 10)
               warning("The number of parameters may be too large.")
          mu <- rbind(parm)
          parm.old <- parm
          colnames(mu) <- Data[["parm.names"]]
          sigma <- sqrt(diag(Covar))
          rule <- SparseGrid(J=parm.len, K=K)
          X <- rule$nodes
          W <- rule$weights
          N <- length(W)
          count <- 0
          expand <- FALSE
          IE <- 1
          tol <- c(1,1)
          ### Begin Iterative Quadrature
          options(warn=-1)
          cat("\nIteration: 1,   K: ", K, ",   Nodes: ", N, sep="")
          for (iter in 2:Iterations) {
               ### New Quadrature Rule
               if({expand == TRUE} & {K < Kmax}) {
                    K <- min(K+1, Kmax)
                    rule <- SparseGrid(J=parm.len, K=K)
                    X <- rule$nodes
                    W <- rule$weights
                    N <- length(W)}
               expand <- FALSE
               cat("\nIteration: ", iter, ",   K: ", K, ",   Nodes: ", N,
                    sep="")
               LP <- rep(LPworst, N)
               LPw <- M <- Z <- matrix(0, N, parm.len)
               mu <- rbind(mu, 0)
               ### Evaluate at the Nodes
               Z <- matrix(mu[iter-1,], N, parm.len, byrow=TRUE) +
                    sqrt(2) * matrix(sigma, N, parm.len, byrow=TRUE) * X
               mod <- parLapply(cl, 1:N, function(x) Model(Z[x,], Data))
               LP <- as.vector(unlist(lapply(mod, function(x) x[["LP"]])))
               Z <- matrix(as.vector(unlist(lapply(mod,
                    function(x) x[["parm"]]))), N, parm.len, byrow=TRUE)
               LP[which(!is.finite(LP))] <- LPworst
               ### Weights
               LPw <- matrix(exp(LP - logadd(LP)), N, parm.len)
               LPw <- LPw / matrix(colSums(LPw), N, parm.len)
               LPw[which(!is.finite(LPw))] <- 0
               if(all(LPw == 0)) {
                    expand <- TRUE
                    LPw <- rep(.Machine$double.eps, N)}
               ### Adapt mu and Sigma
               mu[iter,] <- colSums(LPw * Z)
               temp <- cov.wt(Z, wt=LPw[,1])$cov
               if(all(is.finite(temp))) Covar <- as.symmetric.matrix(temp)
               else expand <- TRUE
               diag(Covar) <- abs(diag(Covar))
               diag(Covar)[which(diag(Covar) < .Machine$double.eps)] <- .Machine$double.eps
               M <- W * exp(X) * sqrt(2) *
                    matrix(sigma, N, parm.len, byrow=TRUE)
               sigma <- sqrt(diag(Covar))
               mod <- Model(mu[iter,], Data)
               Dev <- rbind(Dev, mod[["Dev"]])
               ### Accept Only Improved mu
               m.old <- Model(mu[iter-1,], Data)
               if(all(is.finite(c(mod[["LP"]], mod[["Dev"]],
                    mod[["Monitor"]])))) {
                    if(m.old[["LP"]] >= mod[["LP"]]) {
                         expand <- TRUE
                         mu[iter,] <- mu[iter-1,]}
                    }
               else {
                    expand <- TRUE
                    mu[iter,] <- mu[iter-1,]}
               ### Integration Error and Tolerance
               Mw <- M / matrix(colSums(M), N, parm.len, byrow=TRUE)
               IE <- c(IE, mean(abs(Mw - LPw)))
               tol[1] <- sqrt(sum({mu[iter,] - mu[iter-1,]}^2))
               tol[2] <- abs(IE[iter] - IE[iter-1])
               if(tol[1] <= Stop.Tolerance[1]) {
                    expand <- TRUE
                    if(tol[2] <= Stop.Tolerance[2]) {
                         if(count > 0) break
                         count <- count + 1}
                    }
               else count <- 0
               }
          }
     options(warn=0)
     ### Output
     IQ <- list(Covar=Covar, Dev=Dev, iter=iter, LPw=LPw, M=M,
          mu=mu, N=N, parm.old=parm.old, tol=tol, Z=Z)
     return(IQ)
     }
.iqcagh <- function(Model, parm, Data, Covar, Iterations, Stop.Tolerance,
     CPUs, m.old, N, Nmax, Packages, Dyn.libs)
     {
     if(CPUs == 1) {
          Dev <- matrix(m.old[["Dev"]],1,1)
          LPworst <- m.old[["LP"]]
          parm.len <- length(parm)
          mu <- rbind(parm)
          parm.old <- parm
          colnames(mu) <- Data[["parm.names"]]
          sigma <- sqrt(diag(Covar))
          LP <- rep(LPworst, N)
          LPw <- M <- Z <- matrix(0, N, parm.len)
          rule <- GaussHermiteQuadRule(N)
          count <- 0
          expand <- matrix(0, 4, parm.len)
          IE <- 1
          tol <- c(1,1)
          ### Begin Iterative Quadrature
          options(warn=-1)
          cat("\nIteration: 1,   Nodes: ", N, ",   LP:",
               round(m.old[["LP"]],3), sep="")
          for (iter in 2:Iterations) {
               cat("\nIteration: ", iter, ",   Nodes: ", N,
                    ",   LP:", round(m.old[["LP"]],3), sep="")
               mu <- rbind(mu, 0)
               ### New Quadarature Rule
               if({any(rowSums(expand) == parm.len)} & {N < Nmax}) {
                    N <- min(N + 1, Nmax)
                    LPw <- M <- Z <- matrix(0, N, parm.len)
                    LP <- rep(LPworst, N)
                    rule <- GaussHermiteQuadRule(N)}
               expand <- matrix(0, 4, parm.len)
               X <- matrix(rule$nodes, N, parm.len)
               W <- matrix(rule$weights, N, parm.len)
               for (j in sample(parm.len)) {
                    ### Evaluate at the Nodes
                    Z[,j] <- m.old[["parm"]][j] + sqrt(2)*sigma[j]*X[,j]
                    for (i in 1:N) {
                         prop <- m.old[["parm"]]
                         prop[j] <- Z[i,j]
                         m.new <- Model(prop, Data)
                         if(all(is.finite(c(m.new[["LP"]], m.new[["Dev"]],
                              m.new[["Monitor"]]))))
                              LP[i] <- m.new[["LP"]]
                         Z[i,j] <- m.new[["parm"]][j]}
                    ### Correct for Constraints
                    X[,j] <- (Z[,j] - m.old[["parm"]][j]) / (sqrt(2)*sigma[j])
                    W[,j] <- (2^(N-1) * factorial(N) * sqrt(pi)) /
                         (N^2 * Hermite(X[,j], N-1, prob=FALSE)^2)
                    W[,j][which(!is.finite(W[,j]))] <- 0
                    if(all(W[,j] == 0)) W[,j] <- 1/N
                    ### Weights
                    LPw[,j] <- exp(LP - logadd(LP))
                    LPw[,j] <- LPw[,j] / sum(LPw[,j])
                    if(any(!is.finite(LPw[,j]))) {
                         LPw[,j] <- rep(0, N)
                         expand[1,j] <- 1}
                    ### Adapt mu and sigma
                    if(all(LPw[,j] == 0)) {
                         LPw[,j] <- .Machine$double.eps
                         mu[iter,j] <- m.old[["parm"]][j] + rnorm(1,0,1e-5)
                         expand[2,j] <- 1
                         }
                    else {
                         LPmax <- which.max(LP)[1]
                         if({LPmax == 1} | {LPmax == N})
                              mu[iter,j] <- Z[LPmax,j]
                         else mu[iter,j] <- sum(LPw[,j] * Z[,j])
                         M[,j] <- W[,j] * exp(X[,j]) * sqrt(2) * sigma[j]
                         sigma[j] <- min(max(sqrt(sum(LPw[,j] *
                              {sqrt(2)*sigma[j]*X[,j]}^2)),
                              .Machine$double.eps), 1e5)}
                    ### Accept Increases Only
                    m.new <- Model(mu[iter,], Data)
                    mu[iter,] <- m.new[["parm"]]
                    if(all(is.finite(c(m.new[["LP"]], m.new[["Dev"]],
                         m.new[["Monitor"]])))) {
                         if(m.old[["LP"]] >= m.new[["LP"]]) {
                              expand[3,j] <- 1
                              mu[iter,] <- m.old[["parm"]]
                              }
                         else m.old <- m.new
                         }
                    else {
                         expand[3,j] <- 1
                         mu[iter,] <- m.old[["parm"]]
                         }
                    }
               Dev <- rbind(Dev, m.old[["Dev"]])
               ### Integration Error, Parameter Change, and Tolerance
               Mw <- M / matrix(colSums(M), N, parm.len, byrow=TRUE)
               IE <- c(IE, mean(abs(Mw - LPw)))
               tol[1] <- sqrt(sum({mu[iter,] - mu[iter-1,]}^2))
               tol[2] <- abs(IE[iter] - IE[iter-1])
               if(tol[1] <= Stop.Tolerance[1]) {
                    expand[4,] <- 1
                    if(tol[2] <= Stop.Tolerance[2]) {
                         if(count > 0) break
                         count <- count + 1}
                    }
               else count <- 0
               }
          }
     else {
          detectedCores <- detectCores()
          cat("\n\nCPUs Detected:", detectedCores, "\n")
          if(CPUs > detectedCores) {
               cat("\nOnly", detectedCores, "will be used.\n")
               CPUs <- detectedCores}
          cat("\nLaplace's Demon is preparing environments for CPUs...")
          cat("\n##################################################\n")
          cl <- makeCluster(CPUs)
          cat("\n##################################################\n")
          on.exit(stopCluster(cl))
          varlist <- unique(c(ls(), ls(envir=.GlobalEnv),
               ls(envir=parent.env(environment()))))
          clusterExport(cl, varlist=varlist, envir=environment())
          clusterSetRNGStream(cl)
          wd <- getwd()
          clusterExport(cl, varlist=c("Packages", "Dyn.libs", "wd"),
               envir=environment())
          Dev <- matrix(m.old[["Dev"]],1,1)
          LPworst <- m.old[["LP"]]
          parm.len <- length(parm)
          mu <- rbind(parm)
          parm.old <- parm
          colnames(mu) <- Data[["parm.names"]]
          sigma <- sqrt(diag(Covar))
          LPbest <- rep(-Inf, parm.len)
          LP <- rep(LPworst, N)
          LPw <- M <- Z <- matrix(0, N, parm.len)
          rule <- GaussHermiteQuadRule(N)
          count <- 0
          expand <- matrix(0, 4, parm.len)
          IE <- 1
          tol <- c(1,1)
          ### Begin Iterative Quadrature
          options(warn=-1)
          cat("\nIteration: 1,   Nodes: ", N,
                    ",   LP:", round(m.old[["LP"]],3), sep="")
          for (iter in 2:Iterations) {
               cat("\nIteration: ", iter, ",   Nodes: ", N,
                    ",   LP:", round(m.old[["LP"]],3), sep="")
               mu <- rbind(mu, 0)
               ### New Quadarature Rule
               if({any(rowSums(expand) == parm.len)} & {N < Nmax}) {
                    N <- min(N + 1, Nmax)
                    LPw <- M <- Z <- matrix(0, N, parm.len)
                    LP <- rep(LPworst, N)
                    rule <- GaussHermiteQuadRule(N)}
               expand <- matrix(0, 4, parm.len)
               X <- matrix(rule$nodes, N, parm.len)
               W <- matrix(rule$weights, N, parm.len)
               for (j in sample(parm.len)) {
                    ### Evaluate at the Nodes
                    Z[,j] <- m.old[["parm"]][j] + sqrt(2)*sigma[j]*X[,j]
                    prop <- matrix(m.old[["parm"]], N, parm.len, byrow=TRUE)
                    prop[,j] <- Z[,j]
                    m.new <- parLapply(cl, 1:N, function(x) Model(prop[x,], Data))
                    LP <- as.vector(unlist(lapply(m.new, function(x) x[["LP"]])))
                    Z[,j] <- as.vector(unlist(lapply(m.new,
                         function(x) x[["parm"]][j])))
                    LP[which(!is.finite(LP))] <- LPworst
                    ### Correct for Constraints
                    X[,j] <- (Z[,j] - m.old[["parm"]][j]) / (sqrt(2)*sigma[j])
                    W[,j] <- (2^(N-1) * factorial(N) * sqrt(pi)) /
                         (N^2 * Hermite(X[,j], N-1, prob=FALSE)^2)
                    W[,j][which(!is.finite(W[,j]))] <- 0
                    if(all(W[,j] == 0)) W[,j] <- 1/N
                    ### Weights
                    LPw[,j] <- exp(LP - logadd(LP))
                    LPw[,j] <- LPw[,j] / sum(LPw[,j])
                    if(any(!is.finite(LPw[,j]))) {
                         LPw[,j] <- rep(0, N)
                         expand[1,j] <- 1}
                    ### Adapt mu and sigma
                    if(all(LPw[,j] == 0)) {
                         LPw[,j] <- .Machine$double.eps
                         mu[iter,j] <- m.old[["parm"]][j] + rnorm(1,0,1e-5)
                         expand[2,j] <- 1
                         }
                    else {
                         LPmax <- which.max(LP)[1]
                         if({LPmax == 1} | {LPmax == N})
                              mu[iter,j] <- Z[LPmax,j]
                         else mu[iter,j] <- sum(LPw[,j] * Z[,j])
                         M[,j] <- W[,j] * exp(X[,j]) * sqrt(2) * sigma[j]
                         sigma[j] <- min(max(sqrt(sum(LPw[,j] *
                              {sqrt(2)*sigma[j]*X[,j]}^2)),
                              .Machine$double.eps), 1e5)}
                    ### Accept Increases Only
                    m.new <- Model(mu[iter,], Data)
                    mu[iter,] <- m.new[["parm"]]
                    if(all(is.finite(c(m.new[["LP"]], m.new[["Dev"]],
                         m.new[["Monitor"]])))) {
                         if(m.old[["LP"]] >= m.new[["LP"]]) {
                              expand[3,j] <- 1
                              mu[iter,] <- m.old[["parm"]]
                              }
                         else m.old <- m.new
                         }
                    else {
                         expand[3,j] <- 1
                         mu[iter,] <- m.old[["parm"]]
                         }
                    }
               Dev <- rbind(Dev, m.old[["Dev"]])
               ### Integration Error, Parameter Change, and Tolerance
               Mw <- M / matrix(colSums(M), N, parm.len, byrow=TRUE)
               IE <- c(IE, mean(abs(Mw - LPw)))
               tol[1] <- sqrt(sum({mu[iter,] - mu[iter-1,]}^2))
               tol[2] <- abs(IE[iter] - IE[iter-1])
               if(tol[1] <= Stop.Tolerance[1]) {
                    expand[4,] <- 1
                    if(tol[2] <= Stop.Tolerance[2]) {
                         if(count > 0) break
                         count <- count + 1}
                    }
               else count <- 0
               }
          }
     options(warn=0)
     ### Output
     IQ <- list(Covar=diag(sigma^2), Dev=Dev, iter=iter, LPw=LPw, M=M,
          mu=mu, N=N, parm.old=parm.old, tol=tol, Z=Z)
     return(IQ)
     }

#End

###########################################################################
# MinnesotaPrior                                                          #
#                                                                         #
# The purpose of the MinnesotaPrior function is to return prior           #
# covariance matrices for autoregressive parameters in vector             #
# autoregression (VAR) models.                                            #
###########################################################################

MinnesotaPrior <- function(J, lags=c(1,2), lambda=1, theta=0.5, sigma)
     {
     theta <- max(min(theta, 1), 0)
     Iden <- diag(J)
     L <- length(lags)
     V <- array(0, dim=c(J,J,length(lags)))
     for (l in 1:L) {
          ### Diagonal elements
          V[,,l] <- V[,,l] + Iden * (lambda/lags[l])^2
          ### Off-diagonal elements
          V[,,l] <- V[,,l] + (1 - Iden) *
               ((lambda*theta*matrix(sigma, J, J, byrow=TRUE)) /
               (lags[l]*matrix(sigma, J, J)))^2}
     return(V)
     }

#End

###########################################################################
# CenterScale                                                             #
#                                                                         #
# The purpose of the CenterScale function is to center and scale a        #
# continuous variable. Options are also provided for binary variables.    #
# This function is very similar to Gelman's rescale function in his arm   #
# package.                                                                #
###########################################################################

CenterScale <- function(x, Binary="none", Inverse=FALSE, mu, sigma, Range,
     Min)
     {
     if(identical(Inverse, FALSE)) {
          ### Initial Checks
          if(!is.numeric(x)){
               x <- as.numeric(factor(x))
               x.obs <- x[is.finite(x)]}
          x.obs <- x[is.finite(x)]
          ### Binary Variables
          if(identical(length(unique(x.obs)), 2)){
               if(identical(Binary, "none")){
                    return((x-min(x.obs)) / (max(x.obs)-min(x.obs)))}
               else if(identical(Binary, "center")) {
                    return(x-mean(x.obs))}
               else if(identical(Binary, "center0")) {
                    x <- (x-min(x.obs)) / (max(x.obs)-min(x.obs))
                    return(x-0.5)}
               else if(identical(Binary, "centerscale")) {
                    return({x-mean(x.obs)} / {2*sd(x.obs)})}
               }
          ### Continuous Variables
          else {return({x-mean(x.obs)} / {2*sd(x.obs)})}}
     else {
          ### Initial Checks
          if(!is.numeric(x)){
               x <- as.numeric(factor(x))
               x.obs <- x[is.finite(x)]}
          x.obs <- x[is.finite(x)]
          ### Binary Variables
          if(identical(length(unique(x.obs)), 2)){
               if(identical(Binary, "none")) {
                    return(x * Range + Min)}
               else if(identical(Binary, "center")) {
                    return(x + mu)}
               else if(identical(Binary, "center0")) {
                    return(x * Range + Min)}
               else if(identical(Binary, "centerscale")) {
                    return(x * (2*sigma) + mu)}
               }
          ### Continuous Variables
          else {return(x * (2*sigma) + mu)}
          }
     }

#End

###########################################################################
# Kullback-Leibler Divergence (KLD)                                       #
#                                                                         #
# The purpose of the KLD function is to calculate the Kullback-Leibler    #
# divergences between two probability distributions, p(x) and p(y).       #
###########################################################################

KLD <- function(px, py, base=exp(1))
     {
     ### Initial Checks
     if(!is.vector(px)) px <- as.vector(px)
     if(!is.vector(py)) py <- as.vector(py)
     n1 <- length(px)
     n2 <- length(py)
     if(!identical(n1, n2)) stop("px and py must have the same length.")
     if(any(!is.finite(px)) || any(!is.finite(py))) 
          stop("px and py must have finite values.")
     if(any(px <= 0)) px <- exp(px)
     if(any(py <= 0)) py <- exp(py)
     px[which(px < .Machine$double.xmin)] <- .Machine$double.xmin
     py[which(py < .Machine$double.xmin)] <- .Machine$double.xmin
     ### Normalize
     px <- px / sum(px)
     py <- py / sum(py)
     ### Kullback-Leibler Calculations
     KLD.px.py <- px * (log(px, base=base)-log(py, base=base))
     KLD.py.px <- py * (log(py, base=base)-log(px, base=base))
     sum.KLD.px.py <- sum(KLD.px.py)
     sum.KLD.py.px <- sum(KLD.py.px)
     mean.KLD <- (KLD.px.py + KLD.py.px) / 2
     mean.sum.KLD <- (sum.KLD.px.py + sum.KLD.py.px) / 2
     ### Output
     out <- list(KLD.px.py=KLD.px.py, #KLD[i](p(x[i]) || p(y[i]))
          KLD.py.px=KLD.py.px, #KLD[i](p(y[i]) || p(x[i]))
          mean.KLD=mean.KLD,
          sum.KLD.px.py=sum.KLD.px.py, #KLD(p(x) || p(y))
          sum.KLD.py.px=sum.KLD.py.px, #KLD(p(y) || p(x))
          mean.sum.KLD=mean.sum.KLD,
          intrinsic.discrepancy=min(sum.KLD.px.py, sum.KLD.py.px))
     return(out)
     }

#End

###########################################################################
# as.ppc                                                                  #
#                                                                         #
# The purpose of the as.ppc function is to convert an object of class     #
# demonoid.val to an object of class demonoid.ppc, after which the object #
# is ready for posterior predictive checks.                               #
###########################################################################

as.ppc <- function(x, set=3)
     {
     ### Initial Checks
     if(missing(x)) stop("x is required.")
     if(!identical(class(x), "demonoid.val"))
          stop("x is not of class demonoid.val.")
     set <- round(abs(set))
     if(set < 1) set <- 1
     else if(set > 3) set <- 3
     ### ppc
     if(set == 1) ppc <- list(y=x[[1]]$y, yhat=x[[1]]$yhat)
     else if(set == 2) ppc <- list(y=x[[2]]$y, yhat=x[[2]]$yhat)
     else ppc <- list(y=c(x[[1]]$y, x[[2]]$y), yhat=rbind(x[[1]]$yhat,
          x[[2]]$yhat))
     class(ppc) <- "demonoid.ppc"
     return(ppc)
     }

#End

###########################################################################
# BayesFactor                                                             #
#                                                                         #
# The purpose of the BayesFactor function is to estimate a Bayes factor   #
# from two objects, either of class demonoid, laplace, or pmc.            #
###########################################################################

BayesFactor <- function(x)
     {
     ### Initial Checks
     if(missing(x)) stop("x is required.")
     Model.num <- length(x)
     for (i in 1:Model.num) {
          if(!identical(class(x[[i]]), "demonoid") &
               !identical(class(x[[i]]), "laplace") &
               !identical(class(x[[i]]), "pmc") &
               !identical(class(x[[i]]), "vb"))
               stop("x is not of class demonoid, laplace, pmc, or vb.")
          if(identical(class(x[[i]]), "laplace") &
               identical(x[[i]]$Converged, FALSE)) { 
               stop("LaplaceApproximation() did not converge in ",
                    "M[",i,"].\n", sep="")}
          if(identical(class(x[[i]]), "vb") &
               identical(x[[i]]$Converged, FALSE)) { 
               stop("VariationalBayes() did not converge in ",
                    "M[",i,"].\n", sep="")}
          if(is.na(x[[i]]$LML))
               stop(cat("LML is missing in M[",i,"].", sep=""))
          }
     ### Bayes factor
     B <- matrix(NA, Model.num, Model.num)
     for (i in 1:Model.num) {for (j in 1:Model.num) {
          B[i,j] <- exp(x[[i]]$LML - x[[j]]$LML)}}
     strength <- rep(NA,6)
     strength[1] <- "-Inf  <  B <= 0.1   Strong against"
     strength[2] <- "0.1   <  B <= (1/3) Substantial against"
     strength[3] <- "(1/3) <  B < 1      Barely worth mentioning against"
     strength[4] <- "1     <= B < 3      Barely worth mentioning for"
     strength[5] <- "3     <= B < 10     Substantial for"
     strength[6] <- "10    <= B < Inf    Strong for"
     ### Posterior Probability
     ML <- rep(NA, Model.num)
     for (i in 1:Model.num) {ML[i] <- exp(x[[i]]$LML)}
     Posterior.Probability <- ML / sum(ML)
     ### Output
     BF.out <- list(B=B, Hypothesis="row > column", 
          Strength.of.Evidence=strength,
          Posterior.Probability=Posterior.Probability)
     class(BF.out) <- "bayesfactor"
     return(BF.out)
     }

#End

###########################################################################
# as.parm.names                                                           #
#                                                                         #
# The purpose of the as.parm.names function is to create the vector of    #
# names for the parameters from a list of parameters, which may be any    #
# combination of scalars, vectors, matrices, and upper-triangular         #
# matrices.                                                               #
###########################################################################

as.parm.names <- function(x, uppertri=NULL)
     {
     ### Initial Checks
     if(missing(x)) stop("x is required.")
     if(!is.list(x)) stop("x must be a list.")
     parm.length <- length(x)
     if(is.null(uppertri)) uppertri <- rep(0, parm.length)
     if(!identical(length(uppertri), parm.length))
          stop("Length of uppertri and list attributes differs.")
     ### Length of parm.names
     totlen <- 0
     for (i in 1:parm.length) {
          if(uppertri[i] == 0) {
               xlen <- length(as.vector(which(!is.na(x[[i]]))))
               totlen <- totlen + xlen
               }
          else if(uppertri[i] == 1) {
               if(is.vector(x[[i]])) stop("uppertri=1 found for a vector.")
               xlen <- length(which(!is.na(x[[i]][upper.tri(x[[i]],
                    diag=TRUE)])))
               totlen <- totlen + xlen}}
     ### Assign parm.names
     parm.names <- rep(NA, totlen)
     cnt <- 1
     for (i in 1:parm.length) {
          xname <- names(x)[i]
          xlen <- length(as.vector(x[[i]]))
          ### Scalar
          if(xlen == 1) {
               if(is.na(x[[i]])) stop("scalar has NA.")
               parm.names[cnt] <- paste(xname)
               cnt <- cnt + 1
               }
          ### Vector
          else if(is.vector(x[[i]]) & {xlen > 1}) {
               for (j in which(!is.na(x[[i]]))) {
                    parm.names[cnt] <- paste(xname, "[", j, "]", sep="")
                    cnt <- cnt + 1}
               }
          ### Matrix
          else if(is.matrix(x[[i]]) & (uppertri[i] == 0)) {
               for (k in 1:ncol(x[[i]])) {for (j in 1:nrow(x[[i]])) {
                    if(!is.na(x[[i]][j,k])) {
                         parm.names[cnt] <- paste(xname, "[", j, ",",
                              k, "]", sep = "")
                         cnt <- cnt + 1}}}
               }
          ### Matrix, Upper Triangular
          else if(is.matrix(x[[i]]) & (uppertri[i] == 1)) {
               nr <- nrow(x[[i]])
               nc <- ncol(x[[i]])
               U <- upper.tri(x[[i]], diag=TRUE)
               U[which(is.na(matrix(x[[i]], nr, nc)))] <- FALSE
               for (k in 1:nc) {for (j in 1:nr) {
                    if(U[j, k] == TRUE) {
                         parm.names[cnt] <- paste(xname, "[", j, ",",
                              k, "]", sep = "")
                         cnt <- cnt + 1}}}
               }
          ### Array
          else if(is.array(x[[i]]) & (uppertri[i] == 0)) {
               arrayx <- array(1:prod(dim(x[[i]])), dim(x[[i]]))
               for (j in 1:prod(dim(x[[i]]))) {
                    position <- which(arrayx == j, arr.ind=TRUE)
                    if(!is.na(x[[i]][position])) {
                         parm.names[cnt] <- paste(xname, "[",
                              paste(as.vector(position), sep="",
                              collapse=","), "]", sep="")
                         cnt <- cnt + 1}}
               }
          ### Array, Upper Triangular
          else if(is.array(x[[i]]) & (uppertri[i] == 1))
               stop("upper.tri does not function with arrays.")
          }
     return(parm.names)
     }

#End

###########################################################################
# summary.pmc.ppc                                                         #
#                                                                         #
# The purpose of the summary.pmc.ppc function is to summarize an object   #
# of class pmc.ppc (posterior predictive check).                          #
###########################################################################

summary.pmc.ppc <- function(object=NULL, Categorical=FALSE, Rows=NULL,
     Discrep=NULL, d=0, Quiet=FALSE, ...)
     {
     if(is.null(object)) stop("The object argument is NULL.")
     y <- object$y
     yhat <- object$yhat
     Deviance <- object$Deviance
     if(is.null(Rows)) Rows <- 1:length(y)
     if(any(Rows > length(y)) || any(Rows <= 0)) {
          warning("Invalid Rows argument; All rows included.")
          Rows <- 1:length(y)}
     ### Create Continuous Summary Table
     if(Categorical == FALSE) {
          Summ <- matrix(NA, length(y), 8, dimnames=list(1:length(y),
               c("y","Mean","SD","LB","Median","UB","PQ","Discrep")))
          Summ[,1] <- y
          Summ[,2] <- round(rowMeans(yhat),3)
          Summ[,3] <- round(sqrt(.rowVars(yhat)),3)
          for (i in 1:length(y))
               {
               Summ[i,4] <- round(quantile(yhat[i,], probs=0.025,
                    na.rm=TRUE),3)
               Summ[i,5] <- round(quantile(yhat[i,], probs=0.500,
                    na.rm=TRUE),3)
               Summ[i,6] <- round(quantile(yhat[i,], probs=0.975,
                    na.rm=TRUE),3)
               Summ[i,7] <- round(mean(yhat[i,] >= y[i], na.rm=TRUE),3)
               }
          ### Discrepancy Statistics
          Concordance <- 1 - mean({{Summ[,7] < 0.025} | {Summ[,7] > 0.975}},
               na.rm=TRUE)
          if(identical(yhat,y)) Concordance <- 1
          Discrepancy.Statistic <- 0
          if(!is.null(Discrep) && {Discrep == "Chi-Square"}) {
               Summ[,8] <- round((y - rowMeans(yhat))^2 /
                    .rowVars(yhat),3)
               Discrepancy.Statistic <- round(sum(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "Chi-Square2"}) {
               chisq.obs <- chisq.rep <- yhat
               E.y <- E.yrep <- rowMeans(yhat, na.rm=TRUE)
               for (i in 1:nrow(yhat)) {
                    chisq.obs[i,] <- (y[i] - E.y[i])^2 / E.y[i]
                    chisq.rep[i,] <- (yhat[i,] - E.yrep[i])^2 / E.yrep[i]
                    }
               Summ[,8] <- round(rowMeans(chisq.rep > chisq.obs,
                    na.rm=TRUE),3)
               Discrepancy.Statistic <- round(mean((Summ[,8] < 0.025) |
                    (Summ[,8] > 0.975), na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "DW"}) {
               Summ[,8] <- round((rowMeans(y - yhat, na.rm=TRUE) -
                    c(0, diff(rowMeans(y - yhat, na.rm=TRUE))))^2 /
                    rowMeans(y - yhat, na.rm=TRUE)^2, 3)
               Discrepancy.Statistic <- round(sum(Summ[,8], na.rm=TRUE), 3)}
          if(!is.null(Discrep) && {Discrep == "Kurtosis"}) {
               kurtosis <- function(x) {  
                    m4 <- mean((x-mean(x, na.rm=TRUE))^4, na.rm=TRUE) 
                    kurt <- m4/(sd(x, na.rm=TRUE)^4)-3  
                    return(kurt)}
               for (i in 1:length(y)) {Summ[i,8] <- round(kurtosis(yhat[i,]),3)}
               Discrepancy.Statistic <- round(mean(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "L.criterion"}) {
               Summ[,8] <- round(sqrt(.rowVars(yhat) +
                    (y - rowMeans(yhat))^2),3)
               Discrepancy.Statistic <- round(sum(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "MASE"}) {
               Summ[,8] <- round(abs(rowMeans(y - yhat, na.rm=TRUE) /
                    mean(abs(diff(y)), na.rm=TRUE)), 3)
               Discrepancy.Statistic <- round(mean(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "MSE"}) {
               Summ[,8] <- round(rowMeans((y - yhat)^2, na.rm=TRUE),3)
               Discrepancy.Statistic <- round(mean(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "PPL"}) {
               Summ[,8] <- round(.rowVars(yhat) + (d/(d+1)) *
                    (rowMeans(yhat) - y)^2,3)
               Discrepancy.Statistic <- round(sum(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "Quadratic Loss"}) {
               Summ[,8] <- round(rowMeans((y - yhat)^2, na.rm=TRUE),3)
               Discrepancy.Statistic <- round(mean(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "Quadratic Utility"}) {
               Summ[,8] <- round(rowMeans(-1*(y - yhat)^2, na.rm=TRUE),3)
               Discrepancy.Statistic <- round(mean(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "RMSE"}) {
               Summ[,8] <- round(sqrt(rowMeans((y - yhat)^2, na.rm=TRUE)),3)
               Discrepancy.Statistic <- round(mean(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "Skewness"}) {
               skewness <-  function(x) {
                    m3 <- mean((x-mean(x, na.rm=TRUE))^3, na.rm=TRUE)
                    skew <- m3/(sd(x, na.rm=TRUE)^3)
                    return(skew)}
               for (i in 1:length(y)) {Summ[i,8] <- round(skewness(yhat[i,]),3)}
               Discrepancy.Statistic <- round(mean(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "max(yhat[i,]) > max(y)"}) {
               for (i in 1:length(y)) {Summ[i,8] <- max(yhat[i,]) > max(y)}
               Discrepancy.Statistic <- round(mean(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "mean(yhat[i,]) > mean(y)"}) {
               for (i in 1:length(y)) {Summ[i,8] <- mean(yhat[i,]) > mean(y)}
               Discrepancy.Statistic <- round(mean(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "mean(yhat[i,] > d)"}) {
               for (i in 1:length(y)) {Summ[i,8] <- mean(yhat[i,] > d)}
               Discrepancy.Statistic <- round(mean(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "mean(yhat[i,] > mean(y))"}) {
               for (i in 1:length(y)) {Summ[i,8] <- mean(yhat[i,] > mean(y))}
               Discrepancy.Statistic <- round(mean(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "min(yhat[i,]) < min(y)"}) {
               for (i in 1:length(y)) {Summ[i,8] <- min(yhat[i,]) < min(y)}
               Discrepancy.Statistic <- round(mean(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "round(yhat[i,]) = d"}) {
               for (i in 1:length(y)) {
                    Summ[i,8] <- round(mean(round(yhat[i,]) == d,
                         na.rm=TRUE), 3)}
               Discrepancy.Statistic <- round(mean(Summ[,8], na.rm=TRUE),3)}
          if(!is.null(Discrep) && {Discrep == "sd(yhat[i,]) > sd(y)"}) {
               for (i in 1:length(y)) {Summ[i,8] <- sd(yhat[i,]) > sd(y)}
               Discrepancy.Statistic <- round(mean(Summ[,8], na.rm=TRUE),3)}
          Dbar <- round(mean(Deviance, na.rm=TRUE),3)
          pD <- round(var(Deviance, na.rm=TRUE)/2, 3)
          BPIC <- Dbar + 2*pD
          bpic <- matrix(c(Dbar, pD, BPIC), 1, 3)
          colnames(bpic) <- c("Dbar","pD","BPIC"); rownames(bpic) <- ""
          L <- round(sqrt(.rowVars(yhat) + (y - rowMeans(yhat))^2), 3)
          S.L <- round(sd(L, na.rm=TRUE),3); L <- round(sum(L, na.rm=TRUE),3)
          ### Create Output
          Summ.out <- list(BPIC=bpic,
               Concordance=Concordance,
               Discrepancy.Statistic=round(Discrepancy.Statistic,5),
               L.criterion=L,
               S.L=S.L,
               Summary=Summ[Rows,])
          if(Quiet == FALSE) {
               cat("Bayesian Predictive Information Criterion:\n")
               print(bpic)
               cat("Concordance: ", Concordance, "\n")
               cat("Discrepancy Statistic: ",
                    round(Discrepancy.Statistic,5), "\n")
               cat("L-criterion: ", L, ", S.L: ", S.L, sep="", "\n")
               cat("Records: \n")
               print(Summ[Rows,])}
          }
     ### Create Categorical Summary Table
     else {
          catcounts <- table(y)
          sumnames <- rep(NA, length(catcounts)+3)
          sumnames[1] <- "y"
          for (i in 1:length(catcounts)) {
               sumnames[i+1] <- paste("p(yhat=",names(catcounts)[i],")",sep="")}
          sumnames[length(sumnames)-1] <- "Lift"
          sumnames[length(sumnames)] <- "Discrep"
          Summ <- matrix(NA, length(y), length(sumnames),
               dimnames=list(1:length(y), sumnames))
          Summ[,1] <- y
          for (i in 1:length(catcounts)) {
               Summ[,i+1] <- rowSums(yhat == as.numeric(names(catcounts)[i])) /
                    ncol(yhat)}
          Summ[,{ncol(Summ)-1}] <- 1
          for (i in 1:length(y)) {
               Summ[i,{ncol(Summ)-1}] <- Summ[i,
                    grep(Summ[i,1],names(catcounts))+1] / 
                    {as.vector(catcounts[grep(Summ[i,1],names(catcounts))]) /
                    sum(catcounts)} - 1}
          ### Discrepancy Statistics
          Mean.Lift <- round(mean(Summ[,{ncol(Summ)-1}]),3)
          Discrepancy.Statistic <- 0
          if(!is.null(Discrep) && {Discrep == "p(yhat[i,] != y[i])"}) {
               for (i in 1:length(y)) {Summ[i,ncol(Summ)] <- 1 - 
                    Summ[i, grep(Summ[i,1],names(catcounts))+1]}
               Discrepancy.Statistic <- round(mean(Summ[,ncol(Summ)],
                    na.rm=TRUE),3)}
          Dbar <- round(mean(Deviance, na.rm=TRUE),3)
          pD <- round(var(Deviance, na.rm=TRUE)/2, 3)
          BPIC <- Dbar + 2*pD
          bpic <- matrix(c(Dbar, pD, BPIC), 1, 3)
          colnames(bpic) <- c("Dbar","pD","BPIC"); rownames(bpic) <- ""
          ### Create Output
          Summ.out <- list(BPIC=bpic,
               Mean.Lift=Mean.Lift,
               Discrepancy.Statistic=round(Discrepancy.Statistic,5),
               Summary=Summ[Rows,])
          if(Quiet == FALSE) {
               cat("Bayesian Predictive Information Criterion:\n")
               print(bpic)
               cat("Mean Lift: ", Mean.Lift, "\n")
               cat("Discrepancy Statistic: ",
                    round(Discrepancy.Statistic,5), "\n")
               cat("Records: \n")
               print(Summ[Rows,])}
          }
     return(invisible(Summ.out))
     }

#End

###########################################################################
# is.data                                                                 #
#                                                                         #
# The purpose of the is.data function is to estimate if a list of data is #
# data as far as IterativeQuadrature, LaplaceApproximation,               #
# LaplacesDemon, PMC, and VariationalBayes are concerned.                 #
###########################################################################

is.data <- function(Data)
     {
     if(missing(Data))
          stop("The Data argument is required.")
     isdata <- TRUE
     if(!is.list(Data)) {
          cat("\nData must be a list.\n")
          isdata <- FALSE}
     if(is.null(Data[["mon.names"]])) {
          cat("\nmon.names is NULL.\n")
          isdata <- FALSE}
     if(is.null(Data[["parm.names"]])) {
          cat("\nparm.names is NULL.\n")
          isdata <- FALSE}
     return(isdata)
     }

#End

###########################################################################
# cond.plot                                                               #
#                                                                         #
# The purpose of the cond.plot function is to provide several styles of   #
# conditional plots with base graphics.                                   #
###########################################################################

cond.plot <- function(x, y, z, Style="smoothscatter")
     {
     ### Initial Checks
     if(missing(x)) stop("x is a required argument.")
     if(missing(z)) z <- rep(1, length(x))
     if(Style %in% c("boxplot","scatter","smoothscatter")) {
          if(missing(y)) stop("y is a required argument.")
          if(length(x) != length(y)) stop("length differs for x and y.")}
     if(Style == "boxplot") x <- round(x)
     ### Object Names
     xname <- deparse(substitute(x))
     yname <- deparse(substitute(y))
     zname <- deparse(substitute(z))
     ### Set Plot Window
     znum <- length(unique(z))
     if(znum == 1) par(mfrow=c(1,1))
     else if(znum == 2) par(mfrow=c(2,1))
     else if(znum > 2 & znum < 5) par(mfrow=c(2,2))
     else if(znum > 4 & znum < 7) par(mfrow=c(2,3))
     else par(mfrow=c(3,3))
     if(Style == "boxplot") { ### Conditional Boxplot
          for (i in 1:znum) {
               boxplot(y[which(z == i)] ~ x[which(z == i)], col="red",
                    varwidth=TRUE, main=paste(zname, " = ", i, sep=""),
                    ylab=yname)} #xlab=xname doesn't work here?
          }
     if(Style == "densover") { ### Conditional Density Overlay
          par(mfrow=c(1,1))
          dens <- list()
          for (i in 1:znum) dens[[i]] <- density(x[which(z == i)])
          xmin <- min(sapply(dens, function(x) min(x$x)))
          xmax <- max(sapply(dens, function(x) max(x$x)))
          ymax <- max(sapply(dens, function(x) max(x$y)))
          plot(dens[[1]], xlim=c(xmin, xmax), ylim=c(0,ymax), col=1,
               main="Density Overlay",
               xlab=paste("f(", xname, " | ", zname, ")", sep=""))
          for (i in 2:znum) lines(dens[[i]], col=i)
          }
     else if(Style == "hist") { ### Conditional Histogram
          for (i in 1:znum) {
               hist(x[which(z == i)], col="red", prob=TRUE,
                    main=paste(zname, " = ", i, sep=""),
                    xlab=xname)
               lines(density(x[which(z == i)]))}
          }
     else if(Style == "scatter") { ### Conditional Scatterplot
          mycol <- rgb(100, 100, 100, 50, maxColorValue=255)
          for (i in 1:znum) {
               plot(x[which(z == i)], y[which(z == i)], xlim=range(x),
                    ylim=range(y), col=mycol, pch=16, xlab=xname,
                    ylab=yname, main=paste(zname, " = ", i, sep=""))
               panel.smooth(x[which(z == i)], y[which(z == i)],
                    col=mycol, pch=16)}
          }
     else if(Style == "smoothscatter") { #Conditional smoothScatter
          colramp=colorRampPalette(c("black","red"))
          lowess.na <- function(x, y=NULL, f=2/3, ...) {
               x1 <- subset(x, is.finite(x) & is.finite(y))
               y1 <- subset(y, is.finite(x) & is.finite(y))
               lowess.na <- lowess(x1, y1, f, ...)}
          for (i in 1:znum) {
          smoothScatter(x[which(z == i)], y[which(z == i)],
               colramp=colramp, nrpoints=0, xlab=xname, ylab=yname,
               main=paste(zname, " = ", i, sep=""))
               lines(lowess.na(x[which(z == i)], y[which(z == i)]),
                    col="white")}
          }
     return(invisible())
     }

#End

###########################################################################
# SensitivityAnalysis                                                     #
#                                                                         #
# The purpose of the SensitivityAnalysis function is to perform a         #
# sensitivity analysis on the posterior distributions and posterior       #
# inferences from two models.                                             #
###########################################################################

SensitivityAnalysis <- function(Fit1, Fit2, Pred1, Pred2)
     {
     ### Initial Checks
     if(missing(Fit1)) stop("Fit1 is a required argument.")
     if(missing(Fit2)) stop("Fit2 is a required argument.")
     if(missing(Pred1)) stop("Pred1 is a required argument.")
     if(missing(Pred2)) stop("Pred2 is a required argument.")
     "%!in%" <- function(x,table) match(x, table, nomatch=0) == 0
     if(class(Fit1) %!in% c("demonoid","iterquad","laplace","pmc","vb"))
          stop("Fit1 is not an object of class demonoid, iterquad, laplace, pmc, or vb.")
     if(class(Fit2) %!in% c("demonoid","iterquad","laplace","pmc","vb"))
          stop("Fit2 is not an object of class demonoid, iterquad, laplace, pmc, or vb.")
     if(identical(class(Fit1), "demonoid")) post1 <- Fit1$Posterior2
     if(identical(class(Fit2), "demonoid")) post2 <- Fit2$Posterior2
     if(identical(class(Fit1), "iterquad")) post1 <- Fit1$Posterior
     if(identical(class(Fit2), "iterquad")) post2 <- Fit2$Posterior
     if(identical(class(Fit1), "laplace")) post1 <- Fit1$Posterior
     if(identical(class(Fit2), "laplace")) post2 <- Fit2$Posterior
     if(identical(class(Fit1), "pmc")) post1 <- Fit1$Posterior2
     if(identical(class(Fit2), "pmc")) post2 <- Fit2$Posterior2
     if(identical(class(Fit1), "vb")) post1 <- Fit1$Posterior
     if(identical(class(Fit2), "vb")) post2 <- Fit2$Posterior
     if(class(Pred1) %!in% c("demonoid.ppc","iterquad.ppc","laplace.ppc","pmc.ppc","vb.ppc"))
          stop("Fit1 is not an object of class demonoid.ppc, iterquad.ppc, laplace.ppc, pmc.ppc, or vb.ppc.")
     if(class(Pred2) %!in% c("demonoid.ppc","iterquad.ppc","laplace.ppc","pmc.ppc","vb.ppc"))
          stop("Fit2 is not an object of class demonoid.ppc, iterquad.ppc, laplace.ppc, pmc.ppc, or vb.ppc.")
     if(nrow(post1) != nrow(post2))
          stop("The number of posterior samples differ between Fit1 and Fit2.")
     keep <- which(colnames(post1) %in% colnames(post2))
     post1 <- post1[,keep]
     keep <- which(colnames(post2) %in% colnames(post1))
     post2 <- post2[,keep]
     if(!identical(colnames(post1), colnames(post2)))
          stop("Posterior names differ between Fit1 and Fit2.")
     yhat1 <- Pred1$yhat
     yhat2 <- Pred2$yhat
     if(!identical(dim(yhat1), dim(yhat2)))
          stop("Dimensions of yhat differ between Pred1 and Pred2.")
     ### Sensitivity Analysis
     Posterior <- matrix(NA, ncol(post1), 2)
     rownames(Posterior) <- colnames(post1)
     colnames(Posterior) <- c("p(Fit1 > Fit2)", "var(Fit1) / var(Fit2)")
     Posterior[,1] <- colMeans(post1 > post2)
     Posterior[,2] <- .colVars(post1) / .colVars(post2)
     Post.Pred.Dist <- matrix(NA, nrow(yhat1), 2)
     rownames(Post.Pred.Dist) <- rownames(yhat1)
     colnames(Post.Pred.Dist) <- c("p(Pred1 > Pred2)",
          "var(Pred1) / var(Pred2)")
     Post.Pred.Dist[,1] <- rowMeans(yhat1 > yhat2)
     Post.Pred.Dist[,2] <- .rowVars(yhat1) / .rowVars(yhat2)
     ### Output
     out <- list(Posterior=Posterior, Post.Pred.Dist=Post.Pred.Dist)
     class(out) <- "sensitivity"
     return(out)
     }

#End

###########################################################################
# Precision                                                               #
#                                                                         #
# The purpose of these functions is to facilitate conversions between the #
# precision, standard deviation, and variance of scalars, vectors, and    #
# matrices.                                                               #
###########################################################################

Cov2Prec <- function(Cov)
     {
     if(any(!is.finite(Cov))) stop("Cov must be finite.")
     if(is.matrix(Cov)) {
          if(!is.positive.definite(Cov))
               stop("Cov is not positive-definite.")
          Prec <- as.inverse(Cov)}
     else if(is.vector(Cov)) {
          k <- as.integer(sqrt(length(Cov)))
          Cov <- matrix(Cov, k, k)
          if(!is.positive.definite(Cov))
               stop("Cov is not positive-definite.")
          Prec <- as.inverse(Cov)}
     return(Prec)
     }
Prec2Cov <- function(Prec)
     {
     if(any(!is.finite(Prec))) stop("Prec must be finite.")
     if(is.matrix(Prec)) {
          if(!is.positive.definite(Prec))
               stop("Prec is not positive-definite.")
          Cov <- as.inverse(Prec)}
     else if(is.vector(Prec)) {
          k <- as.integer(sqrt(length(Prec)))
          Prec <- matrix(Prec, k, k)
          if(!is.positive.definite(Prec))
               stop("Prec is not positive-definite.")
          Cov <- as.inverse(Prec)}
     return(Cov)
     }
prec2sd <- function(prec=1)
     {
     prec <- as.vector(prec)
     if(prec <=0) stop("prec must be positive.")
     return(sqrt(1/prec))
     }
prec2var <- function(prec=1)
     {
     prec <- as.vector(prec)
     if(prec <=0) stop("prec must be positive.")
     return(1/prec)
     }
sd2prec <- function(sd=1)
     {
     sd <- as.vector(sd)
     if(sd <=0) stop("sd must be positive.")
     return(1/sd^2)
     }
sd2var <- function(sd=1)
     {
     sd <- as.vector(sd)
     if(sd <=0) stop("sd must be positive.")
     return(sd^2)
     }
var2prec <- function(var=1)
     {
     var <- as.vector(var)
     if(var <=0) stop("var must be positive.")
     return(1/var)
     }
var2sd <- function(var=1)
     {
     var <- as.vector(var)
     if(var <=0) stop("var must be positive.")
     return(sqrt(var))
     }

#End

###########################################################################
# LossMatrix                                                              #
#                                                                         #
# The purpose of the LossMatrix function is facilitate Bayesian decision  #
# theory for discrete actions among discrete states.                      #
###########################################################################

LossMatrix <- function(L, p.theta)
     {
     ### Initial Checks
     if(missing(L)) stop("L is a required argument.")
     if(missing(p.theta)) stop("p.theta is a required argument.")
     if(!is.matrix(L) & !is.array(L)) stop("L must be a matrix or array.")
     if(!is.array(p.theta))
          stop("p.theta must be a vector or matrix.")
     d.L <- dim(L)
     d.p.theta <- dim(p.theta)
     if(any(d.L[1:2] != d.p.theta[1:2]))
          stop("The rows or columns of L and p.theta differ.")
     if(length(d.L) == 3 & length(d.p.theta) == 3)
          if(d.L[3] != d.p.theta[3])
               stop("The number of samples in L and p.theta differ.")
     if(length(d.L) > 3 | length(d.p.theta) > 3)
          stop("L and p.theta may have no more than 3 dimensions.")
     if(length(d.p.theta) == 2) {
          if(any(colSums(p.theta) != 1))
               stop("Each column in p.theta must sum to one.")
          }
     else {
          for (i in 1:d.p.theta[3]) {
               if(any(colSums(p.theta[,,i]) != 1))
                    stop("Each column in p.theta must sum to one.")}}
     ### Expected Loss
     if(length(d.L) == 2 & length(d.p.theta) == 2) {
          E.Loss <- colSums(L * p.theta)
          }
     else if(length(d.L) == 3 & length(d.p.theta) == 2) {
          E.Loss <- rep(0, d.L[3])
          for (i in 1:d.L[3]) {
               E.Loss <- E.Loss + colSums(L[,,i] * p.theta)}
          E.Loss <- E.Loss / d.L[3]
          }
     else if(length(d.L) == 2 & length(d.p.theta) == 3) {
          E.Loss <- rep(0, d.p.theta[3])
          for (i in 1:d.p.theta[3]) {
               E.Loss <- E.Loss + colSums(L * p.theta[,,i])}
          E.Loss <- E.Loss / d.p.theta[3]
          }
     else {
          E.Loss <- rep(0, d.L[3])
          for (i in 1:d.L[3]) {
               E.Loss <- E.Loss + colSums(L[,,i] * p.theta[,,i])}}
     action <- which.min(E.Loss)
     cat("\nAction", action, "minimizes expected loss.\n")
     out <- list(BayesAction=action, E.Loss=E.Loss)
     return(out)
     }

#End

###########################################################################
# predict.pmc                                                             #
#                                                                         #
# The purpose of the predict.pmc function is to predict y[new] or y[rep], #
# and later provide posterior predictive checks for objects of class pmc. #
###########################################################################

predict.pmc <- function(object, Model, Data, CPUs=1, Type="PSOCK", ...)
     {
     ### Initial Checks
     if(missing(object)) stop("The object argument is required.")
     if(missing(Model)) stop("The Model argument is required.")
     if(missing(Data)) stop("The Data argument is required.")
     if(is.null(Data[["y"]]) & is.null(Data[["Y"]]))
          stop("Data must have y or Y.")
     if(!is.null(Data[["y"]])) y <- as.vector(Data[["y"]])
     if(!is.null(Data[["Y"]])) y <- as.vector(Data[["Y"]])
     CPUs <- abs(round(CPUs))
     ### p(y[rep] | y)
     post <- as.matrix(object$Posterior2)
     Dev <- rep(NA, nrow(post))
     yhat <- matrix(NA, length(y), nrow(post))
     lengthcomp <- as.vector(Model(post[1,], Data)[["yhat"]])
     if(!identical(length(lengthcomp), length(y)))
          stop("y and yhat differ in length.")
     ### Non-Parallel Processing
     if(CPUs == 1) {
          for (i in 1:nrow(post)) {
               mod <- Model(post[i,], Data)
               Dev[i] <- as.vector(mod[["Dev"]])
               yhat[,i] <- as.vector(mod[["yhat"]])}
          }
      else { ### Parallel Processing
          detectedCores <- max(detectCores(),
               as.integer(Sys.getenv("NSLOTS")), na.rm=TRUE)
          cat("\n\nCPUs Detected:", detectedCores, "\n")
          if(CPUs > detectedCores) {
               cat("\nOnly", detectedCores, "will be used.\n")
               CPUs <- detectedCores}
          cl <- makeCluster(CPUs, Type)
          varlist <- unique(c(ls(), ls(envir=.GlobalEnv),
               ls(envir=parent.env(environment()))))
          clusterExport(cl, varlist=varlist, envir=environment())
          clusterSetRNGStream(cl)
          mod <- parLapply(cl, 1:nrow(post),
               function(x) Model(post[x,], Data))
          stopCluster(cl)
          Dev <- unlist(lapply(mod,
               function(x) x[["Dev"]]))[1:nrow(post)]
          yhat <- matrix(unlist(lapply(mod,
               function(x) x[["yhat"]])), length(y), nrow(post))
          rm(mod)}
     ### Warnings
     if(any(is.na(yhat))) cat("\nWARNING: Output matrix yhat has ",
          sum(is.na(yhat)), " missing values.\n")
     if(any(is.nan(yhat))) cat("\nWARNING: Output matrix yhat has ",
          sum(is.nan(yhat)), " non-numeric (NaN) values.\n")
     if(any(is.infinite(yhat))) cat("\nWARNING: Output matrix yhat has ",
          sum(is.infinite(yhat)), " infinite values.\n")
     if(any(!is.finite(Dev)))
          cat("\nWARNING: Deviance has non-finite values.")
     ### Create Output
     predicted <- list(y=y, yhat=yhat, Deviance=Dev)
     class(predicted) <- "pmc.ppc"
     return(predicted)
     }

#End

###########################################################################
# plot.vb                                                                 #
#                                                                         #
# The purpose of the plot.vb function is to plot an object of class vb.   #
###########################################################################

plot.vb <- function(x, Data=NULL, PDF=FALSE, Parms=NULL, ...)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     if(!inherits(x, "vb")) stop("x must be of class vb.")
     if(is.null(Data)) stop("The Data argument is NULL.")
     if(any(is.na(x$History))) stop("There is no history to plot.")
     ### Selecting Parms
     if(is.null(Parms)) {
          History1 <- x$History[,,1]
          History2 <- x$History[,,2]
          Posterior <- x$Posterior}
     else {
          Parms <- sub("\\[","\\\\[",Parms)
          Parms <- sub("\\]","\\\\]",Parms)
          Parms <- sub("\\.","\\\\.",Parms)
          if(length(grep(Parms[1], dimnames(x$History)[[2]])) == 0)
               stop("Parameter in Parms does not exist.")
          keepcols <- grep(Parms[1], dimnames(x$History[[2]]))
          if(length(Parms) > 1) {
               for (i in 2:length(Parms)) {
                    if(length(grep(Parms[i],
                         dimnames(x$History)[[2]])) == 0)
                         stop("Parameter in Parms does not exist.")
                    keepcols <- c(keepcols, grep(Parms[i],
                         dimnames(x$History)[[2]]))}}
          History1 <- as.matrix(x$History[,keepcols,1])
          History2 <- as.matrix(x$History[,keepcols,2])
          colnames(History1) <- dimnames(x$History)[[2]][keepcols]
          colnames(History2) <- dimnames(x$History)[[2]][keepcols]
          if(all(!is.na(x$Posterior))) {
               Posterior <- as.matrix(x$Posterior[,keepcols])
               colnames(Posterior) <- colnames(History1)}
          else Posterior <- x$Posterior
          }
     if(PDF == TRUE)
          {
          pdf("VariationalBayes.Plots.pdf")
          par(mfrow=c(3,3))
          }
     else {par(mfrow=c(3,3), ask=TRUE)}
     ### Plot Parameter
     for (j in 1:ncol(History1))
          {
          plot(1:nrow(History1), History1[,j],
               type="l", xlab="Iterations", ylab="Value (Mean)",
               main=colnames(History1)[j])
          plot(1:nrow(History2), History2[,j],
               type="l", xlab="Iterations", ylab="Value (Variance)",
               main=colnames(History2)[j])
          if({x$Converged == TRUE} & !any(is.na(Posterior))) {
               plot(density(Posterior[,j]),
                    xlab="Value", main=colnames(Posterior)[j])
               polygon(density(Posterior[,j]),
                    col="black", border="black")
               abline(v=0, col="red", lty=2)
               }
          else {
               draws <- rnorm(1000, x$Summary1[j,1], x$Summary1[j,2])
               plot(density(draws), xlab="Value",
                    main=colnames(History1)[j])
               polygon(density(draws), col="black", border="black")
               abline(v=0, col="red", lty=2)}
          }
     ### Plot Deviance History
     plot(1:length(x$Deviance), x$Deviance, type="l", xlab="Iterations",
          ylab="Value", main="Deviance")
     ### Plot Monitor
     if({x$Converged == TRUE} & !any(is.na(x$Monitor))) {
          for (j in 1:ncol(x$Monitor)) {
               plot(density(x$Monitor[,j]),
                    xlab="Value", main=Data[["mon.names"]][j])
               polygon(density(x$Monitor[,j]),
                    col="black", border="black")
               abline(v=0, col="red", lty=2)}
          }
     if(PDF == TRUE) dev.off()
     }

#End

###########################################################################
# GIV                                                                     #
#                                                                         #
# GIV stands for ``generate initial values'', and the purpose of the GIV  #
# function is to generate initial values for the IterativeQuadrature,     #
# LaplaceApproximation, LaplacesDemon, PMC, and VariationalBayes          #
# functions.                                                              #
###########################################################################

GIV <- function(Model, Data, n=1000, PGF=FALSE)
     {
     if(missing(Model)) stop("The Model argument is required.")
     if(!is.function(Model)) stop("Model must be a function.")
     if(missing(Data)) stop("The Data argument is required.")
     if(!is.list(Data)) stop("Data must be a list.")
     if(is.null(Data[["parm.names"]])) stop("parm.names missing in Data.")
     LIV <- length(Data[["parm.names"]])
     iv <- rep(NA, LIV)
     if(PGF == TRUE) {
          if(is.null(Data[["PGF"]])) stop("PGF missing in Data.")
          for (i in 1:n) {
               IV <- as.vector(Data$PGF(Data))
               M <- try(Model(IV, Data), silent=TRUE)
               if(!inherits(M, "try-error") & is.finite(M[["LP"]]) &
                    is.finite(M[["Dev"]]) & 
                    identical(as.vector(M[["parm"]]), as.vector(IV))) {
                    iv <- IV; break}
               }
          }
     else if(PGF == FALSE) {
          high <- 100; low <- -100
          a <- try(Model(rep(low, LIV), Data)[["parm"]], silent=TRUE)
          b <- try(Model(rep(high, LIV), Data)[["parm"]], silent=TRUE)
          if(inherits(a, "try-error") | inherits(b, "try-error")) {
               for (i in 1:n) {
                    IV <- rnorm(LIV, runif(1,-100,100), runif(1,0.1,1000))
                    M <- try(Model(IV, Data), silent=TRUE)
                    if(!inherits(M, "try-error") & is.finite(M[["LP"]]) &
                         is.finite(M[["Dev"]]) & 
                         identical(as.vector(M[["parm"]]), as.vector(IV))) {
                         iv <- IV; break}
                    }
               }
          else {
               ab.range <- b - a
               ab.mu <- a + ab.range / 2
               ab.mu <- ifelse({a == 0} & {b == high}, 10, ab.mu)
               Scale <- 1 / ab.range
               Scale[which(ab.range == 0)] <- 0
               for (i in 1:n) {
                    IV <- rnorm(LIV, ab.mu, ab.range * Scale)
                    M <- try(Model(IV, Data), silent=TRUE)
                    if(!inherits(M, "try-error") & is.finite(M[["LP"]]) &
                         is.finite(M[["Dev"]]) & 
                         identical(as.vector(M[["parm"]]), as.vector(IV))) {
                         iv <- IV; break}
                    Scale <- Scale + ab.range / n / 2}
               }
          }
     if((i == n) | any(is.na(iv)))
          cat("\nWARNING: Acceptable initial values were not generated.\n")
     return(iv)
     }
#End

###########################################################################
# Math                                                                    #
#                                                                         #
# This is a collection of functions to facilitate math.                   #
###########################################################################

GaussHermiteQuadRule <- function(N)
     {
     N <- abs(round(N))
     i <- seq(1, N-1, by=1)
     d <- sqrt(i/2)
     Diag <- function(x, k=0) {
          if(!is.numeric(x) && !is.complex(x))
               stop("Argument 'x' must be a real or complex vector or matrix.")
          if(!is.numeric(k) || k != round(k))
               stop("Argument 'k' must be an integer.")
          if(is.matrix(x)) {
               n <- nrow(x); m <- ncol(x)
               if(k >= m || -k >= n) {
                    y <- matrix(0, nrow=0, ncol=0)
               } else {
                    y <- x[col(x) == row(x) + k]
                    }
          } else {
               if(is.vector(x)) {
                    n <- length(x)
                    m <- n + abs(k)
                    y <- matrix(0, nrow=m, ncol=m)
                    y[col(y) == row(y) + k] <- x
               } else {
                    stop("Argument 'x' must be a real or complex vector or matrix.")
                    }
               }
          return(y)
          }
     E <- eigen(Diag(d, 1) + Diag(d, -1), symmetric=TRUE)
     L <- E$values
     V <- E$vectors
     inds <- order(L)
     x <- L[inds]
     V <- t(V[, inds])
     w <- sqrt(pi) * V[, 1]^2
     out <- list(nodes=x, weights=w)
     class(out) <- "gausshermitequadrule"
     return(out)
     }
Hermite <- function(x, N, prob=TRUE)
     {
     N <- abs(round(N))
     isBadLength <- (length(N) != 1) && (length(x) != length(N)) &&
          (length(x) != 1)
     if(isBadLength == TRUE)
     stop(paste("Argument 'n' must be either a vector of same length",
          "as argument 'x',\n  a single integer or 'x' must be a ",
          "single value!", sep=""))
     H <- function(x, N)
          {
          if(N <= 1) return(switch(N + 1, 1, x))
          else return(x * Recall(x, N - 1) - (N - 1) * Recall(x, N - 2))
          }
     scale <- 1
     if(prob == FALSE) {
          x <- sqrt(2) * x
          scale <- 2^(N / 2)}
     return(scale * mapply(H, x, N))
     }
logadd <- function(x, add=TRUE) 
     {
     x <- as.vector(x)
     x <- sort(x[is.finite(x)], decreasing=TRUE)
     x <- c(x[1], x[which(x != x[1])])
     if(length(x) == 1) return(x)
     n <- length(x)
     if(add == TRUE)
          z <- x[1] + log(1 + sum(exp(x[-1] - x[1])))
     else 
          z <- x[1] + sum(log(1 - exp(x[-1] - x[1])))
     return(z)
     }
partial <- function(Model, parm, Data, Interval=1e-6, Method="simple")
     {
     f <- Model(parm, Data)[["LP"]]
     n <- length(parm)
     if(Method == "simple") {
          if(n == 1)
               return({Model(parm + Interval, Data)[["LP"]] - f} / Interval)
          df <- rep(NA, n)
          for (i in 1:n) {
               dx <- parm
               dx[i] <- dx[i] + Interval
               df[i] <- {Model(dx, Data)[["LP"]] - f} / Interval}
          df[which(!is.finite(df))] <- 0
          return(df)
          }
     else if(Method == "Richardson") {
          zero.tol <- sqrt(.Machine$double.eps / 7e-7)
          d <- 0.0001
          r <- 4
          v <- 2
          a <- matrix(NA, r, n)
          h <- abs(d*parm) + Interval*{abs(parm) < zero.tol}
          for (k in 1:r) {
               if(n == 1)
                    a[k,] <- {Model(parm + h, Data)[["LP"]] -
                         Model(parm - h, Data)[["LP"]]} / (2*h)
               else for (i in 1:n) {
                    if((k != 1) && {abs(a[(k-1),i]) < 1e-20}) a[k,i] <- 0
	            else a[k,i] <- (Model(parm + h*(i == seq(n)), Data)[["LP"]] - 
	                 Model(parm - h*(i == seq(n)), Data)[["LP"]]) / (2*h[i])}
               a[k,which(!is.finite(a[k,]))] <- 0
               h <- h / v}
          for (m in 1:(r - 1))
               a <- (a[2:(r+1-m),,drop=FALSE]*(4^m)-a[1:(r-m),,drop=FALSE])/(4^m-1)
          return(c(a))
          }
     else stop("The", Method, "method is unknown.")
     }

#End

###########################################################################
# plot.pmc                                                                #
#                                                                         #
# The purpose of the plot.pmc function is to plot an object of class pmc. #
###########################################################################

plot.pmc <- function(x, BurnIn=0, Data=NULL, PDF=FALSE, Parms=NULL, ...)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     if(!inherits(x, "pmc"))
          stop("x must be of class pmc.")
     if(is.null(Data)) stop("The Data argument is NULL.")
     if(BurnIn >= nrow(x$Posterior2)) BurnIn <- 0
     Stat.at <- BurnIn + 1
     ### Selecting Parms
     if(is.null(Parms)) {
          Posterior <- x$Posterior2
          keepcols <- 1:ncol(Posterior)}
     else {
          Parms <- sub("\\[","\\\\[",Parms)
          Parms <- sub("\\]","\\\\]",Parms)
          Parms <- sub("\\.","\\\\.",Parms)
          if(length(grep(Parms[1], colnames(x$Posterior2))) == 0)
               stop("Parameter in Parms does not exist.")
          keepcols <- grep(Parms[1], colnames(x$Posterior2))
          if(length(Parms) > 1) {
               for (i in 2:length(Parms)) {
                    if(length(grep(Parms[i],
                         colnames(x$Posterior2))) == 0)
                         stop("Parameter in Parms does not exist.")
                    keepcols <- c(keepcols,
                         grep(Parms[i], colnames(x$Posterior2)))}}
          Posterior <- as.matrix(x$Posterior2[,keepcols])
          colnames(Posterior) <- colnames(x$Posterior2)[keepcols]}
     if(PDF == TRUE) {
          pdf("PMC.Plots.pdf")
          par(mfrow=c(2,2))
          }
     else {par(mfrow=c(2,2), ask=TRUE)}
     ### Plot Parameters
     for (j in 1:ncol(Posterior)) {
          ### Plot Parameter Trace Plots
          k <- keepcols[j]
          LL <- Me <- UL <- matrix(0, x$M, x$Iterations)
          for (m in 1:x$M) {for (i in Stat.at:x$Iterations) {
               LL[m,i] <- as.vector(quantile(x$Posterior1[,k,i,m],
                    probs=0.025))
               Me[m,i] <- as.vector(quantile(x$Posterior1[,k,i,m],
                    probs=0.500))
               UL[m,i] <- as.vector(quantile(x$Posterior1[,k,i,m],
                    probs=0.975))}}
          plot(Stat.at:x$Iterations, Me[1,Stat.at:x$Iterations],
               ylim=c(min(LL[,Stat.at:x$Iterations]),
                    max(UL[,Stat.at:x$Iterations])), pch=20,
               xlab="Iterations", ylab="Value",
               main=colnames(Posterior)[j])
          for (i in Stat.at:x$Iterations) lines(c(i,i), c(LL[1,i], UL[1,i]))
          if(x$M > 1) {
               for (m in 2:x$M) {
                    points(Stat.at:x$Iterations+(m-1)*0.1,
                         Me[m,Stat.at:x$Iterations], col=m, pch=20)
                    for (i in Stat.at:x$Iterations) {
                         lines(c(i+(m-1)*0.1,i+(m-1)*0.1),
                              c(LL[m,i], UL[m,i]), col=m)}}}
          ### Plot Parameter Densities
          plot(density(Posterior[Stat.at:x$Thinned.Samples,j]),
               xlab="Value", main=colnames(Posterior)[j])
          polygon(density(Posterior[Stat.at:x$Thinned.Samples,j]),
               col="black", border="black")
          abline(v=0, col="red", lty=2)
          }
     ### Plot Deviance Density
     plot(density(x$Deviance[Stat.at:length(x$Deviance)]),
          xlab="Value", main="Deviance")
     polygon(density(x$Deviance[Stat.at:length(x$Deviance)]), col="black",
               border="black")
     abline(v=0, col="red", lty=2)
     ### Plot Monitored Variable Densities
     if(is.vector(x$Monitor)) {J <- 1; nn <- length(x$Monitor)}
     else if(is.matrix(x$Monitor)) {
          J <- ncol(x$Monitor); nn <- nrow(x$Monitor)}
     for (j in 1:J) {
          plot(density(x$Monitor[Stat.at:nn,j]),
               xlab="Value", main=Data[["mon.names"]][j])
          polygon(density(x$Monitor[Stat.at:nn,j]), col="black",
               border="black")
          abline(v=0, col="red", lty=2)}
     ### Plot Convergence Diagnostics
     plot(x$Perplexity, ylim=c(0,1), type="l", xlab="Iterations", ylab="",
          sub="Perplexity=black; ESSN=red", main="Convergence")
     lines(x$ESSN, col="red")
     W <- Thin(x$W, By=x$Thinning)
     boxplot(W, outline=FALSE, col="red", xlab="Iterations",
          ylab="Importance Weights")
     if(x$M > 1) {
          plot(x$alpha[1,], col=1, ylim=c(0,1), type="l",
               main="Mixture Probabilities", xlab="Iterations",
               ylab="alpha")
          for (m in 2:x$M) {
               lines(x$alpha[m,], col=m)}}
     if(PDF == TRUE) dev.off()
     }

#End

###########################################################################
# server_Listening                                                        #
#                                                                         #
# The server_Listening function is not intended to be called directly by  #
# the user. It is an internal-only function that is intended to prevent   #
# cluster problems while using the INCA algorithm through the             #
# LaplacesDemon.hpc function.                                             #
###########################################################################

server_Listening <- function(n=2, port=19009)
     {
     slist <- vector('list', n)
     for (i in 1:n) {
          slist[[i]] <- socketConnection("localhost", port, server=TRUE,
               open="r+")
          cat("\nClient", i, "Connected")}
     tmp <- NULL
     trow <- 0
     stop_server <- FALSE
     cat("\nStart listening...")
     repeat
          {
          ready <- which(socketSelect(slist, TRUE))
          for (i in ready) {
               #print(paste("Socket", i, "ready to write"))
               con <- slist[[i]]
               #print("Write message...")
               if(is.null(tmp)) serialize(tmp, con)
               else serialize(tmp[-(((i-1)*trow+1):(i*trow))], con)
               #print("Read message...")
               buf <- try(unserialize(con), silent=TRUE)
               if(is.matrix(buf)) {
                    if(is.null(tmp)) {
                         tmp <- matrix(0, nrow=n*nrow(buf), ncol=ncol(buf))
                         trow <- nrow(buf)
                         }
                    tmp[((i-1)*trow+1):(i*trow),] <- buf
                    }
               else {
                    stop_server <- TRUE
                    break
                    }
               }
          if(stop_server == TRUE) break
          }
     for (i in 1:n) {
          close(slist[[i]])
          cat("\nClose connection", i)
          }
     cat("\n")
     }

#End

###########################################################################
# plot.miss                                                               #
#                                                                         #
# The purpose of the plot.miss function is to plot an object of class     #
# miss.                                                                   #
###########################################################################

plot.miss <- function(x, PDF=FALSE, ...)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     if(PDF == TRUE) {
          pdf("MISS.Plots.pdf")
          par(mfrow=c(3,3))
          }
     else par(mfrow=c(3,3), ask=TRUE)
     ### Plot Imputations
     for (i in 1:nrow(x$Imp)) {
          plot(1:ncol(x$Imp), x$Imp[i,], type="l", xlab="Iterations",
               ylab="Value", main=paste("Imp[", i, ",]", sep=""))
          panel.smooth(1:ncol(x$Imp), x$Imp[i,], pch="")
          plot(density(x$Imp[i,]), xlab="Value",
               main=paste("Imp[", i, ",]"))
          polygon(density(x$Imp[i,]), col="black", border="black")
          ### Only plot an ACF if there's > 1 unique values
          if(!is.constant(x$Imp[i,])) {
               z <- acf(x$Imp[i,], plot=FALSE)
               se <- 1/sqrt(length(x$Imp[i,]))
               plot(z$lag, z$acf, ylim=c(min(z$acf,-2*se),1), type="h",
                    main=paste("Imp[", i, ",]"), xlab="Lag",
                    ylab="Correlation")
               abline(h=(2*se), col="red", lty=2)
               abline(h=(-2*se), col="red", lty=2)
               }
          else plot(0, 0, main=paste("Imp[", i, ",]"),
               "is a constant.")}
     if(PDF == TRUE) dev.off()
     }

#End

###########################################################################
# summary.miss                                                            #
#                                                                         #
# The purpose of the summary.miss function is to summarize an object of   #
# class miss.                                                             #
###########################################################################

summary.miss <- function(object=NULL, ...)
     {
     if(is.null(object)) stop("The object argument is NULL.")
     x <- object$Imp
     Summ <- matrix(NA, nrow(x), 7, dimnames=list(1:nrow(x),
          c("Mean","SD","MCSE","ESS","LB","Median","UB")))
     Summ[,1] <- rowMeans(x)
     Summ[,2] <- sqrt(.rowVars(x))
     Summ[,3] <- 0
     Summ[,4] <- 0
     Summ[,5] <- apply(x, 1, quantile, c(0.025), na.rm=TRUE)
     Summ[,6] <- apply(x, 1, quantile, c(0.500), na.rm=TRUE)
     Summ[,7] <- apply(x, 1, quantile, c(0.925), na.rm=TRUE)
     acf.temp <- matrix(1, trunc(10*log10(ncol(x))), nrow(x))
     for (i in 1:nrow(x)) {
          ### MCSE
          temp <- try(MCSE(x[i,]), silent=TRUE)
          if(!inherits(temp, "try-error")) Summ[i,3] <- temp
          else Summ[i,3] <- MCSE(x[i,], method="sample.variance")
          ### ESS
          Summ[i,4] <- ESS(x[i,])}
     print(Summ)
     return(invisible(Summ))
     }

#End

###########################################################################
# KS.Diagnostic                                                           #
#                                                                         #
# The purpose of the KS.Diagnostic is to assess the stationarity of a     #
# MCMC chain, given its posterior samples.                                #
###########################################################################

KS.Diagnostic <- function(x)
     {
     if(missing(x)) stop("The x argument is required.")
     if(!is.vector(x)) x <- as.vector(x)
     n <- length(x)
     half <- round(n/2)
     out <- ks.test(x[1:half], x[-c(1:half)])$p.value
     return(out)
     }

#End

###########################################################################
# as.covar                                                                #
#                                                                         #
# The purpose of the as.covar function is to retrieve the covariance      #
# matrix from an object of class demonoid, demonoid.hpc, iterquad,        #
# laplace, pmc, or vb, or in the case of an object of class pmc with      #
# mixture components, to retrieve multiple covariance matrices.           #
###########################################################################

as.covar <- function(x)
     {
     if(!identical(class(x), "demonoid") &
        !identical(class(x), "demonoid.hpc") &
        !identical(class(x), "iterquad") &
        !identical(class(x), "laplace") &
        !identical(class(x), "pmc") &
        !identical(class(x), "vb"))
          stop("The class of x is unknown.")
     if(identical(class(x), "demonoid")) {
          if(is.matrix(x$Covar)) {
               covar <- x$Covar
               }
          else if(is.vector(x$Covar)) {
               covar <- diag(length(x$Covar))
               diag(covar) <- x$Covar
               }
          else {
               covar <- x$Covar
               if(is.list(x$Covar))
                    cat("\nThe covariance matrix is blocked.\n")
               }
          }
     else if(identical(class(x), "demonoid.hpc")) {
          Chains <- length(x)
          Deviance <- list()
          for (i in 1:Chains) {Deviance[[i]] <- x[[i]][["Deviance"]]}
          j <- which.min(sapply(Deviance, function(x)
               {min(x[length(x)])}))
          cat("\nChain",j,"has the lowest deviance.\n")
          if(is.matrix(x[[j]]$Covar)) {
               covar <- x[[j]]$Covar
               }
          else if(is.vector(x$Covar)) {
               covar <- diag(length(x$Covar))
               diag(covar) <- x$Covar
               }
          else {
               covar <- x$Covar
               if(is.list(x$Covar))
                    cat("\nThe covariance matrix is blocked.\n")
               }
          }
     else if(identical(class(x), "iterquad")) covar <- x$Covar
     else if(identical(class(x), "laplace")) covar <- x$Covar
     else if(identical(class(x), "vb")) covar <- x$Covar
     else covar <- x$Covar[,,x$Iterations,]
     return(covar)
     }

#End

###########################################################################
# BayesTheorem                                                            #
###########################################################################

BayesTheorem <- function(PrA, PrBA)
     {
     if(missing(PrA)) stop("The PrA argument is required.")
     if(missing(PrBA)) stop("The PrBA argument is required.")
     if(any(PrA < 0) | any(PrA > 1))
          stop("PrA is not in the interval [0,1].")
     if(any(PrBA < 0) | any(PrBA > 1))
          stop("PrBA is not in the interval [0,1].")
     PrAB <- (PrBA * PrA) / sum(PrBA * PrA)
     class(PrAB) <- "bayestheorem"
     return(PrAB)
     }

#End

###########################################################################
# Consort                                                                 #
#                                                                         #
# The purpose of the Consort function is to consort with Laplace's Demon  #
# regarding an object of class demonoid.                                  #
###########################################################################

Consort <- function(object=NULL)
     {
     if(is.null(object)) stop("The object argument is empty.")
     if(!identical(class(object), "demonoid"))
          stop("Consort requires an object of class demonoid.")
     oname <- deparse(substitute(object))
     dname <- as.vector(strsplit(as.character(object$Call), "=")[3][[1]])
     cat("\n#############################################################\n")
     cat("# Consort with Laplace's Demon                              #\n")
     cat("#############################################################\n")
     print.demonoid(object)
     ### Check Acceptance.Rate
     Acc.Rate.Level <- 2
     if((object$Algorithm == "Adaptive Hamiltonian Monte Carlo") |
        (object$Algorithm == "Tempered Hamiltonian Monte Carlo")) {
          L <- object$Specs[["L"]]
          if(L == 1) {
               Acc.Rate.Low <- 0.5
               Acc.Rate.High <- 0.65}
          else {
               Acc.Rate.Low <- 0.6
               Acc.Rate.High <- 0.7}}
     else if(object$Algorithm %in% c("Adaptive Griddy-Gibbs",
          "Automated Factor Slice Sampler",
          "Elliptical Slice Sampler", "Griddy-Gibbs",
          "Oblique Hyperrectangle Slice Sampler",
          "Reflective Slice Sampler", "Slice Sampler",
          "Stochastic Gradient Langevin Dynamics",
          "Univariate Eigenvector Slice Sampler")) {
          Acc.Rate.Low <- 1
          Acc.Rate.High <- 1
          }
     else if(object$Algorithm == "Gibbs Sampler") {
          if(is.null(object$Specs[["MWG"]]))
               Acc.Rate.Low <- Acc.Rate.High <- 1
          else {
               Acc.Rate.Low <- 0.15
               Acc.Rate.High <- 0.5}
          }
     else if((object$Algorithm == "Metropolis-Adjusted Langevin Algorithm")) {
          Acc.Rate.Low <- 0.40
          Acc.Rate.High <- 0.80
          }
     else if(object$Algorithm == "Hamiltonian Monte Carlo") {
          L <- object$Specs[["L"]]
          if(L == 1) {
               Acc.Rate.Low <- 0.4
               Acc.Rate.High <- 0.8}
               
          else {
               Acc.Rate.Low <- 0.6
               Acc.Rate.High <- 0.7}
          }
     else if(object$Algorithm == "Hamiltonian Monte Carlo with Dual-Averaging") {
          A <- object$Specs[["A"]]
          delta <- object$Specs[["delta"]]
          Lmax <- object$Specs[["Lmax"]]
          lambda <- object$Specs[["lambda"]]
          Acc.Rate.Low <- max(round(delta - 0.05, 2), 0.01)
          Acc.Rate.High <- min(round(delta + 0.05, 2), 1)
          }
     else if(object$Algorithm == "No-U-Turn Sampler") {
          A <- object$Specs[["A"]]
          delta <- object$Specs[["delta"]]
          Acc.Rate.Low <- max(round(delta - 0.05, 2), 0.01)
          Acc.Rate.High <- min(round(delta + 0.05, 2), 1)
          }
     else if(object$Algorithm == "Refractive Sampler") {
          m <- object$Specs[["m"]]
          Acc.Rate.Low <- 0.6
          Acc.Rate.High <- 0.7
          }
     else if((object$Algorithm %in% c("Reversible-Jump",
          "Multiple-Try Metropolis"))) {
          Acc.Rate.Low <- 0.10
          Acc.Rate.High <- 0.90
          }
     else {
          Acc.Rate.Low <- 0.15
          Acc.Rate.High <- 0.5}
     if(any(object$Acceptance.Rate == 0)) {
          cat("\nWARNING: Acceptance Rate = 0\n\n")
          cat(oname, " <- LaplacesDemon(Model, Data=", dname,
               ", Initial.Values,\n", sep="")
          cat("     Covar=NULL, Iterations=100000, Status=1000, ",
               "Thinning=100,\n", sep="")
          cat("     Algorithm=\"AMM\", Specs=list(Adaptive=500, ",
               "B=NULL, Periodicity=100, w=0.05))\n\n", sep="")
          stop("Try the above code before consorting again.")}
     if(any(object$Acceptance.Rate < Acc.Rate.Low)) {Acc.Rate.Level <- 1}
     else if(any(object$Acceptance.Rate > Acc.Rate.High)) {Acc.Rate.Level <- 3}
     LIV <- object$Parameters
     ### Check MCSE
     MCSE.crit <- 0.0627
     MCSE.temp <- object$Summary2[1:object$Parameters,"MCSE"] /
          object$Summary2[1:LIV,"SD"]
     MCSE.temp2 <- sum(!is.finite(MCSE.temp))
     MCSE.temp[which(!is.finite(MCSE.temp))] <- 0
     MCSE.tot <- 0
     if(MCSE.temp2 < LIV)
          MCSE.tot <- sum(MCSE.temp < MCSE.crit)
     ### Check ESS
     ESS.temp <- object$Summary2[1:LIV,"ESS"]
     ESS.temp[which(!is.finite(ESS.temp))] <- 0
     ESS.min <- min(ESS.temp)
     if(all(is.finite(object$Summary2[1:LIV,"ESS"])))
          ESS.worst <- which.min(object$Summary2[1:LIV,"ESS"])
     else ESS.worst <- which.min(object$Summary1[1:LIV,"ESS"])
     ESS.crit <- 100
     ### Check Stationarity
     Stationarity <- FALSE
     if(object$Rec.BurnIn.Thinned < object$Thinned.Samples)
          Stationarity <- TRUE
     ### Check Diminishing Adaptation (If Adaptive)
     if(nrow(object$CovarDHis) > 1)
          Dim.Adapt <- sum(diff(object$CovarDHis)) <= 0
     else Dim.Adapt <- TRUE
     ### Suggested Values
     Rec.Iterations <- trunc(object$Rec.Thinning / object$Thinning *
          object$Iterations)
     if(any(object$Acceptance.Rate == 0)) {
          Rec.Adaptive <- round(LIV * 1.0E7)
          Rec.Periodicity <- round(object$Rec.Thinning * 1.0E7)}
     else if(all(object$Acceptance.Rate > 0)) {
          Rec.Adaptive <- round(LIV / mean(object$Acceptance.Rate))
          Rec.Periodicity <- round(object$Rec.Thinning /
               mean(object$Acceptance.Rate))}
     if(Rec.Adaptive > Rec.Iterations) Rec.Adaptive <- Rec.Iterations - 1
     if(Rec.Periodicity > Rec.Iterations)
          Rec.Periodicity <- Rec.Iterations - 1
     Status.temp <- trunc(Rec.Iterations / {object$Minutes *
          Rec.Iterations / object$Iterations})
     if(Status.temp < Rec.Iterations) Rec.Status <- Status.temp
     else Rec.Status <- trunc(sqrt(Rec.Iterations))
     if(Rec.Status > Rec.Iterations) Rec.Status <- Rec.Iterations
     Rec.Thinning <- object$Rec.Thinning
     ### The Demonic Suggestion of Laplace's Demon
     cat("\nDemonic Suggestion\n\n")

     cat("Due to the combination of the following conditions,\n\n")

     cat("1. ", object$Algorithm, "\n", sep="")
     if(Acc.Rate.Level == 1) {
          cat("2. The acceptance rate (", min(object$Acceptance.Rate),
               ") is below ", Acc.Rate.Low, ".\n", sep="")}
     else if(Acc.Rate.Level == 2) {
          cat("2. The acceptance rate (", mean(object$Acceptance.Rate),
               ") is within the interval [", Acc.Rate.Low, ",",
               Acc.Rate.High, "].\n", sep="")}
     else {
          cat("2. The acceptance rate (", max(object$Acceptance.Rate),
               ") is above ", Acc.Rate.High, ".\n", sep="")}
     if(MCSE.tot < LIV) {
          cat("3. At least one target MCSE is >= ",
               MCSE.crit * 100,"% of its marginal posterior\n", sep="")
          cat("   standard deviation.\n")}
     else {
          cat("3. Each target MCSE is < ", MCSE.crit * 100,
               "% of its marginal posterior\n", sep="")
          cat("   standard deviation.\n")}
     if(ESS.min < ESS.crit) {
          cat("4. At least one target distribution has an ",
               "effective sample size\n", sep="")
          cat("   (ESS) less than ", ESS.crit,
               ". The worst mixing chain is: ", 
               rownames(object$Summary1)[ESS.worst], " (ESS=",
               object$Summary1[ESS.worst,"ESS"], ").\n", sep="")}
     else {
          cat("4. Each target distribution has an effective ",
               "sample size (ESS)\n", sep="")
          cat("   of at least ", ESS.crit, ".\n", sep="")}
     if(Stationarity == FALSE) {
          cat("5. At least one target distribution is not ",
               "stationary.\n\n", sep="")}
     else if(Stationarity == TRUE & object$Rec.BurnIn.Thinned > 0) {
          cat("5. Each target distribution became stationary by\n")
          cat("   ", object$Rec.BurnIn.Thinned + 1,
               " iterations.\n\n", sep="")}
     else {
          cat("5. Each target distribution became stationary by\n")
          cat("   ", object$Rec.BurnIn.Thinned + 1,
               " iteration.\n\n", sep="")}

     ### Determine if Laplace's Demon is appeased...
     Appeased <- FALSE
     if(!is.null(object$Specs[["Adaptive"]]))
          Adaptive <- object$Specs[["Adaptive"]]
     else Adaptive <- object$Iterations + 1
     if({Adaptive > object$Iterations} &
          {Acc.Rate.Level == 2} & {MCSE.tot >= LIV} &
          {ESS.min >= ESS.crit} & {Stationarity == TRUE}) {
          Appeased <- TRUE
          cat("Laplace's Demon has been appeased, and suggests\n")
          cat("the marginal posterior samples should be plotted\n")
          cat("and subjected to any other MCMC diagnostic deemed\n")
          cat("fit before using these samples for inference.\n\n")
          }
     else {
          if(object$Algorithm %in% c("Adaptive Directional Metropolis-within-Gibbs",
               "Adaptive Griddy-Gibbs",
               "Adaptive Metropolis-within-Gibbs",
               "Componentwise Hit-And-Run Metropolis",
               "Gibbs Sampler",
               "Griddy-Gibbs",
               "Metropolis-within-Gibbs",
               "Multiple-Try Metropolis",
               "Random Dive Metropolis-Hastings",
               "Reversible-Jump",
               "Sequential Adaptive Metropolis-within-Gibbs",
               "Sequential Metropolis-within-Gibbs",
               "Slice Sampler",
               "Updating Sequential Adaptive Metropolis-within-Gibbs",
               "Updating Sequential Metropolis-within-Gibbs")) {
               options(warn=-1)
               postcor <- try(cor(object$Posterior1), silent=TRUE)
               options(warn=0)
               if(!inherits(postcor, "try-error")) {
                    postcor <- try(quantile(abs(postcor)), silent=TRUE)
                    if(!inherits(postcor, "try-error")) {
                         cat("Quantiles of Absolute Posterior1 Correlation:\n")
                         print(postcor)
                         if(postcor["75%"] >= 0.5)
                              cat("\nPossibly excessive posterior correlation for a componentwise algorithm.")
                         cat("\n\n")}}}

          if(Dim.Adapt == FALSE) {
               cat("WARNING: Diminishing adaptation did not occur.\n")
               if(!object$Algorithm %in% c("Automated Factor Slice Sampler",
                    "Interchain Adaptation",
                    "Metropolis-Adjusted Langevin Algorithm",
                    "No-U-Turn Sampler", "Refractive Sampler",
                    "Sequential Adaptive Metropolis-within-Gibbs",
                    "Univariate Eigenvector Slice Sampler",
                    "Updating Sequential Adaptive Metropolis-within-Gibbs"))
                    cat("         A new algorithm will be suggested.\n\n")}
          
          cat("Laplace's Demon has not been appeased, and suggests\n")
          cat("copy/pasting the following R code into the R",
               " console,\n", sep="")
          cat("and running it.\n\n")

          if(object$Algorithm != "Interchain Adaptation")
               cat("Initial.Values <- as.initial.values(", oname, ")\n", sep="")
          if(object$Algorithm %in% c("Adaptive Metropolis-within-Gibbs",
               "Metropolis-within-Gibbs"))
               Time <- object$Iterations / object$Minutes
          else Time <- object$Iterations / object$Minutes / LIV
          if(Time >= 100) Fast <- TRUE
          else Fast <- FALSE

          if({Acc.Rate.Level == 2} & {MCSE.tot >= LIV} &
          {ESS.min >= ESS.crit} & {Stationarity == TRUE}) Ready <- TRUE
          else Ready <- FALSE

          Alg <- switch(object$Algorithm,
               "Adaptive Directional Metropolis-within-Gibbs"="ADMG",
               "Adaptive Griddy-Gibbs"="AGG",
               "Adaptive Hamiltonian Monte Carlo"="AHMC",
               "Adaptive Metropolis"="AM",
               "Adaptive Metropolis-within-Gibbs"="AMWG",
               "Adaptive-Mixture Metropolis"="AMM",
               "Affine-Invariant Ensemble Sampler"="AIES",
               "Automated Factor Slice Sampler"="AFSS",
               "Componentwise Hit-And-Run Metropolis"="CHARM",
               "Delayed Rejection Adaptive Metropolis"="DRAM",
               "Delayed Rejection Metropolis"="DRM",
               "Differential Evolution Markov Chain"="DEMC",
               "Elliptical Slice Sampler"="ESS",
               "Experimental"="Exper",
               "Gibbs Sampler"="Gibbs",
               "Griddy-Gibbs"="GG",
               "Hamiltonian Monte Carlo"="HMC",
               "Hamiltonian Monte Carlo with Dual-Averaging"="HMCDA",
               "Hit-And-Run Metropolis"="HARM",
               "Independence Metropolis"="IM",
               "Interchain Adaptation"="INCA",
               "Metropolis-Adjusted Langevin Algorithm"="MALA",
               "Metropolis-Coupled Markov Chain Monte Carlo"="MCMCMC",
               "Metropolis-within-Gibbs"="MWG",
               "Multiple-Try Metropolis"="MTM",
               "No-U-Turn Sampler"="NUTS",
               "Oblique Hyperrectangle Slice Sampler"="OHSS",
               "Preconditioned Crank-Nicolson"="pCN",
               "Random Dive Metropolis-Hastings"="RDMH",
               "Random-Walk Metropolis"="RWM",
               "Reflective Slice Sampler"="RSS",
               "Refractive Sampler"="Refractive",
               "Reversible-Jump"="RJ",
               "Robust Adaptive Metropolis"="RAM",
               "Sequential Adaptive Metropolis-within-Gibbs"="SAMWG",
               "Sequential Metropolis-within-Gibbs"="SMWG",
               "Slice Sampler"="Slice",
               "Stochastic Gradient Langevin Dynamics"="SGLD",
               "Tempered Hamiltonian Monte Carlo"="THMC",
               "t-walk"="t-walk",
               "Univariate Eigenvector Slice Sampler"="UESS",
               "Updating Sequential Adaptive Metropolis-within-Gibbs"="USAMWG",
               "Updating Sequential Metropolis-within-Gibbs"="USMWG")

          Componentwise <- 0
          if(Alg %in% c("ADMG","AFSS","AGG","AMWG","CHARM","GG","Gibbs",
               "MWG","RJ","SAMWG","SMWG","Slice","USAMWG","USMWG"))
               Componentwise <- 1
          if({(Alg == "ADMG") & !Dim.Adapt} |
               {(Alg == "ADMG") & !Ready}) {
               ### ADMG
               n <- object$Specs[["n"]] + object$Iterations
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"ADMG\", ",
                    "Specs=list(n=", n, ", Periodicity=", Rec.Periodicity,
                    "))\n\n", sep="")
               }
          else if({Alg == "AFSS"} |
               {(Alg == "AMWG") & Dim.Adapt & Fast & !Ready} |
               {(Alg == "AMWG") & Dim.Adapt & !Fast & !Ready} |
               {(Alg == "AMWG") & !Dim.Adapt & Fast & !Ready} | 
               {(Alg == "AMWG") & !Dim.Adapt & !Fast & !Ready}) {
               ### AFSS
               if(Ready == TRUE) A <- 0
               else if(Alg == "AFSS") A <- object$Specs[["A"]]
               else A <- Inf
               block <- "NULL"
               if(Alg == "AFSS") m <- paste(oname, "$Specs$m", sep="")
               else m <- 100
               if(!is.null(object$Specs[["B"]]) &
                    !identical(object$Specs[["B"]],list()))
                    block <- "Block"
               if(Alg == "AFSS")
                    n <- object$Specs[["n"]] + object$Iterations
               else n <- 0
               if(Alg == "AFSS")
                   w <- paste(oname, "$CovarDHis[nrow(", oname,
                        "$CovarDHis),]", sep="")
               else w <- 1
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"AFSS\", ",
                    "Specs=list(A=", A, ", B=", block, ", m=",
                    m, ",\n", sep="")
               cat("     n=", n, ", w=", w, "))\n\n", sep="")
               }
          else if(Alg == "AGG") {
               ### AGG
               Grid <- "GaussHermiteQuadRule(3)$nodes"
               CPUs <- substr(object$Call, 1, nchar(object$Call))[10]
               CPUs <- strsplit(CPUs, " ")
               pos <- grep("CPUs", CPUs[[1]])
               CPUs <- as.numeric(strsplit(CPUs[[1]][pos+2], ","))
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"AGG\", Specs=", oname,
                    "$Specs))\n\n", sep="")
               }
          else if({(Alg == "AHMC") & Dim.Adapt & !Ready} |
               {(Alg == "HMC") & Dim.Adapt & !Ready}) {
               ### AHMC
               if(L > 1) {
                    L <- round(L*(Rec.Iterations/object$Iterations))
                    Rec.Iterations <- object$Iterations
                    Rec.Status <- object$Status
                    Rec.Thinning <- object$Thinning}
               m <- "NULL"
               if(!is.null(object$Specs[["m"]]) &
                    !identical(object$Specs[["m"]],list()))
                    m <- paste(oname, "$Specs$m", sep="")
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"AHMC\",\n", sep="")
               cat("     Specs=list(epsilon=", oname, "$CovarDHis[nrow(",
                    oname, "$CovarDHis),],\n", sep="")
               cat("     L=", object$Specs[["L"]], ", m=", m,
                    ", Periodicity=", Rec.Periodicity, "))\n\n", sep="")
               }
          else if({Alg == "AIES"}) {
               Nc <- 2*LIV
               if(Acc.Rate.Level == 1)
                    Nc <- 2 + 20*(0.15 - object$Acceptance.Rate)
               if(is.null(object$Specs[["Z"]])) Z <- "NULL"
               else Z <- "object$Specs[[\"Z\"]]"
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"AIES\", Specs=list(Nc=", Nc,
                    ", Z=", Z, ",\n", sep="")
               cat("     beta=", object$Specs[["beta"]],
                    ", CPUs=", object$Specs[["CPUs"]],
                    ", Packages=NULL, Dyn.libs=NULL))\n\n", sep="")
               }
          else if({(Alg == "AM") & Dim.Adapt & Fast & !Ready} |
               {(Alg == "AM") & Dim.Adapt & !Fast & !Ready}) {
               ### AM
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"AM\", ",
                    "Specs=list(Adaptive=", Rec.Adaptive, ", Periodicity=",
                    Rec.Periodicity, "))\n\n", sep="")
               }
          else if({(Alg == "AHMC") & !Dim.Adapt} |
               {(Alg == "AM") & !Dim.Adapt & !Fast & Ready} |
               {(Alg == "AMM") & Dim.Adapt & Fast & !Ready} |
               {(Alg == "AMM") & Dim.Adapt & !Fast & !Ready} |
               {(Alg == "DRAM") & !Dim.Adapt & !Fast & Ready} |
               {(Alg == "DRM") & Dim.Adapt & Fast & !Ready} |
               {(Alg == "DRM") & Dim.Adapt & !Fast & !Ready} |
               {(Alg == "RAM") & !Dim.Adapt & !Fast & !Ready} |
               {(Alg == "RAM") & !Dim.Adapt & Fast & !Ready} |
               {(Alg == "RWM") & Dim.Adapt & Fast & !Ready} |
               {(Alg == "RWM") & Dim.Adapt & !Fast & !Ready}) {
               ### AMM
               block <- "NULL"
               if(!is.null(object$Specs[["B"]]) &
                    !identical(object$Specs[["B"]],list()))
                    block <- paste(oname, "$Specs$B", sep="")
               if(Rec.Status > Rec.Iterations)
                    Rec.Status <- Rec.Iterations
               n <- object$Iterations
               if(!is.null(object$Specs[["n"]]))
                    n <- object$Specs[["n"]] + object$Iterations
               w <- 0.05
               if(!is.null(object$Specs[["w"]]))
                    w <- object$Specs[["w"]]
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"AMM\", ",
                    "Specs=list(Adaptive=", Rec.Adaptive,
                    ", B=", block, ", n=", n, ",\n", sep="")
               cat("     Periodicity=", Rec.Periodicity, ", w=", w,
                    "))\n\n", sep="")
               }
          else if({(Alg == "AM") & !Dim.Adapt & Fast & Ready} |
               {(Alg == "AM") & !Dim.Adapt & Fast & !Ready} |
               {(Alg == "AMM") & !Dim.Adapt & Fast & Ready} |
               {(Alg == "AMM") & !Dim.Adapt & Fast & !Ready} |
               {(Alg == "DRAM") & !Dim.Adapt & Fast & Ready} |
               {(Alg == "DRAM") & !Dim.Adapt & Fast & !Ready} |
               {(Alg == "MWG") & Dim.Adapt & Fast & !Ready} |
               {(Alg == "MWG") & Dim.Adapt & !Fast & !Ready}) {
               ### AMWG
               if(Componentwise == 0) {
                    Rec.Iterations <- max(nrow(object$Posterior1),
                         trunc(Rec.Iterations / LIV))
                    Rec.Status <- trunc(interval(trunc(Rec.Status / LIV),
                         1, Rec.Iterations))
                    Rec.Thinning <- trunc(Rec.Iterations /
                         nrow(object$Posterior1))
                    Rec.Iterations <- nrow(object$Posterior1) *
                         Rec.Thinning}
               if(Rec.Periodicity > Rec.Iterations)
                    Rec.Periodicity <- max(trunc(Rec.Iterations * 0.01),1)
               block <- "NULL"
               if(!is.null(object$Specs[["B"]]) &
                    !identical(object$Specs[["B"]],list()))
                    block <- paste(oname, "$Specs$B", sep="")
               n <- object$Iterations
               if(!is.null(object$Specs[["n"]]))
                    n <- object$Specs[["n"]] + object$Iterations
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"AMWG\", ",
                    "Specs=list(B=", block, ", n=", n, ", Periodicity=",
                    Rec.Periodicity, "))\n\n", sep="")
               }
          else if(Alg == "CHARM" & Acc.Rate.Level == 2) {
               ### CHARM
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"CHARM\", ",
                    "Specs=NULL)\n\n", sep="")
               }
          if(Alg == "CHARM" & Acc.Rate.Level != 2) {
               ### CHARM (Adaptive)
               al <- 0.44
               if(!is.na(object$Specs[["alpha.star"]]))
                    al <- object$Specs[["alpha.star"]]
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"CHARM\", ",
                    "Specs=list(alpha.star=", al, "))\n\n", sep="")
               }
          else if(Alg == "DEMC" & Dim.Adapt & !Ready) {
               ### DEMC
               Nc <- max(3, round(LIV * 0.03))
               gamma <- "NULL"
               if(!is.null(object$Specs[["gamma"]]))
                    gamma <- object$Specs[["gamma"]]
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"DEMC\", ",
                    "Specs=list(Nc=", Nc, ", Z=", oname, "$Posterior1, ",
                    "gamma=", gamma,
                    ", w=", object$Specs[["w"]],"))\n\n", sep="")
               }
          else if({(Alg == "DRAM") & Dim.Adapt & Fast & !Ready} |
               {(Alg == "DRAM") & Dim.Adapt & !Fast & !Ready}) {
               ### DRAM
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"DRAM\", ",
                    "Specs=list(Adaptive=", Rec.Adaptive, ", Periodicity=",
                    Rec.Periodicity, "))\n\n", sep="")
               }
          else if({(Alg == "DRM") & Dim.Adapt & Fast & Ready} |
               {(Alg == "DRM") & Dim.Adapt & !Fast & Ready}) {
               ### DRM
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"DRM\", ",
                    "Specs=NULL))\n\n", sep="")
               }
          else if(Alg == "ESS") {
               ### ESS
               block <- "NULL"
               if(!is.null(object$Specs[["B"]]) &
                    !identical(object$Specs[["B"]],list()))
                    block <- "Block"
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"ESS\", Specs=list(B=", block,
                    "))\n\n", sep="")
               }
          else if(Alg == "GG") {
               ### GG
               Grid <- object$Specs[["Grid"]]
               dparm <- object$Specs[["dparm"]]
               CPUs <- object$Specs[["CPUs"]]
               Packages <- object$Specs[["Packages"]]
               Dyn.libs <- object$Specs[["Dyn.libs"]]
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"GG\", Specs=", oname,
                    "$Specs)\n\n", sep="")
               }
          else if(Alg == "Gibbs") {
               ### Gibbs
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"Gibbs\", ",
                    "Specs=", oname, "$Specs)\n\n", sep="")
               }
          else if(Alg == "HARM") {
               ### HARM
               al <- 0.234
               if(is.na(object$Specs[["alpha.star"]]) |
                    !is.null(object$Specs[["alpha.star"]]))
                    al <- object$Specs[["alpha.star"]]
               block <- "NULL"
               if(!is.null(object$Specs[["B"]]) &
                    !identical(object$Specs[["B"]],list()))
                    block <- "Block"
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"HARM\", ",
                    "Specs=list(alpha.star=", al, ", B=", block, ")\n\n",
                    sep="")
               }
          else if({(Alg == "AHMC") & Dim.Adapt & Ready} |
               {(Alg == "HMC") & Dim.Adapt & Ready}) {
               ### HMC
               if(L > 1) {
                    L <- round(L*(Rec.Iterations/object$Iterations))
                    Rec.Iterations <- object$Iterations
                    Rec.Status <- object$Status
                    Rec.Thinning <- object$Thinning}
               m <- "NULL"
               if(!is.null(object$Specs[["m"]]) &
                    !identical(object$Specs[["m"]],list()))
                    m <- paste(oname, "$Specs$m", sep="")
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"HMC\", ",
                    "Specs=list(epsilon=", oname, "$CovarDHis[1,], ",
                    sep="")
               cat("L=", L, ", m=", m, "))\n\n", sep="")
               }
          else if(Alg == "HMCDA" & Dim.Adapt) {
               ### HMCDA
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"HMCDA\", ",
                    "Specs=list(A=", round(Rec.Iterations/2),
                    ", delta=", object$Specs[["delta"]], ",\n", sep="")
               cat("     epsilon=",
                    round(object$CovarDHis[nrow(object$CovarDHis),1],3),
                    ", Lmax=", object$Specs[["Lmax"]],
                    ", lambda=", object$Specs[["lambda"]],
                    "))\n\n", sep="")
               }
          else if(Alg == "IM") {
               ### IM
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"IM\", ",
                    "Specs=list(mu=apply(", oname,
                    "$Posterior1, 2, mean)))\n\n", sep="")
               }
          else if(Alg == "INCA") {
               ### INCA
               detectedcores <- detectCores()
               cat(oname, " <- LaplacesDemon.hpc(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"INCA\", ",
                    "Specs=list(Adaptive=", Rec.Adaptive, ", Periodicity=",
                    Rec.Periodicity, "),\n", sep="")
               cat("     Chains=",detectedcores,
                    ", CPUs=", detectedcores,
                    ", Packages=NULL, Dyn.libs=NULL)\n\n", sep="")
               }
          else if(Alg == "MALA") {
               ### MALA
               A <- object$Specs[["A"]]
               alpha.star <- object$Specs[["alpha.star"]]
               if(Dim.Adapt & Ready) gamma <- 0
               else gamma <- object$Specs[["gamma"]]
               delta <- object$Specs[["delta"]]
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"MALA\", ",
                    "Specs=list(A=", A, ", alpha.star=", alpha.star,
                    ",\n", sep="")
               cat("          gamma=", gamma,
                    ", delta=", delta, ", epsilon=c(1e-6, 1e-7)))\n\n",
                    sep="")
               }
          else if(Alg == "MCMCMC") {
               ### MCMCMC
               lambda <- object$Specs[["lambda"]]
               CPUs <- object$Specs[["CPUs"]]
               Packages <- object$Specs[["Packages"]]
               Dyn.libs <- object$Specs[["Dyn.libs"]]
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname,"$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"MCMCMC\", Specs=list(lambda=",
                    lambda, ", CPUs=", CPUs,
                   ", Packages=NULL, Dyn.libs=NULL))\n\n", sep="")
               }
          else if(Alg == "MTM") {
               ### MTM
               K <- object$Specs[["K"]]
               CPUs <- object$Specs[["CPUs"]]
               Packages <- object$Specs[["Packages"]]
               Dyn.libs <- object$Specs[["Dyn.libs"]]
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname,"$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"MTM\", Specs=list(K=", K,
                    ", CPUs=", CPUs,
                    ", Packages=NULL, Dyn.libs=NULL))\n\n", sep="")
               }
          else if({(Alg == "ADMG") & Dim.Adapt & Fast & Ready} |
               {(Alg == "ADMG") & Dim.Adapt & !Fast & Ready} |
               {(Alg == "AMWG") & Dim.Adapt & Fast & Ready} |
               {(Alg == "AMWG") & Dim.Adapt & !Fast & Ready} |
               {(Alg == "MWG") & Dim.Adapt & Fast & Ready} |
               {(Alg == "MWG") & Dim.Adapt & !Fast & Ready}) {
               ### MWG
               block <- "NULL"
               if(!is.null(object$Specs[["B"]]) &
                    !identical(object$Specs[["B"]],list()))
                    block <- paste(oname, "$Specs$B", sep="")
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"MWG\", ",
                    "Specs=list(B=", block, "))\n\n", sep="")
               }
          else if({Alg == "NUTS"} |
               {(Alg == "HMCDA") & !Dim.Adapt}) {
               ### NUTS
               if(Alg == "HMCDA") delta <- 0.6
               else delta <- object$Specs[["delta"]]
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"NUTS\", ",
                    "Specs=list(A=", round(Rec.Iterations/2),
                    ", delta=", delta, ",\n", sep="")
               cat("     epsilon=",
                    max(round(object$CovarDHis[nrow(object$CovarDHis),1],3),
                    1e-10), ", Lmax=", object$Specs[["Lmax"]],
                    "))\n\n", sep="")
               }
          else if(Alg == "OHSS") {
               ### OHSS
               if(Ready == TRUE) A <- 0
               else A <- object$Specs[["A"]]
               n <- object$Specs[["n"]] + object$Iterations
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"OHSS\", ",
                    "Specs=list(A=", A, ", n=", n,"))\n\n", sep="")
               }
          else if(Alg == "pCN") {
               ### pCN
               beta <- object$Specs[["beta"]]
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"RWM\", ",
                    "Specs=list(beta=", beta, "))\n\n", sep="")
               }
          else if({(Alg == "AM") & !Dim.Adapt & !Fast & !Ready} | 
               {(Alg == "AMM") & !Dim.Adapt & !Fast & Ready} | 
               {(Alg == "AMM") & !Dim.Adapt & !Fast & !Ready} | 
               {(Alg == "DRAM") & !Dim.Adapt & !Fast & !Ready} |
               {(Alg == "RAM") & Dim.Adapt & Fast & !Ready} |
               {(Alg == "RAM") & Dim.Adapt & !Fast & !Ready}) {
               ### RAM
               al <- 0.234
               if(!is.null(object$Specs[["alpha.star"]]))
                    al <- object$Specs[["alpha.star"]]
               block <- "NULL"
               if(!is.null(object$Specs[["B"]]) &
                    !identical(object$Specs[["B"]],list()))
                    block <- paste(oname, "$Specs$B", sep="")
               Dist <- "N"
               if(!is.null(object$Specs[["Dist"]]))
                    Dist <- object$Specs[["Dist"]]
               gamma <- 0.66
               if(!is.null(object$Specs[["gamma"]]))
                    gamma <- object$Specs[["gamma"]]
               n <- object$Iterations
               if(!is.null(object$Specs[["n"]]))
                    n <- object$Specs[["n"]] + object$Iterations
               if(Rec.Status > Rec.Iterations) Rec.Status <- Rec.Iterations
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"RAM\", ",
                    "Specs=list(alpha.star=", al, ", B=", block,
                    ", Dist=\"", Dist, "\",\n", sep="")
               cat("     gamma=", gamma, ", n=", n, "))\n\n", sep="")
               }
          else if(Alg == "RDMH") {
               ### RDMH
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"RDMH\", ",
                    "Specs=NULL)\n\n", sep="")
               }
          else if(Alg == "Refractive") {
               ### Refractive
               Adaptive <- object$Specs[["Adaptive"]]
               m <- object$Specs[["m"]]
               w <- object$Specs[["w"]]
               if(Adaptive < object$Iterations)
                    w <- object$CovarDHis[nrow(object$CovarDHis),1]
               if(!Dim.Adapt) Adaptive <- 1
               else Adaptive <- Rec.Iterations + 1
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"Refractive\", ",
                    "Specs=list(Adaptive=", Adaptive, ", m=", m,
                    ", w=", w, ", r=1.3))\n\n", sep="")
               }
          else if(Alg == "RJ" & (Acc.Rate.Level > 1)) {
               ### RJ
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"RJ\", ",
                    "Specs=list(bin.n=", object$Specs[["bin.n"]],
                    ", bin.p=", object$Specs[["bin.p"]], ",\n", sep="")
               cat("     parm.p=", paste("c(",
                    paste(object$Specs[["parm.p"]], collapse=","),
                    ")", sep=""), ",\n", sep="")
               cat("     selectable=", paste("c(",
                    paste(object$Specs[["selectable"]], collapse=","),
                    ")", sep=""), ",\n", sep="")
               cat("     selected=1*(Initial.Values != 0)))\n\n", sep="")
               }
          else if(Alg == "RSS") {
               ### RSS
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"RSS\", ",
                    "Specs=", oname, "$Specs)\n\n", sep="")
               }
          else if({(Alg == "AM") & Dim.Adapt & Fast & Ready} |
               {(Alg == "AM") & Dim.Adapt & !Fast & Ready} |
               {(Alg == "AMM") & Dim.Adapt & Fast & Ready} |
               {(Alg == "AMM") & Dim.Adapt & !Fast & Ready} |
               {(Alg == "DRAM") & Dim.Adapt & Fast & Ready} |
               {(Alg == "DRAM") & Dim.Adapt & !Fast & Ready} |
               {(Alg == "RAM") & !Dim.Adapt & !Fast & Ready} |
               {(Alg == "RAM") & !Dim.Adapt & Fast & Ready} |
               {(Alg == "RAM") & Dim.Adapt & Fast & Ready} |
               {(Alg == "RAM") & Dim.Adapt & !Fast & Ready} |
               {(Alg == "RWM") & Dim.Adapt & Fast & Ready} |
               {(Alg == "RWM") & Dim.Adapt & !Fast & Ready}) {
               ### RWM
               block <- "NULL"
               if(!is.null(object$Specs[["B"]]) &
                    !identical(object$Specs[["B"]], list()))
                    block <- "Block"
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"RWM\", ",
                    "Specs=list(B=", block, "))\n\n", sep="")
               }
          else if({(Alg == "DEMC") & Dim.Adapt & Fast & Ready} |
               {(Alg == "DEMC") & Dim.Adapt & !Fast & Ready}) {
               ### RWM from DEMC
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=cov(", oname, "$Posterior2), Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"RWM\", ",
                    "Specs=NULL)\n\n", sep="")
               }
          else if({(Alg == "SAMWG") & !Ready} |
               {(Alg == "SMWG") & !Ready}) {
               ### SAMWG
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"SAMWG\", ",
                    "Specs=list(Dyn=Dyn, Periodicity=", Rec.Periodicity,
                    "))\n\n", sep="")
               }
          else if({(Alg == "SAMWG") & Ready} |
               {(Alg == "SMWG") & Ready}) {
               ### SMWG
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"SMWG\", ",
                    "Specs=list(Dyn=Dyn))\n\n", sep="")
               }
          else if(Alg == "SGLD") {
               ### SGLD
               eps <- object$Specs[["epsilon"]]
               Fi <- object$Specs[["file"]]
               nr <- object$Specs[["Nr"]]
               nc <- object$Specs[["Nc"]]
               size <- object$Specs[["size"]]
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"SGLD\", ",
                    "Specs=", oname, "$Specs)\n\n", sep="")
               }
          else if(Alg == "Slice") {
               ### Slice
               block <- "NULL"
               if(!is.null(object$Specs[["B"]]) &
                    !identical(object$Specs[["B"]],list()))
                    block <- paste(oname, "$Specs$B", sep="")
               Bounds <- "c(-Inf,Inf)"
               if(!is.null(object$Specs[["Bounds"]]) &
                    !identical(object$Specs[["Bounds"]],list()))
                    Bounds <- paste(oname, "$Specs$Bounds", sep="")
               m <- "Inf"
               if(!is.null(object$Specs[["m"]]) &
                    !identical(object$Specs[["m"]],list()))
                    m <- paste(oname, "$Specs$m", sep="")
               Type <- "\"Continuous\""
               if(!is.null(object$Specs[["Type"]]) &
                    !identical(object$Specs[["Type"]],list()))
                    Type <- paste(oname, "$Specs$Type", sep="")
               w <- 1
               if(!is.null(object$Specs[["w"]]) &
                    !identical(object$Specs[["w"]],list()))
                    w <- paste(oname, "$Specs$w", sep="")
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"Slice\", ",
                    "Specs=list(B=", block, ", Bounds=", Bounds, ",\n",
                    sep="")
               cat("     m=", m, ", Type=", Type, ", w=", w, "))\n\n",
                    sep="")
               }
          else if(Alg == "THMC") {
               ### THMC
               if(L > 1) {
                    L <- round(L*(Rec.Iterations/object$Iterations))
                    Rec.Iterations <- object$Iterations
                    Rec.Status <- object$Status
                    Rec.Thinning <- object$Thinning}
               m <- "NULL"
               if(!is.null(object$Specs[["m"]]) &
                    !identical(object$Specs[["m"]],list()))
                    m <- paste(oname, "$Specs$m", sep="")
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"THMC\", ",
                    "Specs=list(epsilon=", oname, "$CovarDHis[1,],\n",
                    sep="")
               cat("     L=", L, ", m=", m,
                    ", Temperature=", object$Specs[["Temperature"]],
                    "))\n\n", sep="")
               }
          else if(Alg == "t-walk" |
               {(Alg == "DEMC") & !Dim.Adapt}) {
               ### twalk
               if(Alg == "DEMC") {
                    n1 <- 4
                    at <- 6
                    aw <- 1.5
                    }
               else {
                    n1 <- object$Specs[["n1"]]
                    at <- object$Specs[["at"]]
                    aw <- object$Specs[["aw"]]}
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=NULL, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"twalk\", ",
                    "Specs=list(SIV=NULL, n1=", n1,
                    ", at=", at, ", aw=", aw, "))\n\n", sep="")
               }
          else if(Alg == "UESS") {
               ### UESS
               if(Ready == TRUE) A <- 0
               else A <- object$Specs[["A"]]
               block <- "NULL"
               if(!is.null(object$Specs[["B"]]) &
                    !identical(object$Specs[["B"]],list()))
                    block <- "Block"
               m <- object$Specs[["m"]]
               n <- object$Specs[["n"]] + object$Iterations
               cat(oname, " <- LaplacesDemon(Model, Data=", dname,
                    ", Initial.Values,\n", sep="")
               cat("     Covar=", oname, "$Covar, Iterations=",
                    Rec.Iterations, ", Status=", Rec.Status, ", ",
                    "Thinning=", Rec.Thinning, ",\n", sep="")
               cat("     Algorithm=\"UESS\", ",
                    "Specs=list(A=", A, ", B=", block, ", m=", m,
                    ", n=", n, "))\n\n", sep="")
               }
          else if((Alg == "USAMWG") | (Alg == "USMWG")) {
               ### USAMWG or USMWG
               cat("A Demonic Suggestion will not be made.\n\n")
               }
          }
     cat("Laplace's Demon is finished consorting.\n")
     }

#End

###########################################################################
# plotSamples                                                             #
#                                                                         #
# The purpose of this function is to provide basic plots that are         #
# extended to include samples. This takes a N x S matrix X with N records #
# and S simulations.                                                      #
###########################################################################

plotSamples <- function(X, Style="KDE", LB=0.025, UB=0.975, Title=NULL)
     {
     ### Initial Checks
     if(missing(X)) stop("The X argument is required.")
     if(!is.matrix(X)) stop("X is required to be a matrix.")
     Xname <- deparse(substitute(X))
     N <- nrow(X)
     S <- ncol(X)
     if({LB < 0} | {LB >= 0.5})
          stop("LB must be in the interval [0, 0.5).")
     if({UB <= 0.5} | {UB > 1})
          stop("UB must be in the interval (0.5, 1].")
     if(Style == "barplot") {
          ### Qauntiles
          qLB <- apply(X, 1, quantile, prob=LB, na.rm=TRUE)
          qMed <- apply(X, 1, quantile, prob=0.5, na.rm=TRUE)
          qUB <- apply(X, 1, quantile, prob=UB, na.rm=TRUE)
          ### Plot
          barplot(qUB, names.arg=rownames(X),
               ylim=c(min(qLB), max(qUB)),
               col=rgb(255, 0, 0, 50, maxColorValue=255),
               main=Title, xlab=Xname, ylab="Value")
          barplot(qMed, add=TRUE,
               col=rgb(255, 0, 0, 75, maxColorValue=255))
          barplot(qLB, add=TRUE,
               col=rgb(255, 0, 0, 100, maxColorValue=255))
          }
     else if(Style == "dotchart") {
          ### Qauntiles
          qLB <- apply(X, 1, quantile, prob=LB, na.rm=TRUE)
          qMed <- apply(X, 1, quantile, prob=0.5, na.rm=TRUE)
          qUB <- apply(X, 1, quantile, prob=UB, na.rm=TRUE)
          ### Plot
          dotchart(qMed, xlim=range(c(qLB,qUB)), main=Title)
          for (i in 1:nrow(X)) lines(x=c(qLB[i],qUB[i]), y=c(i,i))
          }
     else if(Style == "hist") {
          ### Histogram counts by Sample
          br <- hist(X, plot=FALSE)
          H <- matrix(0, length(br$counts), S)
          for (s in 1:S)
               H[,s] <- hist(X[,s], breaks=br$breaks, plot=FALSE)$counts
          ### Qauntiles
          qLB <- apply(H, 1, quantile, prob=LB, na.rm=TRUE)
          qMed <- apply(H, 1, quantile, prob=0.5, na.rm=TRUE)
          qUB <- apply(H, 1, quantile, prob=UB, na.rm=TRUE)
          ### Plot
          barplot(qUB, names.arg=br$mids, ylim=c(0, max(qUB)),
               col=rgb(255, 0, 0, 50, maxColorValue=255),
               main=Title, xlab=Xname, ylab="Frequency")
          barplot(qMed, add=TRUE,
               col=rgb(255, 0, 0, 75, maxColorValue=255))
          barplot(qLB, add=TRUE,
               col=rgb(255, 0, 0, 100, maxColorValue=255))
          }
     else if(Style == "KDE") {
          ### KDE by Sample
          D.y <- density(X[,1], na.rm=TRUE)
          D.x <- D.y$x
          D.y <- D.y$y
          D.y <- matrix(D.y, length(D.y), S)
          for (s in 2:S) D.y[,s] <- density(X[,s])$y
          ### Quantiles
          qLB <- apply(D.y, 1, quantile, prob=LB, na.rm=TRUE)
          qMed <- apply(D.y, 1, quantile, prob=0.5, na.rm=TRUE)
          qUB <- apply(D.y, 1, quantile, prob=UB, na.rm=TRUE)
          ### Plot
          plot(D.x, qUB, col=rgb(255, 0, 0, 50, maxColorValue=255),
               type="l", ylim=c(0,max(qUB)), main=Title, xlab=Xname,
               ylab="Density")
          polygon(x=D.x, y=qUB, col=rgb(255, 0, 0, 50, maxColorValue=255),
               border=NULL)
          polygon(x=D.x, y=qMed, col=rgb(255, 0, 0, 75, maxColorValue=255),
               border=NULL)
          polygon(x=D.x, y=qLB, col=rgb(255, 0, 0, 100, maxColorValue=255),
               border=NULL)
          }
     else if(Style == "Time-Series") {
          ### Qauntiles
          qLB <- apply(X, 1, quantile, prob=LB, na.rm=TRUE)
          qMed <- apply(X, 1, quantile, prob=0.5, na.rm=TRUE)
          qUB <- apply(X, 1, quantile, prob=UB, na.rm=TRUE)
          ### Plot
          plot(1:nrow(X), qMed, ylim=range(c(qLB,qUB)), col="white",
                main=Title, xlab="Time", ylab="Value")
          polygon(c(1:nrow(X),rev(1:nrow(X))),c(qLB,rev(qUB)),
                col=rgb(255, 0, 0, 50, maxColorValue=255),
                border=FALSE)
          lines(1:nrow(X), qMed, col="red")
          }
     else stop("Style is unknown.")
     return(invisible())
     }

#End

###########################################################################
# AcceptanceRate                                                          #
#                                                                         #
# The purpose of the AcceptanceRate function is to calculate the          #
# acceptance rate of each chain from its samples.                         #
###########################################################################

AcceptanceRate <- function(x)
     {
     if(missing(x)) stop("x is a required argument.")
     if(!is.matrix(x)) x <- as.matrix(x)
     out <- colMeans(x[-nrow(x),] != x[-1,])
     names(out) <- colnames(x)
     return(out)
     }

x <- matrix(rnorm(10*10),10,10)
colnames(x) <- paste("V", 1:10, sep="")
x[2,] <- x[1,]
AcceptanceRate(x)

#End

###########################################################################
# IAT                                                                     #
#                                                                         #
# The purpose of the IAT function is to estimate the integrated           #
# autocorrelation time of a chain, given its samples. Although the code   #
# is slightly different, it is essentially the same as the IAT function   #
# in the Rtwalk package, which is currently unavailable on CRAN.          #
###########################################################################

IAT <- function(x)
     {
     if(missing(x)) stop("The x argument is required.")
     if(!is.vector(x)) x <- as.vector(x)
     dt <- x
     n <- length(x)
     mu <- mean(dt)
     s2 <- var(dt)
     ### The maximum lag is half the sample size
     maxlag <- max(3, floor(n/2))
     #### The gammas are sums of two consecutive autocovariances
     Ga <- rep(0,2)
     Ga[1] <- s2
     lg <- 1
     Ga[1] <- Ga[1] + sum((dt[1:(n-lg)]-mu)*(dt[(lg+1):n]-mu)) / n
     m <- 1
     lg <- 2*m
     Ga[2] <- sum((dt[1:(n-lg)]-mu)*(dt[(lg+1):n]-mu)) / n
     lg <- 2*m + 1
     Ga[2] <- Ga[2] + sum((dt[1:(n-lg)]-mu)*(dt[(lg+1):n]-mu)) / n
     IAT <- Ga[1]/s2 # Add the autocorrelations
     ### RULE: while Gamma stays positive and decreasing
     while ((Ga[2] > 0.0) & (Ga[2] < Ga[1])) {
          m <- m + 1
          if(2*m + 1 > maxlag) {
               cat("Not enough data, maxlag=", maxlag, "\n")
               break}
          Ga[1] <- Ga[2]
          lg <- 2*m
          Ga[2] <- sum((dt[1:(n-lg)]-mu)*(dt[(lg+1):n]-mu)) / n
          lg <- 2*m + 1
          Ga[2] <- Ga[2] + sum((dt[1:(n-lg)]-mu)*(dt[(lg+1):n]-mu)) / n
          IAT <- IAT + Ga[1] / s2
          }
     IAT <- -1 + 2*IAT #Calculates the IAT from the gammas
     return(IAT)
     }

#End

###########################################################################
# joint.pr.plot                                                           #
#                                                                         #
# The purpose of the joint.pr.plot function is to plot a joint            #
# probability region. The joint.pr.plot function uses modified forms of   #
# the ellipse and dataEllipse functions from the car package.             #
###########################################################################

joint.pr.plot <- function(x, y, quantiles=c(0.25,0.50,0.75,0.95))
     {
     ### Function from car package for ellipses
     ellipse <- function(center, shape, radius, log="", center.pch=19,
          center.cex=1.5, segments=51, draw=TRUE, add=draw, xlab="",
          ylab="", col=palette()[2], lwd=2, fill=FALSE, fill.alpha=0.3,
          ...)
          {
          trans.colors <- function(col, alpha=0.5, names=NULL) {
               # this function by Michael Friendly
               nc <- length(col)
               na <- length(alpha)
               # make lengths conform, filling out to the longest
               if(nc != na) {
                    col <- rep(col, length.out=max(nc,na))
                    alpha <- rep(alpha, length.out=max(nc,na))}
               clr <-rbind(col2rgb(col)/255, alpha=alpha)
               col <- rgb(clr[1,], clr[2,], clr[3,], clr[4,], names=names)
               return(col)}
          logged <- function(axis=c("x", "y")){
               axis <- match.arg(axis)
               0 != length(grep(axis, log))}
          if(!(is.vector(center) && 2==length(center)))
               stop("center must be a vector of length 2")
          if(!(is.matrix(shape) && all(2==dim(shape))))
               stop("shape must be a 2 by 2 matrix")
          if(max(abs(shape - t(shape)))/max(abs(shape)) > 1e-10)
               stop("shape must be a symmetric matrix")
          angles <- (0:segments)*2*pi/segments 
          unit.circle <- cbind(cos(angles), sin(angles)) 
          Q <- chol(shape, pivot=TRUE)
          order <- order(attr(Q, "pivot"))
          ellipse <- t(center + radius*t( unit.circle %*% Q[,order]))
          colnames(ellipse) <- c("x", "y")
          if(logged("x")) ellipse[, "x"] <- exp(ellipse[, "x"])
          if(logged("y")) ellipse[, "y"] <- exp(ellipse[, "y"])
          fill.col <- trans.colors(col, fill.alpha)
          if(draw) {
               if(add) {
                    lines(ellipse, col=col, lwd=lwd, ...) 
                    if(fill) polygon(ellipse, col=fill.col, border=NA)
               }
               else {
                    plot(ellipse, type="n", xlab=xlab, ylab=ylab, ...) 
                    lines(ellipse, col=col, lwd=lwd, ... )
                    if(fill) polygon(ellipse, col=fill.col, border=NA)
               }
               if((center.pch != FALSE) && (!is.null(center.pch)))
                    points(center[1], center[2], pch=center.pch,
                         cex=center.cex, col=col)}
          invisible(ellipse)
          }
     ### Function from car package for ellipses
     dataEllipse <- function(x, y, weights, log="", quantiles=c(0.5, 0.95),
          center.pch=19, center.cex=1.5, draw=TRUE, plot.points=draw,
          add=!plot.points, segments=51, xlab=deparse(substitute(x)),
          ylab=deparse(substitute(y)), col=palette()[1:2], lwd=1,
          fill=FALSE, fill.alpha=0.3, ...)
          {
          if(length(col) == 1) col <- rep(col, 2)
          if(missing(y)) {
               if(is.matrix(x) && ncol(x) == 2) {
                    if(missing(xlab)) xlab <- colnames(x)[1]
                    if(missing(ylab)) ylab <- colnames(x)[2]
                    y <- x[,2]
                    x <- x[,1]
                    }
              else stop("x and y must be vectors, or x must be a 2 column matrix.")
              }
          else if(!(is.vector(x) && is.vector(y) && length(x) == length(y)))
               stop("x and y must be vectors of the same length")
          if(missing(weights)) weights <- rep(1, length(x))
          if(length(weights) != length(x))
               stop("weights must be of the same length as x and y")
          if(draw) {
               if(!add) {
                    mycol <- rgb(0, 100, 0, 50, maxColorValue=255)
                    plot(x, y, type="n", col=mycol, pch=16, xlab=xlab,
                         ylab=ylab, ...)
               if(plot.points) points(x, y, col=mycol, pch=16, ...)}}
          dfn <- 2
          dfd <- length(x) - 1
          v <- cov.wt(cbind(x, y), wt=weights)
          shape <- v$cov
          center <- v$center
          result <- vector("list", length=length(quantiles))
          names(result) <- quantiles
          for (i in seq(along=quantiles)) {
               level <- quantiles[i]
               radius <- sqrt(dfn * qf(level, dfn, dfd))
               result[[i]] <- ellipse(center, shape, radius, log=log,
                    center.pch=center.pch, center.cex=center.cex,
                    segments=segments, col=col[1], lwd=lwd, fill=fill,
                    fill.alpha=fill.alpha, draw=draw, ...)}
          invisible(if(length(quantiles) == 1) result[[1]] else result)
          }
     dataEllipse(x, y, quantiles=quantiles)
     }

#End

###########################################################################
# PosteriorChecks                                                         #
#                                                                         #
# The purpose of the PosteriorChecks function is to provide additional    #
# checks of the posterior, including the probability that each theta is   #
# greater than zero, kurtosis, and skewness. This function requires an    #
# object of class demonoid, laplace, or pmc. The kurtosis and skewness    #
# checks return nothing (NA) for classes laplace and vb, because          #
# parameters from Laplace Approximation and Variational Bayes are         #
# normally distributed, by definition.                                    #
###########################################################################

PosteriorChecks <- function(x, Parms=NULL)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     if(!identical(class(x), "demonoid") &
          !identical(class(x), "iterquad") &
          !identical(class(x), "laplace") &
          !identical(class(x), "pmc") &
          !identical(class(x), "vb"))
          stop("An object of class demonoid, iterquad, laplace, pmc, or vb is required.")
     ### Kurtosis and Skewness Functions
     kurtosis <- function(x) {  
          m4 <- mean((x - mean(x))^4) 
          kurt <- m4 / (sd(x)^4)-3  
          return(kurt)}
     skewness <-  function(x) {
          m3 <- mean((x - mean(x))^3)
          skew <- m3 / (sd(x)^3)
          return(skew)}
     ### Posterior Checks
     if(identical(class(x), "demonoid")) {
          ### Posterior and Monitors
          if(is.matrix(x$Posterior2) == FALSE) {
               post <- cbind(x$Posterior1, x$Monitor)
               cat("\nWARNING: Non-stationary samples used.\n\n")}
          else {
               post <- cbind(x$Posterior2,
                    x$Monitor[(x$Rec.BurnIn.Thinned+1):nrow(x$Monitor),])}
          colnames(post) <- c(colnames(x$Posterior1), colnames(x$Monitor))
          ### Selecting Parms
          if(is.null(Parms)) {keepcols <- 1:ncol(post)}
          else {
               Parms <- sub("\\[","\\\\[",Parms)
               Parms <- sub("\\]","\\\\]",Parms)
               Parms <- sub("\\.","\\\\.",Parms)
               if(length(grep(Parms[1], colnames(post))) == 0)
                    stop("Parameter in Parms does not exist.")
               keepcols <- grep(Parms[1], colnames(post))
               if(length(Parms) > 1) {
                    for (i in 2:length(Parms)) {
                         if(length(grep(Parms[i], colnames(post))) == 0)
                              stop("Parameter in Parms does not exist.")
                         keepcols <- c(keepcols,
                              grep(Parms[i], colnames(post)))}}}
          temp <- colnames(post)[keepcols]
          post <- post[,keepcols]
          colnames(post) <- temp
          ### Correlation Table
          options(warn=-1); postcor <- cor(post); options(warn=0)
          ### Summary Table
          Summ <- matrix(NA, ncol(post), 8)
          rownames(Summ) <- colnames(post)
          colnames(Summ) <- c("p(theta > 0)", "N.Modes", "Kurtosis",
               "Skewness", "Burn-In", "IAT", "ISM", "AR")
          options(warn=-1)
          for (i in 1:ncol(post)) {
               Summ[i,1] <- mean(post[,i] > 0)
               Summ[i,2] <- length(Modes(post[,i])[[1]])
               Summ[i,3] <- round(kurtosis(post[,i]),3)
               Summ[i,4] <- round(skewness(post[,i]),3)
               Summ[i,6] <- round(IAT(post[,i]),3)}
          Summ[,5] <- burnin(post)
          Summ[,7] <- round(ESS(post)/x$Min, 3)
          Summ[,8] <- AcceptanceRate(post)
          options(warn=0)
          }
     else if(identical(class(x), "iterquad")) {
          ### Posterior
          if(any(is.na(x$Posterior)))
               stop("Posterior samples do not exist.")
          post <- x$Summary1
          ### Selecting Parms
          if(is.null(Parms)) {keeprows <- 1:nrow(post)}
          else {
               Parms <- sub("\\[","\\\\[",Parms)
               Parms <- sub("\\]","\\\\]",Parms)
               Parms <- sub("\\.","\\\\.",Parms)
               if(length(grep(Parms[1], rownames(post))) == 0)
                    stop("Parameter in Parms does not exist.")
               keeprows <- grep(Parms[1], rownames(post))
               if(length(Parms) > 1) {
                    for (i in 2:length(Parms)) {
                         if(length(grep(Parms[i], rownames(post))) == 0)
                              stop("Parameter in Parms does not exist.")
                         keeprows <- c(keeprows,
                              grep(Parms[i], rownames(post)))}}}
          temp <- rownames(post)[keeprows]
          post <- post[keeprows,]
          rownames(post) <- temp
          Posterior <- x$Posterior
          colnames(Posterior) <- rownames(post)
          ### Correlation Table
          options(warn=-1); postcor <- cor(Posterior); options(warn=0)
          ### Summary Table
          Summ <- matrix(NA, nrow(post), 8)
          rownames(Summ) <- rownames(post)
          colnames(Summ) <- c("p(theta > 0)", "N.Modes", "Kurtosis",
               "Skewness", "Burn-In", "IAT", "ISM", "AR")
          options(warn=-1)
          for (i in 1:ncol(Posterior)) {
               Summ[i,1] <- mean(Posterior[,i] > 0)
               Summ[i,2] <- length(Modes(Posterior[,i])[[1]])
               Summ[i,3] <- round(kurtosis(Posterior[,i]),3)
               Summ[i,4] <- round(skewness(Posterior[,i]),3)
               Summ[i,5] <- 0
               Summ[i,6] <- round(IAT(Posterior[,i]),3)}
          Summ[,7] <- NA
          Summ[,8] <- 1
          options(warn=0)
          }
     else if(identical(class(x), "laplace")) {
          ### Posterior
          if(any(is.na(x$Posterior)))
               stop("Posterior samples do not exist.")
          post <- x$Summary1
          ### Selecting Parms
          if(is.null(Parms)) {keeprows <- 1:nrow(post)}
          else {
               Parms <- sub("\\[","\\\\[",Parms)
               Parms <- sub("\\]","\\\\]",Parms)
               Parms <- sub("\\.","\\\\.",Parms)
               if(length(grep(Parms[1], rownames(post))) == 0)
                    stop("Parameter in Parms does not exist.")
               keeprows <- grep(Parms[1], rownames(post))
               if(length(Parms) > 1) {
                    for (i in 2:length(Parms)) {
                         if(length(grep(Parms[i], rownames(post))) == 0)
                              stop("Parameter in Parms does not exist.")
                         keeprows <- c(keeprows,
                              grep(Parms[i], rownames(post)))}}}
          temp <- rownames(post)[keeprows]
          post <- post[keeprows,]
          rownames(post) <- temp
          Posterior <- x$Posterior
          colnames(Posterior) <- rownames(post)
          ### Correlation Table
          options(warn=-1); postcor <- cor(Posterior); options(warn=0)
          ### Summary Table
          Summ <- matrix(NA, nrow(post), 8)
          rownames(Summ) <- rownames(post)
          colnames(Summ) <- c("p(theta > 0)", "N.Modes", "Kurtosis",
               "Skewness", "Burn-In", "IAT", "ISM", "AR")
          options(warn=-1)
          for (i in 1:ncol(Posterior)) {
               Summ[i,1] <- mean(Posterior[,i] > 0)
               Summ[i,2] <- length(Modes(Posterior[,i])[[1]])
               Summ[i,3] <- round(kurtosis(Posterior[,i]),3)
               Summ[i,4] <- round(skewness(Posterior[,i]),3)
               Summ[i,5] <- 0
               Summ[i,6] <- round(IAT(Posterior[,i]),3)}
          Summ[,7] <- NA
          Summ[,8] <- 1
          options(warn=0)
          }
     else if(identical(class(x), "vb")) {
          ### Posterior
          if(any(is.na(x$Posterior)))
               stop("Posterior samples do not exist.")
          post <- x$Summary1
          ### Selecting Parms
          if(is.null(Parms)) {keeprows <- 1:nrow(post)}
          else {
               Parms <- sub("\\[","\\\\[",Parms)
               Parms <- sub("\\]","\\\\]",Parms)
               Parms <- sub("\\.","\\\\.",Parms)
               if(length(grep(Parms[1], rownames(post))) == 0)
                    stop("Parameter in Parms does not exist.")
               keeprows <- grep(Parms[1], rownames(post))
               if(length(Parms) > 1) {
                    for (i in 2:length(Parms)) {
                         if(length(grep(Parms[i], rownames(post))) == 0)
                              stop("Parameter in Parms does not exist.")
                         keeprows <- c(keeprows,
                              grep(Parms[i], rownames(post)))}}}
          temp <- rownames(post)[keeprows]
          post <- post[keeprows,]
          rownames(post) <- temp
          Posterior <- x$Posterior
          colnames(Posterior) <- rownames(post)
          ### Correlation Table
          options(warn=-1); postcor <- cor(Posterior); options(warn=0)
          ### Summary Table
          Summ <- matrix(NA, nrow(post), 8)
          rownames(Summ) <- rownames(post)
          colnames(Summ) <- c("p(theta > 0)", "N.Modes", "Kurtosis",
               "Skewness", "Burn-In", "IAT", "ISM", "AR")
          options(warn=-1)
          for (i in 1:ncol(Posterior)) {
               Summ[i,1] <- mean(Posterior[,i] > 0)
               Summ[i,2] <- length(Modes(Posterior[,i])[[1]])
               Summ[i,3] <- round(kurtosis(Posterior[,i]),3)
               Summ[i,4] <- round(skewness(Posterior[,i]),3)
               Summ[i,5] <- 0
               Summ[i,6] <- round(IAT(Posterior[,i]),3)}
          Summ[,7] <- NA
          Summ[,8] <- 1
          options(warn=0)
          }
     if(identical(class(x), "pmc")) {
          ### Posterior and Monitors
          post <- cbind(x$Posterior2, x$Monitor)
          colnames(post) <- c(colnames(x$Posterior2), colnames(x$Monitor))
          ### Selecting Parms
          if(is.null(Parms)) {keepcols <- 1:ncol(post)}
          else {
               Parms <- sub("\\[","\\\\[",Parms)
               Parms <- sub("\\]","\\\\]",Parms)
               Parms <- sub("\\.","\\\\.",Parms)
               if(length(grep(Parms[1], colnames(post))) == 0)
                    stop("Parameter in Parms does not exist.")
               keepcols <- grep(Parms[1], colnames(post))
               if(length(Parms) > 1) {
                    for (i in 2:length(Parms)) {
                         if(length(grep(Parms[i], colnames(post))) == 0)
                              stop("Parameter in Parms does not exist.")
                         keepcols <- c(keepcols,
                              grep(Parms[i], colnames(post)))}}}
          temp <- colnames(post)[keepcols]
          post <- post[,keepcols]
          colnames(post) <- temp
          ### Correlation Table
          options(warn=-1); postcor <- cor(post); options(warn=0)
          ### Summary Table
          Summ <- matrix(NA, ncol(post), 8)
          rownames(Summ) <- colnames(post)
          colnames(Summ) <- c("p(theta > 0)", "N.Modes", "Kurtosis",
               "Skewness", "Burn-In", "IAT", "ISM", "AR")
          options(warn=-1)
          for (i in 1:ncol(post)) {
               Summ[i,1] <- mean(post[,i] > 0)
               Summ[i,2] <- length(Modes(post[,i])[[1]])
               Summ[i,3] <- round(kurtosis(post[,i]),3)
               Summ[i,4] <- round(skewness(post[,i]),3)
               Summ[i,6] <- round(IAT(post[,i]),3)}
          Summ[,5] <- rep(1, nrow(Summ))
          Summ[,7] <- NA
          Summ[,8] <- 1
          options(warn=0)
          }
     ### Output
     out <- list(Posterior.Correlation=postcor, Posterior.Summary=Summ)
     class(out) <- "posteriorchecks"
     return(out)
     }

#End

###########################################################################
# RejectionSampling                                                       #
#                                                                         #
# The purpose of the RejectionSampling function is to perform rejection   #
# sampling.                                                               #
###########################################################################

RejectionSampling <- function(Model, Data, mu, S, df=Inf, logc, n=1000,
     CPUs=1, Type="PSOCK")
     {
     ### Initial Checks
     if(missing(Model)) stop("The Model argument is required.")
     if(missing(Data)) stop("The Data argument is required.")
     if(missing(mu)) stop("The mu argument is required.")
     if(missing(S)) stop("The S argument is required.")
     if(missing(df)) stop("The df argument is required.")
     if(missing(logc)) stop("The logc argument is required.")
     df <- abs(df)
     ### Rejection Sampling
     k <- length(mu)
     if(df == Inf) theta <- rmvn(n, mu, S)
     else theta <- rmvt(n, mu, S, df)
     lf <- rep(0, nrow(theta))
     ### Non-Parallel Processing
     if(CPUs == 1) {
          for (i in 1:nrow(theta)) {
               mod <- Model(theta[i,], Data)
               lf[i] <- mod[["LP"]]
               theta[i,] <- mod[["parm"]]}
          }
     else { ### Parallel Processing
          detectedCores <- max(detectCores(),
               as.integer(Sys.getenv("NSLOTS")), na.rm=TRUE)
          cat("\n\nCPUs Detected:", detectedCores, "\n")
          if(CPUs > detectedCores) {
               cat("\nOnly", detectedCores, "will be used.\n")
               CPUs <- detectedCores}
          cl <- makeCluster(CPUs, Type)
          varlist <- unique(c(ls(), ls(envir=.GlobalEnv),
               ls(envir=parent.env(environment()))))
          clusterExport(cl, varlist=varlist, envir=environment())
          clusterSetRNGStream(cl)
          mod <- parLapply(cl, 1:nrow(theta),
               function(x) Model(theta[x,], Data))
          stopCluster(cl)
          lf <- unlist(lapply(mod,
               function(x) x[["LP"]]))[1:nrow(theta)]
          theta <- matrix(unlist(lapply(mod,
               function(x) x[["parm"]])), nrow(theta), ncol(theta))
          rm(mod)}
     if(df == Inf) lg <- dmvn(theta, mu, S, log=TRUE)
     else lg <- dmvt(theta, mu, S, df, log=TRUE)
     prob <- exp(lf - lg - logc)
     if(k == 1) theta <- theta[runif(n) < prob]
     else theta <- theta[runif(n) < prob,]
     theta <- class("rejection")
     return(theta)
     }

#End

###########################################################################
# plot.laplace.ppc                                                        #
#                                                                         #
# The purpose of the plot.laplace.ppc function is to plot an object of    #
# class laplace.ppc.                                                      #
###########################################################################

plot.laplace.ppc <- function(x, Style=NULL, Data=NULL, Rows=NULL,
     PDF=FALSE, ...)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     if(!inherits(x, "laplace.ppc")) stop("x is not of class laplace.ppc.")
     if(is.null(Style)) Style <- "Density"
     if(is.null(Rows)) Rows <- 1:nrow(x[["yhat"]])
     ### Plots
     if(Style == "Covariates") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.Covariates.pdf")
               par(mfrow=c(3,3))}
          else par(mfrow=c(3,3), ask=TRUE)
          if(is.null(Data))
               stop("Data is required for Style=Covariates.")
          if(is.null(Data[["X"]]) & is.null(Data[["x"]]))
               stop("X or x is required in Data.")
          if(is.null(Data[["X"]]))
               co <- matrix(Data[["x"]], length(Data[["x"]]), 1)
          else if(is.null(Data[["x"]])) co <- Data[["X"]]
          temp <- summary(x, Quiet=TRUE)$Summary
          mycol <- rgb(0, 100, 0, 50, maxColorValue=255)
          for (i in 1:ncol(co)) {
               plot(co[Rows,i], temp[Rows,5], col=mycol, pch=16, cex=0.75,
                    ylim=c(min(temp[Rows,c(1,4:6)]),max(temp[Rows,c(1,4:6)])),
                    xlab=paste("X[,",i,"]", sep=""),
                    ylab="yhat",
                    sub="Gray lines are yhat at 2.5% and 95%.")
               panel.smooth(co[Rows,i], temp[Rows,5], col=mycol, pch=16,
                    cex=0.75)}}
     if(Style == "Covariates, Categorical DV") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.Covariates.Cat.pdf")
               par(mfrow=c(3,3))}
          else par(mfrow=c(3,3), ask=TRUE)
          if(is.null(Data))
               stop("Data is required for Style=Covariates.")
          if(is.null(Data[["X"]]) & is.null(Data[["x"]]))
               stop("X or x is required in Data.")
          if(is.null(Data[["X"]]))
               co <- matrix(Data[["x"]], length(Data[["x"]]), 1)
          else if(is.null(Data[["x"]])) co <- Data[["X"]]
          temp <- summary(x, Categorical=TRUE, Quiet=TRUE)$Summary
          ncat <- length(table(temp[,1]))
          mycol <- rgb(0, 100, 0, 50, maxColorValue=255)
          for (i in 1:ncol(co)) {for (j in 2:(ncat+1)) {
               plot(co[Rows,i], temp[Rows,j], col=mycol, pch=16, cex=0.75,
                    xlab=paste("X[,",i,"]", sep=""),
                    ylab=colnames(temp)[j])
               panel.smooth(co[Rows,i], temp[Rows,j], col=mycol, pch=16,
                    cex=0.75)}}}
     if(Style == "Density") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.Density.pdf")
               par(mfrow=c(3,3))}
          else par(mfrow=c(3,3), ask=TRUE)
          for (j in 1:length(Rows)) {
               plot(density(x[["yhat"]][Rows[j],]),
                    main=paste("Post. Pred. Plot of yhat[", Rows[j],
                         ",]", sep=""), xlab="Value",
                    sub="Black=Density, Red=y")
               polygon(density(x[["yhat"]][Rows[j],]), col="black",
                    border="black")
               abline(v=x[["y"]][Rows[j]], col="red")}}
     if(Style == "DW") {
          if(PDF == TRUE) pdf("PPC.Plots.DW.pdf")
          par(mfrow=c(1,1))
          epsilon.obs <- x[["y"]] - x[["yhat"]]
          N <- nrow(epsilon.obs)
          S <- ncol(epsilon.obs)
          epsilon.rep <- matrix(rnorm(N*S), N, S)
          d.obs <- d.rep <-  rep(0, S)
          for (s in 1:S) {
               d.obs[s] <- sum(c(0,diff(epsilon.obs[,s]))^2, na.rm=TRUE) / 
                    sum(epsilon.obs[,s]^2, na.rm=TRUE)
               d.rep[s] <- sum(c(0,diff(epsilon.rep[,s]))^2, na.rm=TRUE) / 
                    sum(epsilon.rep[,s]^2, na.rm=TRUE)}
          result <- "no"
          if(mean(d.obs > d.rep, na.rm=TRUE) < 0.025) result <- "positive"
          if(mean(d.obs > d.rep, na.rm=TRUE) > 0.975) result <- "negative"
          d.d.obs <- density(d.obs, na.rm=TRUE)
          d.d.rep <- density(d.rep, na.rm=TRUE)
          plot(d.d.obs, xlim=c(0,4),
               ylim=c(0, max(d.d.obs$y, d.d.rep$y)), col="white",
               main="Durbin-Watson test", 
               xlab=paste("d.obs=", round(mean(d.obs, na.rm=TRUE),2), " (",
               round(as.vector(quantile(d.obs, probs=0.025, na.rm=TRUE)),2),
               ", ", round(as.vector(quantile(d.obs, probs=0.975, na.rm=TRUE)),
               2), "), p(d.obs > d.rep) = ", round(mean(d.obs > d.rep,
               na.rm=TRUE),3), " = ", result, " autocorrelation", sep=""))
          polygon(d.d.obs, col=rgb(0,0,0,50,maxColorValue=255), border=NA)
          polygon(d.d.rep, col=rgb(255,0,0,50,maxColorValue=255), border=NA)
          abline(v=2, col="red")}
     if(Style == "DW, Multivariate, C") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.DW.M.pdf")
               par(mfrow=c(1,1))}
          else par(mfrow=c(1,1), ask=TRUE)
          if(is.null(Data))
               stop("Data is required for Style=Fitted, Multivariate, C.")
          if(is.null(Data[["Y"]])) stop("Y is required in Data.")
          M <- nrow(Data[["Y"]])
          J <- ncol(Data[["Y"]])
          epsilon.obs <- x[["y"]] - x[["yhat"]]
          N <- nrow(epsilon.obs)
          S <- ncol(epsilon.obs)
          epsilon.rep <- matrix(rnorm(N*S), N, S)
          d.obs <- d.rep <-  rep(0, S)
          for (j in 1:J) {
               for (s in 1:S) {
                    d.obs[s] <- sum(c(0,diff(epsilon.obs[((j-1)*M+1):(j*M),s]))^2, na.rm=TRUE) / 
                         sum(epsilon.obs[((j-1)*M+1):(j*M),s]^2, na.rm=TRUE)
                    d.rep[s] <- sum(c(0,diff(epsilon.rep[((j-1)*M+1):(j*M),s]))^2, na.rm=TRUE) / 
                         sum(epsilon.rep[((j-1)*M+1):(j*M),s]^2, na.rm=TRUE)}
               result <- "no"
               if(mean(d.obs > d.rep, na.rm=TRUE) < 0.025) result <- "positive"
               if(mean(d.obs > d.rep, na.rm=TRUE) > 0.975) result <- "negative"
               d.d.obs <- density(d.obs, na.rm=TRUE)
               d.d.rep <- density(d.rep, na.rm=TRUE)
               plot(d.d.obs, xlim=c(0,4),
                    ylim=c(0, max(d.d.obs$y, d.d.rep$y)), col="white",
                    main="Durbin-Watson test", 
                    xlab=paste("d.obs=", round(mean(d.obs, na.rm=TRUE),2), " (",
                    round(as.vector(quantile(d.obs, probs=0.025, na.rm=TRUE)),2),
                    ", ", round(as.vector(quantile(d.obs, probs=0.975, na.rm=TRUE)),
                    2), "), p(d.obs > d.rep) = ", round(mean(d.obs > d.rep,
                    na.rm=TRUE),3), " = ", result, " autocorrelation", sep=""),
                    sub=paste("Y[,",j,"]",sep=""))
               polygon(d.d.obs, col=rgb(0,0,0,50,maxColorValue=255),
                    border=NA)
               polygon(d.d.rep, col=rgb(255,0,0,50,maxColorValue=255),
                    border=NA)
               abline(v=2, col="red")}}
     if(Style == "ECDF") {
          if(PDF == TRUE) pdf("PPC.Plots.ECDF.pdf")
          par(mfrow=c(1,1))
          plot(ecdf(x[["y"]][Rows]), verticals=TRUE, do.points=FALSE,
               main="Cumulative Fit",
               xlab="y (black) and yhat (red; gray)",
               ylab="Cumulative Frequency")
          lines(ecdf(apply(x[["yhat"]][Rows,], 1, quantile, probs=0.975)),
               verticals=TRUE, do.points=FALSE, col="gray")
          lines(ecdf(apply(x[["yhat"]][Rows,], 1, quantile, probs=0.025)),
               verticals=TRUE, do.points=FALSE, col="gray")
          lines(ecdf(apply(x[["yhat"]][Rows,], 1, quantile, probs=0.500)),
               verticals=TRUE, do.points=FALSE, col="red")}
     if(Style == "Fitted") {
          if(PDF == TRUE) pdf("PPC.Plots.Fitted.pdf")
          par(mfrow=c(1,1))
          temp <- summary(x, Quiet=TRUE)$Summary
          plot(temp[Rows,1], temp[Rows,5], pch=16, cex=0.75,
               ylim=c(min(temp[Rows,4], na.rm=TRUE),
               max(temp[Rows,6], na.rm=TRUE)),
               xlab="y", ylab="yhat", main="Fitted")
          for (i in Rows) {
               lines(c(temp[Rows[i],1], temp[Rows[i],1]),
                    c(temp[Rows[i],4], temp[Rows[i],6]))}
          panel.smooth(temp[Rows,1], temp[Rows,5], pch=16, cex=0.75)}
     if(Style == "Fitted, Multivariate, C") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.Fitted.M.pdf")
               par(mfrow=c(1,1))}
          else par(mfrow=c(1,1), ask=TRUE)
          if(is.null(Data))
               stop("Data is required for Style=Fitted, Multivariate, C.")
          if(is.null(Data[["Y"]])) stop("Y is required in Data.")
          temp <- summary(x, Quiet=TRUE)$Summary
          for (i in 1:ncol(Data[["Y"]])) {
               temp1 <- as.vector(matrix(temp[,1], nrow(Data[["Y"]]),
                    ncol(Data[["Y"]]))[,i])
               temp2 <- as.vector(matrix(temp[,4], nrow(Data[["Y"]]),
                    ncol(Data[["Y"]]))[,i])
               temp3 <- as.vector(matrix(temp[,5], nrow(Data[["Y"]]),
                    ncol(Data[["Y"]]))[,i])
               temp4 <- as.vector(matrix(temp[,6], nrow(Data[["Y"]]),
                    ncol(Data[["Y"]]))[,i])
               plot(temp1, temp3, pch=16, cex=0.75,
                    ylim=c(min(temp2, na.rm=TRUE),
                         max(temp4, na.rm=TRUE)),
                    xlab=paste("Y[,", i, "]", sep=""), ylab="yhat",
                    main="Fitted")
               for (j in 1:nrow(Data[["Y"]])) {
                    lines(c(temp1[j], temp1[j]),
                         c(temp2[j], temp4[j]))}
               panel.smooth(temp1, temp3, pch=16, cex=0.75)}}
     if(Style == "Fitted, Multivariate, R") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.Fitted.M.pdf")
               par(mfrow=c(1,1))}
          else par(mfrow=c(1,1), ask=TRUE)
          if(is.null(Data))
               stop("Data is required for Style=Fitted, Multivariate, R.")
          if(is.null(Data[["Y"]])) stop("Y is required in Data.")
          temp <- summary(x, Quiet=TRUE)$Summary
          for (i in 1:nrow(Data[["Y"]])) {
               temp1 <- as.vector(matrix(temp[,1], nrow(Data[["Y"]]),
                    ncol(Data[["Y"]]))[i,])
               temp2 <- as.vector(matrix(temp[,4], nrow(Data[["Y"]]),
                    ncol(Data[["Y"]]))[i,])
               temp3 <- as.vector(matrix(temp[,5], nrow(Data[["Y"]]),
                    ncol(Data[["Y"]]))[i,])
               temp4 <- as.vector(matrix(temp[,6], nrow(Data[["Y"]]),
                    ncol(Data[["Y"]]))[i,])
               plot(temp1, temp3, pch=16, cex=0.75,
                    ylim=c(min(temp2, na.rm=TRUE),
                         max(temp4, na.rm=TRUE)),
                    xlab=paste("Y[,", i, "]", sep=""), ylab="yhat",
                    main="Fitted")
               for (j in 1:ncol(Data[["Y"]])) {
                    lines(c(temp1[j], temp1[j]),
                         c(temp2[j], temp4[j]))}
               panel.smooth(temp1, temp3, pch=16, cex=0.75)}}
     if(Style == "Jarque-Bera") {
          if(PDF == TRUE) pdf("PPC.Plots.Jarque.Bera.pdf")
          par(mfrow=c(1,1))
          epsilon.obs <- epsilon.rep <- x[["y"]][Rows] - x[["yhat"]][Rows,]
          kurtosis <- function(x) {  
               m4 <- mean((x-mean(x, na.rm=TRUE))^4, na.rm=TRUE) 
               kurt <- m4/(sd(x, na.rm=TRUE)^4)-3  
               return(kurt)}
          skewness <-  function(x) {
               m3 <- mean((x-mean(x, na.rm=TRUE))^3, na.rm=TRUE)
               skew <- m3/(sd(x, na.rm=TRUE)^3)
               return(skew)}
          JB.obs <- JB.rep <- rep(0, ncol(epsilon.obs))
          N <- nrow(epsilon.obs)
          for (s in 1:ncol(epsilon.obs)) {
               epsilon.rep[,s] <- rnorm(N, mean(epsilon.obs[,s],
                    na.rm=TRUE), sd(epsilon.obs[,s], na.rm=TRUE))
               K.obs <- kurtosis(epsilon.obs[,s])
               S.obs <- skewness(epsilon.obs[,s])
               K.rep <- kurtosis(epsilon.rep[,s])
               S.rep <- skewness(epsilon.rep[,s])
               JB.obs[s] <- (N/6)*(S.obs^2 + ((K.obs-3)^2)/4)
               JB.rep[s] <- (N/6)*(S.rep^2 + ((K.rep-3)^2)/4)}
          p <- round(mean(JB.obs > JB.rep, na.rm=TRUE), 3)
          result <- "Non-Normality"
          if((p >= 0.025) & (p <= 0.975)) result <- "Normality"
          d.obs <- density(JB.obs)
          d.rep <- density(JB.rep)
          plot(d.obs, xlim=c(min(d.obs$x,d.rep$x), max(d.obs$x,d.rep$x)),
               ylim=c(0, max(d.obs$y, d.rep$y)), col="white",
               main="Jarque-Bera Test",
               xlab="JB", ylab="Density",
               sub=paste("JB.obs=", round(mean(JB.obs, na.rm=TRUE),2),
               " (", round(as.vector(quantile(JB.obs, probs=0.025,
               na.rm=TRUE)),2), ",", round(as.vector(quantile(JB.obs,
               probs=0.975, na.rm=TRUE)),2), "), p(JB.obs > JB.rep) = ",
               p, " = ", result, sep=""))
          polygon(d.obs, col=rgb(0,0,0,50,maxColorValue=255), border=NA)
          polygon(d.rep, col=rgb(255,0,0,50,maxColorValue=255), border=NA)}
     if(Style == "Jarque-Bera, Multivariate, C") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.Jarque.Bera.pdf")
               par(mfrow=c(1,1))}
          else par(mfrow=c(1,1), ask=TRUE)
          if(is.null(Data))
               stop("Data is required for Style=Jarque-Bera, Multivariate, C.")
          if(is.null(Data[["Y"]])) stop("Y is required in Data.")
          M <- nrow(Data[["Y"]])
          J <- ncol(Data[["Y"]])
          epsilon.obs <- epsilon.rep <- x[["y"]] - x[["yhat"]]
          kurtosis <- function(x) {  
               m4 <- mean((x-mean(x, na.rm=TRUE))^4, na.rm=TRUE) 
               kurt <- m4/(sd(x, na.rm=TRUE)^4)-3  
               return(kurt)}
          skewness <-  function(x) {
               m3 <- mean((x-mean(x, na.rm=TRUE))^3, na.rm=TRUE)
               skew <- m3/(sd(x, na.rm=TRUE)^3)
               return(skew)}
          JB.obs <- JB.rep <- rep(0, ncol(epsilon.obs))
          N <- nrow(epsilon.obs)
          for (j in 1:J) {
               for (s in 1:ncol(epsilon.obs)) {
                    e.obs <- matrix(epsilon.obs[,s], M, J)
                    e.rep <- rnorm(M, mean(e.obs[,j], na.rm=TRUE),
                         sd(e.obs[,j], na.rm=TRUE))
                    K.obs <- kurtosis(e.obs[,j])
                    S.obs <- skewness(e.obs[,j])
                    K.rep <- kurtosis(e.rep)
                    S.rep <- skewness(e.rep)
                    JB.obs[s] <- (N/6)*(S.obs^2 + ((K.obs-3)^2)/4)
                    JB.rep[s] <- (N/6)*(S.rep^2 + ((K.rep-3)^2)/4)}
               p <- round(mean(JB.obs > JB.rep, na.rm=TRUE), 3)
               result <- "Non-Normality"
               if((p >= 0.025) & (p <= 0.975)) result <- "Normality"
               d.obs <- density(JB.obs)
               d.rep <- density(JB.rep)
               plot(d.obs, xlim=c(min(d.obs$x,d.rep$x), max(d.obs$x,d.rep$x)),
                    ylim=c(0, max(d.obs$y, d.rep$y)), col="white",
                    main="Jarque-Bera Test",
                    xlab=paste("JB for Y[,",j,"]", sep=""), ylab="Density",
                    sub=paste("JB.obs=", round(mean(JB.obs, na.rm=TRUE),2),
                    " (", round(as.vector(quantile(JB.obs, probs=0.025,
                    na.rm=TRUE)),2), ",", round(as.vector(quantile(JB.obs,
                    probs=0.975, na.rm=TRUE)),2), "), p(JB.obs > JB.rep) = ",
                    p, " = ", result, sep=""))
               polygon(d.obs, col=rgb(0,0,0,50,maxColorValue=255),
                    border=NA)
               polygon(d.rep, col=rgb(255,0,0,50,maxColorValue=255),
                    border=NA)}}
     if(Style == "Mardia") {
          if(PDF == TRUE) pdf("PPC.Plots.Mardia.pdf")
          par(mfrow=c(2,1))
          if(is.null(Data))
               stop("Data is required for Style=Mardia, C.")
          if(is.null(Data[["Y"]]))
               stop("Variable Y is required for Style=Mardia, C.")
          epsilon.obs <- x[["y"]] - x[["yhat"]]
          M <- nrow(Data[["Y"]])
          J <- ncol(Data[["Y"]])
          K3.obs <- K3.rep <- K4.obs <- K4.rep <- rep(0, ncol(epsilon.obs))
          for (s in 1:ncol(epsilon.obs)) {
               e.obs <- matrix(epsilon.obs[,s], M, J)
               e.obs.mu <- colMeans(e.obs)
               e.obs.mu.mat <- matrix(e.obs.mu, M, J, byrow=TRUE)
               e.obs.stand <- e.obs - e.obs.mu.mat
               S.obs <- var(e.obs)
               A.obs <- t(chol(S.obs))
               A.inv.obs <- solve(A.obs)
               Z.obs <- t(A.inv.obs %*% t(e.obs.stand))
               Dij.obs <- Z.obs %*% t(Z.obs)
               D2.obs <- diag(Dij.obs)
               K3.obs[s] <- mean(as.vector(Dij.obs)^3)
               K4.obs[s] <- mean(D2.obs^2)
               e.rep <- rmvn(M, e.obs.mu.mat, S.obs)
               e.rep.mu <- colMeans(e.rep)
               e.rep.mu.mat <- matrix(e.rep.mu, M, J, byrow=TRUE)
               e.rep.stand <- e.rep - e.rep.mu.mat
               S.rep <- var(e.rep)
               A.rep <- t(chol(S.rep))
               A.inv.rep <- solve(A.rep)
               Z.rep <- t(A.inv.rep %*% t(e.rep.stand))
               Dij.rep <- Z.rep %*% t(Z.rep)
               D2.rep <- diag(Dij.rep)
               K3.rep[s] <- mean(as.vector(Dij.rep)^3)
               K4.rep[s] <- mean(D2.rep^2)}
          p.K3 <- round(mean(K3.obs > K3.rep), 3)
          p.K4 <- round(mean(K4.obs > K4.rep), 3)
          K3.result <- K4.result <- "Non-Normality"
          if((p.K3 >= 0.025) & (p.K3 <= 0.975)) K3.result <- "Normality"
          if((p.K4 >= 0.025) & (p.K4 <= 0.975)) K4.result <- "Normality"
          d.K3.obs <- density(K3.obs)
          d.K3.rep <- density(K3.rep)
          d.K4.obs <- density(K4.obs)
          d.K4.rep <- density(K4.rep)
          plot(d.K3.obs, xlim=c(min(d.K3.obs$x, d.K3.rep$x),
               max(d.K3.obs$x, d.K3.rep$x)),
               ylim=c(0, max(d.K3.obs$y, d.K3.rep$y)), col="white",
               main="Mardia's Test of MVN Skewness",
               xlab="Skewness Test Statistic (K3)", ylab="Density",
               sub=paste("K3.obs=", round(mean(K3.obs, na.rm=TRUE), 2),
                    " (", round(quantile(K3.obs, probs=0.025, na.rm=TRUE),
                    2), ", ", round(quantile(K3.obs, probs=0.975,
                    na.rm=TRUE), 2), "), p(K3.obs > K3.rep) = ",
                    p.K3, " = ", K3.result, sep=""))
          polygon(d.K3.obs, col=rgb(0,0,0,50,maxColorValue=255), border=NA)
          polygon(d.K3.rep, col=rgb(255,0,0,50,maxColorValue=255), border=NA)
          plot(d.K4.obs, xlim=c(min(d.K4.obs$x, d.K4.rep$x),
               max(d.K4.obs$x, d.K4.rep$x)),
               ylim=c(0, max(d.K4.obs$y, d.K4.rep$y)), col="white",
               main="Mardia's Test of MVN Kurtosis",
               xlab="Kurtosis Test Statistic (K4)", ylab="Density",
               sub=paste("K4.obs=", round(mean(K4.obs, na.rm=TRUE), 2),
                    " (", round(quantile(K4.obs, probs=0.025, na.rm=TRUE),
                    2), ", ", round(quantile(K4.obs, probs=0.975,
                    na.rm=TRUE), 2), "), p(K4.obs > K4.rep) = ",
                    p.K4, " = ", K4.result, sep=""))
          polygon(d.K4.obs, col=rgb(0,0,0,50,maxColorValue=255), border=NA)
          polygon(d.K4.rep, col=rgb(255,0,0,50,maxColorValue=255), border=NA)}
     if(Style == "Predictive Quantiles") {
          if(PDF == TRUE) pdf("PPC.Plots.PQ.pdf")
          par(mfrow=c(1,1))
          temp <- summary(x, Quiet=TRUE)$Summary
          mycol <- rgb(0, 100, 0, 50, maxColorValue=255)
          plot(temp[Rows,1], temp[Rows,7], ylim=c(0,1), col=mycol,
               pch=16, cex=0.75, xlab="y", ylab="PQ",
               main="Predictive Quantiles")
          panel.smooth(temp[Rows,1], temp[Rows,7], col=mycol, pch=16,
               cex=0.75)
          abline(h=0.025, col="gray")
          abline(h=0.975, col="gray")}
     if(Style == "Residual Density") {
          if(PDF == TRUE) pdf("PPC.Plots.Residual.Density.pdf")
          par(mfrow=c(1,1))
          epsilon <- x[["y"]] - x[["yhat"]]
          epsilon.summary <- apply(epsilon, 1, quantile,
               probs=c(0.025,0.500,0.975), na.rm=TRUE)
          dens <- density(epsilon.summary[2,Rows], na.rm=TRUE)
          plot(dens, col="black", main="Residual Density",
               xlab=expression(epsilon), ylab="Density")
          polygon(dens, col="black", border="black")
          abline(v=0, col="red")}
     if(Style == "Residual Density, Multivariate, C") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.Residual.Density.pdf")
               par(mfrow=c(1,1))}
          else par(mfrow=c(1,1), ask=TRUE)
          if(is.null(Data))
               stop("Data is required for Style=Residual Density, Multivariate, C.")
          if(is.null(Data[["Y"]]))
               stop("Variable Y is required for Style=Residual Density, Multivariate, C.")
          epsilon <- x[["y"]] - x[["yhat"]]
          epsilon.summary <- apply(epsilon, 1, quantile,
               probs=c(0.025,0.500,0.975), na.rm=TRUE)
          epsilon.500 <- matrix(epsilon.summary[2,], nrow(Data[["Y"]]),
               ncol(Data[["Y"]]))
          for (i in 1:ncol(Data[["Y"]])) {
               dens <- density(epsilon.500[,i], na.rm=TRUE)
               plot(dens, col="black", main="Residual Density",
                    xlab=paste("epsilon[,", i, "]", sep=""),
                    ylab="Density")
               polygon(dens, col="black", border="black")
               abline(v=0, col="red")}}
     if(Style == "Residual Density, Multivariate, R") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.Residual.Density.pdf")
               par(mfrow=c(1,1))}
          else par(mfrow=c(1,1), ask=TRUE)
          if(is.null(Data))
               stop("Data is required for Style=Residual Density, Multivariate, R.")
          if(is.null(Data[["Y"]]))
               stop("Variable Y is required for Style=Residual Density, Multivariate, R.")
          epsilon <- x[["y"]] - x[["yhat"]]
          epsilon.summary <- apply(epsilon, 1, quantile,
               probs=c(0.025,0.500,0.975), na.rm=TRUE)
          epsilon.500 <- matrix(epsilon.summary[2,], nrow(Data[["Y"]]),
               ncol(Data[["Y"]]))
          for (i in 1:nrow(Data[["Y"]])) {
               dens <- density(epsilon.500[i,], na.rm=TRUE)
               plot(dens, col="black", main="Residual Density",
                    xlab=paste("epsilon[", i, ",]", sep=""),
                    ylab="Density")
               polygon(dens, col="black", border="black")
               abline(v=0, col="red")}}
     if(Style == "Residuals") {
          if(PDF == TRUE) pdf("PPC.Plots.Residuals.pdf")
          par(mfrow=c(1,1))
          epsilon <- x[["y"]] - x[["yhat"]]
          epsilon.summary <- apply(epsilon, 1, quantile,
               probs=c(0.025,0.500,0.975), na.rm=TRUE)
          plot(epsilon.summary[2,Rows], pch=16, cex=0.75,
               ylim=c(min(epsilon.summary[,Rows], na.rm=TRUE),
                    max(epsilon.summary[,Rows], na.rm=TRUE)),
               xlab="y", ylab=expression(epsilon))
          lines(rep(0, ncol(epsilon.summary[,Rows])), col="red")
          for (i in Rows) {
               lines(c(i,i), c(epsilon.summary[1,Rows[i]],
                    epsilon.summary[3,Rows[i]]), col="black")}}
     if(Style == "Residuals, Multivariate, C") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.Residuals.pdf")
               par(mfrow=c(1,1))}
          else par(mfrow=c(1,1), ask=TRUE)
          if(is.null(Data))
               stop("Data is required for Style=Residuals, Multivariate, C.")
          if(is.null(Data[["Y"]]))
               stop("Variable Y is required for Style=Residuals, Multivariate, C.")
          epsilon <- x[["y"]] - x[["yhat"]]
          epsilon.summary <- apply(epsilon, 1, quantile,
               probs=c(0.025,0.500,0.975), na.rm=TRUE)
          epsilon.025 <- matrix(epsilon.summary[1,], nrow(Data[["Y"]]),
               ncol(Data[["Y"]]))
          epsilon.500 <- matrix(epsilon.summary[2,], nrow(Data[["Y"]]),
               ncol(Data[["Y"]]))
          epsilon.975 <- matrix(epsilon.summary[3,], nrow(Data[["Y"]]),
               ncol(Data[["Y"]]))
          for (i in 1:ncol(Data[["Y"]])) {
               plot(epsilon.500[,i], pch=16, cex=0.75,
                    ylim=c(min(epsilon.025[,i], na.rm=TRUE),
                         max(epsilon.975[,i], na.rm=TRUE)),
                    xlab=paste("Y[,", i, "]", sep=""), ylab=expression(epsilon))
               lines(rep(0, nrow(epsilon.500)), col="red")
               for (j in 1:nrow(Data[["Y"]])) {
                    lines(c(j,j), c(epsilon.025[j,i],
                         epsilon.975[j,i]), col="black")}}}
     if(Style == "Residuals, Multivariate, R") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.Residuals.pdf")
               par(mfrow=c(1,1))}
          else par(mfrow=c(1,1), ask=TRUE)
          if(is.null(Data))
               stop("Data is required for Style=Residuals, Multivariate, C.")
          if(is.null(Data[["Y"]]))
               stop("Variable Y is required for Style=Residuals, Multivariate, C.")
          epsilon <- x[["y"]] - x[["yhat"]]
          epsilon.summary <- apply(epsilon, 1, quantile,
               probs=c(0.025,0.500,0.975), na.rm=TRUE)
          epsilon.025 <- matrix(epsilon.summary[1,], nrow(Data[["Y"]]),
               ncol(Data[["Y"]]))
          epsilon.500 <- matrix(epsilon.summary[2,], nrow(Data[["Y"]]),
               ncol(Data[["Y"]]))
          epsilon.975 <- matrix(epsilon.summary[3,], nrow(Data[["Y"]]),
               ncol(Data[["Y"]]))
          for (i in 1:nrow(Data[["Y"]])) {
               plot(epsilon.500[i,], pch=16, cex=0.75,
                    ylim=c(min(epsilon.025[i,], na.rm=TRUE),
                         max(epsilon.975[i,], na.rm=TRUE)),
                    xlab=paste("Y[", i, ",]", sep=""), ylab=expression(epsilon))
               lines(rep(0, ncol(epsilon.500)), col="red")
               for (j in 1:ncol(Data[["Y"]])) {
                    lines(c(j,j), c(epsilon.025[i,j],
                         epsilon.975[i,j]), col="black")}}}
     if(Style == "Space-Time by Space") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.SpaceTime.pdf")
               par(mfrow=c(1,1))}
          else par(mfrow=c(1,1), ask=TRUE)
          if(is.null(Data))
               stop("Data is required for Style=Space-Time by Space.")
          if(is.null(Data[["longitude"]]))
               stop("Variable longitude is required in Data.")
          if(is.null(Data[["latitude"]]))
               stop("Variable latitude is required in Data.")
          if(is.null(Data[["S"]])) stop("Variable S is required in Data.")
          if(is.null(Data[["T"]])) stop("Variable T is required in Data.")
          temp <- summary(x, Quiet=TRUE)$Summary
          for (s in 1:Data[["S"]]) {
               plot(matrix(temp[,1], Data[["S"]], Data[["T"]])[s,],
                    ylim=c(min(c(matrix(temp[,4], Data[["S"]], Data[["T"]])[s,],
                         matrix(temp[,1], Data[["S"]], Data[["T"]])[s,]), na.rm=TRUE),
                         max(c(matrix(temp[,6], Data[["S"]], Data[["T"]])[s,],
                         matrix(temp[,1], Data[["S"]], Data[["T"]])[s,]), na.rm=TRUE)),
                    type="l", xlab="Time", ylab="y",
                    main=paste("Space-Time at Space s=",s," of ",
                         Data[["S"]], sep=""),
                    sub="Actual=Black, Fit=Red, Interval=Transparent Red")
               polygon(c(1:Data[["T"]],rev(1:Data[["T"]])),
                    c(matrix(temp[,4], Data[["S"]], Data[["T"]])[s,],
                    rev(matrix(temp[,6], Data[["S"]], Data[["T"]])[s,])),
                    col=rgb(255, 0, 0, 50, maxColorValue=255), border=FALSE)
               lines(matrix(temp[,5], Data[["S"]], Data[["T"]])[s,], col="red")}}
     if(Style == "Space-Time by Time") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.SpaceTime.pdf")
               par(mfrow=c(1,1))}
          else par(mfrow=c(1,1), ask=TRUE)
          if(is.null(Data))
               stop("Data is required for Style=Space-Time by Time.")
          if(is.null(Data[["longitude"]]))
               stop("Variable longitude is required in Data.")
          if(is.null(Data[["latitude"]]))
               stop("Variable latitude is required in Data.")
          if(is.null(Data[["S"]])) stop("Variable S is required in Data.")
          if(is.null(Data[["T"]])) stop("Variable T is required in Data.")
          Heat <- (1-(x[["y"]]-min(x[["y"]], na.rm=TRUE)) /
               max(x[["y"]]-min(x[["y"]], na.rm=TRUE), na.rm=TRUE)) * 99 + 1
          Heat <- matrix(Heat, Data[["S"]], Data[["T"]])
          for (t in 1:Data[["T"]]) {
               plot(Data[["longitude"]], Data[["latitude"]],
                    col=heat.colors(120)[Heat[,t]],
                    pch=16, cex=0.75, xlab="Longitude", ylab="Latitude",
                    main=paste("Space-Time at t=",t," of ", Data[["T"]],
                         sep=""), sub="Red=High, Yellow=Low")}}
     if(Style == "Spatial") {
          if(PDF == TRUE) pdf("PPC.Plots.Spatial.pdf")
          par(mfrow=c(1,1))
          if(is.null(Data)) stop("Data is required for Style=Spatial.")
          if(is.null(Data[["longitude"]]))
               stop("Variable longitude is required in Data.")
          if(is.null(Data[["latitude"]]))
               stop("Variable latitude is required in Data.")
          heat <- (1-(x[["y"]][Rows]-min(x[["y"]][Rows], na.rm=TRUE)) /
               max(x[["y"]][Rows]-min(x[["y"]][Rows], na.rm=TRUE), na.rm=TRUE)) * 99 + 1
          plot(Data[["longitude"]][Rows], Data[["latitude"]][Rows],
               col=heat.colors(120)[heat],
               pch=16, cex=0.75, xlab="Longitude", ylab="Latitude",
               main="Spatial Plot", sub="Red=High, Yellow=Low")}
     if(Style == "Spatial Uncertainty") {
          if(PDF == TRUE) pdf("PPC.Plots.Spatial.Unc.pdf")
          par(mfrow=c(1,1))
          if(is.null(Data))
               stop("Data is required for Style=Spatial Uncertainty.")
          if(is.null(Data[["longitude"]]))
               stop("Variable longitude is required in Data.")
          if(is.null(Data[["latitude"]]))
               stop("Variable latitude is required in Data.")
          heat <- apply(x[["yhat"]], 1, quantile, probs=c(0.025,0.975))
          heat <- heat[2,] - heat[1,]
          heat <- (1-(heat[Rows]-min(heat[Rows])) /
               max(heat[Rows]-min(heat[Rows]))) * 99 + 1
          plot(Data[["longitude"]][Rows], Data[["latitude"]][Rows],
               col=heat.colors(120)[heat],
               pch=16, cex=0.75, xlab="Longitude", ylab="Latitude",
               main="Spatial Uncertainty Plot",
               sub="Red=High, Yellow=Low")}
     if(Style == "Time-Series") {
          if(PDF == TRUE) pdf("PPC.Plots.TimeSeries.pdf")
          par(mfrow=c(1,1))
          temp <- summary(x, Quiet=TRUE)$Summary
          plot(Rows, temp[Rows,1],
               ylim=c(min(temp[Rows,c(1,4)], na.rm=TRUE),
                    max(temp[Rows,c(1,6)], na.rm=TRUE)),
               type="l", xlab="Time", ylab="y",
               main="Plot of Fitted Time-Series",
               sub="Actual=Black, Fit=Red, Interval=Transparent Red")
          polygon(c(Rows,rev(Rows)),c(temp[Rows,4],rev(temp[Rows,6])),
                col=rgb(255, 0, 0, 50, maxColorValue=255),
                border=FALSE)
          lines(Rows, temp[Rows,1])
          lines(Rows, temp[Rows,5], col="red")}
     if(Style == "Time-Series, Multivariate, C") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.TimeSeries.pdf")
               par(mfrow=c(1,1))}
          else {par(mfrow=c(1,1), ask=TRUE)}
          if(is.null(Data))
               stop("Data is required for Style=Time-Series, Multivariate.")
          if(is.null(Data[["Y"]]))
               stop("Variable Y is required in Data.")
          temp <- summary(x, Quiet=TRUE)$Summary
          for (i in 1:ncol(Data[["Y"]])) {
          tempy <- matrix(temp[Rows,1], nrow(Data[["Y"]]),
               ncol(Data[["Y"]]))[,i]
          qLB <- matrix(temp[Rows,4], nrow(Data[["Y"]]),
               ncol(Data[["Y"]]))[,i]
          qMed <- matrix(temp[Rows,5], nrow(Data[["Y"]]),
               ncol(Data[["Y"]]))[,i]
          qUB <- matrix(temp[Rows,6], nrow(Data[["Y"]]),
               ncol(Data[["Y"]]))[,i]
          plot(1:length(tempy), tempy,
               ylim=c(min(Data[["Y"]][,i],
                    matrix(temp[Rows,4], nrow(Data[["Y"]]),
                         ncol(Data[["Y"]]))[,i], na.rm=TRUE),
                    max(Data[["Y"]][,i],
                    matrix(temp[Rows,6], nrow(Data[["Y"]]),
                         ncol(Data[["Y"]]))[,i], na.rm=TRUE)),
               type="l", xlab="Time", ylab="y",
               main=paste("Time-Series ", i, " of ", ncol(Data[["Y"]]), sep=""),
               sub="Actual=Black, Fit=Red, Interval=Transparent Red")
          polygon(c(1:length(tempy),rev(1:length(tempy))),c(qLB,rev(qUB)),
                col=rgb(255, 0, 0, 50, maxColorValue=255),
                border=FALSE)
          lines(1:length(tempy), tempy)
          lines(1:length(tempy), qMed, col="red")}}
     if(Style == "Time-Series, Multivariate, R") {
          if(PDF == TRUE) {
               pdf("PPC.Plots.TimeSeries.pdf")
               par(mfrow=c(1,1))}
          else par(mfrow=c(1,1), ask=TRUE)
          if(is.null(Data))
               stop("Data is required for Style=Time-Series, Multivariate.")
          if(is.null(Data[["Y"]]))
               stop("Variable Y is required in Data.")
          temp <- summary(x, Quiet=TRUE)$Summary
          for (i in 1:nrow(Data[["Y"]])) {
          tempy <- matrix(temp[Rows,1], nrow(Data[["Y"]]), ncol(Data[["Y"]]))[i,]
          qLB <- matrix(temp[Rows,4], nrow(Data[["Y"]]), ncol(Data[["Y"]]))[i,]
          qMed <- matrix(temp[Rows,5], nrow(Data[["Y"]]), ncol(Data[["Y"]]))[i,]
          qUB <- matrix(temp[Rows,6], nrow(Data[["Y"]]), ncol(Data[["Y"]]))[i,]
          plot(1:length(tempy), tempy,
               ylim=c(min(Data[["Y"]][i,],
                    matrix(temp[Rows,4], nrow(Data[["Y"]]),
                         ncol(Data[["Y"]]))[i,], na.rm=TRUE),
                    max(Data[["Y"]][i,],
                    matrix(temp[Rows,6], nrow(Data[["Y"]]),
                         ncol(Data[["Y"]]))[i,], na.rm=TRUE)),
               type="l", xlab="Time", ylab="y",
               main=paste("Time-Series ", i, " of ", nrow(Data[["Y"]]), sep=""),
               sub="Actual=Black, Fit=Red, Interval=Transparent Red")
          polygon(c(1:length(tempy),rev(1:length(tempy))),c(qLB,rev(qUB)),
                col=rgb(255, 0, 0, 50, maxColorValue=255),
                border=FALSE)
          lines(1:length(tempy), tempy)
          lines(1:length(tempy), qMed, col="red")}}
     if(PDF == TRUE) dev.off()
     }

#End

###########################################################################
# LaplacesDemon.RAM                                                       #
#                                                                         #
# The purpose of the LaplacesDemon.RAM function is to estimate the RAM    #
# required to update a given model and data in LaplacesDemon.             #
###########################################################################

LaplacesDemon.RAM <- function(Model, Data, Iterations, Thinning,
     Algorithm="RWM")
     {
     if(missing(Model))
          stop("The Model argument is required.")
     if(missing(Data))
          stop("The Data argument is required.")
     Const <- 1048600
     LIV <- length(Data[["parm.names"]])
     LM <- length(Data[["mon.names"]])
     Covar <- 0
     if(Algorithm %in% c("ADMG","AFSS","AM","AMM","DRAM","DRM","ESS","IM",
          "INCA","MALA","OHSS","RWM","RAM","UESS")) {
          ### Covariance is required
          Covar <- Covar + as.vector(object.size(matrix(runif(LIV*LIV),
          LIV, LIV))) / Const
          }
     else if(Algorithm %in% c("AGG","AM","AMM","AMWG","DRAM","DRM","INCA",
          "MWG","RWM","SAMWG","SMWG","USAMWG","USMWG")) {
          ### Variance is required
          Covar <- Covar + as.vector(object.size(runif(LIV))) / Const}
     Data <- as.vector(object.size(Data)) / Const
     Deviance <- as.vector(object.size(runif(round(Iterations /
          Thinning)))) / Const
     Initial.Values <- as.vector(object.size(runif(LIV))) / Const
     Model <- as.vector(object.size(Model)) / Const
     Monitor <- as.vector(object.size(matrix(runif(Iterations*LM),
          round(Iterations / Thinning), LM))) / Const
     post <- 0
     if(Algorithm %in% c("AHMC","AM","DRAM","INCA","NUTS","OHSS"))
          post <- as.vector(object.size(matrix(runif(Iterations*LIV),
               Iterations, LIV))) / Const
     Posterior1 <- as.vector(object.size(matrix(runif(round(Iterations /
          Thinning)), round(Iterations / Thinning), LIV))) / Const
     Posterior2 <- as.vector(object.size(matrix(runif(round(Iterations /
          Thinning)), round(Iterations / Thinning), LIV))) / Const
     Summary1 <- as.vector(object.size(matrix(runif((LIV+1+LM)*7),
          LIV+1+LM, 7))) / Const
     Summary2 <- as.vector(object.size(matrix(runif((LIV+1+LM)*7),
          LIV+1+LM, 7))) / Const
     mem.list <- list(Covar=Covar,
          Data=Data,
          Deviance=Deviance,
          Initial.Values=Initial.Values,
          Model=Model,
          Monitor=Monitor,
          post=post,
          Posterior1=Posterior1,
          Posterior2=Posterior2,
          Summary1=Summary1,
          Summary2=Summary2,
          Total=sum(Covar,Data,Deviance,Initial.Values,Model,Monitor,
               post,Posterior1,Posterior2,Summary1,Summary2))
     return(mem.list)
     }

#End

###########################################################################
# BigData                                                                 #
#                                                                         #
# The purpose of the BigData function is to enable the use of a data set  #
# that is larger than the computer memory (RAM).                          #
###########################################################################

BigData <- function(file, nrow, ncol, size=1, Method="add", CPUs=1,
     Type="PSOCK", FUN, ...)
     {
     FUN <- match.fun(FUN)
     N <- trunc(nrow / size)
     ### Non-Parallel Processing
     if(CPUs == 1) {
          con <- file(file, open="r")
          on.exit(close(con))
          for (i in 1:N) {
               ### Read in a Batch
               X <- matrix(scan(file=con, sep=",", #skip=skip.rows[i],
                    nlines=size, quiet=TRUE), size, ncol, byrow=TRUE)
               ### Perform Function
               if(Method == "rbind") {
                    if(i == 1) out <- FUN(X, ...)
                    else out <- rbind(out, FUN(X, ...))}
               else if(Method == "add") {
                    if(i == 1) out <- FUN(X, ...)
                    else out <- out + FUN(X, ...)}}
          }
     else { ### Parallel Processing
          skip.rows <- c(0, size * 1:(N-1))
          batch <- function(x) {
               #seek(con, 0)
               con <- file(file, open="r")
               on.exit(close(con))
               X <- matrix(scan(file=con, sep=",", skip=skip.rows[x],
                    nlines=size, quiet=TRUE), size, ncol, byrow=TRUE)
               if(Method == "add") out <- sum(FUN(X, ...))
               else out <- FUN(X, ...)
               return(out)
               }
          #library(parallel, quietly=TRUE)
          detectedCores <- max(detectCores(),
               as.integer(Sys.getenv("NSLOTS")), na.rm=TRUE)
          if(CPUs > detectedCores) CPUs <- detectedCores
          cl <- makeCluster(CPUs, Type)
          clusterSetRNGStream(cl)
          out <- parLapply(cl, 1:N, function(x) batch(x))
          stopCluster(cl)
          if(Method == "rbind") {
               out <- unlist(out)
               out <- matrix(out, length(out), 1)
               }
          else if(Method == "add") {
               out <- sum(unlist(out))}
          }
     return(out)
     }

#End

###########################################################################
# Blocks                                                                  #
#                                                                         #
# The purpose of the Blocks function is to return a list in which each    #
# component is a block and contains a vector of positions that indicate   #
# parameter membership.                                                   #
###########################################################################

Blocks <- function(Initial.Values, N, PostCor=NULL)
     {
     ### Initial Checks
     if(missing(Initial.Values))
          stop("The Initial.Values argument is required.")
     Initial.Values <- as.vector(Initial.Values)
     LIV <- length(Initial.Values)
     if(LIV <= 1) stop("More Initial.Values are needed to create blocks.")
     if(missing(N)) N <- trunc(sqrt(LIV))
     if(is.null(PostCor)) {
          if(length(N != 1)) N <- N[1]
          N <- max(min(round(N), LIV), 2)
          }
     if(!is.null(PostCor)) {
          if(!is.matrix(PostCor)) stop("PostCor must be a matrix.")
          if(!identical(nrow(PostCor), ncol(PostCor)))
               stop("PostCor must be a square matrix.")
          if(any(diag(PostCor) != 1))
               stop("PostCor requires the diagonal to contain 1s.")
          if(length(N) == 1) N <- max(min(round(N), LIV), 2)
          else {
               N <- N[1:2]
               N[1] <- max(min(round(N[1]), LIV), 2)
               N[2] <- max(min(round(N[2]), LIV), 2)
               if(N[1] >= N[2]) stop("N is incorrect.")
               }
          }
     ### Silhouette (from silhouette.default.R in package cluster)
     silhouette <- function(x, dist, dmatrix, ...)
          {
          cll <- match.call()
          if(is.list(x) && !is.null(cl <- x$clustering)) x <- cl
          n <- length(x)
          if(!all(x == round(x))) stop("'x' must only have integer codes")
          k <- length(clid <- sort(unique(x)))
          if(k <= 1 || k >= n) return(NA)
          if(missing(dist)) {
               if(missing(dmatrix))
                    stop("Need either a dissimilarity 'dist' or diss.matrix 'dmatrix'")
               if(is.null(dm <- dim(dmatrix)) || length(dm) != 2 || !all(n == dm))
                    stop("'dmatrix' is not a dissimilarity matrix compatible to 'x'")
               }
          else {
               dist <- as.dist(dist)
               if(n != attr(dist, "Size"))
                    stop("clustering 'x' and dissimilarity 'dist' are incompatible")
               dmatrix <- as.matrix(dist)}
          wds <- matrix(NA, n,3, dimnames =
               list(names(x), c("cluster","neighbor","sil_width")))
          for (j in 1:k) {
               Nj <- sum(iC <- x == clid[j])
               wds[iC, "cluster"] <- clid[j]
               diC <- rbind(apply(dmatrix[!iC, iC, drop=FALSE], 2,
                    function(r) tapply(r, x[!iC], mean)))
               minC <- apply(diC, 2, which.min)
               wds[iC,"neighbor"] <- clid[-j][minC]
               s.i <- if(Nj > 1) {
                    a.i <- colSums(dmatrix[iC, iC])/(Nj - 1)
                    b.i <- diC[cbind(minC, seq(along=minC))]
                    ifelse(a.i != b.i, (b.i - a.i) / pmax(b.i, a.i), 0)
                    }
               else 0
               wds[iC,"sil_width"] <- s.i}
          return(wds)
          }
     ### Create Blocks
     if(missing(PostCor)) {
          ### Sequential Blocks
          B <- list()
          pos <- 0
          for (i in 1:N) {
               if(i != N) {
                    B[[i]] <- pos + c(1:trunc(LIV / N))
                    pos <- pos + trunc(LIV / N)
                    }
               else B[[i]] <- c((pos + 1):LIV)}
          }
     else {
          ### Hierarchical Clustering
          di <- dist(1-abs(PostCor))
          hc <- hclust(di, "ave")
          if(length(N) == 1) clusters <- as.vector(cutree(hc, N))
          else {
               av.width <- rep(0, (N[2]-N[1])+1)
               names(av.width) <- seq(from=N[1], to=N[2])
               count <- 1
               for (i in N[1]:N[2]) {
                    av.width[count] <- mean(silhouette(cutree(hc, i), di)[,3])
                    count <- count + 1}
               cat("\nMean Silhouette Width per Hierarchical Cluster Solution:\n")
               print(av.width)
               N <- N[1] - 1 + which.max(av.width)[1]
               cat("\n", N, "Blocks will be created.\n")
               clusters <- as.vector(cutree(hc, N))
               }
          B <- list()
          for (i in 1:N) B[[i]] <- which(clusters == i)
          }
     class(B) <- "blocks"
     return(B)
     }

#End

###########################################################################
# LaplaceApproximation                                                    #
#                                                                         #
# The purpose of the LaplaceApproximation function is to maximize the     #
# logarithm of the unnormalized joint posterior distribution of a         #
# Bayesian model with one of many optimization algorithms.                #
###########################################################################

LaplaceApproximation <- function(Model, parm, Data, Interval=1.0E-6,
     Iterations=100, Method="SPG", Samples=1000, CovEst="Hessian",
     sir=TRUE, Stop.Tolerance=1.0E-5, CPUs=1, Type="PSOCK")
     {
     ##########################  Initial Checks  ##########################
     time1 <- proc.time()
     LA.call <- match.call()
     if(missing(Model)) stop("Model is a required argument.")
     if(!is.function(Model)) stop("Model must be a function.")
     if(missing(Data)) stop("Data is a required argument.")
     if(missing(parm)) {
          cat("Initial values were not supplied, and\n")
          cat("have been set to zero prior to LaplaceApproximation().\n")
          parm <- rep(0, length(Data[["parm.names"]]))}
     if(is.null(Data[["mon.names"]]))
          stop("In Data, mon.names is NULL.")
     if(is.null(Data[["parm.names"]]))
          stop("In Data, parm.names is NULL.")
     for (i in 1:length(Data)) {
          if(is.matrix(Data[[i]])) {
               if(all(is.finite(Data[[i]]))) {
                    mat.rank <- qr(Data[[i]], tol=1e-10)$rank
                    if(mat.rank < ncol(Data[[i]])) {
                         cat("WARNING: Matrix", names(Data)[[i]],
                              "may be rank-deficient.\n")}}}}
     if({Interval <= 0} | {Interval > 1}) Interval <- 1.0E-6
     Iterations <- min(max(round(Iterations), 10), 1000000)
     "%!in%" <- function(x,table) return(match(x, table, nomatch=0) == 0)
     if(Method %!in% c("AGA","BFGS","BHHH","CG","DFP","HAR","HJ","LBFGS",
          "LM","NM","NR","PSO","Rprop","SGD","SOMA","SPG","SR1","TR"))
          stop("Method is unknown.")
     if(Stop.Tolerance <= 0) Stop.Tolerance <- 1.0E-5
     as.character.function <- function(x, ... )
          {
          fname <- deparse(substitute(x))
          f <- match.fun(x)
          out <- c(sprintf('"%s" <- ', fname), capture.output(f))
          if(grepl("^[<]", tail(out,1))) out <- head(out, -1)
          return(out)
          }
     acount <- length(grep("apply", as.character.function(Model)))
     if(acount > 0) {
          cat("Suggestion:", acount, " possible instance(s) of apply functions\n")
          cat(     "were found in the Model specification. Iteration speed will\n")
          cat("     increase if apply functions are vectorized in R or coded\n")}
     acount <- length(grep("for", as.character.function(Model)))
     if(acount > 0) {
          cat("Suggestion:", acount, " possible instance(s) of for loops\n")
          cat("     were found in the Model specification. Iteration speed will\n")
          cat("     increase if for loops are vectorized in R or coded in a\n")
          cat("     faster language such as C++ via the Rcpp package.\n")}
     ### Sample Size of Data
     if(!is.null(Data[["n"]])) if(length(Data[["n"]]) == 1) N <- Data[["n"]] 
     if(!is.null(Data[["N"]])) if(length(Data[["N"]]) == 1) N <- Data[["N"]] 
     if(!is.null(Data[["y"]])) N <- nrow(matrix(Data[["y"]]))
     if(!is.null(Data[["Y"]])) N <- nrow(matrix(Data[["Y"]]))
     if(Method == "SGD") if(!is.null(Data[["Nr"]])) N <- Data[["Nr"]]
     if(!is.null(N)) cat("Sample Size: ", N, "\n")
     else stop("Sample size of Data not found in n, N, y, or Y.")
     ###########################  Preparation  ############################
     m.old <- Model(parm, Data)
     if(!is.list(m.old)) stop("Model must return a list.")
     if(length(m.old) != 5) stop("Model must return five components.")
     if(any(names(m.old) != c("LP","Dev","Monitor","yhat","parm")))
          stop("Name mismatch in returned list of Model function.")
     if(length(m.old[["LP"]]) > 1) stop("Multiple joint posteriors exist!")
     if(!identical(length(parm), length(m.old[["parm"]])))
          stop("The number of initial values and parameters differs.")
     if(!is.finite(m.old[["LP"]])) {
          cat("Generating initial values due to a non-finite posterior.\n")
          if(!is.null(Data[["PGF"]]))
               Initial.Values <- GIV(Model, Data, PGF=TRUE)
          else Initial.Values <- GIV(Model, Data, PGF=FALSE)
          m.old <- Model(Initial.Values, Data)
          }
     if(!is.finite(m.old[["LP"]])) stop("The posterior is non-finite.")
     if(!is.finite(m.old[["Dev"]])) stop("The deviance is non-finite.")
     parm <- m.old[["parm"]]
     if(!identical(Model(m.old[["parm"]], Data)[["LP"]], m.old[["LP"]])) {
          cat("WARNING: LP differs when initial values are held constant.\n")
          cat("     Derivatives may be problematic if used.\n")}
     ####################  Begin Laplace Approximation  ###################
     cat("Laplace Approximation begins...\n")
     if(Method == "AGA") {
          LA <- .laaga(Model, parm, Data, Interval, Iterations,
               Stop.Tolerance, m.old)
          }
     else if(Method == "BFGS") {
          LA <- .labfgs(Model, parm, Data, Interval, Iterations,
               Stop.Tolerance, m.old)
          }
     else if(Method == "BHHH") {
          LA <- .labhhh(Model, parm, Data, Interval, Iterations,
               Stop.Tolerance, m.old)
          }
     else if(Method == "CG") {
          LA <- .lacg(Model, parm, Data, Iterations, Stop.Tolerance, m.old)
          }
     else if(Method == "DFP") {
          LA <- .ladfp(Model, parm, Data, Interval, Iterations,
               Stop.Tolerance, m.old)
          }
     else if(Method == "HAR") {
          LA <- .lahar(Model, parm, Data, Iterations, Stop.Tolerance, m.old)
          }
     else if(Method == "HJ") {
          LA <- .lahj(Model, parm, Data, Interval, Iterations, Stop.Tolerance,
               m.old)
          }
     else if(Method == "LBFGS") {
          LA <- .lalbfgs(Model, parm, Data, Iterations, Stop.Tolerance,
               m.old)
          }
     else if(Method == "LM") {
          LA <- .lalm(Model, parm, Data, Iterations, Stop.Tolerance, m.old)
          }
     else if(Method == "NM") {
          LA <- .lanm(Model, parm, Data, Iterations, Stop.Tolerance, m.old)
          }
     else if(Method == "NR") {
          LA <- .lanr(Model, parm, Data, Interval, Iterations,
               Stop.Tolerance, m.old)
          }
     else if(Method == "PSO") {
          LA <- .lapso(Model, parm, Data, Iterations, Stop.Tolerance, m.old)
          }
     else if(Method == "Rprop") {
          LA <- .larprop(Model, parm, Data, Interval, Iterations,
               Stop.Tolerance, m.old)
          }
     else if(Method == "SGD") {
          LA <- .lasgd(Model, parm, Data, Iterations, Stop.Tolerance, m.old)
          }
     else if(Method == "SOMA") {
          LA <- .lasoma(Model, parm, Data,
               options=list(maxiter=Iterations, tol=Stop.Tolerance))
          }
     else if(Method == "SPG") {
          LA <- .laspg(Model, parm, Data, Interval, Iterations,
               Stop.Tolerance, m.old)
          }
     else if(Method == "SR1") {
          LA <- .lasr1(Model, parm, Data, Interval, Iterations,
               Stop.Tolerance, m.old)
          }
     else if(Method == "TR") {
          LA <- .latr(Model, parm, Data, Iterations, m.old)
          }
     Dev <- as.vector(LA$Dev)
     if(is.null(LA$H)) H <- FALSE
     else H <- LA$H
     iter <- LA$iter
     parm.len <- LA$parm.len
     parm.new <- LA$parm.new
     parm.old <- LA$parm.old
     post <- LA$post
     Step.Size <- LA$Step.Size
     tol.new <- LA$tol.new
     rm(LA)
     if(iter == 1) stop("LaplaceApproximation stopped at iteration 1.")
     if(tol.new <= Stop.Tolerance) converged <- TRUE
     else converged <- FALSE
     ### Column names to samples
     if(ncol(post) == length(Data[["parm.names"]]))
          colnames(post) <- Data[["parm.names"]]
     rownames(post) <- 1:nrow(post)
     ########################  Covariance Matirx  #########################
     cat("Estimating the Covariance Matrix\n")
     if(all(H == FALSE)) {
          VarCov <- CovEstim(Model, parm.new, Data, Method=CovEst)
          }
     else {
          VarCov <- -as.inverse(as.symmetric.matrix(H))
          diag(VarCov) <- abs(diag(VarCov))
          }
     #################  Sampling Importance Resampling  ##################
     if({sir == TRUE} & {converged == TRUE}) {
          cat("Sampling from Posterior with Sampling Importance Resampling\n")
          posterior <- SIR(Model, Data, mu=parm.new, Sigma=VarCov,
               n=Samples, CPUs=CPUs, Type=Type)
          Mon <- matrix(0, nrow(posterior), length(Data[["mon.names"]]))
          dev <- rep(0, nrow(posterior))
          for (i in 1:nrow(posterior)) {
               mod <- Model(posterior[i,], Data)
               dev[i] <- mod[["Dev"]]
               Mon[i,] <- mod[["Monitor"]]
               }
          colnames(Mon) <- Data[["mon.names"]]}
     else {
          if({sir == TRUE} & {converged == FALSE})
               cat("Posterior samples are not drawn due to Converge=FALSE\n")
          posterior <- NA; Mon <- NA}
     #####################  Summary, Point-Estimate  ######################
     cat("Creating Summary from Point-Estimates\n")
     Summ1 <- matrix(NA, parm.len, 4, dimnames=list(Data[["parm.names"]],
          c("Mode","SD","LB","UB")))
     Summ1[,1] <- parm.new
     Summ1[,2] <- sqrt(diag(VarCov))
     Summ1[,3] <- parm.new - 2*Summ1[,2]
     Summ1[,4] <- parm.new + 2*Summ1[,2]
     ###################  Summary, Posterior Samples  ####################
     Summ2 <- NA
     if({sir == TRUE} & {converged == TRUE}) {
          cat("Creating Summary from Posterior Samples\n")
          Summ2 <- matrix(NA, ncol(posterior), 7,
               dimnames=list(Data[["parm.names"]],
                    c("Mode","SD","MCSE","ESS","LB","Median","UB")))
          Summ2[,1] <- colMeans(posterior)
          Summ2[,2] <- sqrt(.colVars(posterior))
          Summ2[,3] <- Summ2[,2] / sqrt(nrow(posterior))
          Summ2[,4] <- rep(nrow(posterior), ncol(posterior))
          Summ2[,5] <- apply(posterior, 2, quantile, c(0.025))
          Summ2[,6] <- apply(posterior, 2, quantile, c(0.500))
          Summ2[,7] <- apply(posterior, 2, quantile, c(0.975))
          Deviance <- rep(0, 7)
          Deviance[1] <- mean(dev)
          Deviance[2] <- sd(dev)
          Deviance[3] <- sd(dev) / sqrt(nrow(posterior))
          Deviance[4] <- nrow(posterior)
          Deviance[5] <- as.numeric(quantile(dev, probs=0.025, na.rm=TRUE))
          Deviance[6] <- as.numeric(quantile(dev, probs=0.500, na.rm=TRUE))
          Deviance[7] <- as.numeric(quantile(dev, probs=0.975, na.rm=TRUE))
          Summ2 <- rbind(Summ2, Deviance)
          for (j in 1:ncol(Mon)) {
               Monitor <- rep(NA,7)
               Monitor[1] <- mean(Mon[,j])
               Monitor[2] <- sd(as.vector(Mon[,j]))
               Monitor[3] <- sd(as.vector(Mon[,j])) / sqrt(nrow(Mon))
               Monitor[4] <- nrow(Mon)
               Monitor[5] <- as.numeric(quantile(Mon[,j], probs=0.025,
                    na.rm=TRUE))
               Monitor[6] <- as.numeric(quantile(Mon[,j], probs=0.500,
                    na.rm=TRUE))
               Monitor[7] <- as.numeric(quantile(Mon[,j], probs=0.975,
                    na.rm=TRUE))
               Summ2 <- rbind(Summ2, Monitor)
               rownames(Summ2)[nrow(Summ2)] <- Data[["mon.names"]][j]
               }
          }
     ###############  Logarithm of the Marginal Likelihood  ###############
     LML <- list(LML=NA, VarCov=VarCov)
     if({sir == TRUE} & {converged == TRUE}) {
          cat("Estimating Log of the Marginal Likelihood\n")
          lml <- LML(theta=posterior, LL=(dev*(-1/2)), method="NSIS")
          LML[[1]] <- lml[[1]]}
     else if({sir == FALSE} & {converged == TRUE}) {
          cat("Estimating Log of the Marginal Likelihood\n")
          LML <- LML(Model, Data, Modes=parm.new, Covar=VarCov,
               method="LME")}
     colnames(VarCov) <- rownames(VarCov) <- Data[["parm.names"]]
     time2 <- proc.time()
     #############################  Output  ##############################
     LA <- list(Call=LA.call,
          Converged=converged,
          Covar=VarCov,
          Deviance=as.vector(Dev),
          History=post,
          Initial.Values=parm,
          Iterations=iter,
          LML=LML[[1]],
          LP.Final=as.vector(Model(parm.new, Data)[["LP"]]),
          LP.Initial=m.old[["LP"]],
          Minutes=round(as.vector(time2[3] - time1[3]) / 60, 2),
          Monitor=Mon,
          Posterior=posterior,
          Step.Size.Final=Step.Size,
          Step.Size.Initial=1,
          Summary1=Summ1,
          Summary2=Summ2,
          Tolerance.Final=tol.new,
          Tolerance.Stop=Stop.Tolerance)
     class(LA) <- "laplace"
     cat("Laplace Approximation is finished.\n\n")
     return(LA)
     }
.laaga <- function(Model, parm, Data, Interval, Iterations, Stop.Tolerance,
     m.old)
     {
     alpha.star <- 0.234
     Dev <- matrix(m.old[["Dev"]],1,1)
     parm.len <- length(as.vector(parm))
     parm.new <- parm.old <- m.old[["parm"]]
     names(parm.new) <- names(parm.old) <- Data[["parm.names"]]
     tol.new <- 1
     Step.Size  <- 1 / parm.len
     post <- matrix(parm.new, 1, parm.len)
     for (iter in 1:Iterations) {
          parm.old <- parm.new
          ### Print Status
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(m.old[["LP"]],1), "\n")
          ### Approximate Truncated Gradient
          approx.grad <- partial(Model, parm.old, Data, Interval)
          approx.grad <- interval(approx.grad, -1000, 1000, reflect=FALSE)
          ### Proposal
          parm.new <- parm.old + Step.Size * approx.grad
          if(any(!is.finite(parm.new))) parm.new <- parm.old
          m.new <- Model(parm.new, Data)
          tol.new <- max(sqrt(sum(approx.grad^2)),
               sqrt(sum({parm.new - parm.old}^2)))
          if(any(!is.finite(c(m.new[["LP"]], m.new[["Dev"]],
               m.new[["Monitor"]])))) {
               m.new <- m.old
               parm.new <- parm.old}
          ### Accept/Reject and Adapt
          if(m.new[["LP"]] > m.old[["LP"]]) {
               m.old <- m.new
               parm.new <- m.new[["parm"]]
               Step.Size <- Step.Size + (Step.Size / (alpha.star *
                    (1 - alpha.star))) * (1 - alpha.star) / iter
               }
          else {
               m.new <- m.old
               parm.new <- parm.old
               Step.Size <- abs(Step.Size - (Step.Size / (alpha.star *
                    (1 - alpha.star))) * alpha.star / iter)
               }
          post <- rbind(post, parm.new)
          Dev <- rbind(Dev, m.new[["Dev"]])
          if(tol.new <= Stop.Tolerance) break
          }
     Dev <- Dev[-1,]; post <- post[-1,]
     ### Output
     LA <- list(Dev=Dev, iter=iter, parm.len=parm.len, parm.new=parm.new,
          parm.old=parm.old, post=post, Step.Size=Step.Size,
          tol.new=tol.new)
     return(LA)
     }
.labfgs <- function(Model, parm, Data, Interval, Iterations,
     Stop.Tolerance, m.old) {
     m.new <- m.old
     Dev <- matrix(m.old[["Dev"]],1,1)
     parm.old <- parm
     parm.len <- length(as.vector(parm))
     post <- matrix(m.old[["parm"]], 1, parm.len)
     tol.new <- 1
     keepgoing <- TRUE
     g.old <- g.new <- rep(0, parm.len)
     B <- diag(parm.len) #Approximate Hessian
     options(warn=-1)
     for (iter in 2:Iterations) {
          ### Print Status
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(m.old[["LP"]],1), "\n")
          ### Gradient and Direction p
          g.old <- g.new
          g.new <- -1*partial(Model, m.old[["parm"]], Data, Interval)
          p <- as.vector(tcrossprod(g.new, -B))
          p[which(!is.finite(p))] <- 0
          ### Step-size Line Search
          Step.Size <- 0.8
          changed <- TRUE
          while(m.new[["LP"]] <= m.old[["LP"]] & changed == TRUE) {
               Step.Size <- Step.Size * 0.2
               s <- Step.Size*p
               prop <- m.old[["parm"]] + s
               changed <- !identical(m.old[["parm"]], prop)
               m.new <- Model(prop, Data)
               if(any(!is.finite(c(m.new[["LP"]], m.new[["Dev"]],
                    m.new[["Monitor"]]))))
                    m.new <- m.old
               }
          ### BFGS Update to Approximate Hessian B
          if(m.new[["LP"]] > m.old[["LP"]]) {
               m.old <- m.new
               g <- g.new - g.old
               CC <- sum(s*g) #Curvature condition
               if(CC > 0) {
                    y <- as.vector(crossprod(B, g))
                    D <- as.double(1 + crossprod(g, y)/CC)
                    B <- B - (tcrossprod(s, y) + tcrossprod(y, s) -
                         D * tcrossprod(s, s))/CC}
               if(any(!is.finite(B))) B <- diag(parm.len)
               }
          ### Storage
          post <- rbind(post, m.old[["parm"]])
          Dev <- rbind(Dev, m.old[["Dev"]])
          ### Tolerance
          tol.new <- sqrt(sum(s*s))
          if(keepgoing == FALSE) tol.new <- 0
          if(tol.new <= Stop.Tolerance) break
          }
     options(warn=0)
     ### Output
     LA <- list(Dev=Dev, iter=iter, parm.len=parm.len,
          parm.new=m.old[["parm"]], parm.old=parm.old, post=post,
          Step.Size=Step.Size, tol.new=tol.new)
     return(LA)
     }
.labhhh <- function(Model, parm, Data, Interval, Iterations=100,
     Stop.Tolerance, m.old)
     {
     ### Check data for X and y or Y
     if(is.null(Data[["X"]])) stop("X is required in the data.")
     y <- TRUE
     if(is.null(Data[["y"]])) {
          y <- FALSE
          if(is.null(Data[["Y"]])) stop("y or Y is required in the data.")}
     if(y == TRUE) {
          if(length(Data[["y"]]) != nrow(Data[["X"]]))
               stop("length of y differs from rows in X.")
          }
     else {
          if(nrow(Data[["Y"]]) != nrow(Data[["X"]]))
               stop("The number of rows differs in y and X.")}
     m.new <- m.old
     Dev <- matrix(m.old[["Dev"]],1,1)
     parm.old <- parm
     parm.len <- length(parm)
     post <- matrix(parm, 1, parm.len)
     tol.new <- 1
     options(warn=-1)
     for (iter in 2:Iterations) {
          ### Print Status
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(m.old[["LP"]],1), "\n")
          ### Gradient p, OPG A from gradient g, and direction delta
          p <- partial(Model, m.old[["parm"]], Data, Interval)
          A <- matrix(0, parm.len, parm.len)
          for (i in 1:nrow(Data[["X"]])) {
               Data.temp <- Data
               Data.temp$X <- Data.temp$X[i,,drop=FALSE]
               if(y == TRUE) Data.temp$y <- Data.temp$y[i]
               else Data.temp$Y <- Data.temp$Y[i,]
               g <- partial(Model, m.old[["parm"]], Data.temp, Interval)
               A <- A + tcrossprod(g,g)}
          A <- -as.inverse(as.symmetric.matrix(A))
          delta <- as.vector(tcrossprod(p, -A))
          ### Step-size Line Search
          Step.Size <- 0.8
          changed <- TRUE
          while(m.new[["LP"]] <= m.old[["LP"]] & changed == TRUE) {
               Step.Size <- Step.Size * 0.2
               s <- Step.Size*delta
               prop <- m.old[["parm"]] + s
               changed <- !identical(m.old[["parm"]], prop)
               m.new <- Model(prop, Data)
               if(any(!is.finite(c(m.new[["LP"]], m.new[["Dev"]],
                    m.new[["Monitor"]]))))
                    m.new <- m.old
               }
          if(m.new[["LP"]] > m.old[["LP"]]) m.old <- m.new
          ### Storage
          post <- rbind(post, m.old[["parm"]])
          Dev <- rbind(Dev, m.old[["Dev"]])
          ### Tolerance
          tol.new <- sqrt(sum(delta*delta))
          if(tol.new < Stop.Tolerance) break
          }
     options(warn=0)
     ### Output
     LA <- list(Dev=Dev, iter=iter, parm.len=parm.len,
          parm.new=m.old[["parm"]], parm.old=parm.old, post=post,
          Step.Size=Step.Size, tol.new=tol.new)
     return(LA)
     }
.lacg <- function(Model, parm, Data, Iterations, Stop.Tolerance, m.old)
     {
     m.best <- m.new <- m.old
     parm.len <- length(parm)
     tol <- Stop.Tolerance
     tol.new <- 1
     Dev <- matrix(m.old[["Dev"]],1,1)
     post <- matrix(parm, 1, parm.len)
     eps <- 1e-7
     bvec <- parm.old <- parm
     n <- length(bvec)
     ig <- 0 # count gradient evaluations
     stepredn <- 0.15 # Step reduction in line search
     acctol <- 1e-04 # acceptable point tolerance
     reltest <- 100 # relative equality test
     accpoint <- as.logical(FALSE)  # so far do not have an acceptable point
     cyclimit <- min(2.5 * n, 10 + sqrt(n))
     fmin <- f <- m.old[["LP"]] * -1
     bvec <- m.old[["parm"]]
     keepgoing <- TRUE
     oldstep <- steplength <- 0.8
     fdiff <- NA # initially no decrease
     cycle <- 0 # !! cycle loop counter
     options(warn=-1)
     while (keepgoing == TRUE) {
          t <- as.vector(rep(0, n))  # zero step vector
          c <- t  # zero 'last' gradient
          while ({keepgoing == TRUE} && {cycle < cyclimit}) {
               cycle <- cycle + 1
               parm <- bvec
               ig <- ig + 1
               post <- rbind(post, m.best[["parm"]])
               Dev <- rbind(Dev, m.best[["Dev"]])
               ### Print Status
               if(ig %% round(Iterations / 10) == 0)
                    cat("Iteration: ", ig, " of ", Iterations,
                         ",   LP: ", round(m.old[["LP"]],1), "\n")
               if(ig > Iterations) {
                    ig <- Iterations
                    LA <- list(Dev=Dev, iter=ig, parm.len=parm.len,
                         parm.new=parm, parm.old=parm.old, post=post,
                         Step.Size=steplength, tol.new=tol.new)
                    return(LA)}
               g <- partial(Model, bvec, Data) * -1
               g1 <- sum(g * (g - c))
               g2 <- sum(t * (g - c))
               gradsqr <- sum(g * g)
               c <- g
               g3 <- 1
               if(gradsqr > tol * (abs(fmin) + reltest)) {
                    if(g2 > 0) {
                         betaDY <- gradsqr / g2
                         betaHS <- g1 / g2
                         g3 <- max(0, min(betaHS, betaDY))}
                    }
               else {
                    keepgoing <- FALSE
                    tol.new <- gradsqr
                    break}
               if(g3 == 0 || cycle >= cyclimit) {
                    fdiff <- NA
                    cycle <- 0
                    break
                    }
               else {
                    t <- t * g3 - g
                    gradproj <- sum(t * g)
                    ### Line search
                    OKpoint <- FALSE
                    steplength <- oldstep * 1.5
                    f <- fmin
                    changed <- TRUE
                    while ({f >= fmin} && {changed == TRUE}) {
                         bvec <- parm + steplength * t
                         changed <- !identical((bvec + reltest),
                              (parm + reltest))
                         if(changed == TRUE) {
                              m.old <- m.new
                              m.new <- Model(bvec, Data)
                              if(any(!is.finite(c(m.new[["LP"]],
                                   m.new[["Dev"]], m.new[["Monitor"]])))) {
                                   f <- fmin + 1
                                   m.new <- m.old
                                   }
                              else {
                                   if(m.new[["LP"]] > m.best[["LP"]])
                                        m.best <- m.new
                                   f <- m.new[["LP"]] * -1
                                   tol.new <- max(sqrt(sum(g*g)),
                                        sqrt(sum({m.new[["parm"]] -
                                        m.old[["parm"]]}^2)))
                                   }
                              bvec <- m.new[["parm"]]
                              if(f < fmin) f1 <- f
                              else {
                                   savestep <-steplength
                                   steplength <- steplength * stepredn
                                   if(steplength >= savestep) changed <-FALSE}}}
                    changed1 <- changed
                    if(changed1 == TRUE) {
                         newstep <- 2 * (f - fmin - gradproj * steplength)
                         if(newstep > 0)
                              newstep <- -(gradproj * steplength *
                                   steplength / newstep)
                         bvec <- parm + newstep * t
                         changed <- !identical((bvec + reltest), 
                              (parm + reltest))
                         if(changed == TRUE) {
                              m.old <- m.new
                              m.new <- Model(bvec, Data)
                              if(any(!is.finite(c(m.new[["LP"]],
                                   m.new[["Dev"]], m.new[["Monitor"]])))) {
                                   f <- fmin + 1
                                   m.new <- m.old
                                   }
                              else {
                                   if(m.new[["LP"]] > m.best[["LP"]])
                                        m.best <- m.new
                                   f <- m.new[["LP"]] * -1
                                   tol.new <- max(sqrt(sum(g*g)),
                                        sqrt(sum({m.new[["parm"]] -
                                        m.old[["parm"]]}^2)))
                                   }
                              bvec <- m.new[["parm"]]}
                         if(f < min(fmin, f1)) {
                              OKpoint <- TRUE
                              accpoint <- (f <= fmin + gradproj *
                                   newstep * acctol)
                              fdiff <- (fmin - f)
                              fmin <- f
                              oldstep <- newstep
                              }
                         else {
                              if(f1 < fmin) {
                                   bvec <- parm + steplength * t
                                   accpoint <- (f1 <= fmin + gradproj * 
                                        steplength * acctol)
                                   OKpoint <- TRUE
                                   fdiff <- (fmin - f1)
                                   fmin <- f1
                                   oldstep <- steplength
                                   }
                              else {
                                   fdiff <- NA
                                   accpoint <- FALSE}
                              }
                         if(accpoint == FALSE) {
                              keepgoing <- FALSE
                              break}
                         }
                    else {
                         if(cycle == 1) {
                              keekpgoing <- FALSE
                              break}}}
               }
          if(oldstep < acctol) oldstep <- acctol
          if(oldstep > 1) oldstep <- 1
          }
     options(warn=0)
     Dev <- Dev[-1,]; post <- post[-1,]
     if({ig < Iterations} & {tol.new > Stop.Tolerance})
          tol.new <- Stop.Tolerance
     ### Output
     LA <- list(Dev=Dev, iter=ig, parm.len=parm.len,
          parm.new=m.best[["parm"]], parm.old=parm.old, post=post,
          Step.Size=steplength, tol.new=tol.new)
     return(LA)
     }
.ladfp <- function(Model, parm, Data, Interval, Iterations, Stop.Tolerance,
     m.old) {
     m.new <- m.old
     Dev <- matrix(m.old[["Dev"]],1,1)
     parm.old <- parm
     parm.len <- length(as.vector(parm))
     post <- matrix(m.old[["parm"]], 1, parm.len)
     tol.new <- 1
     keepgoing <- TRUE
     g.old <- g.new <- -1*partial(Model, m.old[["parm"]], Data, Interval)
     B <- Iden <- diag(parm.len) #Approximate Hessian
     options(warn=-1)
     for (iter in 2:Iterations) {
          ### Print Status
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(m.old[["LP"]],1), "\n")
          ### Gradient and Direction p
          g.old <- g.new
          p <- as.vector(tcrossprod(g.old, -B))
          p[which(!is.finite(p))] <- 0
          ### Step-size Line Search
          Step.Size <- 0.8
          changed <- TRUE
          while(m.new[["LP"]] <= m.old[["LP"]] & changed == TRUE) {
               Step.Size <- Step.Size * 0.2
               s <- Step.Size*p
               prop <- m.old[["parm"]] + s
               changed <- !identical(m.old[["parm"]], prop)
               m.new <- Model(prop, Data)
               if(any(!is.finite(c(m.new[["LP"]], m.new[["Dev"]],
                    m.new[["Monitor"]]))))
                    m.new <- m.old
               }
          g.new <- -1*partial(Model, m.new[["parm"]], Data, Interval)
          ### DFP Update to Approximate Hessian B
          if(m.new[["LP"]] > m.old[["LP"]]) {
               m.old <- m.new
               g <- g.new - g.old
               if(sum(s*g) > 0) {
                    denom <- as.vector(t(g) %*% s)
                    B <- (Iden - ((g %*% t(s)) / denom)) %*% B %*%
                         (Iden - ((s %*% t(g)) / denom)) +
                         ((g %*% t(g)) / denom)
                    if(any(!is.finite(B))) B <- diag(parm.len)}}
          ### Storage
          post <- rbind(post, m.old[["parm"]])
          Dev <- rbind(Dev, m.old[["Dev"]])
          ### Tolerance
          tol.new <- sqrt(sum(g.new*g.new))
          if(keepgoing == FALSE) tol.new <- 0
          if(tol.new <= Stop.Tolerance) break
          }
     options(warn=0)
     ### Output
     LA <- list(Dev=Dev, iter=iter, parm.len=parm.len,
          parm.new=m.old[["parm"]], parm.old=parm.old, post=post,
          Step.Size=Step.Size, tol.new=tol.new)
     return(LA)
     }
.lahar <- function(Model, parm, Data, Iterations, Stop.Tolerance, m.old)
     {
     alpha.star <- 0.234
     tau <- 1
     Dev <- matrix(m.old[["Dev"]],1,1)
     parm.len <- length(as.vector(parm))
     parm.new <- parm.old <- parm
     names(parm.new) <- names(parm.old) <- Data[["parm.names"]]
     tol.new <- 1
     post <- matrix(parm.new, 1, parm.len)
     for (iter in 1:Iterations) {
          parm.old <- parm.new
          ### Print Status
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(m.old[["LP"]],1), "\n")
          ### Propose new parameter values
          theta <- rnorm(parm.len)
          d <- theta / sqrt(sum(theta*theta))
          Step.Size <- runif(1,0,tau)
          parm.new <- parm.old + Step.Size * d
          m.new <- Model(parm.new, Data)
          tol.new <- sqrt(sum({m.new[["parm"]] - parm.old}^2))
          if(any(!is.finite(c(m.new[["LP"]], m.new[["Dev"]],
               m.new[["Monitor"]]))))
               m.new <- m.old
          ### Accept/Reject and Adapt
          if(m.new[["LP"]] > m.old[["LP"]]) {
               m.old <- m.new
               parm.new <- m.new[["parm"]]
               tau <- tau + (tau / (alpha.star *
                    (1 - alpha.star))) * (1 - alpha.star) / iter
               }
          else {
               m.new <- m.old
               parm.new <- parm.old
               tau <- abs(tau - (tau / (alpha.star *
                    (1 - alpha.star))) * alpha.star / iter)
               }
          post <- rbind(post, parm.new)
          Dev <- rbind(Dev, m.new[["Dev"]])
          if(tol.new <= Stop.Tolerance) break
          }
     Dev <- Dev[-1,]; post <- post[-1,]
     ### Output
     LA <- list(Dev=Dev, iter=iter, parm.len=parm.len, parm.new=parm.new,
          parm.old=parm.old, post=post, Step.Size=Step.Size,
          tol.new=tol.new)
     return(LA)
     }
.lahj <- function(Model, parm, Data, Interval, Iterations, Stop.Tolerance,
     m.old)
     {
     Dev <- matrix(m.old[["Dev"]],1,1)
     n <- length(parm)
     post <- matrix(parm, 1, n)
     if(n == 1)
          stop("For univariate functions use some different method.")
     tol.new <- 1
     f <- function(x, Data) {
          fun <- Model(x, Data)
          fun[["LP"]] <- fun[["LP"]] * -1
          return(fun)}
     steps  <- 2^c(-(0:(Iterations-1)))
     dir <- diag(1, n, n)
     x <- parm.old <- parm
     fx <- f(x, Data)
     fcount <- 1
     ###  Search with a single scale
     .lahjsearch <- function(xb, f, h, dir, fcount, Data)
          {
          x  <- xb
          xc <- x
          sf <- 0
          finc <- 0
          hje  <- .lahjexplore(xb, xc, f, h, dir, Data=Data)
          x    <- hje$x
          fx   <- hje$fx
          sf   <- hje$sf
          finc <- finc + hje$numf
          ### Pattern move
          while (sf == 1) {
               d  <- x - xb
               xb <- x
               xc <- x + d
               fb <- fx
               hje  <- .lahjexplore(xb, xc, f, h, dir, fb, Data)
               x    <- hje$x
               fx   <- hje$fx
               sf   <- hje$sf
               finc <- finc + hje$numf
               if(sf == 0) {  # pattern move failed
                    hje  <- .lahjexplore(xb, xb, f, h, dir, fb, Data)
                    x    <- hje$x
                    fx   <- hje$fx
                    sf   <- hje$sf
                    finc <- finc + hje$numf}
               }
          return(list(x=x, fx=fx, sf=sf, finc=finc))
          }
     ###  Exploratory move
     .lahjexplore <- function(xb, xc, f, h, dir, fbold, Data)
          {
          n <- length(xb)
          x <- xb
          if(missing(fbold)) {
               fb <- f(x, Data)
               x <- fb[["parm"]]
               numf <- 1
               }
          else {
               fb <- fbold
               numf <- 0}
          fx <- fb
          xt <- xc
          sf <- 0 # do we find a better point ?
          dirh <- h * dir
          fbold <- fx
          for (k in sample.int(n, n)) { # resample orthogonal directions
               p <- xt + dirh[, k]
               ft <- f(p, Data)
               p <- ft[["parm"]]
               numf <- numf + 1
               if(ft[["LP"]] >= fb[["LP"]]) {
                    p <- xt - dirh[, k]
                    ft <- f(p, Data)
                    p <- ft[["parm"]]
                    numf <- numf + 1
                    }
               else {
                    sf <- 1
                    xt <- p
                    fb <- ft}
               }
          if(sf == 1) {
               x <- xt
               fx <- fb}
          return(list(x=x, fx=fx, sf=sf, numf=numf))
          }
     ### Start the main loop
     for (iter in 1:Iterations) {
          ### Print Status
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(fx[["LP"]],1), "\n")
          hjs <- .lahjsearch(x, f, steps[iter], dir, fcount, Data)
          x <- hjs$x
          fx <- hjs$fx
          sf <- hjs$sf
          fcount <- fcount + hjs$finc
          post <- rbind(post, x)
          Dev <- rbind(Dev, fx[["Dev"]])
          tol.new <- sqrt(sum({fx[["parm"]] - parm.old}^2))
          if(tol.new <= Stop.Tolerance) break
          parm.old <- x
          }
     Dev <- Dev[-1,]; post <- post[-1,]
     LA <- list(Dev=Dev, iter=iter, parm.len=n, parm.new=x,
          parm.old=parm, post=post, Step.Size=steps[iter],
          tol.new=tol.new)
     return(LA)
     }
.lalbfgs <- function(Model, parm, Data, Iterations, Stop.Tolerance, m.old)
     {
     Dev <- matrix(m.old[["Dev"]],1,1)
     parm.len <- length(as.vector(parm))
     parm.new <- parm.old <- m.old[["parm"]]
     names(parm.new) <- names(parm.old) <- Data[["parm.names"]]
     tol.new <- 1
     post <- matrix(parm.new, 1, parm.len)
     ModelWrapper <- function(parm.new) {
          out <- Model(parm.new, Data)[["LP"]]
          return(out)
          }
     for (iter in 1:Iterations) {
          parm.old <- parm.new
          ### Print Status
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(m.old[["LP"]],1), "\n")
          ### LBFGS
          Fit <- optim(par=parm.new, fn=ModelWrapper,
               method="L-BFGS-B", control=list(fnscale=-1, maxit=1))
          m.new <- Model(Fit$par, Data)
          tol.new <- sqrt(sum({m.new[["parm"]] - parm.old}^2))
          if(any(!is.finite(c(m.new[["LP"]], m.new[["Dev"]],
               m.new[["Monitor"]]))) | {m.new[["LP"]] < m.old[["LP"]]})
               m.new <- m.old
          m.old <- m.new
          parm.new <- m.new[["parm"]]
          post <- rbind(post, parm.new)
          Dev <- rbind(Dev, m.new[["Dev"]])
          if(tol.new <= Stop.Tolerance) break
          }
     Dev <- Dev[-1,]; post <- post[-1,]
     ### Output
     LA <- list(Dev=Dev, iter=iter, parm.len=parm.len, parm.new=parm.new,
          parm.old=parm.old, post=post, Step.Size=0,
          tol.new=tol.new)
     return(LA)
     }
.lalm <- function(Model, parm, Data, Iterations, Stop.Tolerance, m.old)
     {
     Norm <- function(x, p=2) {
          stopifnot(is.numeric(x) || is.complex(x), is.numeric(p),
               length(p) == 1)
          if(p > -Inf && p < Inf) return(sum(abs(x)^p)^(1/p))
          else if(p ==  Inf) return(max(abs(x)))
          else if(p == -Inf) return(min(abs(x)))
          else return(NULL)
          }
     Dev <- matrix(m.old[["Dev"]],1,1)
     tau <- 1e-3 ### Damping constant
     tolg <- 1e-8
     n <- length(parm)
     m <- 1
     x <- xnew <- parm
     mod <- modbest <- m.old
     r <- r.adj <- mod[["LP"]]
     if(r.adj > 0) r.adj <- r.adj * -1
     dold <- dnew <- mod[["Dev"]]
     x <- parm <- mod[["parm"]]
     post <- matrix(parm, 1, n)
     f <- 0.5 * r * r
     J <- rbind(partial(Model, x, Data))
     g <- t(J) %*% r.adj
     ng <- max(abs(g))
     A <- t(J) %*% J
     mu <- tau * max(diag(A)) ### Damping parameter
     nu <- 2
     nh <- 0
     iter <- 1
     while (iter < Iterations) {
          ### Print Status
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(mod[["LP"]],1), "\n")
          iter <- iter + 1
          R <- chol(A + mu*diag(n))
          h <- c(-t(g) %*% chol2inv(R))
          bad <- !is.finite(h)
          h[bad] <- 1e-10 * J[1,bad] #Small gradient if ill-conditioned
          nh <- Norm(h)
          xnew <- x + h
          h <- xnew - x
          dL <- sum(h*(mu*h - g)) / 2
          mod <- Model(xnew, Data)
          if(all(is.finite(c(mod[["LP"]], mod[["Dev"]],
               mod[["Monitor"]])))) {
               if(mod[["LP"]] > modbest[["LP"]]) modbest <- mod
               rn <- mod[["LP"]]
               xnew <- mod[["parm"]]
               }
          else {
               rn <- r
               xnew <- x}
          fn <- 0.5 * rn * rn
          Jn <- rbind(partial(Model, xnew, Data))
          df <- f - fn
          if(rn > 0) df <- fn - f
          if(dL > 0 && df > 0) {
               tol.new <- sqrt(sum({xnew - x}^2))
               x <- xnew
               f <- fn
               J <- Jn
               r <- r.adj <- rn
               if(r.adj > 0) r.adj <- r.adj * -1
               A <- t(J) %*% J
               g <- t(J) %*% r.adj
               ng <- Norm(g, Inf)
               mu <- mu * max(1/3, 1 - (2*df/dL - 1)^3)
               nu <- 2}
          else {mu <- mu*nu
                nu <- 2*nu}
          post <- rbind(post, modbest[["parm"]])
          Dev <- rbind(Dev, modbest[["Dev"]])
          if(ng <= Stop.Tolerance) {
               tol.new <- ng
               break
               }
          else if(nh <= Stop.Tolerance) {
               tol.new <- nh
               break}
          }
     ### Output
     LA <- list(Dev=Dev, iter=iter, parm.len=n, parm.new=modbest[["parm"]],
          parm.old=parm, post=post, Step.Size=nh, tol.new=tol.new)
     return(LA)
     }
.lanm <- function(Model, parm, Data, Iterations, Stop.Tolerance, m.old)
     {
     Dev <- matrix(m.old[["Dev"]],1,1)
     n <- length(as.vector(parm))
     parm.old <- m.old[["parm"]]
     Step.Size <- 1
     if(!is.numeric(parm) || length(parm) < 2)
          stop("Parameters must be a numeric vector of length > 1.")
     # simplex vertices around parm
     V <- t(1/(2*n) * cbind(diag(n), rep(-1, n)) + parm)
     P <- Q <- c()
     # Function values at vertices
     Y <- numeric(n+1)
     for (j in 1:(n+1)) {
          Mo <- Model(V[j, ], Data)
          Y[j] <- Mo[["LP"]] * -1
          V[j,] <- Mo[["parm"]]}
     ho <- lo <- which.min(Y)
     li <- hi <- which.max(Y)
     for (j in 1:(n+1)) {
          if(j != lo && j != hi && Y[j] <= Y[li]) li <- j
          if(j != hi && j != lo && Y[j] >= Y[ho]) ho <- j}
     iter <- 0
     while(Y[hi] > Y[lo] + Stop.Tolerance && iter < Iterations) {
          S <- numeric(n)
          for (j in 1:(n+1)) S <- S + V[j,1:n]
          M <- (S - V[hi,1:n])/n
          R <- 2*M - V[hi,1:n]
          Mo <- Model(R, Data)
          yR <- Mo[["LP"]] * -1
          R <- Mo[["parm"]]
          if(yR < Y[ho]) {
               if(Y[li] < yR) {
                    V[hi,1:n] <- R
                    Y[hi] <- yR
                    }
               else {
                    E <- 2*R - M
                    Mo <- Model(E, Data)
                    yE <- Mo[["LP"]] * -1
                    E <- Mo[["parm"]]
                    if(yE < Y[li]) {
                         V[hi,1:n] <- E
                         Y[hi] <- yE
                         }
                    else {
                         V[hi,1:n] <- R
                         Y[hi] <- yR
                         }
                    }
               }
          else {
               if(yR < Y[hi]) {
                    V[hi,1:n] <- R
                    Y[hi] <- yR}
               C <- (V[hi,1:n] + M) / 2
               Mo <- Model(C, Data)
               yC <- Mo[["LP"]] * -1
               C <- Mo[["parm"]]
               C2 <- (M + R) / 2
               Mo <- Model(C2, Data)
               yC2 <- Mo[["LP"]] * -1
               C2 <- Mo[["parm"]]
               if(yC2 < yC) {
                    C <- C2
                    yC <- yC2}
               if(yC < Y[hi]) {
                    V[hi,1:n] <- C
                    Y[hi] <- yC
                    }
               else {
                    for (j in 1:(n+1)) {
                         if(j != lo) {
                              V[j,1:n] <- (V[j,1:n] + V[lo,1:n]) / 2
                              Z <- V[j,1:n]
                              Mo <- Model(Z, Data)
                              Y[j] <- Mo[["LP"]] * -1
                              Z <- Mo[["parm"]]
                              V[j,1:n] <- Z}}}}
          ho <- lo <- which.min(Y)
          li <- hi <- which.max(Y)
          for (j in 1:(n+1)) {
               if(j != lo && j != hi && Y[j] <= Y[li]) li <- j
               if(j != hi && j != lo && Y[j] >= Y[ho]) ho <- j}
          iter <- iter + 1
          if(iter %% round(Iterations / 10) == 0) {
               cat("Iteration: ", iter, " of ", Iterations, "\n")}
          P <- rbind(P, V[lo, ])
          Q <- c(Q, Y[lo])
          Dev <- rbind(Dev, Model(V[lo,], Data)[["Dev"]])
          }
     snorm <- 0
     for (j in 1:(n+1)) {
          s <- abs(V[j] - V[lo])
          if(s >= snorm) snorm <- s}
     V0 <- V[lo, 1:n]
     y0 <- Y[lo]
     dV <- snorm
     dy <- abs(Y[hi] - Y[lo])
     Dev <- Dev[-1,]
     ### Output
     LA <- list(Dev=Dev, iter=iter, parm.len=n, parm.new=V0,
          parm.old=parm.old, post=P, Step.Size=Step.Size,
          tol.new=Y[hi] - Y[lo])
     return(LA)
     }
.lanr <- function(Model, parm, Data, Interval, Iterations=100, Stop.Tolerance,
     m.old)
     {
     m.new <- m.old
     Dev <- matrix(m.old[["Dev"]],1,1)
     Step.Size <- 1 / length(parm)
     parm.old <- parm
     parm.len <- length(parm)
     converged <- FALSE
     post <- matrix(parm, 1, parm.len)
     for (iter in 1:Iterations) {
          ### Print Status
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(m.old[["LP"]],1), "\n")
          check1 <- TRUE; check2 <- FALSE
          m.old <- Model(parm, Data)
          p <- partial(Model, m.old[["parm"]], Data, Interval)
          H <- Hessian(Model, m.old[["parm"]], Data)
          if(all(is.finite(H))) {
               Sigma <- -as.inverse(H)
               delta <- as.vector(tcrossprod(p, Sigma))
               }
          else check1 <- FALSE
          if(check1 == TRUE) {
               if(any(!is.finite(delta))) check1 <- FALSE
               if(check1 == TRUE) {
                    temp1 <- temp2 <- temp3 <- parm
                    Step.Size1 <- Step.Size / 2
                    Step.Size3 <- Step.Size * 2
                    temp1 <- temp1 + Step.Size1 * delta
                    temp2 <- temp2 + Step.Size * delta
                    temp3 <- temp3 + Step.Size3 * delta
                    Mo1 <- Model(temp1, Data)
                    Mo2 <- Model(temp2, Data)
                    Mo3 <- Model(temp3, Data)
                    check2 <- FALSE
                    if({Mo1[["LP"]] == max(Mo1[["LP"]], Mo2[["LP"]],
                         Mo3[["LP"]])} & Mo1[["LP"]] > m.old[["LP"]]) {
                         Step.Size <- Step.Size1
                         parm <- parm + Step.Size * delta
                         m.old <- m.new <- Mo1
                         }
                    else if({Mo2[["LP"]] == max(Mo1[["LP"]], Mo2[["LP"]],
                         Mo3[["LP"]])} & Mo2[["LP"]] > m.old[["LP"]]) {
                         parm <- parm + Step.Size * delta
                         m.old <- m.new <- Mo2
                         }
                    else if({Mo3[["LP"]] == max(Mo1[["LP"]], Mo2[["LP"]],
                         Mo3[["LP"]])} & Mo3[["LP"]] > m.old[["LP"]]) {
                         Step.Size <- Step.Size3
                         parm <- parm + Step.Size * delta
                         m.old <- m.new <- Mo3
                         }
                    else check2 <- TRUE}}
          if({check1 == FALSE} | {check2 == TRUE}) {
               delta <- p
               temp1 <- temp2 <- temp3 <- parm
               Step.Size1 <- Step.Size / 2
               Step.Size3 <- Step.Size * 2
               temp1 <- temp1 + Step.Size1 * delta
               temp2 <- temp2 + Step.Size * delta
               temp3 <- temp3 + Step.Size3 * delta
               Mo1 <- Model(temp1, Data)
               Mo2 <- Model(temp2, Data)
               Mo3 <- Model(temp3, Data)
               if({Mo1[["LP"]] == max(Mo1[["LP"]], Mo2[["LP"]],
                    Mo3[["LP"]])} & Mo1[["LP"]] > m.old[["LP"]]) {
                    Step.Size <- Step.Size1
                    parm <- parm + Step.Size * delta
                    m.old <- m.new <- Mo1
                    }
               else if({Mo2[["LP"]] == max(Mo1[["LP"]], Mo2[["LP"]],
                    Mo3[["LP"]])} & Mo2[["LP"]] > m.old[["LP"]]) {
                    parm <- parm + Step.Size * delta
                    f <- Mo2
                    }
               else if({Mo3[["LP"]] == max(Mo1[["LP"]], Mo2[["LP"]],
                    Mo3[["LP"]])} & Mo3[["LP"]] > m.old[["LP"]]) {
                    Step.Size <- Step.Size3
                    parm <- parm + Step.Size * delta
                    m.old <- m.new <- Mo3
                    }
               else { #Jitter in case of failure
                    Step.Size <- Step.Size / 2
                    parm <- parm + Step.Size * rnorm(length(parm))}}
          post <- rbind(post, parm)
          Dev <- rbind(Dev, m.new[["Dev"]])
          Step.Size <- max(Step.Size, .Machine$double.eps)
          tol.new <- sqrt(sum(delta^2))
          if(tol.new < Stop.Tolerance) {converged <- TRUE; break}
          }
     Dev <- Dev[-1,]; post <- post[-1,]
     LA <- list(Dev=Dev, H=H, iter=iter, parm.len=parm.len, parm.new=parm,
          parm.old=parm.old, post=post, Step.Size=Step.Size,
          tol.new=tol.new)
     return(LA)
     }
.lapso <- function(Model, parm, Data, Iterations, Stop.Tolerance, m.old)
     {
     Dev <- matrix(m.old[["Dev"]],1,1)
     LP <- NA
     parm.len <- length(parm)
     parm.new <- parm.old <- parm
     tol.new <- 1
     post <- matrix(parm.new, 1, parm.len)
     p.s <- floor(10 + 2*sqrt(parm.len)) ## Swarm size
     k <- 3 ### Exponent for calculating number of informants
     p.p <- 1-(1-1/p.s)^k ### Average % of informants
     p.w0 <- p.w1 <- 1 / (2*log(2)) ### Exploitation constants
     p.c.p <- 0.5 + log(2) ### Local exploration constant
     p.c.g <- 0.5 + log(2) ### Global exploration constant
     p.randorder <- TRUE ### Randomize Particle Order
     X <- V <- matrix(parm, parm.len, p.s)
     if(!is.null(Data[["PGF"]])) {
          for (i in 2:ncol(X)) X[,i] <- GIV(Model, Data, PGF=TRUE)
          for (i in 1:ncol(V)) V[,i] <- GIV(Model, Data, PGF=TRUE)
          }
     else {
          for (i in 2:ncol(X)) X[,i] <- GIV(Model, Data, PGF=FALSE)
          for (i in 1:ncol(V)) V[,i] <- GIV(Model, Data, PGF=FALSE)}
     V <- (V - X) / 2 ### Velocity
     mods <- apply(X, 2, function(x) Model(x, Data))
     f.x <- sapply(mods, with, LP) * -1
     f.d <- sapply(mods, with, Dev)
     X <- matrix(sapply(mods, with, parm), nrow(X), ncol(X))
     P <- X
     f.p <- f.x
     P.improved <- rep(FALSE, p.s)
     i.best <- which.min(f.p)
     error <- f.p[i.best]
     init.links <- TRUE
     iter <- 1
     while (iter < Iterations && tol.new > Stop.Tolerance) {
          iter <- iter + 1
          ### Print Status
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(mod[["LP"]],1), "\n")
          if(p.p != 1 && init.links == TRUE) {
               links <- matrix(runif(p.s*p.s, 0, 1) <= p.p, p.s, p.s)
               diag(links) <- TRUE}
          if(p.randorder == TRUE) index <- sample(p.s)
          else index <- 1:p.s
          for (i in index) {
               if(p.p == 1) j <- i.best
               else j <- which(links[,i])[which.min(f.p[links[,i]])]
               temp <- p.w0 + (p.w1 - p.w0)*(iter / Iterations)
               V[,i] <- temp*V[,i]
               V[,i] <- V[,i] +
                    runif(parm.len, 0, p.c.p)*(P[,i] - X[,i])
               if(i != j)
                    V[,i] <- V[,i] +
                         runif(parm.len, 0, p.c.g)*(P[,j] - X[,i])
               X[,i] <- X[,i] + V[,i]
               mod <- Model(X[,i], Data)
               f.x[i] <- mod[["LP"]] * -1
               f.d[i] <- mod[["Dev"]]
               X[,i] <- mod[["parm"]]
               if(f.x[i] < f.p[i]) { ### Improvement
                    P[,i] <- X[,i]
                    f.p[i] <- f.x[i]
                    if(f.p[i] < f.p[i.best]) i.best <- i}
               }
          post <- rbind(post, P[,i.best])
          Dev <- rbind(Dev, f.d[i.best])
          tol.new <- mean(abs(V[,i.best]))
          init.links <- f.p[i.best] == error
          error <- f.p[i.best]
          }
     ### Output
     LA <- list(Dev=Dev, iter=iter, parm.len=parm.len, parm.new=P[,i.best],
          parm.old=parm.old, post=post, Step.Size=mean(abs(V[,i.best])),
          tol.new=tol.new)
     return(LA)
     }
.larprop <- function(Model, parm, Data, Interval, Iterations,
     Stop.Tolerance, m.old)
     {
     Dev <- matrix(m.old[["Dev"]],1,1)
     parm.len <- length(as.vector(parm))
     approx.grad.old <- approx.grad.new <- rep(0, parm.len)
     parm.old <- m.old[["parm"]]
     parm.new <- parm.old - 0.1 #First step
     names(parm.new) <- names(parm.old) <- Data[["parm.names"]]
     tol.new <- 1
     post <- matrix(parm.new, 1, parm.len)
     Step.Size <- rep(0.0125, parm.len)
     for (iter in 1:Iterations) {
          approx.grad.old <- approx.grad.new
          parm.old <- parm.new
          ### Print Status
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(m.old[["LP"]],1), "\n")
          ### Approximate Truncated Gradient
          approx.grad.new <- partial(Model, parm.old, Data, Interval)
          approx.grad.new <- interval(approx.grad.new, -1000, 1000,
               reflect=FALSE)
          ### Determine if Gradients Changed Sign
          change <- (approx.grad.old >= 0) == (approx.grad.new >= 0)
          ### Adjust weight (step size) based on sign change
          Step.Size[change == TRUE] <- Step.Size[change == TRUE] * 0.5
          Step.Size[change == FALSE] <- Step.Size[change == FALSE] * 1.2
          Step.Size <- interval(Step.Size, 0.0001, 50, reflect=FALSE)
          ### Propose new state based on weighted approximate gradient
          parm.new <- parm.old + Step.Size * approx.grad.new
          m.new <- Model(parm.new, Data)
          tol.new <- max(sqrt(sum(approx.grad.new^2)),
               sqrt(sum({m.new[["parm"]] - parm.old}^2)))
          if(any(!is.finite(c(m.new[["LP"]], m.new[["Dev"]],
               m.new[["Monitor"]]))) | {m.new[["LP"]] < m.old[["LP"]]}) {
               p.order <- sample(1:length(parm.new))
               parm.temp <- parm.old
               for (i in 1:length(p.order)) {
                    parm.temp[p.order[i]] <- parm.new[p.order[i]]
                    m.new <- Model(parm.temp, Data)
                    if(m.new[["LP"]] < m.old[["LP"]])
                         parm.temp[p.order[i]] <- parm.old[p.order[i]]}
               m.new <- Model(parm.temp, Data)}
          parm.new <- m.new[["parm"]]
          post <- rbind(post, parm.new)
          Dev <- rbind(Dev, m.new[["Dev"]])
          if(tol.new <= Stop.Tolerance) break
          }
     Dev <- Dev[-1,]; post <- post[-1,]
     ### Output
     LA <- list(Dev=Dev, iter=iter, parm.len=parm.len, parm.new=parm.new,
          parm.old=parm.old, post=post, Step.Size=Step.Size,
          tol.new=tol.new)
     return(LA)
     }
.lasgd <- function(Model, parm, Data, Iterations, Stop.Tolerance, m.old)
     {
     Dev <- matrix(m.old[["Dev"]],1,1)
     m.new <- m.old
     parm.len <- length(as.vector(parm))
     parm.new <- parm.old <- parm
     names(parm.new) <- names(parm.old) <- Data[["parm.names"]]
     tol.new <- 1
     post <- matrix(parm.new, 1, parm.len)
     #Read SGD specifications
     if(is.null(Data[["file"]]))
          stop("SGD requires Data$file, which is missing.")
     if(is.null(Data[["Nr"]])) stop("SGD requires Data$Nr, which is missing.")
     if(is.null(Data[["Nc"]])) stop("SGD requires Data$Nc, which is missing.")
     if(is.null(Data[["size"]]))
          stop("SGD requires Data$size, which is missing.")
     if(is.null(Data[["epsilon"]]))
          stop("SGD requires Data$epsilon, which is missing.")
     con <- file(Data[["file"]], open="r")
     on.exit(close(con))
     for (iter in 1:Iterations) {
          parm.old <- parm.new
          ### Print Status
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(m.old[["LP"]],1), "\n")
          ### Sample Data
          seek(con, 0)
          skip.rows <- sample.int(Data[["Nr"]] - Data[["size"]], size=1)
          Data[["X"]] <- matrix(scan(file=con, sep=",", skip=skip.rows,
               nlines=Data[["size"]], quiet=TRUE), Data[["size"]],
               Data[["Nc"]], byrow=TRUE)
          ### Propose new parameter values
          g <- partial(Model, m.old[["parm"]], Data)
          parm.new <- m.new[["parm"]] + {Data[["epsilon"]] / 2} * g
          m.new <- Model(parm.new, Data)
          tol.new <- max(sqrt(sum(g^2)),
               sqrt(sum({m.new[["parm"]] - parm.old}^2)))
          if(any(!is.finite(c(m.new[["LP"]], m.new[["Dev"]],
               m.new[["Monitor"]]))))
               m.new <- m.old
          post <- rbind(post, parm.new)
          Dev <- rbind(Dev, m.new[["Dev"]])
          if(tol.new <= Stop.Tolerance) break
          }
     Dev <- Dev[-1,]; post <- post[-1,]
     ### Output
     LA <- list(Dev=Dev, iter=iter, parm.len=parm.len, parm.new=parm.new,
          parm.old=parm.old, post=post, Step.Size=Data[["epsilon"]],
          tol.new=tol.new)
     return(LA)
     }
.lasoma <- function(Model, parm, Data, bounds, options=list())
     {
     ### Initial Checks
     if(missing(bounds)) {
          bounds <- list(min=rep(-.Machine$double.xmax,
               length(parm)),
               max=rep(.Machine$double.xmax, length(parm)))
          }
     if(!(all(c("min","max") %in% names(bounds))))
          stop("Bounds list must contain \"min\" and \"max\" vector elements")
     if(length(bounds$min) != length(bounds$max))
          stop("Bounds are not of equal length.")
     ### Option Defaults
     defaultOptions <- list(pathLength=3, stepLength=0.11,
          perturbationChance=0.1, tol=1e-3, maxiter=1000,
          populationSize=10)
     defaultsNeeded <- setdiff(names(defaultOptions), names(options))
     spuriousOptions <- setdiff(names(options), names(defaultOptions))
     options[defaultsNeeded] <- defaultOptions[defaultsNeeded]
     if(length(spuriousOptions) > 0)
           warning("The following specified options are unused: ",
                paste(spuriousOptions,collapse=", "))
     ### Setup
     nParams <- length(bounds$min)
     nParamsTotal <- nParams * options$populationSize
     steps <- seq(0, options$pathLength, options$stepLength)
     nSteps <- length(steps)
     steps <- rep(steps, each=nParamsTotal)
     post <- matrix(parm, 1, nParams)
     ### Create Population
     population <- matrix(parm, nrow=nParams, ncol=options$populationSize)
     for (i in 2:ncol(population)) {
          if(!is.null(Data[["PGF"]]))
               population[,i] <- GIV(Model, Data, PGF=TRUE)
          else population[,i] <- GIV(Model, Data)}
     ### Calculate initial LP and Dev per individual in population
     temp <- apply(population, 2, function(x) Model(x, Data))
     population <- sapply(temp, function(l) l$parm)
     LP <- sapply(temp, function(l) l$LP)
     Dev <- sapply(temp, function(l) l$Dev)
     iteration <- 0
     DevHistory <- LPHistory <- numeric(0)
     if((max(LP) - min(LP)) < options$tol) {
          population <- matrix(runif(nParamsTotal,-10,10), nrow=nParams,
               ncol=options$populationSize)}
     if(all(!is.finite(LP))) stop("All individuals have non-finite LPs.")
     ### Evolution Begins
     repeat {
          ### Find the current leader
          leaderIndex <- which.max(LP)
          leaderDev <- Dev[leaderIndex]
          leaderLP <- LP[leaderIndex]
          ### Check termination criteria
          if(iteration == options$maxiter) break
          LPdiff <- max(LP) - min(LP)
          if(!is.finite(LPdiff)) LPdiff <- 1
          if(LPdiff < options$tol) break
          DevHistory <- c(DevHistory, leaderDev)
          LPHistory <- c(LPHistory, leaderLP)
          ### Find the migration direction for each individual
          directionsFromLeader <- apply(population, 2, "-",
               population[,leaderIndex])
          ### Establish which parameters will be changed
          toPerturb <- runif(nParamsTotal) < options$perturbationChance
          # Second line here has a minus, so directions are away from leader
          populationSteps <- array(rep(population, nSteps),
               dim=c(nParams, options$populationSize, nSteps))
          populationSteps <- populationSteps -
               steps * rep(directionsFromLeader * toPerturb, nSteps)
          ### Replace out-of-bounds parameters with random valid values
          outOfBounds <- which(populationSteps < bounds$min |
               populationSteps > bounds$max)
          randomSteps <- array(runif(nParamsTotal*nSteps),
               dim=c(nParams, options$populationSize, nSteps))
          randomSteps <- randomSteps * (bounds$max-bounds$min) + bounds$min
          populationSteps[outOfBounds] <- randomSteps[outOfBounds]
          ### Values over potential locations
          temp <- apply(populationSteps, 2:3, function(x) Model(x, Data))
          population <- sapply(temp, function(l) l$parm)
          LP <- sapply(temp, function(l) l$LP)
          LP <- matrix(LP, length(LP) / nSteps, nSteps)
          Dev <- sapply(temp, function(l) l$Dev)
          Dev <- matrix(Dev, length(Dev) / nSteps, nSteps)
          individualBestLocs <- apply(LP, 1, which.max)
          ### Migrate each individual to its best new location, and update LP
          indexingMatrix <- cbind(seq_len(options$populationSize),
               individualBestLocs)
          population <- t(apply(populationSteps, 1, "[", indexingMatrix))
          LP <- LP[indexingMatrix]
          Dev <- Dev[indexingMatrix]
          post <- rbind(post, population[,leaderIndex])
          iteration <- iteration + 1
          if(iteration %% round(options$maxiter / 10) == 0)
               cat("Iteration: ", iteration, " of ", options$maxiter, "\n")
          }
    ### Output
    LA <- list(Dev=DevHistory, 
         iter=iteration,
         parm.len=nParams,
         parm.new=population[,leaderIndex],
         parm.old=parm,
         post=post[-1,],
         Step.Size=options$stepLength,
         tol.new=signif(LPdiff,3))
    return(LA)
    }
.laspg <- function(Model, parm, Data, Interval, Iterations, Stop.Tolerance,
     m.old)
     {
     parm.old <- parm
     parm.len <- length(parm)
     Dev <- matrix(m.old[["Dev"]],1,1)
     post <- matrix(parm, 1, parm.len)
     M	   <- 10
     ftol     <- 1e-10
     maxfeval <- 10000
     quiet <- FALSE
     feval <- 1
     f0 <- fbest <- f <- m.old[["LP"]] * -1
     nmls <- function(p, f, d, gtd, lastfv, feval, Model, maxfeval, Data)
          {
          # Non-monotone line search of Grippo with safe-guarded quadratic
          # interpolation
          gamma <- 1.e-04
          fmax <- max(lastfv)
          alpha <- 1
          pnew <- p + alpha*d
          m.new <- try(Model(pnew, Data), silent=TRUE)
          feval <- feval + 1
          if(inherits(m.new, "try-error") | !is.finite(m.new[["LP"]]))
               return(list(p=NA, f=NA, feval=NA, lsflag=1))
          fnew <- m.new[["LP"]] * -1
          pnew <- m.new[["parm"]]
          while(fnew > fmax + gamma*alpha*gtd) {
    	       if(alpha <= 0.1) alpha <- alpha/2
    	       else {
    	            atemp <- -(gtd*alpha^2) / (2*(fnew - f - alpha*gtd))
    	            if(atemp < 0.1 | atemp > 0.9*alpha) atemp <- alpha/2
    	            alpha <- atemp
    	            }
               pnew <- p + alpha*d
               m.new <- try(Model(pnew, Data), silent=TRUE)
               feval <- feval + 1
               if(inherits(m.new, "try-error") | !is.finite(m.new[["LP"]]))
                    return(list(p=NA, f=NA, feval=NA, lsflag=1))
               fnew <- m.new[["LP"]] * -1
               pnew <- m.new[["parm"]]
               if(feval > maxfeval)
                    return(list(p=NA, f=NA, feval=NA, lsflag=2))
               }
          return(list(p=pnew, f=fnew, feval=feval, m.new=m.new, lsflag=0))
          }
     ### Initialization
     lmin <- 1e-30
     lmax <- 1e30
     iter <-  geval <- 0
     lastfv <- rep(-1e99, M)
     fbest <- NA
     fchg <- Inf
     if(any(!is.finite(parm))) stop("Failure in initial guess!")
     pbest <- parm
     g <- partial(Model, parm, Data, Interval) * -1
     geval <- geval + 1
     lastfv[1] <- fbest <- f
     pg <- parm - g
     if(any(is.nan(pg))) stop("Failure in initial projection!")
     pg <- pg - parm
     pg2n <- sqrt(sum(pg*pg))
     pginfn <- max(abs(pg))
     gbest <- pg2n
     if(pginfn != 0) lambda <- min(lmax, max(lmin, 1/pginfn))
     ###  Main iterative loop
     lsflag <- NULL
     while ({iter <= Iterations} &
          ({pginfn > Stop.Tolerance} | {fchg > ftol})) {
          iter <- iter + 1
          d <- parm - lambda * g
          d <- d - parm
          gtd <- sum(g * d)
          if(is.infinite(gtd)) {
               lsflag <- 4
               break}
          nmls.ans <- nmls(parm, f, d, gtd, lastfv, feval , Model,
               maxfeval, Data)
          lsflag <- nmls.ans$lsflag
          if(lsflag != 0) break
          fchg <- abs(f - nmls.ans$f)
          f     <- nmls.ans$f
          pnew  <- nmls.ans$p
          feval <- nmls.ans$feval
          m.new <- nmls.ans$m.new
          lastfv[(iter %% M) + 1] <- f
          gnew <- try(partial(Model, pnew, Data, Interval) * -1,
               silent=TRUE)
          geval <- geval + 1
          if(inherits(gnew, "try-error") | any(is.nan(gnew))) {
               lsflag <- 3
               break}
          s <- pnew - parm
          y <- gnew - g
          sts <- sum(s*s)
          yty <- sum(y*y)
          sty <- sum(s*y)
          if(sts == 0 | yty == 0) lambda <- lmax
          else lambda <- min(lmax, max(lmin, sqrt(sts/yty)))
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(m.new[["LP"]],1), "\n")
          post <- rbind(post, pnew)
          Dev <- rbind(Dev, m.new[["Dev"]])
          parm <- pnew
          g   <- gnew
          pg <- parm - g - parm
          pg2n <- sqrt(sum(pg*pg))
          pginfn <- max(abs(pg))
          f.rep <- f * -1
          if(f < fbest) {
               fbest <- f
               pbest <- pnew
               gbest <- pginfn}
          }
     ### Output
     if(iter < Iterations) pginfn <- max(fchg, pginfn)
     if(is.null(lsflag)) lsflag <- 0
     if(lsflag == 0) parm <- pbest
     else {
          parm <- pbest
          pginfn <- gbest
          }
     m.new <- Model(parm, Data)
     Dev[nrow(Dev),] <- m.new[["Dev"]]
     post[nrow(post),] <- m.new[["parm"]]
     Dev <- Dev[-c(1:2),]; post <- post[-c(1:2),]
     LA <- list(Dev=Dev, iter=iter, parm.len=parm.len, parm.new=parm,
          parm.old=parm.old, post=post, Step.Size=1,
          tol.new=pginfn)
     return(LA)
     }
.lasr1 <- function(Model, parm, Data, Interval, Iterations, Stop.Tolerance,
     m.old) {
     m.new <- m.old
     Dev <- matrix(m.old[["Dev"]],1,1)
     parm.old <- parm
     parm.len <- length(as.vector(parm))
     post <- matrix(m.old[["parm"]], 1, parm.len)
     tol.new <- 1
     keepgoing <- TRUE
     g.old <- g.new <- -1*partial(Model, m.old[["parm"]], Data, Interval)
     B <- diag(parm.len) #Approximate Hessian
     options(warn=-1)
     for (iter in 2:Iterations) {
          ### Print Status
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(m.old[["LP"]],1), "\n")
          ### Gradient and Direction p
          g.old <- g.new
          p <- as.vector(tcrossprod(g.old, -as.inverse(B)))
          p[which(!is.finite(p))] <- 0
          ### Step-size Line Search
          Step.Size <- 0.8
          changed <- TRUE
          while(m.new[["LP"]] <= m.old[["LP"]] & changed == TRUE) {
               Step.Size <- Step.Size * 0.2
               s <- Step.Size*p
               prop <- m.old[["parm"]] + s
               changed <- !identical(m.old[["parm"]], prop)
               m.new <- Model(prop, Data)
               if(any(!is.finite(c(m.new[["LP"]], m.new[["Dev"]],
                    m.new[["Monitor"]]))))
                    m.new <- m.old
               }
          g.new <- -1*partial(Model, m.old[["parm"]], Data, Interval)
          ### SR1 Update to B, the Secant Approximation of the Hessian
          if(m.new[["LP"]] > m.old[["LP"]]) {
               m.old <- m.new
               g <- g.new - g.old
               if(sum(s*g) > 0) {
                    part1 <- g - B %*% s
                    if(abs(t(s) %*% part1) >=
                         1e-8*sqrt(sum(s*s))*sqrt(sum(part1*part1)))
                         B <- B + (part1 %*% t(part1)) /
                              as.vector(t(part1) %*% s)
                    if(any(!is.finite(B))) B <- diag(parm.len)}
               else B <- diag(parm.len)}
          ### Storage
          post <- rbind(post, m.old[["parm"]])
          Dev <- rbind(Dev, m.old[["Dev"]])
          ### Tolerance
          tol.new <- sqrt(sum(g.new*g.new))
          if(keepgoing == FALSE) tol.new <- 0
          if(tol.new <= Stop.Tolerance) break
          }
     options(warn=0)
     ### Output
     LA <- list(Dev=Dev, iter=iter, parm.len=parm.len,
          parm.new=m.old[["parm"]], parm.old=parm.old, post=post,
          Step.Size=Step.Size, tol.new=tol.new)
     return(LA)
     }
.latr <- function(Model, parm, Data, Iterations, m.old)
     {
     fterm <- sqrt(.Machine$double.eps)
     mterm <- sqrt(.Machine$double.eps)
     Norm <- function(x) return(sqrt(sum(x^2)))
     parm.len <- length(parm)
     post <- matrix(parm, 1, parm.len)
     Dev <- matrix(m.old[["Dev"]],1,1)
     r <- rinit <- 1
     rmax <- 1e10
     theta <- parm.old <- parm
     Mo <- m.old
     LP <- Mo[["LP"]]
     theta <- Mo[["parm"]]
     grad <- partial(Model, theta, Data)
     H <- Hessian(Model, theta, Data)
     accept <- TRUE
     for (iter in 1:Iterations) {
          ### Print Status
          if(iter %% round(Iterations / 10) == 0)
               cat("Iteration: ", iter, " of ", Iterations,
                    ",   LP: ", round(Mo[["LP"]],1), "\n")
          if(accept == TRUE) {
               B <- H
               g <- grad
               f <- LP
               out.value.save <- f
                    B <- - B
                    g <- - g
                    f <- - f
               eout <- eigen(B, symmetric=TRUE)
               gq <- as.numeric(t(eout$vectors) %*% g)}
          is.newton <- FALSE
          if(all(eout$values > 0)) {
               ptry <- as.numeric(- eout$vectors %*% (gq / eout$values))
               if(Norm(ptry) <= r) is.newton <- TRUE}
          if(is.newton == FALSE) {
               lambda.min <- min(eout$values)
               beta <- eout$values - lambda.min
               imin <- beta == 0
               C1 <- sum((gq / beta)[!imin]^2)
               C2 <- sum(gq[imin]^2)
               C3 <- sum(gq^2)
               if(C2 > 0 || C1 > r^2) {
                    is.easy <- TRUE
                    is.hard <- (C2 == 0)
                    beta.dn <- sqrt(C2) / r
                    beta.up <- sqrt(C3) / r
                    beta.adj <- function(x) {
                         if(x == 0) {
                              if(C2 > 0) return(- 1 / r)
                              else return(sqrt(1 / C1) - 1 / r)}
                         return(sqrt(1 / sum((gq /
                              {beta + x})^2)) - 1 / r)}
                    if(beta.adj(beta.up) <= 0) uout <- list(root=beta.up)
                    else if(beta.adj(beta.dn) >= 0) uout <- list(root=beta.dn)
                    else uout <- uniroot(beta.adj, c(beta.dn, beta.up))
                    wtry <- gq / {beta + uout$root}
                    ptry <- as.numeric(-eout$vectors %*% wtry)
                    }
               else {
                    is.hard <- TRUE
                    is.easy <- FALSE
                    wtry <- gq / beta
                    wtry[imin] <- 0
                    ptry <- as.numeric(- eout$vectors %*% wtry)
                    utry <- sqrt(r^2 - sum(ptry^2))
                    if(utry > 0) {
                         vtry <- eout$vectors[ , imin, drop=FALSE]
                         vtry <- vtry[ , 1]
                         ptry <- ptry + utry * vtry}
                    }
               }
          preddiff <- sum(ptry * {g + as.numeric(B %*% ptry) / 2})
          theta.try <- theta + ptry
          Mo <- Model(theta.try, Data)
          LP <- Mo[["LP"]]
          theta.try <- Mo[["parm"]]
          grad <- partial(Model, theta.try, Data)
          H <- Hessian(Model, theta.try, Data)
          ftry <- LP
          ftry <- ftry * -1
          rho <- {ftry - f} / preddiff
          if(ftry < Inf) {
               is.terminate <- {abs(ftry - f) < fterm} || {
                    abs(preddiff) < mterm}
               }
          else {
               is.terminate <- FALSE
               rho <- -Inf}
          if(is.terminate == TRUE) {
               if(ftry < f) {
                    accept <- TRUE
                    theta <- theta.try
                    post <- rbind(post, theta)
                    Dev <- rbind(Dev, Mo[["Dev"]])}
               }
          else {
               if(rho < 1 / 4) {
                    accept <- FALSE
                    r <- r / 4
                    }
               else {
                    accept <- TRUE
                    theta <- theta.try
                    post <- rbind(post, theta)
                    Dev <- rbind(Dev, Mo[["Dev"]])
                    if({rho > 3 / 4} && {is.newton == FALSE})
                    r <- min(2 * r, rmax)}}
          if(is.terminate == TRUE) break
          }
     Mo <- Model(theta, Data)
     theta <- Mo[["parm"]]
     LA <- list(Dev=Dev, H=H, iter=iter, parm.len=parm.len,
          parm.new=theta, parm.old=parm.old, post=post,
          Step.Size=abs(ftry-f), tol.new=abs(preddiff))
     return(LA)
     }
#End

###########################################################################
# PMC                                                                     #
#                                                                         #
# The purpose of the PMC function is update a model with Population       #
# Monte Carlo.                                                            #
###########################################################################

PMC <- function(Model, Data, Initial.Values, Covar=NULL, Iterations=10,
     Thinning=1, alpha=NULL, M=1, N=1000, nu=9, CPUs=1, Type="PSOCK")
     {
     cat("\nPMC was called on ", date(), "\n", sep="")
     time1 <- proc.time()
     pmc.call <- match.call()
     ##########################  Initial Checks  ##########################
     cat("\nPerforming initial checks...\n")
     if(missing(Model)) stop("A function must be entered for Model.")
     if(!is.function(Model)) stop("Model must be a function.")
     if(missing(Data))
          stop("A list containing data must be entered for Data.")
     if(is.null(Data[["mon.names"]])) stop("In Data, mon.names is NULL.")
     if(is.null(Data[["parm.names"]])) stop("In Data, parm.names is NULL.")
     for (i in 1:length(Data)) {
          if(is.matrix(Data[[i]])) {
               if(all(is.finite(Data[[i]]))) {
                    mat.rank <- qr(Data[[i]], tol=1e-10)$rank
                    if(mat.rank < ncol(Data[[i]])) {
                         cat("WARNING: Matrix", names(Data)[[i]],
                              "may be rank-deficient.\n")}}}}
     M <- max(round(abs(M)), 1)
     N <- max(round(abs(N)), 1)
     if(missing(Initial.Values)) {
          cat("WARNING: Initial Values were not supplied.\n")
          Initial.Values <- matrix(0, M, length(Data[["parm.names"]]))}
     if(is.vector(Initial.Values)) {
          if(!identical(length(Initial.Values),
               length(Data[["parm.names"]]))) {
               cat("WARNING: The length of Initial Values differed from",
                    "Data$parm.names.\n")
          Initial.Values <- matrix(0, M, length(Data[["parm.names"]]))}}
     else {
          if(!identical(ncol(Initial.Values),
               length(Data[["parm.names"]]))) {
               cat("WARNING: Columns in Initial Values differed from",
                    "Data$parm.names.\n")
          Initial.Values <- matrix(0, M, length(Data[["parm.names"]]))}}
     if(any(!is.finite(Initial.Values))) {
          cat("WARNING: Initial Values contain non-finite values.\n")
          Initial.Values <- matrix(0, M, length(Data[["parm.names"]]))}
     if(is.vector(Initial.Values)) {
          LIV <- length(Initial.Values)
          Initial.Values <- matrix(Initial.Values, M, LIV, byrow=TRUE)}
     else if(nrow(Initial.Values) != M)
               stop("nrow(Initial.Values != M.")
     else LIV <- ncol(Initial.Values)
     ### Covar
     ScaleF <- 2.381204 * 2.381204 / LIV
     if(is.null(Covar))
          Covar <- array(diag(LIV)*ScaleF, dim=c(LIV,LIV,Iterations,M))
     else Covar <- array(Covar, dim=c(LIV, LIV, Iterations, M))
     ### Iterations and Thinning
     Iterations <- max(round(abs(Iterations)), 1)
     Thinning <- min(max(round(abs(Thinning)), 1), N*M/2)
     ### alpha
     if(is.null(alpha)) alpha <- matrix(rep(1/M, M), M, Iterations)
     else if(any(!is.finite(alpha))) {
          stop("\nWARNING: alpha had non-finite values. Creating alpha...\n.")
          alpha <- matrix(rep(1/M, M), M, Iterations)}
     if(is.vector(alpha)) alpha <- matrix(alpha, M, Iterations, byrow=TRUE)
     if(nrow(alpha) != M) stop("nrow(alpha) != M.")
     else if(ncol(alpha) != Iterations) stop("ncol(alpha) != Iterations.")
     if(any(colSums(alpha) != 1)) 
          alpha <- alpha / matrix(colSums(alpha), M, Iterations,
               byrow=TRUE)
     ### mu
     mu <- array(0, dim=c(Iterations, LIV, M))
     for (m in 1:M) {
          mu[,,m] <- matrix(Initial.Values[m,], Iterations, LIV,
               byrow=TRUE)}
     ### Miscellaneous
     nu <- max(abs(nu), 2.01)
     post <- array(0, dim=c(N, LIV, Iterations, M))
     LH <- LP <- array(0, dim=c(N, Iterations, M))
     LW <- matrix(0, N, Iterations)
     essn <- perp <- rep(0, Iterations)
     ##########################  Test the Model  ##########################
     for (m in 1:M) {
          M0 <- Model(as.vector(Initial.Values[m,]), Data)
          if(!is.list(M0)) stop("Model must return a list.")
          if(length(M0) != 5) stop("Model must return five components.")
          if(length(M0[["LP"]]) > 1) stop("Multiple joint posteriors exist!")
          if(!identical(length(M0[["Monitor"]]),
               length(Data[["mon.names"]])))
               stop("Length of mon.names differs from length of monitors.")
          if(any(!is.finite(c(M0[["LP"]],M0[["Dev"]],M0[["parm"]]))))
               stop("Model produces non-finite results.")}
     ### Looking for apply functions and for loops
     as.character.function <- function(x, ... )
          {
          fname <- deparse(substitute(x))
          f <- match.fun(x)
          out <- c(sprintf('"%s" <- ', fname), capture.output(f))
          if(grepl("^[<]", tail(out, 1))) out <- head(out, -1)
          return(out)
          }
     acount <- length(grep("apply", as.character.function(Model)))
     if(acount > 0) {
          cat("Suggestion:", acount, "possible instance(s) of apply functions\n")
          cat("     were found in the Model specification. Sampling speed will\n")
          cat("     increase if apply functions are vectorized in R or coded\n")
          cat("     in a faster language such as C++ via the Rcpp package.\n")}
     acount <- length(grep("for", as.character.function(Model)))
     if(acount > 0) {
          cat("Suggestion:", acount, "possible instance(s) of for loops\n")
          cat("     were found in the Model specification. Sampling speed will\n")
          cat("     increase if for loops are vectorized in R or coded in a\n")
          cat("     faster language such as C++ via the Rcpp package.\n")}
     ######################  Laplace Approximation  #######################
     ### Sample Size of Data
     if(!is.null(Data[["n"]])) if(length(Data[["n"]]) == 1) NN <- Data[["n"]] 
     if(!is.null(Data[["N"]])) if(length(Data[["N"]]) == 1) NN <- Data[["N"]] 
     if(!is.null(Data[["y"]])) NN <- nrow(matrix(Data[["y"]]))
     if(!is.null(Data[["Y"]])) NN <- nrow(matrix(Data[["Y"]]))
     if(is.null(NN)) stop("Sample size of Data not found in n, N, y, or Y.")
     if({sum(abs(Initial.Values[1,]) == 0) == ncol(Initial.Values)} &
          {NN >= 5*ncol(Initial.Values)}) {
          cat("\nLaplace Approximation will be used on initial values.\n")
          Fit.LA <- LaplaceApproximation(Model, Initial.Values[1,], Data,
               Method="SPG", CovEst="Hessian", sir=FALSE)
          Covar <- array(Fit.LA$Covar, dim=c(LIV, LIV, Iterations, M))
          Initial.Values <- matrix(Fit.LA$Summary1[1:ncol(Initial.Values),1],
               M, LIV)}
     ### Covar symmetry and positive-definiteness
     for (m in 1:M) {
          if(!is.symmetric.matrix(Covar[,,1,m]))
               Covar[,,1,m] <- as.symmetric.matrix(Covar[,,1,m])
          if(!is.positive.definite(Covar[,,1,m]))
               Covar[,,1,m] <- as.positive.definite(Covar[,,1,m])}
     ###################  Prepare for Parallelization  ####################
     CPUs <- abs(round(CPUs))
     if(CPUs > 1) {
          detectedCores <- max(detectCores(),
               as.integer(Sys.getenv("NSLOTS")), na.rm=TRUE)
          cat("\n\nCPUs Detected:", detectedCores, "\n")
          if(CPUs > detectedCores) {
               cat("\nOnly", detectedCores, "will be used.\n")
               CPUs <- detectedCores}}
     ############################  Begin PMC  #############################
     cat("\nPMC is beginning to update...\n")
     for (iter in 1:Iterations) {
          cat("Iteration: ", iter, sep="")
          bad <- FALSE
          ### Importance Sampling
          for (m in 1:M) {
               S <- nu/(nu-2) * Covar[,,iter,m]
               if(!is.symmetric.matrix(S))
                    S <- as.symmetric.matrix(S)
               if(!is.positive.definite(S))
                    S <- as.positive.definite(S)
               post[,,iter,m] <- rmvt(N, mu[iter,,m], S, nu)
               if(sum(!is.finite(post[,,iter,m]) > 0))
                    stop("Bad draws from importance distribution.")
               ### Non-Parallel Processing
               if(CPUs == 1) {
                    for (i in 1:N) {
                         mod <- Model(post[i,,iter,m], Data)
                         if(all(is.finite(c(mod[["LP"]], mod[["Dev"]],
                              mod[["parm"]]))))
                              M0 <- mod
                         LP[i,iter,m] <- M0[["LP"]]
                         post[i,,iter,m] <- M0[["parm"]]}
                    }
               else { ### Parallel Processing
                    cl <- makeCluster(CPUs, Type)
                    varlist <- unique(c(ls(), ls(envir=.GlobalEnv),
                         ls(envir=parent.env(environment()))))
                    clusterExport(cl, varlist=varlist, envir=environment())
                    clusterSetRNGStream(cl)
                    mod <- parLapply(cl, 1:nrow(post),
                         function(x) Model(post[x,,iter,m], Data))
                    stopCluster(cl)
                    LP[,iter,m] <- unlist(lapply(mod,
                         function(x) x[["LP"]]))[1:nrow(post)]
                    post[,,iter,m] <- matrix(unlist(lapply(mod,
                         function(x) x[["parm"]])), dim(post)[1],
                         dim(post)[2], byrow=TRUE)}
               ### Proposal Sampling Distribution H(theta) ~ MVT
               LH[,iter,m] <- dmvt(post[,,iter,m], mu[iter,,m],
                    S, nu, log=TRUE)
               if(any(!is.finite(LH[,iter,m]))) {
                    if(iter > 1) LH[,iter,m] <- LH[,iter-1,m]
                    else stop("Sampling density H has non-finite values.")}}
          ### Weights
          LW[,iter] <- apply(LP[,iter,] -
               matrix(apply(matrix(alpha[,iter], N, M, byrow=TRUE) +
                    LH[,iter,], 1, logadd), N, M, byrow=TRUE), 1, logadd)
          if(all(!is.finite(LW[,iter]))) {
               bad <- TRUE; LW[,iter] <- 0}
          else if(any(!is.finite(LW[,iter]))) {
               w <- which(!is.finite(LW[,iter]))
               LW[w,iter] <- 1e-800}
          ### Normalize Weights
          LW[,iter] <- LW[,iter] - logadd(LW[,iter])
          if(any(!is.finite(LW[,iter]))){
               bad <- TRUE
               if(iter == 1) {
                    cat(", WARNING: Bad weights, setting to 1/N.\n")
                    LW[,iter] <- rep(log(1/N), N)}
               else {
                    cat(", WARNING: Bad weights, using last iteration.\n")
                    alpha[,iter] <- alpha[,iter-1]
                    for (m in 1:M) Covar[,,iter,m] <- Covar[,,iter-1,m]
                    LH[,iter,] <- LH[,iter-1,]
                    LP[,iter,] <- LP[,iter-1,]
                    post[,,iter,] <- post[,,iter-1,]
                    LW[,iter] <- LW[,iter-1]}}
          ### Convergence
          essn[iter] <- (1 / sum(exp(LW[,iter])^2)) / N
          perp[iter] <- exp(-sum(exp(LW[,iter]) * LW[,iter])) / N
          if(all(LW[,iter] == log(1/N))) essn[iter] <- perp[iter] <- 0
          if(bad == FALSE)
               cat(",   ESSN: ", round(essn[iter],5),
                    ",   Perplexity: ", round(perp[iter],5), "\n", sep="")
          ### Adaptation
          LpM <- matrix(log(alpha[,iter]), N, M, byrow=TRUE) + LH[,iter,]
          LpM <- LpM - matrix(apply(LpM, 1, logadd), N, M)
          if(iter < Iterations) {
               alpha[,iter+1] <- exp(apply(LW[,iter] + LpM, 2, logadd))
               if(any(!is.finite(alpha[,iter+1])))
                    alpha[,iter+1] <- ifelse(!is.finite(alpha[,iter+1]),
                         alpha[,iter], alpha[,iter+1])
                    alpha[which(alpha[,iter+1] < 0.001),iter+1] <- 0.001
                    alpha[,iter+1] <- alpha[,iter+1] / sum(alpha[,iter+1])
               for (m in 1:M) {
                    mu[iter+1,,m] <- colSums(exp(LW[,iter]) *
                         post[,,iter,m] * exp(LpM[,m])) / alpha[m,iter+1]
                    if(any(!is.finite(mu[iter+1,,m])))
                         mu[iter+1,,m] <- mu[iter,,m]}
               for (m in 1:M) {
                    Covar[,,iter+1,m] <- 0
                    for (i in 1:N) {
                         Covar[,,iter+1,m] <- Covar[,,iter+1,m] +
                         (exp(LW[i,iter] + LpM[i,m]) *
                         ((post[i,,iter,m] - mu[iter+1,,m]) %*%
                         t(post[i,,iter,m] - mu[iter+1,,m]))) /
                         alpha[m,iter+1]}
                    if(any(!is.finite(diag(Covar[,,iter+1,m]))))
                         Covar[,,iter+1,m] <- Covar[,,iter,m]
                    if(any(diag(Covar[,,iter+1,m] < 1e-100)))
                         Covar[,,iter+1,m] <- Covar[,,iter,m]
                    Covar[,,iter+1,m] <- as.symmetric.matrix(Covar[,,iter+1,m])
                    if(!is.positive.definite(Covar[,,iter+1,m]))
                         Covar[,,iter+1,m] <- Covar[,,iter,m]
                    if(bad == TRUE) Covar[,,iter+1,m] <- Covar[,,iter,m]}}
          }
     ### Combine Samples from Mixture Components
     Posterior2 <- post[,,Iterations,1]
     colnames(Posterior2) <- Data[["parm.names"]]
     if(M > 1) {
          for (m in 2:M) {
               if(alpha[m,Iterations] >= 0.002)
                    Posterior2 <- rbind(Posterior2,
                         post[,,Iterations,m])}}
     Posterior2 <- Thin(Posterior2, By=Thinning)
     ### Final Sampling
     cat("Final Sampling\n")
     Dev <- rep(0, nrow(Posterior2))
     Mon <- matrix(0, nrow(Posterior2), length(Data[["mon.names"]]))
     colnames(Mon) <- Data[["mon.names"]]
     for (i in 1:nrow(Posterior2)) {
          temp <- Model(Posterior2[i,], Data)
          Dev[i] <- temp[["Dev"]]
          Mon[i,] <- temp[["Monitor"]]}
     ### Posterior Summary Table
     cat("Creating Summaries\n")
     Summ <- matrix(NA, LIV, 7, dimnames=list(Data[["parm.names"]],
          c("Mean","SD","MCSE","ESS","LB","Median","UB")))
     Summ[,1] <- colMeans(Posterior2)
     Summ[,2] <- sqrt(.colVars(Posterior2))
     Summ[,3] <- 0
     Summ[,4] <- ESS(Posterior2)
     Summ[,5] <- apply(Posterior2, 2, quantile, c(0.025), na.rm=TRUE)
     Summ[,6] <- apply(Posterior2, 2, quantile, c(0.500), na.rm=TRUE)
     Summ[,7] <- apply(Posterior2, 2, quantile, c(0.975), na.rm=TRUE)
     for (i in 1:ncol(Posterior2)) {
          temp <- try(MCSE(Posterior2[,i]), silent=TRUE)
          if(!inherits(temp, "try-error")) Summ[i,3] <- temp
          else Summ[i,3] <- MCSE(Posterior2[,i], method="sample.variance")}
     Deviance <- rep(NA,7)
     Deviance[1] <- mean(Dev)
     Deviance[2] <- sd(as.vector(Dev))
     temp <- try(MCSE(as.vector(Dev)))
     if(inherits(temp, "try-error"))
          temp <- MCSE(as.vector(Dev), method="sample.variance")
     Deviance[3] <- temp
     Deviance[4] <- ESS(Dev)
     Deviance[5] <- as.numeric(quantile(Dev, probs=0.025, na.rm=TRUE))
     Deviance[6] <- as.numeric(quantile(Dev, probs=0.500, na.rm=TRUE))
     Deviance[7] <- as.numeric(quantile(Dev, probs=0.975, na.rm=TRUE))
     Summ <- rbind(Summ, Deviance)
     for (j in 1:ncol(Mon)) {
          Monitor <- rep(NA,7)
          Monitor[1] <- mean(Mon[,j])
          Monitor[2] <- sd(as.vector(Mon[,j]))
          temp <- try(MCSE(Mon[,j]), silent=TRUE)
          if(inherits(temp, "try-error")) 
               temp <- MCSE(Mon[,j], method="sample.variance")
          Monitor[3] <- temp
          Monitor[4] <- ESS(Mon[,j])
          Monitor[5] <- as.numeric(quantile(Mon[,j], probs=0.025,
               na.rm=TRUE))
          Monitor[6] <- as.numeric(quantile(Mon[,j], probs=0.5,
               na.rm=TRUE))
          Monitor[7] <- as.numeric(quantile(Mon[,j], probs=0.975,
               na.rm=TRUE))
          Summ <- rbind(Summ, Monitor)
          rownames(Summ)[nrow(Summ)] <- Data[["mon.names"]][j]}
     ### Logarithm of the Marginal Likelihood
     LML <- list(LML=NA, VarCov=NA)
     cat("Estimating Log of the Marginal Likelihood\n")
     LML <- LML(theta=Posterior2,
          LL=as.vector(Dev)*(-1/2), method="NSIS")
     time2 <- proc.time()
     ### Compile Output
     cat("Creating Output\n")
     pmc.out <- (list(alpha=alpha,
          Call=pmc.call,
          Covar=Covar,
          Deviance=Dev,
          DIC=c(mean(as.vector(Dev)),
               var(as.vector(Dev))/2,
               mean(as.vector(Dev)) + var(as.vector(Dev))/2),
          ESSN=essn,
          Initial.Values=Initial.Values,
          Iterations=Iterations,
          LML=LML[[1]],
          M=M,
          Minutes=round(as.vector(time2[3] - time1[3]) / 60,2),
          Model=Model,
          N=N,
          nu=nu,
          Mu=mu,
          Monitor=Mon,
          Parameters=LIV,
          Perplexity=perp,
          Posterior1=post,
          Posterior2=Posterior2,
          Summary=Summ,
          Thinned.Samples=nrow(Posterior2),
          Thinning=Thinning,
          W=exp(LW)))
     class(pmc.out) <- "pmc"
     return(pmc.out)
     }

#End

###########################################################################
# plot.demonoid.hpc                                                       #
#                                                                         #
# The purpose of the plot.demonoid.hpc function is to plot an object of   #
# class demonoid.hpc.                                                     #
###########################################################################

plot.demonoid.hpc <- function(x, BurnIn=0, Data=NULL, PDF=FALSE, Parms=NULL,
                              FileName = paste0("laplacesDemon-plot_", format(Sys.time(), "%Y-%m-%d_%T"), ".pdf"), ...)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     if(!inherits(x, "demonoid.hpc"))
          stop("x must be of class demonoid.hpc.")
     Chains <- length(x)
     if(is.null(Data)) stop("The Data argument is NULL.")
     nn <- nrow(x[[1]]$Posterior1)
     if(BurnIn >= nn) BurnIn <- 0
     Stat.at <- BurnIn + 1
     ### Selecting Parms
     if(is.null(Parms)) {
          Posterior <- list()
          for (i in 1:Chains) {
               Posterior[[i]] <- x[[i]][["Posterior1"]]}
          }
     else {
          Parms <- sub("\\[","\\\\[",Parms)
          Parms <- sub("\\]","\\\\]",Parms)
          Parms <- sub("\\.","\\\\.",Parms)
          if(length(grep(Parms[1], colnames(x[[1]]$Posterior1))) == 0)
               stop("Parameter in Parms does not exist.")
          keepcols <- grep(Parms[1], colnames(x[[1]]$Posterior1))
          if(length(Parms) > 1) {
               for (i in 2:length(Parms)) {
                    if(length(grep(Parms[i],
                         colnames(x[[1]]$Posterior1))) == 0)
                         stop("Parameter in Parms does not exist.")
                    keepcols <- c(keepcols,
                         grep(Parms[i], colnames(x[[1]]$Posterior1)))}}
          Posterior <- list()
          for (i in 1:Chains) {
               Posterior[[i]] <- matrix(x[[i]][["Posterior1"]][,keepcols],
                    nn, length(keepcols))}
          }
     if(PDF == TRUE) {
          pdf(FileName)
          par(mfrow=c(3,3))
          }
     else {par(mfrow=c(3,3), ask=TRUE)}
     ### Plot Parameters
     for (j in 1:ncol(Posterior[[1]])) {
          plot(Stat.at:nn, Posterior[[1]][Stat.at:nn,j],
               ylim=c(min(matrix(sapply(Posterior, function(x) {
                         min = min(x[,j])}), nn, Chains)[Stat.at:nn,]),
                    max(matrix(sapply(Posterior, function(x) {
                         max = max(x[,j])}), nn, Chains)[Stat.at:nn,])),
               col=rgb(0,0,0,50,maxColorValue=255), type="l",
               xlab="Thinned Samples", ylab="Value",
               main=colnames(Posterior[[1]])[j])
          for (n in 2:Chains) {
               lines(Stat.at:nn, Posterior[[n]][Stat.at:nn,j],
                    col=rgb(col2rgb(n)[1],col2rgb(n)[2],col2rgb(n)[3],50,
                    maxColorValue=255))}
          plot(density(Posterior[[1]][Stat.at:nn,j]), col="white",
               xlab="Value", main=colnames(Posterior[[1]])[j])
          polygon(density(Posterior[[1]][Stat.at:nn,j]),
               col=rgb(0,0,0,50,maxColorValue=255), border=NA)
          for (n in 2:Chains) {
               polygon(density(Posterior[[n]][Stat.at:nn,j]),
               col=rgb(col2rgb(n)[1],col2rgb(n)[2],col2rgb(n)[3],50,
                    maxColorValue=255), border=NA)}
          abline(v=0, col="red", lty=2)
          ### Only plot an ACF if there's > 1 unique values
          if(!is.constant(Posterior[[1]][Stat.at:nn,j])) {
               z <- acf(Posterior[[1]][Stat.at:nn,j], plot=FALSE)
               se <- 1/sqrt(length(Posterior[[1]][Stat.at:nn,j]))
               plot(z$lag, z$acf, ylim=c(min(z$acf,-2*se),1),
                    col=rgb(0,0,0,50,maxColorValue=255), type="h",
                    main=colnames(Posterior[[1]])[j], xlab="Lag",
                    ylab="Correlation")
               abline(h=(2*se), col="red", lty=2)
               abline(h=(-2*se), col="red", lty=2)
               for (n in 2:Chains) {
                    z <- acf(Posterior[[n]][Stat.at:nn,j], plot=FALSE)
                    se <- 1/sqrt(length(Posterior[[n]][Stat.at:nn,j]))
                    lines(z$lag, z$acf, col=rgb(col2rgb(n)[1],
                         col2rgb(n)[2],col2rgb(n)[3],50,
                         maxColorValue=255))}
               }
          else {plot(0, 0, main=paste(colnames(Posterior[[1]])[j],
               "is a constant."))}
          }
     rm(Posterior)
     ### Plot Deviance
     Deviance <- list()
     for (i in 1:Chains) {Deviance[[i]] <- x[[i]][["Deviance"]]}
     plot(Stat.at:nn, Deviance[[1]][Stat.at:nn],
          ylim=c(min(sapply(Deviance, function(x) {min(x[Stat.at:nn])})),
               max(sapply(Deviance, function(x) {max(x[Stat.at:nn])}))),
          col=rgb(0,0,0,50,maxColorValue=255),
          type="l", xlab="Thinned Samples", ylab="Value", main="Deviance")
     for (n in 2:Chains) {
          lines(Stat.at:nn, Deviance[[n]][Stat.at:nn],
               col=rgb(col2rgb(n)[1], col2rgb(n)[2],col2rgb(n)[3],50,
                    maxColorValue=255))}
     plot(density(Deviance[[1]][Stat.at:nn]), col="white",
          xlab="Value", main="Deviance")
     polygon(density(Deviance[[1]][Stat.at:nn]),
          col=rgb(0,0,0,50,maxColorValue=255), border=NA)
     for (n in 2:Chains) {
          polygon(density(Deviance[[n]][Stat.at:nn]),
               col=rgb(col2rgb(n)[1], col2rgb(n)[2],col2rgb(n)[3],50,
                    maxColorValue=255), border=NA)}
     abline(v=0, col="red", lty=2)
     #### Only plot an ACF if there's > 1 unique values
     if(!is.constant(Deviance[[1]][Stat.at:nn])) {
          z <- acf(Deviance[[1]][Stat.at:nn], plot=FALSE)
          se <- 1/sqrt(length(Deviance[[1]][Stat.at:nn]))
          plot(z$lag, z$acf, ylim=c(min(z$acf,-2*se),1),
               col=rgb(0,0,0,50,maxColorValue=255), type="h",
               main="Deviance", xlab="Lag", ylab="Correlation")
          abline(h=(2*se), col="red", lty=2)
          abline(h=(-2*se), col="red", lty=2)
          for (n in 2:Chains) {
               z <- acf(Deviance[[n]][Stat.at:nn], plot=FALSE)
               se <- 1/sqrt(length(Deviance[[n]][Stat.at:nn]))
               lines(z$lag, z$acf, col=rgb(col2rgb(n)[1],
                      col2rgb(n)[2],col2rgb(n)[3],50,
                      maxColorValue=255))}
          }
     else {plot(0, 0, main="Deviance is a constant.")}
     rm(Deviance)
     #### Plot Monitored Variables
     J <- length(Data[["mon.names"]])
     Monitor <- list()
     for (i in 1:Chains) {
          Monitor[[i]] <- matrix(x[[i]][["Monitor"]], nn, J)}
     for (j in 1:J) {
          plot(Stat.at:nn, Monitor[[1]][Stat.at:nn,j],
               ylim=c(min(sapply(Monitor, function(x) {min(x[Stat.at:nn,j])})),
                    max(sapply(Monitor, function(x) {max(x[Stat.at:nn,j])}))),
               col=rgb(0,0,0,50,maxColorValue=255),
               type="l", xlab="Thinned Samples", ylab="Value",
               main=Data[["mon.names"]][j])
          for (n in 2:Chains) {
               lines(Stat.at:nn, Monitor[[n]][Stat.at:nn,j],
                    col=rgb(col2rgb(n)[1],col2rgb(n)[2],col2rgb(n)[3],50,
                    maxColorValue=255))}
          plot(density(Monitor[[1]][Stat.at:nn,j]), col="white",
               xlab="Value", main=Data[["mon.names"]][j])
          polygon(density(Monitor[[1]][Stat.at:nn,j]),
               col=rgb(0,0,0,50,maxColorValue=255), border=NA)
          for (n in 2:Chains) {
               polygon(density(Monitor[[n]][Stat.at:nn,j]),
                    col=rgb(col2rgb(n)[1],col2rgb(n)[2],col2rgb(n)[3],50,
                         maxColorValue=255), border=NA)}
          abline(v=0, col="red", lty=2)
          ### Only plot an ACF if there's > 1 unique values
          if(!is.constant(Monitor[[1]][Stat.at:nn,j])) {
               z <- acf(Monitor[[1]][Stat.at:nn,j], plot=FALSE)
               se <- 1/sqrt(length(Monitor[[1]][Stat.at:nn,j]))
               plot(z$lag, z$acf, ylim=c(min(z$acf,-2*se),1),
                    col=rgb(0,0,0,50,maxColorValue=255), type="h",
                    main=Data[["mon.names"]][j], xlab="Lag",
                    ylab="Correlation")
               abline(h=(2*se), col="red", lty=2)
               abline(h=(-2*se), col="red", lty=2)
               for (n in 2:Chains) {
                    z <- acf(Monitor[[n]][Stat.at:nn,j], plot=FALSE)
                    se <- 1/sqrt(length(Monitor[[n]][Stat.at:nn,j]))
                    lines(z$lag, z$acf, col=rgb(col2rgb(n)[1],
                         col2rgb(n)[2],col2rgb(n)[3],50,
                         maxColorValue=255))}
               }
          else {plot(0, 0, main=paste(Data[["mon.names"]][j],
                     "is a constant."))}
          }
     rm(Monitor)
     #### Diminishing Adaptation
     if(nrow(x[[1]]$CovarDHis) > 1) {
          Diff <- abs(diff(x[[1]]$CovarDHis))
          adaptchange <- matrix(NA, nrow(Diff), 3)
          for (i in 1:nrow(Diff)) {
               adaptchange[i,1:3] <- as.vector(quantile(Diff[i,],
                    probs=c(0.025, 0.500, 0.975)))}
          plot(adaptchange[,2], ylim=c(min(adaptchange), max(adaptchange)),
               type="l", col=rgb(0,0,0,50,maxColorValue=255),
               xlab="Adaptations", ylab="Absolute Difference",
               main="Proposal Variance", sub="Median=Red, 95% Bounds=Gray")
          for (n in 2:Chains) {
               Diff <- abs(diff(x[[n]]$CovarDHis))
               adaptchange <- matrix(NA, nrow(Diff), 3)
               for (i in 1:nrow(Diff)) {
                    adaptchange[i,1:3] <- as.vector(quantile(Diff[i,],
                         probs=c(0.025, 0.500, 0.975)))}
               lines(adaptchange[,2], col=rgb(col2rgb(n)[1],
                    col2rgb(n)[2],col2rgb(n)[3],50, maxColorValue=255))}
          }
     if(PDF == TRUE) dev.off()
     }

#End

###########################################################################
# WAIC                                                                    #
#                                                                         #
# The purpose of the WAIC function is to calculate the Widely Applicable  #
# Information Criterion.                                                  #
###########################################################################

WAIC <- function(x)
     {
     lppd <- sum (log(rowMeans(exp(x))))
     pWAIC1 <- 2*sum(log(rowMeans(exp(x))) - rowMeans(x))
     pWAIC2 <- sum(.rowVars(x))
     WAIC <- -2*lppd + 2*pWAIC2
     return(list(WAIC=WAIC, lppd=lppd, pWAIC=pWAIC2, pWAIC1=pWAIC1))
     }

#End

###########################################################################
# plot.importance                                                         #
#                                                                         #
# The purpose of the plot.importance function is to plot variable         #
# importance according either to BPIC or the L-criterion in an object of  #
# class importance.                                                       #
###########################################################################

plot.importance <- function(x, Style="BPIC", ...)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     if(!identical(Style, "BPIC") & !identical(Style, "Concordance") &
          !identical(Style, "Discrep") & !identical(Style, "L-criterion"))
          stop("Style is unrecognized.")
     if(!identical(class(x), "importance"))
          stop("x must be of class importance.")
     if(identical(Style, "BPIC"))
          dotchart(x[,1], main="Variable Importance", xlab="BPIC", pch=20)
     else if(identical(Style, "Concordance"))
          dotchart(x[,2], main="Variable Importance", xlab="Concordance",
               pch=20)
     else if(identical(Style, "Discrep"))
          dotchart(x[,3], main="Variable Importance",
               xlab="Discrepancy Statistic", pch=20)
     else dotchart(x[,4], main="Variable Importance", xlab="L-criterion",
               pch=20)
     return(invisible())
     }

#End

###########################################################################
# plot.demonoid                                                           #
#                                                                         #
# The purpose of the plot.demonoid function is to plot an object of class #
# demonoid.                                                               #
###########################################################################

plot.demonoid <- function(x, BurnIn=0, Data=NULL, PDF=FALSE, Parms=NULL,
                          FileName = paste0("laplacesDemon-plot_", format(Sys.time(), "%Y-%m-%d_%T"), ".pdf"), ...)
{
  ### Initial Checks
  if(missing(x)) stop("The x argument is required.")
  if(is.null(Data)) stop("The Data argument is NULL.")
  if(BurnIn >= nrow(x$Posterior1)) BurnIn <- 0
  Stat.at <- BurnIn + 1
  ### Selecting Parms
  if(is.null(Parms)) {Posterior <- x$Posterior1}
  else {
    Parms <- sub("\\[","\\\\[",Parms)
    Parms <- sub("\\]","\\\\]",Parms)
    Parms <- sub("\\.","\\\\.",Parms)
    if(length(grep(Parms[1], colnames(x$Posterior1))) == 0)
      stop("Parameter in Parms does not exist.")
    keepcols <- grep(Parms[1], colnames(x$Posterior1))
    if(length(Parms) > 1) {
      for (i in 2:length(Parms)) {
        if(length(grep(Parms[i],
                       colnames(x$Posterior1))) == 0)
          stop("Parameter in Parms does not exist.")
        keepcols <- c(keepcols,
                      grep(Parms[i], colnames(x$Posterior1)))}}
    Posterior <- as.matrix(x$Posterior1[,keepcols])
    colnames(Posterior) <- colnames(x$Posterior1)[keepcols]}
  if(PDF == TRUE) {
    pdf(FileName)
    par(mfrow=c(3,3))
  }
  else {par(mfrow=c(3,3), ask=TRUE)}
  ### Plot Parameters
  for (j in 1:ncol(Posterior))
  {
    plot(Stat.at:x$Thinned.Samples,
         Posterior[Stat.at:x$Thinned.Samples,j],
         type="l", xlab="Thinned Samples", ylab="Value",
         main=colnames(Posterior)[j])
    panel.smooth(Stat.at:x$Thinned.Samples,
                 Posterior[Stat.at:x$Thinned.Samples,j], pch="")
    plot(density(Posterior[Stat.at:x$Thinned.Samples,j]),
         xlab="Value", main=colnames(Posterior)[j])
    polygon(density(Posterior[Stat.at:x$Thinned.Samples,j]),
            col="black", border="black")
    abline(v=0, col="red", lty=2)
    ### Only plot an ACF if there's > 1 unique values
    if(!is.constant(Posterior[Stat.at:x$Thinned.Samples,j])) {
      z <- acf(Posterior[Stat.at:x$Thinned.Samples,j], plot=FALSE)
      se <- 1/sqrt(length(Posterior[Stat.at:x$Thinned.Samples,j]))
      plot(z$lag, z$acf, ylim=c(min(z$acf,-2*se),1), type="h",
           main=colnames(Posterior)[j], xlab="Lag",
           ylab="Correlation")
      abline(h=(2*se), col="red", lty=2)
      abline(h=(-2*se), col="red", lty=2)
    }
    else {plot(0,0, main=paste(colnames(Posterior)[j],
                               "is a constant."))}
  }
  ### Plot Deviance
  plot(Stat.at:length(x$Deviance),
       x$Deviance[Stat.at:length(x$Deviance)],
       type="l", xlab="Thinned Samples", ylab="Value", main="Deviance")
  panel.smooth(Stat.at:length(x$Deviance),
               x$Deviance[Stat.at:length(x$Deviance)], pch="")
  plot(density(x$Deviance[Stat.at:length(x$Deviance)]),
       xlab="Value", main="Deviance")
  polygon(density(x$Deviance[Stat.at:length(x$Deviance)]), col="black",
          border="black")
  abline(v=0, col="red", lty=2)
  ### Only plot an ACF if there's > 1 unique values
  if(!is.constant(x$Deviance[Stat.at:length(x$Deviance)])) {
    z <- acf(x$Deviance[Stat.at:length(x$Deviance)], plot=FALSE)
    se <- 1/sqrt(length(x$Deviance[Stat.at:length(x$Deviance)]))
    plot(z$lag, z$acf, ylim=c(min(z$acf,-2*se),1), type="h",
         main="Deviance", xlab="Lag", ylab="Correlation")
    abline(h=(2*se), col="red", lty=2)
    abline(h=(-2*se), col="red", lty=2)
  }
  else {plot(0,0, main="Deviance is a constant.")}
  ### Plot Monitored Variables
  if(is.vector(x$Monitor)) {J <- 1; nn <- length(x$Monitor)}
  else if(is.matrix(x$Monitor)) {
    J <- ncol(x$Monitor); nn <- nrow(x$Monitor)}
  for (j in 1:J)
  {
    plot(Stat.at:nn, x$Monitor[Stat.at:nn,j],
         type="l", xlab="Thinned Samples", ylab="Value",
         main=Data[["mon.names"]][j])
    panel.smooth(Stat.at:nn, x$Monitor[Stat.at:nn,j], pch="")
    plot(density(x$Monitor[Stat.at:nn,j]),
         xlab="Value", main=Data[["mon.names"]][j])
    polygon(density(x$Monitor[Stat.at:nn,j]), col="black",
            border="black")
    abline(v=0, col="red", lty=2)
    ### Only plot an ACF if there's > 1 unique values
    if(!is.constant(x$Monitor[Stat.at:nn,j])) {
      z <- acf(x$Monitor[Stat.at:nn,j], plot=FALSE)
      se <- 1/sqrt(length(x$Monitor[Stat.at:nn,j]))
      plot(z$lag, z$acf, ylim=c(min(z$acf,-2*se),1), type="h",
           main=Data[["mon.names"]][j], xlab="Lag",
           ylab="Correlation")
      abline(h=(2*se), col="red", lty=2)
      abline(h=(-2*se), col="red", lty=2)
    }
    else {plot(0,0, main=paste(Data[["mon.names"]][j],
                               "is a constant."))}
  }
  ### Diminishing Adaptation
  if(nrow(x$CovarDHis) > 1) {
    if(x$Algorithm %in% c("Adaptive Hamiltonian Monte Carlo",
                          "Hamiltonian Monte Carlo with Dual-Averaging",
                          "No-U-Turn Sampler")) {
      plot(x$CovarDHis[,1], type="l", xlab="Adaptations",
           main="Step-Size", ylab=expression(epsilon))}
    else {
      if(x$Algorithm %in% c("Oblique Hyperrectangle Slice Sampler",
                            "Univariate Eigenvector Slice Sampler"))
        title <- "Eigenvectors"
      else if(x$Algorithm %in% c("Metropolis-Adjusted Langevin Algorithm"))
        title <- "Lambda"
      else if(x$Algorithm %in% c("Componentwise Hit-And-Run Metropolis",
                                 "Hit-And-Run Metropolis"))
        title <- "Proposal Distance"
      else if(x$Algorithm %in% c("Adaptive Griddy-Gibbs",
                                 "Adaptive Metropolis-within-Gibbs",
                                 "Sequential Adaptive Metropolis-within-Gibbs",
                                 "Updating Sequential Adaptive Metropolis-within-Gibbs"))
        title <- "Proposal S.D."
      else if(x$Algorithm %in% c("Differential Evolution Markov Chain"))
        title <- "Z"
      else if(x$Algorithm %in% c("Adaptive Factor Slice Sampler",
                                 "Refractive Sampler"))
        title <- "Step-Size"
      else title <- "Proposal Variance"
      Diff <- abs(diff(x$CovarDHis))
      adaptchange <- matrix(NA, nrow(Diff), 3)
      for (i in 1:nrow(Diff)) {
        adaptchange[i,1:3] <- as.vector(quantile(Diff[i,],
                                                 probs=c(0.025, 0.500, 0.975)))}
      plot(1:nrow(Diff), adaptchange[,2],
           ylim=c(min(adaptchange), max(adaptchange)),
           type="l", col="red", xlab="Adaptations",
           ylab="Absolute Difference", main=title,
           sub="Median=Red, Interval=Transparent Red")
      polygon(c(1:nrow(Diff),rev(1:nrow(Diff))),
              c(adaptchange[,1], rev(adaptchange[,3])),
              col=rgb(255, 0, 0, 50, maxColorValue=255),
              border=FALSE)
      lines(adaptchange[,2], col="red")}
  }
  if(PDF == TRUE) dev.off()
}

# End

###########################################################################
# is.stationary                                                           #
#                                                                         #
# The purpose of the is.stationary function is to provide a logical test  #
# regarding whether or not a vector, matrix, or demonoid object is        #
# stationary. The Geweke.Diagnostic function is used.                     #
###########################################################################

is.stationary <- function(x)
  {
  if(missing(x)) stop("The x argument is required.")
  stationary <- FALSE
  if(is.vector(x)) {
       if(is.constant(x)) return(TRUE)
       options(warn=-1)
       test <- try(as.vector(Geweke.Diagnostic(x)), silent=TRUE)
       options(warn=0)
       if(!inherits(test, "try-error") & is.finite(test))
            if((test > -2) & (test < 2)) stationarity <- TRUE
       }
  else if(is.matrix(x)) {
       options(warn=-1)
       test <- try(as.vector(Geweke.Diagnostic(x)), silent=TRUE)
       options(warn=0)
       if(!inherits(test, "try-error") & all(is.finite(test)))
            if(all(test > -2) & all(test < 2)) stationary <- TRUE
       }
  else if(identical(class(x), "demonoid")) {
       if(x$Rec.BurnIn.Thinned < nrow(x$Posterior1)) stationary <- TRUE}
  else if(identical(class(x), "laplace")) {
       warning("x is an object of class laplace.")
       stationary <- TRUE}
  else warning("x is an unrecognized object.")
  return(stationary)
  }

#End

###########################################################################
# ESS                                                                     #
#                                                                         #
# The purpose of the ESS function is to estimate the effective sample     #
# size (ESS) of a target distribution after taking autocorrelation into   #
# account. Although the code is slightly different, it is essentially the #
# same as the effectiveSize function in the coda package.                 #
###########################################################################

ESS <- function(x)
     {
     x <- as.matrix(x)
     v0 <- order <- rep(0, ncol(x))
     names(v0) <- names(order) <- colnames(x)
     N <- nrow(x)
     z <- 1:N
     for (i in 1:ncol(x)) {
          lm.out <- lm(x[, i] ~ z)
          if(!identical(all.equal(sd(residuals(lm.out)), 0), TRUE)) {
               ar.out <- try(ar(x[,i], aic=TRUE), silent=TRUE)
               if(!inherits(ar.out, "try-error")) {
                    v0[i] <- ar.out$var.pred / {1 - sum(ar.out$ar)}^2
                    order[i] <- ar.out$order}}}
     spec <- list(spec=v0, order=order)
     spec <- spec$spec
     temp <- N * .colVars(x) / spec
     out <- spec
     out[which(spec != 0)] <- temp[which(spec != 0)]
     out[which(out < .Machine$double.eps)] <- .Machine$double.eps
     out[which(out > N)] <- N
     return(out)
     }

#End

###########################################################################
# MCSE                                                                    #
#                                                                         #
# The purpose of the MCSE function is to estimate the Monte Carlo         #
# Standard Error of a vector of posterior samples. Multiple methods are   #
# provided. The purpose of the MCSS function is to calculate the required #
# sample size `n' to achieve acceptable error `a' given a vector `x' of   #
# Monte Carlo samples and the sample variance (rather than the asymptotic #
# variance).                                                              #
###########################################################################

MCSE <- function(x, method="IMPS", batch.size="sqrt", warn=FALSE)
     {
     if(missing(x)) stop("The x argument is required.")
     if(method == "sample.variance") {
          ess <- try(ESS(x), silent=TRUE)
          if(inherits(ess, "try-error")) ess <- length(x)
          se <- sd(x) / sqrt(ess)
          return(se)}
     else if(method == "batch.means") {
          N <- length(x)
          if(N < 1000) if(warn) warning("Samples must be >= 1000.")
          if(N < 10) return(NA)
          if(batch.size == "sqrt") {
               b <- floor(sqrt(N)) # batch size
               a <- floor(N/b) # number of batches
               }
          else if(batch.size == "cuberoot") {
                    b <- floor(N^(1/3)) # batch size
                    a <- floor(N/b) # number of batches
                    }
          else { #Batch size is provided numerically
               stopifnot(is.numeric(batch.size))  
               b <- floor(batch.size) # batch size
               if(b > 1) a <- floor(N/b) # number of batches
               else stop("batch.size is invalid.")
               }
          Ys <- sapply(1:a, function(k) return(mean(x[((k-1)*b+1):(k*b)])))
          muhat <- mean(Ys)
          sigmahatsq <- b*sum((Ys - muhat)^2) / (a-1)
          bmse <- sqrt(sigmahatsq / N)
          return(list(est=muhat, se=bmse))
          }
     else if(method == "IMPS") {
          chainAC <- acf(x, type="covariance" ,plot=FALSE)$acf
          AClen <- length(chainAC)
          gammaAC <- chainAC[1:(AClen-1)] + chainAC[2:AClen]
          m <- 1
          currgamma <- gammaAC[1]
          k <- 1
          while ((k < length(gammaAC)) && (gammaAC[k+1] > 0) &&
               (gammaAC[k] >= gammaAC[k+1]))
               k <- k + 1
          if({k == length(gammaAC)} & {warn == TRUE})
               warning("May need to compute more autocovariances for IMPS.")
          options(warn=-1)
          sigmasq <- -chainAC[1] + 2*sum(gammaAC[1:k])
          se <- sqrt(sigmasq / length(x))
          options(warn=0)
          return(se)
          }
     else stop("The method is unknown.")
     }
MCSS <- function(x, a)
     {
     if(missing(x)) stop("The x argument is required.")
     if(missing(a)) stop("The a argument is required.")
     ess <- ESS(x)
     ratio <- length(x) / ess
     fx <- function(sdx, n, a) ((sdx / sqrt(n)) - a)^2
     optimized <- optimize(f=fx, interval=c(0, .Machine$integer.max),
          maximum=FALSE, a=a, sdx=sd(x))
     return(round(optimized$minimum * ratio))
     }

#End

###########################################################################
# joint.density.plot                                                      #
#                                                                         #
# The purpose of the joint.density.plot function is to produce a joint    #
# density plot from samples of two marginal posterior distributions. This #
# function is derived from the kde2d and bandwidth.nrd functions from the #
# MASS package.                                                           #
###########################################################################

joint.density.plot <- function(x, y, Title=NULL, contour=TRUE, color=FALSE,
     Trace=NULL)
     {
     ### Initial Checks
     xname <- deparse(substitute(x))
     yname <- deparse(substitute(y))
     x <- as.vector(x)
     y <- as.vector(y)
     if(!identical(length(y), length(x)))
          stop("vectors x and y must be the same length.")
     if(any(!is.finite(x)))
          stop("x must have finite values.")
     if(any(!is.finite(y)))
          stop("y must have finite values.")
     ### Two-Dimensional Kernel Density Estimates
     kde2d <- function(x, y, h, n=25, lims=c(range(x), range(y)))
          {
          nx <- length(x)
          if(any(!is.finite(lims)))
               stop("x and y must have finite values.")
          n <- rep(n, length.out=2L)
          gx <- seq.int(lims[1L], lims[2L], length.out=n[1L])
          gy <- seq.int(lims[3L], lims[4L], length.out=n[2L])
          h <- if(missing(h)) c(bandwidth.nrd(x), bandwidth.nrd(y))
          else rep(h, length.out=2L)
          h <- h / 4 # for S's bandwidth scale
          ax <- outer(gx, x, "-" ) / h[1L]
          ay <- outer(gy, y, "-" ) / h[2L]
          z <- tcrossprod(matrix(dnorm(ax), , nx),
               matrix(dnorm(ay), , nx)) / (nx * h[1L] * h[2L])
          list(x=gx, y=gy, z=z)
          }
     bandwidth.nrd <- function(x)
          {
          r <- quantile(x, c(0.25, 0.75))
          h <- (r[2L] - r[1L]) / 1.34
          4 * 1.06 * min(sqrt(var(x)), h) * length(x) ^ (-1/5)
          }
     dd <- kde2d(x,y)
     if(color == FALSE) {
     plot(x, y, cex=0.1, main=Title, xlab=xname, ylab=yname, col="gray")}
     else if(color == TRUE) {
          crp <- colorRampPalette(c("black","red","yellow","white"), space="rgb")
          image(dd, main=Title, xlab=xname, ylab=yname, col=crp(200))
          }
     if(contour == TRUE) {contour(dd, nlevels=10, add=TRUE)}
     if(!is.null(Trace)) {
          if(length(Trace) != 2) stop("Trace requires 2 elements.")
          if(Trace[1] >= Trace[2])
             stop("Trace[1] not smaller than Trace[2].")
          if(Trace[1] < 1) stop("Trace[1] < 1.")
          if(Trace[2] > length(x)) stop("Trace[2] > length(x).")
          lines(x[Trace[1]:Trace[2]], y[Trace[1]:Trace[2]], col="green")
          points(x[Trace[1]], y[Trace[1]], cex=0.5, col="green")
          }
     }

#End

###########################################################################
# is.constrained                                                          #
#                                                                         #
# The purpose of the is.constrained function is to provide a logical test #
# of whether or not initial values change as they are passed through the  #
# Model specification function, given data.                               #
###########################################################################

is.constrained <- function(Model, Initial.Values, Data)
     {
     if(missing(Model))
          stop("The Model argument is required.")
     if(missing(Initial.Values))
          stop("The Initial.Values argument is required.")
     if(missing(Data))
          stop("The Data argument is required.")
     Mo <- Model(Initial.Values, Data)
     constr <- Initial.Values != Mo[["parm"]]
     return(constr)
     }

#End

###########################################################################
# plotMatrix                                                              #
#                                                                         #
# The purpose of the plotMatrix function is to plot a numerical matrix.   #
###########################################################################

plotMatrix <- function(x, col=colorRampPalette(c("red","black","green"))(100),
     cex=1, circle=TRUE, order=FALSE, zlim=NULL, title="", PDF=FALSE, ...)
     {
     ### Initial Checks
     if(missing(x)) stop("x is a required argument.")
     if(identical(class(x), "bayesfactor")) {
          x <- x$B
          title <- "Bayes Factors"}
     else if(identical(class(x), "demonoid")) {
          if(is.null(x$Covar) | is.list(x$Covar) | is.vector(x$Covar))
               stop("Covar=NULL.")
          x <- x$Covar
          title <- "Covariance"}
     else if(identical(class(x), "iterquad")) {
          if(is.null(x$Covar)) stop("Covar=NULL.")
          x <- x$Covar
          title <- "Covariance"}
     else if(identical(class(x), "laplace")) {
          if(is.null(x$Covar)) stop("Covar=NULL.")
          x <- x$Covar
          title <- "Covariance"}
     else if(identical(class(x), "pmc")) {
          if(is.null(x$Covar)) stop("Covar=NULL.")
          x <- x$Covar
          title <- "Covariance"}
     else if(identical(class(x), "posteriorchecks")) {
          x <- x$Posterior.Correlation
          title <- "Posterior Correlation"}
     else if(identical(class(x), "vb")) {
          if(is.null(x$Covar)) stop("Covar=NULL.")
          x <- x$Covar
          title <- "Covariance"}
     else if(!is.matrix(x)) x <- as.matrix(x)
     min <- min(x)
     max <- max(x)
     yLabels <- rownames(x)
     xLabels <- colnames(x)
     if(is.null(rownames(x))) xLabels <- 1:nrow(x)
     if(is.null(colnames(x))) yLabels <- 1:ncol(x)
     if(!is.null(zlim)) {
          if(length(zlim) != 2) stop("zlim must have length 2.")
          if(zlim[1] >= zlim[2])
               stop("zlim[1] must be lower than zlim[2].")
          min <- zlim[1]
          max <- zlim[2]}
     if(circle == TRUE & order == TRUE) {
          if(!nrow(x) == ncol(x)) 
               stop("The matrix must be square if order is TRUE.")
          x.eigen <- eigen(x)$vectors[, 1:2]
          e1 <- x.eigen[, 1]
          e2 <- x.eigen[, 2]
          alpha <- ifelse(e1 > 0, atan(e2/e1), atan(e2/e1) + pi)
          x <- x[order(alpha), order(alpha)]
          yLabels <- rownames(x)
          xLabels <- colnames(x)}
     ### Plot Matrix
     if(PDF == TRUE) pdf("plotMatrix.pdf")
     if(circle == FALSE) {
          ### Layout and Colors
          layout(matrix(data=c(1,2), nrow=1, ncol=2), widths=c(4,1),
               heights=c(1,1))
          ColorRamp <- col
          ColorLevels <- seq(min, max, length=length(ColorRamp))
          ### Reverse y-axis
          reverse <- nrow(x):1
          yLabels <- yLabels[reverse]
          x <- x[reverse,]
          ### Data Map
          par(mar = c(3,5,2.5,2))
          image(1:length(xLabels), 1:length(yLabels), t(x), col=ColorRamp,
               xlab="", ylab="", axes=FALSE, zlim=c(min,max))
          if(!is.null(title)) title(main=title)
          axis(BELOW<-1, at=1:length(xLabels), labels=xLabels, cex.axis=0.7)
          axis(LEFT <-2, at=1:length(yLabels), labels=yLabels,
               las=HORIZONTAL<-1, cex.axis=0.7)
          par(mar=c(3,2.5,2.5,2))
          image(1, ColorLevels,
               matrix(data=ColorLevels, ncol=length(ColorLevels), nrow=1),
               col=ColorRamp, xlab="", ylab="", xaxt="n")
          layout(1)}
     else {
          col <- col[length(col):1]
          ### Scale Covariance/Precision Matrices
          maxel <- max(abs(x))
          if(maxel > 1) x <- x * (1/maxel)
          ### Plot Setup
          par(mar=c(0, 0, 2, 0), bg="white")
          plot.new()
          plot.window(c(0, ncol(x)), c(0, nrow(x)), asp=1)
          cex.x <- 1 / {log(length(xLabels))/5 + 1}
          cex.y <- 1 / {log(length(yLabels))/5 + 1}
          xlabwidth <- max(strwidth(yLabels, cex=cex))
          ylabwidth <- max(strwidth(xLabels, cex=cex))
          plot.window(c(-xlabwidth + 0.5, ncol(x) + 0.5),
               c(0, nrow(x) + 1 + ylabwidth),
               asp=1, xlab="", ylab="")
          bg <- "gray10"
          rect(0.5, 0.5, ncol(x) + 0.5, nrow(x) + 0.5, col=bg)
          text(rep(-xlabwidth/2, nrow(x)), nrow(x):1, xLabels,
               col="black", cex=cex.x)
          text(1:ncol(x), rep(nrow(x) + 1 + ylabwidth/2, ncol(x)),
               yLabels, srt=90, col="black", cex=cex.y)
          ### Add grid
          lines <- "gray30"
          segments(rep(0.5, nrow(x) + 1), 0.5 + 0:nrow(x),
               rep(ncol(x) + 0.5, nrow(x) + 1),
               0.5 + 0:nrow(x), col=lines)
          segments(0.5 + 0:ncol(x), rep(0.5, ncol(x) + 1),
               0.5 + 0:ncol(x), rep(nrow(x) + 0.5, ncol(x)), col=lines)
          ### Assign circles' fill color
          nc <- length(col)
          if(nc==1) bg <- rep(col, prod(dim(x)))
          else {
               ff <- seq(-1,1, length=nc+1)
               bg2 <- rep(0, prod(dim(x)))
               for (i in 1:prod(dim((x)))) {
                    bg2[i] <- rank(c(ff[2:nc], as.vector(x)[i]),
                         ties.method="random")[nc]}
               bg <- (col[nc:1])[bg2]}
          ### Plot n*m circles using vector language, suggested by Yihui Xie
          ### the area of circles denotes the absolute value of coefficient
          symbols(rep(1:ncol(x), each=nrow(x)), rep(nrow(x):1, ncol(x)),
             add=TRUE, inches=F,
             circles=as.vector(sqrt(abs(x))/2), bg=as.vector(bg))
          }
     if(PDF == TRUE) dev.off()
     }

#End

###########################################################################
# Thin                                                                    #
#                                                                         #
# The purpose of the Thin function is to facilitate the thinning of a     #
# matrix of posterior samples.                                            #
###########################################################################

Thin <- function(x, By=1)
     {
     ### Initial Checks
     if(!is.matrix(x)) x <- as.matrix(x)
     rownum <- nrow(x)
     By <- abs(round(By))
     if(By > rownum) stop("By exceeds number of rows in x.")
     ### Thin
     keeprows <- which(rep(1:By, len=rownum) == By)
     z <- x[keeprows,]
     return(z)
     }

#End

###########################################################################
# is.bayesian                                                             #
#                                                                         #
# The purpose of the is.bayesian function is to determine whether or not  #
# a model is Bayesian by comparing the log-posterior (LP) and the LL.     #
###########################################################################

is.bayesian <- function(Model, Initial.Values, Data)
     {
     if(missing(Model)) stop("The Model argument is required.")
     if(missing(Initial.Values))
          stop("The Initial.Values argument is required.")
     if(missing(Data)) stop("The Data argument is required.")
     bayesian <- FALSE
     Mo <- Model(Initial.Values, Data)
     LL <- Mo[["Dev"]] / -2
     if(Mo[["LP"]] != LL) bayesian <- TRUE
     return(bayesian)
     }

#End

###########################################################################
# Sampling Importance Resampling (SIR)                                    #
#                                                                         #
# The purpose of the SIR function is to perform sampling importance       #
# re-sampling, usually to draw samples from the posterior as output from  #
# LaplaceApproxmation function. This function is similar to the sir       #
# function in the LearnBayes package.                                     #
###########################################################################

SIR <- function(Model, Data, mu, Sigma, n=1000, CPUs=1, Type="PSOCK") 
     {
     if(missing(Model)) stop("The Model function is required.")
     if(missing(Data)) stop("The Data argument is required.")
     if(missing(mu)) stop("The mu argument is required.")
     if(!is.vector(mu)) mu <- as.vector(mu)
     if(missing(Sigma)) stop("The Sigma argument is required.")
     if(!is.symmetric.matrix(Sigma)) Sigma <- as.symmetric.matrix(Sigma)
     if(!is.positive.definite(Sigma)) Sigma <- as.positive.definite(Sigma)
     if(length(mu) != nrow(Sigma)) stop("mu and Sigma are incompatible.")
     ### Sampling
     k <- length(mu)
     theta <- rmvn(n, mu, Sigma)
     theta[which(!is.finite(theta))] <- 0
     colnames(theta) <- Data[["parm.names"]]
     ### Importance
     lf <- matrix(0, n, 1)
     ### Non-Parallel Processing
     if(CPUs == 1) {
          for (i in 1:n) {
               mod <- Model(theta[i,], Data)
               lf[i] <- mod[["LP"]]
               theta[i,] <- mod[["parm"]]}
          }
     else { ### Parallel Processing
          detectedCores <- max(detectCores(),
               as.integer(Sys.getenv("NSLOTS")), na.rm=TRUE)
          cat("\n\nCPUs Detected:", detectedCores, "\n")
          if(CPUs > detectedCores) {
               cat("\nOnly", detectedCores, "will be used.\n")
               CPUs <- detectedCores}
          cl <- makeCluster(CPUs, Type)
          varlist <- unique(c(ls(), ls(envir=.GlobalEnv),
               ls(envir=parent.env(environment()))))
          clusterExport(cl, varlist=varlist, envir=environment())
          clusterSetRNGStream(cl)
          mod <- parLapply(cl, 1:nrow(theta),
               function(x) Model(theta[x,], Data))
          stopCluster(cl)
          lf <- unlist(lapply(mod,
               function(x) x[["LP"]]))[1:nrow(theta)]
          theta <- matrix(unlist(lapply(mod,
               function(x) x[["parm"]])), nrow(theta), ncol(theta))
          rm(mod)}
     lp <- dmvn(theta, mu, Sigma, log=TRUE)
     md <- max(lf - lp)
     lw <- lf - lp - md
     if(any(!is.finite(lw))) 
          lw[!is.finite(lw)] <- min(lw[is.finite(lw)])
     probs <- exp(lw - logadd(lw))
     ### Resampling
     options(warn=-1)
     indices <- try(sample.int(n, size=n, replace=TRUE, prob=probs),
          silent=TRUE)
     options(warn=0)
     if(inherits(indices, "try-error")) indices <- 1:n
     if(k > 1) theta <- theta[indices,]
     else theta <- theta[indices]
     return(theta)
     }

#End

###########################################################################
# Approximate Bayesian Bootstrap (ABB)                                    #
#                                                                         #
# The purpose of the ABB function is to perform Multiple Imputation (MI)  #
# with the Approximate Bayesian Bootstrap (ABB).                          #
###########################################################################

ABB <- function(X, K=1)
     {
     ### Initial Checks
     if(missing(X)) stop("X is a required argument.")
     if(!is.matrix(X)) X <- as.matrix(X)
     J <- ncol(X)
     N <- nrow(X)
     ### Missingness Indicator
     M <- X*0
     M[which(is.na(X))] <- 1
     if(sum(M) == 0) stop("There are no missing values to impute.")
     M.sums <- colSums(M)
     ### Approximate Bayesian Bootstrap
     MI <- list()
     for (k in 1:K) {
          imp <- NULL
          for (j in 1:J) {
               if(M.sums[j] > 0) {
                    ### Sample X.star.obs | X.obs
                    X.obs <- X[which(M[,j] == 0),j]
                    X.star.obs <- sample(X.obs, length(X.obs),
                         replace=TRUE)
                    ### Sample X.star.mis | X.star.obs
                    X.star.mis <- sample(X.star.obs, M.sums[j],
                         replace=TRUE)
                    if(length(imp) > 0) imp <- c(imp, X.star.mis)
                    else imp <- X.star.mis}
               }
          MI[[k]] <- imp
          }
     return(MI)
     }

#End

###########################################################################
# Validate                                                                #
#                                                                         #
# The purpose of the Validate function is to perform hold-out validation  #
# with BPIC.                                                              #
###########################################################################

Validate <- function(object, Model, Data, plot=FALSE, PDF=FALSE)
     {
     ### Initial Checks
     if(missing(object)) stop("The object argument is required.")
     if(missing(Model)) stop("The Model argument is required.")
     if(missing(Data)) stop("The Data argument is required.")
     if(!identical(class(object), "demonoid") &
        !identical(class(object), "pmc"))
          stop("object must be of class demonoid or pmc.")
     if(!is.list(Data)) stop("Data must be a list.")
     if(length(Data) != 2) stop("Data must have length 2.")
     if(is.null(Data[[1]][["y"]]) & is.null(Data[[1]][["Y"]]))
          stop("Data must have y or Y.")
     if(!is.null(Data[[1]][["y"]])) y1 <- as.vector(Data[[1]][["y"]])
     if(!is.null(Data[[1]][["Y"]])) y1 <- as.vector(Data[[1]][["Y"]])
     if(!is.null(Data[[2]][["y"]])) y2 <- as.vector(Data[[2]][["y"]])
     if(!is.null(Data[[2]][["Y"]])) y2 <- as.vector(Data[[2]][["Y"]])
     ### p(y[rep] | y)
     if(identical(class(object), "demonoid")) {
          post <- as.matrix(object$Posterior1)
          if(is.matrix(object$Posterior2) == TRUE) {
               post <- as.matrix(object$Posterior2)}}
     else {post <- as.matrix(object$Posterior2)}
     dev1 <- dev2 <- rep(NA, nrow(post))
     yhat1 <- matrix(NA, length(y1), nrow(post))
     yhat2 <- matrix(NA, length(y2), nrow(post))
     lengthcomp <- as.vector(Model(post[1,], Data[[1]])[["yhat"]])
     if(!identical(length(lengthcomp), length(y1)))
          stop("y and yhat differ in length.")
     for (i in 1:nrow(post)) {
          temp1 <- Model(post[i,], Data[[1]])
          temp2 <- Model(post[i,], Data[[2]])
          dev1[i] <- temp1[["Dev"]]
          dev2[i] <- temp2[["Dev"]]
          yhat1[,i] <- as.vector(temp1[["yhat"]])
          yhat2[,i] <- as.vector(temp2[["yhat"]])}
     ### BPIC
     Dbar.M <- round(mean(dev1, na.rm=TRUE),3)
     pD.M <- round(var(dev1, na.rm=TRUE)/2,3)
     BPIC.M <- Dbar.M + 2*pD.M
     Dbar.V <- round(mean(dev2, na.rm=TRUE),3)
     pD.V <- round(var(dev2, na.rm=TRUE)/2,3)
     BPIC.V <- Dbar.V + 2*pD.V
     bpic <- matrix(c(Dbar.M, pD.M, BPIC.M, Dbar.V, pD.V, BPIC.V), 3, 2)
     rownames(bpic) <- c("Dbar","pD","BPIC")
     colnames(bpic) <- c("Modeled","Validation")
     cat("\n")
     print(bpic)
     shorter <- min(length(dev1),length(dev2))
     cat("\np(Deviance.V > Deviance.M):",
          round(mean(dev2[1:shorter] > dev1[1:shorter], na.rm=TRUE),3))
     cat("\nE(Change in Deviance):",
          round(mean(dev2[1:shorter] - dev1[1:shorter], na.rm=TRUE),3),
          "\n\n")
     ### Warnings
     if(is.matrix(object$Posterior2) == FALSE) {
          warning("Non-stationary samples were used.")}
     if(any(is.na(yhat1))) cat("\nWARNING: Output matrix yhat.M has ",
          sum(is.na(yhat1)), " missing values.\n")
     if(any(is.na(yhat2))) cat("\nWARNING: Output matrix yhat.V has ",
          sum(is.na(yhat2)), " missing values.\n")
     if(any(is.nan(yhat1))) cat("\nWARNING: Output matrix yhat.M has ",
          sum(is.nan(yhat1)), " non-numeric (NaN) values.\n")
     if(any(is.nan(yhat2))) cat("\nWARNING: Output matrix yhat.V has ",
          sum(is.nan(yhat2)), " non-numeric (NaN) values.\n")
     if(any(is.infinite(yhat1))) cat("\nWARNING: Output matrix yhat.M has ",
          sum(is.infinite(yhat1)), " infinite values.\n")
     if(any(is.infinite(yhat2))) cat("\nWARNING: Output matrix yhat.V has ",
          sum(is.infinite(yhat2)), " infinite values.\n")
     ### Plot
     if(plot == TRUE) {
          if(PDF == TRUE) pdf("Validation.Plot.pdf")
          par(mfrow=c(2,1))
          kde1 <- density(dev1)
          kde2 <- density(dev2)
          plot(kde1, xlim=c(min(kde1$x,kde2$x),max(kde1$x,kde2$x)),
               ylim=c(min(kde1$y,kde2$y),max(kde1$y,kde2$y)), main="",
               xlab="Deviance: Modeled (Black), Validation (Red)",
               ylab="Density")
          polygon(kde1, col="black", border="black")
          lines(kde2, col="red")
          polygon(kde2, col="red", border="red")
          lines(kde1, col="black")
          dens.diff <- dev2[1:shorter] - dev1[1:shorter]
          kde <- density(dens.diff, na.rm=TRUE)
          plot(kde, col="gray", xlab="Change in Deviance", ylab="Density",
               main="", sub=paste("(D.V-D.M)=",
               round(mean(dens.diff, na.rm=TRUE),2), " (",
               round(as.vector(quantile(dens.diff, probs=0.025,
               na.rm=TRUE)),2), ",", round(as.vector(quantile(dens.diff,
               probs=0.975, na.rm=TRUE)),2), "), p(D.V > D.M) = ",
               round(mean(dev2[1:shorter] > dev1[1:shorter],
               na.rm=TRUE),2), sep=""))
          polygon(kde, col="gray", border="gray")
          abline(v=0, col="red", lty=2)
          if(PDF == TRUE) dev.off()}
     ### Create Output
     predicted <- list(list(y=y1, yhat=yhat1, Deviance=dev1),
          list(y=y2, yhat=yhat2, Deviance=dev2),
          BPIC=bpic)
     names(predicted) <- c("Modeled","Validation","BPIC")
     if(inherits(object, "demonoid")) class(predicted) <- "demonoid.val"
     else class(predicted) <- "pmc.val"
     return(predicted)
     }

#End

###########################################################################
# is.proper                                                               #
#                                                                         #
# The purpose of the is.proper function is to provide a logical check of  #
# whether or not a probability distribution is proper, meaning whether or #
# not it integrates to one.                                               #
###########################################################################

is.proper <- function(f, a, b, tol=1e-5)
       {
       ### Initial Checks
       if(!is.function(f) & !inherits(f, "demonoid") &
            !inherits(f, "iterquad") & !inherits(f, "laplace") &
            !inherits(f, "pmc"))
            stop("f is not a function or object of class demonoid, iterquad, laplace, or pmc.")
       ### Propriety
       propriety <- FALSE
       if(is.function(f)) {
            if(a >= b) stop("a >= b.")
            area <- integrate(f,a,b)$value
            if((area >= (1-tol)) & (area <= (1+tol))) propriety <- TRUE
            }
       else if(is.finite(f$LML)) propriety <- TRUE
       return(propriety)
       }

#End

###########################################################################
# print.miss                                                              #
#                                                                         #
# The purpose of the print.miss function is to print the contents of an   #
# object of class miss to the screen.                                     #
###########################################################################

print.miss <- function(x, ...)
     {
     if(missing(x)) stop("The x argument is required.")
     cat("\nAlgorithm:", x$Algorithm)
     cat("\nImp:")
     cat("\n  Missing Values:", nrow(x$Imp))
     cat("\n  Iterations:", ncol(x$Imp))
     cat("\nparm: (NOT SHOWN HERE)")
     cat("\nPostMode: (NOT SHOWN HERE)")
     cat("\nType: (NOT SHOWN HERE)\n")
     invisible(x)
     }

#End

###########################################################################
# print.pmc                                                               #
#                                                                         #
# The purpose of the print.pmc function is to print the contents of an    #
# object of class pmc to the screen.                                      #
###########################################################################

print.pmc <- function(x, ...)
     {
     if(missing(x)) stop("The x argument is required.")
     cat("Call:\n")
     print(x$Call)
     cat("\nalpha:\n", sep="")
     print(x$alpha)
     cat("Covariance Matrix: (NOT SHOWN HERE)\n")
     cat("Deviance: (NOT SHOWN HERE)\n")
     cat("Deviance Information Criterion (DIC):\n")
     DIC <- matrix(c(round(x$DIC[1],3), round(x$DIC[2],3),
          round(x$DIC[3],3)), 3, 1,
          dimnames=list(c("Dbar","pD","DIC"),c("All")))
     print(DIC)
     cat("ESSN:\n")
     print(x$ESSN)
     cat("Initial Values:\n")
     print(x$Initial.Values)
     cat("\nIterations: ", x$Iterations, "\n", sep="")
     cat("Log(Marginal Likelihood): ", x$LML, "\n", sep="")
     cat("M (Mixture Components): ", x$M, "\n", sep="")
     cat("Minutes of run-time: ", round(x$Minutes,2), "\n",
          sep="")
     cat("Model: (NOT SHOWN HERE)\n")
     cat("Monitor: (NOT SHOWN HERE)\n")
     cat("Mu: (NOT SHOWN HERE)\n")
     cat("Number of Samples: ", x$N, "\n", sep="")
     cat("nu: ", x$nu, "\n", sep="")
     cat("Parameters (Number of): ", x$Parameters, "\n",
          sep="")
     cat("Perpexity, Normalized:\n")
     print(x$Perplexity)
     cat("Posterior1: (NOT SHOWN HERE)\n")
     cat("Posterior2: (NOT SHOWN HERE)\n")
     cat("Summary: (SHOWN BELOW)\n")
     cat("Thinned Samples: ", x$Thinned.Samples, "\n",
          sep="")
     cat("Thinning: ", x$Thinning, "\n", sep="")
     cat("Weights: (NOT SHOWN HERE)\n")
     cat("\n\nSummary:\n")
     print(x$Summary)
     invisible(x)
     }

#End

###########################################################################
# Levene.Test                                                             #
#                                                                         #
# The purpose of the Levene.Test function is to apply Levene's test to    #
# residuals as a test of homoegeneity.                                    #
###########################################################################

Levene.Test <- function(x, Method="U", G=NULL, Data=NULL)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     if({class(x) != "demonoid.ppc"} & {class(x) != "iterquad.ppc"} &
        {class(x) != "laplace.ppc"} & {class(x) != "pmc.ppc"} &
        {class(x) != "vb.ppc"})
          stop("x is not of class demonoid.ppc, iterquad.ppc, laplace.ppc, pmc.ppc, or vb.ppc.")
     if({Method == "C"} & is.null(Data))
          stop("Data is required for Method C.")
     if({Method == "R"} & is.null(Data))
          stop("Data is required for Method R.")
     if({Method == "C" | Method == "R"} & is.null(Data[["Y"]]))
          stop("Y is required in Data.")
     y <- x[["y"]]
     yhat <- x[["yhat"]]
     if(is.null(Data) & is.null(G)) {
          G <- rep(1:4, each=round(nrow(yhat)/4), len=nrow(yhat))
          if(length(G) != length(y))
               stop("Lengths of G and y differ.")}
     if(!is.null(Data) & is.null(G) & {Method == "C" || Method == "R"}) {
          if(Method == "C") {
               G <- matrix(rep(1:4, each=round(nrow(Data[["Y"]])/4),
                    len=nrow(Data[["Y"]])), nrow(Data[["Y"]]),
                    ncol(Data[["Y"]]))}
          if(Method == "R") {
               G <- matrix(rep(1:4, each=round(ncol(Data[["Y"]])/4),
                    len=ncol(Data[["Y"]])), nrow(Data[["Y"]]),
                    ncol(Data[["Y"]]), byrow=TRUE)}}
     if(Method == "U") K <- length(unique(G))
     if(Method == "C") K <- length(unique(G[,1]))
     if(Method == "R") K <- length(unique(G[1,]))
     N <- nrow(yhat)
     S <- ncol(yhat)
     epsilon.obs <- y - yhat
     if(Method == "U") {
          epsilon.rep <- matrix(rnorm(N*S, mean(epsilon.obs, na.rm=TRUE),
               sd(as.vector(epsilon.obs), na.rm=TRUE)), N, S)}
     if(Method == "C") {
          epsilon.rep <- epsilon.obs
          for (j in 1:ncol(Data[["Y"]])) {
               point <- matrix(FALSE, nrow(Data[["Y"]]), ncol(Data[["Y"]]))
               point[,j] <- TRUE
               point <- as.vector(point)
               epsilon.rep[point,] <- rnorm(nrow(Data[["Y"]])*S,
                    mean(epsilon.obs[point,], na.rm=TRUE),
                    sd(as.vector(epsilon.obs[point,]), na.rm=TRUE))}}
     if(Method == "R") {
          epsilon.rep <- epsilon.obs
          for (i in 1:nrow(Data[["Y"]])) {
               point <- matrix(FALSE, nrow(Data[["Y"]]), ncol(Data[["Y"]]))
               point[i,] <- TRUE
               point <- as.vector(point)
               epsilon.rep[point,] <- rnorm(ncol(Data[["Y"]])*S,
                    mean(epsilon.obs[point,], na.rm=TRUE),
                    sd(as.vector(epsilon.obs[point,]), na.rm=TRUE))}}
     ### Levene Test
     if(Method == "U") {
          W.obs <- W.rep <- rep(0, S)
          for (s in 1:S) {
               epsilon.G.obs <- as.vector(by(epsilon.obs[,s], G, mean,
                    na.rm=TRUE))
               epsilon.G.rep <- as.vector(by(epsilon.rep[,s], G, mean,
                    na.rm=TRUE))
               Z.obs <- abs(epsilon.obs[,s] - epsilon.G.obs[G])
               Z.rep <- abs(epsilon.rep[,s] - epsilon.G.rep[G])
               Zbar.obs <- mean(Z.obs, na.rm=TRUE)
               Zbar.rep <- mean(Z.rep, na.rm=TRUE)
               Zbar.G.obs <- as.vector(by(Z.obs, G, mean, na.rm=TRUE))
               Zbar.G.rep <- as.vector(by(Z.rep, G, mean, na.rm=TRUE))
               W.obs[s] <- ((N-K)*sum((Zbar.G.obs[G] - Zbar.obs)^2,
                    na.rm=TRUE)) / ((K-1)*sum((Z.obs - Zbar.G.obs[G])^2,
                    na.rm=TRUE))
               W.rep[s] <- ((N-K)*sum((Zbar.G.rep[G] - Zbar.rep)^2,
                    na.rm=TRUE)) / ((K-1)*sum((Z.rep - Zbar.G.rep[G])^2,
                    na.rm=TRUE))}
          p <- round(mean(W.obs > W.rep, na.rm=TRUE),3)
          result <- "Homoskedastic"
          if((p < 0.025) | (p > 0.975)) result <- "Heteroskedastic"
          par(mfrow=c(1,1))
          d.W.obs <- density(W.obs)
          d.W.rep <- density(W.rep)
          plot(d.W.obs, xlim=c(min(d.W.obs$x, d.W.rep$x),
                    max(d.W.obs$x, d.W.rep$x)),
               ylim=c(0, max(d.W.obs$y, d.W.rep$y)),
               col=rgb(0,0,0,50,maxColorValue=255),
               main="Levene's Test",
               xlab="W",
               sub=paste("W.obs=", round(mean(W.obs, na.rm=TRUE),2),
                    " (", round(as.vector(quantile(W.obs, probs=0.025,
                    na.rm=TRUE)),2), ",", round(as.vector(quantile(W.obs,
                    probs=0.975, na.rm=TRUE)),2), "), p(W.obs > W.rep) = ",
                    p, " = ", result, sep=""),
               ylab="Density")
          polygon(d.W.obs, col=rgb(0,0,0,50,maxColorValue=255), border=NA)
          polygon(d.W.rep, col=rgb(255,0,0,50,maxColorValue=255),
               border=NA)}
     if(Method == "C") {
          par(mfrow=c(1,1), ask=TRUE)
          p <- rep(0, ncol(Data[["Y"]]))
          for (j in 1:ncol(Data[["Y"]])) {
               point <- matrix(FALSE, nrow(Data[["Y"]]), ncol(Data[["Y"]]))
               point[,j] <- TRUE
               point <- as.vector(point)
               W.obs <- W.rep <- rep(0, S)
               for (s in 1:S) {
                    epsilon.G.obs <- as.vector(by(epsilon.obs[point,s],
                         G[,j], mean, na.rm=TRUE))
                    epsilon.G.rep <- as.vector(by(epsilon.rep[point,s],
                         G[,j], mean, na.rm=TRUE))
                    Z.obs <- abs(epsilon.obs[point,s] -
                         epsilon.G.obs[G[,j]])
                    Z.rep <- abs(epsilon.rep[point,s] -
                         epsilon.G.rep[G[,j]])
                    Zbar.obs <- mean(Z.obs, na.rm=TRUE)
                    Zbar.rep <- mean(Z.rep, na.rm=TRUE)
                    Zbar.G.obs <- as.vector(by(Z.obs, G[,j], mean,
                         na.rm=TRUE))
                    Zbar.G.rep <- as.vector(by(Z.rep, G[,j], mean,
                         na.rm=TRUE))
                    W.obs[s] <- ((N-K)*sum((Zbar.G.obs[G[,j]] -
                         Zbar.obs)^2, na.rm=TRUE)) / ((K-1)*sum((Z.obs -
                         Zbar.G.obs[G[,j]])^2, na.rm=TRUE))
                    W.rep[s] <- ((N-K)*sum((Zbar.G.rep[G[,j]] -
                         Zbar.rep)^2, na.rm=TRUE)) / ((K-1)*sum((Z.rep -
                         Zbar.G.rep[G[,j]])^2, na.rm=TRUE))}
               p[j] <- round(mean(W.obs > W.rep, na.rm=TRUE),3)
               result <- "Homoskedastic"
               if((p[j] < 0.025) | (p[j] > 0.975))
                    result <- "Heteroskedastic"
               d.W.obs <- density(W.obs)
               d.W.rep <- density(W.rep)
               plot(d.W.obs, xlim=c(min(d.W.obs$x, d.W.rep$x),
                         max(d.W.obs$x, d.W.rep$x)),
                    ylim=c(0, max(d.W.obs$y, d.W.rep$y)),
                    col=rgb(col2rgb(j)[1], col2rgb(j)[2], col2rgb(j)[3],
                         50, maxColorValue=255),
                    main="Levene's Test",
                    xlab=paste("W.obs=", round(mean(W.obs, na.rm=TRUE),2),
                         " (", round(as.vector(quantile(W.obs, probs=0.025,
                         na.rm=TRUE)),2), ",",
                         round(as.vector(quantile(W.obs, probs=0.975,
                         na.rm=TRUE)),2), "), p(W.obs > W.rep) = ",
                         p[j], " = ", result, sep=""),
                    sub=paste("Y[,", j, "]", sep=""),
                    ylab="Density")
               polygon(d.W.obs, col=rgb(0,0,0,50,maxColorValue=255),
                    border=NA)
               polygon(d.W.rep, col=rgb(255,0,0,50,maxColorValue=255),
                    border=NA)}}
     if(Method == "R") {
          par(mfrow=c(1,1), ask=TRUE)
          p <- rep(0, nrow(Data[["Y"]]))
          for (i in 1:nrow(Data[["Y"]])) {
               point <- matrix(FALSE, nrow(Data[["Y"]]), ncol(Data[["Y"]]))
               point[i,] <- TRUE
               point <- as.vector(point)
               W.obs <- W.rep <- rep(0, S)
               for (s in 1:S) {
                    epsilon.G.obs <- as.vector(by(epsilon.obs[point,s],
                         G[i,], mean, na.rm=TRUE))
                    epsilon.G.rep <- as.vector(by(epsilon.rep[point,s],
                         G[i,], mean, na.rm=TRUE))
                    Z.obs <- abs(epsilon.obs[point,s] -
                         epsilon.G.obs[G[i,]])
                    Z.rep <- abs(epsilon.rep[point,s] -
                         epsilon.G.rep[G[i,]])
                    Zbar.obs <- mean(Z.obs, na.rm=TRUE)
                    Zbar.rep <- mean(Z.rep, na.rm=TRUE)
                    Zbar.G.obs <- as.vector(by(Z.obs, G[i,], mean,
                         na.rm=TRUE))
                    Zbar.G.rep <- as.vector(by(Z.rep, G[i,], mean,
                         na.rm=TRUE))
                    W.obs[s] <- ((N-K)*sum((Zbar.G.obs[G[i,]] -
                         Zbar.obs)^2, na.rm=TRUE)) /
                         ((K-1)*sum((Z.obs - Zbar.G.obs[G[i,]])^2,
                         na.rm=TRUE))
                    W.rep[s] <- ((N-K)*sum((Zbar.G.rep[G[i,]] -
                         Zbar.rep)^2, na.rm=TRUE)) /
                         ((K-1)*sum((Z.rep -
                         Zbar.G.rep[G[i,]])^2, na.rm=TRUE))}
               p[i] <- round(mean(W.obs > W.rep, na.rm=TRUE),3)
               result <- "Homoskedastic"
               if((p[i] < 0.025) | (p[i] > 0.975))
                    result <- "Heteroskedastic"
               d.W.obs <- density(W.obs)
               d.W.rep <- density(W.rep)
               plot(d.W.obs, xlim=c(min(d.W.obs$x, d.W.rep$x),
                         max(d.W.obs$x, d.W.rep$x)),
                    ylim=c(0, max(d.W.obs$y, d.W.rep$y)),
                    col=rgb(col2rgb(j)[1], col2rgb(j)[2], col2rgb(j)[3],
                         50, maxColorValue=255),
                    main="Levene's Test",
                    xlab=paste("W.obs=", round(mean(W.obs, na.rm=TRUE),2),
                         " (", round(as.vector(quantile(W.obs, probs=0.025,
                         na.rm=TRUE)),2), ",",
                         round(as.vector(quantile(W.obs, probs=0.975,
                         na.rm=TRUE)),2), "), p(W.obs > W.rep) = ",
                         p[i], " = ", result, sep=""),
                    sub=paste("Y[", i, ",]", sep=""),
                    ylab="Density")
               polygon(d.W.obs, col=rgb(0,0,0,50,maxColorValue=255),
                    border=NA)
               polygon(d.W.rep, col=rgb(255,0,0,50,maxColorValue=255),
                    border=NA)}}
     return(p)
     }

#End

###########################################################################
# Juxtapose                                                               #
#                                                                         #
# The purpose of the Juxtapose function is to compare the inefficiency of #
# multiple updates of LaplacesDemon, each with a different algorithm, but #
# the same model, initial values, and data. The internal ar.act and       #
# ar.act1 functions are slightly modified versions of those in the        #
# SamplerCompare package (modified to use the rmvn function), and the     #
# method of assessing inefficiency differs.                               #
###########################################################################

Juxtapose <- function(x)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     if(!is.list(x)) stop("The x argument must be a list.")
     ### IAT with probability intervals
     ar.act1  <- function(y)
          {
          stopifnot(NCOL(y) == 1)
          if(length(unique(y)) < 5)
               return(list(act=NA, act.025=NA, act.975=NA, se=NA,
                    order=NA))
          order.max <- NULL
          repeat {
               A <- ar.yw(y, demean=FALSE, order.max=order.max)
               if(A$order == 0)
                    A <- ar.yw(y, demean=FALSE, order.max=1, aic=FALSE)
               pi <- A$ar
               pi.var <- A$asy.var.coef
               if(kappa(pi.var) < 1 / sqrt(.Machine$double.eps) ||
                    isTRUE(order.max == 1))
                    break
               order.max <- floor(A$order * 0.7)}
          acf <- matrix(ARMAacf(ar=pi)[2:(A$order+1)])
          act <- (1-sum(pi*acf))/(1-sum(pi))^2
          simulation.length <- min(max(40, length(y)), 5000)
          pi.var2 <- as.symmetric.matrix(pi.var)
          AX <- rmvn(simulation.length, pi, pi.var2)
          act.sim <- numeric(simulation.length)
          for (i in 1:simulation.length) {
               pi.sim <- AX[i,]
               acf.sim <- ARMAacf(ar=pi.sim)[2:(A$order+1)]
               if(any(abs(polyroot(c(-1,pi.sim)))<1)) act.sim[i] <- Inf
               else act.sim[i] <- (1-sum(pi.sim*acf.sim)) /
                    (1-sum(pi.sim))^2}
          act.sim[is.na(act.sim)] <- Inf
          act.025 <- as.numeric(quantile(act.sim, 0.025))
          act.975 <- as.numeric(quantile(act.sim, 0.975))
          se <- (act.975-act.025)/(2*1.96)
          return(list(act=act, se=se, act.025=act.025, act.975=act.975,
               order=A$order))
          }
     ar.act <- function(Y, true.mean=NULL)
          {
          Y <- as.matrix(Y)
          stopifnot(is.null(true.mean) || ncol(Y)==length(true.mean))
          if(is.null(true.mean)) mu <- colMeans(Y)
          else mu <- true.mean
          acts <- sapply(1:ncol(Y), function(i) ar.act1(Y[,i]-mu[i]))
          max.i <- which.max(unlist(acts['act',]))
          if(length(max.i)!=1) {
               return(list(act=NA, act.025=NA, act.975=NA, se=NA,
                    order=NA))}
          else return(acts[,max.i])
          }
     ### Back to Juxtapose
     lenx <- length(x)
     ### Set up Output
     out <- matrix(NA, 9, lenx)
     algs <- rep(NA, lenx)
     for (i in 1:lenx) {
          if(!identical(class(x[[i]]), "demonoid"))
               stop("A component of x was found not be of class demonoid.")
          ### Use the abbreviated name of the algorithm
          algs[i] <- switch(x[[i]][["Algorithm"]],
               "Adaptive Directional Metropolis-within-Gibbs"="ADMG",
               "Adaptive Griddy-Gibbs"="AGG",
               "Adaptive Hamiltonian Monte Carlo"="AHMC",
               "Adaptive Metropolis"="AM",
               "Adaptive Metropolis-within-Gibbs"="AMWG",
               "Adaptive-Mixture Metropolis"="AMM",
               "Affine-Invariant Ensemble Sampler"="AIES",
               "Automated Factor Slice Sampler"="AFSS",
               "Componentwise Hit-And-Run Metropolis"="CHARM",
               "Delayed Rejection Adaptive Metropolis"="DRAM",
               "Delayed Rejection Metropolis"="DRM",
               "Differential Evolution Markov Chain"="DEMC",
               "Elliptical Slice Sampler"="ESS",
               "Experimental"="Exper",
               "Gibbs Sampler"="Gibbs",
               "Griddy-Gibbs"="GG",
               "Hamiltonian Monte Carlo"="HMC",
               "Hamiltonian Monte Carlo with Dual-Averaging"="HMCDA",
               "Hit-And-Run Metropolis"="HARM",
               "Independence Metropolis"="IM",
               "Interchain Adaptation"="INCA",
               "Metropolis-Adjusted Langevin Algorithm"="MALA",
               "Metropolis-Coupled Markov Chain Monte Carlo"="MCMCMC",
               "Metropolis-within-Gibbs"="MWG",
               "Multiple-Try Metropolis"="MTM",
               "No-U-Turn Sampler"="NUTS",
               "Oblique Hyperrectangle Slice Sampler"="OHSS",
               "Preconditioned Crank-Nicolson"="pCN",
               "Random Dive Metropolis-Hastings"="RDMH",
               "Random-Walk Metropolis"="RWM",
               "Reflective Slice Sampler"="RSS",
               "Refractive Sampler"="Refractive",
               "Reversible-Jump"="RJ",
               "Robust Adaptive Metropolis"="RAM",
               "Sequential Adaptive Metropolis-within-Gibbs"="SAMWG",
               "Sequential Metropolis-within-Gibbs"="SMWG",
               "Slice Sampler"="Slice",
               "Stochastic Gradient Langevin Dynamics"="SGLD",
               "Tempered Hamiltonian Monte Carlo"="THMC",
               "t-walk"="t-walk",
               "Univariate Eigenvector Slice Sampler"="UESS",
               "Updating Sequential Adaptive Metropolis-within-Gibbs"="USAMWG",
               "Updating Sequential Metropolis-within-Gibbs"="USMWG")
          }
     colnames(out) <- algs
     rownames(out) <- c("iter.min","t.iter.min","prop.stat","IAT.025",
          "IAT.500","IAT.975","ISM.025","ISM.500","ISM.975")
     class(out) <- "juxtapose"
     ### Juxtapose Algorithms
     for (i in 1:lenx) {
          iter.min <- x[[i]][["Iterations"]] / x[[i]][["Minutes"]]
          t.iter.min <- x[[i]][["Iterations"]] / x[[i]][["Thinning"]] /
               x[[i]][["Minutes"]]
          if(x[[i]][["Rec.BurnIn.Thinned"]] >= x[[i]][["Thinned.Samples"]])
               prop.stat <- 0
          else prop.stat <- 1 - (x[[i]][["Rec.BurnIn.Thinned"]] /
               x[[i]][["Thinned.Samples"]])
          if(all(is.na(x[[i]][["Summary2"]]))) {
               iat.500 <- iat.025 <- iat.975 <- Inf}
          else {
               iat.temp <- ar.act(x[[i]][["Posterior2"]])
               iat.500 <- iat.temp$act
               iat.025 <- iat.temp$act.025
               iat.975 <- iat.temp$act.975}
          ism.025 <- prop.stat * t.iter.min / iat.975
          ism.500 <- prop.stat * t.iter.min / iat.500
          ism.975 <- prop.stat * t.iter.min / iat.025
          out[1,i] <- round(iter.min,2)
          out[2,i] <- round(t.iter.min,2)
          out[3,i] <- round(prop.stat,2)
          out[4,i] <- round(iat.025,2)
          out[5,i] <- round(iat.500,2)
          out[6,i] <- round(iat.975,2)
          out[7,i] <- round(ism.025,2)
          out[8,i] <- round(ism.500,2)
          out[9,i] <- round(ism.975,2)
          }
     return(out)
     }

#End

###########################################################################
# Raftery.Diagnostic                                                      #
#                                                                         #
# The purpose of the Raftery.Diagnostic function is to perform MCMC       #
# diagnostics on an object of class demonoid.                             #
###########################################################################

Raftery.Diagnostic <- function(x, q=0.025, r=0.005, s=0.95, eps=0.001)
     {
     if(missing(x)) stop("x is a required argument")
     if(!identical(class(x), "demonoid"))
          stop("x must be an object of class demonoid.")
     if(all(is.na(x$Posterior2))) post <- x$Posterior1
     else post <- x$Posterior2
     Thinning <- x$Thinning
     resmatrix <- matrix(nrow=ncol(post), ncol=4,
          dimnames=list(colnames(post),
          c("M", "N", "Nmin", "I")))
     phi <- qnorm(0.5 * (1 + s))
     nmin <- as.integer(ceiling((q * (1 - q) * phi^2) / r^2))
     if(nmin > nrow(post)) resmatrix <- c("Error", nmin)
     else for (i in 1:ncol(post)) {
          quant <- quantile(post[, i, drop=TRUE], probs=q)
          dichot <- post[, i, drop=TRUE] <= quant
          kthin <- 0
          bic <- 1
          while (bic >= 0) {
               kthin <- kthin + Thinning
               testres <- as.vector(Thin(dichot, By=kthin))
               testres <- factor(testres, levels=c(FALSE,TRUE))
               newdim <- length(testres)
               testtran <- table(testres[1:(newdim - 2)],
                    testres[2:(newdim - 1)], testres[3:newdim])
               testtran <- array(as.double(testtran), dim=dim(testtran))
               g2 <- 0
               for (i1 in 1:2) {
                     for (i2 in 1:2) {
                          for (i3 in 1:2) {
                               if(testtran[i1, i2, i3] != 0) {
                                    fitted <- (sum(testtran[i1, i2, 1:2]) * 
                                         sum(testtran[1:2, i2, i3])) /
                                         (sum(testtran[1:2, i2, 1:2]))
                                    g2 <- g2 + testtran[i1, i2, i3] *
                                         log(testtran[i1, i2, i3]/fitted) * 2
                                    }}}}
               bic <- g2 - log(newdim - 2) * 2
               }
          finaltran <- table(testres[1:(newdim - 1)], testres[2:newdim])
          alpha <- finaltran[1, 2]/(finaltran[1, 1] + finaltran[1, 2])
          beta <- finaltran[2, 1]/(finaltran[2, 1] + finaltran[2, 2])
          tempburn <- log((eps * (alpha + beta))/max(alpha, 
               beta))/(log(abs(1 - alpha - beta)))
          nburn <- as.integer(ceiling(tempburn) * kthin)
          tempprec <- ((2 - alpha - beta) * alpha * beta * phi^2) /
               (((alpha + beta)^3) * r^2)
          nkeep <- as.integer(ceiling(tempprec) * kthin)
          iratio <- (nburn + nkeep) / nmin
          resmatrix[i, 1] <- nburn
          resmatrix[i, 2] <- nkeep + nburn
          resmatrix[i, 3] <- nmin
          resmatrix[i, 4] <- signif(iratio, digits=3)
          }
     y <- list(params=c(q=q, r=r, s=s), resmatrix=resmatrix)
     class(y) <- "raftery"
     return(y)
     }

#End

###########################################################################
# PMC.RAM                                                                 #
#                                                                         #
# The purpose of the PMC.RAM function is to estimate the RAM required to  #
# update a given model and data in PMC.                                   #
###########################################################################

PMC.RAM <- function(Model, Data, Iterations, Thinning, M, N)
     {
     if(missing(Model))
          stop("The Model argument is required.")
     if(missing(Data))
          stop("The Data argument is required.")
     Const <- 1048600
     LIV <- length(Data[["parm.names"]])
     LM <- length(Data[["mon.names"]])
     alpha <- as.vector(object.size(matrix(rep(1/M, M), M, Iterations))) /
          Const
     Covar <- as.vector(object.size(array(0,
          dim=c(LIV,LIV,Iterations,M)))) / Const
     Data <- as.vector(object.size(Data)) / Const
     Deviance <- as.vector(object.size(rep(0,N))) / Const
     Initial.Values <- as.vector(object.size(matrix(0, M,
          length(Data[["parm.names"]])))) / Const
     LH <- as.vector(object.size(array(0, dim=c(N, Iterations, M)))) /
          Const
     LP <- as.vector(object.size(array(0, dim=c(N, Iterations, M)))) /
          Const
     Model <- as.vector(object.size(Model)) / Const
     Monitor <- as.vector(object.size(matrix(runif(N*LM), N, LM))) / Const
     Mu <- as.vector(object.size(array(0, dim=c(Iterations, LIV, M)))) /
          Const                  
     Posterior1 <- as.vector(object.size(array(0,
          dim=c(N, LIV, Iterations, M)))) / Const
     Posterior2 <- Posterior1[,,Iterations,1]
     Posterior2 <- as.vector(object.size(Posterior2)) / Const
     #Note: Posterior2 gets thinned, but at one point it's this large.
     Summary <- as.vector(object.size(matrix(0, LIV+1+LM, 7))) / Const
     W <- as.vector(object.size(matrix(0, N, Iterations))) / Const
     mem.list <- list(alpha=alpha,
          Covar=Covar,
          Data=Data,
          Deviance=Deviance,
          Initial.Values=Initial.Values,
          LH=LH,
          LP=LP,
          Model=Model,
          Monitor=Monitor, 
          Mu=Mu,
          Posterior1=Posterior1,
          Posterior2=Posterior2,
          Summary=Summary,
          W=W,
          Total=sum(alpha,Covar,Data,Deviance,Initial.Values,LH,LP,
               Model,Monitor,Mu,Posterior1,Posterior2,Summary,W))
     return(mem.list)
     }

#End

###########################################################################
# as.initial.values                                                       #
#                                                                         #
# The purpose of the as.initial.values function is to retrieve the last   #
# posterior samples from an object of class demonoid, demonoid.hpc,       #
# iterquad, laplace, or pmc to serve as initial values for future         #
# updating.                                                               #
###########################################################################

as.initial.values <- function(x)
     {
     if(!identical(class(x), "demonoid") &
        !identical(class(x), "demonoid.hpc") &
        !identical(class(x), "iterquad") &
        !identical(class(x), "laplace") &
        !identical(class(x), "pmc") &
        !identical(class(x), "vb"))
          stop("The class of x is unknown.")
     if(identical(class(x), "demonoid")) {
          initial.values <- as.vector(x$Posterior1[x$Thinned.Samples,])
          }
     else if(identical(class(x), "demonoid.hpc")) {
          Chains <- length(x)
          LIV <- x[[1]][["Parameters"]]
          initial.values <- matrix(0, Chains, LIV)
          for (i in 1:Chains) {
               initial.values[i,] <- as.vector(x[[i]][["Posterior1"]][x[[i]][["Thinned.Samples"]],])}}
     else if(identical(class(x), "iterquad"))
          initial.values <- as.vector(x$Summary1[,"Mean"])
     else if(identical(class(x), "laplace"))
          initial.values <- as.vector(x$Summary1[,"Mode"])
     else if(identical(class(x), "vb"))
          initial.values <- as.vector(x$Summary1[,"Mean"])
     else if(x$M == 1)
          initial.values <- colMeans(x$Posterior2)
     else if(x$M > 1)
          initial.values <- t(x$Mu[dim(x$Mu)[1],,])
     return(initial.values)
     }

#End

###########################################################################
# is.constant                                                             #
#                                                                         #
# The purpose of the is.constant function is to provide a logical test of #
# whether or not a vector is a constant.                                  #
###########################################################################

is.constant <-  function(x)
     {
     if(missing(x)) stop("The x argument is required.")
     if(!is.vector(x)) x <- as.vector(x)
     uni <- length(unique(x))
     return(uni <= 1)
     }

#End

###########################################################################
# burnin                                                                  #
#                                                                         #
# The purpose of the burnin function is to estimate the duration of       #
# burn-in in iterations for one or more MCMC chains.                      #
###########################################################################

burnin <- function(x, method="BMK")
     {
     if(missing(x)) stop("The x argument is required.")
     if(is.vector(x)) x <- matrix(x, length(x), 1)
     n <- nrow(x)
     burn <- rep(0,ncol(x))
     if(method == "BMK") {
          if(n %% 10 == 0) x2 <- x
          if(n %% 10 != 0) x2 <- x[1:(10*trunc(n/10)),]
          HD <- BMK.Diagnostic(x2, 10)
          Ind <- 1 * (HD > 0.5)
          burn <- n
          batch.list <- seq(from=1, to=nrow(x2), by=floor(nrow(x2)/10))
          for (i in 1:9) {
               if(sum(Ind[,i:9]) == 0) {
                    burn <- batch.list[i] - 1
                    break
                    }
               }
          }
     else {
          for (i in 1:ncol(x)) {
               iter <- 1
               stationary <- 0
               jump <- round(n/10)
               while(stationary == 0) {
                    if(method == "KS") {
                         p <- KS.Diagnostic(x[iter:n])
                         if(p <= 0.05) stationary <- 1}
                    else { #method == Geweke
                         z <- try(Geweke.Diagnostic(x[iter:n]),
                              silent=TRUE)
                         if(inherits(z, "try-error")) z <- 3
                         if(abs(z < 2)) stationary <- 1}
                    if(stationary == 0) iter <- iter + jump
                    if(iter >= n) stationary <- 1}
               if(iter > 1) iter <- iter - 1
               if(iter > n) iter <- n
               burn[i] <- iter
               }
          }
     return(burn)
     }

#End

###########################################################################
# Mode                                                                    #
#                                                                         #
# The purpose of these functions is to return the mode or modes of a      #
# vector, or test for the presence or number of modes.                    #
###########################################################################

is.amodal <- function(x, min.size=0.1)
     {
     if(any(is.na(Modes(x, min.size)[[1]]))) return(TRUE)
     else return(FALSE)
     }
is.bimodal <- function(x, min.size=0.1)
     {
     if(length(Modes(x, min.size)[[1]]) == 2) return(TRUE)
     else return(FALSE)
     }
is.multimodal <- function(x, min.size=0.1)
     {
     if(length(Modes(x, min.size)[[1]]) > 1) return(TRUE)
     else return(FALSE)
     }
is.trimodal <- function(x, min.size=0.1)
     {
     if(length(Modes(x, min.size)[[1]]) == 3) return(TRUE)
     else return(FALSE)
     }
is.unimodal <- function(x, min.size=0.1)
     {
     if((length(Modes(x, min.size)[[1]]) == 1) &
          (!any(is.na(Modes(x, min.size)[[1]])))) return(TRUE)
     else return(FALSE)
     }
Mode <- function(x)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     if(!is.vector(x)) x <- as.vector(x)
     x <- x[is.finite(x)]
     ### Amodal
     if(is.constant(x)) return(NA)
     ### Discrete
     if(all(x == round(x))) {
          Mode <- as.numeric(names(which.max(table(x))))}
     ### Continuous (using kernel density)
     else {
          x <- as.vector(as.numeric(as.character(x)))
          kde <- density(x)
          Mode <- kde$x[kde$y == max(kde$y)][1]
          }
     return(Mode)
     }
Modes <- function(x, min.size=0.1) {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     x <- as.vector(as.numeric(as.character(x)))
     x <- x[is.finite(x)]
     ### Amodal
     if(is.constant(x))
          return(list(modes=NA, mode.dens=NA, size=1))
     ### Differentiate kernel density by x
     length(density(x)$y)
     dens.y.diff <- density(x)$y[-1] - density(x)$y[-length(density(x)$y)]
     incr <- dens.y.diff
     incr[which(dens.y.diff > 0)] <- 1
     incr[which(dens.y.diff <= 0)] <- 0
     ### Kernel density by increasing/decreasing density regions
     begin <- 1; count <- 1
     for (i in 2:length(incr)) {
          if(incr[i] != incr[i-1]) {
               count <- count + 1
               begin <- c(begin, i)}
          }
     begin <- c(begin, length(incr))
     size <- modes <- mode.dens <- rep(0, count/2)
     init <- 1
     dens <- density(x); sumdens <- sum(dens$y)
     if(incr[1] == 0) {
          size[1] <- sum(dens$y[1:begin[2]]) / sumdens
          init <- 2}
     j <- init
     for (i in init:length(size)) {
          size[i] <- sum(dens$y[begin[j]:begin[j+2]]) / sumdens
          kde <- dens
          kde$x <- kde$x[begin[j]:begin[j+2]]
          kde$y <- kde$y[begin[j]:begin[j+2]]
          modes[i] <- kde$x[kde$y == max(kde$y)][1]
          mode.dens[i] <- kde$y[kde$y == max(kde$y)][1]
          j <- j + 2
          }
     ### Order everything by density
     size <- size[order(mode.dens, decreasing=TRUE)]
     modes <- modes[order(mode.dens, decreasing=TRUE)]
     mode.dens <- mode.dens[order(mode.dens, decreasing=TRUE)]
     ### Remove modes with size < 10%
     if(any(size < 0.1)) {
         modes <- modes[-which(size < min.size)]
         mode.dens <- mode.dens[-which(size < min.size)]
         size <- size[-which(size < min.size)]
         }
     if(sum(size) > 1) size <- size / sum(size)
     #Output
     return(list(modes=modes, mode.dens=mode.dens, size=size))
     }


#End

###########################################################################
# Combine                                                                 #
#                                                                         #
# The purpose of the Combine function is to combine multiple objects of   #
# class demonoid.                                                         #
###########################################################################

Combine <- function(x, Data, Thinning=1)
     {
     ### Initial Checks
     if(missing(x)) stop("x is a required argument.")
     if(missing(Data)) stop("Data is a required argument.")
     Thinning <- abs(round(Thinning))
     len.x <- length(x)
     if(!all(sapply(x, class) == "demonoid")) {
          stop("At least one item in list x is not of class demonoid.")}
     Acceptance.Rate <- round(sum(sapply(x, with, Acceptance.Rate) *
               sapply(x, with, Iterations)) /
               sum(sapply(x, with, Iterations)),5)
     Algorithm <- x[[1]]$Algorithm
     Covar <- x[[len.x]]$Covar
     Iterations <- sum(sapply(x, with, Iterations))
     Model <- x[[1]]$Model
     Minutes <- sum(sapply(x, with, Minutes))
     Status <- round(mean(sapply(x, with, Status)))
     LIV <- x[[1]]$Parameters
     ### Combine
     thinned <- x[[1]]$Posterior1
     Dev <- matrix(x[[1]]$Deviance)
     Mon <- x[[1]]$Monitor
     for (i in 2:len.x) {
          thinned <- rbind(thinned, x[[i]]$Posterior1)
          Dev <- rbind(Dev, matrix(x[[i]]$Deviance))
          Mon <- rbind(Mon, x[[i]]$Monitor)
          }
     ### Thinning
     if(Thinning > 1) {
          thinned <- Thin(thinned, By=Thinning)
          Dev <- matrix(Thin(Dev, By=Thinning))
          Mon <- Thin(Mon, By=Thinning)}
     thinned.rows <- nrow(thinned)
     ### Assess Stationarity
     cat("\nAssessing Stationarity\n")
     if(thinned.rows %% 10 == 0) thinned2 <- thinned
     if(thinned.rows %% 10 != 0) thinned2 <- thinned[1:(10*trunc(thinned.rows/10)),]
     HD <- BMK.Diagnostic(thinned2, batches=10)
     Ind <- 1 * (HD > 0.5)
     BurnIn <- thinned.rows
     batch.list <- seq(from=1, to=nrow(thinned2), by=floor(nrow(thinned2)/10))
     for (i in 1:9) {
          if(sum(Ind[,i:9]) == 0) {
               BurnIn <- batch.list[i] - 1
               break
               }
          }
     Stat.at <- BurnIn + 1
     rm(batch.list, HD, Ind, thinned2)
     ### Assess Thinning and ESS Size for all parameter samples
     cat("Assessing Thinning and ESS\n")
     acf.rows <- trunc(10*log10(thinned.rows))
     acf.temp <- matrix(1, acf.rows, LIV)
     ESS1 <- Rec.Thin <- rep(1, LIV)
     for (j in 1:LIV) {
          temp0 <- acf(thinned[,j], lag.max=acf.rows, plot=FALSE)
          if(length(temp0$acf[-1,1,1]) == acf.rows)
               acf.temp[,j] <- abs(temp0$acf[-1,1,1])
          ESS1[j] <- ESS(thinned[,j])
          Rec.Thin[j] <- which(acf.temp[,j] <= 0.1)[1]*Thinning}
     Rec.Thin[which(is.na(Rec.Thin))] <- nrow(acf.temp)
     ### Assess ESS for all deviance and monitor samples
     ESS2 <- ESS(Dev)
     ESS3 <- ESS(Mon)
     ### Assess ESS for stationary samples
     if(Stat.at < thinned.rows) {
          ESS4 <- ESS(thinned[Stat.at:thinned.rows,])
          ESS5 <- ESS(Dev[Stat.at:thinned.rows,])
          ESS6 <- ESS(Mon[Stat.at:thinned.rows,]) }
     ### Posterior Summary Table 1: All Thinned Samples
     cat("Creating Summaries\n")
     Num.Mon <- ncol(Mon)
     Summ1 <- matrix(NA, LIV, 7, dimnames=list(Data[["parm.names"]],
          c("Mean","SD","MCSE","ESS","LB","Median","UB")))
     Summ1[,1] <- colMeans(thinned)
     Summ1[,2] <- sqrt(.colVars(thinned))
     Summ1[,3] <- 0
     Summ1[,4] <- ESS1
     Summ1[,5] <- apply(thinned, 2, quantile, c(0.025), na.rm=TRUE)
     Summ1[,6] <- apply(thinned, 2, quantile, c(0.500), na.rm=TRUE)
     Summ1[,7] <- apply(thinned, 2, quantile, c(0.975), na.rm=TRUE)
     for (i in 1:ncol(thinned)) {
          temp <- try(MCSE(thinned[,i]), silent=TRUE)
          if(!inherits(temp, "try-error")) Summ1[i,3] <- temp
          else Summ1[i,3] <- MCSE(thinned[,i], method="sample.variance")}
     Deviance <- rep(NA,7)
     Deviance[1] <- mean(Dev)
     Deviance[2] <- sd(as.vector(Dev))
     temp <- try(MCSE(as.vector(Dev)), silent=TRUE)
     if(inherits(temp, "try-error"))
          temp <- MCSE(as.vector(Dev), method="sample.variance")
     Deviance[3] <- temp
     Deviance[4] <- ESS2
     Deviance[5] <- as.numeric(quantile(Dev, probs=0.025, na.rm=TRUE))
     Deviance[6] <- as.numeric(quantile(Dev, probs=0.500, na.rm=TRUE))
     Deviance[7] <- as.numeric(quantile(Dev, probs=0.975, na.rm=TRUE))
     Summ1 <- rbind(Summ1, Deviance)
     for (j in 1:Num.Mon) {
          Monitor <- rep(NA,7)
          Monitor[1] <- mean(Mon[,j])
          Monitor[2] <- sd(as.vector(Mon[,j]))
          temp <- try(MCSE(as.vector(Mon[,j])), silent=TRUE)
          if(inherits(temp, "try-error"))
               temp <- MCSE(Mon[,j], method="sample.variance")
          Monitor[3] <- temp
          Monitor[4] <- ESS3[j]
          Monitor[5] <- as.numeric(quantile(Mon[,j], probs=0.025,
               na.rm=TRUE))
          Monitor[6] <- as.numeric(quantile(Mon[,j], probs=0.500,
               na.rm=TRUE))
          Monitor[7] <- as.numeric(quantile(Mon[,j], probs=0.975,
               na.rm=TRUE))
          Summ1 <- rbind(Summ1, Monitor)
          rownames(Summ1)[nrow(Summ1)] <- Data[["mon.names"]][j]
          }
     ### Posterior Summary Table 2: Stationary Samples
     Summ2 <- matrix(NA, LIV, 7, dimnames=list(Data[["parm.names"]],
          c("Mean","SD","MCSE","ESS","LB","Median","UB")))
     if(Stat.at < thinned.rows) {
          thinned2 <- matrix(thinned[Stat.at:thinned.rows,],
               thinned.rows-Stat.at+1, ncol(thinned))
          Dev2 <- matrix(Dev[Stat.at:thinned.rows,],
               thinned.rows-Stat.at+1, ncol(Dev))
          Mon2 <- matrix(Mon[Stat.at:thinned.rows,],
               thinned.rows-Stat.at+1, ncol(Mon))
          Summ2[,1] <- colMeans(thinned2)
          Summ2[,2] <- sqrt(.colVars(thinned2))
          Summ2[,3] <- 0
          Summ2[,4] <- ESS4
          Summ2[,5] <- apply(thinned2, 2, quantile, c(0.025), na.rm=TRUE)
          Summ2[,6] <- apply(thinned2, 2, quantile, c(0.500), na.rm=TRUE)
          Summ2[,7] <- apply(thinned2, 2, quantile, c(0.975), na.rm=TRUE)
          for (i in 1:ncol(thinned2)) {
               temp <- try(MCSE(thinned2[,i]), silent=TRUE)
               if(!inherits(temp, "try-error")) Summ2[i,3] <- temp
               else Summ2[i,3] <- MCSE(thinned2[,i],
                    method="sample.variance")}
          Deviance <- rep(NA,7)
          Deviance[1] <- mean(Dev2)
          Deviance[2] <- sd(as.vector(Dev2))
          temp <- try(MCSE(as.vector(Dev2)), silent=TRUE)
          if(inherits(temp, "try-error"))
               temp <- MCSE(as.vector(Dev2), method="sample.variance")
          Deviance[3] <- temp
          Deviance[4] <- ESS5
          Deviance[5] <- as.numeric(quantile(Dev2, probs=0.025,
               na.rm=TRUE))
          Deviance[6] <- as.numeric(quantile(Dev2, probs=0.500,
               na.rm=TRUE))
          Deviance[7] <- as.numeric(quantile(Dev2, probs=0.975,
               na.rm=TRUE))
          Summ2 <- rbind(Summ2, Deviance)
          for (j in 1:Num.Mon) {
               Monitor <- rep(NA,7)
               Monitor[1] <- mean(Mon2[,j])
               Monitor[2] <- sd(as.vector(Mon2[,j]))
               temp <- try(MCSE(as.vector(Mon[,j])), silent=TRUE)
               if(inherits(temp, "try-error"))
                    temp <- MCSE(as.vector(Mon[,j]),
                    method="sample.variance")
               Monitor[3] <- temp
               Monitor[4] <- ESS6[j]
               Monitor[5] <- as.numeric(quantile(Mon2[,j],
                    probs=0.025, na.rm=TRUE))
               Monitor[6] <- as.numeric(quantile(Mon2[,j],
                    probs=0.500, na.rm=TRUE))
               Monitor[7] <- as.numeric(quantile(Mon2[,j],
                    probs=0.975, na.rm=TRUE))
               Summ2 <- rbind(Summ2, Monitor)
               rownames(Summ2)[nrow(Summ2)] <- Data[["mon.names"]][j]}
          }
     ### Column names to samples
     if(identical(ncol(Mon), length(Data[["mon.names"]])))
          colnames(Mon) <- Data[["mon.names"]]
     if(identical(ncol(thinned), length(Data[["parm.names"]]))) {
          colnames(thinned) <- Data[["parm.names"]]}
     ### Logarithm of the Marginal Likelihood
     LML <- list(LML=NA, VarCov=NA)
     if(Algorithm %in% c("Adaptive Griddy-Gibbs",
          "Affine-Invariant Ensemble Sampler",
          "Automated Factor Slice Sampler",
          "Componentwise Hit-And-Run Metropolis",
          "Delayed Rejection Metropolis",
          "Elliptical Slice Sampler",
          "Gibbs Sampler",
          "Griddy-Gibbs",
          "Hamiltonian Monte Carlo",
          "Hit-And-Run Metropolis",
          "Independence Metropolis",
          "Metropolis-Adjusted Langevin Algorithm",
          "Metropolis-Coupled Markov Chain Monte Carlo",
          "Metropolis-within-Gibbs",
          "Multiple-Try Metropolis",
          "No-U-Turn Sampler",
          "Oblique Hyperrectangle Slice Sampler",
          "Preconditioned Crank-Nicolson",
          "Random Dive Metropolis-Hastings",
          "Random-Walk Metropolis",
          "Reflective Slice Sampler",
          "Refractive Sampler",
          "Reversible-Jump",
          "Sequential Metropolis-within-Gibbs",
          "Slice Sampler",
          "Stochastic Gradient Langevin Dynamics",
          "Tempered Hamiltonian Monte Carlo",
          "t-walk",
          "Univariate Eigenvector Slice Sampler") &
          {Stat.at < thinned.rows}) {
          cat("Estimating Log of the Marginal Likelihood\n")
          LML <- LML(theta=thinned2, LL=as.vector(Dev2)*(-1/2),
               method="NSIS")}
     ### Compile Output
     cat("Creating Output\n")
     LaplacesDemon.out <- list(Acceptance.Rate=Acceptance.Rate,
          Algorithm=Algorithm,
          Call=x[[1]]$Call,
          Covar=Covar,
          CovarDHis=x[[len.x]]$CovarDHis,
          Deviance=as.vector(Dev),
          DIC1=c(mean(as.vector(Dev)),
               var(as.vector(Dev))/2,
               mean(as.vector(Dev)) + var(as.vector(Dev))/2),
          DIC2=if(Stat.at < thinned.rows) {
               c(mean(as.vector(Dev2)),
               var(as.vector(Dev2))/2,
               mean(as.vector(Dev2)) + 
               var(as.vector(Dev2))/2)}
               else rep(NA,3),
          Initial.Values=x[[len.x]]$Initial.Values,
          Iterations=Iterations,
          LML=LML[[1]],
          Minutes=Minutes,
          Model=Model,
          Monitor=Mon,
          Parameters=LIV,
          Posterior1=thinned,
          Posterior2=if(Stat.at < thinned.rows) {
               thinned[Stat.at:thinned.rows,]}
               else thinned[thinned.rows,],
          Rec.BurnIn.Thinned=BurnIn,
          Rec.BurnIn.UnThinned=BurnIn*Thinning,
          Rec.Thinning=min(1000, max(Rec.Thin)),
          Specs=x[[1]]$Specs,
          Status=Status,
          Summary1=Summ1,
          Summary2=Summ2,
          Thinned.Samples=thinned.rows,
          Thinning=Thinning)
     class(LaplacesDemon.out) <- "demonoid"
     cat("\nLaplace's Demon has finished.\n")
     return(LaplacesDemon.out)
     }

#End

###########################################################################
# Gelfand.Diagnostic                                                      #
#                                                                         #
# The Gelfand.Diagnostic function is an interpretation of Gelfand's       #
# ``thick felt-tip pen'' MCMC convergence diagnostic (Gelfand et al.,     #
# 1990).                                                                  #
###########################################################################

Gelfand.Diagnostic <- function(x, k=3, pen=FALSE)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     if(!is.vector(x)) x <- as.vector(x)
     if(k < 2) k <- 2
     if(k > length(x)/2) k <- round(length(x)/2)
     if({length(x)/k} < 2) stop("k is too large relative to length(x).")
     ### KDE
     quantiles <- seq(from=0, to=1, by=1/k)
     breaks <- round(as.vector(quantiles)*length(x))
     breaks <- breaks[-1]
     d.temp <- density(x)
     d <- array(c(d.temp$x, d.temp$y), dim=c(length(d.temp$x), 2,
          length(breaks)))
     d.temp <- density(x[1:breaks[1]])
     d[,,1] <- c(d.temp$x, d.temp$y)
     for (i in 2:length(breaks)) {
          d.temp <- density(x[1:breaks[i]])
          d[,,i] <- c(d.temp$x, d.temp$y)}
     ### Plots
     ymax <- max(d[,2,])
     col.list <- c("red", "green", "blue", "yellow", "purple", "orange",
          "brown", "gray", "burlywood", "aquamarine")
     col.list <- rep(col.list, len=length(breaks))
     rgb.temp <- as.vector(col2rgb(col.list[1]))
     mycol <- rgb(red=rgb.temp[1], green=rgb.temp[2], blue=rgb.temp[3],
          alpha=50, maxColorValue=255)
     plot(d[,1,1], d[,2,1], type="l", col=mycol, xlim=c(range(d[,1,])),
          ylim=c(0,ymax), main="Gelfand Diagnostic",
          xlab=deparse(substitute(x)), ylab="Density")
     polygon(x=d[,1,1], y=d[,2,1], col=mycol, border=NULL)
     for (i in 2:length(breaks)) {
          rgb.temp <- as.vector(col2rgb(col.list[i]))
          mycol <- rgb(red=rgb.temp[1], green=rgb.temp[2],
               blue=rgb.temp[3], alpha=50, maxColorValue=255)
          lines(d[,1,i], d[,2,i], col=mycol)
          polygon(x=d[,1,i], y=d[,2,i], col=mycol, border=mycol)
          lines(d[,1,i], d[,2,i], lty=i)}
     if(pen == TRUE) abline(v=mean(range(d[,1,])), col="black", lwd=10)
     legend(quantile(d[,1,], probs=0.025), round(ymax*0.9,2),
          legend=paste("1:",breaks,sep=""), lty=1:k, title="Samples")
     return(invisible(x))
     }

#End

###########################################################################
# plot.juxtapose                                                          #
#                                                                         #
# The purpose of the plot.juxtapose function is to plot a comparison of   #
# MCMC algorithms according either to IAT or ISM.                         #
###########################################################################

plot.juxtapose <- function(x, Style="ISM", ...)
     {
     ### Initial Checks
     if(missing(x)) stop("The x argument is required.")
     if(!identical(class(x), "juxtapose"))
          stop("The x argument is not of class juxtapose.")
     if((Style != "IAT") & (Style != "ISM"))
          stop("Style must be IAT or ISM")
     Title <- "MCMC Juxtaposition"
     if(identical(Style, "IAT")) {
          ### Basic Plot
          plot(0, 0, ylim=c(0, ncol(x) + 1), xlim=c(1, max(x[6,])),
               main=Title, sub="", xlab="Integrated Autocorrelation Time",
               ylab="", type="n", ann=TRUE, yaxt="n")
          abline(v=1, col="gray")
          ### Add Medians
          points(x[5,], ncol(x):1, pch=20)
          ### Add Horizontal Lines for 2.5%-97.5% Quantiles
          for (i in 1:ncol(x)) {
               lines(x[c(4,6),i], c(ncol(x)-i+1, ncol(x)-i+1))}
          ### Add y-axis labels
          yy <- ncol(x):1
          cex.labels <- 1 / {log(ncol(x))/5 + 1}
          axis(2, labels=colnames(x), tick=FALSE, las=1, at=yy,
               cex.axis=cex.labels)}
     else {
          ### Basic Plot
          plot(0, 0, ylim=c(0, ncol(x) + 1), xlim=c(0, max(x[9,])),
               main=Title, sub="", xlab="Independent Samples per Minute",
               ylab="", type="n", ann=TRUE, yaxt="n")
          abline(v=0, col="gray")
          ### Add Medians
          points(x[8,], ncol(x):1, pch=20)
          ### Add Horizontal Lines for 2.5%-97.5% Quantiles
          for (i in 1:ncol(x)) {
               lines(x[c(7,9),i], c(ncol(x)-i+1, ncol(x)-i+1))}
          ### Add y-axis labels
          yy <- ncol(x):1
          cex.labels <- 1 / {log(ncol(x))/5 + 1}
          axis(2, labels=colnames(x), tick=FALSE, las=1, at=yy,
               cex.axis=cex.labels)}
     }

#End

###########################################################################
# LPL.interval                                                            #
#                                                                         #
# The purpose of the LPL.interval function is to estimate the lowest      #
# posterior loss (LPL) interval.                                          #
###########################################################################

LPL.interval <- function(Prior, Posterior, prob=0.95, plot=FALSE,
     PDF=FALSE)
     {
     ### Initial Checks
     if(missing(Prior)) stop("Prior is required.")
     if(missing(Posterior)) stop("Posterior is required.")
     if(!is.vector(Prior)) Prior <- as.vector(Prior)
     if(!is.vector(Posterior)) Posterior <- as.vector(Posterior)
     if(length(Prior) != length(Posterior))
          stop("Length mismatch between Prior and Posterior.")
     if(any(!is.finite(Prior) | !is.finite(Posterior)))
          stop("Non-finite values found in Prior or Posterior.")
     name <- names(Posterior)
     if(is.null(name)) name <- "Value"
     ### Expected Posterior Loss
     ord <- order(Posterior)
     Prior <- Prior[ord]
     Posterior <- Posterior[ord]
     loss <- KLD(Prior, Posterior)[[3]]
     ### Plot Expected Posterior Loss
     if(plot == TRUE) {
          if(PDF == TRUE) pdf("LPL.Plot.pdf")
          par(mfrow=c(2,1))
          plot(Posterior, loss, type="l", main="Posterior Loss",
               xlab=name, ylab="E(Posterior Loss)")
          polygon(c(min(Posterior), Posterior, max(Posterior)),
               c(min(loss), loss, min(loss)),
               col="gray", border="gray")}
     ### Find LPL Interval
     n <- length(loss)
     gap <- max(1, min(n - 1, round(n * prob)))
     loss.sum <- init <- 1:(n - gap)
     for (i in 1:length(init)) {
          loss.sum[i] <- sum(loss[init[i]:(init[i]+gap)])}
     min.init <- init[which.min(loss.sum)]
     ans <- cbind(Posterior[min.init], Posterior[min.init+gap])
     colnames(ans) <- c("Lower","Upper")
     attr(ans, "LPL.Interval") <- prob
     ### Shade LPL Area
     if(plot == TRUE) {
          polygon(c(min(Posterior[min.init]),
               Posterior[min.init:(min.init+gap)],
               max(Posterior[min.init+gap])),
               c(min(loss[min.init:(min.init+gap)]),
               loss[min.init:(min.init+gap)],
               min(loss[min.init:(min.init+gap)])),
               col="black", border="black")
          abline(v=0, col="red", lty=2)
          kde <- kde.low <- kde.high <- density(Posterior)
          kde.low$x <- kde$x[kde$x < ans[1,1]]
          kde.low$y <- kde$y[which(kde$x < ans[1,1])]
          kde.high$x <- kde$x[kde$x > ans[1,2]]
          kde.high$y <- kde$y[which(kde$x > ans[1,2])]
          plot(kde, xlab=name, ylab="Density",
               main="LPL Probability Interval")
          polygon(kde, col="black", border="black")
          polygon(c(min(kde.low$x), kde.low$x, max(kde.low$x)),
               c(min(kde.low$y), kde.low$y, min(kde.low$y)),
               col="gray", border="gray")
          polygon(c(min(kde.high$x), kde.high$x, max(kde.high$x)),
               c(min(kde.high$y), kde.high$y, min(kde.high$y)),
               col="gray", border="gray")
          abline(v=0, col="red", lty=2)
          if(PDF == TRUE) dev.off()}
     return(ans)
     }

#End

###########################################################################
# Geweke.Diagnostic                                                       #
#                                                                         #
# The purpose of the Geweke.Diagnostic function is to estimate            #
# stationarity in samples according to Geweke's diagnostic. Although the  #
# code is slightly different, it is essentially the same as the           #
# geweke.diag function in the coda package.                               #
###########################################################################

Geweke.Diagnostic <- function(x) 
     {
     x <- as.matrix(x)
     if(nrow(x) < 100) return(rep(NA, ncol(x)))
     frac1 <- 0.1; frac2 <- 0.5
     startx <- 1; endx <- nrow(x)
     xstart <- c(startx, endx - frac2 * {endx - startx})
     xend <- c(startx + frac1 * {endx - startx}, endx)
     y.variance <- y.mean <- vector("list", 2)
     for (i in 1:2) {
          y <- x[xstart[i]:xend[i],]
          y.mean[[i]] <- colMeans(as.matrix(y))
          yy <- as.matrix(y)
          y <- as.matrix(y)
          max.freq <- 0.5; order <- 1; max.length <- 200
          if(nrow(yy) > max.length) {
               batch.size <- ceiling(nrow(yy) / max.length)
               yy <- aggregate(ts(yy, frequency=batch.size), nfreq=1, 
                    FUN=mean)}
          else {batch.size <- 1}
          yy <- as.matrix(yy)
          fmla <- switch(order + 1,
               spec ~ one,
               spec ~ f1,
               spec ~ f1 + f2)
          if(is.null(fmla)) stop("Invalid order.")
          N <- nrow(yy)
          Nfreq <- floor(N/2)
          freq <- seq(from=1/N, by=1/N, length=Nfreq)
          f1 <- sqrt(3) * {4 * freq - 1}
          f2 <- sqrt(5) * {24 * freq * freq - 12 * freq + 1}
          v0 <- numeric(ncol(yy))
          for (j in 1:ncol(yy)) {
               zz <- yy[,j]
               if(var(zz) == 0) v0[j] <- 0
               else {
                    yfft <- fft(zz)
                    spec <- Re(yfft * Conj(yfft)) / N
                    spec.data <- data.frame(one=rep(1, Nfreq), f1=f1,
                         f2=f2, spec=spec[1 + {1:Nfreq}],
                         inset=I(freq <= max.freq))
                    glm.out <- try(glm(fmla, family=Gamma(link="log"),
                         data=spec.data), silent=TRUE)
                    if(!inherits(glm.out, "try-error")) 
                         v0[j] <- predict(glm.out, type="response",
                              newdata=data.frame(spec=0, one=1,
                              f1=-sqrt(3), f2=sqrt(5)))
                    }
               }
          spec <- list(spec=v0)
          spec$spec <- spec$spec * batch.size
          y.variance[[i]] <- spec$spec / nrow(y)
          }
     z <- {y.mean[[1]] - y.mean[[2]]} /
          sqrt(y.variance[[1]] + y.variance[[2]])
     return(z)
     }

#End

\name{LaplacesDemon-package}
\alias{LaplacesDemon-package}
\alias{.colVars}
\alias{.iqagh}
\alias{.iqaghsg}
\alias{.iqcagh}
\alias{.laaga}
\alias{.labfgs}
\alias{.labhhh}
\alias{.lacg}
\alias{.ladfp}
\alias{.lahar}
\alias{.lahj}
\alias{.lalbfgs}
\alias{.lalm}
\alias{.lanm}
\alias{.lanr}
\alias{.lapso}
\alias{.larprop}
\alias{.lasgd}
\alias{.lasoma}
\alias{.laspg}
\alias{.lasr1}
\alias{.latr}
\alias{.mcmcadmg}
\alias{.mcmcafss}
\alias{.mcmcagg}
\alias{.mcmcahmc}
\alias{.mcmcaies}
\alias{.mcmcam}
\alias{.mcmcamm}
\alias{.mcmcamm.b}
\alias{.mcmcamwg}
\alias{.mcmccharm}
\alias{.mcmcdemc}
\alias{.mcmcdram}
\alias{.mcmcdrm}
\alias{.mcmcess}
\alias{.mcmcgibbs}
\alias{.mcmcgg}
\alias{.mcmcggcp}
\alias{.mcmcggcpp}
\alias{.mcmcggdp}
\alias{.mcmcggdpp}
\alias{.mcmcharm}
\alias{.mcmchmc}
\alias{.mcmchmcda}
\alias{.mcmcim}
\alias{.mcmcinca}
\alias{.mcmcmala}
\alias{.mcmcmcmcmc}
\alias{.mcmcmtm}
\alias{.mcmcmwg}
\alias{.mcmcnuts}
\alias{.mcmcohss}
\alias{.mcmcram}
\alias{.mcmcrdmh}
\alias{.mcmcrefractive}
\alias{.mcmcrj}
\alias{.mcmcrss}
\alias{.mcmcrwm}
\alias{.mcmcsamwg}
\alias{.mcmcsgld}
\alias{.mcmcslice}
\alias{.mcmcsmwg}
\alias{.mcmcthmc}
\alias{.mcmctwalk}
\alias{.mcmcuess}
\alias{.mcmcusamwg}
\alias{.mcmcusmwg}
\alias{.rowVars}
\alias{.vbsalimans2}
\docType{package}
\title{
\packageTitle{LaplacesDemon}
}
\description{
\packageDescription{LaplacesDemon}
}
\details{

The DESCRIPTION file:
\packageDESCRIPTION{LaplacesDemon}
\packageIndices{LaplacesDemon}

  The goal of LaplacesDemon, often referred to as LD, is to provide a
  complete and self-contained Bayesian environment within R. For
  example, this package includes dozens of MCMC algorithms, Laplace
  Approximation, iterative quadrature, variational Bayes,
  parallelization, big data, PMC, over 100 examples in the ``Examples''
  vignette, dozens of additional probability distributions, numerous
  MCMC diagnostics, Bayes factors, posterior predictive checks, a
  variety of plots, elicitation, parameter and variable importance,
  Bayesian forms of test statistics (such as Durbin-Watson, Jarque-Bera,
  etc.), validation, and numerous additional utility functions, such as
  functions for multimodality, matrices, or timing your model
  specification. Other vignettes include an introduction to Bayesian
  inference, as well as a tutorial.

  No further development of this package is currently being done as the original
  maintainer has stopped working on the package. Contributions to this
  package are welcome at \url{https://github.com/LaplacesDemonR/LaplacesDemon}.

  The main function in this package is the \code{LaplacesDemon}
  function, and the best place to start is probably with the
  LaplacesDemon Tutorial vignette.
  }
\author{
\packageAuthor{LaplacesDemon}

Maintainer: \packageMaintainer{LaplacesDemon}
}
\keyword{package}

\name{plotSamples}
\alias{plotSamples}
\title{Plot Samples}
\description{
  This function provides basic plots that are extended to include
  samples.
}
\usage{
plotSamples(X, Style="KDE", LB=0.025, UB=0.975, Title=NULL)
}
\arguments{
  \item{X}{This required argument is a \eqn{N \times S}{N x S} numerical
    matrix of \eqn{N} records and \eqn{S} samples.}
  \item{Style}{This argument accepts the following quoted strings:
    "barplot", "dotchart", "hist", "KDE", or "Time-Series". It defaults
    to \code{Style="KDE"}.}
  \item{LB}{This argument accepts the lower bound of a probability
    interval, which must be in the interval [0,0.5).}
  \item{UB}{This argument accepts the upper bound of a probability
    interval, which must be in the interval (0.5,1].}
  \item{Title}{This argument defaults to \code{NULL}, and otherwise
    accepts a quoted string that will be the title of the plot.}
}
\details{
  The \code{plotSamples} function extends several basic plots from
  points to samples. For example, it is common to use the \code{hist}
  function to plot a histogram from a column vector. However, the user
  may desire to plot a histogram of a column vector that was sampled
  numerous times, rather than a simple column vector, in which a
  (usually 95\%) probability interval is also plotted to show the
  uncertainty around the sampled median of each bin in the histogram.

  The \code{plotSamples} function extends the \code{barplot},
  \code{dotchart}, and \code{hist} functions to include uncertainty due
  to samples. The \code{KDE} style of plot is added so that a
  probability interval is shown around a sampled kernel density estimate
  of a distribution, and the \code{Time-Series} style of plot is added
  so that a probability interval is shown around a sampled univariate
  time-series.

  For each style of plot, three quantiles are plotted: the lower bound
  (LB), median, and upper bound (UB).

  One of many potential Bayesian applications is to examine the
  uncertainty in a predictive distribution.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\examples{
#library(LaplacesDemon)
#N <- 100
#S <- 100
#X <- matrix(rnorm(N*S),N,S)
#rownames(X) <- 1:100
#plotSamples(X, Style="barplot", LB=0.025, UB=0.975)
#plotSamples(X[1:10,], Style="dotchart", LB=0.025, UB=0.975)
#plotSamples(X, Style="hist", LB=0.025, UB=0.975)
#plotSamples(X, Style="KDE", LB=0.025, UB=0.975)
#plotSamples(X, Style="Time-Series", LB=0.025, UB=0.975)
}
\keyword{Plot}

\name{Importance}
\alias{Importance}
\title{Variable Importance}
\description{
  The \code{Importance} function considers variable importance (or
  predictor importance) to be the effect that the variable has on
  replicates \eqn{\textbf{y}^{rep}}{y^rep} (or
  \eqn{\textbf{Y}^{rep}}{Y^rep}) when the variable is removed from the
  model by setting it equal to zero. Here, variable importance is
  considered in terms of the comparison of posterior predictive
  checks. This may be considered to be a form of sensitivity analysis,
  and can be useful for model revision, variable selection, and model
  interpretation.

  Currently, this function only tests the variable importance of design
  matrix \eqn{\textbf{X}}{X}.
}
\usage{
Importance(object, Model, Data, Categorical=FALSE, Discrep, d=0, CPUs=1,
  Type="PSOCK")
}
\arguments{
  \item{object}{An object of class \code{demonoid}, \code{iterquad},
    \code{laplace}, \code{pmc}, or \code{vb} is required.}
  \item{Model}{The model specification function is required.}
  \item{Data}{A data set in a list is required. The dependent variable
    is required to be named either \code{y} or \code{Y}. The
    \code{Importance} function will sequentially remove each column
    vector in \code{X}, so \code{X} is required to be in data set
    \code{Data}.}
  \item{Categorical}{Logical. If \code{TRUE}, then \code{y} and
    \code{yhat} are considered to be categorical (such as y=0 or y=1),
    rather than continuous. This defaults to \code{FALSE}.}
  \item{Discrep}{This optional argument allows a discrepancy statistic to
    be included. For more information on discrepancy statistics, see
    \code{\link{summary.demonoid.ppc}}.}
  \item{d}{This is an optional integer to be used with the
    \code{Discrep} argument above, and it defaults to \code{d=0}. For
    more information on discrepancy, see
    \code{\link{summary.demonoid.ppc}}.}
  \item{CPUs}{This argument accepts an integer that specifies the number
    of central processing units (CPUs) of the multicore computer or
    computer cluster. This argument defaults to \code{CPUs=1}, in which
    parallel processing does not occur.}
  \item{Type}{This argument specifies the type of parallel processing to
    perform, accepting either \code{Type="PSOCK"} or
    \code{Type="MPI"}.}
}
\details{Variable importance is defined here as the impact of each
  variable (predictor, or column vector) in design matrix
  \eqn{\textbf{X}}{X} on \eqn{\textbf{y}^{rep}}{y^rep} (or
  \eqn{\textbf{Y}^{rep}}{Y^rep}), when the variable is removed.

  First, the full model is predicted with the
  \code{\link{predict.demonoid}}, \code{\link{predict.iterquad}},
  \code{\link{predict.laplace}}, \code{\link{predict.pmc}}, or
  \code{\link{predict.vb}} function, and summarized with the
  \code{\link{summary.demonoid.ppc}},
  \code{\link{summary.iterquad.ppc}}, \code{\link{summary.laplace.ppc}},
  \code{\link{summary.pmc.ppc}}, or \code{\link{summary.vb.ppc}}
  function, respectively. The results are stored in the first row of the
  output. Each successive row in the output corresponds to the
  application of \code{predict} and \code{summary} functions, but with
  each variable in design matrix \eqn{\textbf{X}}{X} being set to zero
  and effectively removed. The results show the impact of sequentially
  removing each predictor.

  The criterion for variable importance may differ from model to
  model. As a default, BPIC is recommended. The Bayesian Predictive
  Information Criterion (BPIC) was introduced by Ando (2007). BPIC is a
  variation of the Deviance Information Criterion (DIC) that has been
  modified for predictive distributions. For more information on DIC
  (Spiegelhalter et al., 2002), see the accompanying vignette entitled
  "Bayesian Inference". \eqn{BPIC = Dbar + 2pD}.

  With BPIC, variable importance has a positive relationship, such that
  larger values indicate a more important variable, because removing
  that variable resulted in a worse fit to the data. The best model
  has the lowest BPIC.

  In a model in which the dependent variable is not categorical, it is
  also recommended to consider the L-criterion (Laud and Ibrahim, 1995),
  provided that sample size is small enough that it does not result in
  \code{Inf}. For more information on the L-criterion, see the
  accompanying vignette entitled "Bayesian Inference".

  With the L-criterion, variable importance has a positive relationship, 
  such that larger values indicate a more important variable, because
  removing that variable resulted in a worse fit to the data. Ibrahim
  (1995) recommended considering the model with the lowest
  L-criterion, say as \eqn{L_1}{L[1]}, and the model with the closest
  L-criterion, say as \eqn{L_2}{L[2]}, and creating a comparison score
  as \eqn{\phi = (L_2-L_1)/S_L}{phi = (L[2]-L[1])/S[L]}, where
  \code{S.L} is from the \eqn{L_1}{L[1]} model. If the comparison score,
  \eqn{\phi}{phi} is less than 2, then \eqn{L_2}{L[2]} is within 2
  standard deviations of \eqn{L_1}{L[1]}, and is the recommended cut-off
  for model choice.

  The \code{Importance} function may suggest that a model fits the data
  better with a variable removed. In which case, the user may
  choose to leave the variable in the model (perhaps the model is
  misspecified without the variable), investigate and possibly
  re-specify the relationship between the independent and dependent
  variable(s), or remove the variable and update the model again.

  In contrast to variable importance, the \code{\link{PosteriorChecks}}
  function calculates parameter importance, which is the probability
  that each parameter's marginal posterior distribution is greater than
  zero, where an important parameter does not include zero in its
  probability interval (see \code{\link{p.interval}}). Parameter
  importance and variable importance may disagree, and both should be
  studied.

  The \code{Importance} function tends to indicate that a model fits the
  data better when variables are removed that have parameters with
  marginal posterior distributions that include 0 in the 95\%
  probability interval (variables associated with lower parameter
  importance).

  Often, in complicated models, it is difficult to assess variable
  importance by examining the marginal posterior distribution of the
  associated parameter(s). Consider polynomial regression, in which each
  variable may have multiple parameters.

  The information provided by the \code{Importance} function may be used
  for model revision, or reporting the relative importance of variables.

  The \code{\link{plot.importance}} function is available to plot the
  output of the \code{Importance} function according to BPIC, predictive
  concordance (Gelfand, 1996), the selected discrepancy statistic
  (Gelman et al., 1996), or the L-criterion.

  Parallel processing may be performed when the user specifies
  \code{CPUs} to be greater than one, implying that the specified number
  of CPUs exists and is available. Parallelization may be performed on a
  multicore computer or a computer cluster. Either a Simple Network of
  Workstations (SNOW) or Message Passing Interface is used (MPI). With
  small data sets and few samples, parallel processing may be slower,
  due to computer network communication. With larger data sets and more
  samples, the user should experience a faster run-time.
}
\value{
  \code{Importance} returns an object of class \code{importance}, which
  is a matrix with a number of rows equal to the number of columns in
  design matrix \eqn{\textbf{X}}{X} + 1 (including the full model), and
  4 columns, which are BPIC, Concordance (or Mean.Lift if categorical),
  Discrep, and L-criterion. Each row represents a model with a predictor
  in \eqn{\textbf{X}}{X} removed (except for the first row, which is the
  full model), and the resulting posterior predictive checks. For
  non-categorical dependent variables, an attribute is returned with the
  object, and the attribute is a vector of \code{S.L}, the calibration
  number of the L-criterion.
}
\references{
  Ando, T. (2007). "Bayesian Predictive Information Criterion for
  the Evaluation of Hierarchical Bayesian and Empirical Bayes Models".
  \emph{Biometrika}, 94(2), p. 443--458.

  Gelfand, A. (1996). "Model Determination Using Sampling Based
  Methods". In Gilks, W., Richardson, S., Spiegehalter, D., Chapter 9 in
  Markov Chain Monte Carlo in Practice. Chapman and Hall: Boca Raton, FL.

  Laud, P.W. and Ibrahim, J.G. (1995). "Predictive Model
  Selection". \emph{Journal of the Royal Statistical Society}, B 57,
  p. 247--262.

  Spiegelhalter, D.J., Best, N.G., Carlin, B.P., and van der Linde, A.
  (2002). "Bayesian Measures of Model Complexity and Fit (with
  Discussion)". \emph{Journal of the Royal Statistical Society}, B 64,
  p. 583--639.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{is.importance}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{PMC}},
  \code{\link{plot.importance}},
  \code{\link{PosteriorChecks}},
  \code{\link{p.interval}},
  \code{\link{predict.demonoid}},
  \code{\link{predict.iterquad}},
  \code{\link{predict.laplace}},
  \code{\link{predict.pmc}},
  \code{\link{predict.vb}},
  \code{\link{summary.demonoid.ppc}},
  \code{\link{summary.iterquad.ppc}},
  \code{\link{summary.laplace.ppc}},
  \code{\link{summary.pmc.ppc}},
  \code{\link{summary.vb.ppc}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
#First, update the model with the LaplacesDemon function, such as
#the example with linear regression, creating an object called Fit.
#Then
#Importance(Fit, Model, MyData, Discrep="Chi-Square", CPUs=1)
}
\keyword{BPIC}
\keyword{High Performance Computing}
\keyword{L-criterion}
\keyword{Model Selection}
\keyword{Posterior Predictive Checks}

\name{as.initial.values}
\alias{as.initial.values}
\title{Initial Values}
\description{
  This function returns the most recent posterior samples from an object
  of class \code{demonoid} or \code{demonoid.hpc}, the posterior means
  of an object of class \code{iterquad}, the posterior modes of an
  object of class \code{laplace} or \code{vb}, the posterior means of an
  object of class \code{pmc} with one mixture component, or the latest
  means of the importance sampling distribution of an object of class
  \code{pmc} with multiple mixture components. The returned values are
  intended to be the initial values for future updates.
}
\usage{
as.initial.values(x)
}
\arguments{
  \item{x}{This is an object of class \code{demonoid},
    \code{demonoid.hpc}, \code{iterquad}, \code{laplace}, \code{pmc}, or
    \code{vb}.}
}
\details{
  Unless it is known beforehand how many iterations are required for
  \code{\link{IterativeQuadrature}}, \code{\link{LaplaceApproximation}},
  or \code{\link{VariationalBayes}} to converge, MCMC in
  \code{\link{LaplacesDemon}} to appear converged, or the normalized
  perplexity to stabilize in \code{\link{PMC}}, multiple updates are
  necessary. An additional update, however, should not begin with the
  same initial values as the original update, because it will have to
  repeat the work already accomplished. For this reason, the
  \code{as.initial.values} function may be used at the end of an update
  to change the previous initial values to the latest values.

  When using \code{\link{LaplacesDemon.hpc}}, \code{as.initial.values}
  should be used when the output is of class \code{demonoid.hpc}, before
  the \code{\link{Combine}} function is used to combine the multiple
  chains for use with \code{\link{Consort}} and other functions, because
  the \code{\link{Combine}} function returns an object of class
  \code{demonoid}, and the number of chains will become unknown. The
  \code{\link{Consort}} function may suggest using
  \code{as.initial.values}, but when applied to an object of class
  \code{demonoid}, it will return the latest values as if there were
  only one chain.
}
\value{
  The returned value is a vector (or matrix in the case of an object of
  class \code{demonoid.hpc}, or \code{pmc} with multiple mixture
  components) of the latest values, which may now be used as initial
  values for a future update.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{Combine}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{LaplacesDemon.hpc}},
  \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\keyword{Initial Values}

\name{dist.Laplace}
\alias{dlaplace}
\alias{plaplace}
\alias{qlaplace}
\alias{rlaplace}
\title{Laplace Distribution: Univariate Symmetric}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the univariate, symmetric, Laplace
  distribution with location parameter \eqn{\mu}{mu} and scale
  parameter \eqn{\lambda}{\lambda}.
}
\usage{
dlaplace(x, location=0, scale=1, log=FALSE)
plaplace(q, location=0, scale=1)
qlaplace(p, location=0, scale=1)
rlaplace(n, location=0, scale=1)
}
\arguments{
  \item{x, q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{location}{This is the location parameter \eqn{\mu}{mu}.}
  \item{scale}{This is the scale parameter \eqn{\lambda}{lambda}, which
    must be positive.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density: \eqn{p(\theta) = \frac{1}{2 \lambda} \exp(-\frac{|\theta - \mu|}{\lambda})}{p(theta) = (1 / (2*lambda)) * exp(-(abs(theta - mu) / lambda))}
  \item Inventor: Pierre-Simon Laplace (1774)
  \item Notation 1: \eqn{\theta \sim
    \mathrm{Laplace}(\mu,\lambda)}{theta ~ Laplace(mu, lambda)}
  \item Notation 2: \eqn{\theta \sim \mathcal{L}(\mu, \lambda)}{theta ~
    L(mu, lambda)}
  \item Notation 3: \eqn{p(\theta) = \mathrm{Laplace}(\theta | \mu,
    \lambda)}{p(theta) = Laplace(theta | mu, lambda)}
  \item Notation 4: \eqn{p(\theta) = \mathcal{L}(\theta | \mu,
    \lambda)}{p(theta) = L(theta | mu, lambda)}
  \item Parameter 1: location parameter \eqn{\mu}{mu}
  \item Parameter 2: scale parameter \eqn{\lambda > 0}{lambda > 0}
  \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}
  \item Variance: \eqn{var(\theta) = 2 \lambda^2}{var(theta) = 2*lambda^2}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The Laplace distribution (Laplace, 1774) is also called the double
exponential distribution, because it looks like two exponential
distributions back to back with respect to location \eqn{\mu}{mu}. It is
also called the ``First Law of Laplace'', just as the normal
distribution is referred to as the ``Second Law of Laplace''. The
Laplace distribution is symmetric with respect to \eqn{\mu}{mu}, though
there are asymmetric versions of the Laplace distribution. The PDF of
the Laplace distribution is reminiscent of the normal distribution;
however, whereas the normal distribution is expressed in terms of the
squared difference from the mean \eqn{\mu}{mu}, the Laplace density is
expressed in terms of the absolute difference from the mean,
\eqn{\mu}{mu}. Consequently, the Laplace distribution has fatter
tails than the normal distribution. It has been argued that the Laplace
distribution fits most things in nature better than the normal
distribution.

There are many extensions to the Laplace distribution, such as the
asymmetric Laplace, asymmetric log-Laplace, Laplace (re-parameterized
for precision), log-Laplace, multivariate Laplace, and skew-Laplace,
among many more.

These functions are similar to those in the \code{VGAM} package.
}
\value{
  \code{dlaplace} gives the density,
  \code{plaplace} gives the distribution function,
  \code{qlaplace} gives the quantile function, and
  \code{rlaplace} generates random deviates.
}
\references{
  Laplace, P. (1774). "Memoire sur la Probabilite des Causes par les
  Evenements." l'Academie Royale des Sciences, 6, 621--656. English
  translation by S.M. Stigler in 1986 as "Memoir on the Probability
  of the Causes of Events" in \emph{Statistical Science}, 1(3),
  p. 359--378.
}
\seealso{
  \code{\link{dalaplace}},
  \code{\link{dallaplace}},
  \code{\link{dexp}},
  \code{\link{dlaplacep}},
  \code{\link{dllaplace}},
  \code{\link{dmvl}},
  \code{\link{dnorm}},
  \code{\link{dnormp}},
  \code{\link{dnormv}},
  \code{\link{dsdlaplace}}, and
  \code{\link{dslaplace}}.
}
\examples{
library(LaplacesDemon)
x <- dlaplace(1,0,1)
x <- plaplace(1,0,1)
x <- qlaplace(0.5,0,1)
x <- rlaplace(100,0,1)

#Plot Probability Functions
x <- seq(from=-5, to=5, by=0.1)
plot(x, dlaplace(x,0,0.5), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dlaplace(x,0,1), type="l", col="green")
lines(x, dlaplace(x,0,2), type="l", col="blue")
legend(2, 0.9, expression(paste(mu==0, ", ", lambda==0.5),
     paste(mu==0, ", ", lambda==1), paste(mu==0, ", ", lambda==2)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{dist.Multivariate.Normal}
\alias{dmvn}
\alias{rmvn}
\title{Multivariate Normal Distribution}
\description{
  These functions provide the density and random number generation for
  the multivariate normal distribution.
}
\usage{
dmvn(x, mu, Sigma, log=FALSE) 
rmvn(n=1, mu, Sigma)
}
\arguments{
  \item{x}{This is data or parameters in the form of a vector of length
       \eqn{k} or a matrix with \eqn{k} columns.}
  \item{n}{This is the number of random draws.}
  \item{mu}{This is mean vector \eqn{\mu}{mu} with length \eqn{k} or
       matrix with \eqn{k} columns.}
  \item{Sigma}{This is the \eqn{k \times k}{k x k} covariance matrix
       \eqn{\Sigma}{Sigma}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
       density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Multivariate
  \item Density: \eqn{p(\theta) = \frac{1}{(2\pi)^{k/2}|\Sigma|^{1/2}}
    \exp(-\frac{1}{2}(\theta - \mu)'\Sigma^{-1}(\theta - \mu))}{p(theta)
    = (1/((2*pi)^(k/2)*|Sigma|^(1/2))) *
    exp(-(1/2)*(theta-mu)'*Sigma^(-1)*(theta-mu))}
  \item Inventors: Robert Adrain (1808), Pierre-Simon Laplace (1812), and Francis Galton (1885)
  \item Notation 1: \eqn{\theta \sim \mathcal{MVN}(\mu, \Sigma)}{theta ~ MVN(mu, Sigma)}
  \item Notation 2: \eqn{\theta \sim \mathcal{N}_k(\mu, \Sigma)}{theta ~ N[k](mu, Sigma)}
  \item Notation 3: \eqn{p(\theta) = \mathcal{MVN}(\theta | \mu, \Sigma)}{p(theta) = MVN(theta | mu, Sigma)}
  \item Notation 4: \eqn{p(\theta) = \mathcal{N}_k(\theta | \mu, \Sigma)}{p(theta) = N[k](theta | mu, Sigma)}
  \item Parameter 1: location vector \eqn{\mu}{mu}
  \item Parameter 2: positive-definite \eqn{k \times k}{k x k} covariance matrix \eqn{\Sigma}{Sigma}
  \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}
  \item Variance: \eqn{var(\theta) = \Sigma}{var(theta) = Sigma}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The multivariate normal distribution, or multivariate Gaussian
distribution, is a multidimensional extension of the one-dimensional
or univariate normal (or Gaussian) distribution. A random vector is
considered to be multivariate normally distributed if every linear
combination of its components has a univariate normal distribution.
This distribution has a mean parameter vector \eqn{\mu}{mu} of length
\eqn{k} and a \eqn{k \times k}{k x k} covariance matrix
\eqn{\Sigma}{Sigma}, which must be positive-definite.

The conjugate prior of the mean vector is another multivariate normal
distribution. The conjugate prior of the covariance matrix is the
inverse Wishart distribution (see \code{\link{dinvwishart}}).

When applicable, the alternative Cholesky parameterization should be
preferred. For more information, see \code{\link{dmvnc}}.

For models where the dependent variable, Y, is specified to be
distributed multivariate normal given the model, the Mardia test (see
\code{\link{plot.demonoid.ppc}}, \code{\link{plot.laplace.ppc}}, or
\code{\link{plot.pmc.ppc}}) may be used to test the residuals.
}
\value{
  \code{dmvn} gives the density and 
  \code{rmvn} generates random deviates.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dinvwishart}},
  \code{\link{dmatrixnorm}},
  \code{\link{dmvnc}},
  \code{\link{dmvnp}},
  \code{\link{dnorm}},
  \code{\link{dnormp}},
  \code{\link{dnormv}},
  \code{\link{plot.demonoid.ppc}},
  \code{\link{plot.laplace.ppc}}, and
  \code{\link{plot.pmc.ppc}}.
}
\examples{
library(LaplacesDemon)
x <- dmvn(c(1,2,3), c(0,1,2), diag(3))
X <- rmvn(1000, c(0,1,2), diag(3))
joint.density.plot(X[,1], X[,2], color=TRUE)
}
\keyword{Distribution}

\name{dist.HalfNormal}
\alias{dhalfnorm}
\alias{phalfnorm}
\alias{qhalfnorm}
\alias{rhalfnorm}
\title{Half-Normal Distribution}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the half-normal distribution.
}
\usage{
dhalfnorm(x, scale=sqrt(pi/2), log=FALSE)
phalfnorm(q, scale=sqrt(pi/2), lower.tail=TRUE, log.p=FALSE)
qhalfnorm(p, scale=sqrt(pi/2), lower.tail=TRUE, log.p=FALSE)
rhalfnorm(n, scale=sqrt(pi/2))
}
\arguments{
  \item{x,q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{scale}{This is the scale parameter \eqn{\sigma}{sigma}, which
    must be positive.}
  \item{log,log.p}{Logical. If \code{log=TRUE}, then the logarithm of the
    density or result is returned.}
  \item{lower.tail}{Logical. If \code{lower.tail=TRUE} (default),
    probabilities are \eqn{Pr[X \le x]}{Pr[X <= x]}, otherwise,
    \eqn{Pr[X > x]}{Pr[X > x]}.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density: \eqn{p(\theta) = \frac{2 \sigma}{\pi}
    \exp(-\frac{\theta^2 \sigma^2}{\pi}), \quad \theta \ge 0}{p(theta) = 2*sigma/pi e^-(theta^2*sigma^2/pi), theta >= 0}
  \item Inventor: Derived from the normal or Gaussian
  \item Notation 1: \eqn{\theta \sim \mathcal{HN}(\sigma)}{theta ~ HALF-N(sigma)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{HN}(\theta | \sigma)}{p(theta) = HN(theta | sigma)}
  \item Parameter 1: scale parameter \eqn{\sigma > 0}{sigma > 0}
  \item Mean: \eqn{E(\theta) = \frac{1}{\sigma}}{E(theta) = 1 / sigma}
  \item Variance: \eqn{var(\theta) = \frac{\pi-2}{2 \sigma^2}}{var(theta) = (pi-2)/(2*sigma^2)}
  \item Mode: \eqn{mode(\theta) = 0}{mode(theta) = 0}
}

The half-normal distribution is recommended as a weakly informative prior
distribution for a scale parameter that may be useful as an alternative
to the half-Cauchy, half-t, or vague gamma.
}
\value{
  \code{dhalfnorm} gives the density,
  \code{phalfnorm} gives the distribution function,
  \code{qhalfnorm} gives the quantile function, and
  \code{rhalfnorm} generates random deviates.
}
\seealso{
  \code{\link{dnorm}},
  \code{\link{dnormp}}, and
  \code{\link{dnormv}}.
}
\examples{
library(LaplacesDemon)
x <- dhalfnorm(1)
x <- phalfnorm(1)
x <- qhalfnorm(0.5)
x <- rhalfnorm(10)

#Plot Probability Functions
x <- seq(from=0.1, to=20, by=0.1)
plot(x, dhalfnorm(x,0.1), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dhalfnorm(x,0.5), type="l", col="green")
lines(x, dhalfnorm(x,1), type="l", col="blue")
legend(2, 0.9, expression(sigma==0.1, sigma==0.5, sigma==1),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{print.iterquad}
\alias{print.iterquad}
\title{Print an object of class \code{iterquad} to the screen}
\description{
  This may be used to print the contents of an object of class
  \code{iterquad} to the screen.
}
\usage{\method{print}{iterquad}(x, \dots)}
\arguments{
  \item{x}{An object of class \code{iterquad} is required.}
  \item{\dots}{Additional arguments are unused.}
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{\code{\link{IterativeQuadrature}}}
\examples{### See the IterativeQuadrature function for an example.}
\keyword{print}

\name{dist.Multivariate.Normal.Precision}
\alias{dmvnp}
\alias{rmvnp}
\title{Multivariate Normal Distribution: Precision Parameterization}
\description{
  These functions provide the density and random number generation for
  the multivariate normal distribution, given the precision
  parameterization.
}
\usage{
dmvnp(x, mu, Omega, log=FALSE) 
rmvnp(n=1, mu, Omega)
}
\arguments{
  \item{x}{This is data or parameters in the form of a vector of length
       \eqn{k} or a matrix with \eqn{k} columns.}
  \item{n}{This is the number of random draws.}
  \item{mu}{This is mean vector \eqn{\mu}{mu} with length \eqn{k} or
       matrix with \eqn{k} columns.}
  \item{Omega}{This is the \eqn{k \times k}{k x k} precision matrix
       \eqn{\Omega}{Omega}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
       density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Multivariate
  \item Density: \eqn{p(\theta) = (2\pi)^{-p/2} |\Omega|^{1/2}
    \exp(-\frac{1}{2} (\theta-\mu)^T \Omega (\theta-\mu))}{p(theta) = (2*pi)^(-p/2) * |Omega|^(1/2) * exp(-(1/2)*(theta-mu)^t Omega (theta-mu))}
  \item Inventor: Unknown (to me, anyway)
  \item Notation 1: \eqn{\theta \sim \mathcal{MVN}(\mu, \Omega^{-1})}{theta ~ MVN(mu, Omega^(-1))}
  \item Notation 2: \eqn{\theta \sim \mathcal{N}_k(\mu, \Omega^{-1})}{theta ~ N[k](mu, Omega^(-1))}
  \item Notation 3: \eqn{p(\theta) = \mathcal{MVN}(\theta | \mu, \Omega^{-1})}{p(theta) = MVN(theta | mu, Omega^(-1))}
  \item Notation 4: \eqn{p(\theta) = \mathcal{N}_k(\theta | \mu, \Omega^{-1})}{p(theta) = N[k](theta | mu, Omega^(-1))}
  \item Parameter 1: location vector \eqn{\mu}{mu}
  \item Parameter 2: positive-definite \eqn{k \times k}{k x k} precision matrix \eqn{\Omega}{Omega}
  \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}
  \item Variance: \eqn{var(\theta) = \Omega^{-1}}{var(theta) = Omega^(-1)}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The multivariate normal distribution, or multivariate Gaussian
distribution, is a multidimensional extension of the one-dimensional
or univariate normal (or Gaussian) distribution. It is usually
parameterized with mean and a covariance matrix, or in Bayesian
inference, with mean and a precision matrix, where the precision matrix
is the matrix inverse of the covariance matrix. These functions
provide the precision parameterization for convenience and
familiarity. It is easier to calculate a multivariate normal density
with the precision parameterization, because a matrix inversion can be
avoided.

A random vector is considered to be multivariate normally distributed
if every linear combination of its components has a univariate normal
distribution. This distribution has a mean parameter vector
\eqn{\mu}{mu} of length \eqn{k} and a \eqn{k \times k}{k x k}
precision matrix \eqn{\Omega}{Omega}, which must be positive-definite.

The conjugate prior of the mean vector is another multivariate normal
distribution. The conjugate prior of the precision matrix is the
Wishart distribution (see \code{\link{dwishart}}).

When applicable, the alternative Cholesky parameterization should be
preferred. For more information, see \code{\link{dmvnpc}}.

For models where the dependent variable, Y, is specified to be
distributed multivariate normal given the model, the Mardia test (see
\code{\link{plot.demonoid.ppc}}, \code{\link{plot.laplace.ppc}}, or
\code{\link{plot.pmc.ppc}}) may be used to test the residuals.
}
\value{
  \code{dmvnp} gives the density and 
  \code{rmvnp} generates random deviates.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dmvn}},
  \code{\link{dmvnc}},
  \code{\link{dmvnpc}},
  \code{\link{dnorm}},
  \code{\link{dnormp}},
  \code{\link{dnormv}},
  \code{\link{dwishart}},
  \code{\link{plot.demonoid.ppc}},
  \code{\link{plot.laplace.ppc}}, and
  \code{\link{plot.pmc.ppc}}.
}
\examples{
library(LaplacesDemon)
x <- dmvnp(c(1,2,3), c(0,1,2), diag(3))
X <- rmvnp(1000, c(0,1,2), diag(3))
joint.density.plot(X[,1], X[,2], color=TRUE)
}
\keyword{Distribution}

\name{MCSE}
\alias{MCSE}
\alias{MCSS}
\title{Monte Carlo Standard Error}
\description{
  Monte Carlo Standard Error (MCSE) is an estimate of the inaccuracy of
  Monte Carlo samples, usually regarding the expectation of posterior
  samples, \eqn{\mathrm{E}(\theta)}{E(theta)}, from Monte Carlo or
  Markov chain Monte Carlo (MCMC) algorithms, such as with the
  \code{\link{LaplacesDemon}} or \code{\link{LaplacesDemon.hpc}}
  functions. MCSE approaches zero as the number of independent posterior
  samples approaches infinity. MCSE is essentially a standard deviation
  around the posterior mean of the samples,
  \eqn{\mathrm{E}(\theta)}{E(theta)}, due to uncertainty associated with
  using an MCMC algorithm, or Monte Carlo methods in general.

  The acceptable size of the MCSE depends on the acceptable uncertainty
  associated around the marginal posterior mean,
  \eqn{\mathrm{E}(\theta)}{E(theta)}, and the goal of inference. It has
  been argued that MCSE is generally unimportant when the goal of
  inference is \eqn{\theta}{theta} rather than
  \eqn{\mathrm{E}(\theta)}{E(theta)} (Gelman et al., 2004, p. 277), and
  that a sufficient \code{\link{ESS}} is more important. Others perceive
  MCSE to be a vital part of reporting any Bayesian model, and as a
  stopping rule (Flegal et al., 2008).

  In \code{\link{LaplacesDemon}}, MCSE is part of the posterior
  summaries because it is easy to estimate, and Laplace's Demon prefers
  to continue updating until each MCSE is less than 6.27\% of its
  associated marginal posterior standard deviation (for more information
  on this stopping rule, see the \code{\link{Consort}} function), since
  MCSE has been demonstrated to be an excellent stopping rule.

  Acceptable error may be specified, if known, in the \code{MCSS}
  (Monte Carlo Sample Size) function to estimate the required number of
  posterior samples.

  \code{MCSE} is a univariate function that is often applied to each
  marginal posterior distribution. A multivariate form is not
  included. By chance alone due to multiple independent tests, 5\% of
  the parameters should indicate unacceptable MSCEs, even when
  acceptable. Assessing convergence is difficult.
}
\usage{
MCSE(x, method="IMPS", batch.size="sqrt", warn=FALSE)
MCSS(x, a)
}
\arguments{
  \item{x}{This is a vector of posterior samples for which MCSE or MCSS
    will be estimated.}
  \item{a}{This is a scalar argument of acceptable error for the mean of
    \code{x}, and \code{a} must be positive. As acceptable error
    decreases, the required number of samples increases.}
  \item{method}{This is an optional argument for the method of MCSE
    estimation, and defaults to Geyer's \code{"IMPS"} method. Optional
    methods include \code{"sample.variance"} and \code{"batch.mean"}.
    Note that \code{"batch.mean"} is recommended only when the number of
    posterior samples is at least 1,000.}
  \item{batch.size}{This is an optional argument that corresponds only
    with \code{method="batch.means"}, and determines either the size of
    the batches (accepting a numerical argument) or the method of
    creating the size of batches, which is either \code{"sqrt"} or
    \code{"cuberoot"}, and refers to the length of \code{x}. The default
    argument is \code{"sqrt"}.}
  \item{warn}{Logical. If \code{warn=TRUE}, then a warning is provided
    with \code{method="batch.means"} whenever posterior sample size is
    less than 1,000, or a warning is produced when more autcovariance
    is recommended with \code{method="IMPS"}.}
}
\details{
  The default method for estimating MCSE is Geyer's Initial Monotone
  Positive Sequence (IMPS) estimator (Geyer, 1992), which takes the
  asymptotic variance into account and is time-series based. This method
  goes by other names, such as Initial Positive Sequence (IPS).

  The simplest method for estimating MCSE is to modify the formula for
  standard error, \eqn{\sigma(\textbf{x}) / \sqrt{N}}{sigma(x) /
  sqrt(N)}, to account for non-independence in the sequence
  \eqn{\textbf{x}}{x} of posterior samples. Non-independence is
  estimated with the \code{ESS} function for Effective Sample Size (see
  the \code{\link{ESS}} function for more details), where \eqn{M =
  ESS(\textbf{x})}{m = ESS(x)}, and MCSE is
  \eqn{\sigma(\textbf{x}) / \sqrt{M}}{sigma(x) / sqrt(M)}. Although this
  is the fastest and easiest method of estimation, it does not
  incorporate an estimate of the asymptotic variance of
  \eqn{\textbf{x}}{x}.

  The batch means method (Jones et al., 2006; Flegal et al., 2008)
  separates elements of \eqn{\textbf{x}}{x} into batches and estimates
  MCSE as a function of multiple batches. This method is excellent, but
  is not recommended when the number of posterior samples is less than
  1,000. These journal articles also assert that MCSE is a better
  stopping rule than MCMC convergence diagnostics.

  The \code{MCSS} function estimates the required number of posterior
  samples, given the user-specified acceptable error, posterior samples
  \code{x}, and the observed variance (rather than asymptotic
  variance). Due to the observed variance, this is a rough estimate.
}
\references{
  Flegal, J.M., Haran, M., and Jones, G.L. (2008). "Markov chain Monte
  Carlo: Can We Trust the Third Significant Figure?". \emph{Statistical
  Science}, 23, p. 250--260.

  Gelman, A., Carlin, J., Stern, H., and Rubin, D. (2004). "Bayesian
  Data Analysis, Texts in Statistical Science, 2nd ed.". Chapman and
  Hall, London.

  Geyer, C.J. (1992). "Practical Markov Chain Monte Carlo".
  \emph{Statistical Science}, 7, 4, p. 473--483.
  
  Jones, G.L., Haran, M., Caffo, B.S., and Neath, R. (2006). "Fixed-Width
  Output Analysis for Markov chain Monte Carlo". \emph{Journal of the
  American Statistical Association}, 101(1), p. 1537--1547.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{Consort}},
  \code{\link{ESS}},
  \code{\link{LaplacesDemon}}, and
  \code{\link{LaplacesDemon.hpc}}.
}
\examples{
library(LaplacesDemon)
x <- rnorm(1000)
MCSE(x)
MCSE(x, method="batch.means")
MCSE(x, method="sample.variance")
MCSS(x, a=0.01)
}
\keyword{Diagnostic}
\keyword{MCMC}
\keyword{MCSE}
\keyword{MCSS}
\keyword{Utility}

\name{Consort}
\alias{Consort}
\title{Consort with Laplace's Demon}
\description{
  This may be used to consort with Laplace's Demon regarding an object
  of class \code{demonoid}. Laplace's Demon will offer suggestions.
}
\usage{Consort(object)}
\arguments{
  \item{object}{This required argument is an object of class
    \code{demonoid}. For more information, see the
    \code{\link{LaplacesDemon}} function.}
}
\details{
  First, \code{Consort} calls \code{print.demonoid}, which prints most
  of the components to the screen from the supplied object of class
  \code{demonoid}.

  Second, Laplace's Demon considers a combination of five conditions
  when making the largest part of its suggestion. These conditions are:
  the algorithm, acceptance rate, MCSE, ESS, and stationarity. Other
  things are considered as well, such as the recommended thinning value
  is used to suggest a new number of iterations, how fast the algorithm
  is expected to be, and if the condition of diminishing adaptation
  (also called the vanishing adaptation condition) was met (for an
  adaptive algorithm). Diminishing adaptation occurs only when the
  absolute value of the proposed variances trends downward (toward zero)
  over the course of all adaptations. When an algorithm is adaptive and
  it does not have diminishing adaptations, the \code{Consort} function
  will suggest a different adaptive algorithm. The \code{Periodicity}
  argument is suggested to be set equal to the value of
  \code{Rec.Thinning}.

  Appeasement applies only when all parameters are continuous.The
  \code{\link{Hangartner.Diagnostic}} should be considered for discrete
  parameters.

  Appeasement Conditions
  \itemize{
    \item Algorithm: The final algorithm must be non-adaptive, so that
    the Markov property holds. This is conservative. A user may have
    an adaptive (non-final) algorithm in which adaptations in the latest
    update are stationary, or no longer diminishing. Laplace's Demon is
    unaware of previous updates, and conservatively interprets this as
    failing to meet the condition of diminishing adaptation, when the
    output may be satisfactory. On the other hand, if the adaptive
    algorithm has essentially stopped adapting, and if there is a
    non-adaptive version, then the user should consider switching to
    the non-adaptive algorithm. User discretion is advised.
    \item Acceptance Rate: The acceptance rate is considered
    satisfactory if it is within the interval [15\%,50\%] for most
    algorithms. Some algorithms have different recommended intervals.
    \item MCSE: The Monte Carlo Standard Error (MCSE) is considered
    satisfactory for each target distribution if it is less than 6.27\%
    of the standard deviation of the target distribution. This allows
    the true mean to be within 5\% of the area under a Gaussian
    distribution around the estimated mean. The \code{\link{MCSE}}
    function is used. Toft et al. (2007) propose a stricter criterion of
    5\%. The criterion of 6.27\% for this stopping rule is arbitrary,
    and may be too lenient or strict, depending on the needs of the
    user. Nonetheless, it has performed well, and this type of stopping
    rule has been observed to perform better than MCMC convergence
    diagnostics (Flegal et al., 2008).
    \item ESS: The effective sample size (ESS) is considered
    satisfactory for each target distribution if it is at least 100,
    which is usually enough to describe 95\% probability intervals (see
    \code{\link{p.interval}} and \code{\link{LPL.interval}} for more
    information). The \code{\link{ESS}} function is used. When this
    criterion is unmet, the name of the worst mixing chain in Summary1
    appears.
    \item Stationarity: Each target distribution is considered
    satisfactory if it is estimated to be stationary with the
    \code{\link{BMK.Diagnostic}} function.
}
  Bear in mind that the MCSE, ESS, and stationarity criteria are all
  univariate measures applied to each marginal posterior
  distribution. Multivariate forms are not included. By chance alone due
  to multiple independent tests, 5\% of these diagnostics should
  indicate non-convergence when 'convergence' exists. In contrast, even
  one non-convergent nuisance parameter is associated with
  non-convergence in all other parameters. Assessing convergence is
  difficult.

  If all five conditions are satisfactory, then Laplace's Demon is
  appeased. Otherwise, Laplace's Demon will suggest and supply R
  code that is ready to be copy/pasted and executed.

  To visualize the MCSE-based stopping rule, run the following code:

  \code{x <- seq(from=-3, to=3, by=0.1);}
  \code{plot(x, dnorm(x,0,1), type="l");}
  \code{abline(v=-0.0627); abline(v=0.0627);}
  \code{abline(v=2*-0.0627, col="red"); abline(v=2*0.0627, col="red")}

  The black vertical lines show the standard error, and the red vertical
  lines show the 95\% interval.

  If the user has an object of class \code{demonoid.hpc}, then the
  \code{Consort} function may be still be applied, but a particular
  chain in the object must be specified as a component in a list. For
  example, with an object called \code{Fit} and a goal of consorting
  over the second chain, the code would be: \code{Consort(Fit[[2]])}.

  The Demonic Suggestion is usually very helpful, but should not be
  followed blindly. Do not let it replace critical thinking. For
  example, \code{Consort} may find that diminishing adaptation is unmet,
  and recommend a different algorithm. However, the user may be
  convinced that the current algorithm is best, and believe instead that
  MCMC found a local solution, and is leaving it to find the global
  solution, in which case adaptations may increase again. Diminishing
  adaptation may have occurred in a previous run, and is not found in
  the current run because adaptation is essentially finished. If either
  of these is true, then it may be best to ignore the newly suggested
  algorithm, and continue with the current algorithm. The suggested code
  may be helpful, but it is merely a suggestion.

  If achieving the appeasement of Laplace's Demon is difficult, consider
  ignoring the MCSE criterion and terminate when all other criteria have
  been met, placing special emphasis on ESS.
}
\references{
  Flegal, J.M., Haran, M., and Jones, G.L. (2008). "Markov chain Monte
  Carlo: Can We Trust the Third Significant Figure?". \emph{Statistical
  Science}, 23, p. 250--260.

  Toft, N., Innocent, G., Gettinby, G., and Reid, S. (2007). "Assessing
  the Convergence of Markov Chain Monte Carlo Methods: An Example from
  Evaluation of Diagnostic Tests in Absence of a Gold Standard".
  \emph{Preventive Veterinary Medicine}, 79, p. 244--256.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{BMK.Diagnostic}},
  \code{\link{ESS}},
  \code{\link{Hangartner.Diagnostic}},
  \code{\link{LaplacesDemon}},
  \code{\link{LaplacesDemon.hpc}},
  \code{\link{LPL.interval}},
  \code{\link{MCSE}}, and
  \code{\link{p.interval}}.
}
\keyword{Diminishing Adaptation}
\keyword{MCMC}

\name{dist.Multivariate.t.Cholesky}
\alias{dmvtc}
\alias{rmvtc}
\title{Multivariate t Distribution: Cholesky Parameterization}
\description{
  These functions provide the density and random number generation for
  the multivariate t distribution, otherwise called the multivariate
  Student distribution, given the Cholesky parameterization.
}
\usage{
dmvtc(x, mu, U, df=Inf, log=FALSE)
rmvtc(n=1, mu, U, df=Inf)
}
\arguments{
  \item{x}{This is either a vector of length \eqn{k} or a matrix with
    a number of columns, \eqn{k}, equal to the number of columns in
    scale matrix \eqn{\textbf{S}}{S}.}
  \item{n}{This is the number of random draws.}
  \item{mu}{This is a numeric vector or matrix representing the location
    parameter,\eqn{\mu}{mu} (the mean vector), of the multivariate
    distribution (equal to the expected value when \code{df > 1},
    otherwise represented as \eqn{\nu > 1}{nu > 1}). When a vector, it
    must be of length \eqn{k}, or must have \eqn{k} columns as a matrix,
    as defined above.}
  \item{U}{This is the \eqn{k \times k}{k x k} upper-triangular matrix
    that is Cholesky factor \eqn{\textbf{U}}{U} of scale matrix
    \eqn{\textbf{S}}{S}, such that \code{S*df/(df-2)} is the
    variance-covariance matrix when \code{df > 2}.}
  \item{df}{This is the degrees of freedom, and is often represented
    with \eqn{\nu}{nu}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Multivariate
  \item Density: \deqn{p(\theta) =
    \frac{\Gamma[(\nu+k)/2]}{\Gamma(\nu/2)\nu^{k/2}\pi^{k/2}|\Sigma|^{1/2}[1
      + (1/\nu)(\theta-\mu)^{\mathrm{T}} \Sigma^{-1}
      (\theta-\mu)]^{(\nu+k)/2}}}{p(theta) = Gamma[(nu+k)/2] /
    {Gamma(nu/2)nu^(k/2)pi^(k/2)|Sigma|^(1/2)[1 +
      (1/nu)(theta-mu)^T*Sigma^(-1)(theta-mu)]^[(nu+k)/2]}}
  \item Inventor: Unknown (to me, anyway)
  \item Notation 1: \eqn{\theta \sim \mathrm{t}_k(\mu,
    \Sigma, \nu)}{theta ~ t[k](mu, Sigma, nu)}
  \item Notation 2: \eqn{p(\theta) = \mathrm{t}_k(\theta | \mu,
    \Sigma, \nu)}{p(theta) = t[k](theta | mu, Sigma, nu)}
  \item Parameter 1: location vector \eqn{\mu}{mu}
  \item Parameter 2: positive-definite \eqn{k \times k}{k x k} scale
    matrix \eqn{\Sigma}{Sigma}
  \item Parameter 3: degrees of freedom \eqn{\nu > 0}{nu > 0} (df in the
    functions)
  \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}, for \eqn{\nu > 1}{nu
    > 1}, otherwise undefined
  \item Variance: \eqn{var(\theta) = \frac{\nu}{\nu - 2}
    \Sigma}{var(theta) = (nu / (nu - 2))*Sigma}, for \eqn{\nu > 2}{nu > 2}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The multivariate t distribution, also called the multivariate Student or
multivariate Student t distribution, is a multidimensional extension of the
one-dimensional or univariate Student t distribution. A random vector is
considered to be multivariate t-distributed if every linear
combination of its components has a univariate Student t-distribution.
This distribution has a mean parameter vector \eqn{\mu}{mu} of length
\eqn{k}, and an upper-triangular \eqn{k \times k}{k x k} matrix that is
Cholesky factor \eqn{\textbf{U}}{U}, as per the \code{\link{chol}}
function for Cholesky decomposition. When degrees of freedom
\eqn{\nu=1}{nu=1}, this is the multivariate Cauchy distribution.

In practice, \eqn{\textbf{U}}{U} is fully unconstrained for proposals
when its diagonal is log-transformed. The diagonal is exponentiated
after a proposal and before other calculations. Overall, the Cholesky
parameterization is faster than the traditional parameterization.
Compared with \code{dmvt}, \code{dmvtc} must additionally
matrix-multiply the Cholesky back to the scale matrix, but it
does not have to check for or correct the scale matrix to
positive-definiteness, which overall is slower. The same is true when
comparing \code{rmvt} and \code{rmvtc}.
}
\value{
  \code{dmvtc} gives the density and 
  \code{rmvtc} generates random deviates.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{chol}},
  \code{\link{dinvwishartc}},
  \code{\link{dmvc}},
  \code{\link{dmvcp}},
  \code{\link{dmvtp}},
  \code{\link{dst}},
  \code{\link{dstp}}, and
  \code{\link{dt}}.
}
\examples{
library(LaplacesDemon)
x <- seq(-2,4,length=21)
y <- 2*x+10
z <- x+cos(y) 
mu <- c(1,12,2)
S <- matrix(c(1,2,0,2,5,0.5,0,0.5,3), 3, 3)
U <- chol(S)
df <- 4
f <- dmvtc(cbind(x,y,z), mu, U, df)
X <- rmvtc(1000, c(0,1,2), U, 5)
joint.density.plot(X[,1], X[,2], color=TRUE)
}
\keyword{Distribution}

\name{SensitivityAnalysis}
\alias{SensitivityAnalysis}
\title{Sensitivity Analysis}
\description{
  This function performs an elementary sensitivity analysis for two
  models regarding marginal posterior distributions and posterior
  inferences.
}
\usage{
SensitivityAnalysis(Fit1, Fit2, Pred1, Pred2)
}
\arguments{
  \item{Fit1}{This argument accepts an object of class \code{demonoid},
    \code{iterquad}, \code{laplace}, \code{pmc}, or \code{vb}.}
  \item{Fit2}{This argument accepts an object of class \code{demonoid},
    \code{iterquad}, \code{laplace}, \code{pmc}, or \code{vb}.}
  \item{Pred1}{This argument accepts an object of class
    \code{demonoid.ppc}, \code{iterquad.ppc}, \code{laplace.ppc},
    \code{pmc.ppc}, or \code{vb.ppc}.}
  \item{Pred2}{This argument accepts an object of class
    \code{demonoid.ppc}, \code{iterquad.ppc}, \code{laplace.ppc},
    \code{pmc.ppc}, or \code{vb.ppc}.}
}
\details{
  Sensitivity analysis is concerned with the influence from changes to the
  inputs of a model on the output. Comparing differences resulting from
  different prior distributions is the most common application of
  sensitivity analysis, though results from different likelihoods may be
  compared as well. The outputs of interest are the marginal posterior
  distributions and posterior inferences.

  There are many more methods of conducting a sensitivity analysis than
  exist in the \code{SensitivityAnalysis} function. For more
  information, see Oakley and O'Hagan (2004). The \code{\link{SIR}}
  function is useful for approximating changes in the posterior due to
  small changes in prior distributions.

  The \code{SensitivityAnalysis} function compares marginal posterior
  distributions and posterior predictive distributions. Specifically,
  it calculates the probability that each distribution in \code{Fit1}
  and \code{Pred1} is greater than the associated distribution in
  \code{Fit2} and \code{Pred2}, and returns a variance ratio of each
  pair of distributions. If the probability is \eqn{0.5} that a
  distribution is greater than another, or if the variance ratio is
  \eqn{1}, then no difference is found due to the inputs.

  Additional comparisons and methods are currently outside the scope of
  the \code{SensitivityAnalysis} function. The \code{\link{BayesFactor}}
  function may also be considered, as well as comparing posterior
  predictive checks resulting from \code{\link{summary.demonoid.ppc}},
  \code{\link{summary.iterquad.ppc}},
  \code{\link{summary.laplace.ppc}}, \code{\link{summary.pmc.ppc}}, or
  \code{\link{summary.vb.ppc}}.

  Regarding marginal posterior distributions, the
  \code{SensitivityAnalysis} function compares only distributions with
  identical parameter names. For example, suppose a statistician
  conducts a sensitivity analysis to study differences resulting from
  two prior distributions: a normal distribution and a Student t
  distribution. These distributions have two and three parameters,
  respectively. The statistician has named the parameters \code{beta}
  and \code{sigma} for the normal distribution, while for the Student
  t distribution, the parameters are named \code{beta}, \code{sigma},
  and \code{nu}. In this case, the \code{SensitivityAnalysis} function
  compares the marginal posterior distributions for \code{beta} and
  \code{sigma}, though \code{nu} is ignored because it is not in both
  models. If the statistician does not want certain parameters compared,
  then differing parameter names should be assigned.

  Robust Bayesian analysis is a very similar topic, and often called
  simply Bayesian sensitivity analysis. In robust Bayesian analysis, the
  robustness of answers from a Bayesian analysis to uncertainty about
  the precise details of the analysis is studied. An answer is
  considered robust if it does not depend sensitively on the assumptions
  and inputs on which it is based. Robust Bayes methods acknowledge that
  it is sometimes very difficult to come up with precise distributions
  to be used as priors. Likewise the appropriate likelihood function
  that should be used for a particular problem may also be in doubt. In
  a robust Bayesian analysis, a standard Bayesian analysis is applied to
  all possible combinations of prior distributions and likelihood
  functions selected from classes of priors and likelihoods considered
  empirically plausible by the statistician.
}
\value{
  This function returns a list with the following components:
  \item{Posterior}{This is a \eqn{J \times 2}{J x 2} matrix of \eqn{J}
    marginal posterior distributions. Column names are "p(Fit1 > Fit2)"
    and "var(Fit1) / var(Fit2)".}
  \item{Post.Pred.Dist}{This is a \eqn{N \times 2}{N x 2} matrix of \eqn{N}
    posterior predictive distributions. Column names are
    "p(Pred1 > Pred2)" and "var(Pred1) / var(Pred2)".}
}
\references{
  Berger, J.O. (1984). "The Robust Bayesian Viewpoint (with
  discussion)". In J. B. Kadane, editor, Robustness of Bayesian
  Analyses, p. 63--144. North-Holland, Amsterdam.

  Berger, J.O. (1985). "Statistical Decision Theory and Bayesian
  Analysis". Springer-Verlag, New York.

  Berger, J.O. (1994). "An Overview of Robust Bayesian Analysis
  (with discussion)". Test, 3, p. 5--124.
  
  Oakley, J. and O'Hagan, A. (2004). "Probabilistic Sensitivity Analysis
  of Complex Models: a Bayesian Approach". \emph{Journal of the Royal
  Statistical Society, Series B}, 66, p. 751--769.

  Weiss, R. (1995). "An Approach to Bayesian Sensitivity Analysis".
  \emph{Journal of the Royal Statistical Society, Series B}, 58,
  p. 739--750.
}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{BayesFactor}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{PMC}},
  \code{\link{predict.demonoid}},
  \code{\link{predict.iterquad}},
  \code{\link{predict.laplace}},
  \code{\link{predict.pmc}},
  \code{\link{SIR}},
  \code{\link{summary.demonoid.ppc}},
  \code{\link{summary.iterquad.ppc}},
  \code{\link{summary.laplace.ppc}},
  \code{\link{summary.pmc.ppc}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
#sa <- SensitivityAnalysis(Fit1, Fit2, Pred1, Pred2)
#sa
}
\keyword{Sensitivity}

\name{dist.Laplace.Mixture}
\alias{dlaplacem}
\alias{plaplacem}
\alias{rlaplacem}
\title{Mixture of Laplace Distributions}
\description{
  These functions provide the density, cumulative, and random generation
  for the mixture of univariate Laplace distributions with probability
  \eqn{p}, location \eqn{\mu}{mu} and scale \eqn{\sigma}{sigma}.
}
\usage{
dlaplacem(x, p, location, scale, log=FALSE)
plaplacem(q, p, location, scale)
rlaplacem(n, p, location, scale)
}
\arguments{
  \item{x,q}{This is vector of values at which the density will be evaluated.}
  \item{p}{This is a vector of length \eqn{M} of probabilities for \eqn{M}
    components. The sum of the vector must be one.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{location}{This is a vector of length \eqn{M} that is the location
    parameter \eqn{\mu}{mu}.}
  \item{scale}{This is a vector of length \eqn{M} that is the scale
    parameter \eqn{\sigma}{sigma}, which must be positive.}
  \item{log}{Logical. If \code{TRUE}, then the logarithm of the density is
    returned.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density: \eqn{p(\theta) = \sum p_i \mathcal{L}(\mu_i,
    \sigma_i)}{p(theta) = sum p[i] L(mu[i], sigma[i])}
  \item Inventor: Unknown
  \item Notation 1: \eqn{\theta \sim \mathcal{L}(\mu, \sigma)}{theta ~ L(mu, sigma)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{L}(\theta | \mu,
    \sigma)}{p(theta) = L(theta | mu, sigma)}
  \item Parameter 1: location parameters \eqn{\mu}{mu}
  \item Parameter 2: scale parameters \eqn{\sigma > 0}{sigma > 0}
  \item Mean: \eqn{E(\theta) = \sum p_i \mu_i}{E(theta) = sum p[i] mu[i]}
  \item Variance: 
  \item Mode: 
}

A mixture distribution is a probability distribution that is a
combination of other probability distributions, and each distribution is
called a mixture component, or component. A probability (or weight)
exists for each component, and these probabilities sum to one. A
mixture distribution (though not these functions here in particular) may
contain mixture components in which each component is a different
probability distribution. Mixture distributions are very flexible, and
are often used to represent a complex distribution with an unknown
form. When the number of mixture components is unknown, Bayesian
inference is the only sensible approach to estimation.

A Laplace mixture distribution is a combination of Laplace probability
distributions.

One of many applications of Laplace mixture distributions is the Laplace
Mixture Model (LMM).
}
\value{
  \code{dlaplacem} gives the density,
  \code{plaplacem} returns the CDF, and
  \code{rlaplacem} generates random deviates.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{ddirichlet}} and
  \code{\link{dlaplace}}.
}
\examples{
library(LaplacesDemon)
p <- c(0.3,0.3,0.4)
mu <- c(-5, 1, 5)
sigma <- c(1,2,1)
x <- seq(from=-10, to=10, by=0.1)
plot(x, dlaplacem(x, p, mu, sigma, log=FALSE), type="l") #Density
plot(x, plaplacem(x, p, mu, sigma), type="l") #CDF
plot(density(rlaplacem(10000, p, mu, sigma))) #Random Deviates
}
\keyword{Distribution}

\name{dist.Multivariate.Polya}
\alias{dmvpolya}
\alias{rmvpolya}
\title{Multivariate Polya Distribution}
\description{
  These functions provide the density and random number generation for
  the multivariate Polya distribution.
}
\usage{
dmvpolya(x, alpha, log=FALSE)
rmvpolya(n, alpha)
}
\arguments{
  \item{x}{This is data or parameters in the form of a vector of length
    \eqn{k}.}
  \item{n}{This is the number of random draws to take from the
    distribution.}
  \item{alpha}{This is shape vector \eqn{\alpha}{alpha} with length
    \eqn{k}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
       density is returned.}
}
\details{
  \itemize{
  \item Application: Discrete Multivariate
  \item Density: \deqn{p(\theta) = \frac{N!}{\prod_k N_k!} \frac{(\sum_k
      \alpha_k - 1)!}{(\sum_k \theta_k + \sum_k \alpha_k - 1)!}
    \frac{\prod (\theta + \alpha - 1)!}{(\alpha - 1)!}}{p(theta) = (N! /
    prod(N[k]!)) * ((sum alpha[k] - 1)! / (sum theta[k] + sum alpha[k] -
    1)!) * prod((theta + alpha - 1)! / (alpha - 1)!)}
  \item Inventor: George Polya (1887-1985)
  \item Notation 1: \eqn{\theta \sim \mathcal{MPO}(\alpha)}{theta ~ MPO(alpha)}
  \item Notation 3: \eqn{p(\theta) = \mathcal{MPO}(\theta |
    \alpha)}{p(theta) = MPO(theta | alpha)}
  \item Parameter 1: shape parameter vector \eqn{\alpha}{alpha}
  \item Mean: \eqn{E(\theta) = }{E(theta) = }
  \item Variance: \eqn{var(\theta) =}{var(theta) = }
  \item Mode: \eqn{mode(\theta) = }{mode(theta) = }
}

The multivariate Polya distribution is named after George Polya
(1887-1985). It is also called the Dirichlet compound multinomial
distribution or the Dirichlet-multinomial distribution. The multivariate
Polya distribution is a compound probability distribution, where a
probability vector \eqn{p} is drawn from a Dirichlet distribution with
parameter vector \eqn{\alpha}{alpha}, and a set of \eqn{N} discrete
samples is drawn from the categorical distribution with probability
vector \eqn{p} and having \eqn{K} discrete categories. The compounding
corresponds to a Polya urn scheme. In document classification, for
example, the distribution is used to represent probabilities over word
counts for different document types. The multivariate Polya distribution
is a multivariate extension of the univariate Beta-binomial distribution.
}
\value{
  \code{dmvpolya} gives the density and \code{rmvpolya} generates random
  deviates.
}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dcat}},
  \code{\link{ddirichlet}}, and
  \code{\link{dmultinom}}.
}
\examples{
library(LaplacesDemon)
dmvpolya(x=1:3, alpha=1:3, log=TRUE)
x <- rmvpolya(1000, c(0.1,0.3,0.6))
}
\keyword{Distribution}

\name{plotMatrix}
\alias{plotMatrix}
\title{Plot a Numerical Matrix}
\description{
  This function plots a numerical matrix, and is often used to plot the
  following matrices: correlation, covariance, distance, and
  precision.
}
\usage{
plotMatrix(x, col=colorRampPalette(c("red","black","green"))(100),
     cex=1, circle=TRUE, order=FALSE, zlim=NULL, title="", PDF=FALSE, \dots)
}
\arguments{
  \item{x}{This required argument is a numerical matrix, or an
    object of class \code{bayesfactor}, \code{demonoid}, \code{iterquad},
    \code{laplace}, \code{pmc}, \code{posteriorchecks}, or \code{vb}.
    See more information below regarding these classes. One component of
    a blocked proposal covariance matrix must be pointed to explicitly,
    rather than to the object of class \code{demonoid}.}
  \item{col}{This argument specifies the colors of the circles. By
    default, the \code{colorRampPalette} function colors strong positive
    correlation as \code{green}, zero correlation as \code{black}, and
    strong negative correlation as \code{red}, and provides 100 color
    gradations.}
  \item{cex}{When \code{circle=TRUE}, this argument specifies the size
    of the marginal text, the names of the parameters or variables, and
    defaults to 1.}
  \item{circle}{Logical. When \code{TRUE}, each element in the numeric
    matrix is represented with a circle, and a larger circle is assigned
    to elements that are farther from zero. Also, when \code{TRUE}, the
    gradation scale does not appear to the right of the plot.}
  \item{order}{Logical. This argument defaults to \code{FALSE}, and
    presents the parameters or variables in the same order as in the
    numeric matrix. When \code{TRUE}, the parameters or variables
    are ordered using principal components analysis.}
  \item{zlim}{When \code{circle=FALSE}, the gradation scale may be
    constrained to an interval by \code{zlim}, such as
    \code{zlim=c(-1,1)}, and only values within the interval are
    plotted.}
  \item{title}{This argument specifies the title of the plot, and the
    default does not include a title. When \code{x} is of class
    \code{posteriorchecks}, the title is changed to \code{Posterior
      Correlation}.}
  \item{PDF}{Logical. When \code{TRUE}, the plot is saved as a .pdf
    file.}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  The \code{plotMatrix} function produces one of two styles of plots,
  depending on the \code{circle} argument. A \eqn{K \times K}{K x K}
  numeric matrix of \eqn{K} parameters or variables is plotted. The plot
  is a matrix of the same dimensions, in which each element is colored
  (and sized, when \code{circle=TRUE}) according to its value.

  Although \code{plotMatrix} does not provide the same detail as a
  numeric matrix, it is easier to discover elements of interest
  according to color (and size when \code{circle=TRUE}).
  
  The \code{plotMatrix} function is not inherently Bayesian, and does
  not include uncertainty in matrices. Nonetheless, it is included
  because it is a useful graphical presentation of a numeric matrices,
  and is recommended to be used with the posterior correlation matrix in
  an object of class \code{posteriorchecks}.

  When \code{x} is an object of class \code{bayesfactor}, matrix
  \code{B} is plotted. When \code{x} is an object of class
  \code{demonoid} (if it is a matrix), \code{iterquad}, \code{laplace},
  \code{pmc}, or \code{vb}, the covariance matrix \code{Covar} is
  plotted. When \code{x} is an object of class \code{posteriorchecks},
  the posterior correlation matrix is plotted.

  This is a modified version of the \code{circle.corr} function
  of Taiyun Wei.
}
\author{Taiyun Wei}
\seealso{\code{\link{PosteriorChecks}}}
\examples{
library(LaplacesDemon)
### Although it is most commonly used with an object of class
### posteriorchecks, it is applied here to a different correlation matrix.
data(mtcars)
plotMatrix(cor(mtcars), col=colorRampPalette(c("green","gray10","red"))(100),
     cex=1, circle=FALSE, order=TRUE)
plotMatrix(cor(mtcars), col=colorRampPalette(c("green","gray10","red"))(100),
     cex=1, circle=TRUE, order=TRUE)}
\keyword{Plot}

\name{PMC.RAM}
\alias{PMC.RAM}
\title{PMC RAM Estimate}
\description{
  This function estimates the random-access memory (RAM) required to
  update a given model and data with \code{\link{PMC}}.

  \emph{Warning:} Unwise use of this function may crash a computer, so
  please read the details below.
}
\usage{
PMC.RAM(Model, Data, Iterations, Thinning, M, N)
}
\arguments{
  \item{Model}{This is a model specification function. For more
    information, see \code{\link{PMC}}.}
  \item{Data}{This is a list of Data. For more information, see
    \code{\link{PMC}}.}
  \item{Iterations}{This is the number of iterations for which
    \code{\link{PMC}} would update. For more information,
    see \code{\link{PMC}}.}
  \item{Thinning}{This is the amount of thinning applied to the samples
    in \code{\link{PMC}}.For more information, see \code{\link{PMC}}.}
  \item{M}{This is the number of mixture components in
    \code{\link{PMC}}.}
  \item{N}{This is the number of samples in \code{\link{PMC}}.}
}
\details{
  The \code{PMC.RAM} function uses the
  \code{\link{object.size}} function to estimate the size in MB of RAM
  required to update in \code{\link{PMC}} for a given model and data,
  and for a number of iterations and specified thinning. When RAM is
  exceeded, the computer will crash. This function can be useful when
  trying to estimate how many samples and iterations to update a
  model without crashing the computer. However, when estimating the
  required RAM, \code{PMC.RAM} actually creates several large
  objects, such as \code{post} (see below). If too many iterations are
  given as an argument to \code{PMC.RAM}, for example, then it
  will crash the computer while trying to estimate the required RAM.

  The best way to use this function is as follows. First, prepare the
  model specification and list of data. Second, observe how much RAM the
  computer is using at the moment, as well as the maximum available
  RAM. The majority of the difference of these two is the amount of RAM
  the computer may dedicate to updating the model. Next, use this
  function with a small number of iterations. Note the estimated
  RAM. Increase the number of iterations, and again note the
  RAM. Continue to increase the number of iterations until, say,
  arbitrarily within 90\% of the above-mentioned difference in RAM.

  The computer operating system uses RAM, as does any other software
  running at the moment. R is currently using RAM, and other functions
  in the \code{LaplacesDemon} package, and any other package that is
  currently activated, are using RAM. There are numerous small objects
  that are not included in the returned list, that use RAM. For example,
  perplexity is a small vector, etc.

  A potentially large objects that is not included is a matrix used for
  estimating \code{\link{LML}}.
}
\value{
  \code{PMC.RAM} returns a list with several components. Each component
  is an estimate in MB for an object. The list has the following
  components:
  \item{alpha}{This is the estimated size in MB of RAM required for the
  matrix of mixture probabilities by iteration.}
  \item{Covar}{This is the estimated size in MB of RAM required for the
    covariance matrix or matrices.}
  \item{Data}{This is the estimated size in MB of RAM required for the
    list of data.}
  \item{Deviance}{This is the estimated size in MB of RAM required for
    the deviance vector before thinning.}
  \item{Initial.Values}{This is the estimated size in MB of RAM required
    for the matrix or vector of initial values.}
  \item{LH}{This is the estimated size in MB of RAM required for the
    \eqn{N \times T \times M}{N x T x M} array \code{LH}, where \eqn{N}
    is the number of samples, \eqn{T} is the number of iterations, and
    \eqn{M} is the number of mixture components. The \code{LH} array is
    not returned by \code{\link{PMC}}.}
  \item{LP}{This is the estimated size in MB of RAM required for the
    \eqn{N \times T \times M}{N x T x M} array \code{LP}, where \eqn{N}
    is the number of samples, \eqn{T} is the number of iterations, and
    \eqn{M} is the number of mixture components. The \code{LP} array is
    not returned by \code{\link{PMC}}.}
  \item{Model}{This is the estimated size in MB of RAM required for the
    model specification function.}
  \item{Monitor}{This is the estimated size in MB of RAM required for
    the \eqn{N \times J}{N x J} matrix \code{Monitor}, where \eqn{N} is
    the number of unthinned samples and J is the number of monitored
    variables. Although it is thinned later in the algorithm, the full
    matrix is created.}
  \item{Posterior1}{This is the estimated size in MB of RAM required for
    the \eqn{N \times J \times T \times M}{N x J x T x M} array
    \code{Posterior1}, where \eqn{N} is the number of samples, \eqn{J}
    is the number of parameters, \eqn{T} is the number of iterations,
    and \eqn{M} is the number of mixture components.}
  \item{Posterior2}{This is the estimated size in MB of RAM required for
    the \eqn{N \times J}{N x J} matrix \code{Posterior2}, where \eqn{N}
    is the number of samples and \eqn{J} is the number of initial values
    or parameters. Although this is thinned later, at one point it is
    un-thinned.}
  \item{Summary}{This is the estimated size in MB of RAM required for
    the summary table.}
  \item{W}{This is the estimated size in MB of RAM required for the
    matrix of importance weights.}
  \item{Total}{This is the estimated size in MB of RAM required in total
    to update with \code{\link{PMC}} for a given model and data, and for
    a number of iterations, specified thinning, mixture components, and
    number of samples.}
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{BigData}},
  \code{\link{LML}},
  \code{\link{object.size}}, and
  \code{\link{PMC}}.
}
\keyword{Memory}

\name{KLD}
\alias{KLD}
\title{Kullback-Leibler Divergence (KLD)}
\description{
  This function calculates the Kullback-Leibler divergence (KLD) between
  two probability distributions, and has many uses, such as in lowest
  posterior loss probability intervals, posterior predictive checks,
  prior elicitation, reference priors, and Variational Bayes.
}
\usage{
KLD(px, py, base)
}
\arguments{
   \item{px}{This is a required vector of probability densities,
     considered as \eqn{p(\textbf{x})}{p(x)}. Log-densities are also
     accepted, in which case both \code{px} and \code{py} must be
     log-densities.}
   \item{py}{This is a required vector of probability densities,
     considered as \eqn{p(\textbf{y})}{p(y)}. Log-densities are also
     accepted, in which case both \code{px} and \code{py} must be
     log-densities.}
   \item{base}{This optional argument specifies the logarithmic base,
     which defaults to \code{base=exp(1)} (or \eqn{e}) and represents
     information in natural units (nats), where \code{base=2} represents
     information in binary units (bits).}
}
\details{
  The Kullback-Leibler divergence (KLD) is known by many names, some of
  which are Kullback-Leibler distance, K-L, and logarithmic divergence.
  KLD is an asymmetric measure of the difference, distance, or direct
  divergence between two probability distributions
  \eqn{p(\textbf{y})}{p(y)} and \eqn{p(\textbf{x})}{p(x)} (Kullback and
  Leibler, 1951). Mathematically, however, KLD is not a distance,
  because of its asymmetry.
  
  Here, \eqn{p(\textbf{y})}{p(y)} represents the
  ``true'' distribution of data, observations, or theoretical
  distribution, and \eqn{p(\textbf{x})}{p(x)} represents a theory,
  model, or approximation of \eqn{p(\textbf{y})}{p(y)}.

  For probability distributions \eqn{p(\textbf{y})}{p(y)} and
  \eqn{p(\textbf{x})}{p(x)} that are discrete (whether the underlying
  distribution is continuous or discrete, the observations themselves
  are always discrete, such as from \eqn{i=1,\dots,N}{i=1,...,N}),

  \deqn{\mathrm{KLD}[p(\textbf{y}) || p(\textbf{x})] = \sum^N_i
  p(\textbf{y}_i)
  \log\frac{p(\textbf{y}_i)}{p(\textbf{x}_i)}}{KLD[p(y)||p(x)] = sum of
  p(y[i]) log(p(y[i]) / p(x[i]))}

  In Bayesian inference, KLD can be used as a measure of the information
  gain in moving from a prior distribution, \eqn{p(\theta)}{p(theta)},
  to a posterior distribution, \eqn{p(\theta | \textbf{y})}{p(theta |
  y)}. As such, KLD is the basis of reference priors and lowest
  posterior loss intervals (\code{\link{LPL.interval}}), such as in
  Berger, Bernardo, and Sun (2009) and Bernardo (2005). The intrinsic
  discrepancy was introduced by Bernardo and Rueda (2002). For more
  information on the intrinsic discrepancy, see
  \code{\link{LPL.interval}}.
}
\value{
  \code{KLD} returns a list with the following components:
  \item{KLD.px.py}{This is \eqn{\mathrm{KLD}_i[p(\textbf{x}_i) ||
      p(\textbf{y}_i)]}{KLD[i](p(x[i]) || p(y[i]))}.}
  \item{KLD.py.px}{This is \eqn{\mathrm{KLD}_i[p(\textbf{y}_i) ||
      p(\textbf{x}_i)]}{KLD[i](p(y[i]) || p(x[i]))}.}
  \item{mean.KLD}{This is the mean of the two components above. This is
      the expected posterior loss in \code{\link{LPL.interval}}.}
  \item{sum.KLD.px.py}{This is \eqn{\mathrm{KLD}[p(\textbf{x}) ||
      p(\textbf{y})]}{KLD(p(x) || p(y))}. This is a directed
      divergence.}
  \item{sum.KLD.py.px}{This is \eqn{\mathrm{KLD}[p(\textbf{y}) ||
      p(\textbf{x})]}{KLD(p(y) || p(x))}. This is a directed divergence.}
  \item{mean.sum.KLD}{This is the mean of the two components above.}
  \item{intrinsic.discrepancy}{This is minimum of the two directed
      divergences.}
}
\references{
  Berger, J.O., Bernardo, J.M., and Sun, D. (2009). "The Formal
  Definition of Reference Priors". \emph{The Annals of Statistics},
  37(2), p. 905--938.

  Bernardo, J.M. and Rueda, R. (2002). "Bayesian Hypothesis Testing: A
  Reference Approach". \emph{International Statistical Review}, 70,
  p. 351--372.

  Bernardo, J.M. (2005). "Intrinsic Credible Regions: An Objective
  Bayesian Approach to Interval Estimation". \emph{Sociedad de
  Estadistica e Investigacion Operativa}, 14(2), p. 317--384.

  Kullback, S. and Leibler, R.A. (1951). "On Information and
  Sufficiency". \emph{The Annals of Mathematical Statistics}, 22(1),
  p. 79--86.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{LPL.interval}} and
  \code{\link{VariationalBayes}}.
}
\examples{
library(LaplacesDemon)
px <- dnorm(runif(100),0,1)
py <- dnorm(runif(100),0.1,0.9)
KLD(px,py)
}
\keyword{Distribution}
\keyword{Elicitation}
\keyword{Reference Priors}

\name{dist.Wishart.Cholesky}
\alias{dwishartc}
\alias{rwishartc}
\title{Wishart Distribution: Cholesky Parameterization}
\description{
  These functions provide the density and random number generation
  for the Wishart distribution with the Cholesky parameterization.
}
\usage{
   dwishartc(U, nu, S, log=FALSE)
   rwishartc(nu, S)
}
\arguments{
  \item{U}{This is the upper-triangular \eqn{k \times k}{k x k} matrix
    for the Cholesky factor \eqn{\textbf{U}}{U} of precision matrix
    \eqn{\Omega}{Omega}.}
  \item{nu}{This is the scalar degrees of freedom \eqn{\nu}{nu}.}
  \item{S}{This is the symmetric, positive-semidefinite, \eqn{k \times
      k}{k x k} scale matrix \eqn{\textbf{S}}{S}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Multivariate
    \item Density: \eqn{p(\theta) = (2^{\nu k/2} \pi^{k(k-1)/4}
      \prod^k_{i=1} \Gamma(\frac{\nu+1-i}{2}))^{-1} |\textbf{S}|^{-nu/2}
      |\Omega|^{(nu-k-1)/2} \exp(-\frac{1}{2} tr(\textbf{S}^{-1}
      \Omega))}{p(theta) = (2^(nu*k/2) * pi^(k(k-1)/4) *
      [Gamma((nu+1-i)/2) * ... * Gamma((nu+1-k)/2)])^(-1) * |S|^(-nu/2)
      * |Omega|^((nu-k-1)/2) * exp(-(1/2) * tr(S^(-1) Omega))}
    \item Inventor: John Wishart (1928)
    \item Notation 1: \eqn{\Omega \sim
      \mathcal{W}_{\nu}(\textbf{S})}{Omega ~ W[nu](S)}
    \item Notation 2: \eqn{p(\Omega) = \mathcal{W}_{\nu}(\Omega |
      \textbf{S})}{p(Omega) = W[nu](Omega | S)}
    \item Parameter 1: degrees of freedom \eqn{\nu \ge k}{nu >= k}
    \item Parameter 2: symmetric, positive-semidefinite
    \eqn{k \times k}{k x k} scale matrix \eqn{\textbf{S}}{S}
    \item Mean: \eqn{E(\Omega) = \nu \textbf{S}}{E(Omega) = nuS}
    \item Variance: \eqn{var(\Omega) = \nu (\textbf{S}^2_{i,j} +
      \textbf{S}_{i,i} \textbf{S}_{j,j})}{var(Omega) = nu(S[i,j]^2 +
      S[i,i]S[j,j])}
    \item Mode: \eqn{mode(\Omega) = (\nu - k - 1)
      \textbf{S}}{mode(Omega) = (nu-k-1)S}, for \eqn{\nu \ge k + 1}{nu
      >= k + 1}
}

The Wishart distribution is a generalization to multiple dimensions of
the chi-square distribution, or, in the case of non-integer degrees of
freedom, of the gamma distribution. However, the Wishart distribution is
not called the multivariate chi-squared distribution because the
marginal distribution of the off-diagonal elements is not chi-squared.

The Wishart is the conjugate prior distribution for the precision matrix
\eqn{\Omega}{Omega}, the inverse of which (covariance matrix
\eqn{\Sigma}{Sigma}) is used in a multivariate normal distribution. In
this parameterization, \eqn{\Omega}{Omega} has been decomposed to the
upper-triangular Cholesky factor \eqn{\textbf{U}}{U}, as per
\code{\link{chol}}.

The integral is finite when \eqn{\nu \ge k}{nu >= k}, where
\eqn{\nu}{nu} is the scalar degrees of freedom parameter, and \eqn{k} is
the dimension of scale matrix \eqn{\textbf{S}}{S}. The density is finite
when \eqn{\nu ge k + 1}{nu >= k + 1}, which is recommended.

The degrees of freedom, \eqn{\nu}{nu}, is equivalent to specifying a
prior sample size, indicating the confidence in \eqn{\textbf{S}}{S},
where \eqn{\textbf{S}}{S} is a prior guess at the order of covariance
matrix \eqn{\Sigma}{Sigma}. A flat prior distribution is
obtained as \eqn{\nu \rightarrow 0}{nu -> 0}.

In practice, \eqn{\textbf{U}}{U} is fully unconstrained for proposals
when its diagonal is log-transformed. The diagonal is exponentiated
after a proposal and before other calculations. Overall, the
Cholesky parameterization is faster than the traditional
parameterization. Compared with \code{dwishart}, \code{dwishartc}
must additionally matrix-multiply the Cholesky back to the precision
matrix, but it does not have to check for or correct the precision
matrix to positive-semidefiniteness, which overall is slower. Compared
with \code{rwishart}, \code{rwishartc} must additionally
calculate a Cholesky decomposition, and is therefore slower.

The Wishart prior lacks flexibility, having only one parameter,
\eqn{\nu}{nu}, to control the variability for all \eqn{k(k + 1)/2}
elements. Popular choices for the scale matrix \eqn{\textbf{S}}{S}
include an identity matrix or sample covariance matrix. When the model
sample size is small, the specification of the scale matrix can be
influential.

Although the related inverse Wishart distribution has a dependency
between variance and correlation, the Wishart distribution does not
have this dependency.

The matrix gamma (\code{\link{dmatrixgamma}}) distribution is a more
general version of the Wishart distribution, and the Yang-Berger
(\code{\link{dyangberger}}) distribution is an alterative that is a
least informative prior (LIP).
}
\value{
  \code{dwishartc} gives the density and
  \code{rwishartc} generates random deviates.
}
\references{
  Wishart, J. (1928). "The Generalised Product Moment Distribution in
  Samples from a Normal Multivariate Population". \emph{Biometrika},
  20A(1-2), p. 32--52.
}
\seealso{
  \code{\link{chol}},
  \code{\link{dchisq}},
  \code{\link{dgamma}},
  \code{\link{dinvwishart}},
  \code{\link{dinvwishartc}},
  \code{\link{dmatrixgamma}},
  \code{\link{dmvnp}},
  \code{\link{dmvnpc}},
  \code{\link{Prec2Cov}}, and
  \code{\link{dyangbergerc}}.
}
\examples{
library(LaplacesDemon)
Omega <- matrix(c(2,-.3,-.3,4),2,2)
U <- chol(Omega)
x <- dwishartc(U, 3, matrix(c(1,.1,.1,1),2,2))
x <- rwishartc(3, matrix(c(1,.1,.1,1),2,2))
}
\keyword{Distribution}

\name{is.constrained}
\alias{is.constrained}
\title{Logical Check of Constraints}
\description{
  This function provides a logical test of constraints for each initial
  value or parameter for a model specification, given data.
}
\usage{
is.constrained(Model, Initial.Values, Data)
}
\arguments{
  \item{Model}{This is a model specification function. For more
    information, see the \code{\link{LaplacesDemon}} function.}
  \item{Initial.Values}{This is a vector of initial values, or current
    parameter values. For more information, see the
    \code{\link{LaplacesDemon}} function.}
  \item{Data}{This is a list of data. For more information, see the
    \code{\link{LaplacesDemon}} function.}
}
\details{
  This function is useful for testing whether or not initial values
  changed due to constraints when being passed through a \code{Model}
  specification function. If any initial value changes, then the
  constrained values that are ouput in the fifth component of the
  \code{Model} specification are suitable as initial values, not the
  tested initial values.

  A parameter may be constrained and this function may not discover the
  constraint, since the discovery depends on the initial values and
  whether or not they change as they are passed through the model.
}
\value{
  The \code{is.constrained} function returns a logical vector, equal in
  length to the number of initial values. Each element receives
  \code{TRUE} if the corresponding initial value changed due to a
  constraint, or \code{FALSE} if it did not.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{LaplacesDemon}}.
}
\keyword{Utility}

\name{dist.Inverse.Wishart}
\alias{dinvwishart}
\alias{rinvwishart}
\title{Inverse Wishart Distribution}
\description{
  These functions provide the density and random number generation
     for the inverse Wishart distribution.
}
\usage{
   dinvwishart(Sigma, nu, S, log=FALSE)
   rinvwishart(nu, S)
}
\arguments{
  \item{Sigma}{This is the symmetric, positive-definite
    \eqn{k \times k}{k x k} matrix \eqn{\Sigma}{Sigma}.}
  \item{nu}{This is the scalar degrees of freedom, \eqn{\nu}{nu}.}
  \item{S}{This is the symmetric, positive-semidefinite
    \eqn{k \times k}{k x k} scale matrix \eqn{\textbf{S}}{S}.}
    \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Multivariate
    \item Density: \eqn{p(\theta) = (2^{\nu k/2} \pi^{k(k-1)/4}
      \prod^k_{i=1} \Gamma(\frac{\nu+1-i}{2}))^{-1} |\textbf{S}|^{\nu/2}
      |\Sigma|^{-(\nu+k+1)/2} \exp(-\frac{1}{2} tr(\textbf{S}
      \Sigma^{-1}))}{p(theta) = (2^(nu*k/2) * pi^(k(k-1)/4) *
      [Gamma((nu+1-i)/2) * ... * Gamma((nu+1-k)/2)])^(-1) * |S|^(nu/2) *
      |Sigma|^(-(nu+k+1)/2) * exp(-(1/2) * tr(S Sigma^(-1)))}
    \item Inventor: John Wishart (1928)
    \item Notation 1: \eqn{\Sigma \sim
      \mathcal{W}^{-1}_{\nu}(\textbf{S}^{-1})}{Sigma ~
      W^(-1)[nu](S^(-1))}
    \item Notation 2: \eqn{p(\Sigma) = \mathcal{W}^{-1}_{\nu}(\Sigma |
      \textbf{S}^{-1})}{p(Sigma) = W^-1[nu](Sigma | S^(-1))}
    \item Parameter 1: degrees of freedom \eqn{\nu}{nu}
    \item Parameter 2: symmetric, positive-semidefinite
      \eqn{k \times k}{k x k} scale matrix \eqn{\textbf{S}}{S}
    \item Mean: \eqn{E(\Sigma) = \frac{\textbf{S}}{\nu - k -
	1}}{E(Sigma) = S / (nu - k - 1)}
    \item Variance: 
    \item Mode: \eqn{mode(\Sigma) = \frac{\textbf{S}}{\nu + k +
	1}}{mode(Sigma) = S / (nu + k + 1)}
}

The inverse Wishart distribution is a probability distribution defined on
real-valued, symmetric, positive-definite matrices, and is used as the
conjugate prior for the covariance matrix, \eqn{\Sigma}{Sigma}, of a
multivariate normal distribution. The inverse-Wishart density is always
finite, and the integral is always finite. A degenerate form occurs when
\eqn{\nu < k}{nu < k}.

When applicable, the alternative Cholesky parameterization should be
preferred. For more information, see \code{\link{dinvwishartc}}.

The inverse Wishart prior lacks flexibility, having only one parameter,
\eqn{\nu}{nu}, to control the variability for all \eqn{k(k + 1)/2}
elements. Popular choices for the scale matrix \eqn{\textbf{S}}{S}
include an identity matrix or sample covariance matrix. When the model
sample size is small, the specification of the scale matrix can be
influential.

The inverse Wishart distribution has a dependency between variance and
correlation, although its relative for a precision matrix (inverse
covariance matrix), the Wishart distribution, does not have this
dependency. This relationship becomes weaker with more degrees of
freedom.

Due to these limitations (lack of flexibility, and dependence between
variance and correlation), alternative distributions have been
developed. Alternative distributions that are available here include
Huang-Wand (\code{\link{dhuangwand}}), inverse matrix gamma
(\code{\link{dinvmatrixgamma}}), Scaled Inverse Wishart
(\code{\link{dsiw}}), and Yang-Berger (\code{\link{dyangberger}}).

These functions are parameterized as per Gelman et al. (2004).
}
\value{
  \code{dinvwishart} gives the density and
  \code{rinvwishart} generates random deviates.
}
\references{  
  Gelman, A., Carlin, J., Stern, H., and Rubin, D. (2004). "Bayesian
  Data Analysis, Texts in Statistical Science, 2nd ed.". Chapman and
  Hall, London.

  Wishart, J. (1928). "The Generalised Product Moment Distribution in
  Samples from a Normal Multivariate Population". \emph{Biometrika},
  20A(1-2), p. 32--52.
}
\seealso{
  \code{\link{dhuangwand}},
  \code{\link{dinvmatrixgamma}},
  \code{\link{dinvwishartc}},
  \code{\link{dmvn}},
  \code{\link{dsiw}},
  \code{\link{dwishart}}, and
  \code{\link{dyangberger}}.
}
\examples{
library(LaplacesDemon)
x <- dinvwishart(matrix(c(2,-.3,-.3,4),2,2), 3, matrix(c(1,.1,.1,1),2,2))
x <- rinvwishart(3, matrix(c(1,.1,.1,1),2,2))
}
\keyword{Distribution}

\name{is.data}
\alias{is.data}
\title{Logical Check of Data}
\description{
  This function provides a logical test of whether or not a given list
  of data meets minimum criteria to be considered data for
  \code{\link{IterativeQuadrature}}, \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, \code{\link{PMC}}, or
  \code{\link{VariationalBayes}}.
}
\usage{
is.data(Data)
}
\arguments{
  \item{Data}{This is a list of data. For more information, see the
    \code{\link{LaplacesDemon}} function.}
}
\details{
  This function is useful for testing whether or not a list of data
  meets minimum criteria to be considered data in this package. The
  minimum requirements are that \code{Data} is a list, and it contains
  \code{mon.names} and \code{parm.names}.

  This function is not extensive. For example, it does not match the
  length of \code{parm.names} with the length of \code{Initial.Values},
  or compare the length of \code{mon.names} to the number of monitored
  variables output from the \code{Model} specification
  function. Additional checks are conducted in
  \code{\link{IterativeQuadrature}}, \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\value{
  The \code{is.data} function returns a logical value. It returns
  \code{TRUE} if \code{Data} meets minimum requirements to be considered
  data in this package, and \code{FALSE} otherwise.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{IterativeQuadrature}}
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\keyword{Utility}

\name{dist.Matrix.Normal}
\alias{dmatrixnorm}
\alias{rmatrixnorm}
\title{Matrix Normal Distribution}
\description{
  These functions provide the density and random number generation for
  the matrix normal distribution.
}
\usage{
dmatrixnorm(X, M, U, V, log=FALSE) 
rmatrixnorm(M, U, V)
}
\arguments{
  \item{X}{This is data or parameters in the form of a matrix with
    \eqn{n} rows and \eqn{k} columns.}
  \item{M}{This is mean matrix with \eqn{n} rows and \eqn{k} columns.}
  \item{U}{This is a \eqn{n \times n}{n x n} positive-definite scale
    matrix.}
  \item{V}{This is a \eqn{k \times k}{k x k} positive-definite scale
    matrix.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
       density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Multivariate Matrix
    \item Density: \eqn{p(\theta) =
      \frac{\exp(-0.5tr[V^{-1}(X-M)'U^{-1}(X-M)])}{(2\pi)^{nk/2}|V|^{n/2}|U|^{k/2}}}{p(theta) = exp(-0.5tr[V^(-1)(X-M)'U^(-1)(X-M)])/[(2*pi)^(nk/2)|V|^(n/2)|U|(k/2)]}
  \item Inventors: Unknown
  \item Notation 1: \eqn{\theta \sim \mathcal{MN}_{n \times k}(M, U,
    V)}{theta ~ MN[n x k](M, U, V)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{MN}_{n \times k}(\theta |
    M, U, V)}{p(theta) = MN[n x k](theta | M, U, V)}
  \item Parameter 1: location \eqn{n \times k}{n x k} matrix \eqn{M}
  \item Parameter 2: positive-definite \eqn{n \times n}{n x n} scale matrix \eqn{U}
  \item Parameter 3: positive-definite \eqn{k \times k}{k x k} scale matrix \eqn{V}
  \item Mean: \eqn{E(\theta) = M}{E(theta) = M}
  \item Variance: Unknown
  \item Mode: Unknown
}

The matrix normal distribution is also called the matrix Gaussian,
matrix-variate normal, or matrix-variate Gaussian distribution. It is a
generalization of the multivariate normal distribution to matrix-valued
random variables.

An example of the use of a matrix normal distribution is multivariate
regression, in which there is a \eqn{j \times k}{j x k} matrix of
regression effects of \eqn{j} predictors for \eqn{k} dependent
variables. For univariate regression, having only one dependent
variable, the \eqn{j} regression effects may be multivariate normally
distributed. For multivariate regression, this multivariate normal
distribution may be extended to a matrix normal distribution to account
for relationships of the regression effects across \eqn{k} dependent
variables. In this example, the matrix normal distribution is the
conjugate prior distribution for these regression effects.

The matrix normal distribution has two covariance matrices, one for the
rows and one for the columns. When \eqn{U} is diagonal, the rows are
independent. When \eqn{V} is diagonal, the columns are independent.
}
\value{
  \code{dmatrixnorm} gives the density and 
  \code{rmatrixnorm} generates random deviates.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dinvmatrixgamma}},
  \code{\link{dmatrixgamma}}, and
  \code{\link{dmvn}}.
}
\examples{
library(LaplacesDemon)
N <- 10
K <- 4
U <- as.positive.definite(matrix(rnorm(N*N),N,N))
V <- as.positive.definite(matrix(rnorm(K*K),K,K))
x <- dmatrixnorm(matrix(0,N,K), matrix(0,N,K), U, V)
X <- rmatrixnorm(matrix(0,N,K), U, V)
joint.density.plot(X[,1], X[,2], color=TRUE)
}
\keyword{Distribution}

\name{WAIC}
\alias{WAIC}
\title{Widely Applicable Information Criterion}
\description{
  This function calculates the Widely Applicable Information Criterion
  (WAIC), also known as the Widely Available Information Criterion or
  the Watanable-Akaike, of Watanabe (2010).
}
\usage{
WAIC(x)
}
\arguments{
  \item{x}{This required argument accepts a \eqn{N \times S}{N x S}
    matrix of log-likelihood (LL) for \eqn{N} records and \eqn{S}
    samples.}
}
\details{
  WAIC is an extension of the Akaike Information Criterion (AIC) that is
  more fully Bayesian than the Deviance Information Criterion (DIC).

  Like DIC, WAIC estimates the effective number of parameters to adjust
  for overfitting. Two adjustments have been proposed. pWAIC1 is similar
  to pD in the original DIC. In contrast, pWAIC2 is calculated with
  variance more similarly to pV, which Gelman proposed for DIC. Gelman
  et al. (2014, p.174) recommends pWAIC2 because its results are closer
  in practice to the results of leave-one-out cross-validation
  (LOO-CV). pWAIC is considered an approximation to the number of
  unconstrained and uninformed parameters, where a parameter counts as 1
  when estimated without contraint or any prior information, 0 if fully
  constrained or all information comes from the prior distribution, or
  an intermediate number if both the data and prior are informative,
  which is usually the case.

  Gelman et al. (2014, p. 174) scale the WAIC of Watanabe (2010) by a
  factor of 2 so that it is comparable to AIC and DIC. WAIC is then
  reported as \eqn{-2(lppd - pWAIC)}. Gelman et al. (2014) prefer WAIC
  to AIC or DIC when feasible, which is less often than AIC or DIC.
  The \code{\link{LaplacesDemon}} function requires the model
  specification function to return the model-level deviance, which is
  \eqn{-2(LL)}, where \eqn{LL} is the sum of the record-level
  log-likelihood. Therefore, if the user desires to calculate WAIC, then
  the record-level log-likelihood must be monitored.
}
\value{
  The \code{WAIC} argument returns a list with four components:
  \item{WAIC}{This is the Widely Applicable Information Criterion
  (WAIC), which is \eqn{-2(lppd - pWAIC)}.}
  \item{lppd}{This is the log pointwise predictive density. For more
  information, see Gelman et al. (2014, p. 168).}
  \item{pWAIC}{This is the effective number of parameters preferred by
    Gelman et al. (2014).}
  \item{pWAIC1}{This is the effective number of parameters, is
    calculated with an alternate method, and is included here for
    completeness. It is not used to calculate WAIC in the \code{WAIC}
    function.}
}
\references{
  Gelman, A., Carlin, J.B., Stern, H.S., Dunson, D.B., Vehtari, A., and
  Rubin, D.B. (2014). "Bayesian Data Analysis, 3rd ed.". CRC Press:
  Boca Raton, FL.
  
  Watanabe, S. (2010). "Asymptotic Equivalence of Bayes Cross Validation
  and Widely Applicable Information Criterion in Singular Learning
  Theory". \emph{Journal of Machine Learning Research}, 11,
  p. 3571--3594.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{LaplacesDemon}}
}
\examples{
#library(LaplacesDemon)
#N <- 10 #Number of records
#S <- 1000 #Number of samples
#LL <- t(rmvn(S, -70+rnorm(N),
#     as.positive.definite(matrix(rnorm(N*N),N,N))))
#WAIC(LL)
### Compare with DIC:
#Dev <- -2*colSums(LL)
#DIC <- list(DIC=mean(Dev) + var(Dev)/2, Dbar=mean(Dev), pV=var(Dev)/2)
#DIC
}
\keyword{Diagnostic}
\keyword{Information Criterion}
\keyword{MCMC}
\keyword{Monte Carlo}
\keyword{Utility}

\name{burnin}
\alias{burnin}
\title{Burn-in}
\description{
  The \code{burnin} function estimates the duration of burn-in in
  iterations for one or more Markov chains. ``Burn-in'' refers to the
  initial portion of a Markov chain that is not stationary and is still
  affected by its initial value.
}
\usage{
burnin(x, method="BMK")
}
\arguments{
  \item{x}{This is a vector or matrix of posterior samples for which a
    the number of burn-in iterations will be estimated.}
  \item{method}{This argument defaults to \code{"BMK"}, in which case
    stationarity is estimated with the \code{\link{BMK.Diagnostic}}
    function. Alternatively, the \code{\link{Geweke.Diagnostic}}
    function may be used when \code{method="Geweke"} or the
    \code{\link{KS.Diagnostic}} function may be used when
    \code{method="KS"}.}
}
\details{
  Burn-in is a colloquial term for the initial iterations in a Markov
  chain prior to its convergence to the target distribution. During
  burn-in, the chain is not considered to have ``forgotten'' its initial
  value.

  Burn-in is not a theoretical part of MCMC, but its use is the norm
  because of the need to limit the number of posterior samples due to
  computer memory. If burn-in were retained rather than discarded, then
  more posterior samples would have to be retained. If a Markov chain
  starts anywhere close to the center of its target distribution, then
  burn-in iterations do not need to be discarded.

  In the \code{\link{LaplacesDemon}} function, stationarity is estimated
  with the \code{\link{BMK.Diagnostic}} function on all thinned
  posterior samples of each chain, beginning at cumulative 10\% intervals
  relative to the total number of samples, and the lowest number in
  which all chains are stationary is considered the burn-in.

  The term, ``burn-in'', originated in electronics regarding the initial
  testing of component failure at the factory to eliminate initial
  failures (Geyer, 2011). Although ``burn-in' has been the standard term
  for decades, some are referring to these as ``warm-up'' iterations.
}
\value{
  The \code{burnin} function returns a vector equal in length to the
  number of MCMC chains in \code{x}, and each element indicates the
  maximum iteration in burn-in.
}
\references{
  Geyer, C.J. (2011). "Introduction to Markov Chain Monte Carlo". In
  S Brooks, A Gelman, G Jones, and M Xiao-Li (eds.), "Handbook of
  Markov Chain Monte Carlo", p. 3--48. Chapman and Hall, Boca Raton, FL.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{BMK.Diagnostic}},
  \code{\link{deburn}},
  \code{\link{Geweke.Diagnostic}},
  \code{\link{KS.Diagnostic}}, and
  \code{\link{LaplacesDemon}}.
}
\examples{
library(LaplacesDemon)
x <- rnorm(1000)
burnin(x)
}
\keyword{Diagnostic}
\keyword{Stationarity}
\keyword{Utility}

\name{Juxtapose}
\alias{Juxtapose}
\title{Juxtapose MCMC Algorithm Inefficiency}
\description{
  This function gives a side-by-side comparison of (or juxtaposes) the
  inefficiency of MCMC algorithms in \code{\link{LaplacesDemon}} for
  applied use, and is a valuable tool for selecting what is likely to be
  the least inefficient algorithm for the user's current model, prior to
  updating the final, intended model.
}
\usage{
Juxtapose(x)
}
\arguments{
  \item{x}{This is a list of multiple components. Each component must
    be an object of class \code{demonoid}.}
}
\details{
  Laplace's Demon recommends using the \code{Juxtapose} function on the
  user's model (or most likely a simplified version of it) with a
  smaller, simulated data set to select the least inefficient MCMC
  algorithm before using real data and updating the model for numerous
  iterations. The least inefficient MCMC algorithm differs for different
  models and data sets. Using \code{Juxtapose} in this way does not
  guarantee that the selected algorithm will remain the best choice with
  real data, but it should be better than otherwise selecting an
  algorithm.

  The user must make a decision regarding their model and data. The more
  similar the model and data is to the final, intended model and data,
  the more appropriate will be the results of the \code{Juxtapose}
  function. However, if the full model and data are used, then the user
  may as well instead skip using \code{Juxtapose} and proceed directly
  to \code{\link{LaplacesDemon}}. Replacing the actual data set with a
  smaller, simulated set is fairly straightforward, but the
  decision-making will most likely focus on what is the best way to
  reduce the full model specification. A simple approach may be to
  merely reduce the number of predictors. However, complicated models
  may have several components that slow down estimation time, and extend
  the amount of time until global stationarity is estimated. Laplace's
  Demon offers no guidance here, and leaves it in the realm of user
  discretion.

  First, the user should simulate a smaller data set, and if best,
  reduce the model specification. Next, the user must select candidate
  algorithms. Then, the user must update each algorithm with
  \code{\link{LaplacesDemon}} for numerous iterations, with the goal of
  achieving stationarity for all parameters early in the
  iterations. Each update should begin with the same model specification
  function, vector of initial values, and data. Each output object of
  class \code{demonoid} should be renamed. An example follows.

  Suppose a user considers three candidate algorithms for their model:
  AMWG, NUTS, and twalk. The user updates each model, saving the model
  that used the AMWG algorithm as, say, \code{Fit1}, the NUTS model as
  \code{Fit2}, and the twalk model as \code{Fit3}.

  Next, the output model objects are put in a list and passed to the
  \code{Juxtapose} function. See the example below.

  The \code{Juxtapose} function uses an internal version of the
  \code{\link{IAT}}, which is a slightly modified version of that found
  in the \code{SamplerCompare} package. The \code{Juxtapose} function
  returns an object of class \code{juxtapose}. It is a matrix in which
  each row is a result and each column is an algorithm.

  The rows are:
  \itemize{
    \item \code{iter.min}: This is the iterations per minute.
    \item \code{t.iter.min}: This is the thinned iterations per minute.
    \item \code{prop.stat}: This is the proportion of iterations that
    were stationary.
    \item \code{IAT.025}: This is the 2.5\% quantile of the integrated
    autocorrelation time of the worst parameter, estimated only on
    samples when all parameters are estimated to be globally stationary.
    \item \code{IAT.500}: This is the median integrated autocorrelation
    time of the worst parameter, estimated only on samples when all
    parameters are estimated to be globally stationary.
    \item \code{IAT.975}: This is the 97.5\% quantile of the integrated
    autocorrelation time of the worst parameter, estimated only on
    samples when all parameters are estimated to be globally stationary.
    \item \code{ISM.025}: This is the 2.5\% quantile of the number of
    independent samples per minute.
    \item \code{ISM.500}: This is the median of the number of the
    independent samples per minute. The least inefficient MCMC algorithm
    has the highest \code{ISM.500}.
    \item \code{ISM.975}: This is the 97.5\% quantile of the number of
    the independent samples per minute.
  }

  As for calculating \eqn{ISM}, let \eqn{TIM} be the observed number of
  thinned iterations per minute, \eqn{PS} be the percent of iterations
  in which all parameters were estimated to be globally stationary, and
  \eqn{IAT_q}{IAT[q]} be a quantile from a simulated distribution of the
  integrated autocorrelation time among the parameters.

  \deqn{ISM = \frac{PS \times TIM}{IAT_q}}{ISM = (PS x TIM) / IAT[q]}
  
  There are various ways to measure the inefficiency of MCMC
  samplers. \code{\link{IAT}} is used perhaps most often. As with the
  \code{SamplerCompare} package, Laplace's Demon uses the worst
  parameter, in terms of \code{\link{IAT}}. Often, the number of
  evaluations or number of parameters is considered. The
  \code{Juxtapose} function, instead considers the final criterion of
  MCMC efficiency, in an applied context, to be \code{ISM}, or the
  number of Independent (thinned) Samples per Minute. The algorithm with
  the highest \code{ISM.500} is the best, or least inefficient,
  algorithm with respect to its worst \code{\link{IAT}}, the proportion
  of iterations required to seem to have global stationarity, and the
  number of (thinned) iterations per minute.

  A disadvantage of using time is that it will differ by computer, and
  is less likely to be reported in a journal. The advantage, though, is
  that it is more meaningful to a user. Increases in the number of
  evaluations, parameters, and time should all correlate well, but time
  may enlighten a user as to expected run-time given the model just
  studied, even though the real data set will most likely be larger than
  the simulated data used initially. NUTS is an example of a sampler in
  which the number of evaluations varies per iteration. For an
  alternative approach, see Thompson (2010).

  The \code{Juxtapose} function also adjusts \code{ISM} by
  \code{prop.stat}, the proportion of the iterations in which all chains
  were estimated to be stationary. This adjustment is weighted by
  burn-in iterations, penalizing an algorithm that took longer to
  achieve global stationarity. The goal, again, is to assist the user in
  selecting the least inefficient MCMC algorithm in an applied setting.

  The \code{Juxtapose} function has many other potential uses than those
  described above. One additional use of the \code{Juxtapose} function is
  to compare inefficiencies within a single algorithm in which
  algorithmic specifications varied with different model
  updates. Another use is to investigate parallel chains in an object of
  class \code{demonoid.hpc}, as returned from the
  \code{\link{LaplacesDemon.hpc}} function. Yet another use is to
  compare the effects of small changes to a model specification
  function, such as with priors, or due to an increase in the amount of
  simulated data.

  An object of class \code{juxtapose} may be plotted with the
  \code{\link{plot.juxtapose}} function, which displays \code{ISM} by
  default, or optionally \code{IAT}. For more information, see the
  \code{\link{plot.juxtapose}} function.

  Independent samples per minute, calculated as \code{\link{ESS}}
  divided by minutes of run-time, are also available by parameter in the
  \code{\link{PosteriorChecks}} function.
}
\value{
  This function returns an object of class \code{juxtapose}. It is a
  \eqn{9 \times J}{9 x J} matrix with nine results for \eqn{J} MCMC
  algorithms.
}
\references{
  Thompson, M. (2010). "Graphical Comparison of MCMC Performance". ArXiv
  e-prints, eprint 1011.4458.
}
\seealso{
  \code{\link{IAT}},
  \code{\link{is.juxtapose}},
  \code{\link{LaplacesDemon}},
  \code{\link{LaplacesDemon.hpc}},
  \code{\link{plot.juxtapose}}, and
  \code{\link{PosteriorChecks}}.
}
\examples{
### Update three demonoid objects, each from different MCMC algorithms.
### Suppose Fit1 was updated with AFSS, Fit2 with AMWG, and
### Fit3 with NUTS. Then, compare the inefficiencies:
#Juxt <- Juxtapose(list(Fit1=Fit1, Fit2=Fit2, Fit3=Fit3)); Juxt
#plot(Juxt, Style="ISM")
}
\keyword{Diagnostic}
\keyword{Utilities}

\name{plot.demonoid.ppc}
\alias{plot.demonoid.ppc}
\title{Plots of Posterior Predictive Checks}
\description{
  This may be used to plot, or save plots of, samples in an object of
  class \code{demonoid.ppc}. A variety of plots is provided.
}
\usage{\method{plot}{demonoid.ppc}(x, Style=NULL, Data=NULL, Rows=NULL,
     PDF=FALSE, \dots)}
\arguments{
  \item{x}{
    This required argument is an object of class \code{demonoid.ppc}.}
  \item{Style}{
    This optional argument specifies one of several styles of plots,
    and defaults to \code{NULL} (which is the same as
    \code{"Density"}). Styles of plots are indicated in quotes. Optional
    styles include \code{"Covariates"}, \code{"Covariates, Categorical DV"},
    \code{"Density"}, \code{"DW"}, \code{"DW, Multivariate, C"},
    \code{"ECDF"}, \code{"Fitted"}, \code{"Fitted, Multivariate, C"},
    \code{"Fitted, Multivariate, R"}, \code{"Jarque-Bera"},
    \code{"Jarque-Bera, Multivariate, C"},
    \code{"Mardia"}, \code{"Predictive Quantiles"},
    \code{"Residual Density"}, \code{"Residual Density, Multivariate, C"},
    \code{"Residual Density, Multivariate, R"},
    \code{"Residuals"}, \code{"Residuals, Multivariate, C"},
    \code{"Residuals, Multivariate, R"}, \code{"Space-Time by Space"},
    \code{"Space-Time by Time"}, \code{"Spatial"},
    \code{"Spatial Uncertainty"}, \code{"Time-Series"},
    \code{"Time-Series, Multivariate, C"}, and
    \code{"Time-Series, Multivariate, R"}. Details are given below.}
  \item{Data}{
    This optional argument accepts the data set used when updating the
    model. Data is required only with certain plot styles, including
    \code{"Covariates"}, \code{"Covariates, Categorical DV"},
    \code{"DW, Multivariate, C"}, \code{"Fitted, Multivariate, C"},
    \code{"Fitted, Multivariate, R"},
    \code{"Jarque-Bera, Multivariate, C"}, \code{"Mardia"},
    \code{"Residual Density, Multivariate, C"},
    \code{"Residual Density, Multivariate, R"},
    \code{"Residuals, Multivariate, C"},
    \code{"Residuals, Multivariate, R"},
    \code{"Space-Time by Space"}, \code{"Space-Time by Time"},
    \code{"Spatial"}, \code{"Spatial Uncertainty"},
    \code{"Time-Series, Multivariate, C"}, and
    \code{"Time-Series, Multivariate, R"}.}
  \item{Rows}{
    This optional argument is for a vector of row numbers that 
    specify the records associated by row in the object of class
    \code{demonoid.ppc}. Only these rows are plotted. The default is to
    plot all rows. Some plots do not allow rows to be specified.}
  \item{PDF}{
    This logical argument indicates whether or not the user wants
    Laplace's Demon to save the plots as a .pdf file.}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  This function can be used to produce a variety of posterior predictive
  plots, and the style of plot is selected with the \code{Style}
  argument. Below are some notes on the styles of plots.

  \code{Covariates} requires \code{Data} to be specified, and also
  requires that the covariates are named \code{X} or \code{x}. A plot
  is produced for each covariate column vector against yhat, and is
  appropriate when y is not categorical.

  \code{Covariates, Categorical DV} requires \code{Data} to be
  specified, and also requires that the covariates are named \code{X} or
  \code{x}. A plot is produced for each covariate column vector against
  yhat, and is appropriate when y is categorical.
  
  \code{Density} plots show the kernel density of the posterior
  predictive distribution for each selected row of y (all are selected
  by default). A vertical red line indicates the position of the
  observed y along the x-axis. When the vertical red line is close to
  the middle of a normal posterior predictive distribution, then there
  is little discrepancy between y and the posterior predictive
  distribution. When the vertical red line is in the tail of the
  distribution, or outside of the kernel density altogether, then
  there is a large discrepancy between y and the posterior predictive
  distribution. Large discrepancies may be considered outliers, and
  moreover suggest that an improvement in model fit should be
  considered.

  \code{DW} plots the distributions of the Durbin-Watson (DW) test
  statistics (Durbin and Watson, 1950), both observed
  (\eqn{d^{obs}}{d.obs} as a transparent, black density) and replicated
  (\eqn{d^{rep}}{d.rep} as a transparent, red density). The distribution
  of \eqn{d^{obs}}{d.obs} is estimated from the model, and
  \eqn{d^{rep}}{d.rep} is simulated from normal residuals without
  autocorrelation, where the number of simulations are the same as the
  observed number. This DW test may be applied to the residuals of
  univariate time-series models (or otherwise ordered residuals) to
  detect first-order autocorrelation. Autocorrelated residuals are not
  independent. The DW test is applicable only when the residuals are
  normally-distributed, higher-order autocorrelation is not present, and
  y is not used also as a lagged predictor. The DW test statistic,
  \eqn{d^{obs}}{d[obs]}, occurs in the interval (0,4), where 0 is
  perfect positive autocorrelation, 2 is no autocorrelation, and 4 is
  perfect negative autocorrelation. The following summary is reported on
  the plot: the mean of \eqn{d^{obs}}{d[obs]} (and its 95\% probability
  interval), the probability that \eqn{d^{obs} > d^{rep}}{d[obs] >
  d[rep]}, and whether or not autocorrelation is found. Positive
  autocorrelation is reported when the observed process is greater than
  the replicated process in 2.5\% of the samples, and negative
  autocorrelation is reported when the observed process is greater than
  the replicated process in 97.5\% of the samples.

  \code{DW, Multivariate, C} requires \code{Data} to be
  specified, and also requires that variable \code{Y} exist in the data
  set with exactly that name. These plots compare each column-wise
  vector of residuals with a univariate Durbin-Watson test, as in
  \code{DW} above. This plot is appropriate when Y is multivariate, not
  categorical, and residuals are desired to be tested column-wise for
  first-order autocorrelation.
  
  \code{ECDF} (Empirical Cumulative Distribution Function) plots compare
  the ECDF of y with three ECDFs of yhat based on the 2.5\%, 50\%
  (median), and 97.5\% of its distribution. The ECDF(y) is defined as
  the proportion of values less than or equal to y. This plot is
  appropriate when y is univariate and at least ordinal.

  \code{Fitted} plots compare y with the probability interval of its
  replicate, and provide loess smoothing. This plot is appropriate when
  y is univariate and not categorical.

  \code{Fitted, Multivariate, C} requires \code{Data} to be
  specified, and also requires that variable \code{Y} exists in the data
  set with exactly that name. These plots compare each column-wise
  vector of y in Y with its replicates and provide loess smoothing.
  This plot is appropriate when Y is multivariate, not categorical, and
  desired to be seen column-wise.
  
  \code{Fitted, Multivariate, R} requires \code{Data} to be
  specified, and also requires that variable \code{Y} exists in the data
  set with exactly that name. These plots compare each row-wise
  vector of y in Y with its replicates and provide loess smoothing.
  This plot is appropriate when Y is multivariate, not categorical, and
  desired to be seen row-wise.

  \code{Jarque-Bera} plots the distributions of the Jarque-Bera (JB)
  test statistics (Jarque and Bera, 1980), both observed
  (\eqn{JB^{obs}}{JB.obs} as a transparent black density) and replicated
  (\eqn{JB^{rep}}{JB.rep} as a transparent red density). The
  distribution of \eqn{JB^{obs}}{JB.obs} is estimated from the model,
  and \eqn{JB^{rep}}{JB.rep} is simulated from normal residuals, where
  the number of simulations are the same as the observed number. This
  Jarque-Bera test may be applied to the residuals of 
  univariate models to test for normality. The Jarque-Bera test does not
  test normality per se, but whether or not the distribution has
  kurtosis and skewness that match a normal distribution, and is
  therefore a test of the moments of a normal distribution. The
  following summary is reported on the plot: the mean of
  \eqn{JB^{obs}}{JB[obs]} (and its 95\% probability interval), the
  probability that \eqn{JB^{obs} > JB^{rep}}{JB[obs] > JB[rep]}, and
  whether or not normality is indicated. Non-normality is reported when
  the observed process is greater than the replicated process in either
  2.5\% or 97.5\% of the samples.

  \code{Jarque-Bera, Multivariate, C} requires \code{Data} to be
  specified, and also requires that variable \code{Y} exist in the data
  set with exactly that name. These plots compare each column-wise
  vector of residuals with a univariate Jarque-Bera test, as in
  \code{Jarque-Bera} above. This plot is appropriate when Y is
  multivariate, not categorical, and residuals are desired to be
  tested column-wise for normality.

  \code{Mardia} plots the distributions of the skewness (K3) and
  kurtosis (K4) test statistics (Mardia, 1970), both observed
  (\eqn{K3^{obs}}{K3.obs} and \eqn{K4^{obs}}{K4.obs} as transparent
  black density) and replicated (\eqn{K3^{rep}}{K3.rep} and
  \eqn{K4^{rep}}{K4.rep} as transparent red density). The distributions
  of \eqn{K3^{obs}}{K3.obs} and \eqn{K4^{obs}}{K4.obs} are estimated
  from the model, and both \eqn{K3^{rep}}{K3.rep} \eqn{K4^{rep}}{K4.rep}
  are simulated from multivariate normal residuals, where the number of
  simulations are the same as the observed number. This Mardia's test
  may be applied to the residuals of multivariate models to test for
  multivariate normality. Mardia's test does not test for multivariate
  normality per se, but whether or not the distribution has kurtosis and
  skewness that match a multivariate normal distribution, and is 
  therefore a test of the moments of a multivariate normal
  distribution. The following summary is reported on the plots: the
  means of \eqn{K3^{obs}}{K3[obs]} and \eqn{K4^{obs}}{K4[obs]} (and
  the associated 95\% probability intervals), the probabilities that
  \eqn{K3^{obs} > K3^{rep}}{K3[obs] > K3[rep]} and
  \eqn{K4^{obs} > K4^{rep}}{K4[obs] > K4[rep]}, and whether or not
  multivariate normality is indicated. Non-normality is reported when 
  the observed process is greater than the replicated process in either
  2.5\% or 97.5\% of the samples. \code{Mardia} requires \code{Data} to
  be specified, and also requires that variable \code{Y} exist in the
  data set with exactly that name. \code{Y} must be a \eqn{N \times P}{N
  x P} matrix of \eqn{N}{N} records and \eqn{P}{P} variables. Source
  code was modified from the deprecated package QRMlib.
  
  \code{Predictive Quantiles} plots compare y with the predictive
  quantile (PQ) of its replicate. This may be useful in looking for
  patterns with outliers. Instances outside of the gray lines are
  considered outliers.

  \code{Residual Density} plots the residual density of the median of
  the samples. A vertical red line occurs at zero. This plot may be
  useful for inspecting a distributional assumption of residual
  variance. This plot is appropriate when y is univariate and
  continuous.

  \code{Residual Density, Multivariate C} requires \code{Data} to be
  specified, and also requires that variable \code{Y} exist in the data
  set with exactly that name. These are column-wise plots of residual
  density, given the median of the samples. These plots may be useful
  for inspecting a distributional assumption of residual variance.
  This plot is appropriate when Y is multivariate, continuous, and
  densities are desired to be seen column-wise.

  \code{Residual Density, Multivariate R} requires \code{Data} to be
  specified, and also requires that variable \code{Y} exist in the data
  set with exactly that name. These are row-wise plots of residual
  density, given the median of the samples. These plots may be useful
  for inspecting a distributional assumption of residual variance.
  This plot is appropriate when Y is multivariate, continuous, and
  densities are desired to be seen row-wise.

  \code{Residuals} plots compare y with its residuals. The probability
  interval is plotted as a line. This plot is appropriate when y
  is univariate.

  \code{Residuals, Multivariate, C} requires \code{Data} to be
  specified, and also requires that variable \code{Y} exist in the data
  set with exactly that name. These are plots of each column-wise
  vector of residuals. The probability interval is plotted as a
  line. This plot is appropriate when Y is multivariate, not
  categorical, and the residuals are desired to be seen column-wise.

  \code{Residuals, Multivariate, R} requires \code{Data} to be
  specified, and also requires that variable \code{Y} exist in the data
  set with exactly that name. These are plots of each row-wise
  vector of residuals. The probability interval is plotted as a
  line. This plot is appropriate when Y is multivariate, not
  categorical, and the residuals are desired to be seen row-wise.
  
  \code{Space-Time by Space} requires \code{Data} to be specified, and
  also requires that the following variables exist in the data set with
  exactly these names: \code{latitude}, \code{longitude}, \code{S}, and
  \code{T}. These space-time plots compare the S x T matrix Y with the S
  x T matrix Yrep, producing one time-series plot per point s in space,
  for a total of S plots. Therefore, these are time-series plots for
  each point s in space across T time-periods. See \code{Time-Series}
  plots below.

  \code{Space-Time by Time} requires \code{Data} to be specified, and
  also requires that the following variables exist in the data set with
  exactly these names: \code{latitude}, \code{longitude}, \code{S}, and
  \code{T}. These space-time plots compare the S x T matrix Y with the S
  x T matrix Yrep, producing one spatial plot per time-period, and T
  plots will be produced. See \code{Spatial} plots below.

  \code{Spatial} requires \code{Data} to be specified, and also requires
  that the following variables exist in the data set with exactly these
  names: \code{latitude} and \code{longitude}. This spatial plot shows
  yrep plotted according to its coordinates, and is color-coded so that
  higher values of yrep become more red, and lower values become more
  yellow.

  \code{Spatial Uncertainty} requires \code{Data} to be specified, and
  also requires that the following variables exist in the data set with
  exactly these names: \code{latitude} and \code{longitude}. This
  spatial plot shows the probability interval of yrep plotted according
  to its coordinates, and is color-coded so that wider probability
  intervals become more red, and lower values become more yellow.

  \code{Time-Series} plots compare y with its replicate, including the
  median and probability interval quantiles. This plot is appropriate
  when y is univariate and ordered by time.

  \code{Time-Series, Multivariate, C} requires \code{Data} to be
  specified, and also requires that variable \code{Y} exist in the data
  set with exactly that name. These plots compare each column-wise
  time-series in Y with its replicate, including the median and
  probability interval quantiles. This plot is appropriate when y is
  multivariate and each time-series is indexed by column in Y.

  \code{Time-Series, Multivariate, R} requires \code{Data} to be
  specified, and also requires that variable \code{Y} exist in the data
  set with exactly that name. These plots compare each row-wise
  time-series in Y with its replicate, including the median and
  probability interval quantiles. This plot is appropriate when y is
  multivariate and each time-series is indexed by row in Y, such as is
  typically true in panel models.
}
\references{
  Durbin, J., and Watson, G.S. (1950). "Testing for Serial Correlation
  in Least Squares Regression, I." \emph{Biometrika}, 37, p. 409--428.
  
  Jarque, C.M. and Bera, A.K. (1980). "Efficient Tests for Normality,
  Homoscedasticity and Serial Independence of Regression Residuals".
  \emph{Economics Letters}, 6(3), p. 255--259.

  Mardia, K.V. (1970). "Measures of Multivariate Skewness and Kurtosis
  with Applications". \emph{Biometrika}, 57(3), p. 519--530.
}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{LaplacesDemon}} and
  \code{\link{predict.demonoid}}.
}
\examples{### See the LaplacesDemon function for an example.}
\keyword{Plot}

\name{Geweke.Diagnostic}
\alias{Geweke.Diagnostic}
\title{Geweke's Convergence Diagnostic}
\description{
  Geweke (1992) proposed a convergence diagnostic for Markov chains.
  This diagnostic is based on a test for equality of the means of the
  first and last part of a Markov chain (by default the first 10\% and
  the last 50\%). If the samples are drawn from a stationary
  distribution of the chain, then the two means are equal and Geweke's
  statistic has an asymptotically standard normal distribution.

  The test statistic is a standard Z-score: the difference between the
  two sample means divided by its estimated standard error. The
  standard error is estimated from the spectral density at zero, and so
  takes into account any autocorrelation.

  The Z-score is calculated under the assumption that the two parts of
  the chain are asymptotically independent.

  The \code{Geweke.Diagnostic} is a univariate diagnostic that is
  usually applied to each marginal posterior distribution. A
  multivariate form is not included. By chance alone due to multiple
  independent tests, 5\% of the marginal posterior distributions should
  appear non-stationary when stationarity exists. Assessing multivariate
  convergence is difficult.
}
\usage{Geweke.Diagnostic(x)}
\arguments{
  \item{x}{This required argument is a vector or matrix of posterior
    samples, such as from the output of the \code{\link{LaplacesDemon}}
    function. Each column vector in a matrix is a chain to be assessed.
    A minimum of 100 samples are required.}
}
\details{  
  The \code{Geweke.Diagnostic} is essentially the same as the
  \code{geweke.diag} function in the \code{coda} package, but
  programmed to accept a simple vector or matrix, so it does not require
  an \code{mcmc} object.
}
\value{
  A vector is returned, in which each element is a Z-score for a test of
  equality that compares the means of the first and last parts of each
  chain supplied as \code{x} to \code{Geweke.Diagnostic}.
}
\references{
  Geweke, J. (1992). "Evaluating the Accuracy of Sampling-Based
  Approaches to Calculating Posterior Moments". In
  \emph{Bayesian Statistics 4} (ed JM Bernardo, JO Berger, AP Dawid,
  and AFM Smith). Clarendon Press, Oxford, UK.
}
\seealso{
  \code{\link{burnin}},
  \code{\link{is.stationary}}, and 
  \code{\link{LaplacesDemon}}
}
\examples{
library(LaplacesDemon)
Geweke.Diagnostic(rnorm(100))
Geweke.Diagnostic(matrix(rnorm(100),10,10))
}
\keyword{Diagnostic}
\keyword{MCMC}
\keyword{Stationarity}
\keyword{Utility}

\name{LML}
\alias{LML}
\title{Logarithm of the Marginal Likelihood}
\description{
  This function approximates the logarithm of the marginal likelihood
  (LML), where the marginal likelihood is also called the integrated
  likelihood or the prior predictive distribution of \eqn{\textbf{y}}{y}
  in Bayesian inference. The marginal likelihood is

  \deqn{p(\textbf{y}) = \int p(\textbf{y} | \Theta)p(\Theta) d\Theta}{p(y)
    = integral p(y | Theta)p(Theta) d Theta}

  The prior predictive distribution indicates what \eqn{\textbf{y}}{y}
  should look like, given the model, before \eqn{\textbf{y}}{y} has been
  observed. The presence of the marginal likelihood of
  \eqn{\textbf{y}}{y} normalizes the joint posterior distribution,
  \eqn{p(\Theta|\textbf{y})}{p(Theta|y)}, ensuring it is a proper
  distribution and integrates to one (see \code{\link{is.proper}}). The
  marginal likelihood is the denominator of Bayes' theorem, and is often
  omitted, serving as a constant of proportionality. Several methods of
  approximation are available.
}
\usage{
LML(Model=NULL, Data=NULL, Modes=NULL, theta=NULL, LL=NULL, Covar=NULL,
method="NSIS")
}
\arguments{
  \item{Model}{This is the model specification for the model that was
    updated either in \code{\link{IterativeQuadrature}},
    \code{\link{LaplaceApproximation}}, \code{\link{LaplacesDemon}},
    \code{\link{LaplacesDemon.hpc}}, or \code{\link{VariationalBayes}}.
    This argument is used only with the \code{LME} method.}
  \item{Data}{This is the list of data passed to the model
    specification. This argument is used only with the \code{LME}
    method.}
  \item{Modes}{This is a vector of the posterior modes (or medians, in
    the case of MCMC). This argument is used only with the \code{GD} or
    \code{LME} methods.}
  \item{theta}{This is a matrix of posterior samples (parameters only),
    and is specified only with the \code{GD}, \code{HME}, or
    \code{NSIS} methods.}
  \item{LL}{This is a vector of MCMC samples of the log-likelihood, and
    is specified only with the \code{GD}, \code{HME}, or \code{NSIS}
    methods.}
  \item{Covar}{This argument accepts the covariance matrix of the
    posterior modes, and is used only with the \code{GD} or \code{LME}
    methods.}
  \item{method}{The method may be \code{"GD"}, \code{"HME"},
    \code{"LME"}, or \code{"NSIS"}, and defaults to \code{"NSIS"}.
    \code{"GD"} uses the Gelfand-Dey estimator, \code{"HME"} uses the
    Harmonic Mean Estimator, \code{"LME"} uses the Laplace-Metropolis
    Estimator, and \code{"NSIS"} uses nonparametric self-normalized
    importance sampling (NSIS).}
}
\details{
  Generally, a user of \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, \code{\link{LaplacesDemon.hpc}},
  \code{\link{PMC}}, or \code{\link{VariationalBayes}} does not need to
  use the \code{LML} function, because these methods already include
  it. However, \code{LML} may be called by the user, should the user
  desire to estimate the logarithm of the marginal likelihood with a
  different method, or with non-stationary chains. The
  \code{\link{LaplacesDemon}} and \code{\link{LaplacesDemon.hpc}}
  functions only call \code{LML} when all parameters are stationary, and
  only with non-adaptive algorithms.

  The \code{GD} method, where GD stands for Gelfand-Dey (1994), is a
  modification of the harmonic mean estimator (HME) that results in a
  more stable estimator of the logarithm of the marginal
  likelihood. This method is unbiased, simulation-consistent, and
  usually satisfies the Gaussian central limit theorem.

  The \code{HME} method, where HME stands for harmonic mean estimator,
  of Newton-Raftery (1994) is the easiest, and therefore fastest,
  estimation of the logarithm of the marginal likelihood. However, it is
  an unreliable estimator and should be avoided, because small
  likelihood values can overly influence the estimator, variance is
  often infinite, and the Gaussian central limit theorem is usually not
  satisfied. It is included here for completeness. There is not a
  function in this package that uses this method by default. Given
  \eqn{N} samples, the estimator is
  \eqn{1/[\frac{1}{N} \sum_N \exp(-LL)]}{1 / [1/N sum_N exp(-LL)]}.

  The \code{LME} method uses the Laplace-Metropolis Estimator (LME), in
  which the estimation of the Hessian matrix is approximated
  numerically. It is the slowest method here, though it returns an
  estimate in more cases than the other methods. The supplied
  \code{Model} specification must be executed a number of times equal
  to \eqn{k^2 \times 4}{k^2 x 2}, where \eqn{k} is the number of
  parameters. In large dimensions, this is very slow. The
  Laplace-Metropolis Estimator is inappropriate with hierarchical
  models. The \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}}, and \code{\link{VariationalBayes}}
  functions use \code{LME} when it has converged and \code{sir=FALSE},
  in which case it uses the posterior means or modes, and is itself
  Laplace Approximation.

  The Laplace-Metropolis Estimator (LME) is the logarithmic form of
  equation 4 in Lewis and Raftery (1997). In a non-hierarchical model,
  the marginal likelihood may easily be approximated with the
  Laplace-Metropolis Estimator for model \eqn{m} as
  
  \deqn{p(\textbf{y}|m) =
  (2\pi)^{d_m/2}|\Sigma_m|^{1/2}p(\textbf{y}|\Theta_m,m)p(\Theta_m|m)}{p(y|m)
  = (2*pi)^(d_m/2) |Sigma_m|^(1/2) p(y|Theta_m, m)p(Theta_m|m)}

  where \eqn{d} is the number of parameters and \eqn{\Sigma}{Sigma} is
  the inverse of the negative of the approximated Hessian matrix of
  second derivatives.
  
  As a rough estimate of Kass and Raftery (1995), LME is worrisome when
  the sample size of the data is less than five times the number of
  parameters, and LME should be adequate in most problems when the
  sample size of the data exceeds twenty times the number of parameters
  (p. 778).

  The \code{NSIS} method is essentially the \code{MarginalLikelihood}
  function in the \code{MargLikArrogance} package. After \code{HME},
  this is the fastest method available here. The
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}}, and \code{\link{VariationalBayes}}
  functions use \code{NSIS} when converged and \code{sir=TRUE}. The
  \code{\link{LaplacesDemon}}, \code{\link{LaplacesDemon.hpc}}, and
  \code{\link{PMC}} functions use \code{NSIS}. At least 301 stationary
  samples are required, and the number of parameters cannot exceed half
  the number of stationary samples.
}
\value{
  \code{LML} returns a list with two components:
  \item{LML}{
    This is an approximation of the logarithm of the marginal
    likelihood (LML), which is notoriously difficult to estimate. For this
    reason, several methods are provided. The marginal likelihood is
    useful when comparing models, such as with Bayes factors in the
    \code{\link{BayesFactor}} function. When the method fails, \code{NA}
    is returned, and it is most likely that the joint posterior is
  improper (see \code{\link{is.proper}}).}
  \item{VarCov}{
    This is a variance-covariance matrix, and is the negative inverse of
    the Hessian matrix, if estimated. The \code{GD}, \code{HME}, and
    \code{NSIS} methods do not estimate \code{VarCov}, and return
    \code{NA}.}
}
\references{
  Gelfand, A.E. and Dey, D.K. (1994). "Bayesian Model Choice:
  Asymptotics and Exact Calculations". \emph{Journal of the Royal
  Statistical Society}, Series B 56, p. 501--514.

  Kass, R.E. and Raftery, A.E. (1995). "Bayes Factors". \emph{Journal
  of the American Statistical Association}, 90(430), p. 773--795.

  Lewis, S.M. and Raftery, A.E. (1997). "Estimating Bayes Factors via
  Posterior Simulation with the Laplace-Metropolis Estimator".
  \emph{Journal of the American Statistical Association}, 92,
  p. 648--655.

  Newton, M.A. and Raftery, A.E. (1994). "Approximate Bayesian
  Inference by the Weighted Likelihood Bootstrap". \emph{Journal of the
  Royal Statistical Society}, Series B 3, p. 3--48.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{BayesFactor}},
  \code{\link{is.proper}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{LaplacesDemon.hpc}},
  \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
### If a model object were created and called Fit, then:
#
### Applying HME to an object of class demonoid or pmc:
#LML(LL=Fit$Deviance*(-1/2), method="HME")
#
### Applying LME to an object of class demonoid:
#LML(Model, MyData, Modes=apply(Fit$Posterior1, 2, median), method="LME")
#
### Applying NSIS to an object of class demonoid
#LML(theta=Fit$Posterior1, LL=Fit$Deviance*-(1/2), method="NSIS")
#
### Applying LME to an object of class iterquad:
#LML(Model, MyData, Modes=Fit$Summary1[,1], method="LME")
#
### Applying LME to an object of class laplace:
#LML(Model, MyData, Modes=Fit$Summary1[,1], method="LME")
#
### Applying LME to an object of class vb:
#LML(Model, MyData, Modes=Fit$Summary1[,1], method="LME")
}
\keyword{Harmonic Mean Estimator}
\keyword{Integrated Likelihood}
\keyword{Laplace-Metropolis Estimator}
\keyword{Marginal Likelihood}
\keyword{Nonparametric Self-Normalized Importance Sampling}
\keyword{Prior Predictive Distribution}

\name{predict.pmc}
\alias{predict.pmc}
\title{Posterior Predictive Checks}
\description{
  This may be used to predict either new, unobserved instances of
  \eqn{\textbf{y}}{y} (called \eqn{\textbf{y}^{new}}{y[new]}) or
  replicates of \eqn{\textbf{y}}{y} (called
  \eqn{\textbf{y}^{rep}}{y[rep]}), and then perform posterior
  predictive checks. Either \eqn{\textbf{y}^{new}}{y[new]} or
  \eqn{\textbf{y}^{rep}}{y[rep]} is predicted given an object of
  class \code{demonoid}, the model specification, and data.
}
\usage{\method{predict}{pmc}(object, Model, Data, CPUs=1, Type="PSOCK", \dots)}
\arguments{
  \item{object}{An object of class \code{pmc} is required.}
  \item{Model}{The model specification function is required.}
  \item{Data}{A data set in a list is required. The dependent variable
    is required to be named either \code{y} or \code{Y}.}
  \item{CPUs}{This argument accepts an integer that specifies the number
    of central processing units (CPUs) of the multicore computer or
    computer cluster. This argument defaults to \code{CPUs=1}, in which
    parallel processing does not occur.}
  \item{Type}{This argument specifies the type of parallel processing to
    perform, accepting either \code{Type="PSOCK"} or
    \code{Type="MPI"}.}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  This function passes each iteration of marginal posterior samples
  along with data to \code{Model}, where the fourth component in the
  return list is labeled \code{yhat}, and is a vector of expectations of
  \eqn{\textbf{y}}{y}, given the samples, model specification, and
  data. Stationary samples are used if detected, otherwise
  non-stationary samples will be used. To predict
  \eqn{\textbf{y}^{rep}}{y[rep]}, simply supply the data set used to
  estimate the model. To predict \eqn{\textbf{y}^{new}}{y[new]}, supply
  a new data set instead (though for some model specifications, this
  cannot be done, and \eqn{\textbf{y}_{new}}{y[new]} must be specified
  in the \code{Model} function). If the new data set does not have
  \eqn{\textbf{y}}{y}, then create \code{y} in the list and set it
  equal to something sensible, such as \code{mean(y)} from the
  original data set.

  The variable \code{y} must be a vector. If instead it is matrix
  \code{Y}, then it will be converted to vector \code{y}. The vectorized
  length of \code{y} or \code{Y} must be equal to the vectorized length
  of \code{yhat}, the fourth component of the return list of the
  \code{Model} function.

  Parallel processing may be performed when the user specifies
  \code{CPUs} to be greater than one, implying that the specified number
  of CPUs exists and is available. Parallelization may be performed on a
  multicore computer or a computer cluster. Either a Simple Network of
  Workstations (SNOW) or Message Passing Interface is used (MPI). With
  small data sets and few samples, parallel processing may be slower,
  due to computer network communication. With larger data sets and more
  samples, the user should experience a faster run-time.

  For more information on posterior predictive checks, see
  \url{https://web.archive.org/web/20150215050702/http://www.bayesian-inference.com/posteriorpredictivechecks}.
}
\value{
  This function returns an object of class \code{pmc.ppc} (where
  ppc stands for posterior predictive checks). The returned object is
  a list with the following components:

  \item{y}{This stores the vectorized form of \eqn{\textbf{y}}{y}, the
    dependent variable.}
  \item{yhat}{This is a \eqn{N \times S}{N x S} matrix, where \eqn{N} is
    the number of records of \eqn{\textbf{y}}{y} and \eqn{S} is the
    number of posterior samples.}
  \item{Deviance}{This is a vector of predictive deviance.}
}
\author{Statisticat, LLC.}
\seealso{
  \code{\link{PMC}}
}
\keyword{High Performance Computing}
\keyword{Posterior Predictive Checks}
\keyword{Predict}

\name{interval}
\alias{interval}
\title{Constrain to Interval}
\description{
  This function constrains the value(s) of a scalar, vector, matrix, or
  array to a specified interval, \eqn{[a,b]}. In Bayesian inference, it
  is often used both to truncate a parameter to an interval, such as
  \eqn{p(\theta) \in [a,b]}{p(theta) in [a,b]}. The \code{interval}
  function is often used in conjunction with the \code{\link{dtrunc}} 
  function to truncate the prior probability distribution associated
  with the constrained parameter. While \code{\link{dtrunc}} prevents
  assigning density outside of its interval and re-estimates density
  within the interval, the \code{interval} function is used to prevent
  the parameter from moving outside of the interval in the first place.

  After the parameter is constrained to an interval in
  \code{\link{IterativeQuadrature}}, \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, \code{\link{PMC}}, or
  \code{\link{VariationalBayes}}, the constrained parameter should be
  updated back into the \code{parm} vector, so the algorithm knows it
  has been constrained.

  This is unrelated to the probability interval (see
  \code{\link{p.interval}} and \code{\link{LPL.interval}}).
}
\usage{
interval(x, a=-Inf, b=Inf, reflect=TRUE)
}
\arguments{
  \item{x}{This required argument is a scalar, vector, matrix or array,
    and its elements will be constrained to the interval
    [\code{a},\code{b}].}
  \item{a}{This optional argument allows the specification of the lower
    bound of the interval, and defaults to \code{-Inf}.}
  \item{b}{This optional argument allows the specification of the upper
    bound of the interval, and defaults to \code{Inf}.}
  \item{reflect}{Logical. When \code{TRUE}, a value outside of the
  constrained interval is reflected or bounced back into the
  interval. When \code{FALSE}, a value outside of the interval is
  assigned the nearest boundary of the interval. This argument defaults
  to \code{TRUE}.}
}
\details{
  It is common for a parameter to be constrained to an interval. The
  \code{interval} function provides two methods of constraining
  proposals. The default is to reflect an out-of-bounds proposal off of
  the boundaries until the proposal is within the specified
  interval. This is rare in the literature but works very well in
  practice. The other method does not reflect off of boundaries, but
  sets the value equal to the violated boundary. This is also rare in
  the literature and is not generally recommended.

  If the \code{interval} function is unacceptable, then there are
  several alternatives.

  It is common to re-parameterize by transforming the constrained
  parameter to the real line. For example, a positive-only scale
  parameter may be log-transformed. A parameter that is re-parameterized
  to the real line often mixes better in MCMC, exhibiting a higher
  effective sample size (\code{\link{ESS}}), and each evaluation of the
  model specification function is faster as well. However, without a
  hard constraint, it remains possible for the transformed parameter
  still become problematic, such as a log-transformed scale parameter
  that reaches negative infinity. This is much more common in the
  literature.

  Another method is to allow the parameters to move outside of the
  desired, constrained interval in MCMC during the model update, and
  when the model update is finished, to discard any samples outside of
  the constraint boundaries. This is a method of rejecting unacceptable
  proposals in regions of zero probability. However, it is possible for
  parameters to remain outside of acceptable bounds long enough to be
  problematic.

  In \code{\link{LaplacesDemon}}, the Gibbs sampler allows more control
  in the FC function, where a user can customize how constraints are
  handled.
}
\value{
  The \code{interval} function returns a scalar, vector, matrix, or
  array in accord with its argument, \code{x}. Each element is
  constrained to the interval [\code{a},\code{b}].
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dtrunc}},
  \code{\link{ESS}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{LPL.interval}},
  \code{\link{PMC}},
  \code{\link{p.interval}},
  \code{\link{VariationalBayes}}.
}
\examples{
#See the Examples vignette for numerous examples.
library(LaplacesDemon)
x <- 2
interval(x,0,1)
X <- matrix(runif(25,-2,2),5,5)
interval(X,-1,1)
}
\keyword{Utility}

\name{dist.Power.Exponential}
\alias{dpe}
\alias{ppe}
\alias{qpe}
\alias{rpe}
\title{Power Exponential Distribution: Univariate Symmetric}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the univariate, symmetric, power
  exponential distribution with location parameter \eqn{\mu}{mu}, scale
  parameter \eqn{\sigma}{sigma}, and kurtosis parameter
  \eqn{\kappa}{kappa}.
}
\usage{
dpe(x, mu=0, sigma=1, kappa=2, log=FALSE)
ppe(q, mu=0, sigma=1, kappa=2, lower.tail=TRUE, log.p=FALSE)
qpe(p, mu=0, sigma=1, kappa=2, lower.tail=TRUE, log.p=FALSE)
rpe(n, mu=0, sigma=1, kappa=2)
}
\arguments{
  \item{x, q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{mu}{This is the location parameter \eqn{\mu}{mu}.}
  \item{sigma}{This is the scale parameter \eqn{\sigma}{sigma}, which
    must be positive.}
  \item{kappa}{This is the kurtosis parameter \eqn{\kappa}{kappa}, which
    must be positive.}
  \item{log,log.p}{Logical. If \code{log=TRUE}, then the logarithm of the
    density or result is returned.}
  \item{lower.tail}{Logical. If \code{lower.tail=TRUE} (default),
    probabilities are \eqn{Pr[X \le x]}{Pr[X <= x]}, otherwise,
    \eqn{Pr[X > x]}{Pr[X > x]}.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density: \eqn{p(\theta) = \frac{1}{2 \kappa^{1/\kappa}
    \Gamma(1+\frac{1}{\kappa}) \sigma}
  \exp(-\frac{|\theta-\mu|^{\kappa}}{\kappa \sigma^\kappa})}{p(theta) = 1/(2
  kappa^(1/kappa) Gamma(1+1/kappa) sigma) * exp(-(|theta -
  mu|^kappa)/(kappa sigma^kappa))}
  \item Inventor: Subbotin, M.T. (1923)
  \item Notation 1: \eqn{\theta \sim \mathcal{PE}(\mu, \sigma, \kappa)}{theta ~
    PE(mu, sigma, kappa)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{PE}(\theta | \mu,
    \sigma, \kappa)}{p(theta) = PE(theta | mu, sigma, kappa)}
  \item Parameter 1: location parameter \eqn{\mu}{mu}
  \item Parameter 2: scale parameter \eqn{\sigma > 0}{sigma > 0}
  \item Parameter 3: kurtosis parameter \eqn{\kappa > 0}{kappa > 0}
  \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}
  \item Variance: \eqn{var(\theta) = }{var(theta) = }
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The power exponential distribution is also called the exponential power
distribution, generalized error distribution, generalized Gaussian
distribution, and generalized normal distribution. The original form was
introduced by Subbotin (1923) and re-parameterized by Lunetta
(1963). These functions use the more recent parameterization by Lunetta
(1963). A shape parameter, \eqn{\kappa > 0}{kappa > 0}, is added to the
normal distribution. When \eqn{\kappa=1}{kappa=1}, the power exponential
distribution is the same as the Laplace distribution. When
\eqn{\kappa=2}{kappa=2}, the power exponential distribution is the same
as the normal distribution. As \eqn{\kappa \rightarrow \infty}{kappa ->
  infinity}, this becomes a uniform distribution \eqn{\in (\mu-\sigma,
  \mu+\sigma)}{in (mu-sigma, mu+sigma)}. Tails that are heavier than
normal occur when \eqn{\kappa < 2}{kappa < 2}, or lighter than normal
when \eqn{\kappa > 2}{kappa > 2}. This distribution is univariate and
symmetric, and there exist multivariate and asymmetric versions.

These functions are similar to those in the \code{normalp} package.
}
\value{
  \code{dpe} gives the density,
  \code{ppe} gives the distribution function,
  \code{qpe} gives the quantile function, and
  \code{rpe} generates random deviates.
}
\references{
  Lunetta, G. (1963). "Di una Generalizzazione dello Schema della Curva
  Normale". \emph{Annali della Facolt`a di Economia e Commercio di
  Palermo}, 17, p. 237--244.

  Subbotin, M.T. (1923). "On the Law of Frequency of Errors".
  \emph{Matematicheskii Sbornik}, 31, p. 296--301.
}
\seealso{
  \code{\link{dlaplace}},
  \code{\link{dlaplacep}},
  \code{\link{dmvpe}},
  \code{\link{dnorm}},
  \code{\link{dnormp}},
  \code{\link{dnormv}}, and
  \code{\link{dunif}}.
}
\examples{
library(LaplacesDemon)
x <- dpe(1,0,1,2)
x <- ppe(1,0,1,2)
x <- qpe(0.5,0,1,2)
x <- rpe(100,0,1,2)

#Plot Probability Functions
x <- seq(from=0.1, to=3, by=0.01)
plot(x, dpe(x,0,1,0.1), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dpe(x,0,1,2), type="l", col="green")
lines(x, dpe(x,0,1,5), type="l", col="blue")
legend(1.5, 0.9, expression(paste(mu==0, ", ", sigma==1, ", ", kappa==0.1),
     paste(mu==0, ", ", sigma==1, ", ", kappa==2),
     paste(mu==0, ", ", sigma==1, ", ", kappa==5)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{dist.Asymmetric.Laplace}
\alias{dalaplace}
\alias{palaplace}
\alias{qalaplace}
\alias{ralaplace}
\title{Asymmetric Laplace Distribution: Univariate}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the univariate, asymmetric Laplace
  distribution with location parameter \code{location}, scale parameter
  \code{scale}, and asymmetry or skewness parameter \code{kappa}.
}
\usage{
dalaplace(x, location=0, scale=1, kappa=1, log=FALSE)
palaplace(q, location=0, scale=1, kappa=1)
qalaplace(p, location=0, scale=1, kappa=1)
ralaplace(n, location=0, scale=1, kappa=1)
}
\arguments{
  \item{x, q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{location}{This is the location parameter \eqn{\mu}{mu}.}
  \item{scale}{This is the scale parameter \eqn{\lambda}{lambda}, which
    must be positive.}
  \item{kappa}{This is the asymmetry or skewness parameter
    \eqn{\kappa}{kappa}, which must be positive.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density: \eqn{p(\theta) = \frac{\kappa \sqrt{2}}{\lambda
      (1+\kappa^2)} \exp(-|\theta-\mu| \frac{\sqrt{2}}{\lambda}
    \kappa^{|\theta-\mu|} |\theta-\mu|)}{((kappa*sqrt(2)) / (lambda*(1+kappa^2))) * exp(-|x-mu| * (sqrt(2)/lambda) * kappa^(|x-mu|) * |x-mu|)}
  \item Inventor: Kotz, Kozubowski, and Podgorski (2001)
  \item Notation 1: \eqn{\theta \sim \mathcal{AL}(\mu, \lambda,
    \kappa)}{theta ~ AL(mu, lambda, kappa)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{AL}(\theta | \mu, \lambda,
  \kappa)}{p(theta) = AL(theta | mu, lambda, kappa)}
  \item Parameter 1: location parameter \eqn{\mu}{mu}
  \item Parameter 2: scale parameter \eqn{\lambda > 0}{lambda > 0}
  \item Parameter 3: skewness parameter \eqn{\kappa > 0}{kappa > 0}
  \item Mean: \eqn{E(\theta) = \mu + \lambda \frac{1/\kappa - \kappa}{\sqrt{2}}}{E(theta) = mu + lambda*(1/kappa - kappa) / sqrt(2)}
  \item Variance: \eqn{var(\theta) = \lambda^2 \frac{1 + \kappa^4}{2 \kappa^2}}{var(theta) = lambda^2 * (1 + kappa^4) / (2*kappa^2)}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The asymmetric Laplace of Kotz, Kozubowski, and Podgorski (2001), also
referred to as AL, is an extension of the univariate, symmetric Laplace
distribution to allow for skewness. It is parameterized according to
three parameters: location parameter \eqn{\mu}{mu}, scale parameter
\eqn{\lambda}{lambda}, and asymmetry or skewness parameter
\eqn{\kappa}{kappa}. The special case of \eqn{\kappa=1}{kappa=1} is the
symmetric Laplace distribution. Values of \eqn{\kappa}{kappa} in the
intervals \eqn{(0, 1)}{(0,1)} and \eqn{(1, \infty)}{(1, Inf)},
correspond to positive (right) and negative (left) skewness,
respectively. The AL distribution is leptokurtic, and its kurtosis
ranges from 3 to 6 as \eqn{\kappa}{kappa} ranges from 1 to infinity. The
skewness of the AL has been useful in engineering and finance. As an
example, the AL distribution has been used as a replacement for
Gaussian-distributed GARCH residuals. There is also an extension to the
asymmetric multivariate Laplace distribution.

The asymmetric Laplace distribution is demonstrated in Kozubowski and
Podgorski (2001) to be well-suited for financial modeling, specifically
with currency exchange rates.

These functions are similar to those in the \code{VGAM} package.
}
\value{
  \code{dalaplace} gives the density,
  \code{palaplace} gives the distribution function,
  \code{qalaplace} gives the quantile function, and
  \code{ralaplace} generates random deviates.
}
\references{
  Kotz, S., Kozubowski, T.J., and Podgorski, K. (2001). "The Laplace
  Distribution and Generalizations: a Revisit with Applications to
  Communications, Economics, Engineering, and Finance". Boston:
  Birkhauser.

  Kozubowski, T.J. and Podgorski, K. (2001). "Asymmetric Laplace
  Laws and Modeling Financial Data". \emph{Mathematical and Computer
  Modelling}, 34, p. 1003-1021.
}
\seealso{\code{\link{dlaplace}} and
  \code{\link{dallaplace}}
}
\examples{
library(LaplacesDemon)
x <- dalaplace(1,0,1,1)
x <- palaplace(1,0,1,1)
x <- qalaplace(0.5,0,1,1)
x <- ralaplace(100,0,1,1)

#Plot Probability Functions
x <- seq(from=-5, to=5, by=0.1)
plot(x, dalaplace(x,0,1,0.5), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dalaplace(x,0,1,1), type="l", col="green")
lines(x, dalaplace(x,0,1,5), type="l", col="blue")
legend(1, 0.9, expression(paste(mu==0, ", ", lambda==1, ", ", kappa==0.5),
     paste(mu==0, ", ", lambda==1, ", ", kappa==1),
     paste(mu==0, ", ", lambda==1, ", ", kappa==5)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{LaplaceApproximation}
\alias{LaplaceApproximation}
\title{Laplace Approximation}
\description{
  The \code{LaplaceApproximation} function deterministically maximizes
  the logarithm of the unnormalized joint posterior density with one of
  several optimization algorithms. The goal of Laplace Approximation is
  to estimate the posterior mode and variance of each parameter. This
  function is useful for optimizing initial values and estimating a
  covariance matrix to be input into the
  \code{\link{IterativeQuadrature}}, \code{\link{LaplacesDemon}},
  \code{\link{PMC}}, or \code{\link{VariationalBayes}} function, or
  sometimes for model estimation in its own right.
}
\usage{
LaplaceApproximation(Model, parm, Data, Interval=1.0E-6,
     Iterations=100, Method="SPG", Samples=1000, CovEst="Hessian",
     sir=TRUE, Stop.Tolerance=1.0E-5, CPUs=1, Type="PSOCK")
}
\arguments{
  \item{Model}{This required argument receives the model from a
    user-defined function. The user-defined function is where the model
    is specified. \code{LaplaceApproximation} passes two arguments to
    the model function, \code{parms} and \code{Data}. For more
    information, see the \code{\link{LaplacesDemon}} function and
    ``LaplacesDemon Tutorial'' vignette.}
  \item{parm}{This argument requires a vector of initial values equal in
    length to the number of parameters. \code{LaplaceApproximation} will
    attempt to optimize these initial values for the parameters, where
    the optimized values are the posterior modes, for later use with the
    \code{\link{IterativeQuadrature}}, \code{\link{LaplacesDemon}},
    \code{\link{PMC}}, or the \code{\link{VariationalBayes}} function.
    The \code{\link{GIV}} function may be used to randomly generate
    initial values. Parameters must be continuous.}
  \item{Data}{This required argument accepts a list of data. The list of
    data must include \code{mon.names} which contains monitored variable
    names, and \code{parm.names} which contains parameter
    names. \code{LaplaceApproximation} must be able to determine the
    sample size of the data, and will look for a scalar sample size
    variable \code{n} or \code{N}. If not found, it will look for
    variable \code{y} or \code{Y}, and attempt to take its number of
    rows as sample size. \code{LaplaceApproximation} needs to determine
    sample size due to the asymptotic nature of this method. Sample size
    should be at least \eqn{\sqrt{J}}{sqrt(J)} with \eqn{J} exchangeable
    parameters.}
  \item{Interval}{This argument receives an interval for estimating
    approximate gradients. The logarithm of the unnormalized joint
    posterior density of the Bayesian model is evaluated at the current
    parameter value, and again at the current parameter value plus this
    interval.}
  \item{Iterations}{This argument accepts an integer that determines the
    number of iterations that \code{LaplaceApproximation} will attempt
    to maximize the logarithm of the unnormalized joint posterior
    density. \code{Iterations} defaults to 100.
    \code{LaplaceApproximation} will stop before this number of
    iterations if the tolerance is less than or equal to the
    \code{Stop.Tolerance} criterion. The required amount of computer
    memory increases with \code{Iterations}. If computer memory is
    exceeded, then all will be lost.}
  \item{Method}{This optional argument accepts a quoted string that
    specifies the method used for Laplace Approximation. The default
    method is \code{Method="SPG"}. Options include \code{"AGA"} for
    adaptive gradient ascent, \code{"BFGS"} for the
    Broyden-Fletcher-Goldfarb-Shanno algorithm, \code{"BHHH"} for the
    algorithm of Berndt et al., \code{"CG"} for conjugate gradient,
    \code{"DFP"} for the Davidon-Fletcher-Powell algorithm,
    \code{"HAR"} for adaptive hit-and-run, \code{"HJ"} for
    Hooke-Jeeves, \code{"LBFGS"} for limited-memory BFGS, \code{"LM"}
    for Levenberg-Marquardt, \code{"NM"} for Nelder-Mead, \code{"NR"}
    for Newton-Raphson, \code{"PSO"} for Particle Swarm Optimization,
    \code{"Rprop"} for resilient backpropagation, \code{"SGD"} for
    Stochastic Gradient Descent, \code{"SOMA"} for the Self-Organizing
    Migration Algorithm, \code{"SPG"} for Spectral Projected Gradient,
    \code{"SR1"} for Symmetric Rank-One, and \code{"TR"} for Trust
    Region.}
  \item{Samples}{This argument indicates the number of posterior samples
    to be taken with sampling importance resampling via the
    \code{\link{SIR}} function, which occurs only when
    \code{sir=TRUE}. Note that the number of samples should increase
    with the number and intercorrelations of the parameters.}
  \item{CovEst}{This argument accepts a quoted string that indicates how
    the covariance matrix is estimated after the model finishes. This
    covariance matrix is used to obtain the standard deviation of each
    parameter, and may also be used for posterior sampling via Sampling
    Importance Resampling (SIR) (see the \code{sir} argument below), if
    converged. By default, the covariance matrix is approximated as the
    negative inverse of the \code{"Hessian"} matrix of second
    derivatives, estimated with Richardson extrapolation. Alternatives
    include \code{CovEst="Identity"}, \code{CovEst="OPG"}, or
    \code{CovEst="Sandwich"}. When \code{CovEst="Identity"}, the
    covariance matrix is not estimated, and is merely assigned an
    identity matrix. When \code{\link{LaplaceApproximation}} is
    performed internally by \code{\link{LaplacesDemon}}, an identity
    matrix is returned and scaled. When \code{CovEst="OPG"}, the
    covariance matrix is approximated with the inverse of the sum of the
    outer products of the gradient, which requires \code{X}, and either
    \code{y} or \code{Y} in the list of data. For OPG, a partial
    derivative is taken for each row in \code{X}, and each element in
    \code{y} or row in \code{Y}. Therefore, this requires \eqn{N + NJ}
    model evaluations for a data set with \eqn{N} records and \eqn{J}
    variables. The OPG method is an asymptotic approximation of
    the Hessian, and usually requires fewer calculations with a small
    data set, or more with large data sets. Both methods require a
    matrix inversion, which becomes costly as dimension grows. The
    Richardson-based Hessian method is more accurate, but requires more
    calculation in large dimensions. An alternative approach to
    obtaining covariance is to use the \code{\link{BayesianBootstrap}}
    on the data, or sample the posterior with iterative quadrature
    (\code{\link{IterativeQuadrature}}), MCMC
    (\code{\link{LaplacesDemon}}), or \code{\link{VariationalBayes}}.}
  \item{sir}{This logical argument indicates whether or not Sampling
    Importance Resampling (SIR) is conducted via the \code{\link{SIR}}
    function to draw independent posterior samples. This argument
    defaults to \code{TRUE}. Even when \code{TRUE}, posterior samples
    are drawn only when \code{LaplaceApproximation} has
    converged. Posterior samples are required for many other functions,
    including \code{plot.laplace} and \code{predict.laplace}. The only
    time that it is advantageous for \code{sir=FALSE} is when
    \code{LaplaceApproximation} is used to help the initial values for
    \code{\link{IterativeQuadrature}}, \code{\link{LaplacesDemon}},
    \code{\link{PMC}}, or \code{\link{VariationalBayes}}, and it is
    unnecessary for time to be spent on sampling. Less time can be spent
    on sampling by increasing \code{CPUs}, which parallelizes the
    sampling.}
  \item{Stop.Tolerance}{This argument accepts any positive number and
    defaults to 1.0E-5. Tolerance is calculated each iteration, and the
    criteria varies by algorithm. The algorithm is considered to have
    converged to the user-specified \code{Stop.Tolerance} when the
    tolerance is less than or equal to the value of
    \code{Stop.Tolerance}, and the algorithm terminates at the end of
    the current iteration. Often, multiple criteria are used, in
    which case the maximum of all criteria becomes the tolerance. For
    example, when partial derivatives are taken, it is commonly required
    that the Euclidean norm of the partial derivatives is a criterion,
    and another common criterion is the Euclidean norm of the
    differences between the current and previous parameter values.
    Several algorithms have other, specific tolerances.}
  \item{CPUs}{This argument accepts an integer that specifies the number
    of central processing units (CPUs) of the multicore computer or
    computer cluster. This argument defaults to \code{CPUs=1}, in which
    parallel processing does not occur. Parallelization occurs only for
    sampling with \code{\link{SIR}} when \code{sir=TRUE}.}
  \item{Type}{This argument specifies the type of parallel processing to
    perform, accepting either \code{Type="PSOCK"} or
    \code{Type="MPI"}.}
}
\details{
  The Laplace Approximation or Laplace Method is a family of asymptotic
  techniques used to approximate integrals. Laplace's method accurately
  approximates unimodal posterior moments and marginal posterior
  distributions in many cases. Since it is not applicable in
  all cases, it is recommended here that Laplace Approximation is used
  cautiously in its own right, or preferably, it is used before MCMC.

  After introducing the Laplace Approximation (Laplace, 1774,
  p. 366--367), a proof was published later (Laplace, 1814) as part of
  a mathematical system of inductive reasoning based on probability.
  Laplace used this method to approximate posterior moments.

  Since its introduction, the Laplace Approximation has been applied
  successfully in many disciplines. In the 1980s, the Laplace
  Approximation experienced renewed interest, especially in statistics,
  and some improvements in its implementation were introduced (Tierney
  et al., 1986; Tierney et al., 1989). Only since the 1980s has the
  Laplace Approximation been seriously considered by statisticians in
  practical applications.

  There are many variations of Laplace Approximation, with an effort
  toward replacing Markov chain Monte Carlo (MCMC) algorithms as the
  dominant form of numerical approximation in Bayesian inference. The
  run-time of Laplace Approximation is a little longer than Maximum
  Likelihood Estimation (MLE), usually shorter than variational Bayes,
  and much shorter than MCMC (Azevedo and Shachter, 1994).

  The speed of Laplace Approximation depends on the optimization
  algorithm selected, and typically involves many evaluations of the
  objective function per iteration (where an MCMC algorithm with a
  multivariate proposal usually evaluates once per iteration), making
  many MCMC algorithms faster per iteration. The attractiveness
  of Laplace Approximation is that it typically improves the objective
  function better than iterative quadrature, MCMC, and PMC when the
  parameters are in low-probability regions. Laplace Approximation is
  also typically faster than MCMC and PMC because it is seeking
  point-estimates, rather than attempting to represent the target
  distribution with enough simulation draws. Laplace Approximation
  extends MLE, but shares similar limitations, such as its asymptotic
  nature with respect to sample size and that marginal posterior
  distributions are Gaussian. Bernardo and Smith (2000) note that
  Laplace Approximation is an attractive family of numerical
  approximation algorithms, and will continue to develop.

  \code{LaplaceApproximation} seeks a global maximum of the logarithm of
  the unnormalized joint posterior density. The approach differs by
  \code{Method}. The \code{\link{LaplacesDemon}} function uses the
  \code{LaplaceApproximation} algorithm to optimize initial values and
  save time for the user.

  Most optimization algorithms assume that the logarithm of the
  unnormalized joint posterior density is defined and differentiable.
  Some methods calculate an approximate gradient for each initial value
  as the difference in the logarithm of the unnormalized joint posterior
  density due to a slight increase in the parameter.

  When \code{Method="AGA"}, the direction and distance for each
  parameter is proposed based on an approximate truncated gradient and
  an adaptive step size. The step size parameter, which is often plural
  and called rate parameters in other literature, is adapted each
  iteration with the univariate version of the Robbins-Monro stochastic
  approximation in Garthwaite (2010). The step size shrinks when a
  proposal is rejected and expands when a proposal is accepted.

  Gradient ascent is criticized for sometimes being relatively slow when
  close to the maximum, and its asymptotic rate of convergence is
  inferior to other methods. However, compared to other popular
  optimization algorithms such as Newton-Raphson, an advantage of the
  gradient ascent is that it works in infinite dimensions, requiring
  only sufficient computer memory. Although Newton-Raphson converges in
  fewer iterations, calculating the inverse of the negative Hessian
  matrix of second-derivatives is more computationally expensive and
  subject to singularities. Therefore, gradient ascent takes longer to
  converge, but is more generalizable.

  When \code{Method="BFGS"}, the BFGS algorithm is used, which was
  proposed by Broyden (1970), Fletcher (1970), Goldfarb (1970), and
  Shanno (1970), independently. BFGS may be the most efficient and
  popular quasi-Newton optimiziation algorithm. As a quasi-Newton
  algorithm, the Hessian matrix is approximated using rank-one updates
  specified by (approximate) gradient evaluations. Since BFGS is very
  popular, there are many variations of it. This is a version by Nash
  that has been adapted from the Rvmmin package, and is used in the
  \code{optim} function of base R. The approximate Hessian is not
  guaranteed to converge to the Hessian. When BFGS is used, the
  approximate Hessian is not used to calculate the final covariance
  matrix.

  When \code{Method="BHHH"}, the algorithm of Berndt et al. (1974) is
  used, which is commonly pronounced B-triple H. The BHHH algorithm is a
  quasi-Newton method that includes a step-size parameter, partial
  derivatives, and an approximation of a covariance matrix that is
  calculated as the inverse of the sum of the outer product of the
  gradient (OPG), calculated from each record. The OPG method becomes
  more costly with data sets with more records. Since partial
  derivatives must be calculated per record of data, the list of data
  has special requirements with this method, and must include design
  matrix \code{X}, and dependent variable \code{y} or \code{Y}. Records
  must be row-wise. An advantage of BHHH over NR (see below) is that
  the covariance matrix is necessarily positive definite, and gauranteed
  to provide an increase in LP each iteration (given a small enough
  step-size), even in convex areas. The covariance matrix is better
  approximated with larger data sample sizes, and when closer to the
  maximum of LP. Disadvantages of BHHH include that it can give small
  increases in LP, especially when far from the maximum or when LP is
  highly non-quadratic.

  When \code{Method="CG"}, a nonlinear conjugate gradient algorithm is
  used. CG uses partial derivatives, but does not use the Hessian matrix
  or any approximation of it. CG usually requires more iterations to
  reach convergence than other algorithms that use the Hessian or an
  approximation. However, since the Hessian becomes computationally
  expensive as the dimension of the model grows, CG is applicable to
  large dimensional models when \code{CovEst="Hessian"} is avoided.
  CG was originally developed by Hestenes and Stiefel (1952), though
  this version is adapted from the \code{Rcgminu} function in package
  Rcgmin.

  When \code{Method="DFP"}, the Davidon-Fletcher-Powell algorithm is
  used. DFP was the first popular, multidimensional, quasi-Newton
  optimization algorithm. The DFP update of an approximate Hessian
  matrix maintains symmetry and positive-definiteness. The approximate
  Hessian is not guaranteed to converge to the Hessian. When DFP is
  used, the approximate Hessian is not used to calculate the final
  covariance matrix. Although DFP is very effective, it was superseded
  by the BFGS algorithm.

  When \code{Method="HAR"}, a hit-and-run algorithm with a multivariate
  proposal and adaptive length is used. The length parameter is adapted
  each iteration with the univariate version of the Robbins-Monro
  stochastic approximation in Garthwaite (2010). The length shrinks when
  a proposal is rejected and expands when a proposal is accepted. This
  is the same algorithm as the HARM or Hit-And-Run Metropolis MCMC
  algorithm with adaptive length, except that a Metropolis step is not
  used.

  When \code{Method="HJ"}, the Hooke-Jeeves (1961) algorithm is used.
  This was adapted from the \code{HJK} algorithm in package dfoptim.
  Hooke-Jeeves is a derivative-free, direct search method. Each iteration
  involves two steps: an exploratory move and a pattern move. The
  exploratory move explores local behavior, and the pattern move takes
  advantage of pattern direction. It is sometimes described as a
  hill-climbing algorithm. If the solution improves, it accepts the
  move, and otherwise rejects it. Step size decreases with each
  iteration. The decreasing step size can trap it in local maxima, where
  it gets stuck and convergences erroneously. Users are encouraged to
  attempt again after what seems to be convergence, starting from the
  latest point. Although getting stuck at local maxima can be
  problematic, the Hooke-Jeeves algorithm is also attractive because it
  is simple, fast, does not depend on derivatives, and is otherwise
  relatively robust.

  When \code{Method="LBFGS"}, the limited-memory BFGS
  (Broyden-Fletcher-Goldfarb-Shanno) algorithm is called in
  \code{optim}, once per iteration.

  When \code{Method="LM"}, the Levenberg-Marquardt algorithm (Levenberg,
  1944; Marquardt, 1963) is used. Also known as the Levenberg-Marquardt
  Algorithm (LMA) or the Damped Least-Squares (DLS) method, LM is a
  trust region (not to be confused with TR below) quasi-Newton
  optimization algorithm that provides minimizes nonlinear least
  squares, and has been adapted here to maximize LP. LM uses partial
  derivatives and approximates the Hessian with outer-products. It is
  suitable for nonlinear optimization up to a few hundred parameters,
  but loses its efficiency in larger problems due to matrix inversion.
  LM is considered between the Gauss-Newton algorithm and gradient
  descent. When far from the solution, LM moves slowly like gradient
  descent, but is guaranteed to converge. When LM is close to the
  solution, LM becomes a damped Gauss-Newton method. This was adapted
  from the \code{lsqnonlin} algorithm in package pracma.

  When \code{Method="NM"}, the Nelder-Mead (1965) algorithm is
  used. This was adapted from the \code{nelder_mead} function in package
  pracma. Nelder-Mead is a derivative-free, direct search method that is
  known to become inefficient in large-dimensional problems. As the
  dimension increases, the search direction becomes increasingly
  orthogonal to the steepest ascent (usually descent)
  direction. However, in smaller dimensions, it is a popular algorithm.
  At each iteration, three steps are taken to improve a simplex:
  reflection, extension, and contraction.

  When \code{Method="NR"}, the Newton-Raphson optimization algorithm,
  also known as Newton's Method, is used. Newton-Raphson uses
  derivatives and a Hessian matrix. The algorithm is included for its
  historical significance, but is known to be problematic when starting
  values are far from the targets, and calculating and inverting the
  Hessian matrix can be computationally expensive. As programmed here,
  when the Hessian is problematic, it tries to use only the derivatives,
  and when that fails, a jitter is applied. Newton-Raphson should not
  be the first choice of the user, and BFGS should always be preferred.

  When \code{Method="PSO"}, the Standard Particle Swarm Optimization
  2007 algorithm is used. A swarm of particles is moved according
  to velocity, neighborhood, and the best previous solution. The
  neighborhood for each particle is a set of informing particles. PSO
  is derivative-free. PSO has been adapted from the \code{psoptim}
  function in package pso.

  When \code{Method="Rprop"}, the approximate gradient is taken for each
  parameter in each iteration, and its sign is compared to the
  approximate gradient in the previous iteration. A weight element in a
  weight vector is associated with each approximate gradient. A weight
  element is multiplied by 1.2 when the sign does not change, or by 0.5
  if the sign changes. The weight vector is the step size, and is
  constrained to the interval [0.001, 50], and initial weights are
  0.0125. This is the resilient backpropagation algorithm, which is
  often denoted as the ``Rprop-'' algorithm of Riedmiller (1994).

  When \code{Method="SGD"}, a stochastic gradient descent algorithm is
  used that is designed only for big data, and gained popularity after
  successful use in the NetFlix competition. This algorithm has special
  requirements for the \code{Model} specification function and the
  \code{Data} list. See the ``LaplacesDemon Tutorial'' vignette for more
  information.

  When \code{Method="SOMA"}, a population of ten particles or
  individuals moves in the direction of the best particle, the leader.
  The leader does not move in each iteration, and a line-search is used
  for each non-leader, up to three times the difference in parameter
  values between each non-leader and leader. This algorithm is
  derivative-free and often considered in the family of evolution
  algorithms. Numerous model evaluations are performed per non-leader
  per iteration. This algorithm was adapted from package soma.

  When \code{Method="SPG"}, a Spectral Projected Gradient algorithm
  is used. SPG is a non-monotone algorithm that is suitable for
  high-dimensional models. The approximate gradient is used, but the
  Hessian matrix is not. When used with large models,
  \code{CovEst="Hessian"} should be avoided. SPG has been adapted from
  the \code{spg} function in package BB.

  When \code{Method="SR1"}, the Symmetric Rank-One (SR1) algorithm is
  used. SR1 is a quasi-Newton algorithm, and the Hessian matrix is
  approximated, often without being positive-definite. At the posterior
  modes, the true Hessian is usually positive-definite, but this is
  often not the case during optimization when the parameters have not
  yet reached the posterior modes. Other restrictions, including
  constraints, often result in the true Hessian being indefinite at the
  solution. For these reasons, SR1 often outperforms BFGS. The
  approximate Hessian is not guaranteed to converge to the Hessian. When
  SR1 is used, the approximate Hessian is not used to calculate the
  final covariance matrix.

  When \code{Method="TR"}, the Trust Region algorithm of Nocedal and
  Wright (1999) is used. The TR algorithm attempts to reach its
  objective in the fewest number of iterations, is therefore very
  efficient, as well as safe. The efficiency of TR is attractive when
  model evaluations are expensive. The Hessian is approximated each
  iteration, making TR best suited to models with small to medium
  dimensions, say up to a few hundred parameters. TR has been adapted
  from the \code{trust} function in package trust.
}
\value{
  \code{LaplaceApproximation} returns an object of class \code{laplace}
  that is a list with the following components:
  \item{Call}{This is the matched call of \code{LaplaceApproximation}.}
  \item{Converged}{This is a logical indicator of whether or not
    \code{LaplaceApproximation} converged within the specified
    \code{Iterations} according to the supplied \code{Stop.Tolerance}
    criterion. Convergence does not indicate that the global maximum has
    been found, but only that the tolerance was less than or equal to
    the \code{Stop.Tolerance} criterion.}
  \item{Covar}{This covariance matrix is estimated according to the
    \code{CovEst} argument. The \code{Covar} matrix may be scaled and
    input into the \code{Covar} argument of the
    \code{\link{LaplacesDemon}} or \code{\link{PMC}} function for
    further estimation, or the diagonal of this matrix may be used
    to represent the posterior variance of the parameters, provided the
    algorithm converged and matrix inversion was successful. To scale
    this matrix for use with Laplace's Demon or PMC, multiply it by
    \eqn{2.38^2/d}, where \eqn{d} is the number of initial values.}
  \item{Deviance}{This is a vector of the iterative history of the
    deviance in the \code{LaplaceApproximation} function, as it sought
    convergence.}
  \item{History}{This is a matrix of the iterative history of the
    parameters in the \code{LaplaceApproximation} function, as it sought
    convergence.}
  \item{Initial.Values}{This is the vector of initial values that was
    originally given to \code{LaplaceApproximation} in the \code{parm}
    argument.}
  \item{LML}{This is an approximation of the logarithm of the marginal
    likelihood of the data (see the \code{\link{LML}} function for more
    information). When the model has converged and \code{sir=TRUE}, the
    NSIS method is used. When the model has converged and
    \code{sir=FALSE}, the LME method is used. This is the
    logarithmic form of equation 4 in Lewis and Raftery (1997). As a
    rough estimate of Kass and Raftery (1995), the LME-based LML is
    worrisome when the sample size of the data is less than five times
    the number of parameters, and \code{LML} should be adequate in most
    problems when the sample size of the data exceeds twenty times the
    number of parameters (p. 778). The LME is inappropriate with
    hierarchical models. However \code{LML} is estimated, it is useful
    for comparing multiple models with the \code{BayesFactor} function.}
  \item{LP.Final}{This reports the final scalar value for the logarithm
    of the unnormalized joint posterior density.}
  \item{LP.Initial}{This reports the initial scalar value for the
    logarithm of the unnormalized joint posterior density.}
  \item{Minutes}{This is the number of minutes that
    \code{LaplaceApproximation} was running, and this includes the
    initial checks as well as drawing posterior samples and creating
    summaries.}
  \item{Monitor}{When \code{sir=TRUE}, a number of independent
    posterior samples equal to \code{Samples} is taken, and the draws
    are stored here as a matrix. The rows of the matrix are the samples,
    and the columns are the monitored variables.}
  \item{Posterior}{When \code{sir=TRUE}, a number of independent
    posterior samples equal to \code{Samples} is taken, and the draws
    are stored here as a matrix. The rows of the matrix are the samples,
    and the columns are the parameters.}
  \item{Step.Size.Final}{This is the final, scalar \code{Step.Size}
    value at the end of the \code{LaplaceApproximation} algorithm.}
  \item{Step.Size.Initial}{This is the initial, scalar \code{Step.Size}.}
  \item{Summary1}{This is a summary matrix that summarizes the
    point-estimated posterior modes. Uncertainty around the posterior
    modes is estimated from the covariance matrix. Rows are parameters.
    The following columns are included: Mode, SD (Standard Deviation),
    LB (Lower Bound), and UB (Upper Bound). The bounds constitute a 95\%
    probability interval.}
  \item{Summary2}{This is a summary matrix that summarizes the
    posterior samples drawn with sampling importance resampling
    (\code{\link{SIR}}) when \code{sir=TRUE}, given the point-estimated
    posterior modes and the covariance matrix. Rows are parameters. The
    following columns are included: Mode, SD (Standard Deviation),
    LB (Lower Bound), and UB (Upper Bound). The bounds constitute a 95\%
    probability interval.}
  \item{Tolerance.Final}{This is the last \code{Tolerance} of the
    \code{LaplaceApproximation} algorithm.}
  \item{Tolerance.Stop}{This is the \code{Stop.Tolerance} criterion.}
}
\references{
  Azevedo-Filho, A. and Shachter, R. (1994). "Laplace's Method
  Approximations for Probabilistic Inference in Belief Networks with
  Continuous Variables". In "Uncertainty in Artificial Intelligence",
  Mantaras, R. and Poole, D., Morgan Kauffman, San Francisco, CA,
  p. 28--36.
     
  Bernardo, J.M. and Smith, A.F.M. (2000). "Bayesian Theory". John
  Wiley & Sons: West Sussex, England.

  Berndt, E., Hall, B., Hall, R., and Hausman, J. (1974), "Estimation
  and Inference in Nonlinear Structural Models". \emph{Annals of
  Economic and Social Measurement}, 3, p. 653--665.

  Broyden, C.G. (1970). "The Convergence of a Class of Double Rank
  Minimization Algorithms: 2. The New Algorithm". Journal of the
  Institute of Mathematics and its Applications, 6, p.76--90.

  Fletcher, R. (1970). "A New Approach to Variable Metric Algorithms".
  Computer Journal, 13(3), p. 317--322.
  
  Garthwaite, P., Fan, Y., and Sisson, S. (2010). "Adaptive Optimal
  Scaling of Metropolis-Hastings Algorithms Using the Robbins-Monro
  Process."

  Goldfarb, D. (1970). "A Family of Variable Metric Methods Derived
  by Variational Means". Mathematics of Computation, 24(109), p. 23--26.

  Hestenes, M.R. and Stiefel, E. (1952). "Methods of Conjugate Gradients
  for Solving Linear Systems". \emph{Journal of Research of the National
  Bureau of Standards}, 49(6), p. 409--436.

  Hooke, R. and Jeeves, T.A. (1961). "'Direct Search' Solution of
  Numerical and Statistical Problems". \emph{Journal of the Association
  for Computing Machinery}, 8(2), p. 212--229. 

  Kass, R.E. and Raftery, A.E. (1995). "Bayes Factors". \emph{Journal
  of the American Statistical Association}, 90(430), p. 773--795.
     
  Laplace, P. (1774). "Memoire sur la Probabilite des Causes par les
  Evenements." l'Academie Royale des Sciences, 6, 621--656. English
  translation by S.M. Stigler in 1986 as "Memoir on the Probability
  of the Causes of Events" in Statistical Science, 1(3), 359--378.

  Laplace, P. (1814). "Essai Philosophique sur les Probabilites."
  English translation in Truscott, F.W. and Emory, F.L. (2007) from
  (1902) as "A Philosophical Essay on Probabilities". ISBN
  1602063281, translated from the French 6th ed. (1840).

  Levenberg, K. (1944). "A Method for the Solution of Certain
  Non-Linear Problems in Least Squares". \emph{Quarterly of Applied
  Mathematics}, 2, p. 164--168.

  Lewis, S.M. and Raftery, A.E. (1997). "Estimating Bayes Factors via
  Posterior Simulation with the Laplace-Metropolis
  Estimator". \emph{Journal of the American Statistical Association},
  92, p. 648--655.

  Marquardt, D. (1963). "An Algorithm for Least-Squares Estimation of
  Nonlinear Parameters". \emph{SIAM Journal on Applied Mathematics},
  11(2), p. 431--441.

  Nelder, J.A. and Mead, R. (1965). "A Simplex Method for Function
  Minimization". \emph{The Computer Journal}, 7(4), p. 308--313.

  Nocedal, J. and Wright, S.J. (1999). "Numerical Optimization".
  Springer-Verlag.

  Riedmiller, M. (1994). "Advanced Supervised Learning in Multi-Layer
  Perceptrons - From Backpropagation to Adaptive Learning
  Algorithms". \emph{Computer Standards and Interfaces}, 16,
  p. 265--278.

  Shanno, D.F. (1970). "Conditioning of quasi-Newton Methods for
  Function Minimization". Mathematics of Computation, 24(111),
  p. 647--650.
     
  Tierney, L. and Kadane, J.B. (1986). "Accurate Approximations for
  Posterior Moments and Marginal Densities". \emph{Journal of the
  American Statistical Association}, 81(393), p. 82--86.

  Tierney, L., Kass. R., and Kadane, J.B. (1989). "Fully Exponential
  Laplace Approximations to Expectations and Variances of Nonpositive
  Functions". \emph{Journal of the American Statistical Association},
  84(407), p. 710--716.

  Zelinka, I. (2004). "SOMA - Self Organizing Migrating Algorithm". In:
  Onwubolu G.C. and Babu, B.V., editors. "New Optimization
  Techniques in Engineering". Springer: Berlin, Germany.

}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{BayesFactor}},
  \code{\link{BayesianBootstrap}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplacesDemon}},
  \code{\link{GIV}},
  \code{\link{LML}},
  \code{\link{optim}},
  \code{\link{PMC}},
  \code{\link{SIR}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
# The accompanying Examples vignette is a compendium of examples.
####################  Load the LaplacesDemon Library  #####################
library(LaplacesDemon)

##############################  Demon Data  ###############################
data(demonsnacks)
y <- log(demonsnacks$Calories)
X <- cbind(1, as.matrix(log(demonsnacks[,10]+1)))
J <- ncol(X)
for (j in 2:J) X[,j] <- CenterScale(X[,j])

#########################  Data List Preparation  #########################
mon.names <- "mu[1]"
parm.names <- as.parm.names(list(beta=rep(0,J), sigma=0))
pos.beta <- grep("beta", parm.names)
pos.sigma <- grep("sigma", parm.names)
PGF <- function(Data) {
     beta <- rnorm(Data$J)
     sigma <- runif(1)
     return(c(beta, sigma))
     }
MyData <- list(J=J, PGF=PGF, X=X, mon.names=mon.names,
     parm.names=parm.names, pos.beta=pos.beta, pos.sigma=pos.sigma, y=y)

##########################  Model Specification  ##########################
Model <- function(parm, Data)
     {
     ### Parameters
     beta <- parm[Data$pos.beta]
     sigma <- interval(parm[Data$pos.sigma], 1e-100, Inf)
     parm[Data$pos.sigma] <- sigma
     ### Log-Prior
     beta.prior <- sum(dnormv(beta, 0, 1000, log=TRUE))
     sigma.prior <- dhalfcauchy(sigma, 25, log=TRUE)
     ### Log-Likelihood
     mu <- tcrossprod(Data$X, t(beta))
     LL <- sum(dnorm(Data$y, mu, sigma, log=TRUE))
     ### Log-Posterior
     LP <- LL + beta.prior + sigma.prior
     Modelout <- list(LP=LP, Dev=-2*LL, Monitor=mu[1],
          yhat=rnorm(length(mu), mu, sigma), parm=parm)
     return(Modelout)
     }

############################  Initial Values  #############################
#Initial.Values <- GIV(Model, MyData, PGF=TRUE)
Initial.Values <- rep(0,J+1)

Fit <- LaplaceApproximation(Model, Initial.Values, Data=MyData,
     Iterations=100, Method="NM", CPUs=1)
Fit
print(Fit)
#PosteriorChecks(Fit)
#caterpillar.plot(Fit, Parms="beta")
#plot(Fit, MyData, PDF=FALSE)
#Pred <- predict(Fit, Model, MyData, CPUs=1)
#summary(Pred, Discrep="Chi-Square")
#plot(Pred, Style="Covariates", Data=MyData)
#plot(Pred, Style="Density", Rows=1:9)
#plot(Pred, Style="Fitted")
#plot(Pred, Style="Jarque-Bera")
#plot(Pred, Style="Predictive Quantiles")
#plot(Pred, Style="Residual Density")
#plot(Pred, Style="Residuals")
#Levene.Test(Pred)
#Importance(Fit, Model, MyData, Discrep="Chi-Square")

#Fit$Covar is scaled (2.38^2/d) and submitted to LaplacesDemon as Covar.
#Fit$Summary[,1] is submitted to LaplacesDemon as Initial.Values.
#End
}
\keyword{Adaptive}
\keyword{Bayesian Inference}
\keyword{BFGS}
\keyword{BHHH}
\keyword{Conjugate Gradient}
\keyword{DFP}
\keyword{Gradient Ascent}
\keyword{High Performance Computing}
\keyword{Hit-And-Run}
\keyword{Hooke-Jeeves}
\keyword{Initial Values}
\keyword{Limited-Memory BFGS}
\keyword{Levenberg-Marquardt}
\keyword{Nelder-Mead}
\keyword{Newton-Raphson}
\keyword{Optimization}
\keyword{Particle Swarm Optimization}
\keyword{Resilient Backpropagation}
\keyword{Self-Organizing Migration Algorithm}
\keyword{Spectral Projected Gradient}
\keyword{Stochastic Gradient Descent}
\keyword{Symmetric Rank-One}
\keyword{Trust Region}

\name{data.demonsessions}
\alias{demonsessions}
\title{Demon Sessions Data Set}
\usage{data(demonsessions)}
\description{
  These are the monthly number of user sessions at
  \url{https://web.archive.org/web/20141224051720/http://www.bayesian-inference.com/index} by continent. Additional data
  may be added in the future.
}
\format{
  This data frame contains 26 rows (with row names) and 6 columns. The
  following data dictionary describes each variable or column.
  \describe{
    \item{\code{Africa}}{This is the African continent.}
    \item{\code{Americas}}{This is North and South America.}
    \item{\code{Asia}}{This is the Asian continent.}
    \item{\code{Europe}}{This is Europe as a continent.}
    \item{\code{Oceania}}{This is Oceania, such as Australia.}
    \item{\code{Not.Set}}{This includes sessions in which the continent
      was not set, or is unknown.}
    }
}
\source{\url{https://web.archive.org/web/20141224051720/http://www.bayesian-inference.com/index}}
\keyword{datasets}

\name{LaplacesDemon}
\alias{LaplacesDemon}
\alias{LaplacesDemon.hpc}
\title{Laplace's Demon}
\description{
  The \code{LaplacesDemon} function is the main function of Laplace's
  Demon. Given data, a model specification, and initial values,
  \code{LaplacesDemon} maximizes the logarithm of the unnormalized joint
  posterior density with MCMC and provides samples of the marginal
  posterior distributions, deviance, and other monitored variables.

  The \code{LaplacesDemon.hpc} function extends \code{LaplacesDemon} to
  parallel chains for multicore or cluster high performance computing.
}
\usage{
LaplacesDemon(Model, Data, Initial.Values, Covar=NULL, Iterations=10000,
     Status=100, Thinning=10, Algorithm="MWG", Specs=list(B=NULL),
     Debug=list(DB.chol=FALSE, DB.eigen=FALSE, DB.MCSE=FALSE,
     DB.Model=TRUE), LogFile="", ...)
LaplacesDemon.hpc(Model, Data, Initial.Values, Covar=NULL,
     Iterations=10000, Status=100, Thinning=10, Algorithm="MWG",
     Specs=list(B=NULL), Debug=list(DB.chol=FALSE, DB.eigen=FALSE,
     DB.MCSE=FALSE, DB.Model=TRUE), LogFile="", Chains=2, CPUs=2,
     Type="PSOCK", Packages=NULL, Dyn.libs=NULL)
}
\arguments{
  \item{Model}{This required argument receives the model from a
    user-defined function that must be named Model. The user-defined
    function is where the model is specified. \code{LaplacesDemon}
    passes two arguments to the model function, \code{parms} and
    \code{Data}, and receives five arguments from the model function:
    \code{LP} (the logarithm of the unnormalized joint posterior),
    \code{Dev} (the deviance), \code{Monitor} (the monitored variables),
    \code{yhat} (the variables for posterior predictive checks), and
    \code{parm}, the vector of parameters, which may be constrained in
    the model function. More information on the Model specification
    function may be found in the "LaplacesDemon Tutorial" vignette, and
    the \code{\link{is.model}} function. Many examples of model
    specification functions may be found in the "Examples" vignette.}
  \item{Data}{This required argument accepts a list of data. The list of
    data must contain \code{mon.names} which contains monitored variable
    names, and must contain \code{parm.names} which contains parameter
    names. The \code{\link{as.parm.names}} function may be helpful for
    preparing the data, and the \code{\link{is.data}} function may be
    helpful for checking data.}
  \item{Initial.Values}{For \code{LaplacesDemon}, this argument requires
    a vector of initial values equal in length to the number of
    parameters. For \code{LaplacesDemon.hpc}, this argument also accepts
    a vector, in which case the same initial values will be applied to
    all parallel chains, or the argument accepts a matrix in which each
    row is a parallel chain and the number of columns is equal in length
    to the number of parameters. When a matrix is supplied for
    \code{LaplacesDemon.hpc}, each parallel chain begins with its own
    initial values that are preferably dispersed. For both
    \code{LaplacesDemon} and \code{LaplacesDemon.hpc}, each initial
    value will be the starting point for an adaptive chain or a
    non-adaptive Markov chain of a parameter. Parameters are assumed to
    be continuous, unless specified to be discrete (see \code{dparm}
    below), which is not accepted by all algorithms (see
    \code{\link{dcrmrf}} for an alternative). If all initial values are
    set to zero, then Laplace's Demon will attempt to optimize the
    initial values with the \code{\link{LaplaceApproximation}}
    function. After Laplace's Demon finishes updating, it may be desired
    to continue updating from where it left off. To continue, this
    argument should receive the last iteration of the previous update.
    For example, if the output object is called Fit, then
    \code{Initial.Values=as.initial.values(Fit)}. Initial values may be
    generated randomly with the \code{\link{GIV}} function.}
  \item{Covar}{This argument defaults to \code{NULL}, but may otherwise
    accept a \eqn{K \times K}{K x K} proposal covariance matrix
    (where \eqn{K} is the number of dimensions or parameters), a
    variance vector, or a list of covariance matrices (for blockwise
    sampling in some algorithms). When the model is updated for the
    first time and prior variance or covariance is unknown, then
    \code{Covar=NULL} should be used. Some algorithms require
    covariance, some only require variance, and some require neither.
    Laplace's Demon automatically converts the user input to the
    required form. Once Laplace's Demon has finished updating, it may
    be desired to continue updating where it left off, in which case
    the proposal covariance matrix from the last run can be input into
    the next run. The covariance matrix may also be input from the
    \code{\link{LaplaceApproximation}} function, if used.}
  \item{Iterations}{This required argument accepts integers larger than
    10, and determines the number of iterations that Laplace's Demon
    will update the parameters while searching for target
    distributions. The required amount of computer memory will increase
    with \code{Iterations}. If computer memory is exceeded, then all
    will be lost. The \code{\link{Combine}} function can be used later
    to combine multiple updates.}
  \item{Status}{This argument accepts an integer between 1 and the
    number of iterations, and indicates how often, in iterations, the
    user would like the status printed to the screen or log
    file. Usually, the following is reported: the number of iterations,
    the proposal type (for example, multivariate or componentwise, or
    mixture, or subset), and LP. For example, if a model is updated for
    1,000 iterations and \code{Status=200}, then a status message will
    be printed at the following iterations: 200, 400, 600, 800, and
    1,000.}
  \item{Thinning}{This argument accepts integers between 1 and the
    number of iterations, and indicates that every nth iteration will be
    retained, while the other iterations are discarded. If
    \code{Thinning=5}, then every 5th iteration will be
    retained. Thinning is performed to reduce autocorrelation and the
    number of marginal posterior samples.}
  \item{Algorithm}{This argument accepts the abbreviated name of the
    MCMC algorithm, which must appear in quotes. A list of MCMC
    algorithms appears below in the Details section, and the
    abbreviated name is in parenthesis.}
  \item{Specs}{This argument defaults to \code{NULL}, and accepts a list
    of specifications for the MCMC algorithm declared in the
    \code{Algorithm} argument. The specifications associated with each
    algorithm may be seen below in the examples, must appear in the
    order shown, and are described in the details section below.}
  \item{Debug}{This argument accepts a list of logical scalars that
    control whether or not errors or warnings are reported due to a
    \code{try} function or non-finite values. List components include
    \code{DB.chol} regarding \code{chol}, \code{DB.eigen} regarding
    \code{eigen}, \code{DB.MCSE} regarding \code{\link{MCSE}}, and
    \code{DB.Model} regarding the Model specification function. Errors
    and warnings should be investigated, but do not necessarily indicate
    a faulty Model specification function or a bug in the software. For
    example, a sampler may make a proposal that would result in a matrix
    that is not positive definite, when it should be. This kind of error
    or warning is acceptable, provided the sampler handles it correctly
    by rejecting the proposal, and provided the Model specification
    function is not causing the issue. Oftentimes, blockwise sampling
    with carefully chosen blocks will mostly or completely eliminate
    errors or warnings that occur otherwise in larger, multivariate
    proposals. Similarly, debugged componentwise algorithms tend to
    provide more information than multivariate algorithms, since
    usually the parameter and both its current and proposed values may
    be reported. If confident in the Model specification function, and
    errors or warnings are produced frequently that are acceptable,
    then consider setting \code{DB.Model=FALSE} for cleaner output and
    faster sampling. If the Model specification function is not faulty
    and there is a bug in \code{LaplacesDemon}, then please report it
    with a bug description and reproducible code on 
    \url{https://github.com/LaplacesDemonR/LaplacesDemon/issues}.}
  \item{LogFile}{This argument is used to specify a log file name in
    quotes in the working directory as a destination, rather than the
    console, for the output messages of \code{cat} and \code{stop}
    commands. It is helpful to assign a log file name when
    using multiple cores, such as with \code{LaplacesDemon.hpc}. Doing
    so allows the user to check the progress in the log. A number of log
    files are created, one for each chain, and one for the overall
    process.}
  \item{Chains}{This argument is required only for
    \code{LaplacesDemon.hpc}, and indicates the number of parallel
    chains.}
  \item{CPUs}{This argument is required for parallel independent or
    interactive chains in \code{LaplacesDemon} or
    \code{LaplacesDemon.hpc}, and indicates the number of central
    processing units (CPUs) of the computer or cluster. For example,
    when a user has a quad-core computer, \code{CPUs=4}.}
  \item{Type}{This argument defaults to \code{"PSOCK"} and uses the
    Simple Network of Workstations (SNOW) for parallelization.
    Alternatively, \code{Type="MPI"} may be specified to use Message
    Passing Interface (MPI) for parallelization.}
  \item{Packages}{This optional argument is for use with parallel
    independent or interacting chains, and defaults to \code{NULL}. This
    argument accepts a vector of package names to load into each
    parallel chain. If the \code{Model} specification depends on any
    packages, then these package names need to be in this vector.}
  \item{Dyn.libs}{This optional argument is for use with parallel
    independent or interacting chain, and defaults to \code{NULL}. This
    argument accepts a vector of the names of dynamic link libraries
    (shared objects) to load into each parallel chain. The libraries
    must be located in the working directory.}
  \item{...}{Additional arguments are unused.}
}
\details{
  \code{LaplacesDemon} offers numerous MCMC algorithms for numerical
  approximation in Bayesian inference. The algorithms are

  \itemize{
    \item Adaptive Directional Metropolis-within-Gibbs (ADMG)
    \item Adaptive Griddy-Gibbs (AGG)
    \item Adaptive Hamiltonian Monte Carlo (AHMC)
    \item Adaptive Metropolis (AM)
    \item Adaptive Metropolis-within-Gibbs (AMWG)
    \item Adaptive-Mixture Metropolis (AMM)
    \item Affine-Invariant Ensemble Sampler (AIES)
    \item Componentwise Hit-And-Run Metropolis (CHARM)
    \item Delayed Rejection Adaptive Metropolis (DRAM)
    \item Delayed Rejection Metropolis (DRM)
    \item Differential Evolution Markov Chain (DEMC)
    \item Elliptical Slice Sampler (ESS)
    \item Gibbs Sampler (Gibbs)
    \item Griddy-Gibbs (GG)
    \item Hamiltonian Monte Carlo (HMC)
    \item Hamiltonian Monte Carlo with Dual-Averaging (HMCDA)
    \item Hit-And-Run Metropolis (HARM)
    \item Independence Metropolis (IM)
    \item Interchain Adaptation (INCA)
    \item Metropolis-Adjusted Langevin Algorithm (MALA)
    \item Metropolis-Coupled Markov Chain Monte Carlo (MCMCMC)
    \item Metropolis-within-Gibbs (MWG)
    \item Multiple-Try Metropolis (MTM)
    \item No-U-Turn Sampler (NUTS)
    \item Oblique Hyperrectangle Slice Sampler (OHSS)
    \item Preconditioned Crank-Nicolson (pCN)
    \item Random Dive Metropolis-Hastings (RDMH)
    \item Random-Walk Metropolis (RWM)
    \item Reflective Slice Sampler (RSS)
    \item Refractive Sampler (Refractive)
    \item Reversible-Jump (RJ)
    \item Robust Adaptive Metropolis (RAM)
    \item Sequential Adaptive Metropolis-within-Gibbs (SAMWG)
    \item Sequential Metropolis-within-Gibbs (SMWG)
    \item Slice Sampler (Slice)
    \item Stochastic Gradient Langevin Dynamics (SGLD)
    \item Tempered Hamiltonian Monte Carlo (THMC)
    \item t-walk (twalk)
    \item Univariate Eigenvector Slice Sampler (UESS)
    \item Updating Sequential Adaptive Metropolis-within-Gibbs (USAMWG)
    \item Updating Sequential Metropolis-within-Gibbs (USMWG)
  }

  It is a goal for the documentation in the \pkg{LaplacesDemon} to be
  extensive. However, details of MCMC algorithms are best explored
  online at \url{https://web.archive.org/web/20150206014000/http://www.bayesian-inference.com/mcmc}, as well
  as in the "LaplacesDemon Tutorial" vignette, and the "Bayesian
  Inference" vignette. Algorithm specifications (\code{Specs}) are
  listed below:

  \itemize{
    \item \code{A} is used in AFSS, HMCDA, MALA, NUTS, OHSS, and UESS. In
    MALA, it is the maximum acceptable value of the Euclidean norm of
    the adaptive parameters mu and sigma, and the Frobenius norm of the
    covariance matrix. In AFSS, HMCDA, NUTS, OHSS, and UESS, it is the
    number of initial, adaptive iterations to be discarded as burn-in.
    \item \code{Adaptive} is the iteration in which adaptation begins,
    and is used in AM, AMM, DRAM, INCA, and Refractive. Most of these
    algorithms adapt according to an observed covariance matrix, and
    should sample before beginning to adapt.
    \item \code{alpha.star} is the target acceptance rate in MALA and
    RAM, and is optional in CHARM and HARM. The recommended value for
    multivariate proposals is \code{alpha.star=0.234}, for componentwise
    proposals is \code{alpha.star=0.44}, and for MALA is
    \code{alpha.star=0.574}.
    \item \code{at} affects the traverse move in twalk. \code{at=6} is
    recommended. It helps when some parameters are highly correlated,
    and the correlation structure may change through the
    state-space. The traverse move is associated with an acceptance rate
    that decreases as the number of parameters increases, and is the
    reason that \code{n1} is used to select a subset of parameters each
    iteration. If adjusted, it is recommended to stay in the interval
    [2,10].
    \item \code{aw} affects the walk move in twalk, and \code{aw=1.5} is
    recommended. If adjusted, it is recommended to stay in the
    interval [0.3,2].
    \item \code{beta} is a scale parameter for AIES, and defaults to 2,
    or an autoregressive parameter for pCN.
    \item \code{bin.n} is the scalar size parameter for a binomial prior
    distribution of model size for the RJ algorithm.
    \item \code{bin.p} is the scalar probability parameter for a
    binomial prior distribution of model size for the RJ algorithm.
    \item \code{B} is a list of blocked parameters. Each component of
    the list represents a block of parameters, and contains a vector in
    which each element is the position of the associated parameter in
    parm.names. This function is optional in the AFSS, AMM, AMWG, ESS,
    HARM, MWG, RAM, RWM, Slice, and UESS algorithms. For more
    information on blockwise sampling, see the \code{\link{Blocks}}
    function.
    \item \code{Begin} indicates the time-period in which to begin
    updating (filtering or predicting) in the USAMWG and USMWG
    algorithms.
    \item \code{Bounds} is used in the Slice algorithm. It is a vector
    of length two with the lower and upper boundary of the slice. For
    continuous parameters, it is often set to negative and positive
    infinity, while for discrete parameters it is set to the minimum
    and maximum discrete values to be sampled. When blocks are used,
    this must be supplied as a list with the same number of list
    components as the number of blocks.
    \item \code{delta} is used in HMCDA, MALA, and NUTS. In HMCDA and
    NUTS, it is the target acceptance rate, and the recommended value is
    0.65 in HMCDA and 0.6 in NUTS. In MALA, it is a constant in the
    bounded drift function, may be in the interval [1e-10,1000], and 1
    is the default.
    \item \code{Dist} is the proposal distribution in RAM, and may
    either be \code{Dist="t"} for t-distributed or \code{Dist="N"} for
    normally-distributed.
    \item \code{dparm} accepts a vector of integers that indicate
    discrete parameters. This argument is for use with the AGG or GG
    algorithm.
    \item \code{Dyn} is a \eqn{T \times K}{T x K} matrix of dynamic
    parameters, where \eqn{T} is the number of time-periods and \eqn{K}
    is the number of dynamic parameters. \code{Dyn} is used by SAMWG,
    SMWG, USAMWG, and USMWG. Non-dynamic parameters are updated first in
    each sampler iteration, then dynamic parameters are updated in a
    random order in each time-period, and sequentially by time-period.
    \item \code{epsilon} is used in AHMC, HMC, HMCDA, MALA, NUTS, SGLD,
    and THMC. It is the step-size in all algorithms except MALA. It is
    a vector equal in length to the number of parameters in AHMC, HMC,
    and THMC. It is a scalar in HMCDA and NUTS. It is either a scalar
    or a vector equal in length to the number of iterations in SGLD.
    When \code{epsilon=NULL} in HMCDA or NUTS (only), a reasonable
    initial value is found. In MALA, it is a vector of length two. The
    first element is the acceptable minimum of adaptive scale sigma, and
    the second element is added to the diagonal of the covariance matrix
    for regularization.
    \item \code{FC} is used in Gibbs and accepts a function that
    receives two arguments: the vector of all parameters and the list of
    data (similar to the Model specification function). FC must return
    the updated vector of all parameters. The user specifies FC to
    calculate the full conditional distribution of one or more
    parameters.
    \item \code{file} is the quoted name of a numeric matrix of data,
    without headers, for SGLD. The big data set must be a .csv
    file. This matrix has \code{Nr} rows and \code{Nc} columns. Each
    iteration, SGLD will randomly select a block of rows, where the
    number of rows is specified by the \code{size} argument.
    \item \code{Fit} is an object of class \code{demonoid} in the USAMWG
    and USMWG algorithms. Posterior samples before the time-period
    specified in the \code{Begin} argument are not updated, and are used
    instead from \code{Fit}.
    \item \code{gamma} controls the step size in DEMC or the decay of
    adaptation in MALA and RAM. In DEMC, it is positive and defaults to
    \eqn{2.38 / \sqrt{2J}}{2.38/sqrt(2J)} when \code{NULL}, where
    \eqn{J} is the length of initial values. For RAM, it is in the
    interval (0.5,1], and 0.66 is recommended. For MALA, it is in the
    interval (1,\code{Iterations}), and defaults to 1.
    \item \code{Grid} accepts either a vector or a list of vectors of
    evenly-spaced points on a grid for the AGG or GG algorithm. When the
    argument is a vector, the same grid is applied to all
    parameters. When the argument is a list, each component in the list
    has a grid that is applied to the corresponding parameter. The
    algorithm will evaluate each continuous parameter at the latest
    value plus each point in the grid, or each discrete parameter (see
    \code{dparm}) at each grid point (which should be each discrete
    value).
    \item \code{K} is a scalar number of proposals in MTM.
    \item \code{L} is a scalar number of leapfrog steps in AHMC, HMC, and
    THMC. When \code{L=1}, the algorithm reduces to Langevin Monte Carlo
    (LMC).
    \item \code{lambda} is used in HMCDA and MCMCMC. In HMCDA, it is a
    scalar trajectory length. In MCMCMC, it is either a scalar that
    controls temperature spacing, or a vector of temperature spacings.
    \item \code{Lmax} is a scalar maximum for \code{L} (see above) in
    HMCDA and NUTS.
    \item \code{m} is used in the AFSS, AHMC, HMC, Refractive, RSS, Slice,
    THMC, and UESS algorithms. In AHMC, HMC, and THMC, it is a
    \eqn{J \times J}{J x J} mass matrix for \eqn{J} initial values. In
    AFSS and UESS, it is a scalar, and is the maximum number of steps
    for creating the slice interval. In Refractive and RSS, it is a
    scalar, and is the number of steps to take per iteration. In Slice,
    it is either a scalar or a list with as many list components as
    blocks. It must be an integer in [1,Inf], and indicates the maximum
    number of steps for creating the slice interval.
    \item \code{mu} is a vector that is equal in length to the initial
    values. This vector will be used as the mean of the proposal
    distribution, and is usually the posterior mode of a
    previously-updated \code{\link{LaplaceApproximation}}.
    \item \code{MWG} is used in Gibbs to specify a vector of parameters
    that are to receive Metropolis-within-Gibbs updates. Each element is
    an integer that indicates the parameter.
    \item \code{Nc} is either the number of (un-parallelized) parallel
    chains in DEMC (and must be at least 3) or the number of columns of
    big data in SGLD.
    \item \code{Nr} is the number of rows of big data in SGLD.
    \item \code{n} is the number of previous iterations in ADMG, AFSS,
    AMM, AMWG, OHSS, RAM, and UESS.
    \item \code{n1} affects the size of the subset of each set of points
    to adjust, and is used in twalk. It relates to the number of
    parameters, and \code{n1=4} is recommended. If adjusted, it is
    recommended to stay in the interval [2,20].
    \item \code{parm.p} is a vector of probabilities for parameter
    selection in the RJ algorithm, and must be equal in length to
    the number of initial values.
    \item \code{r} is a scalar used in the Refractive algorithm to
    indicate the ratio between r1 and r2.
    \item \code{Periodicity} specifies how often in iterations the
    adaptive algorithm should adapt, and is used by AHMC, AM, AMM, AMWG,
    DRAM, INCA, SAMWG, and USAMWG. If \code{Periodicity=10}, then the
    algorithm adapts every 10th iteration. A higher \code{Periodicity}
    is associated with an algorithm that runs faster, because it does
    not have to calculate adaptation as often, though the algorithm
    adapts less often to the target distributions, so it is a
    trade-off. It is recommended to use the lowest value that runs
    fast enough to suit the user, or provide sufficient adaptation.
    \item \code{selectable} is a vector of indicators of whether or not
    a parameter is selectable for variable selection in the RJ
    algorithm. Non-selectable parameters are assigned a zero, and are
    always in the model. Selectable parameters are assigned a one. This
    vector must be equal in length to the number of initial values.
    \item \code{selected} is a vector of indicators of whether or not
    each parameter is selected when the RJ algorithm begins, and
    must be equal in length to the number of initial values.
    \item \code{SIV} stands for secondary initial values and is used by
    twalk. \code{SIV} must be the same length as \code{Initial.Values},
    and each element of these two vectors must be unique from each
    other, both before and after being passed to the \code{Model}
    function. \code{SIV} defaults to \code{NULL}, in which case values
    are generated with \code{\link{GIV}}.
    \item \code{size} is the number of rows of big data to be read into
    SGLD each iteration.
    \item \code{smax} is the maximum allowable tuning parameter sigma,
    the standard deviation of the conditional distribution, in the AGG
    algorithm.
    \item \code{Temperature} is used in the THMC algorithm to heat up
    the momentum in the first half of the leapfrog steps, and then cool
    down the momentum in the last half. \code{Temperature} must be
    positive. When greater than 1, THMC should explore more diffuse
    distributions, and may be helpful with multimodal distributions.
    \item \code{Type} is used in the Slice algorithm. It is either a
    scalar or a list with the same number of list components as blocks.
    This accepts \code{"Continuous"} for continuous parameters,
    \code{"Nominal"} for discrete parameters that are unordered, and
    \code{"Ordinal"} for discrete parameters that are ordered.
    \item \code{w} is used in AFSS, AMM, DEMC, Refractive, RSS, and
    Slice. It is a mixture weight for both the AMM and DEMC algorithms,
    and in these algorithms it is in the interval (0,1]. For AMM, it is
    recommended to use \code{w=0.05}, as per Roberts and Rosenthal
    (2009). The two mixture components in AMM are adaptive multivariate
    and static/symmetric univariate proposals. The mixture is determined
    at each iteration with mixture weight \code{w}. In the AMM
    algorithm, a higher value of \code{w} is associated with more
    static/symmetric univariate proposals, and a lower \code{w} is
    associated with more adaptive multivariate proposals. AMM will be
    unable to include the multivariate mixture component until it has
    accumulated some history, and models with more parameters will take
    longer to be able to use adaptive multivariate proposals. In DEMC,
    it indicates the probability that each iteration uses a snooker
    update, rather than a projection update, and the recommended default
    is \code{w=0.1}. In the Refractive algorithm, \code{w} is a scalar
    step size parameter. In AFSS, RSS, and the Slice algorithms, this is
    a step size interval for creating the slice interval. In AFSS and
    RSS, a scalar or vector equal in length the number of initial values
    is accepted. In Slice, a scalar or a list with a number of list
    components equal to the number of blocks is accepted.
    \item \code{Z} accepts a \eqn{T \times J}{T x J} matrix or \eqn{T
    \times J \times Nc}{T x J x Nc} array of thinned samples for \eqn{T}
    thinned iterations, \eqn{J} parameters, and \eqn{Nc} chains for
    DEMC. \code{Z} defaults to \code{NULL}. The matrix of thinned
    posterior samples from a previous run may be used, in which case the
    samples are copied across the chains.
  }
}
\value{
  \code{LaplacesDemon} returns an object of class \code{demonoid}, and
  \code{LaplacesDemon.hpc} returns an object of class
  \code{demonoid.hpc} that is a list of objects of class
  \code{demonoid}, where the number of components in the list
  is the number of parallel chains. Each object of class \code{demonoid} 
  is a list with the following components:
  \item{Acceptance.Rate}{This is the acceptance rate of the MCMC
    algorithm, indicating the percentage of iterations in which the
    proposals were accepted. For more information on acceptance rates,
    see the \code{\link{AcceptanceRate}} function.}
  \item{Algorithm}{This reports the specific algorithm used.}
  \item{Call}{This is the matched call of \code{LaplacesDemon}.}
  \item{Covar}{This stores the \eqn{K \times K}{K x K} proposal
    covariance matrix (where \eqn{K} is the dimension or number of
    parameters), variance vector, or list of covariance matrices.
    If variance or covariance is used for adaptation, then this
    covariance is returned. Otherwise, the variance of the samples of
    each parameter is returned. If the model is updated in the future,
    then this vector, matrix, or list can be used to start the next
    update where the last update left off. Only the diagonal of this
    matrix is reported in the associated \code{print} function.}
  \item{CovarDHis}{This \eqn{N \times K}{N x K} matrix stores the
    diagonal of the proposal covariance matrix of each adaptation in
    each of \eqn{N} rows for \eqn{K} dimensions, where the dimension is
    the number of parameters or length of the initial values vector. The
    proposal covariance matrix should change less over time. An
    exception is that the AHMC algorithm stores an algorithm
    specification here, which is not the diagonal of the proposal
    covariance matrix.}
  \item{Deviance}{This is a vector of the deviance of the model, with a
    length equal to the number of thinned samples that were retained.
    Deviance is useful for considering model fit, and is equal to the
    sum of the log-likelihood for all rows in the data set, which is
    then multiplied by negative two.}
  \item{DIC1}{This is a vector of three values: Dbar, pD, and DIC. Dbar
    is the mean deviance, pD is a measure of model complexity indicating
    the effective number of parameters, and DIC is the Deviance
    Information Criterion, which is a model fit statistic that is the
    sum of Dbar and pD. \code{DIC1} is calculated over all retained
    samples. Note that pD is calculated as \code{var(Deviance)/2} as in
    Gelman et al. (2004).}
  \item{DIC2}{This is identical to \code{DIC1} above, except that it is
    calculated over only the samples that were considered by the
    \code{BMK.Diagnostic} to be stationary for all parameters. If
    stationarity (or a lack of trend) is not estimated for all
    parameters, then \code{DIC2} is set to missing values.}
  \item{Initial.Values}{This is the vector of \code{Initial.Values},
    which may have been optimized with the
    \code{\link{IterativeQuadrature}} or
    \code{\link{LaplaceApproximation}} function.}
  \item{Iterations}{This reports the number of \code{Iterations} for
    updating.}
  \item{LML}{This is an approximation of the logarithm of the marginal
    likelihood of the data (see the \code{\link{LML}} function for more
    information). \code{LML} is estimated only with stationary samples,
    and only with a non-adaptive algorithm, including Adaptive
    Griddy-Gibbs (AGG), Affine-Invariant Ensemble Sampler (AIES),
    Componentwise Hit-And-Run (CHARM), Delayed Rejection Metropolis
    (DRM), Elliptical Slice Sampling (ESS), Gibbs Sampler (Gibbs),
    Griddy-Gibbs (GG), Hamiltonian Monte Carlo (HMC), Hit-And-Run
    Metropolis (HARM), Independence Metropolis (IM), Metropolis-Coupled
    Markov Chain Monte Carlo (MCMCMC), Metropolis-within-Gibbs (MWG),
    Multiple-Try Metropolis, No-U-Turn Sampler (NUTS), Random Dive
    Metropolis-Hastings (RDMH), Random-Walk Metropolis (RWM), Reflective
    Slice Sampler (RSS), Refractive Sampler (Refractive),
    Reversible-Jump (RJ), Sequential Metropolis-within-Gibbs (SMWG),
    Slice Sampler (Slice), Stochastic Gradient Langevin Dynamics (SGLD),
    Tempered Hamiltonian Monte Carlo (THMC), or t-walk (twalk).
    \code{LML} is estimated with nonparametric self-normalized
    importance sampling (NSIS), given LL and the marginal posterior
    samples of the parameters. \code{LML} is useful for comparing
    multiple models with the \code{\link{BayesFactor}} function.}
  \item{Minutes}{This indicates the number of minutes that
    \code{LaplacesDemon} was running, and includes the initial checks as
    well as time it took the \code{\link{LaplaceApproximation}}
    function, assessing stationarity, effective sample size (ESS), and
    creating summaries.}
  \item{Model}{This contains the model specification \code{Model}.}
  \item{Monitor}{This is a vector or matrix of one or more monitored
    variables, which are variables that were specified in the
    \code{Model} function to be observed as chains (or Markov chains,
    if \code{Adaptive=0}), but that were not deviance or parameters.}
  \item{Parameters}{This reports the number of parameters.}
  \item{Posterior1}{This is a matrix of marginal posterior distributions
    composed of thinned samples, with a number of rows equal to the
    number of thinned samples and a number of columns equal to the
    number of parameters. This matrix includes all thinned samples.}
  \item{Posterior2}{This is a matrix equal to \code{Posterior1}, except
    that rows are included only if stationarity (a lack of trend) is
    indicated by the \code{\link{BMK.Diagnostic}} for all parameters.
    If stationarity did not occur, then this matrix is missing.}
  \item{Rec.BurnIn.Thinned}{This is the recommended burn-in for the
    thinned samples, where the value indicates the first row that was
    stationary across all parameters, and previous rows are discarded
    as burn-in. Samples considered as burn-in are discarded because they
    do not represent the target distribution and have not adequately
    forgotten the initial value of the chain (or Markov chain, if
    \code{Adaptive=0}).}
  \item{Rec.BurnIn.UnThinned}{This is the recommended burn-in for all
    samples, in case thinning will not be necessary.}
  \item{Rec.Thinning}{This is the recommended value for the
    \code{Thinning} argument according to the autocorrelation in the
    thinned samples, and it is limited to the interval [1,1000].}
  \item{Specs}{This is an optional list of algorithm specifications.}
  \item{Status}{This is the value in the \code{Status} argument.}
  \item{Summary1}{This is a matrix that summarizes the marginal
    posterior distributions of the parameters, deviance, and monitored
    variables over all samples in \code{Posterior1}. The following
    summary statistics are included: mean, standard deviation, MCSE
    (Monte Carlo Standard Error), ESS is the effective sample size due
    to autocorrelation, and finally the 2.5\%, 50\%, and 97.5\%
    quantiles are reported. MCSE is essentially a standard deviation
    around the marginal posterior mean that is due to uncertainty
    associated with using MCMC. The acceptable size of the MCSE
    depends on the acceptable uncertainty associated around the
    marginal posterior mean. Laplace's Demon prefers to continue
    updating until each MCSE is less than 6.27\% of each marginal
    posterior standard deviation (see the \code{\link{MCSE}} and
    \code{\link{Consort}} functions). The default \code{IMPS} method
    is used. Next, the desired precision of ESS depends on the user's
    goal, and Laplace's Demon prefers to continue until each ESS is at
    least 100, which should be enough to describe 95\% boundaries of an
    approximately Gaussian distribution (see the \code{\link{ESS}} for
    more information).}
  \item{Summary2}{This matrix is identical to the matrix in
    \code{Summary1}, except that it is calculated only on the
    stationary samples found in \code{Posterior2}. If universal
    stationarity was not estimated for the parameters, then this matrix
    is set to missing values.}
  \item{Thinned.Samples}{This is the number of thinned samples that
    were retained.}
  \item{Thinning}{This is the value of the \code{Thinning} argument.}
}
\references{
  Atchade, Y.F. (2006). "An Adaptive Version for the Metropolis Adjusted
  Langevin Algorithm with a Truncated Drift". \emph{Methodology and
  Computing in Applied Probability}, 8, p. 235--254.

  Bai, Y. (2009). "An Adaptive Directional Metropolis-within-Gibbs
  Algorithm". Technical Report in Department of Statistics at the
  University of Toronto.

  Beskos, A., Roberts, G.O., Stuart, A.M., and Voss, J. (2008). "MCMC
  Methods for Diffusion Bridges". Stoch. Dyn., 8, p. 319--350.

  Boyles, L.B. and Welling, M. (2012). "Refractive Sampling".

  Craiu, R.V., Rosenthal, J., and Yang, C. (2009). "Learn From Thy
  Neighbor: Parallel-Chain and Regional Adaptive MCMC". \emph{Journal
  of the American Statistical Assocation}, 104(488), p. 1454--1466.
  
  Christen, J.A. and Fox, C. (2010). "A General Purpose Sampling
  Algorithm for Continuous Distributions (the t-walk)". \emph{Bayesian
  Analysis}, 5(2), p. 263--282.

  Dutta, S. (2012). "Multiplicative Random Walk Metropolis-Hastings
  on the Real Line". \emph{Sankhya B}, 74(2), p. 315--342.

  Duane, S., Kennedy, A.D., Pendleton, B.J., and Roweth, D. (1987).
  "Hybrid Monte Carlo". \emph{Physics Letters}, B, 195, p. 216--222.
  
  Gelman, A., Carlin, J., Stern, H., and Rubin, D. (2004). "Bayesian
  Data Analysis, Texts in Statistical Science, 2nd ed.". Chapman and
  Hall, London.

  Geman, S. and Geman, D. (1984). "Stochastic Relaxation, Gibbs
  Distributions, and the Bayesian Restoration of Images". \emph{IEEE
  Transactions on Pattern Analysis and Machine Intelligence}, 6(6),
  p. 721--741.

  Geyer, C.J. (1991). "Markov Chain Monte Carlo Maximum Likelihood". In
  Keramidas, E.M. Computing Science and Statistics: Proceedings of the
  23rd Symposium of the Interface. Fairfax Station VA: Interface
  Foundation. p. 156--163.

  Goodman J, and Weare, J. (2010). "Ensemble Samplers with Affine
  Invariance". \emph{Communications in Applied Mathematics and
  Computational Science}, 5(1), p. 65--80.

  Green, P.J. (1995). "Reversible Jump Markov Chain Monte Carlo
  Computation and Bayesian Model Determination". \emph{Biometrika},
  82, p. 711--732.

  Haario, H., Laine, M., Mira, A., and Saksman, E. (2006). "DRAM:
  Efficient Adaptive MCMC". \emph{Statistical Computing}, 16,
  p. 339--354.
     
  Haario, H., Saksman, E., and Tamminen, J. (2001). "An Adaptive
  Metropolis Algorithm". \emph{Bernoulli}, 7, p. 223--242.

  Hoffman, M.D. and Gelman. A. (2012). "The No-U-Turn Sampler:
  Adaptively Setting Path Lengths in Hamiltonian Monte
  Carlo". \emph{Journal of Machine Learning Research}, p. 1--30.

  Kass, R.E. and Raftery, A.E. (1995). "Bayes Factors". \emph{Journal
  of the American Statistical Association}, 90(430), p. 773--795.

  Lewis, S.M. and Raftery, A.E. (1997). "Estimating Bayes Factors via
  Posterior Simulation with the Laplace-Metropolis Estimator".
  \emph{Journal of the American Statistical Association}, 92,
  p. 648--655.

  Liu, J., Liang, F., and Wong, W. (2000). "The Multiple-Try Method and
  Local Optimization in Metropolis Sampling". \emph{Journal of the
  American Statistical Association}, 95, p. 121--134.

  Metropolis, N., Rosenbluth, A.W., Rosenbluth, M.N., and Teller,
  E. (1953). "Equation of State Calculations by Fast Computing
  Machines". \emph{Journal of Chemical Physics}, 21, p. 1087--1092.

  Mira, A. (2001). "On Metropolis-Hastings Algorithms with Delayed
  Rejection". \emph{Metron}, Vol. LIX, n. 3-4, p. 231--241.

  Murray, I., Adams, R.P., and MacKay, D.J. (2010). "Elliptical Slice
  Sampling". \emph{Journal of Machine Learning Research}, 9,
  p. 541--548.
  
  Neal, R.M. (2003). "Slice Sampling" (with discussion). \emph{Annals of
  Statistics}, 31(3), p. 705--767.

  Ritter, C. and Tanner, M. (1992), "Facilitating the Gibbs Sampler: the
  Gibbs Stopper and the Griddy-Gibbs Sampler", \emph{Journal of the
  American Statistical Association}, 87, p. 861--868.

  Roberts, G.O. and Rosenthal, J.S. (2009). "Examples of Adaptive
  MCMC". \emph{Computational Statistics and Data Analysis}, 18,
  p. 349--367.

  Roberts, G.O. and Tweedie, R.L. (1996). "Exponential Convergence of
  Langevin Distributions and Their Discrete Approximations".
  \emph{Bernoulli}, 2(4), p. 341--363.
  
  Rosenthal, J.S. (2007). "AMCMC: An R interface for adaptive MCMC".
  \emph{Computational Statistics and Data Analysis}, 51, p. 5467--5470.

  Smith, R.L. (1984). "Efficient Monte Carlo Procedures for Generating
  Points Uniformly Distributed Over Bounded Region". \emph{Operations
  Research}, 32, p. 1296--1308.

  Ter Braak, C.J.F. and Vrugt, J.A. (2008). "Differential Evolution
  Markov Chain with Snooker Updater and Fewer Chains", \emph{Statistics
  and Computing}, 18(4), p. 435--446.

  Tibbits, M., Groendyke, C., Haran, M., Liechty, J. (2014). "Automated
  Factor Slice Sampling". \emph{Journal of Computational and Graphical
  Statistics}, 23(2), p. 543--563.

  Thompson, M.D. (2011). "Slice Sampling with Multivariate Steps".
  \url{https://utoronto.scholaris.ca/items/161206c5-af25-41a8-99d8-8d622f950cb3}

  Vihola, M. (2011). "Robust Adaptive Metropolis Algorithm with Coerced
  Acceptance Rate". \emph{Statistics and Computing}. Springer,
  Netherlands.

  Welling, M. and Teh, Y.W. (2011). "Bayesian Learning via Stochastic
  Gradient Langevin Dynamics". \emph{Proceedings of the 28th
  International Conference on Machine Learning (ICML)}, p. 681--688.

}
\author{Statisticat, LLC.,
     Silvere Vialet-Chabrand \email{silvere@vialet-chabrand.com}}
\seealso{
  \code{\link{AcceptanceRate}},
  \code{\link{as.initial.values}},
  \code{\link{as.parm.names}},
  \code{\link{BayesFactor}},
  \code{\link{Blocks}},
  \code{\link{BMK.Diagnostic}},
  \code{\link{Combine}},
  \code{\link{Consort}},
  \code{\link{dcrmrf}},
  \code{\link{ESS}},
  \code{\link{GIV}},
  \code{\link{is.data}},
  \code{\link{is.model}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon.RAM}},
  \code{\link{LML}}, and
  \code{\link{MCSE}}.
}
\examples{
# The accompanying Examples vignette is a compendium of examples.
####################  Load the LaplacesDemon Library  #####################
library(LaplacesDemon)

##############################  Demon Data  ###############################
data(demonsnacks)
y <- log(demonsnacks$Calories)
X <- cbind(1, as.matrix(log(demonsnacks[,c(1,4,10)]+1)))
J <- ncol(X)
for (j in 2:J) X[,j] <- CenterScale(X[,j])

#########################  Data List Preparation  #########################
mon.names <- "LP"
parm.names <- as.parm.names(list(beta=rep(0,J), sigma=0))
pos.beta <- grep("beta", parm.names)
pos.sigma <- grep("sigma", parm.names)
PGF <- function(Data) {
     beta <- rnorm(Data$J)
     sigma <- runif(1)
     return(c(beta, sigma))
     }
MyData <- list(J=J, PGF=PGF, X=X, mon.names=mon.names,
     parm.names=parm.names, pos.beta=pos.beta, pos.sigma=pos.sigma, y=y)

##########################  Model Specification  ##########################
Model <- function(parm, Data)
     {
     ### Parameters
     beta <- parm[Data$pos.beta]
     sigma <- interval(parm[Data$pos.sigma], 1e-100, Inf)
     parm[Data$pos.sigma] <- sigma
     ### Log-Prior
     beta.prior <- sum(dnormv(beta, 0, 1000, log=TRUE))
     sigma.prior <- dhalfcauchy(sigma, 25, log=TRUE)
     ### Log-Likelihood
     mu <- tcrossprod(Data$X, t(beta))
     LL <- sum(dnorm(Data$y, mu, sigma, log=TRUE))
     ### Log-Posterior
     LP <- LL + beta.prior + sigma.prior
     Modelout <- list(LP=LP, Dev=-2*LL, Monitor=LP,
          yhat=rnorm(length(mu), mu, sigma), parm=parm)
     return(Modelout)
     }
#library(compiler)
#Model <- cmpfun(Model) #Consider byte-compiling for more speed

set.seed(666)

############################  Initial Values  #############################
Initial.Values <- GIV(Model, MyData, PGF=TRUE)

###########################################################################
# Examples of MCMC Algorithms                                             #
###########################################################################

####################  Automated Factor Slice Sampler  #####################
Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
     Algorithm="AFSS", Specs=list(A=Inf, B=NULL, m=100, n=0, w=1))
Fit
print(Fit)
#Consort(Fit)
#plot(BMK.Diagnostic(Fit))
#PosteriorChecks(Fit)
#caterpillar.plot(Fit, Parms="beta")
#BurnIn <- Fit$Rec.BurnIn.Thinned
#plot(Fit, BurnIn, MyData, PDF=FALSE)
#Pred <- predict(Fit, Model, MyData, CPUs=1)
#summary(Pred, Discrep="Chi-Square")
#plot(Pred, Style="Covariates", Data=MyData)
#plot(Pred, Style="Density", Rows=1:9)
#plot(Pred, Style="ECDF")
#plot(Pred, Style="Fitted")
#plot(Pred, Style="Jarque-Bera")
#plot(Pred, Style="Predictive Quantiles")
#plot(Pred, Style="Residual Density")
#plot(Pred, Style="Residuals")
#Levene.Test(Pred)
#Importance(Fit, Model, MyData, Discrep="Chi-Square")

#############  Adaptive Directional Metropolis-within-Gibbs  ##############
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="ADMG", Specs=list(n=0, Periodicity=50))

########################  Adaptive Griddy-Gibbs  ##########################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="AGG", Specs=list(Grid=GaussHermiteQuadRule(3)$nodes,
#     dparm=NULL, smax=Inf, CPUs=1, Packages=NULL, Dyn.libs=NULL))

##################  Adaptive Hamiltonian Monte Carlo  #####################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="AHMC", Specs=list(epsilon=0.02, L=2, m=NULL,
#     Periodicity=10))

##########################  Adaptive Metropolis  ##########################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="AM", Specs=list(Adaptive=500, Periodicity=10))

###################  Adaptive Metropolis-within-Gibbs  ####################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="AMWG", Specs=list(B=NULL, n=0, Periodicity=50))

######################  Adaptive-Mixture Metropolis  ######################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="AMM", Specs=list(Adaptive=500, B=NULL, n=0,
#     Periodicity=10, w=0.05))

###################  Affine-Invariant Ensemble Sampler  ###################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="AIES", Specs=list(Nc=2*length(Initial.Values), Z=NULL,
#     beta=2, CPUs=1, Packages=NULL, Dyn.libs=NULL))

#################  Componentwise Hit-And-Run Metropolis  ##################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="CHARM", Specs=NULL)

###########  Componentwise Hit-And-Run (Adaptive) Metropolis  #############
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="CHARM", Specs=list(alpha.star=0.44))

#################  Delayed Rejection Adaptive Metropolis  #################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="DRAM", Specs=list(Adaptive=500, Periodicity=10))

#####################  Delayed Rejection Metropolis  ######################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="DRM", Specs=NULL)

##################  Differential Evolution Markov Chain  ##################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="DEMC", Specs=list(Nc=3, Z=NULL, gamma=NULL, w=0.1))

#######################  Elliptical Slice Sampler  ########################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="ESS", Specs=list(B=NULL))

#############################  Gibbs Sampler  #############################
### NOTE: Unlike the other samplers, Gibbs requires specifying a
### function (FC) that draws from full conditionals.
#FC <- function(parm, Data)
#     {
#     ### Parameters
#     beta <- parm[Data$pos.beta]
#     sigma <- interval(parm[Data$pos.sigma], 1e-100, Inf)
#     sigma2 <- sigma*sigma
#     ### Hyperparameters
#     betamu <- rep(0,length(beta))
#     betaprec <- diag(length(beta))/1000
#     ### Update beta
#     XX <- crossprod(Data$X)
#     Xy <- crossprod(Data$X, Data$y)
#     IR <- backsolve(chol(XX/sigma2 + betaprec), diag(length(beta)))
#     btilde <- crossprod(t(IR)) %*% (Xy/sigma2 + betaprec %*% betamu)
#     beta <- btilde + IR %*% rnorm(length(beta))
#     return(c(beta,sigma))
#     }
##library(compiler)
##FC <- cmpfun(FC) #Consider byte-compiling for more speed
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="Gibbs", Specs=list(FC=FC, MWG=pos.sigma))


#############################  Griddy-Gibbs  ##############################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="GG", Specs=list(Grid=seq(from=-0.1, to=0.1, len=5),
#     dparm=NULL, CPUs=1, Packages=NULL, Dyn.libs=NULL))

#######################  Hamiltonian Monte Carlo  #########################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="HMC", Specs=list(epsilon=0.001, L=2, m=NULL))

#############  Hamiltonian Monte Carlo with Dual-Averaging  ###############
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=1, Thinning=1,
#     Algorithm="HMCDA", Specs=list(A=500, delta=0.65, epsilon=NULL,
#     Lmax=1000, lambda=0.1))

#######################  Hit-And-Run Metropolis  ##########################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="HARM", Specs=NULL)

##################  Hit-And-Run (Adaptive) Metropolis  ####################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="HARM", Specs=list(alpha.star=0.234, B=NULL))

########################  Independence Metropolis  ########################
### Note: the mu and Covar arguments are populated from a previous Laplace
### Approximation.
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=Fit$Covar, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="IM",
#     Specs=list(mu=Fit$Summary1[1:length(Initial.Values),1]))

#########################  Interchain Adaptation  #########################
#Initial.Values <- rbind(Initial.Values, GIV(Model, MyData, PGF=TRUE))
#Fit <- LaplacesDemon.hpc(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="INCA", Specs=list(Adaptive=500, Periodicity=10),
#     LogFile="MyLog", Chains=2, CPUs=2, Type="PSOCK", Packages=NULL,
#     Dyn.libs=NULL)

################  Metropolis-Adjusted Langevin Algorithm  #################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="MALA", Specs=list(A=1e7, alpha.star=0.574, gamma=1,
#          delta=1, epsilon=c(1e-6,1e-7)))

#############  Metropolis-Coupled Markov Chain Monte Carlo  ###############
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="MCMCMC", Specs=list(lambda=1, CPUs=2, Packages=NULL,
#     Dyn.libs=NULL))

#######################  Metropolis-within-Gibbs  #########################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="MWG", Specs=list(B=NULL))

########################  Multiple-Try Metropolis  ########################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="MTM", Specs=list(K=4, CPUs=1, Packages=NULL, Dyn.libs=NULL))

##########################  No-U-Turn Sampler  ############################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=1, Thinning=1,
#     Algorithm="NUTS", Specs=list(A=500, delta=0.6, epsilon=NULL,
#     Lmax=Inf))

#################  Oblique Hyperrectangle Slice Sampler  ##################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="OHSS", Specs=list(A=Inf, n=0))

#####################  Preconditioned Crank-Nicolson  #####################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="pCN", Specs=list(beta=0.1))

######################  Robust Adaptive Metropolis  #######################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="RAM", Specs=list(alpha.star=0.234, B=NULL, Dist="N",
#     gamma=0.66, n=0))

###################  Random Dive Metropolis-Hastings  ####################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="RDMH", Specs=NULL)

##########################  Refractive Sampler  ###########################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="Refractive", Specs=list(Adaptive=1, m=2, w=0.1, r=1.3))

###########################  Reversible-Jump  #############################
#bin.n <- J-1
#bin.p <- 0.2
#parm.p <- c(1, rep(1/(J-1),(J-1)), 1)
#selectable <- c(0, rep(1,J-1), 0)
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="RJ", Specs=list(bin.n=bin.n, bin.p=bin.p,
#          parm.p=parm.p, selectable=selectable,
#          selected=c(0,rep(1,J-1),0)))

########################  Random-Walk Metropolis  #########################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="RWM", Specs=NULL)

########################  Reflective Slice Sampler  #######################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="RSS", Specs=list(m=5, w=1e-5))

##############  Sequential Adaptive Metropolis-within-Gibbs  ##############
#NOTE: The SAMWG algorithm is only for state-space models (SSMs)
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="SAMWG", Specs=list(Dyn=Dyn, Periodicity=50))

##################  Sequential Metropolis-within-Gibbs  ###################
#NOTE: The SMWG algorithm is only for state-space models (SSMs)
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="SMWG", Specs=list(Dyn=Dyn))

#############################  Slice Sampler  #############################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=1, Thinning=1,
#     Algorithm="Slice", Specs=list(B=NULL, Bounds=c(-Inf,Inf), m=100,
#     Type="Continuous", w=1))

#################  Stochastic Gradient Langevin Dynamics  #################
#NOTE: The Data and Model functions must be coded differently for SGLD.
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=10, Thinning=10,
#     Algorithm="SGLD", Specs=list(epsilon=1e-4, file="X.csv", Nr=1e4,
#     Nc=6, size=10))

###################  Tempered Hamiltonian Monte Carlo  ####################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="THMC", Specs=list(epsilon=0.001, L=2, m=NULL,
#     Temperature=2))

###############################  t-walk  #################################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="twalk", Specs=list(SIV=NULL, n1=4, at=6, aw=1.5))

#################  Univariate Eigenvector Slice Sampler  #################
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values,
#     Covar=NULL, Iterations=1000, Status=100, Thinning=1,
#     Algorithm="UESS", Specs=list(A=Inf, B=NULL, m=100, n=0))

##########  Updating Sequential Adaptive Metropolis-within-Gibbs  #########
#NOTE: The USAMWG algorithm is only for state-space model updating
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values, 
#     Covar=NULL, Iterations=100000, Status=100, Thinning=100,
#     Algorithm="USAMWG", Specs=list(Dyn=Dyn, Periodicity=50, Fit=Fit,
#     Begin=T.m))

##############  Updating Sequential Metropolis-within-Gibbs  ##############
#NOTE: The USMWG algorithm is only for state-space model updating
#Fit <- LaplacesDemon(Model, Data=MyData, Initial.Values, 
#     Covar=NULL, Iterations=100000, Status=100, Thinning=100,
#     Algorithm="USMWG", Specs=list(Dyn=Dyn, Fit=Fit, Begin=T.m))

#End
}
\keyword{Adaptive}
\keyword{Adaptive Directional Metropolis-within-Gibbs}
\keyword{Adaptive Griddy-Gibbs}
\keyword{Adaptive Hamiltonian Monte Carlo}
\keyword{Adaptive Metropolis}
\keyword{Adaptive-Mixture Metropolis}
\keyword{Adaptive Metropolis-within-Gibbs}
\keyword{Affine-Invariant Ensemble Sampler}
\keyword{Bayesian Inference}
\keyword{Delayed Rejection Adaptive Metropolis}
\keyword{Delayed Rejection Metropolis}
\keyword{Differential Evolution Markov Chain}
\keyword{Elliptical Slice Sampler}
\keyword{Gibbs Sampler}
\keyword{Griddy-Gibbs}
\keyword{Hamiltonian Monte Carlo}
\keyword{High Performance Computing}
\keyword{Hit-And-Run}
\keyword{Independence Metropolis}
\keyword{Interchain Adaptation}
\keyword{MCMC}
\keyword{Metropolis-Adjusted Langevin Algorithm}
\keyword{Metropolis-Coupled Markov Chain Monte Carlo}
\keyword{Metropolis-within-Gibbs}
\keyword{Multiple-Try Metropolis}
\keyword{Multiple Chains}
\keyword{No-U-Turn Sampler}
\keyword{Oblique Hyperrectangle Slice Sampler}
\keyword{Optimization}
\keyword{Parallel Chains}
\keyword{Preconditioned Crank-Nicolson}
\keyword{Reflective Slice Sampler}
\keyword{Refractive Sampler}
\keyword{Reversible-Jump}
\keyword{Robust Adaptive Metropolis}
\keyword{Random-Walk Metropolis}
\keyword{Sequential Adaptive Metropolis-within-Gibbs}
\keyword{Sequential Metropolis-within-Gibbs}
\keyword{Slice Sampler}
\keyword{Stochastic Gradient Langevin Dynamics}
\keyword{Tempered Hamiltonian Monte Carlo}
\keyword{t-walk}
\keyword{Univariate Eigenvector Slice Sampler}
\keyword{Updating Sequential Adaptive Metropolis-within-Gibbs}
\keyword{Updating Sequential Metropolis-within-Gibbs}

\name{RejectionSampling}
\alias{RejectionSampling}
\title{Rejection Sampling}
\description{
  The \code{RejectionSampling} function implements rejection sampling
  of a target density given a proposal density.
}
\usage{
RejectionSampling(Model, Data, mu, S, df=Inf, logc, n=1000, CPUs=1, Type="PSOCK")
}
\arguments{
  \item{Model}{This is a model specification function. For more
    information, see \code{\link{LaplaceApproximation}}.}
  \item{Data}{This is a list of data. For more information, see
    \code{\link{LaplaceApproximation}}.}
  \item{mu}{This is a mean vector \eqn{\mu}{mu} for the multivariate
    normal or multivariate t proposal density.}
  \item{S}{This is a convariance matrix \eqn{\Sigma}{Sigma} for the
    multivariate normal or multivariate t proposal density.}
  \item{df}{This is a scalar degrees of freedom parameter
    \eqn{\nu}{nu}. It defaults to infinity, in which case the
    multivariate normal density is used.}
  \item{logc}{This is the logarithm of the rejection sampling constant.}
  \item{n}{This is the number of independent draws to be simulated from
    the proposal density.}
  \item{CPUs}{This argument accepts an integer that specifies the number
    of central processing units (CPUs) of the multicore computer or
    computer cluster. This argument defaults to \code{CPUs=1}, in which
    parallel processing does not occur.}
  \item{Type}{This argument specifies the type of parallel processing to
    perform, accepting either \code{Type="PSOCK"} or
    \code{Type="MPI"}.}
}
\details{
  Rejection sampling (von Neumann, 1951) is a Monte Carlo method for
  drawing independent samples from a distribution that is proportional
  to the target distribution, \eqn{f(x)}, given a sampling distribution,
  \eqn{g(x)}, from which samples can readily be drawn, and for which
  there is a finite constant \eqn{c}.

  Here, the target distribution, \eqn{f(x)}, is the result of the
  \code{Model} function. The sampling distribution, \eqn{g(x)}, is
  either a multivariate normal or multivariate t-distribution. The
  parameters of \eqn{g(x)} (\code{mu}, \code{S}, and \code{df}) are used
  to create random draws, \eqn{\theta}{theta}, of the sampling
  distribution, \eqn{g(x)}. These draws, \eqn{\theta}{theta}, are used
  to evaluate the target distribution, \eqn{f(x)}, via the \code{Model}
  specification function. The evaluations of the target distribution,
  sampling distribution, and the constant are used to create a
  probability of acceptance for each draw, by comparing to a vector of
  \eqn{n} uniform draws, \eqn{u}. Each draw, \eqn{\theta}{theta} is
  accepted if \deqn{u \le \frac{f(\theta|\textbf{y})}{cg(\theta)}}{u <=
    f(theta|y)/[cg(theta)]}

  Before beginning rejection sampling, a goal of the user is to find the
  bounding constant, \eqn{c}, such that \eqn{f(\theta|\textbf{y}) \le
  cg(\theta)}{f(theta|y) <= cg(theta)} for all
  \eqn{\theta}{theta}. These are all expressed in logarithms, so the
  goal is to find \eqn{\log f(\theta|\textbf{y}) - \log g(\theta) \le
  \log(c)}{log[f(theta|y)] - log[g(theta)] <= log(c)} for all
  \eqn{\theta}{theta}. This is done by maximizing \eqn{\log
  f(\theta|\textbf{y}) - \log g(\theta)}{log[f(theta|y)] -
  log[g(theta)]} over all \eqn{\theta}{theta}. By using, say,
  \code{\link{LaplaceApproximation}} to find the modes of the
  parameters of interest, and using the resultant \code{LP}, the mode
  of the logarithm of the joint posterior distribution, as
  \eqn{\log(c)}{log(c)}.

  The \code{RejectionSampling} function performs one iteration of
  rejection sampling. Rejection sampling is often iterated, then called
  the rejection sampling algorithm, until a sufficient number or
  proportion of \eqn{\theta}{theta} is accepted. An efficient
  rejection sampling algorithm has a high acceptance rate. However,
  rejection sampling becomes less efficient as the model dimension (the
  number of parameters) increases.
  
  Extensions of rejection sampling include Adaptive Rejection
  Sampling (ARS) (either derivative-based or derivative-free) and
  Adaptive Rejection Metropolis Sampling (ARMS), as in Gilks et
  al. (1995). The random-walk Metropolis algorithm (Metropolis et al.,
  1953) combined the rejection sampling (a method of Monte Carlo
  simulation) of von Neumann (1951) with Markov chains.

  Parallel processing may be performed when the user specifies
  \code{CPUs} to be greater than one, implying that the specified number
  of CPUs exists and is available. Parallelization may be performed on a
  multicore computer or a computer cluster. Either a Simple Network of
  Workstations (SNOW) or Message Passing Interface (MPI) is used. With
  small data sets and few samples, parallel processing may be slower,
  due to computer network communication. With larger data sets and more
  samples, the user should experience a faster run-time.

  This function is similar to the \code{rejectsampling} function in the
  \code{LearnBayes} package.
}
\value{
  The \code{RejectionSampling} function returns an object of class
  \code{rejection}, which is a matrix of accepted, independent,
  simulated draws from the target distribution.
}
\references{
  Gilks, W.R., Best, N.G., Tan, K.K.C. (1995). "Adaptive Rejection
  Metropolis Sampling within Gibbs Sampling". Journal of the Royal
  Statistical Society. Series C (Applied Statistics), Vol. 44, No. 4,
  p. 455--472.

  Metropolis, N., Rosenbluth, A.W., Rosenbluth, M.N., and Teller,
  E. (1953). "Equation of State Calculations by Fast Computing
  Machines". Journal of Chemical Physics, 21, p. 1087-1092.

  von Neumann, J. (1951). "Various Techniques Used in Connection with
  Random Digits. Monte Carlo Methods". National Bureau Standards, 12,
  p. 36--38.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dmvn}},
  \code{\link{dmvt}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
library(LaplacesDemon)
### Suppose an output object of class laplace is called Fit:
#rs <- RejectionSampling(Model, MyData, mu=Fit$Summary1[,1],
#     S=Fit$Covar, df=Inf, logc=Fit$LP.Final, n=1000)
#rs
}
\keyword{High Performance Computing}
\keyword{Monte Carlo}

\name{dist.YangBerger}
\alias{dyangberger}
\alias{dyangbergerc}
\title{Yang-Berger Distribution}
\description{
  This is the density function for the Yang-Berger prior distribution
  for a covariance matrix or precision matrix.
}
\usage{
dyangberger(x, log=FALSE)
dyangbergerc(x, log=FALSE)
}
\arguments{
  \item{x}{This is the \eqn{k \times k}{k x k} positive-definite
    covariance matrix or precision matrix for \code{dyangberger} or the
    Cholesky factor \eqn{\textbf{U}}{U} of the covariance matrix or
    precision matrix for \code{dyangbergerc}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Multivariate
    \item Density: \eqn{p(\theta) = \frac{1}{|\theta|^{\prod (d_j -
      d_{j-1})}}}{p(theta) = 1 / |theta|^(prod (d[j] - d[j-1]))},
      where \eqn{d} are increasing eigenvalues. See equation 13 in
      Yang and Berger (1994).
    \item Inventor: Yang and Berger (1994)
    \item Notation 1: \eqn{\theta \sim \mathcal{YB}}{p(theta) ~ YB}
    \item Mean: 
    \item Variance: 
    \item Mode: 
}

Yang and Berger (1994) derived a least informative prior (LIP) for a
covariance matrix or precision matrix. The Yang-Berger (YB) distribution
does not have any parameters. It is a reference prior for objective
Bayesian inference. The Cholesky parameterization is also provided here.

The YB prior distribution results in a proper posterior. It involves an
eigendecomposition of the covariance matrix or precision matrix. It is
difficult to interpret a model that uses the YB prior, due to a lack of
intuition regarding the relationship between eigenvalues and
correlations.

Compared to Jeffreys prior for a covariance matrix, this reference prior
encourages equal eigenvalues, and therefore results in a covariance
matrix or precision matrix with a better shrinkage of its
eigenstructure.
}
\value{
  \code{dyangberger} and \code{dyangbergerc} give the density.
}
\references{
  Yang, R. and Berger, J.O. (1994). "Estimation of a Covariance Matrix
  using the Reference Prior". \emph{Annals of Statistics}, 2,
  p. 1195-1211.
}
\seealso{
  \code{\link{dinvwishart}} and
  \code{\link{dwishart}}
}
\examples{
library(LaplacesDemon)
X <- matrix(c(1,0.8,0.8,1), 2, 2)
dyangberger(X, log=TRUE)
}
\keyword{Distribution}

\name{dist.ContinuousRelaxation}
\alias{dcrmrf}
\alias{rcrmrf}
\title{Continuous Relaxation of a Markov Random Field Distribution}
\description{
  This is the density function and random generation from the continuous
  relaxation of a Markov random field (MRF) distribution.
}
\usage{
dcrmrf(x, alpha, Omega, log=FALSE)
rcrmrf(n, alpha, Omega)
}
\arguments{
  \item{x}{This is a vector of length \eqn{k}.}
  \item{n}{This is the number of random deviates to generate.}
  \item{alpha}{This is a vector of length \eqn{k} of shape parameters.}
  \item{Omega}{This is the \eqn{k \times k}{k x k} precision matrix
       \eqn{\Omega}{Omega}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Multivariate
    \item Density: \deqn{p(\theta) \propto \exp(-\frac{1}{2} \theta^T
      \Omega^{-1} \theta) \prod_i (1 + \exp(\theta_i +
      alpha_i))}{p(theta) = exp(-0.5 theta^T Omega^(-1) theta) prod i=1
      (1 + exp(theta[i] + alpha[i]))}
    \item Inventor: Zhang et al. (2012)
    \item Notation 1: \eqn{\theta \sim \mathcal{CRMRF}(\alpha,
      \Omega)}{theta ~ CRMRF(alpha, Omega)}
    \item Notation 2: \eqn{p(\theta) = \mathcal{CRMRF}(\theta | \alpha,
      \Omega)}{p(theta) = CRMRF(theta | alpha, Omega)}
    \item Parameter 1: shape vector \eqn{\alpha}{alpha}
    \item Parameter 2: positive-definite \eqn{k \times k}{k x k} matrix
      \eqn{\Omega}{Omega}
    \item Mean: \eqn{E(\theta)}{E(theta)}
    \item Variance: \eqn{var(\theta)}{var(theta)}
    \item Mode: \eqn{mode(\theta)}{mode(theta)}
}

It is often easier to solve or optimize a problem with continuous
variables rather than a problem that involves discrete variables. A
continuous variable may also have a gradient, contour, and curvature
that may be useful for optimization or sampling. Continuous MCMC
samplers are far more common.

Zhang et al. (2012) introduced a generalized form of the Gaussian
integral trick from statistical physics to transform a discrete variable
so that it may be estimated with continuous variables. An auxiliary
Gaussian variable is added to a discrete Markov random field (MRF) so
that discrete dependencies cancel out, allowing the discrete variable to
be summed away, and leaving a continuous problem. The resulting
continuous representation of the problem allows the model to be updated
with a continuous MCMC sampler, and may benefit from a MCMC sampler that
uses derivatives. Another advantage of continuous MCMC is that
stationarity of discrete Markov chains is problematic to assess.

A disadvantage of solving a discrete problem with continuous parameters
is that the continuous solution requires more parameters.
}
\value{
  \code{dcrmrf} gives the density and
  \code{rcrmrf} generates random deviates.
}
\references{
  Zhang, Y., Ghahramani, Z., Storkey, A.J., and Sutton, C.A. (2012).
  "Continuous Relaxations for Discrete Hamiltonian Monte Carlo".
  \emph{Advances in Neural Information Processing Systems}, 25,
  p. 3203--3211.
}
\seealso{
  \code{\link{dmvn}}
}
\examples{
library(LaplacesDemon)
x <- dcrmrf(rnorm(5), rnorm(5), diag(5))
x <- rcrmrf(10, rnorm(5), diag(5))
}
\keyword{Distribution}

\name{dist.Student.t}
\alias{dst}
\alias{pst}
\alias{qst}
\alias{rst}
\title{Student t Distribution: Univariate}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the univariate Student t
  distribution with location parameter \eqn{\mu}{mu}, scale parameter
  \eqn{\sigma}{sigma}, and degrees of freedom parameter \eqn{\nu}{nu}.
}
\usage{
dst(x, mu=0, sigma=1, nu=10, log=FALSE)
pst(q, mu=0, sigma=1, nu=10, lower.tail=TRUE, log.p=FALSE)
qst(p, mu=0, sigma=1, nu=10, lower.tail=TRUE, log.p=FALSE)
rst(n, mu=0, sigma=1, nu=10)
}
\arguments{
  \item{x, q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{mu}{This is the location parameter \eqn{\mu}{mu}.}
  \item{sigma}{This is the scale parameter \eqn{\sigma}{sigma}, which
    must be positive.}
  \item{nu}{This is the degrees of freedom parameter \eqn{\nu}{nu},
    which must be positive.}
  \item{lower.tail}{Logical. If \code{lower.tail=TRUE}, then
    probabilities are \eqn{Pr[X \le x]}{Pr[X <= x]}, otherwise,
    \eqn{Pr[X > x]}{Pr[X > x]}.}
  \item{log, log.p}{Logical. If \code{log=TRUE}, then the logarithm of the
    density or probability is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density: \eqn{p(\theta) =
    \frac{\Gamma[(\nu+1)/2]}{\Gamma(\nu/2)} \sqrt{\nu \pi} \sigma[1 +
    \frac{1}{\nu}[\frac{\theta - \mu}{\sigma}]^2]^{(-\nu + 1)/2}}{p(theta) =
    [{Gamma[(nu + 1) / 2]} / Gamma(nu/2)]*sqrt(nu*pi)*sigma*[1 +
    (1/nu)*[(theta - mu)/sigma]^2]^[(-nu+ 1)/2]}
  \item Inventor: William Sealy Gosset (1908)
  \item Notation 1: \eqn{\theta \sim \mathrm{t}(\mu, \sigma, \nu)}{theta
    ~ t(mu, sigma,nu)}
  \item Notation 2: \eqn{p(\theta) = \mathrm{t}(\theta | \mu, \sigma,
    \nu)}{p(theta) = t(theta | mu, sigma,nu)}
  \item Parameter 1: location parameter \eqn{\mu}{mu}
  \item Parameter 2: scale parameter \eqn{\sigma > 0}{sigma > 0}
  \item Parameter 3: degrees of freedom \eqn{\nu > 0}{nu > 0}
  \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}, for \eqn{\nu > 1}{nu
    > 1}, otherwise undefined
  \item Variance: \eqn{var(\theta) = \frac{\nu}{\nu -
      2}\sigma^2}{var(theta) = [nu / (nu - 2)]sigma^2}, for \eqn{\nu >
    2}{nu> 2}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The Student t-distribution is often used as an alternative to the normal
distribution as a model for data. It is frequently the case that real
data have heavier tails than the normal distribution allows for. The
classical approach was to identify outliers and exclude or downweight
them in some way. However, it is not always easy to identify outliers
(especially in high dimensions), and the Student t-distribution is a
natural choice of model-form for such data. It provides a parametric
approach to robust statistics.

The degrees of freedom parameter, \eqn{\nu}{nu}, controls the kurtosis
of the distribution, and is correlated with the scale parameter
\eqn{\sigma}{sigma}. The likelihood can have multiple local maxima
and, as such, it is often necessary to fix \eqn{\nu}{nu} at a fairly
low value and estimate the other parameters taking this as given.
Some authors report that values between 3 and 9 are often good
choices, and some authors suggest 5 is often a good choice.

In the limit \eqn{\nu \rightarrow \infty}{nu -> infinity}, the
Student t-distribution approaches
\eqn{\mathcal{N}(\mu, \sigma^2)}{N(mu, sigma^2)}. The
case of \eqn{\nu = 1}{nu = 1} is the Cauchy distribution.

The \code{pst} and \code{qst} functions are similar to those in the
\code{gamlss.dist} package.
}
\value{
  \code{dst} gives the density,
  \code{pst} gives the distribution function,
  \code{qst} gives the quantile function, and
  \code{rst} generates random deviates.
}
\seealso{
  \code{\link{dcauchy}},
  \code{\link{dmvt}},
  \code{\link{dmvtp}},
  \code{\link{dnorm}},
  \code{\link{dnormp}},
  \code{\link{dnormv}},
  \code{\link{dstp}}, and
  \code{\link{dt}}.
}
\examples{
library(LaplacesDemon)
x <- dst(1,0,1,10)
x <- pst(1,0,1,10)
x <- qst(0.5,0,1,10)
x <- rst(100,0,1,10)

#Plot Probability Functions
x <- seq(from=-5, to=5, by=0.1)
plot(x, dst(x,0,1,0.1), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dst(x,0,1,1), type="l", col="green")
lines(x, dst(x,0,1,10), type="l", col="blue")
legend(1, 0.9, expression(paste(mu==0, ", ", sigma==1, ", ", nu==0.5),
     paste(mu==0, ", ", sigma==1, ", ", nu==1),
     paste(mu==0, ", ", sigma==1, ", ", nu==10)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{dist.Asymmetric.Multivariate.Laplace}
\alias{daml}
\alias{raml}
\title{Asymmetric Multivariate Laplace Distribution}
\description{
  These functions provide the density and random generation for the
  asymmetric multivariate Laplace distribution with location and skew
  parameter \eqn{\mu}{mu} and covariance \eqn{\Sigma}{Sigma}.
}
\usage{
daml(x, mu, Sigma, log=FALSE)
raml(n, mu, Sigma)
}
\arguments{
  \item{x}{This is a \eqn{N \times K}{N x K} matrix of data, or a
    vector of length \eqn{K}.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{mu}{This is the location and skew parameter \eqn{\mu}{mu}. This
    may be a \eqn{N \times K}{N x K} matrix, or a vector of length
    \eqn{K}.}
  \item{Sigma}{This is the \eqn{K \times K}{K x K} positive-definite
    covariance matrix \eqn{\Sigma}{Sigma}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Multivariate
    \item Density: \eqn{p(\theta) =
      \frac{2\exp(\theta\Omega\theta)}{(2\pi)^{k/2}|\Sigma|^0.5}
      \frac{\theta\Omega\theta}{2 + \mu\Omega\mu}^{(2-k)/4}
      K_{(2-k)/2}(\sqrt{(2 + \mu\Omega\mu)(\theta\Omega\theta)})}{
      p(theta) = ((2exp(theta Omega theta)) /
      ((2pi)^(k/2)|Sigma|^(0.5)))
      ((theta Omega theta) / (2 + mu Omega mu))^((k-2)/4)
      K[(2-k)/2](sqrt((2 + mu Omega mu)(theta Omega theta)))}
    \item Inventor: Kotz, Kozubowski, and Podgorski (2003)
    \item Notation 1: \eqn{\theta \sim \mathcal{AL}_K(\mu,
      \Sigma)}{theta ~ AL[K](mu, Sigma)}
    \item Notation 2: \eqn{p(\theta) = \mathcal{AL}_K(\theta | \mu,
      \Sigma)}{p(theta) = AL[K](theta | mu, Sigma)}
    \item Parameter 1: location-skew parameter \eqn{\mu}{mu}
    \item Parameter 2: positive-definite covariance matrix
      \eqn{\Sigma}{Sigma}
    \item Mean: Unknown
    \item Variance: Unknown
    \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The asymmetric multivariate Laplace distribution of Kotz, Kozubowski,
and Podgorski (2003) is a multivariate extension of the univariate,
asymmetric Laplace distribution. It is parameterized according to
two parameters: location-skew parameter \eqn{\mu}{mu} and positive-definite
covariance matrix \eqn{\Sigma}{Sigma}. Location and skew occur in the
same parameter. When \eqn{\mu=0}{mu=0}, the density is the (symmetric)
multivariate Laplace of Anderson (1992). As each location deviates from
zero, the marginal distribution becomes more skewed. Since location and
skew are combined, it is appropriate for zero-centered variables, such
as a matrix of centered and scaled dependent variables in cluster
analysis, factor analysis, multivariate regression, or multivariate
time-series.

The asymmetric multivariate Laplace distribution is also discussed
earlier in Kozubowski and Podgorski (2001), and is well-suited for
financial modeling via multivariate regression, specifically with
currency exchange rates. Cajigas and Urga (2005) fit residuals in a
multivariate GARCH model with the asymmetric multivariate Laplace
distribution, regarding stocks and bonds. They find that it
"overwhelmingly outperforms" normality.
}
\value{
  \code{daml} gives the density, and
  \code{raml} generates random deviates.
}
\references{
  Anderson, D.N. (1992). "A Multivariate Linnik Distribution".
  \emph{Statistical Probability Letters}, 14, p. 333--336.
  
  Cajigas, J.P. and Urga, G. (2005) "Dynamic Conditional Correlation
  Models with Asymmetric Laplace Innovations". Centre for Economic
  Analysis: Cass Business School.
  
  Kotz, S., Kozubowski, T.J., and Podgorski, K. (2003). "An
  Asymmetric Multivariate Laplace Distribution". Working Paper.
    
  Kozubowski, T.J. and Podgorski, K. (2001). "Asymmetric Laplace
  Laws and Modeling Financial Data". \emph{Mathematical and Computer
  Modelling}, 34, p. 1003--1021.
}
\seealso{\code{\link{dalaplace}} and
  \code{\link{dmvl}}
}
\examples{
library(LaplacesDemon)
x <- daml(c(1,2,3), c(0,1,2), diag(3))
X <- raml(1000, c(0,1,2), diag(3))
joint.density.plot(X[,1], X[,2], color=FALSE)
}
\keyword{Distribution}

\name{dist.Generalized.Poisson}
\alias{dgpois}
\title{Generalized Poisson Distribution}
\description{
  The density function is provided for the univariate, discrete,
  generalized Poisson distribution with location parameter
  \eqn{\lambda}{lambda} and scale parameter \eqn{\omega}{omega}.
}
\usage{
dgpois(x, lambda=0, omega=0, log=FALSE)
}
\arguments{
  \item{x}{This is a vector of quantiles.}
  \item{lambda}{This is the parameter \eqn{\lambda}{lambda}.}
  \item{omega}{This is the parameter \eqn{\omega}{omega}, which
    should be in the interval [0,1) for positive counts.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
  \item Application: Discrete Univariate
  \item Density: \eqn{p(\theta) = (1 - \omega) \lambda \frac{[(1 -
      \omega) \lambda + \omega \theta]^{\theta - 1}}{\theta!} \exp{-[(1
      - \omega) \lambda + \omega \theta]}}{(1 - omega) lambda (((1 -
    omega) lambda + omega theta)^(y-1) / theta!) e(-((1 - omega) lambda
    + omega theta))}
  \item Inventor: Consul (1989) and Ntzoufras et al. (2005)
  \item Notation 1: \eqn{\theta \sim
    \mathrm{GP}(\lambda,\omega)}{theta ~ GP(lambda, omega)}
  \item Notation 2: \eqn{p(\theta) = \mathrm{GP}(\theta | \lambda,
    \omega)}{p(theta) = GP(theta | lambda, omega)}
  \item Parameter 1: location parameter \eqn{\lambda}{lambda}
  \item Parameter 2: scale parameter \eqn{\omega \in [0,1)}{omega in [0,1)}
  \item Mean: \eqn{E(\theta) = \lambda}{E(theta) = lambda}
  \item Variance: \eqn{var(\theta) = \lambda(1 -
    \omega)^{-2}}{var(theta) = lambda(1 - omega)^(-2)}
}

The generalized Poisson distribution (Consul, 1989) is also called the
Lagrangian Poisson distribution. The simple Poisson distribution is a
special case of the generalized Poisson distribution. The generalized
Poisson distribution is used in generalized Poisson regression as an
extension of Poisson regression that accounts for overdispersion.

The \code{dgpois} function is parameterized according to Ntzoufras et
al. (2005), which is easier to interpret and estimates better with MCMC.

Valid values for omega are in the interval [0,1) for positive counts.
For \eqn{\omega = 0}{omega = 0}, the generalized Poisson reduces to a
simple Poisson with mean \eqn{\lambda}{lambda}. Note that it is possible
for \eqn{\omega < 0}{omega < 0}, but this implies underdispersion in
count data, which is uncommon. The \code{dgpois} function returns
warnings or errors, so \eqn{\omega}{omega} should be non-negative here.

The dispersion index (DI) is a variance-to-mean ratio, and is \eqn{DI =
  (1 - \omega)^{-2}}{DI = (1 - omega)^(-2)}. A simple Poisson has DI=1.
When DI is far from one, the assumption that the variance equals the
mean of a simple Poisson is violated.
}
\value{
  \code{dgpois} gives the density.
}
\references{
  Consul, P. (1989). `"Generalized Poisson Distribution: Properties and
  Applications". Marcel Decker: New York, NY.

  Ntzoufras, I., Katsis, A., and Karlis, D. (2005). "Bayesian Assessment
  of the Distribution of Insurance Claim Counts using Reversible Jump
  MCMC", \emph{North American Actuarial Journal}, 9, p. 90--108.
}
\seealso{
  \code{\link{dnbinom}} and
  \code{\link{dpois}}.
}
\examples{
library(LaplacesDemon)
y <- rpois(100, 5)
lambda <- rpois(100, 5)
x <- dgpois(y, lambda, 0.5)

#Plot Probability Functions
x <- seq(from=0, to=20, by=1)
plot(x, dgpois(x,1,0.5), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dlaplace(x,1,0.6), type="l", col="green")
lines(x, dlaplace(x,1,0.7), type="l", col="blue")
legend(2, 0.9, expression(paste(lambda==1, ", ", omega==0.5),
     paste(lambda==1, ", ", omega==0.6), paste(lambda==1, ", ", omega==0.7)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{de.Finetti.Game}
\alias{de.Finetti.Game}
\title{de Finetti's Game}
\description{
  The \code{de.Finetti.Game} function estimates the interval of a
  subjective probability regarding a possible event in the near future.
}
\usage{
de.Finetti.Game(width)
}
\arguments{
  \item{width}{This is the maximum acceptable width of the interval for
    the returned subjective probability. The user must specify a width
    between 0 and 1.}
}
\details{
  This function is a variation on the game introduced by de Finetti,
  who is one of the main developers of subjective probability,
  along with Ramsey and Savage. In the original context, de Finetti
  proposed a gamble regarding life on Mars one billion years ago.

  The frequentist interpretation of probability defines the probability
  of an event as the limit of its relative frequency in a large number
  of trials. Frequentist inference is undefined, for example, when there
  are no trials from which to calculate a probability. By defining
  probability relative to frequencies of physical events, frequentists
  attempt to objectify probability. However, de Finetti asserts that the
  frequentist (or objective) interpretation always reduces to a subjective
  interpretation of probability, because probability is a human
  construct and does not exist independently of humans in
  nature. Therefore, probability is a degree of belief, and is called
  subjective or personal probability.
}
\value{
  The \code{de.Finetti.Game} function returns a vector of length
  two. The respective elements are the lower and upper bounds of the
  subjective probability of the participant regarding the possible event
  in the near future.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{elicit}}
}
\keyword{Personal Probability}
\keyword{Subjective Probability}
\keyword{Utility}

\name{deburn}
\alias{deburn}
\title{De-Burn}
\description{
  The \code{deburn} function discards or removes a user-specified number
  of burn-in iterations from an object of class \code{demonoid}.
}
\usage{
deburn(x, BurnIn=0)
}
\arguments{
  \item{x}{This is an object of class \code{demonoid}.}
  \item{BurnIn}{This argument defaults to \code{BurnIn=0}, and accepts
    an integer that indicates the number of iterations to discard as
    burn-in.}
}
\details{
  Documentation for the \code{\link{burnin}} function provides an
  introduction to the concept of burn-in as it relates to Markov chains.

  The \code{deburn} function discards a number of the first posterior
  samples, as specified by the \code{BurnIn} argument. Stationarity is
  not checked, because it is assumed the user has a reason for using the
  \code{deburn} function, rather than using the results from the object
  of class \code{demonoid}. Therefore, the posterior samples in
  \code{Posterior1} and \code{Posterior2} are identical, as are
  \code{Summary1} and \code{Summary2}.
}
\value{
  The \code{deburn} function returns an object of class \code{demonoid}.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{burnin}} and
  \code{\link{LaplacesDemon}}.
}
\examples{
### Assuming the user has Fit which is an object of class demonoid:
#library(LaplacesDemon)
#Fit2 <- deburn(Fit, BurnIn=100)
}
\keyword{Stationarity}
\keyword{Utility}

\name{as.covar}
\alias{as.covar}
\title{Proposal Covariance}
\description{
  This function returns the most recent covariance matrix or a list of
  blocking covariance matrices from an object of class \code{demonoid},
  the most recent covariance matrix from \code{iterquad},
  \code{laplace}, or \code{vb}, the most recent covariance matrix from
  the chain with the lowest deviance in an object of class
  \code{demonoid.hpc}, and a number of covariance matrices of an object
  of class \code{pmc} equal to the number of mixture components. The
  returned covariance matrix or matrices are intended to be the initial
  proposal covariance matrix or matrices for future updates. A variance
  vector from an object of class \code{demonoid} or \code{demonoid.hpc}
  is converted to a covariance matrix.
}
\usage{
as.covar(x)
}
\arguments{
  \item{x}{This is an object of class \code{demonoid},
    \code{demonoid.hpc}, \code{iterquad}, \code{laplace}, \code{pmc}, or
    \code{vb}.}
}
\details{
  Unless it is known beforehand how many iterations are required for
  iterative quadrature, Laplace Approximation, or Variational Bayes to
  converge, MCMC to appear converged, or the normalized perplexity to
  stabilize in PMC, multiple updates are necessary. An additional
  update, however, should not begin with the same proposal covariance
  matrix or matrices as the original update, because it will have to
  repeat the work already accomplished. For this reason, the
  \code{as.covar} function may be used at the end of an update to change
  the previous initial values to the latest values.

  The \code{as.covar} function is most helpful with objects of class
  \code{pmc} that have multiple mixture components. For more
  information, see \code{\link{PMC}}.
}
\value{
  The returned value is a matrix (or array in the case of PMC with
  multiple mixture components) of the latest observed or proposal
  covariance, which may now be used as an initial proposal covariance
  matrix or matrices for a future update.
}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{LaplacesDemon.hpc}},
  \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\keyword{Utility}

\name{Levene.Test}
\alias{Levene.Test}
\title{Levene's Test}
\description{
  The \code{Levene.Test} function is a Bayesian form of Levene's test
  (Levene, 1960) of equality of variances.
}
\usage{
Levene.Test(x, Method="U", G=NULL, Data=NULL)
}
\arguments{
  \item{x}{This required argument must be an object of class
    \code{demonoid.ppc}, \code{iterquad.ppc}, \code{laplace.ppc},
    \code{pmc.ppc}, or \code{vb.ppc}.}
  \item{Method}{The method defaults to \code{U} for a univariate
    dependent variable (DV), y. When the DV is multivariate,
    \code{Method="C"} applies Levene's test to each column associated
    in Y. When \code{Method="R"}, Levene's test is applied to each
    row associated in Y.}
  \item{G}{This argument defaults to \code{NULL}, or is required to
    have the same dimensions as the DV. For example, if the DV is
    univariate, then G must have a length equal to y, which is usually
    represented with a length of N for the number of records or T for
    the number of time-periods. If the DV is multivariate, then
    \code{G} must be a matrix, like Y, and have the same number of
    rows and columns. The purpose of the \code{G} argument is to allow
    the user to specify each element of y or Y to be in a particular
    group, so the variance of the groups can be tested. As such, each
    element of \code{G} must consist of an integer from one to the
    number of groups desired to be tested. The reason this test allows
    this degree of specificity is so the user can specify groups, such
    as according to covariate levels. By default, 4 groups are
    specified, where the first quarter of the records are group 1 and
    the last quarter of the records are group 4.}
  \item{Data}{This argument is required when the DV is multivariate,
    hence when \code{Method="C"} or \code{Method="R"}. The DV is
    required to be named Y.}
}
\details{
  This function is a Bayesian form of Levene's test. Levene's test is
  used to assess the probability of the equality of residual variances
  in different groups. When residual variance does not differ by group,
  it is often called homoscedastic (or homoskedastic) residual variance.
  Homoskedastic residual variance is a common assumption. An advantage
  of Levene's test to other tests of homoskedastic residual variance is
  that Levene's test does not require normality of the residuals.

  The \code{Levene.Test} function estimates the test statistic,
  \eqn{W}{W}, as per Levene's test. This Bayesian form, however,
  estimates \eqn{W}{W} from the observed residuals as
  \eqn{W^{obs}}{W.obs}, and \eqn{W}{W} from residuals that are
  replicated from a homoskedastic process as \eqn{W^{rep}}{W.rep}.
  Further, \eqn{W^{obs}}{W.obs} and \eqn{W^{rep}}{W.rep} are
  estimated for each posterior sample. Finally, the probability that
  the distribution of \eqn{W^{obs}}{W.obs} is greater than the
  distribution of \eqn{W^{rep}}{W.rep} is reported (see below).
}
\value{
  The \code{Levene.Test} function returns a plot (or for multivariate Y,
  a series of plots), and a vector with a length equal to the number of
  Levene's tests conducted.

  One plot is produced per univariate application of Levene's test. Each
  plot shows the test statistic W, both from the observed process
  (W.obs as a black density) and the replicated process (W.rep as a red
  line). The mean of W.obs is reported, along with its 95\% quantile-based
  probability interval (see \code{\link{p.interval}}), the probability
  \eqn{p(W^{obs} > W^{rep})}{p(W.obs > W.rep)}, and the indicated
  results, either homoskedastic or heteroskedastic.

  Each element of the returned vector is the probability
  \eqn{p(W^{obs} > W^{rep})}{p(W.obs > W.rep)}. When the probability
  is \eqn{p < 0.025}{p < 0.025} or \eqn{p > 0.975}{p > 0.975},
  heteroskedastic variance is indicated. Otherwise, the variances of
  the groups are assumed not to differ effectively.
}
\references{
  Levene, H. (1960). "Robust Tests for Equality of Variances". In
  I. Olkins, S. G. Ghurye, W. Hoeffding, W. G. Madow, & H. B. Mann
  (Eds.), \emph{Contributions to Probability and Statistics},
  p. 278--292. Stanford University Press: Stanford, CA.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{PMC}},
  \code{\link{p.interval}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
#First, update the model with IterativeQuadrature, LaplaceApproximation,
#  LaplacesDemon, PMC, or VariationalBayes.
#Then, use the predict function, creating, say, object Pred.
#Finally:
#Levene.Test(Pred)
}
\keyword{Homoskedasticity}
\keyword{Utility}

\name{dist.Inverse.Wishart.Cholesky}
\alias{dinvwishartc}
\alias{rinvwishartc}
\title{Inverse Wishart Distribution: Cholesky Parameterization}
\description{
  These functions provide the density and random number generation
  for the inverse Wishart distribution with the Cholesky
  parameterization.
}
\usage{
   dinvwishartc(U, nu, S, log=FALSE)
   rinvwishartc(nu, S)
}
\arguments{
  \item{U}{This is the upper-triangular \eqn{k \times k}{k x k} matrix
    for the Cholesky factor \eqn{\textbf{U}}{U} of covariance matrix
    \eqn{\Sigma}{Sigma}.}
  \item{nu}{This is the scalar degrees of freedom, \eqn{\nu}{nu}.}
  \item{S}{This is the symmetric, positive-semidefinite
    \eqn{k \times k}{k x k} scale matrix \eqn{\textbf{S}}{S}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Multivariate
    \item Density: \eqn{p(\theta) = (2^{\nu k/2} \pi^{k(k-1)/4}
      \prod^k_{i=1} \Gamma(\frac{\nu+1-i}{2}))^{-1} |\textbf{S}|^{\nu/2}
      |\Sigma|^{-(\nu+k+1)/2} \exp(-\frac{1}{2} tr(\textbf{S}
      \Sigma^{-1}))}{p(theta) = (2^(nu*k/2) * pi^(k(k-1)/4) *
      [Gamma((nu+1-i)/2) * ... * Gamma((nu+1-k)/2)])^(-1) * |S|^(nu/2) *
      |Sigma|^(-(nu+k+1)/2) * exp(-(1/2) * tr(S Sigma^(-1)))}
    \item Inventor: John Wishart (1928)
    \item Notation 1: \eqn{\Sigma \sim
      \mathcal{W}^{-1}_{\nu}(\textbf{S}^{-1})}{Sigma ~
      W^(-1)[nu](S^(-1))}
    \item Notation 2: \eqn{p(\Sigma) = \mathcal{W}^{-1}_{\nu}(\Sigma |
      \textbf{S}^{-1})}{p(Sigma) = W^-1[nu](Sigma | S^(-1))}
    \item Parameter 1: degrees of freedom \eqn{\nu}{nu}
    \item Parameter 2: symmetric, positive-semidefinite
      \eqn{k \times k}{k x k} scale matrix \eqn{\textbf{S}}{S}
    \item Mean: \eqn{E(\Sigma) = \frac{\textbf{S}}{\nu - k -
	1}}{E(Sigma) = S / (nu - k - 1)}
    \item Variance: 
    \item Mode: \eqn{mode(\Sigma) = \frac{\textbf{S}}{\nu + k +
	1}}{mode(Sigma) = S / (nu + k + 1)}
}

The inverse Wishart distribution is a probability distribution defined on
real-valued, symmetric, positive-definite matrices, and is used as the
conjugate prior for the covariance matrix, \eqn{\Sigma}{Sigma}, of a
multivariate normal distribution. In this parameterization,
\eqn{\Sigma}{Sigma} has been decomposed to the upper-triangular Cholesky
factor \eqn{\textbf{U}}{U}, as per \code{\link{chol}}. The
inverse-Wishart density is always finite, and the integral is always
finite. A degenerate form occurs when \eqn{\nu < k}{nu < k}.

In practice, \eqn{\textbf{U}}{U} is fully unconstrained for proposals
when its diagonal is log-transformed. The diagonal is exponentiated
after a proposal and before other calculations. Overall, the
Cholesky parameterization is faster than the traditional
parameterization. Compared with \code{dinvwishart}, \code{dinvwishartc}
must additionally matrix-multiply the Cholesky back to the covariance
matrix, but it does not have to check for or correct the covariance
matrix to positive-semidefiniteness, which overall is slower. Compared
with \code{rinvwishart}, \code{rinvwishartc} must additionally
calculate a Cholesky decomposition, and is therefore slower.

The inverse Wishart prior lacks flexibility, having only one parameter,
\eqn{\nu}{nu}, to control the variability for all \eqn{k(k + 1)/2}
elements. Popular choices for the scale matrix \eqn{\textbf{S}}{S}
include an identity matrix or sample covariance matrix. When the model
sample size is small, the specification of the scale matrix can be
influential.

The inverse Wishart distribution has a dependency between variance and
correlation, although its relative for a precision matrix (inverse
covariance matrix), the Wishart distribution, does not have this
dependency. This relationship becomes weaker with more degrees of
freedom.

Due to these limitations (lack of flexibility, and dependence between
variance and correlation), alternative distributions have been
developed. Alternative distributions that are available here include the
inverse matrix gamma (\code{\link{dinvmatrixgamma}}), Scaled Inverse
Wishart (\code{\link{dsiw}}) and Huang-Wand (\code{\link{dhuangwand}}).
Huang-Wand is recommended.
}
\value{
  \code{dinvwishartc} gives the density and
  \code{rinvwishartc} generates random deviates.
}
\references{
  Wishart, J. (1928). "The Generalised Product Moment Distribution in
  Samples from a Normal Multivariate Population". \emph{Biometrika},
  20A(1-2), p. 32--52.
}
\seealso{
  \code{\link{chol}},
  \code{\link{Cov2Prec}},
  \code{\link{dhuangwand}},
  \code{\link{dinvmatrixgamma}},
  \code{\link{dmvn}},
  \code{\link{dmvnc}},
  \code{\link{dmvtc}},
  \code{\link{dsiw}},
  \code{\link{dwishart}},
  \code{\link{dwishartc}}, and
  \code{\link{dyangbergerc}}.
}
\examples{
library(LaplacesDemon)
Sigma <- matrix(c(2,-.3,-.3,4),2,2)
U <- chol(Sigma)
x <- dinvwishartc(U, 3, matrix(c(1,.1,.1,1),2,2))
x <- rinvwishartc(3, matrix(c(1,.1,.1,1),2,2))
}
\keyword{Distribution}

\name{LPL.interval}
\alias{LPL.interval}
\title{Lowest Posterior Loss Interval}
\description{
  This function returns the Lowest Posterior Loss (LPL) interval for one
  parameter, given samples from the density of its prior distribution
  and samples of the posterior distribution.
}
\usage{
LPL.interval(Prior, Posterior, prob=0.95, plot=FALSE, PDF=FALSE)
}
\arguments{
  \item{Prior}{This is a vector of samples of the prior density.}
  \item{Posterior}{This is a vector of posterior samples.}
  \item{prob}{This is a numeric scalar in the interval (0,1) giving the
    Lowest Posterior Loss (LPL) interval, and defaults to 0.95,
    representing a 95\% LPL interval.}
  \item{plot}{Logical. When \code{plot=TRUE}, two plots are
    produced. The top plot shows the expected posterior loss. The LPL
    region is shaded black, and the area outside the region is gray. The
    bottom plot shows LPL interval of \eqn{\theta}{theta} on the kernel
    density of \eqn{\theta}{theta}. Again, the LPL region is shaded
    black, and the outside area is gray. A vertical, red, dotted line is
    added at zero for both plots. The \code{plot} argument defaults to
    \code{FALSE}. The plot treats the distribution as if it were
    unimodal; disjoint regions are not estimated here. If multimodality
    should result in disjoint regions, then consider using HPD intervals
    in the \code{\link{p.interval}} function.}
  \item{PDF}{Logical. When \code{PDF=TRUE}, and only when
    \code{plot=TRUE}, plots are saved as a .pdf file in the working
    directory.}
}
\details{
  The Lowest Posterior Loss (LPL) interval (Bernardo, 2005), or LPLI, is
  a probability interval based on intrinsic discrepancy loss between
  prior and posterior distributions. The expected posterior loss
  is the loss associated with using a particular value
  \eqn{\theta_i \in \theta}{theta[i] in theta} of the parameter as the
  unknown true value of \eqn{\theta}{theta} (Bernardo, 2005). Parameter
  values with smaller expected posterior loss should always be
  preferred. The LPL interval includes a region in which all parameter
  values have smaller expected posterior loss than those outside the
  region.
 
  Although any loss function could be used, the loss function should be
  invariant under reparameterization. Any intrinsic loss function is
  invariant under reparameterization, but not necessarily invariant
  under one-to-one transformations of data \eqn{\textbf{x}}{x}. When a
  loss function is also invariant under one-to-one transformations, it
  is usually also invariant when reduced to a sufficient statistic. Only
  an intrinsic loss function that is invariant when reduced to a
  sufficient statistic should be considered.

  The intrinsic discrepancy loss is easily a superior loss function to
  the overused quadratic loss function, and is more appropriate than
  other popular measures, such as Hellinger distance, Kullback-Leibler
  divergence (\code{\link{KLD}}), and Jeffreys logarithmic
  divergence. The intrinsic discrepancy loss is also an
  information-theory related divergence measure. Intrinsic discrepancy
  loss is a symmetric, non-negative loss function, and is a continuous,
  convex function. Intrinsic discrepancy loss was introduced
  by Bernardo and Rueda (2002) in a different context: hypothesis
  testing. Formally, it is:

  \deqn{\delta f(p_2,p_1) = min[\kappa(p_2 | p_1), \kappa(p_1 |
    p_2)]}{delta f(p[2],p[1]) = min[kappa(p[2] | p[1]), kappa(p[1] | p[2])]}

  where \eqn{\delta}{delta} is the discrepancy, \eqn{\kappa}{kappa} is
  the \code{\link{KLD}}, and \eqn{p_1}{p[1]} and \eqn{p_2}{p[2]} are the
  probability distributions. The intrinsic discrepancy loss is the loss
  function, and the expected posterior loss is the mean of the directed
  divergences.
  
  The LPL interval is also called an intrinsic credible interval or
  intrinsic probability interval, and the area inside the interval is
  often called an intrinsic credible region or intrinsic probability
  region.

  In practice, whether a reference prior or weakly informative prior
  (WIP) is used, the LPL interval is usually very close to the HPD
  interval, though the posterior losses may be noticeably different. If
  LPL used a zero-one loss function, then the HPD interval would be
  produced. An advantage of the LPL interval over HPD interval (see
  \code{\link{p.interval}}) is that the LPL interval is invariant to
  reparameterization. This is due to the invariant reparameterization
  property of reference priors. The quantile-based probability interval
  is also invariant to reparameterization. The LPL interval enjoys the
  same advantage as the HPD interval does over the quantile-based
  probability interval: it does not produce equal tails when
  inappropriate.

  Compared with probability intervals, the LPL interval is slightly less
  convenient to calculate. Although the prior distribution is specified
  within the \code{Model} specification function, the user must specify
  it for the \code{LPL.interval} function as well. A comparison of the
  quantile-based probability interval, HPD interval, and LPL interval is
  available here: \url{https://web.archive.org/web/20150214090353/http://www.bayesian-inference.com/credible}.
}
\value{
  A matrix is returned with one row and two columns. The row represents
  the parameter and the column names are \code{"Lower"} and
  \code{"Upper"}. The elements of the matrix are the lower and upper
  bounds of the LPL interval.
}
\references{
  Bernardo, J.M. (2005). "Intrinsic Credible Regions: An Objective
  Bayesian Approach to Interval Estimation". \emph{Sociedad de
  Estadistica e Investigacion Operativa}, 14(2), p. 317--384.

  Bernardo, J.M. and Rueda, R. (2002). "Bayesian Hypothesis Testing: A
  Reference Approach". \emph{International Statistical Review}, 70,
  p. 351--372.
}
\author{Statisticat, LLC.}
\seealso{
  \code{\link{KLD}},
  \code{\link{p.interval}},
  \code{\link{LaplacesDemon}}, and
  \code{\link{PMC}}.}
\examples{
library(LaplacesDemon)
#Although LPL is intended to be applied to output from LaplacesDemon or
#PMC, here is an example in which p(theta) ~ N(0,100), and
#p(theta | y) ~ N(1,10), given 1000 samples.
theta <- rnorm(1000,1,10)
LPL.interval(Prior=dnorm(theta,0,100^2), Posterior=theta, prob=0.95,
  plot=TRUE)
#A more practical example follows, but it assumes a model has been
#updated with LaplacesDemon or PMC, the output object is called Fit, and
#that the prior for the third parameter is normally distributed with
#mean 0 and variance 100: 
#temp <- Fit$Posterior2[,3]
#names(temp) <- colnames(Fit$Posterior2)[3]
#LPL.interval(Prior=dnorm(temp,0,100^2), Posterior=temp, prob=0.95,
#     plot=TRUE, PDF=FALSE)
}
\keyword{Credible Interval}
\keyword{Credible Region}
\keyword{Credible Set}
\keyword{LPL}
\keyword{LPLI}
\keyword{Probability Interval}
\keyword{Probability Region}
\keyword{Probability Set}

\name{log-log}
\alias{cloglog}
\alias{invcloglog}
\alias{invloglog}
\alias{loglog}
\title{The log-log and complementary log-log functions}
\description{
  The log-log and complementary log-log functions, as well as the
  inverse functions, are provided.
}
\usage{
cloglog(p)
invcloglog(x)
invloglog(x)
loglog(p)
}
\arguments{
  \item{x}{This is a vector of real values that will be transformed to
    the interval [0,1].}
  \item{p}{This is a vector of probabilities p in the interval [0,1]
    that will be transformed to the real line.}
}
\details{
  The logit and probit links are symmetric, because the probabilities
  approach zero or one at the same rate. The log-log and complementary
  log-log links are asymmetric. Complementary log-log links approach
  zero slowly and one quickly. Log-log links approach zero quickly and
  one slowly. Either the log-log or complementary log-log link will tend
  to fit better than logistic and probit, and are frequently used when
  the probability of an event is small or large. A mixture of the two
  links, the log-log and complementary log-log is often used, where each
  link is weighted. The reason that logit is so prevalent is because
  logistic parameters can be interpreted as odds ratios.
}
\value{
  \code{cloglog} returns \code{x},
  \code{invcloglog} and \code{invloglog} return probability \code{p},
  and \code{loglog} returns \code{x}.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{\code{\link{LaplacesDemon}}}
\examples{
library(LaplacesDemon)
x <- -5:5
p <- invloglog(x)
x <- loglog(p)
}
\keyword{Complementary log-log}
\keyword{Link Function}
\keyword{log-log}
\keyword{Transformation}

\name{Mode}
\alias{is.amodal}
\alias{is.bimodal}
\alias{is.multimodal}
\alias{is.trimodal}
\alias{is.unimodal}
\alias{Mode}
\alias{Modes}
\title{The Mode(s) of a Vector}
\description{
  The mode is a measure of central tendency. It is the value that occurs
  most frequently, or in a continuous probability distribution, it is
  the value with the most density. A distribution may have no modes
  (such as with a constant, or in a uniform distribution when no value
  occurs more frequently than any other), or one or more modes.
}
\usage{
is.amodal(x, min.size=0.1)
is.bimodal(x, min.size=0.1)
is.multimodal(x, min.size=0.1)
is.trimodal(x, min.size=0.1)
is.unimodal(x, min.size=0.1)
Mode(x)
Modes(x, min.size=0.1)
}
\arguments{
  \item{x}{This is a vector in which a mode (or modes) will be sought.}
  \item{min.size}{This is the minimum size that can be considered a
    mode, where size means the proportion of the distribution between
    areas of increasing kernel density estimates.}
}
\details{
  The \code{is.amodal} function is a logical test of whether or not
  \code{x} has a mode. If \code{x} has a mode, then \code{TRUE} is
  returned, otherwise \code{FALSE}.

  The \code{is.bimodal} function is a logical test of whether or not
  \code{x} has two modes. If \code{x} has two modes, then \code{TRUE}
  is returned, otherwise \code{FALSE}.

  The \code{is.multimodal} function is a logical test of whether or not
  \code{x} has multiple modes. If \code{x} has multiple modes, then
  \code{TRUE} is returned, otherwise \code{FALSE}.

  The \code{is.trimodal} function is a logical test of whether or not
  \code{x} has three modes. If \code{x} has three modes, then \code{TRUE}
  is returned, otherwise \code{FALSE}.

  The \code{is.unimodal} function is a logical test of whether or not
  \code{x} has one mode. If \code{x} has one mode, then \code{TRUE}
  is returned, otherwise \code{FALSE}.
  
  The \code{Mode} function returns the most frequent value when \code{x}
  is discrete. If \code{x} is a constant, then it is considered amodal,
  and \code{NA} is returned. If multiple modes exist, this function
  returns only the mode with the highest density, or if two or more
  modes have the same density, then it returns the first mode found.
  Otherwise, the \code{Mode} function returns the value of \code{x}
  associated with the highest kernel density estimate, or the first
  one found if multiple modes have the same density.

  The \code{Modes} function is a simple, deterministic function that
  differences the kernel density of \code{x} and reports a number of
  modes equal to half the number of changes in direction, although the
  \code{min.size} function can be used to reduce the number of modes
  returned, and defaults to 0.1, eliminating modes that do not have at
  least 10\% of the distributional area. The \code{Modes} function
  returns a list with three components: \code{modes}, \code{modes.dens},
  and \code{size}. The elements in each component are ordered according
  to the decreasing density of the modes. The \code{modes} component is
  a vector of the values of \code{x} associated with the modes. The
  \code{modes.dens} component is a vector of the kernel density
  estimates at the modes. The \code{size} component is a vector of the
  proportion of area underneath each mode.

  The \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}}, and \code{\link{VariationalBayes}}
  functions characterize the marginal posterior distributions by
  posterior modes (means) and variance. A related topic is MAP or
  maximum \emph{a posteriori} estimation.

  Otherwise, the results of Bayesian inference tend to report the
  posterior mean or median, along with probability intervals (see
  \code{\link{p.interval}} and \code{\link{LPL.interval}}), rather than
  posterior modes. In many types of models, such as mixture models, the
  posterior may be multimodal. In such a case, the usual recommendation
  is to choose the highest mode if feasible and possible. However, the
  highest mode may be uncharacteristic of the majority of the posterior.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{LPL.interval}},
  \code{\link{p.interval}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
library(LaplacesDemon)
### Below are distributions with different numbers of modes.
x <- c(1,1) #Amodal
x <- c(1,2,2,2,3) #Unimodal
x <- c(1,2) #Bimodal
x <- c(1,3,3,3,3,4,4,4,4,4) #min.size affects the answer
x <- c(1,1,3,3,3,3,4,4,4,4,4) #Trimodal

### And for each of the above, the functions below may be applied.
Mode(x)
Modes(x)
is.amodal(x)
is.bimodal(x)
is.multimodal(x)
is.trimodal(x)
is.unimodal(x)
}
\keyword{Mode}
\keyword{Utility}

\name{Validate}
\alias{Validate}
\title{Holdout Validation}
\description{
  This function performs holdout validation on an object of class
  \code{demonoid} or \code{pmc}, given both a modeled and validation
  data set.
}
\usage{
Validate(object, Model, Data, plot=FALSE, PDF=FALSE)
}
\arguments{
  \item{object}{This is an object of class \code{demonoid} or
    \code{pmc}.}
  \item{Model}{This is a model specification function for
    \code{\link{LaplacesDemon}} or \code{\link{PMC}}.}
  \item{Data}{This is a list that contains two lists of data, as
    specified for \code{\link{LaplacesDemon}}. The first component in
    the list is the list of modeled data, and the second component in
    the list is the list of validation data.}
  \item{plot}{Logical. When \code{plot=TRUE}, two plots are
    displayed. The upper plot shows the density of the modeled deviance
    in black and the density of the validation deviance in red. The
    lower plot shows the density of the change in deviance in gray. The
    \code{plot} argument defaults to \code{FALSE}.}
  \item{PDF}{Logical. When \code{PDF=TRUE} (and \code{plot=TRUE}), the
    plot is saved as a .pdf file. The \code{PDF} argument defaults to
    \code{FALSE}.}
}
\details{
  There are numerous ways to validate a model. In this context,
  validation means to assess the predictive performance of a model on
  out-of-sample data. If reasonable, leave-one-out cross-validation
  (LOOCV) via the conditional predictive ordinate (CPO) should be
  considered when using \code{\link{LaplacesDemon}} or
  \code{\link{PMC}}. For more information on CPO, see the accompanying
  vignettes entitled "Bayesian Inference" and "Examples". CPO is
  unavailable when using \code{\link{LaplaceApproximation}} or
  \code{\link{VariationalBayes}}.

  For \code{\link{LaplaceApproximation}} or
  \code{\link{VariationalBayes}}, it is recommended that the user
  perform holdout validation by comparing posterior predictive checks,
  comparing the differences in the specified discrepancy measure.

  When LOOCV is unreasonable, popular alternatives include k-fold
  cross-validation and holdout validation. Although k-fold
  cross-validation is not performed explicitly here, the user may
  accomplish it with some effort. Of these methods, holdout validation
  includes the most bias, but is the most common in applied use, since
  only one model is fitted, rather than \eqn{k-1} models in k-fold
  cross-validation. The \code{Validate} function performs holdout
  validation.

  For holdout validation, the observed data is sampled randomly into two
  data sets of approximately equal size, or three data sets that
  consists of two data sets of approximately equal size and a remainder
  data set. Of the two data sets approximately equal in size, one is
  called the modeled (or training) data set, and the other is called the
  validation (or test) data set. The modeled data set is used when
  updating the model. After the model is updated, both data sets are
  predicted in the \code{Validate} function, given the model. Predictive
  loss is estimated for the validation data set, relative to the modeled
  data set.

  Predictive loss is associated with overfitting, differences between
  the model and validation data set, or model misspecification. Bayesian
  inference is reputed to be much more robust to overfitting than
  frequentist inference.

  There are many ways to measure predictive loss, and within each
  approach, there are usually numerous possible loss functions. The
  log-likelihood of the model is a popular approximate utility function,
  and consequently, the deviance of the model is a popular loss
  function.

  A vector of model-level (rather than record-level) deviance
  samples is returned with each object of class \code{demonoid} or
  \code{pmc}. The \code{Validate} function obtains this vector for each
  data set, and then calculates the Bayesian Predictive Information
  Criterion (BPIC), as per Ando (2007). BPIC is a variation of the
  Deviance Information Criterion (DIC) that has been modified for
  predictive distributions. For more information on DIC (Spiegelhalter
  et al., 2002), see the accompanying vignette entitled "Bayesian
  Inference". The goal is to minimize BPIC.

  When DIC is applied after the model, such as with a predictive
  distribution, it is positively biased, or too small. The bias is due
  to the same data \eqn{\textbf{y}}{y} being used both to construct the
  posterior distributions and to evaluate pD, the penalty term for model
  complexity. For example, for validation data set
  \eqn{\textbf{y}_{new}}{ynew}, BPIC is:

  \deqn{BPIC = -2\mathrm{log}[p(\textbf{y}_{new}|\textbf{y},\Theta)] +
    2pD}{BPIC = -2log[p(ynew|y,Theta)] + 2pD}

  When \code{plot=TRUE}, the distributions of the modeled and validation
  deviances are plotted above, and the lower plot is the modeled
  deviance subtracted from the validation deviance. When positive, this
  distribution of the change in deviance is the loss in predictive
  deviance associated with moving from the modeled data set to the
  validation data set.

  After using the \code{Validate} function, the user is encouraged to
  perform posterior predictive checks on each data set via the
  \code{\link{summary.demonoid.ppc}} or \code{\link{summary.pmc.ppc}}
  function.
}
\value{
  This function returns a list with three components. The first two
  components are also lists. Each list consists of \code{y},
  \code{yhat}, and \code{Deviance}. The third component is a matrix that
  reports the expected deviance, pD, and BPIC. The object is of class
  \code{demonoid.val} for \code{\link{LaplacesDemon}}, or \code{pmc.val}
  when associated with \code{\link{PMC}}.
}
\references{
  Ando, T. (2007). "Bayesian Predictive Information Criterion for
  the Evaluation of Hierarchical Bayesian and Empirical Bayes Models".
  \emph{Biometrika}, 94(2), p. 443--458.
  
  Spiegelhalter, D.J., Best, N.G., Carlin, B.P., and van der Linde, A.
  (2002). "Bayesian Measures of Model Complexity and Fit (with
  Discussion)". \emph{Journal of the Royal Statistical Society}, B 64,
  p. 583--639.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
library(LaplacesDemon)
#Given an object called Fit of class demonoid, a Model specification,
#and a modeled data set (MyData.M) and validation data set (MyData.V):
#Validate(Fit, Model, Data=list(MyData.M=MyData.M, MyData.V=MyData.V))
}
\keyword{BPIC}
\keyword{Model Selection}
\keyword{Posterior Predictive Checks}
\keyword{Utility}

\name{is.proper}
\alias{is.proper}
\title{Logical Check of Propriety}
\description{
  This function provides a logical check of the propriety of a
  univariate prior probability distribution or the joint posterior
  distribution.
}
\usage{
is.proper(f, a, b, tol=1e-5)
}
\arguments{
  \item{f}{This is either a probability density function or an object of
    class \code{demonoid}, \code{laplace}, \code{pmc}, or \code{vb}.}
  \item{a}{This is the lower limit of integration, and may be negative
    infinity.}
  \item{b}{This is the upper limit of integration, and may be positive
    infinity.}
  \item{tol}{This is the tolerance, and indicates the allowable
    difference from one.}
}
\details{
  A proper probability distribution is a probability distribution that
  integrates to one, and an improper probability distribution does not
  integrate to one. If a probability distribution integrates to any
  positive and finite value other than one, then it is an improper
  distribution, but is merely unnormalized. An unnormalized distribution
  may be multiplied by a constant so that it integrates to one.

  In Bayesian inference, the posterior probability distribution should
  be proper. An improper prior distribution can cause an improper
  posterior distribution. When the posterior distribution is improper,
  inferences are invalid, it is non-integrable, and Bayes factors cannot
  be used (though there are exceptions).

  To avoid these problems, it is suggested that the prior probability
  distribution should be proper, though it is possible to use an
  improper prior distribution and have it result in a proper posterior
  distribution.

  To check the propriety of a univariate prior probability distribution,
  create a function \code{f}. For example, to check the propriety of a
  vague normal distribution, such as

  \deqn{\theta \sim \mathcal{N}(0,1000)}{theta ~ N(0,1000)}

  the function is \code{function(x){dnormv(x,0,1000)}}. Next, set the
  lower and upper limits of integration, \code{a} and
  \code{b}. Internally, this function calls \code{integrate} from base
  R, which uses adaptive quadrature. By using \eqn{f(x)} as shorthand
  for the specified function, \code{is.proper} will check to see if the
  area of the following integral is one:

  \deqn{\int^b_a f(x)dx}{integral from a to b of f(x)dx}

  Multivariate prior probability distributions currently cannot be
  checked for approximate propriety. This is currently unavailable in
  this package.

  To check the propriety of the joint posterior distribution, the only
  argument to be supplied is an object of class \code{demonoid},
  \code{iterquad}, \code{laplace}, \code{pmc}, or \code{vb}. The
  \code{is.proper} function checks the logarithm of the marginal
  likelihood (see \code{\link{LML}}) for a finite value, and returns
  \code{TRUE} when the LML is finite. This indicates that the marginal
  likelihood is finite for all observed \eqn{\textbf{y}}{y} in the model
  data set. This implies:

  \deqn{\int p(\theta|\textbf{y})p(\theta)d\theta < \infty}{integral of
  p(theta|y)p(theta)dtheta < Inf}

  If the object is of class \code{demonoid} and the algorithm was
  adaptive, or if the object is of class \code{iterquad},
  \code{laplace}, or \code{vb} and the algorithm did not converge, then
  \code{is.proper} will return \code{FALSE} because LML was not
  estimated. In this case, it is possible for the joint posterior to be
  proper, but \code{is.proper} will be unable to determine propriety
  without the estimate of LML. If desired, the \code{\link{LML}} may be
  estimated by the user, and if it is finite, then the joint posterior
  distribution is proper.
}
\value{
  The \code{is.proper} function returns a logical value indicating
  whether or not the univariate prior or joint posterior probability
  distribution integrates to one within its specified limits.
  \code{TRUE} is returned for a proper univariate probability
  distribution.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dnormv}},
  \code{\link{integrate}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{LML}},
  \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
library(LaplacesDemon)
### Prior Probability Distribution
is.proper(function(x) {dnormv(x,0,1000)}, -Inf, Inf) #x ~ N(0,1000)
is.proper(function(x) {dhalfcauchy(x,25)}, 0, Inf) #x ~ HC(25)
is.proper(function(x) {dunif(x,0,1)}, 0, 1) #x ~ U(0,1)
is.proper(function(x) {dunif(x,-Inf,Inf)}, -Inf, Inf) #x ~ U(-Inf,Inf)
### Joint Posterior Distribution
##This assumes that Fit is an object of class demonoid, iterquad,
##  laplace, or pmc
#is.proper(Fit)
}
\keyword{Diagnostic}
\keyword{Distribution}
\keyword{Utility}

\name{Gelman.Diagnostic}
\alias{Gelman.Diagnostic}
\title{Gelman and Rubin's MCMC Convergence Diagnostic}
\description{
  Gelman and Rubin (1992) proposed a general approach to monitoring
  convergence of MCMC output in which \eqn{m > 1} parallel chains are
  updated with initial values that are overdispersed relative to each
  target distribution, which must be normally distributed. Convergence
  is diagnosed when the chains have `forgotten' their initial values,
  and the output from all chains is indistinguishable. The
  \code{Gelman.Diagnostic} function makes a comparison of within-chain
  and between-chain variances, and is similar to a classical analysis of
  variance. A large deviation between these two variances indicates
  non-convergence.

  This diagnostic is popular as a stopping rule, though it requires
  parallel chains. The \code{\link{LaplacesDemon.hpc}} function is an
  extension of \code{\link{LaplacesDemon}} to enable parallel chains.
  As an alternative, the popular single-chain stopping rule is based on
  \code{\link{MCSE}}.
}
\usage{Gelman.Diagnostic(x, confidence=0.95, transform=FALSE)}
\arguments{
  \item{x}{This required argument accepts an object of class
    \code{demonoid.hpc}, or a list of multiple objects of class
    \code{demonoid}, where the number of components in the list 
    is the number of chains.}
  \item{confidence}{This is the coverage probability of the confidence
    interval for the potential scale reduction factor (PSRF).}
  \item{transform}{Logical. If \code{TRUE}, then marginal posterior
    distributions in \code{x} may be transformed to improve the
    normality of the distribution, which is assumed. A log-transform is
    applied to marginal posterior distributions in the interval \eqn{(0,
      \infty]}{(0, Inf]}, or a logit-transform is applied to marginal
    posterior distributions in the interval \eqn{(0,1)}.}
}
\value{
  A list is returned with the following components:
  
  \item{PSRF}{This is a list containing the point-estimates of the
    potential scale reduction factor (labelled \code{Point Est.}) and
    the associated upper confidence limits (labelled \code{Upper C.I.}).}
  \item{MPSRF}{This is the point-estimate of the multivariate potential
    scale reduction factor.}
}
\details{
  To use the \code{Gelman.Diagnostic} function, the user must first have
  multiple MCMC chains for the same model, and three chains is usually
  sufficient. The easiest way to obtain multiple chains is with the
  \code{\link{LaplacesDemon.hpc}} function.

  Although the \code{\link{LaplacesDemon}} function does not
  simultaneously update multiple MCMC chains, it is easy enough to
  obtain multiple chains, and if the computer has multiple processors
  (which is common), then multiple chains may be obtained simultaneously
  as follows. The model file may be opened in separate, concurrent R
  sessions, and it is recommended that a maximum number of sessions is
  equal to the number of processors, minus one. Each session constitutes
  its own chain, and the code is identical, except the initial values
  should be randomized with the \code{\link{GIV}} function so the chains
  begin in different places. The resulting object of class
  \code{demonoid} for each chain is saved, all objects are read into one
  session, put into a list, and passed to the \code{Gelman.Diagnostic}
  function.

  Initial values must be overdispersed with respect to each target
  distribution, though these distributions are unknown in the
  beginning. Since the \code{Gelman.Diagnostic} function relies heavily
  on overdispersion with respect to the target distribution, the user
  should consider using MCMC twice, first to estimate the target
  distributions, and secondly to overdisperse initial values with
  respect to them. This may help identify multimodal target
  distributions. If multiple modes are found, it remain possible that
  more modes exist. When multiple modes are found, and if chains are
  combined with the \code{\link{Combine}} function, each mode is
  probably not represented in a proportion correct to the distribution.
    
  The `potential scale reduction factor' (PSRF) is an estimated factor
  by which the scale of the current distribution for the target
  distribution might be reduced if the simulations were continued for
  an infinite number of iterations. Each PSRF declines to 1 as the
  number of iterations approaches infinity. PSRF is also often
  represented as R-hat. PSRF is calculated for each marginal posterior
  distribution in \code{x}, together with upper and lower confidence
  limits. Approximate convergence is diagnosed when the upper limit is
  close to 1. The recommended proximity of each PSRF to 1 varies with
  each problem, but a general goal is to achieve PSRF < 1.1. PSRF is an
  estimate of how much narrower the posterior might become with an
  infinite number of iterations. When PSRF = 1.1, for example, it may be
  interpreted as a potential reduction of 10\% in posterior interval
  width, given infinite iterations. The multivariate form bounds above
  the potential scale reduction factor for any linear combination of the
  (possibly transformed) variables.
  
  The confidence limits are based on the assumption that the
  target distribution is stationary and normally distributed. The
  \code{transform} argument may be used to improve the normal
  approximation.

  A large PSRF indicates that the between-chain variance is
  substantially greater than the within-chain variance, so that longer
  simulation is needed. If a PSRF is close to 1, then the associated
  chains are likely to have converged to one target distribution. A
  large PSRF (perhaps generally when a PSRF > 1.2) indicates convergence
  failure, and can indicate the presence of a multimodal marginal
  posterior distribution in which different chains may have converged
  to different local modes (see \code{\link{is.multimodal}}), or the
  need to update the associated chains longer, because burn-in (see
  \code{\link{burnin}}) has yet to be completed.
  
  The \code{Gelman.Diagnostic} is essentially the same as the
  \code{gelman.diag} function in the \code{coda} package, but here it is 
  programmed to work with objects of class \code{demonoid}.

  There are two ways to estimate the variance of the stationary
  distribution: the mean of the empirical variance within each chain,
  \eqn{W}, and the empirical variance from all chains combined, which
  can be expressed as

  \deqn{ \widehat{\sigma}^2 = \frac{(n-1) W}{n} +
    \frac{B}{n}}{sigma.hat^2 = (n-1)W/n + B/n} 

  where \eqn{n} is the number of iterations and \eqn{B/n} is the
  empirical between-chain variance.

  If the chains have converged, then both estimates are
  unbiased. Otherwise the first method will \emph{underestimate} the
  variance, since the individual chains have not had time to range all
  over the stationary distribution, and the second method will
  \emph{overestimate} the variance, since the initial values were
  chosen to be overdispersed (and this assumes the target distribution
  is known, see above).

  This convergence diagnostic is based on the assumption that each
  target distribution is normal. A Bayesian probability interval (see
  \code{\link{p.interval}}) can be constructed using a t-distribution
  with mean
  
  \deqn{\widehat{\mu}=\mbox{Sample mean of all chains combined,}}{mu.hat
    = Sample mean of all chains combined,}

  variance

  \deqn{\widehat{V} = \widehat{\sigma}^2 + \frac{B}{mn},}{V.hat =
    sigma.hat2 + B/(mn),}

  and degrees of freedom estimated by the method of moments

  \deqn{d = \frac{2\widehat{V}^2}{\mbox{Var}(\widehat{V})}}{d = 2*V.hat^2 /
    Var(V.hat)}

  Use of the t-distribution accounts for the fact that the mean and
  variance of the posterior distribution are estimated. The convergence
  diagnostic itself is

  \deqn{R=\sqrt{\frac{(d+3) \widehat{V}}{(d+1)W}}}{R=sqrt((d+3) V.hat /((d+1)W)}

  Values substantially above 1 indicate lack of convergence. If the
  chains have not converged, then Bayesian probability intervals based
  on the t-distribution are too wide, and have the potential to shrink
  by this factor if the MCMC run is continued.

  The multivariate version of Gelman and Rubin's diagnostic was proposed
  by Brooks and Gelman (1998). Unlike the univariate proportional scale
  reduction factor, the multivariate version does not include an
  adjustment for the estimated number of degrees of freedom.
}
\references{
  Brooks, S.P. and Gelman, A. (1998). "General Methods for Monitoring
  Convergence of Iterative Simulations". \emph{Journal of Computational
  and Graphical Statistics}, 7, p. 434--455.

  Gelman, A. and Rubin, D.B. (1992). "Inference from Iterative Simulation
  using Multiple Sequences". \emph{Statistical Science}, 7, p. 457--511.
}
\seealso{
  \code{\link{Combine}},
  \code{\link{GIV}},
  \code{\link{is.multimodal}},
  \code{\link{LaplacesDemon}},
  \code{\link{LaplacesDemon.hpc}},
  \code{\link{MCSE}}, and
  \code{\link{p.interval}}.
}
\examples{
#library(LaplacesDemon)
###After updating multiple chains with LaplacesDemon.hpc, do:
#Gelman.Diagnostic(Fit)
}
\keyword{Diagnostic}
\keyword{MCMC}
\keyword{Multiple Chains}
\keyword{Parallel Chains}

\name{is.class}
\alias{is.bayesfactor}
\alias{is.blocks}
\alias{is.bmk}
\alias{is.demonoid}
\alias{is.demonoid.hpc}
\alias{is.demonoid.ppc}
\alias{is.demonoid.val}
\alias{is.hangartner}
\alias{is.heidelberger}
\alias{is.importance}
\alias{is.iterquad}
\alias{is.iterquad.ppc}
\alias{is.juxtapose}
\alias{is.laplace}
\alias{is.laplace.ppc}
\alias{is.miss}
\alias{is.pmc}
\alias{is.pmc.ppc}
\alias{is.pmc.val}
\alias{is.posteriorchecks}
\alias{is.raftery}
\alias{is.rejection}
\alias{is.sensitivity}
\alias{is.vb}
\alias{is.vb.ppc}
\title{Logical Check of Classes}
\description{
  These functions each provide a logical test of the class of an object.
}
\usage{
is.bayesfactor(x)
is.blocks(x)
is.bmk(x)
is.demonoid(x)
is.demonoid.hpc(x)
is.demonoid.ppc(x)
is.demonoid.val(x)
is.hangartner(x)
is.heidelberger(x)
is.importance(x)
is.iterquad(x)
is.iterquad.ppc(x)
is.juxtapose(x)
is.laplace(x)
is.laplace.ppc(x)
is.miss(x)
is.pmc(x)
is.pmc.ppc(x)
is.pmc.val(x)
is.posteriorchecks(x)
is.raftery(x)
is.rejection(x)
is.sensitivity(x)
is.vb(x)
is.vb.ppc(x)
}
\arguments{
  \item{x}{This is an object that will be subjected to a logical test of
    its class.}
}
\details{
  Functions in Laplace's Demon often assigns a class to an output
  object. For example, after updating a model with the
  \code{\link{LaplacesDemon}} or \code{\link{LaplacesDemon.hpc}}
  function, an output object is created. The output object is of class
  \code{demonoid} or \code{demonoid.hpc}, respectively. Likewise, after
  passing a model to the \code{\link{LaplaceApproximation}} function, an
  output object is created, and it is of class \code{laplace}. The class
  of these and other objects may be logically tested.

  By assigning a class to an output object, the package is able to
  discern which other functions are appropriate for it. For example,
  after updating a model with \code{\link{LaplacesDemon}}, which creates
  an object of class \code{demonoid}, the user may desire to plot its
  output. Since it is assigned a class, the user may use the generic
  \code{plot} function, which internally selects the
  \code{\link{plot.demonoid}} function, which differs from
  \code{\link{plot.laplace}} for objects of class \code{laplace}.

  For more information on object classes, see the \code{\link{class}}
  function.
}
\value{
  The \code{is.bayesfactor} function returns a logical value indicating
  whether or not the supplied object is of class \code{bayesfactor}.
  
  The \code{is.blocks} function returns a logical value indicating
  whether or not the supplied object is of class \code{blocks}.

  The \code{is.bmk} function returns a logical value indicating
  whether or not the supplied object is of class \code{bmk}.
  
  The \code{is.demonoid} function returns a logical value indicating
  whether or not the supplied object is of class \code{demonoid}.

  The \code{is.demonoid.hpc} function returns a logical value indicating
  whether or not the supplied object is of class \code{demonoid.hpc}.

  The \code{is.demonoid.ppc} function returns a logical value indicating
  whether or not the supplied object is of class \code{demonoid.ppc}.

  The \code{is.demonoid.val} function returns a logical value indicating
  whether or not the supplied object is of class \code{demonoid.val}.

  The \code{is.hangartner} function returns a logical value indicating
  whether or not the supplied object is of class \code{hangartner}.

  The \code{is.heidelberger} function returns a logical value indicating
  whether or not the supplied object is of class \code{heidelberger}.

  The \code{is.importance} function returns a logical value indicating
  whether or not the supplied object is of class \code{importance}.

  The \code{is.iterquad} function returns a logical value indicating
  whether or not the supplied object is of class \code{iterquad}.

  The \code{is.iterquad.ppc} function returns a logical value indicating
  whether or not the supplied object is of class \code{iterquad.ppc}.

  The \code{is.juxtapose} function returns a logical value indicating
  whether or not the supplied object is of class \code{juxtapose}.

  The \code{is.laplace} function returns a logical value indicating
  whether or not the supplied object is of class \code{laplace}.

  The \code{is.laplace.ppc} function returns a logical value indicating
  whether or not the supplied object is of class \code{laplace.ppc}.

  The \code{is.miss} function returns a logical value indicating
  whether or not the supplied object is of class \code{miss}.

  The \code{is.pmc} function returns a logical value indicating
  whether or not the supplied object is of class \code{pmc}.

  The \code{is.pmc.ppc} function returns a logical value indicating
  whether or not the supplied object is of class \code{pmc.ppc}.

  The \code{is.pmc.val} function returns a logical value indicating
  whether or not the supplied object is of class \code{pmc.val}.

  The \code{is.posteriorchecks} function returns a logical value
  indicating whether or not the supplied object is of class
  \code{posteriorchecks}.

  The \code{is.raftery} function returns a logical value indicating
  whether or not the supplied object is of class \code{raftery}.

  The \code{is.rejection} function returns a logical value indicating
  whether or not the supplied object is of class \code{rejection}.

  The \code{is.sensitivity} function returns a logical value indicating
  whether or not the supplied object is of class \code{sensitivity}.

  The \code{is.vb} function returns a logical value indicating
  whether or not the supplied object is of class \code{vb}.

  The \code{is.vb.ppc} function returns a logical value indicating
  whether or not the supplied object is of class \code{vb.ppc}.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{BayesFactor}},
  \code{\link{Blocks}},
  \code{\link{BMK.Diagnostic}},
  \code{\link{class}},
  \code{\link{Hangartner.Diagnostic}},
  \code{\link{Heidelberger.Diagnostic}},
  \code{\link{Importance}},
  \code{\link{IterativeQuadrature}},
  \code{\link{Juxtapose}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{LaplacesDemon.hpc}},
  \code{\link{MISS}},
  \code{\link{PMC}},
  \code{\link{PosteriorChecks}},
  \code{\link{predict.demonoid}},
  \code{\link{predict.laplace}},
  \code{\link{predict.pmc}},
  \code{\link{predict.vb}},
  \code{\link{Raftery.Diagnostic}},
  \code{\link{RejectionSampling}},
  \code{\link{SensitivityAnalysis}},
  \code{\link{Validate}}, and
  \code{\link{VariationalBayes}}.
}
\keyword{Utility}

\name{CenterScale}
\alias{CenterScale}
\title{Centering and Scaling}
\description{
  This function either centers and scales a continuous variable and
  provides options for binary variables, or returns an untransformed
  variable from a centered and scaled variable.
}
\usage{
CenterScale(x, Binary="none", Inverse=FALSE, mu, sigma, Range, Min)
}
\arguments{
  \item{x}{This is a vector to be centered and scaled, or to be
    untransformed if \code{Inverse=TRUE}.}
  \item{Binary}{This argument indicates how binary variables will be
    treated, and defaults to \code{"none"}, which keeps the original
    scale, or transforms the variable to the 0-1 range, if not already
    there. With \code{"center"}, it will center the binary variable by
    subtracting the mean. With \code{"center0"}, it centers the binary
    variable at zero, recoding a 0 to -0.5, and a 1 to 0.5. Finally,
    \code{"centerscale"} will center and scale the binary variable,
    subtracting the mean and dividing by two standard deviations.}
  \item{Inverse}{Logical. If \code{TRUE}, then a centered and scaled
    variable \code{x} will be transformed to its original, un-centered
    and un-scaled state. This defaults to \code{FALSE}.}
  \item{mu, sigma, Range, Min}{These arguments are required only when
    \code{Inverse=TRUE}, where \code{mu} is the mean, \code{sigma} is
    the standard deviation, \code{Range} is the range, and \code{Min} is
    the minimum of the original \code{x}. \code{Range} and \code{Min}
    are used only when \code{Binary="none"} or \code{Binary="center0"}.}
}
\details{
  Gelman (2008) recommends centering and scaling continuous predictors
  to facilitate MCMC convergence and enable comparisons between
  coefficients of centered and scaled continuous predictors with
  coefficients of untransformed binary predictors. A continuous
  predictor is centered and scaled as follows: \code{x.cs <- (x -
    mean(x)) / (2*sd(x))}. This is an improvement over the usual
  practice of standardizing predictors, which is \code{x.z <- (x -
    mean(x)) / sd(x)}, where coefficients cannot be validly compared
  between binary and continuous predictors.

  In MCMC, such as in \code{\link{LaplacesDemon}}, a centered and
  scaled predictor often results in a higher effective sample size
  (\code{\link{ESS}}), and therefore the chain mixes better. Centering
  and scaling is a method of re-parameterization to improve mixing.

  Griffin and Brown (2013) also assert that the user may not want to
  scale predictors that are measured on the same scale, since scaling
  in this case may increase noisy, low signals. In this case, centering
  (without scaling) is recommended. To center a predictor, subtract its
  mean.

}
\value{
  The \code{CenterScale} function returns a centered and scaled vector,
  or the untransformed vector.
}
\references{
  Gelman, A. (2008). "Scaling Regression Inputs by Dividing by Two Standard
  Devations". \emph{Statistics in Medicine}, 27, p. 2865--2873.

  Griffin, J.E. and Brown, P.J. (2013) "Some Priors for Sparse
  Regression Modelling". \emph{Bayesian Analysis}, 8(3), p. 691--702.
}
\seealso{
  \code{\link{ESS}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, and
  \code{\link{PMC}}.
}
\examples{
### See the LaplacesDemon function for an example in use.
library(LaplacesDemon)
x <- rnorm(100,10,1)
x.cs <- CenterScale(x)
x.orig <- CenterScale(x.cs, Inverse=TRUE, mu=mean(x), sigma=sd(x))
}
\keyword{Center}
\keyword{Centering}
\keyword{Predictor}
\keyword{Scale}
\keyword{Scaling}
\keyword{Transformation}

\name{dist.Categorical}
\alias{dcat}
\alias{qcat}
\alias{rcat}
\title{Categorical Distribution}
\description{
  This is the density and random deviates function for the categorical
  distribution with probabilities parameter \eqn{p}.
}
\usage{
dcat(x, p, log=FALSE)
qcat(pr, p, lower.tail=TRUE, log.pr=FALSE)
rcat(n, p)
}
\arguments{
  \item{x}{This is a vector of discrete data with \eqn{k} discrete
    categories, and is of length \eqn{n}. This function also accepts
    \eqn{x} after it has been converted to an \eqn{n \times k}{n x k}
    indicator matrix, such as with the \code{as.indicator.matrix} function.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1. When \code{p} is supplied to \code{rcat}
    as a matrix, \code{n} must equal the number of rows in \code{p}.}
  \item{p}{This is a vector of length \eqn{k} or \eqn{n \times k}{n x k}
    matrix of probabilities. The \code{qcat} function requires a
    vector.}
  \item{pr}{This is a vector of probabilities, or log-probabilities.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
  \item{log.pr}{Logical. if \code{TRUE}, probabilities \eqn{pr} are
    given as \eqn{\log(pr)}{log(pr)}.}
  \item{lower.tail}{Logical. if \code{TRUE} (default), probabilities
    are \eqn{Pr[X \le x]}{Pr[X <= x]}, otherwise,
    \eqn{Pr[X > x]}{Pr[X > x]}.}
}
\details{
    \itemize{
  \item Application: Discrete Univariate
  \item Density: \eqn{p(\theta) = \sum \theta p}{p(theta) = Sum (theta * p)}
  \item Inventor: Unknown (to me, anyway)
  \item Notation 1: \eqn{\theta \sim \mathcal{CAT}(p)}{theta ~ CAT(p)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{CAT}(\theta | p)}{p(theta) = CAT(theta | p)}
  \item Parameter 1: probabilities \eqn{p}
  \item Mean: \eqn{E(\theta)}{E(theta)} = Unknown
  \item Variance: \eqn{var(\theta)}{var(theta)} = Unknown
  \item Mode: \eqn{mode(\theta)}{mode(theta)} = Unknown
}
Also called the discrete distribution, the categorical distribution
describes the result of a random event that can take on one of \eqn{k}
possible outcomes, with the probability \eqn{p} of each outcome
separately specified. The vector \eqn{p} of probabilities for each
event must sum to 1. The categorical distribution is often used, for
example, in the multinomial logit model. The conjugate prior is the
Dirichlet distribution.
}
\value{
  \code{dcat} gives the density and
  \code{rcat} generates random deviates.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{as.indicator.matrix}},
  \code{\link{ddirichlet}}, and
  \code{\link{dmultinom}}.
}
\examples{
library(LaplacesDemon)
dcat(x=1, p=c(0.3,0.3,0.4))
rcat(n=10, p=c(0.1,0.3,0.6))
}
\keyword{Distribution}

\name{logit}
\alias{invlogit}
\alias{logit}
\title{The logit and inverse-logit functions}
\description{
  The logit and inverse-logit (also called the logistic function) are
  provided.
}
\usage{
invlogit(x)
logit(p)
}
\arguments{
  \item{x}{This object contains real values that will be transformed to
    the interval [0,1].}
  \item{p}{This object contains of probabilities p in the interval [0,1]
    that will be transformed to the real line.}
}
\details{
  The \code{logit} function is the inverse of the sigmoid or logistic
  function, and transforms a continuous value (usually probability
  \eqn{p}) in the interval [0,1] to the real line (where it is usually
  the logarithm of the odds). The \code{logit} function is \eqn{\log(p /
    (1-p))}{log(p / (1 - p))}.
  
  The \code{invlogit} function (called either the inverse logit or the
  logistic function) transforms a real number (usually the logarithm of
  the odds) to a value (usually probability \eqn{p}) in the interval
  [0,1]. The \code{invlogit} function is \eqn{\frac{1}{1 + \exp(-x)}}{1
    / (1 + exp(-x))}.

  If \eqn{p} is a probability, then \eqn{\frac{p}{1-p}}{p/(1-p)} is the
  corresponding odds, while the \code{logit} of \eqn{p} is the logarithm
  of the odds. The difference between the logits of two probabilities is
  the logarithm of the odds ratio. The derivative of probability \eqn{p}
  in a logistic function (such as \code{invlogit}) is: \eqn{\frac{d}{dx}
    = p(1-p)}{(d / dx) = p * (1 - p)}.

  In the LaplacesDemon package, it is common to re-parameterize a model
  so that a parameter that should be in an interval can be updated from
  the real line by using the \code{logit} and \code{invlogit} functions,
  though the \code{\link{interval}} function provides an
  alternative. For example, consider a parameter \eqn{\theta}{theta}
  that must be in the interval [0,1]. The algorithms in
  \code{\link{IterativeQuadrature}}, \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, \code{\link{PMC}}, and
  \code{\link{VariationalBayes}} are unaware of the desired interval,
  and may attempt \eqn{\theta}{theta} outside of this interval. One
  solution is to have the algorithms update \code{logit(theta)} rather
  than \code{theta}. After \code{logit(theta)} is manipulated by the
  algorithm, it is transformed via \code{invlogit(theta)} in the model
  specification function, where \eqn{\theta \in [0,1]}{theta in [0,1]}.
}
\value{
  \code{invlogit} returns probability \code{p}, and
  \code{logit} returns \code{x}.
}
\seealso{
  \code{\link{interval}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{plogis}},
  \code{\link{PMC}},
  \code{\link{qlogis}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
library(LaplacesDemon)
x <- -5:5
p <- invlogit(x)
x <- logit(p)
}
\keyword{Inverse Logit}
\keyword{Link Function}
\keyword{Logistic Function}
\keyword{Logit}
\keyword{Transformation}

\name{dist.Normal.Wishart}
\alias{dnormwishart}
\alias{rnormwishart}
\title{Normal-Wishart Distribution}
\description{
  These functions provide the density and random number generation for
  the normal-Wishart distribution.
}
\usage{
dnormwishart(mu, mu0, lambda, Omega, S, nu, log=FALSE) 
rnormwishart(n=1, mu0, lambda, S, nu)
}
\arguments{
  \item{mu}{This is data or parameters in the form of a vector of length
    \eqn{k} or a matrix with \eqn{k} columns.}
  \item{mu0}{This is mean vector \eqn{\mu_0}{mu[0]} with length \eqn{k} or
    matrix with \eqn{k} columns.}
  \item{lambda}{This is a positive-only scalar.}
  \item{n}{This is the number of random draws.}
  \item{nu}{This is the scalar degrees of freedom \eqn{\nu}{nu}.}
  \item{Omega}{This is a \eqn{k \times k}{k x k} precision matrix
    \eqn{\Omega}{Omega}.}
  \item{S}{This is the symmetric, positive-semidefinite, \eqn{k \times
      k}{k x k} scale matrix \eqn{\textbf{S}}{S}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
       density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Multivariate
    \item Density: \eqn{p(\mu, \Omega) = \mathcal{N}(\mu | \mu_0,
      (\lambda\Omega)^{-1}) \mathcal{W}(\Omega | \nu, \textbf{S})}{p(mu,
      Omega) = N(mu | mu[0], (lambda Omega)^(-1)) W(Omega | nu, S)}
  \item Inventors: Unknown
  \item Notation 1: \eqn{(\mu, \Omega) \sim \mathcal{NW}(\mu_0, \lambda,
    \textbf{S}, \nu)}{(mu, Omega) ~ NW(mu[0], lambda, S, nu)}
  \item Notation 2: \eqn{p(\mu, \Omega) = \mathcal{NW}(\mu, \Omega |
    \mu_0, \lambda, \textbf{S}, \nu)}{p(mu, Omega) = NW(mu, Omega |
    mu[0], lambda, S, nu)}
  \item Parameter 1: location vector \eqn{\mu_0}{mu[0]}
  \item Parameter 2: \eqn{\lambda > 0}{lambda > 0}
  \item Parameter 3: symmetric, positive-semidefinite
    \eqn{k \times k}{k x k} scale matrix \eqn{\textbf{S}}{S}
  \item Parameter 4: degrees of freedom \eqn{\nu \ge k}{nu >= k}
  \item Mean: Unknown
  \item Variance: Unknown
  \item Mode: Unknown
}

The normal-Wishart distribution, or Gaussian-Wishart distribution, is a
multivariate four-parameter continuous probability distribution. It is
the conjugate prior of a multivariate normal distribution with unknown
mean and precision matrix.
}
\value{
  \code{dnormwishart} gives the density and 
  \code{rnormwishart} generates random deviates and returns a list with
    two components.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dmvnp}} and
  \code{\link{dwishart}}.
}
\examples{
library(LaplacesDemon)
K <- 3
mu <- rnorm(K)
mu0 <- rnorm(K)
nu <- K + 1
S <- diag(K)
lambda <- runif(1) #Real scalar
Omega <- as.positive.definite(matrix(rnorm(K^2),K,K))
x <- dnormwishart(mu, mu0, lambda, Omega, S, nu, log=TRUE)
out <- rnormwishart(n=10, mu0, lambda, S, nu)
joint.density.plot(out$mu[,1], out$mu[,2], color=TRUE)
}
\keyword{Distribution}

\name{dist.Multivariate.Power.Exponential.Cholesky}
\alias{dmvpec}
\alias{rmvpec}
\title{Multivariate Power Exponential Distribution: Cholesky Parameterization}
\description{
  These functions provide the density and random number generation for
  the multivariate power exponential distribution, given the Cholesky
  parameterization.
}
\usage{
dmvpec(x=c(0,0), mu=c(0,0), U, kappa=1, log=FALSE)
rmvpec(n, mu=c(0,0), U, kappa=1)
}
\arguments{
  \item{x}{This is data or parameters in the form of a vector of length
    \eqn{k} or a matrix with \eqn{k} columns.}
  \item{n}{This is the number of random draws.}
  \item{mu}{This is mean vector \eqn{\mu}{mu} with length \eqn{k} or
    matrix with \eqn{k} columns.}
  \item{U}{This is the \eqn{k \times k}{k x k} upper-triangular matrix
    that is Cholesky factor \eqn{\textbf{U}}{U} of covariance matrix
    \eqn{\Sigma}{Sigma}.}
  \item{kappa}{This is the kurtosis parameter, \eqn{\kappa}{kappa}, and
    must be positive.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Multivariate
  \item Density: \deqn{p(\theta) = \frac{k\Gamma(k/2)}{\pi^{k/2}
      \sqrt{|\Sigma|} \Gamma(1 + k/(2\kappa)) 2^{1 + k/(2\kappa)}}
    \exp(-\frac{1}{2}(\theta-\mu)^T \Sigma
    (\theta-\mu))^{\kappa}}{p(theta) = ((k*Gamma(k/2)) / (pi^(k/2) * 
    sqrt(|Sigma|) * Gamma(1 + k/(2*kappa)) * 2^(1 + k/(2*kappa)))) *
          exp(-(1/2)*(theta-mu)^T Sigma (theta-mu))^kappa}
  \item Inventor: Gomez, Gomez-Villegas, and Marin (1998)
  \item Notation 1: \eqn{\theta \sim \mathcal{MPE}(\mu, \Sigma,
    \kappa)}{theta ~ MPE(mu, Sigma, kappa)}
  \item Notation 2: \eqn{\theta \sim \mathcal{PE}_k(\mu, \Sigma,
    \kappa)}{theta ~ PE[k](mu, Sigma, kappa)}
  \item Notation 3: \eqn{p(\theta) = \mathcal{MPE}(\theta | \mu, \Sigma,
    \kappa)}{p(theta) = MPE(theta | mu, Sigma, kappa)}
  \item Notation 4: \eqn{p(\theta) = \mathcal{PE}_k(\theta | \mu,
    \Sigma, \kappa)}{p(theta) = PE[k](theta | mu, Sigma, kappa)}
  \item Parameter 1: location vector \eqn{\mu}{mu}
  \item Parameter 2: positive-definite \eqn{k \times k}{k x k}
  covariance matrix \eqn{\Sigma}{Sigma}
  \item Parameter 3: kurtosis parameter \eqn{\kappa}{kappa}
  \item Mean: \eqn{E(\theta) = }{E(theta) = }
  \item Variance: \eqn{var(\theta) =}{var(theta) = }
  \item Mode: \eqn{mode(\theta) = }{mode(theta) = }
}

The multivariate power exponential distribution, or multivariate
exponential power distribution, is a multidimensional extension of
the one-dimensional or univariate power exponential distribution.
Gomez-Villegas (1998) and Sanchez-Manzano et al. (2002) proposed
multivariate and matrix generalizations of the PE family of
distributions and studied their properties in relation to multivariate
Elliptically Contoured (EC) distributions.

The multivariate power exponential distribution includes the
multivariate normal distribution (\eqn{\kappa = 1}{kappa = 1}) and
multivariate Laplace distribution (\eqn{\kappa = 0.5}{kappa = 0.5}) as
special cases, depending on the kurtosis or \eqn{\kappa}{kappa}
parameter. A multivariate uniform occurs as
\eqn{\kappa \rightarrow \infty}{kappa -> infinity}.

If the goal is to use a multivariate Laplace distribution, the
\code{dmvlc} function will perform faster and more accurately.

In practice, \eqn{\textbf{U}}{U} is fully unconstrained for proposals
when its diagonal is log-transformed. The diagonal is exponentiated
after a proposal and before other calculations. Overall, the Cholesky
parameterization is faster than the traditional parameterization.
Compared with \code{dmvpe}, \code{dmvpec} must additionally
matrix-multiply the Cholesky back to the covariance matrix, but it
does not have to check for or correct the covariance matrix to
positive-definiteness, which overall is slower. Compared with
\code{rmvpe}, \code{rmvpec} is faster because the Cholesky decomposition
has already been performed.

The \code{rmvpec} function is a modified form of the rmvpowerexp function
in the MNM package.
}
\value{
  \code{dmvpec} gives the density and
  \code{rmvpec} generates random deviates.
}
\references{
  Gomez, E., Gomez-Villegas, M.A., and Marin, J.M. (1998). "A
  Multivariate Generalization of the Power Exponential Family of
  Distributions". \emph{Communications in Statistics-Theory and
  Methods}, 27(3), p. 589--600.

  Sanchez-Manzano, E.G., Gomez-Villegas, M.A., and Marn-Diazaraque,
  J.M. (2002). "A Matrix Variate Generalization of the Power Exponential
  Family of Distributions". \emph{Communications in Statistics, Part A -
  Theory and Methods} [Split from: J(CommStat)], 31(12), p. 2167--2182.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{chol}},
  \code{\link{dlaplace}},
  \code{\link{dmvlc}},
  \code{\link{dmvnc}},
  \code{\link{dmvnpc}},
  \code{\link{dnorm}},
  \code{\link{dnormp}},
  \code{\link{dnormv}}, and
  \code{\link{dpe}}.
}
\examples{
library(LaplacesDemon)
n <- 100
k <- 3
x <- matrix(runif(n*k),n,k)
mu <- matrix(runif(n*k),n,k)
Sigma <- diag(k)
U <- chol(Sigma)
dmvpec(x, mu, U, kappa=1)
X <- rmvpec(n, mu, U, kappa=1)
joint.density.plot(X[,1], X[,2], color=TRUE)
}
\keyword{Distribution}

\name{Combine}
\alias{Combine}
\title{Combine Demonoid Objects}
\description{
  This function combines objects of class \code{demonoid}.
}
\usage{
Combine(x, Data, Thinning=1)
}
\arguments{
  \item{x}{This is a list of objects of class \code{demonoid}, and this
    list may be an object of class \code{demonoid.hpc}.}
  \item{Data}{This is the data, and must be identical to the data used
    to create the \code{demonoid} objects with
    \code{\link{LaplacesDemon}}.}
  \item{Thinning}{This is the amount of thinning to apply to the
    posterior samples after appending them together. \code{Thinning}
    defaults to 1, in which case all samples are retained. For example,
    in the case of, say, \code{Thinning=10}, then only every 10th sample
    would be retained. When combining parallel chains, \code{Thinning}
    is often left to its default. When combining consecutive updates,
    \code{Thinning} is usually applied, with the value equal to the
    number of objects of class \code{demonoid}. For more information on
    thinning, see the \code{Thin} function.}
}
\details{
  The purpose of the \code{Combine} function is to enable a user to
  combine objects of class \code{demonoid} for one of three
  reasons. First, parallel chains from \code{\link{LaplacesDemon.hpc}}
  may be combined after convergence is assessed with
  \code{\link{Gelman.Diagnostic}}. Second, consecutive updates of single
  chains from \code{\link{LaplacesDemon}} or parallel chains from
  \code{\link{LaplacesDemon.hpc}} may be combined when the computer has
  insufficient random-access memory (RAM) for the user to update once 
  with enough iterations. Third, consecutive single-chain or
  parallel-chain updates may be combined when it seems that the
  logarithm of the joint posterior distribution, \code{LP}, seems to be
  oscillating up and down, which is described in more detail below.

  The most common use regards the combination of parallel chains output
  from \code{\link{LaplacesDemon.hpc}}. Typically, a user with parallel
  chains examines them graphically with the
  \code{\link{caterpillar.plot}} and \code{plot} (actually,
  \code{\link{plot.demonoid}}) functions, and assesses convergence
  with the \code{\link{Gelman.Diagnostic}} function. Thereafter, the
  parallel chain output in the object of class \code{demonoid.hpc}
  should be combined into a single object of class \code{demonoid},
  before doing posterior predictive checks and making inferences. In
  this case, the \code{Thinning} argument usually is recommended to
  remain at its default.

  It is also common with a high-dimensional model (a model with a large
  number of parameters) to need more posterior samples than allowed by
  the random-access memory (RAM) of the computer. In this case, it is
  best to use the \code{\link{LaplacesDemon.RAM}} function to estimate
  the amount of RAM that a given model will require with a given number
  of iterations, and then update \code{\link{LaplacesDemon}} almost as
  much as RAM allows, and save the output object of class
  \code{demonoid}. Then, the user is advised to continue onward with a
  consecutive update (after using \code{\link{as.initial.values}} and
  anything else appropriate to prepare for the consecutive
  update). Suppose a user desires to update a gigantic model with
  thousands of parameters, and with the aid of
  \code{\link{LaplacesDemon.RAM}}, estimates that they can safely update
  only 100,000 iterations, and that 150,000 iterations would exceed RAM
  and crash the computer. The patient user can update several
  consecutive models, each with retaining only 1,000 thinned posterior
  samples, and combine them later with the \code{Combine} function, by
  placing multiple objects into a list, as described below. In this way,
  it is possible for a user to update models that otherwise far exceed
  computer RAM.

  Less commonly, multiple updates of single-chain objects should be
  combined into a single object of class \code{demonoid}. This is most
  useful in complicated models that are run for large numbers of
  iterations, where it may be suspected that stationarity has been
  achieved, but that thinning is insufficient, and the samples may be
  combined and thinned. If followed, then these suggestions may continue
  seemingly to infinity, and the unnormalized logarithm of the joint
  posterior density, \code{LP}, may seem to oscillate, sometimes
  improving and getting higher, and getting lower during other updates.
  For this purpose, the prior covariance matrix of the last model is
  retained (rather than combining them). This may be an unpleasant
  surprise for combining parallel updates, so be aware of it.

  In these cases, which usually involve complicated models with high
  autocorrelation in the chains, the user may opt to use parallel
  processing with the \code{\link{LaplacesDemon.hpc}} function, or may
  use the \code{\link{LaplacesDemon}} function as follows. The user
  should save (meaning, not overwrite) each object of class
  \code{demonoid}, place multiple objects into a list, and use the
  \code{Combine} function to combine these objects.

  For example, suppose a user names the object Fit, as in the
  \code{\link{LaplacesDemon}} example. Now, rather than overwriting
  object Fit, object Fit is renamed, after updating a million
  iterations, to Fit1. As suggested by \code{\link{Consort}}, another
  million iterations are used, but now to create object Fit2. Further
  suppose this user specified \code{Thinning=1000} in
  \code{\link{LaplacesDemon}}, meaning that the million iterations are
  thinned by 1,000, so only 1,000 iterations are retained in each
  object, Fit1 and Fit2. In this case, \code{Combine} combines the
  information in Fit1 and Fit2, and returns an object the user names
  Fit3. Fit3 has only 1,000 iterations, which is the result of appending
  the iterations in Fit1 and Fit2, and thinning by 2. If 2,000,000
  iterations were updated from the beginning, and were thinned by 2,000,
  then the same information exists now in Fit3. The \code{\link{Consort}}
  function can now be applied to Fit3, to see if stationarity is found.
  If not, then more objects of class \code{demonoid} can be collected and
  combined.
}
\value{
  This function returns an object of class \code{demonoid}. For more
  information on an object of class \code{demonoid}, see the
  \code{LaplacesDemon} function.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{\code{\link{caterpillar.plot}},
  \code{\link{Gelman.Diagnostic}},
  \code{\link{LaplacesDemon}}, 
  \code{\link{LaplacesDemon.hpc}}, and
  \code{\link{Thin}}.}
\keyword{MCMC}
\keyword{Multiple Chains}
\keyword{Parallel Chains}
\keyword{Utility}

\name{caterpillar.plot}
\alias{caterpillar.plot}
\title{Caterpillar Plot}
\description{
  A caterpillar plot is a horizontal plot of 3 quantiles of selected
  distributions. This may be used to produce a caterpillar plot of
  posterior samples (parameters and monitored variables) from an object
  either of class \code{demonoid}, \code{demonoid.hpc}, \code{iterquad},
  \code{laplace}, \code{pmc}, \code{vb}, or a matrix.
}
\usage{caterpillar.plot(x, Parms=NULL, Title=NULL)}
\arguments{
  \item{x}{
    This required argument is an object of class \code{demonoid},
    \code{demonoid.hpc}, \code{iterquad}, \code{laplace}, \code{pmc},
    \code{vb}, or a \eqn{S \times J}{S x J} matrix of \eqn{S} samples
    and \eqn{J} variables. For an object of class \code{demonoid}, the
    distributions of the stationary posterior summary (\code{Summary2})
    will be attempted first, and if missing, then the parameters of all
    posterior samples (\code{Summary1}) will be plotted. For an object
    of class \code{demonoid.hpc}, stationarity may differ by chain, so
    all posterior samples (\code{Summary1}) are used. For an object of
    class \code{laplace} or \code{vb}, the distributions in the
    posterior summary, \code{Summary}, are plotted according to the
    posterior draws, sampled with sampling importance resampling in the
    \code{\link{SIR}} function. When a generic matrix is supplied,
    unimodal 95\% HPD intervals are estimated with the
    \code{\link{p.interval}} function.}
  \item{Parms}{
    This argument accepts a vector of quoted strings to be matched for
    selecting parameters and monitored variables for plotting (though
    all parameters are selected when a generic matrix is supplied). This
    argument defaults to \code{NULL} and selects every parameter for
    plotting. Each quoted string is matched to one or more parameter
    names with the \code{grep} function. For example, if the user specifies
    \code{Parms=c("eta", "tau")}, and if the parameter names are
    beta[1], beta[2], eta[1], eta[2], and tau, then all parameters will
    be selected, because the string \code{eta} is within \code{beta}.
    Since \code{grep} is used, string matching uses regular
    expressions, so beware of meta-characters, though these are
    acceptable: ".", "[", and "]".}
  \item{Title}{
    This argument accepts a title for the plot.}
}
\details{
  Caterpillar plots are popular plots in Bayesian inference for
  summarizing the quantiles of posterior samples. A caterpillar plot is
  similar to a horizontal boxplot, though without quartiles, making it
  easier for the user to study more distributions in a single plot. The
  following quantiles are plotted as a line for each parameter: 0.025 and
  0.975, with the exception of a generic matrix, where unimodal 95\% HPD
  intervals are estimated (for more information, see
  \code{\link{p.interval}}). A vertical, gray line is included at zero.
  For all but class \code{demonoid.hpc}, the median appears as a black
  dot, and the quantile line is black. For class \code{demonoid.hpc}, the
  color of the median and quantile line differs by chain; the first
  chain is black and additional chains appear beneath.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{LaplacesDemon.hpc}},
  \code{\link{PMC}},
  \code{\link{p.interval}},
  \code{\link{SIR}}, and
  \code{\link{VariationalBayes}}.
}
\examples{#An example is provided in the LaplacesDemon function.}
\keyword{Plot}

\name{dist.Bernoulli}
\alias{dbern}
\alias{pbern}
\alias{qbern}
\alias{rbern}
\title{Bernoulli Distribution}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the Bernoulli distribution.
}
\usage{
dbern(x, prob, log=FALSE)
pbern(q, prob, lower.tail=TRUE, log.p=FALSE)
qbern(p, prob, lower.tail=TRUE, log.p=FALSE)
rbern(n, prob)
}
\arguments{
  \item{x, q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations. If \code{length(n) > 1},
    then the length is taken to be the number required.}
  \item{prob}{This is the probability of success on each trial.}
  \item{log, log.p}{Logical. if \code{TRUE}, probabilities \eqn{p} are
    given as \eqn{\log(p)}{log(p)}.}
  \item{lower.tail}{Logical. if \code{TRUE} (default), probabilities
    are \eqn{Pr[X \le x]}{Pr[X <= x]}, otherwise,
    \eqn{Pr[X > x]}{Pr[X > x]}.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density: \eqn{p(\theta) = {p}^{\theta}
    {(1-p)}^{1-\theta}}{p(theta) = p^theta (1-p)^(1-theta)}, \eqn{\theta = 0,1}{theta = 0,1}
  \item Inventor: Jacob Bernoulli
  \item Notation 1: \eqn{\theta \sim \mathcal{BERN}(p)}{theta ~ Bern(p)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{BERN}(\theta | p)}{p(theta) = Bern(theta | p)}
  \item Parameter 1: probability parameter \eqn{0 \le p \le 1}{0 <= p <=
  1}
  \item Mean: \eqn{E(\theta) = p}{E(theta) = p}
  \item Variance: \eqn{var(\theta) = \frac{p}{1-p}}{var(theta) = p / (1-p)}
  \item Mode: \eqn{mode(\theta) =}{mode(theta) =}
}

The Bernoulli distribution is a binomial distribution with
\eqn{n=1}{n=1}, and one instance of a Bernoulli distribution is called a
Bernoulli trial. One coin flip is a Bernoulli trial, for example. The
categorical distribution is the generalization of the Bernoulli
distribution for variables with more than two discrete values. The
beta distribution is the conjugate prior distribution of the Bernoulli
distribution. The geometric distribution is the number of Bernoulli
trials needed to get one success.
}
\value{
  \code{dbern} gives the density,
  \code{pbern} gives the distribution function,
  \code{qbern} gives the quantile function, and
  \code{rbern} generates random deviates.
}
\seealso{
  \code{\link{dbinom}}
}
\examples{
library(LaplacesDemon)
dbern(1, 0.7)
rbern(10, 0.5)
}
\keyword{Distribution}

\name{dist.Laplace.Precision}
\alias{dlaplacep}
\alias{plaplacep}
\alias{qlaplacep}
\alias{rlaplacep}
\title{Laplace Distribution: Precision Parameterization}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the univariate, symmetric, Laplace
  distribution with location parameter \eqn{\mu}{mu} and precision
  parameter \eqn{\tau}{tau}, which is the inverse of the usual scale
  parameter, \eqn{\lambda}{lambda}.
}
\usage{
dlaplacep(x, mu=0, tau=1, log=FALSE)
plaplacep(q, mu=0, tau=1)
qlaplacep(p, mu=0, tau=1)
rlaplacep(n, mu=0, tau=1)
}
\arguments{
  \item{x, q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{mu}{This is the location parameter \eqn{\mu}{mu}.}
  \item{tau}{This is the precision parameter \eqn{\tau}{tau}, which
    must be positive.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density: \eqn{p(\theta) = \frac{\tau}{2} \exp(-\tau |\theta-\mu|)}{p(theta) = (tau/2) * exp(-tau*abs(theta-mu))}
  \item Inventor: Pierre-Simon Laplace (1774)
  \item Notation 1: \eqn{\theta \sim \mathrm{Laplace}(\mu,\tau^{-1})}{theta ~
  Laplace(mu, tau^(-1))}
  \item Notation 2: \eqn{\theta \sim \mathcal{L}(\mu, \tau^{-1})}{theta ~
    L(mu, tau^(-1))}
  \item Notation 3: \eqn{p(\theta) =
    \mathrm{Laplace}(\mu,\tau^{-1})}{p(theta) = Laplace(mu, tau^(-1))}
  \item Notation 4: \eqn{p(\theta) = \mathcal{L}(\theta | \mu,
    \tau^{-1})}{p(theta) = L(theta | mu, tau(-1))}
  \item Parameter 1: location parameter \eqn{\mu}{mu}
  \item Parameter 2: precision parameter \eqn{\tau > 0}{tau > 0}
  \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}
  \item Variance: \eqn{var(\theta) = 2 \tau^{-2}}{var(theta) = 2*tau^{-2}}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The Laplace distribution is also called the double exponential 
distribution, because it looks like two exponential distributions back to
back with respect to location \eqn{\mu}{mu}. It is also called the
``First Law of Laplace'', just as the normal distribution is referred to
as the ``Second Law of Laplace''. The Laplace distribution is
symmetric with respect to \eqn{\mu}{mu}, though there are asymmetric
versions of the Laplace distribution. The PDF of the Laplace
distribution is reminiscent of the normal distribution; however,
whereas the normal distribution is expressed in terms of the squared
difference from the mean \eqn{\mu}{mu}, the Laplace density is
expressed in terms of the absolute difference from the mean,
\eqn{\mu}{mu}. Consequently, the Laplace distribution has fatter
tails than the normal distribution. It has been argued that the Laplace
distribution fits most things in nature better than the normal
distribution. Elsewhere, there are a large number of extensions to the
Laplace distribution, including asymmetric versions and
multivariate versions, among many more. These functions provide the
precision parameterization for convenience and familiarity in Bayesian
inference.
}
\value{
  \code{dlaplacep} gives the density,
  \code{plaplacep} gives the distribution function,
  \code{qlaplacep} gives the quantile function, and
  \code{rlaplacep} generates random deviates.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dalaplace}},
  \code{\link{dexp}},
  \code{\link{dlaplace}},
  \code{\link{dmvl}},
  \code{\link{dnorm}},
  \code{\link{dnormp}}, and
  \code{\link{dnormv}}.
}
\examples{
library(LaplacesDemon)
x <- dlaplacep(1,0,1)
x <- plaplacep(1,0,1)
x <- qlaplacep(0.5,0,1)
x <- rlaplacep(100,0,1)

#Plot Probability Functions
x <- seq(from=-5, to=5, by=0.1)
plot(x, dlaplacep(x,0,0.5), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dlaplacep(x,0,1), type="l", col="green")
lines(x, dlaplacep(x,0,2), type="l", col="blue")
legend(2, 0.9, expression(paste(mu==0, ", ", tau==0.5),
     paste(mu==0, ", ", tau==1), paste(mu==0, ", ", tau==2)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{plot.miss}
\alias{plot.miss}
\title{Plot samples from the output of MISS}
\description{
  This may be used to plot, or save plots of, samples in an object of
  class \code{miss}. Plots include a trace plot, density plot, and
  autocorrelation or ACF plot.
}
\usage{
\method{plot}{miss}(x, PDF=FALSE, \dots)
}
\arguments{
  \item{x}{
    This required argument is an object of class \code{miss}.}
  \item{PDF}{
    This logical argument indicates whether or not the user wants
    Laplace's Demon to save the plots as a .pdf file.}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  The plots are arranged in a \eqn{3 \times 3}{3 x 3} matrix. Each row
  represents the predictive distribution of a missing value. The
  left column displays trace plots, the middle column displays kernel
  density plots, and the right column displays autocorrelation (ACF)
  plots.

  Trace plots show the thinned history of the predictive distribution,
  with its value in the y-axis moving by iteration across the x-axis.
  Simulations of a predictive distribution with good properties do not
  suggest a trend upward or downward as it progresses across the x-axis
  (it should appear stationary), and it should mix well, meaning it
  should appear as though random samples are being taken each time from
  the same target distribution. Visual inspection of a trace plot cannot
  verify convergence, but apparent non-stationarity or poor mixing can
  certainly suggest non-convergence. A red, smoothed line also appears
  to aid visual inspection.

  Kernel density plots depict the marginal posterior distribution.
  There is no distributional assumption about this density.

  Autocorrelation plots show the autocorrelation or serial correlation
  between sampled values at nearby iterations. Samples with
  autocorrelation do not violate any assumption, but are inefficient
  because they reduce the effective sample size (\code{\link{ESS}}), and
  indicate that the chain is not mixing well, since each value is
  influenced by values that are previous and nearby. The x-axis
  indicates lags with respect to samples by iteration, and the y-axis
  represents autocorrelation. The ideal autocorrelation plot shows
  perfect correlation at zero lag, and quickly falls to zero
  autocorrelation for all other lags.
}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{MISS}}.}
\examples{### See the MISS function for an example.}
\keyword{Plot}

\name{data.demonsnacks}
\alias{demonsnacks}
\title{Demon Snacks Data Set}
\usage{data(demonsnacks)}
\description{
  Late one night, after witnessing Laplace's Demon in action, I followed
  him back to what seemed to be his lair. Minutes later, he left again.
  I snuck inside and saw something labeled 'Demon Snacks'. Hurriedly, I
  recorded the 39 items, each with a name and 10 nutritional attributes.
}
\format{
  This data frame contains 39 rows (with row names) and 10 columns. The
  following data dictionary describes each variable or column.
  \describe{
    \item{\code{Serving.Size}}{This is serving size in grams.}
    \item{\code{Calories}}{This is the number of calories.}
    \item{\code{Total.Fat}}{This is total fat in grams.}
    \item{\code{Saturated.Fat}}{This is saturated fat in grams.}
    \item{\code{Cholesterol}}{This is cholesterol in milligrams.}
    \item{\code{Sodium}}{This is sodium in milligrams.}
    \item{\code{Total.Carbohydrate}}{This is the total carbohydrates in grams.}
    \item{\code{Dietary.Fiber}}{This is dietary fiber in grams.}
    \item{\code{Sugars}}{This is sugar in grams.}
    \item{\code{Protein}}{This is protein in grams.}
    }
}
\source{This data was obtained from the lair of Laplace's Demon!}
\keyword{datasets}

\name{is.bayesian}
\alias{is.bayesian}
\title{Logical Check of a Bayesian Model}
\description{
  This function provides a logical test of whether or not a \code{Model}
  specification function is Bayesian.
}
\usage{
is.bayesian(Model, Initial.Values, Data)
}
\arguments{
  \item{Model}{This is a model specification function. For more
    information, see the \code{\link{LaplacesDemon}} function.}
  \item{Initial.Values}{This is a vector of initial values, or current
    parameter values. For more information, see the
    \code{\link{LaplacesDemon}} function.}
  \item{Data}{This is a list of data. For more information, see the
    \code{\link{LaplacesDemon}} function.}
}
\details{
  This function tests whether or not a model is Bayesian by comparing
  the first two returned arguments: the logarithm of the unnormalized
  joint posterior density (\code{LP}) and deviance (\code{Dev}). The
  deviance (D) is
  
  \deqn{\mathrm{D} = -2 \mathrm{LL}}{D = -2 LL},

  where LL is the log-likelihood. Consequently,

  \deqn{\mathrm{LL} = \mathrm{D} / -2}{LL = D / -2},

  and LP is the sum of LL and prior probability densities. If LP = LL,
  then the model is not Bayesian, because prior densities are absent.
}
\value{
  The \code{is.bayesian} function returns a logical value of \code{TRUE}
  when the model is Bayesian, and \code{FALSE} otherwise.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{LaplacesDemon}}.
}
\keyword{Utility}

\name{plot.bmk}
\alias{plot.bmk}
\title{Plot Hellinger Distances}
\description{
  This function plots Hellinger distances in an object of class \code{bmk}.
}
\usage{
\method{plot}{bmk}(x, col=colorRampPalette(c("black","red"))(100),
     title="", PDF=FALSE, Parms=NULL, \dots)
}
\arguments{
  \item{x}{This required argument is an object of class \code{bmk}. See
    the \code{\link{BMK.Diagnostic}} function for more information.}
  \item{col}{This argument specifies the colors of the cells. By
    default, the \code{colorRampPalette} function colors large Hellinger
    distances as \code{red}, small as \code{black}, and provides 100
    color gradations.}
  \item{title}{This argument specifies the title of the plot, and the
    default does not include a title.}
  \item{PDF}{Logical. When \code{TRUE}, the plot is saved as a .pdf
    file.}
  \item{Parms}{
    This argument accepts a vector of quoted strings to be matched for
    selecting parameters for plotting. This argument defaults to
    \code{NULL} and selects every parameter for plotting. Each quoted
    string is matched to one or more parameter names with the
    \code{grep} function. For example, if the user specifies
    \code{Parms=c("eta", "tau")}, and if the parameter names
    are beta[1], beta[2], eta[1], eta[2], and tau, then all parameters
    will be selected, because the string \code{eta} is within
    \code{beta}. Since \code{grep} is used, string matching uses
    regular expressions, so beware of meta-characters, though these are
    acceptable: ".", "[", and "]".}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  The \code{plot.bmk} function plots the Hellinger distances in an
  object of class \code{bmk}. This is useful for quickly finding
  portions of chains with large Hellinger distances, which indicates
  non-stationarity and non-convergence.
}
\seealso{\code{\link{BMK.Diagnostic}}}
\examples{
library(LaplacesDemon)
N <- 1000 #Number of posterior samples
J <- 10 #Number of parameters
Theta <- matrix(runif(N*J),N,J)
colnames(Theta) <- paste("beta[", 1:J, "]", sep="")
for (i in 2:N) {Theta[i,1] <- Theta[i-1,1] + rnorm(1)}
HD <- BMK.Diagnostic(Theta, batches=10)
plot(HD, title="Hellinger distance between batches")
}
\keyword{Plot}

\name{print.heidelberger}
\alias{print.heidelberger}
\title{Print an object of class \code{heidelberger} to the screen}
\description{
  This may be used to print the contents of an object of class
  \code{heidelberger} to the screen.
}
\usage{\method{print}{heidelberger}(x, digits=3, \dots)}
\arguments{
  \item{x}{An object of class \code{heidelberger} is required.}
  \item{digits}{This is the number of digits to print.}
  \item{\dots}{Additional arguments are unused.}
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{Heidelberger.Diagnostic}}.
}
\examples{### See the Heidelberger.Diagnostic function for an example.}
\keyword{print}

\name{plot.iterquad}
\alias{plot.iterquad}
\title{Plot the output of \code{\link{IterativeQuadrature}}}
\description{
  This may be used to plot, or save plots of, the iterated history of
  the parameters and, if posterior samples were taken, density plots of
  parameters and monitors in an object of class \code{iterquad}.
}
\usage{\method{plot}{iterquad}(x, Data, PDF=FALSE, Parms, \dots)}
\arguments{
  \item{x}{
    This required argument is an object of class \code{iterquad}.}
  \item{Data}{
    This required argument must receive the list of data that was
    supplied to \code{\link{IterativeQuadrature}} to create the object
    of class \code{iterquad}.}
  \item{PDF}{
    This logical argument indicates whether or not the user wants
    Laplace's Demon to save the plots as a .pdf file.}
  \item{Parms}{
    This argument accepts a vector of quoted strings to be matched for
    selecting parameters for plotting. This argument defaults to
    \code{NULL} and selects every parameter for plotting. Each quoted
    string is matched to one or more parameter names with the
    \code{grep} function. For example, if the user specifies
    \code{Parms=c("eta", "tau")}, and if the parameter names
    are beta[1], beta[2], eta[1], eta[2], and tau, then all parameters
    will be selected, because the string \code{eta} is within
    \code{beta}. Since \code{grep} is used, string matching uses
    regular expressions, so beware of meta-characters, though these are
    acceptable: ".", "[", and "]".}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  The plots are arranged in a \eqn{2 \times 2}{2 x 2} matrix. The
  purpose of the iterated history plots is to show how the value of each
  parameter and the deviance changed by iteration as the
  \code{\link{IterativeQuadrature}} attempted to fit a normal
  distribution to the marginal posterior distributions.

  The plots on the right show several densities, described below.

  \itemize{
    \item The transparent black density is the normalized quadrature
    weights for non-standard normal distributions, \eqn{M}. For
    multivariate quadrature, there are often multiple weights at a given
    node, and the average \eqn{M} is shown. Vertical black lines
    indicate the nodes.
    \item The transparent red density is the normalized LP weights. For
    multivariate quadrature, there are often multiple weights at a given
    node, and the average normalized and weighted LP is shown. Vertical
    red lines indicate the nodes.
    \item The transparent green density is the normal density implied
    given the conditional mean and conditional variance.
    \item The transparent blue density is the kernel density estimate
    of posterior samples generated with Sampling Importance Resampling.
    This is plotted only if the algorithm converged, and if
    \code{sir=TRUE}.
  }
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{\code{\link{IterativeQuadrature}}}
\examples{### See the IterativeQuadrature function for an example.}
\keyword{Plot}

\name{Heidelberger.Diagnostic}
\alias{Heidelberger.Diagnostic}
\title{Heidelberger and Welch's MCMC Convergence Diagnostic}
\description{
  Heidelberger and Welch (1981; 1983) proposed a two-part MCMC
  convergence diagnostic that calculates a test statistic (based on the
  Cramer-von Mises test statistic) to accept or reject the null
  hypothesis that the Markov chain is from a stationary distribution.
}
\usage{Heidelberger.Diagnostic(x, eps=0.1, pvalue=0.05)}
\arguments{
  \item{x}{This required argument accepts an object of class
    \code{demonoid}. It attempts to use \code{Posterior2}, but when this
    is missing it uses \code{Posterior1}.}
  \item{eps}{This argument specifies the target value for the ratio of
    halfwidth to sample mean.}
  \item{pvalue}{This argument specifies the level of statistical
    significance.}
}
\details{
  The Heidelberg and Welch MCMC convergence diagnostic consists of two
  parts:

  First Part
  1. Generate a chain of \eqn{N} iterations and define an alpha level.
  2. Calculate the test statistic on the whole chain. Accept or reject
  the null hypothesis that the chain is from a stationary distribution.
  3. If the null hypothesis is rejected, then discard the first 10\% of
  the chain. Calculate the test statistic and accept or reject the null
  hypothesis.
  4. If the null hypothesis is rejected, then discard the next 10\% and
  calculate the test statistic.
  5. Repeat until the null hypothesis is accepted or 50\% of the chain
  is discarded. If the test still rejects the null hypothesis, then
  the chain fails the test and needs to be run longer.

  Second Part
  If the chain passes the first part of the diagnostic, then the part of
  the chain that was not discarded from the first part is used to test
  the second part.

  The halfwidth test calculates half the width of the (1 - alpha)\%
  probability interval (credible interval) around the mean.

  If the ratio of the halfwidth and the mean is lower than \code{eps},
  then the chain passes the halfwidth test. Otherwise, the chain fails
  the halfwidth test and must be updated for more iterations until
  sufficient accuracy is obtained. In order to avoid problems caused by
  sequential testing, the test should not be repeated too frequently.
  Heidelberger and Welch (1981) suggest increasing the run length by a
  factor I > 1.5, each time, so that estimate has the same, reasonably
  large, proportion of new data.

  The Heidelberger and Welch MCMC convergence diagnostic conducts
  multiple hypothesis tests. The number of potentially wrong results
  increases with the number of non-independent hypothesis tests
  conducted.

  The \code{Heidelberger.Diagnostic} is a univariate diagnostic that is
  usually applied to each marginal posterior distribution. A
  multivariate form is not included. By chance alone due to multiple
  independent tests, 5\% of the marginal posterior distributions should
  appear non-stationary when stationarity exists. Assessing multivariate
  convergence is difficult.
}
\note{
  The \code{Heidelberger.Diagnostic} function was adapted from the
  \code{heidel.diag} function in the coda package.
}
\value{
  The \code{Heidelberger.Diagnostic} function returns an object of class
  \code{heidelberger}. This object is a \eqn{J \times 6}{J x 6}
  matrix, and it is intended to be summarized with the
  \code{\link{print.heidelberger}} function. Nonetheless, this object of
  class \code{heidelberger} has \eqn{J} rows, each of which corresponds
  to a Markov chain. The column names are \code{stest}, \code{start},
  \code{pvalue}, \code{htest}, \code{mean}, and \code{halfwidth}. The
  \code{stest} column indicates convergence with a one, and
  non-convergence with a zero, regarding the stationarity test. When
  non-convergence is indicated, the remaining columns have missing
  values. The \code{start} column indicates the starting iteration, and
  the \code{pvalue} column shows the p-value associated with the first
  test. The \code{htest} column indicates convergence for the halfwidth
  test. The \code{mean} and \code{halfwidth} columns report the mean and
  halfwidth.
}
\references{
Heidelberger, P. and Welch, P.D. (1981). "A Spectral Method for
Confidence Interval Generation and Run Length Control in Simulations".
\emph{Comm. ACM.}, 24, p. 233--245.

Heidelberger, P. and Welch, P.D. (1983). "Simulation Run Length Control
in the Presence of an Initial Transient". \emph{Opns Res.}, 31,
p. 1109--1144.

Schruben, L.W. (1982). "Detecting Initialization Bias in Simulation
Experiments". \emph{Opns. Res.}, 30, p. 569--590.
}
\seealso{
  \code{\link{burnin}},
  \code{\link{is.stationary}},
  \code{\link{LaplacesDemon}}, and
  \code{\link{print.heidelberger}}.
}
\examples{
#library(LaplacesDemon)
###After updating with LaplacesDemon, do:
#hd <- Heidelberger.Diagnostic(Fit)
#print(hd)
}
\keyword{Diagnostic}
\keyword{MCMC}

\name{p.interval}
\alias{p.interval}
\title{Probability Interval}
\description{
  This function returns one or more probability intervals of posterior
  samples.
}
\usage{
p.interval(obj, HPD=TRUE, MM=TRUE, prob=0.95, plot=FALSE, PDF=FALSE, \dots)
}
\arguments{
  \item{obj}{This can be either a vector or matrix of posterior samples,
    or an object of class \code{demonoid}, \code{iterquad},
    \code{laplace}, \code{pmc}, or \code{vb}. If it is an object of
    class \code{demonoid}, then it will use only stationary posterior
    samples and monitored target distributions (automatically discarding
    the burn-in; if stationarity does not exist, then it will use all
    samples).}
  \item{HPD}{Logical. This argument defaults to \code{TRUE}, in which
    case one or more Highest Posterior Density (HPD) intervals is
    returned. When \code{FALSE}, one or more quantile-based probability
    intervals is returned.}
  \item{MM}{Logical. This argument defaults to \code{TRUE}, in which
    case each column vector is checked for multimodality, and if found,
    the multimodal form of a Highest Posterior Density (HPD) interval is
    additionally estimated, even when \code{HPD=FALSE}.}
  \item{prob}{This is a numeric scalar in the interval (0,1) giving the
    target probability interval, and defaults to 0.95, representing a
    95\% probability interval. A 95\% probability interval, for example,
    is an interval that contains 95\% of a posterior probability
    distribution.}
  \item{plot}{Logical. When \code{plot=TRUE}, each kernel density is
    plotted and shaded gray, and the area under the curve within the
    probability interval is shaded black. If the kernel density is
    considered to be multimodal, then up to three intervals are shaded
    black. A vertical, red, dotted line is added at zero. The
    \code{plot} argument defaults to \code{FALSE}.}
  \item{PDF}{Logical. When \code{PDF=TRUE}, and only when
    \code{plot=TRUE}, plots are saved as a .pdf file in the working
    directory.}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  A probability interval, also called a credible interval or Bayesian
  confidence interval, is an interval in the domain of a posterior
  probability distribution. When generalized to multivariate forms, it
  is called a probability region (or credible region), though some
  sources refer to a probability region (or credible region) as the
  area within the probability interval. Bivariate probability regions
  may be plotted with the \code{\link{joint.pr.plot}} function.

  The \code{p.interval} function may return different probability
  intervals: a quantile-based probability interval, a unimodal
  Highest Posterior Density (HPD) interval, and multimodal HPD
  intervals. Another type of probability interval is the Lowest
  Posterior Loss (LPL) interval, which is calculated with the
  \code{\link{LPL.interval}} function.

  The quantile-based probability interval is used most commonly,
  possibly because it is simple, the fastest to calculate, invariant
  under transformation, and more closely resembles the frequentist
  confidence interval. The lower and upper bounds of the
  quantile-based probability interval are calculated with the
  \code{quantile} function. A 95\% quantile-based probability interval
  reports the values of the posterior probability distribution that
  indicate the 2.5\% and 97.5\% quantiles, which contain the central
  95\% of the distribution. The quantile-based probability interval is
  centered around the median and has equal-sized tails.
  
  The HPD (highest posterior density) interval is identical to
  the quantile-based probability interval when the posterior probability
  distribution is unimodal and symmetric. Otherwise, the HPD interval
  is the smallest interval, because it is estimated as the interval
  that contains the highest posterior density. Unlike the quantile-based
  probability interval, the HPD interval could be one-tailed or
  two-tailed, whichever is more appropriate. However, unlike the
  quantile-based interval, the HPD interval is not invariant to
  reparameterization (Bernardo, 2005).

  The unimodal HPD interval is estimated from the empirical CDF of the
  sample for each parameter (or deviance or monitored variable) as the
  shortest interval for which the difference in the ECDF values of the
  end-points is the user-specified probability width. This assumes the
  distribution is not severely multimodal.

  As an example, imagine an exponential posterior distribution. A
  quantile-based probability interval would report the highest density
  region near zero to be outside of its interval. In contrast, the
  unimodal HPD interval is recommended for such skewed posterior
  distributions.

  When \code{MM=TRUE}, the \code{\link{is.multimodal}} function is
  applied to each column vector after the unimodal interval (either
  quantile-based or HPD) is estimated. If multimodality is found, then
  multimodal HPD intervals are estimated with kernel density and
  printed to the screen as a character string. The original unimodal
  intervals are returned in the output matrix, because the matrix is
  constrained to have a uniform number of columns per row, and because
  multimodal HPD intervals may be disjoint.

  Disjoint multimodal HPD intervals have multiple intervals for one
  posterior probability distribution. An example may be when there is a
  bimodal, Gaussian distribution with means -10 and 10, variances of 1
  and 1, and a 95\% probability interval is specified. In this case,
  there is not enough density between these two distant modes to have
  only one probability interval.

  The user should also consider \code{\link{LPL.interval}}, since it is
  invariant to reparameterization like the quantile-based probability
  interval, but could be one- or two-tailed, whichever is more
  appropriate, like the HPD interval. A comparison of the quantile-based
  probability interval, HPD interval, and LPL interval is available
  here: \url{https://web.archive.org/web/20150214090353/http://www.bayesian-inference.com/credible}.
}
\value{
  A matrix is returned with rows corresponding to the parameters (or
  deviance or monitored variables), and columns \code{"Lower"} and
  \code{"Upper"}. The elements of the matrix are the unimodal
  probability intervals. The attribute \code{"Probability"} is the
  user-selected probability width. If \code{MM=TRUE} and multimodal
  posterior distributions are found, then multimodal HPD intervals are
  printed to the screen in a character string.
}
\references{
  Bernardo, J.M. (2005). "Intrinsic Credible Regions: An Objective
  Bayesian Approach to Interval Estimation". \emph{Sociedad de
  Estadistica e Investigacion Operativa}, 14(2), p. 317--384.
}
\author{Statisticat, LLC}
\seealso{
  \code{\link{is.multimodal}},
  \code{\link{IterativeQuadrature}},
  \code{\link{joint.pr.plot}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{LPL.interval}},
  \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
##First, update the model with the LaplacesDemon function.
##Then
#p.interval(Fit, HPD=TRUE, MM=TRUE, prob=0.95)
}
\keyword{Credible Interval}
\keyword{Credible Region}
\keyword{Credible Set}
\keyword{HPD}
\keyword{HPDI}
\keyword{Probability Interval}
\keyword{Probability Region}
\keyword{Probability Set}

\name{dist.Multivariate.Cauchy.Precision}
\alias{dmvcp}
\alias{rmvcp}
\title{Multivariate Cauchy Distribution: Precision Parameterization}
\description{
  These functions provide the density and random number generation for
  the multivariate Cauchy distribution. These functions use the
  precision parameterization.
}
\usage{
dmvcp(x, mu, Omega, log=FALSE)
rmvcp(n=1, mu, Omega)
}
\arguments{
  \item{x}{This is either a vector of length \eqn{k} or a matrix with 
       a number of columns, \eqn{k}, equal to the number of columns in
       precision matrix \eqn{\Omega}{Omega}.}
  \item{n}{This is the number of random draws.}
  \item{mu}{This is a numeric vector representing the location parameter,
       \eqn{\mu}{mu} (the mean vector), of the multivariate
       distribution. It must be of length \eqn{k}, as defined above.}
  \item{Omega}{This is a \eqn{k \times k}{k x k} positive-definite
       precision matrix \eqn{\Omega}{Omega}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
       density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Multivariate
  \item Density: \deqn{p(\theta) =
    \frac{\Gamma((1+k)/2)}{\Gamma(1/2)1^{k/2}\pi^{k/2}} |\Omega|^{1/2}
    (1 + (\theta-\mu)^T \Omega (\theta-\mu))^{-(1+k)/2}}{p(theta) =
    (Gamma((nu+k)/2) / (Gamma(1/2)*1^(k/2)*pi^(k/2))) * |Omega|^(1/2) *
    (1 + (theta-mu)^T Omega (theta-mu))^(-(1+k)/2)}
  \item Inventor: Unknown (to me, anyway)
  \item Notation 1: \eqn{\theta \sim \mathcal{MC}_k(\mu,
    \Omega^{-1})}{theta ~ MC[k](mu, Omega^(-1))}
  \item Notation 2: \eqn{p(\theta) = \mathcal{MC}_k(\theta | \mu,
    \Omega^{-1})}{p(theta) = MC[k](theta | mu, Omega^(-1))}
  \item Parameter 1: location vector \eqn{\mu}{mu}
  \item Parameter 2: positive-definite \eqn{k \times k}{k x k} precision
  matrix \eqn{\Omega}{Omega}
  \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}
  \item Variance: \eqn{var(\theta) = undefined}{var(theta) = undefined}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The multivariate Cauchy distribution is a multidimensional extension
of the one-dimensional or univariate Cauchy distribution. A random
vector is considered to be multivariate Cauchy-distributed if every
linear combination of its components has a univariate Cauchy
distribution. The multivariate Cauchy distribution is equivalent to a
multivariate t distribution with 1 degree of freedom.

The Cauchy distribution is known as a pathological distribution because
its mean and variance are undefined, and it does not satisfy the central
limit theorem.

It is usually parameterized with mean and a covariance matrix, or in
Bayesian inference, with mean and a precision matrix, where the
precision matrix is the matrix inverse of the covariance matrix. These
functions provide the precision parameterization for convenience and
familiarity. It is easier to calculate a multivariate Cauchy density
with the precision parameterization, because a matrix inversion can be
avoided.

This distribution has a mean parameter vector \eqn{\mu}{mu} of length
\eqn{k}, and a \eqn{k \times k}{k x k} precision matrix \eqn{\Omega}{Omega},
which must be positive-definite.
}
\value{
  \code{dmvcp} gives the density and 
  \code{rmvcp} generates random deviates.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dcauchy}},
  \code{\link{dmvc}},
  \code{\link{dmvt}},
  \code{\link{dmvtp}}, and
  \code{\link{dwishart}}.
}
\examples{
library(LaplacesDemon)
x <- seq(-2,4,length=21)
y <- 2*x+10
z <- x+cos(y) 
mu <- c(1,12,2)
Omega <- matrix(c(1,2,0,2,5,0.5,0,0.5,3), 3, 3)
f <- dmvcp(cbind(x,y,z), mu, Omega)

X <- rmvcp(1000, rep(0,2), diag(2))
X <- X[rowSums((X >= quantile(X, probs=0.025)) &
     (X <= quantile(X, probs=0.975)))==2,]
joint.density.plot(X[,1], X[,2], color=TRUE)
}
\keyword{Distribution}

\name{Gelfand.Diagnostic}
\alias{Gelfand.Diagnostic}
\title{Gelfand's Convergence Diagnostic}
\description{
  Gelfand et al. (1990) proposed a convergence diagnostic for Markov
  chains. The \code{Gelfand.Diagnostic} function is an interpretation of
  Gelfand's ``thick felt-tip pen'' MCMC convergence diagnostic. This
  diagnostic plots a series of kernel density plots at \eqn{k}
  intervals of cumulative samples. Given a vector of \eqn{S} samples
  from a marginal posterior distribution, \eqn{\theta}{theta}, multiple
  kernel density lines are plotted together, where each includes samples
  from a different interval. It is assumed that \code{\link{burnin}}
  iterations have been discarded.

  Gelfand et al. (1990) assert that convergence is violated when the
  plotted lines are farther apart than the width of a thick, felt-tip
  pen. This depends on the size of the plot, and, of course, the
  pen. The estimated width of a ``thick felt-tip pen'' is included as a
  black, vertical line. The pen in \code{Gelfand.Diagnostic} is included
  for historical reasons. This diagnostic requires numerous samples.
}
\usage{
Gelfand.Diagnostic(x, k=3, pen=FALSE)
}
\arguments{
  \item{x}{This required argument is a vector of marginal posterior
    samples, such as selected from the output of
    \code{\link{LaplacesDemon}}.}
  \item{k}{This argument specifies the number \eqn{k} of kernel
    density plots given cumulative intervals of samples. This argument
    defaults to \eqn{k=3}{k=3}.}
  \item{pen}{Logical. This argument defaults to \code{pen=FALSE}. When
    \code{pen=TRUE}, the thick felt-tip pen is included as a black,
    vertical line.}
}
\value{
  The \code{Gelfand.Diagnostic} returns a plot.
}
\references{
  Gelfand, A.E., Hills, S., Racine-Poon, A., and Smith,
  A.F.M. (1990). "Illustration of Bayesian Inference in Normal Data
  Models Using Gibbs Sampling". \emph{Journal of the American
  Statistical Association}, 85, p. 972--985.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{burnin}} and
  \code{\link{LaplacesDemon}}.
}
\examples{
library(LaplacesDemon)
x <- rnorm(1000)
Gelfand.Diagnostic(x)
}
\keyword{Diagnostic}
\keyword{MCMC}

\name{dist.Multivariate.Normal.Cholesky}
\alias{dmvnc}
\alias{rmvnc}
\title{Multivariate Normal Distribution: Cholesky Parameterization}
\description{
  These functions provide the density and random number generation for
  the multivariate normal distribution, given the Cholesky
  parameterization.
}
\usage{
dmvnc(x, mu, U, log=FALSE) 
rmvnc(n=1, mu, U)
}
\arguments{
  \item{x}{This is data or parameters in the form of a vector of length
       \eqn{k} or a matrix with \eqn{k} columns.}
  \item{n}{This is the number of random draws.}
  \item{mu}{This is mean vector \eqn{\mu}{mu} with length \eqn{k} or
       matrix with \eqn{k} columns.}
  \item{U}{This is the \eqn{k \times k}{k x k} upper-triangular matrix
    that is Cholesky factor \eqn{\textbf{U}}{U} of covariance matrix
    \eqn{\Sigma}{Sigma}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
       density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Multivariate
  \item Density: \eqn{p(\theta) = \frac{1}{(2\pi)^{k/2}|\Sigma|^{1/2}}
    \exp(-\frac{1}{2}(\theta - \mu)'\Sigma^{-1}(\theta - \mu))}{p(theta)
    = (1/((2*pi)^(k/2)*|Sigma|^(1/2))) *
    exp(-(1/2)*(theta-mu)'*Sigma^(-1)*(theta-mu))}
  \item Inventor: Unknown (to me, anyway)
  \item Notation 1: \eqn{\theta \sim \mathcal{MVN}(\mu, \Sigma)}{theta ~ MVN(mu, Sigma)}
  \item Notation 2: \eqn{\theta \sim \mathcal{N}_k(\mu, \Sigma)}{theta ~ N[k](mu, Sigma)}
  \item Notation 3: \eqn{p(\theta) = \mathcal{MVN}(\theta | \mu, \Sigma)}{p(theta) = MVN(theta | mu, Sigma)}
  \item Notation 4: \eqn{p(\theta) = \mathcal{N}_k(\theta | \mu, \Sigma)}{p(theta) = N[k](theta | mu, Sigma)}
  \item Parameter 1: location vector \eqn{\mu}{mu}
  \item Parameter 2: \eqn{k \times k}{k x k} positive-definite matrix \eqn{\Sigma}{Sigma}
  \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}
  \item Variance: \eqn{var(\theta) = \Sigma}{var(theta) = Sigma}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The multivariate normal distribution, or multivariate Gaussian
distribution, is a multidimensional extension of the one-dimensional
or univariate normal (or Gaussian) distribution. A random vector is
considered to be multivariate normally distributed if every linear
combination of its components has a univariate normal distribution.
This distribution has a mean parameter vector \eqn{\mu}{mu} of length
\eqn{k} and an upper-triangular \eqn{k \times k}{k x k} matrix that is
Cholesky factor \eqn{\textbf{U}}{U}, as per the \code{\link{chol}}
function for Cholesky decomposition.

In practice, \eqn{\textbf{U}}{U} is fully unconstrained for proposals
when its diagonal is log-transformed. The diagonal is exponentiated
after a proposal and before other calculations. Overall, the Cholesky
parameterization is faster than the traditional parameterization.
Compared with \code{dmvn}, \code{dmvnc} must additionally
matrix-multiply the Cholesky back to the covariance matrix, but it
does not have to check for or correct the covariance matrix to
positive-definiteness, which overall is slower. Compared with
\code{rmvn}, \code{rmvnc} is faster because the Cholesky decomposition
has already been performed.

For models where the dependent variable, Y, is specified to be
distributed multivariate normal given the model, the Mardia test (see
\code{\link{plot.demonoid.ppc}}, \code{\link{plot.laplace.ppc}}, or
\code{\link{plot.pmc.ppc}}) may be used to test the residuals.
}
\value{
  \code{dmvnc} gives the density and 
  \code{rmvnc} generates random deviates.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{chol}},
  \code{\link{dinvwishartc}},
  \code{\link{dmvn}},
  \code{\link{dmvnp}},
  \code{\link{dmvnpc}},
  \code{\link{dnorm}},
  \code{\link{dnormp}},
  \code{\link{dnormv}},
  \code{\link{plot.demonoid.ppc}},
  \code{\link{plot.laplace.ppc}}, and
  \code{\link{plot.pmc.ppc}}.
}
\examples{
library(LaplacesDemon)
Sigma <- diag(3)
U <- chol(Sigma)
x <- dmvnc(c(1,2,3), c(0,1,2), U)
X <- rmvnc(1000, c(0,1,2), U)
joint.density.plot(X[,1], X[,2], color=TRUE)
}
\keyword{Distribution}

\name{dist.Multivariate.Normal.Precision.Cholesky}
\alias{dmvnpc}
\alias{rmvnpc}
\title{Multivariate Normal Distribution: Precision-Cholesky Parameterization}
\description{
  These functions provide the density and random number generation for
  the multivariate normal distribution, given the precision-Cholesky
  parameterization.
}
\usage{
dmvnpc(x, mu, U, log=FALSE) 
rmvnpc(n=1, mu, U)
}
\arguments{
  \item{x}{This is data or parameters in the form of a vector of length
       \eqn{k} or a matrix with \eqn{k} columns.}
  \item{n}{This is the number of random draws.}
  \item{mu}{This is mean vector \eqn{\mu}{mu} with length \eqn{k} or
       matrix with \eqn{k} columns.}
  \item{U}{This is the \eqn{k \times k}{k x k} upper-triangular of the
    precision matrix that is Cholesky factor \eqn{\textbf{U}}{U} of
    precision matrix \eqn{\Omega}{Omega}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
       density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Multivariate
  \item Density: \eqn{p(\theta) = (2\pi)^{-p/2} |\Omega|^{1/2}
    \exp(-\frac{1}{2} (\theta-\mu)^T \Omega (\theta-\mu))}{p(theta) = (2*pi)^(-p/2) * |Omega|^(1/2) * exp(-(1/2)*(theta-mu)^t Omega (theta-mu))}
  \item Inventor: Unknown (to me, anyway)
  \item Notation 1: \eqn{\theta \sim \mathcal{MVN}(\mu, \Omega^{-1})}{theta ~ MVN(mu, Omega^(-1))}
  \item Notation 2: \eqn{\theta \sim \mathcal{N}_k(\mu, \Omega^{-1})}{theta ~ N[k](mu, Omega^(-1))}
  \item Notation 3: \eqn{p(\theta) = \mathcal{MVN}(\theta | \mu, \Omega^{-1})}{p(theta) = MVN(theta | mu, Omega^(-1))}
  \item Notation 4: \eqn{p(\theta) = \mathcal{N}_k(\theta | \mu, \Omega^{-1})}{p(theta) = N[k](theta | mu, Omega^(-1))}
  \item Parameter 1: location vector \eqn{\mu}{mu}
  \item Parameter 2: positive-definite \eqn{k \times k}{k x k} precision matrix \eqn{\Omega}{Omega}
  \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}
  \item Variance: \eqn{var(\theta) = \Omega^{-1}}{var(theta) = Omega^(-1)}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The multivariate normal distribution, or multivariate Gaussian
distribution, is a multidimensional extension of the one-dimensional
or univariate normal (or Gaussian) distribution. It is usually
parameterized with mean and a covariance matrix, or in Bayesian
inference, with mean and a precision matrix, where the precision matrix
is the matrix inverse of the covariance matrix. These functions
provide the precision-Cholesky parameterization for convenience and
familiarity. It is easier to calculate a multivariate normal density
with the precision parameterization, because a matrix inversion can be
avoided. The precision matrix is replaced with an upper-triangular
\eqn{k \times k}{k x k} matrix that is Cholesky factor
\eqn{\textbf{U}}{U}, as per the \code{\link{chol}} function for Cholesky
decomposition.

A random vector is considered to be multivariate normally distributed
if every linear combination of its components has a univariate normal
distribution. This distribution has a mean parameter vector
\eqn{\mu}{mu} of length \eqn{k} and a \eqn{k \times k}{k x k}
precision matrix \eqn{\Omega}{Omega}, which must be positive-definite.

In practice, \eqn{\textbf{U}}{U} is fully unconstrained for proposals
when its diagonal is log-transformed. The diagonal is exponentiated
after a proposal and before other calculations. Overall, Cholesky
parameterization is faster than the traditional parameterization.
Compared with \code{dmvnp}, \code{dmvnpc} must additionally
matrix-multiply the Cholesky back to the covariance matrix, but it
does not have to check for or correct the precision matrix to
positive-definiteness, which overall is slower. Compared with
\code{rmvnp}, \code{rmvnpc} is faster because the Cholesky decomposition
has already been performed.

For models where the dependent variable, Y, is specified to be
distributed multivariate normal given the model, the Mardia test (see
\code{\link{plot.demonoid.ppc}}, \code{\link{plot.laplace.ppc}}, or
\code{\link{plot.pmc.ppc}}) may be used to test the residuals.
}
\value{
  \code{dmvnpc} gives the density and 
  \code{rmvnpc} generates random deviates.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{chol}},
  \code{\link{dmvn}},
  \code{\link{dmvnc}},
  \code{\link{dmvnp}},
  \code{\link{dnorm}},
  \code{\link{dnormp}},
  \code{\link{dnormv}},
  \code{\link{dwishartc}},
  \code{\link{plot.demonoid.ppc}},
  \code{\link{plot.laplace.ppc}}, and
  \code{\link{plot.pmc.ppc}}.
}
\examples{
library(LaplacesDemon)
Omega <- diag(3)
U <- chol(Omega)
x <- dmvnpc(c(1,2,3), c(0,1,2), U)
X <- rmvnpc(1000, c(0,1,2), U)
joint.density.plot(X[,1], X[,2], color=TRUE)
}
\keyword{Distribution}

\name{GIV}
\alias{GIV}
\title{Generate Initial Values}
\description{
  The \code{GIV} function generates initial values for use with the
  \code{\link{IterativeQuadrature}}, \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, \code{\link{PMC}}, and
  \code{\link{VariationalBayes}} functions.
}
\usage{
GIV(Model, Data, n=1000, PGF=FALSE)
}
\arguments{
  \item{Model}{This required argument is a model specification
    function. For more information, see \code{\link{LaplacesDemon}}.}
  \item{Data}{This required argument is a list of data. For more
    information, see \code{\link{LaplacesDemon}}.}
  \item{n}{This is the number of attempts to generate acceptable
    initial values.}
  \item{PGF}{Logical. When \code{TRUE}, a Parameter-Generating Function
    (PGF) is required to be in \code{Data}, and \code{GIV} will generate
    initial values according to the user-specified PGF. This argument
    defaults to \code{FALSE}, in which case initial values are generated
    randomly without respect to a user-specified function.}
}
\details{
  Initial values are required for optimization or sampling algorithms. A
  user may specify initial values, or use the \code{GIV} function for
  random generation. Initial values determined by the user may fail to
  produce a finite posterior in complicated models, and the \code{GIV}
  function is here to help.

  \code{GIV} has several uses. First, the
  \code{\link{IterativeQuadrature}}, \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, and \code{\link{VariationalBayes}}
  functions use \code{GIV} internally if unacceptable initial values are
  discovered. Second, the user may use \code{GIV} when developing their
  model specification function, \code{Model}, to check for potential
  problems. Third, the user may prefer to randomly generate acceptable
  initial values. Lastly, \code{GIV} is recommended when running
  multiple or parallel chains with the \code{\link{LaplacesDemon.hpc}}
  function (such as for later use with the \code{Gelman.Diagnostic}) for
  dispersed starting locations. For dispersed starting locations,
  \code{GIV} should be run once for each parallel chain, and the results
  should be stored per row in a matrix of initial values. For more
  information, see the \code{LaplacesDemon.hpc} documentation for
  initial values.

  It is strongly recommended that the user specifies a
  Parameter-Generating Function (PGF), and includes this function in the
  list of data. Although the PGF may be specified according to the prior
  distributions (possibly considered as a Prior-Generating Function), it
  is often specified with a more restricted range. For example, if a
  user has a model with the following prior distributions

  \deqn{\beta_j \sim \mathcal{N}(0, 1000), j=1,\dots,5}{beta_j ~ N(0,
    1000), j=1,\dots,5}
  \deqn{\sigma \sim \mathcal{HC}(25)}{sigma ~ HC(25)}

  then the PGF, given the prior distributions, is

  \code{PGF <- function(Data) return(c(rnormv(5,0,1000),rhalfcauchy(1,25)))}

  However, the user may not want to begin with initial values that could
  be so far from zero (as determined by the variance of 1000), and may
  instead prefer

  \code{PGF <- function(Data) return(c(rnormv(5,0,10),rhalfcauchy(1,5)))}

  When \code{PGF=FALSE}, initial values are attempted to be constrained
  to the interval \eqn{[-100,100]}. This is done to prevent numeric
  overflows with parameters that are exponentiated within the model
  specification function. First, \code{GIV} passes the upper and lower
  bounds of this interval to the model, and any changed parameters are
  noted.

  At this point, it is hoped that a non-finite posterior is not
  found. If found, then the remainder of the process is random and
  without the previous bounds. This can be particularly problematic in
  the case of, say, initial values that are the elements of a matrix
  that must be positive-definite, especially with large matrices. If a
  random solution is not found, then \code{GIV} will fail.
  
  If the posterior is finite and \code{PGF=FALSE}, then initial values
  are randomly generated with a normal proposal and a small variance at
  the center of the returned range of each parameter. As \code{GIV}
  fails to find acceptable initial values, the algorithm iterates toward
  its maximum number of iterations, \code{n}. In each iteration, the
  variance increases for the proposal.
  
  Initial values are considered acceptable only when the first two
  returned components of \code{Model} (which are \code{LP} and
  \code{Dev}) are finite, and when initial values do not change through
  constraints, as returned in the fifth component of the list:
  \code{parm}.

  If \code{GIV} fails to return acceptable initial values, then it is
  best to study the model specification function. When the model is
  complicated, here is a suggestion. Remove the log-likelihood,
  \code{LL}, from the equation that calculates the logarithm of the
  unnormalized joint posterior density, \code{LP}. For example, convert
  \code{LP <- LL + beta.prior} to \code{LP <- beta.prior}. Now, maximize
  \code{LP}, which is merely the set of prior densities, with any
  optimization algorithm. Replace \code{LL}, and run the model with
  initial values that are in regions of high prior density (preferably
  with \code{PGF=TRUE}. If this fails, then the model specification
  should be studied closely, because a non-finite posterior should
  (especially) never be associated with regions of high prior density.
}
\value{
  The \code{GIV} function returns a vector equal in length to the
  number of parameters, and each element is an initial value for the
  associated parameter in \code{Data$parm.names}. When \code{GIV} fails
  to find acceptable initial values, each returned element is \code{NA}.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{as.initial.values}},
  \code{\link{Gelman.Diagnostic}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{LaplacesDemon.hpc}},
  \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
library(LaplacesDemon)

##############################  Demon Data  ###############################
data(demonsnacks)
y <- log(demonsnacks$Calories)
X <- cbind(1, as.matrix(log(demonsnacks[,c(1,4,10)]+1)))
J <- ncol(X)
for (j in 2:J) X[,j] <- CenterScale(X[,j])

#########################  Data List Preparation  #########################
mon.names <- c("LP","sigma")
parm.names <- as.parm.names(list(beta=rep(0,J), sigma=0))
pos.beta <- grep("beta", parm.names)
pos.sigma <- grep("sigma", parm.names)
PGF <- function(Data) {
     beta <- rnorm(Data$J)
     sigma <- runif(1)
     return(c(beta, sigma))
     }
MyData <- list(J=J, PGF=PGF, X=X, mon.names=mon.names,
     parm.names=parm.names, pos.beta=pos.beta, pos.sigma=pos.sigma, y=y)

##########################  Model Specification  ##########################
Model <- function(parm, Data)
     {
     ### Parameters
     beta <- parm[Data$pos.beta]
     sigma <- interval(parm[Data$pos.sigma], 1e-100, Inf)
     parm[Data$pos.sigma] <- sigma
     ### Log-Prior
     beta.prior <- sum(dnormv(beta, 0, 1000, log=TRUE))
     sigma.prior <- dhalfcauchy(sigma, 25, log=TRUE)
     ### Log-Likelihood
     mu <- tcrossprod(Data$X, t(beta))
     LL <- sum(dnorm(Data$y, mu, sigma, log=TRUE))
     ### Log-Posterior
     LP <- LL + beta.prior + sigma.prior
     Modelout <- list(LP=LP, Dev=-2*LL, Monitor=LP,
          yhat=rnorm(length(mu), mu, sigma), parm=parm)
     return(Modelout)
     }

########################  Generate Initial Values  ########################
Initial.Values <- GIV(Model, MyData, PGF=TRUE)
}
\keyword{Initial Values}
\keyword{Utility}

\name{dist.Skew.Discrete.Laplace}
\alias{dsdlaplace}
\alias{psdlaplace}
\alias{qsdlaplace}
\alias{rsdlaplace}
\title{Skew Discrete Laplace Distribution: Univariate}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the univariate, skew discrete
  Laplace distribution with parameters \eqn{p}{p} and \eqn{q}{q}.
}
\usage{
dsdlaplace(x, p, q, log=FALSE)
psdlaplace(x, p, q)
qsdlaplace(prob, p, q)
rsdlaplace(n, p, q)
}
\arguments{
  \item{x}{This is a vector of data.}
  \item{p}{This is a scalar or vector of parameter \eqn{p \in [0,1]}{p
      in [0,1]}.}
  \item{q}{This is a scalar or vector of parameter \eqn{q \in [0,1]}{q
      in [0,1]}.}
  \item{prob}{This is a probability scalar or vector.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Discrete Univariate
    \item Density 1: \eqn{p(\theta) = \frac{(1-p)(1-q)}{1-pq}p^\theta;
      \theta=0,1,2,3,\dots}{p(theta) = (1-p)(1-q) / (1-pq)p^theta,
      theta=0,1,2,3,...}
    \item Density 2: \eqn{p(\theta) = \frac{(1-p)(1-q)}{1-pq}q^{|\theta|};
      x=0,-1,-2,-3,\dots}{p(theta) = (1-p)(1-q) / (1-pq)q^(|theta|),;
      x=0,-1,-2,-3,...}
    \item Inventor: Kozubowski, T.J. and Inusah, S. (2006)
    \item Notation 1: \eqn{\theta \sim \mathcal{DL}(p, q)}{theta ~ DL(p,
      q)}
    \item Notation 2: \eqn{p(\theta) = \mathcal{DL}(\theta | p,
      q)}{p(theta) = DL(theta | p, q)}
    \item Parameter 1: \eqn{p \in [0,1]}{p in [0,1]}
    \item Parameter 2: \eqn{q \in [0,1]}{q in [0,1]}
    \item Mean 1: \eqn{E(\theta) =
      \frac{1}{1-p}-\frac{1}{1-q}=\frac{p}{1-p}-\frac{q}{1-q}}{E(theta)
      = (1 / (1-p)) - (1 / (1-q)) = (p / (1-p)) - (q / (1-q))}
    \item Mean 2: \eqn{E(|\theta|) =
      \frac{q(1-p)^2+p(1-q)^2}{(1-qp)(1-q)(1-p)}}{E(|theta|) =
      (q(1-p)^2+p(1-q)^2) / ((1-qp)(1-q)(1-p))}
    \item Variance: \eqn{var(\theta) =
      \frac{1}{(1-p)^2(1-q)^2}[\frac{q(1-p)^3(1+q)+p(1-q)^3(1+p)}{1-pq}-(p-q)^2]}{var(theta)
      = (1 / ((1-p)^2(1-q)^2))[(q(1-p)^3(1+q)+p(1-q)^3(1+p)) / (1-pq) -
      (p-q)^2]}
    \item Mode: 
}

This is a discrete form of the skew-Laplace distribution. The symmetric
discrete Laplace distribution occurs when \eqn{p=q}{p=q}. DL(p,0) is a
geometric distribution, and DL(0,q) is a geometric distribution of
non-positive integers. The distribution is degenerate when
DL(0,0). Since the geometric distribution is a discrete analog of the
exponential distribution, the distribution of the difference of two
geometric variables is a discrete Laplace distribution.


These functions are similar to those in the \code{DiscreteLaplace} package.
}
\value{
  \code{dslaplace} gives the density,
  \code{pslaplace} gives the distribution function,
  \code{qslaplace} gives the quantile function, and
  \code{rslaplace} generates random deviates.
}
\references{
  Kozubowski, T.J. and Inusah, S. (2006). "A Skew Laplace Distribution
  on Integers". \emph{AISM}, 58, p. 555--571.
}
\seealso{
  \code{\link{dalaplace}},
  \code{\link{dexp}},
  \code{\link{dlaplace}},
  \code{\link{dlaplacep}}, and
  \code{\link{dslaplace}}.
}
\examples{
library(LaplacesDemon)
x <- dsdlaplace(1,0.5,0.5)
x <- psdlaplace(1,0.5,0.5)
x <- qsdlaplace(0.5,0.5,0.5)
x <- rsdlaplace(5,0.5,0.5)

#Plot Probability Functions
     x <- c(-3:3)
     plot(x, dsdlaplace(x,0.5,0.5), ylim=c(0,0.6), type="l", main="Probability Function",
          ylab="density", col="red")
     lines(x, dsdlaplace(x,0.3,0.6), type="l", col="green")
     lines(x, dsdlaplace(x,0.9,0.1), type="l", col="blue")
     legend(-2.5, 0.5, expression(paste(p==0.5, ", ", q==0.5),
          paste(p==0.3, ", ", q==0.6),
          paste(p==0.9, ", ", q==0.1)),
          lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{CSF}
\alias{CSF}
\title{Cumulative Sample Function}
\description{
  The Cumulative Sample Function (CSF) is a visual MCMC diagnostic in
  which the user may select a measure (such as a variable, summary
  statistic, or other diagnostic), and observe a plot of how the measure
  changes over cumulative posterior samples from MCMC, such as the
  output of \code{\link{LaplacesDemon}}. This may be considered to be a
  generalized extension of the \code{cumuplot} in the coda package,
  which is a more restrictive form of the cusum diagnostic introduced by
  Yu and Myckland (1998).

  Yu and Myckland (1998) suggest that CSF plots should be examined after
  traditional trace plots seem convergent, and assert that faster mixing
  chains (which are more desirable) result in CSF plots that are more
  `hairy' (as opposed to smooth), though this is subjective and has been
  debated. The \code{LaplacesDemon} package neither supports nor
  contradicts the suggestion of mixing and `hairiness', but suggests
  that CSF plots may be used to provide additional information about a
  chain. For example, a user may decide on a practical
  \code{\link{burnin}} given when a conditional mean obtains a certain
  standard error.
}
\usage{
CSF(x, name, method="Quantiles", quantiles=c(0.025,0.500,0.975), output=FALSE)
}
\arguments{
  \item{x}{This is a vector of posterior samples from MCMC.}
  \item{name}{This is an optional name for vector \code{x}, and is input
    as a quoted string, such as \code{name="theta"}.}
  \item{method}{This is a measure that will be observed over the course
    of cumulative samples of \code{x}. It defaults to
    \code{method="Quantiles"}, and optional methods include: \code{"ESS"},
    \code{"Geweke.Diagnostic"}, \code{"HPD"}, \code{"is.stationary"},
    \code{"Kurtosis"}, \code{"MCSE"}, \code{"MCSE.bm"},
    \code{"MCSE.sv"}, \code{"Mean"}, \code{"Mode"}, \code{"N.Modes"},
    \code{"Precision"}, \code{"Quantiles"}, and \code{"Skewness"}.}
  \item{quantiles}{This optional argument applies only when
    \code{method="Quantiles"}, in which case this vector indicates the
    probabilities that will be observed. It defaults to the median and
    95\% probability interval bounds (see \code{\link{p.interval}} for
    more information).}
  \item{output}{Logical. If \code{output=TRUE}, then the results of the
    measure over the course of the cumulative samples will be output as
    an object, either a vector or matrix, depending on the \code{method}
  argument. The \code{output} argument defaults to \code{FALSE}.}
}
\details{
  When \code{method="ESS"}, the effective sample size (ESS) is observed
  as a function of the cumulative samples of \code{x}. For more
  information, see the \code{\link{ESS}} function.

  When \code{method="Geweke.Diagnostic"}, the Z-score output of the
  Geweke diagnostic is observed as a function of the cumulative samples
  of \code{x}. For more information, see the
  \code{\link{Geweke.Diagnostic}} function.

  When \code{method="HPD"}, the Highest Posterior Density (HPD) interval
  is observed as a function of the cumulative samples of \code{x}. For
  more information, see the \code{\link{p.interval}} function.

  When \code{method="is.stationary"}, stationarity is logically
  tested and the result is observed as a function of the cumulative
  samples of \code{x}. For more information, see the
  \code{\link{is.stationary}} function.

  When \code{method="Kurtosis"}, kurtosis is observed as a function of
  the cumulative samples of \code{x}.

  When \code{method="MCSE"}, the Monte Carlo Standard Error (MCSE)
  estimated with the \code{IMPS} method is observed as a function of
  the cumulative samples of \code{x}. For more information, see the
  \code{\link{MCSE}} function.

  When \code{method="MCSE.bm"}, the Monte Carlo Standard Error (MCSE)
  estimated with the \code{batch.means} method is observed as a
  function of the cumulative samples of \code{x}. For more information,
  see the \code{\link{MCSE}} function.

  When \code{method="MCSE.sv"}, the Monte Carlo Standard Error (MCSE)
  estimated with the \code{sample.variance} method is observed as a
  function of the cumulative samples of \code{x}. For more information,
  see the \code{\link{MCSE}} function.

  When \code{method="Mean"}, the mean is observed as a function of
  the cumulative samples of \code{x}.

  When \code{method="Mode"}, the estimated mode is observed as a
  function of the cumulative samples of \code{x}. For more information,
  see the \code{\link{Mode}} function.

  When \code{method="N.Modes"}, the estimated number of modes is
  observed as a function of the cumulative samples of \code{x}. For
  more information, see the \code{\link{Modes}} function.

  When \code{method="Precision"}, the precision (inverse variance) is
  observed as a function of the cumulative samples of \code{x}.

  When \code{method="Quantiles"}, the quantiles selected with the
  \code{quantiles} argument are observed as a function of the
  cumulative samples of \code{x}.

  When \code{method="Skewness"}, skewness is observed as a function of
  the cumulative samples of \code{x}.
}
\references{
  Yu, B. and Myckland, P. (1997). "Looking at Markov Samplers through
  Cusum Path Plots: A Simple Diagnostic Idea". \emph{Statistics and
  Computing}, 8(3), p. 275--286. 
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{burnin}},
  \code{\link{ESS}},
  \code{\link{Geweke.Diagnostic}},
  \code{\link{is.stationary}},
  \code{\link{LaplacesDemon}},
  \code{\link{MCSE}},
  \code{\link{Mode}},
  \code{\link{Modes}}, and
  \code{\link{p.interval}}.
}
\examples{
#Commented-out because of run-time for package builds
#library(LaplacesDemon)
#x <- rnorm(1000)
#CSF(x, method="ESS")
#CSF(x, method="Geweke.Diagnostic")
#CSF(x, method="HPD")
#CSF(x, method="is.stationary")
#CSF(x, method="Kurtosis")
#CSF(x, method="MCSE")
#CSF(x, method="MCSE.bm")
#CSF(x, method="MCSE.sv")
#CSF(x, method="Mean")
#CSF(x, method="Mode")
#CSF(x, method="N.Modes")
#CSF(x, method="Precision")
#CSF(x, method="Quantiles")
#CSF(x, method="Skewness")
}
\keyword{Diagnostic}
\keyword{MCMC}
\keyword{Plot}
\keyword{Utility}

\name{dist.Halft}
\alias{dhalft}
\alias{phalft}
\alias{qhalft}
\alias{rhalft}
\title{Half-t Distribution}
\description{
  These functions provide the density, distribution function, quantile function, and random generation for the half-t distribution.
}
\usage{
dhalft(x, scale=25, nu=1, log=FALSE)
phalft(q, scale=25, nu=1)
qhalft(p, scale=25, nu=1)
rhalft(n, scale=25, nu=1)
}
\arguments{
  \item{x,q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{scale}{This is the scale parameter \eqn{\alpha}{alpha}, which
    must be positive.}
  \item{nu}{This is the scalar degrees of freedom parameter, which is
    usually represented as \eqn{\nu}{nu}.}
  \item{log}{Logical. If \code{log=TRUE} then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density: \eqn{p(\theta) = (1 + \frac{1}{\nu} (\theta /
    \alpha)^2)^{-(\nu+1)/2}, \quad \theta \ge 0}{p(theta) = (1 +
    (1/nu)*(theta/alpha)^2)^(-(nu+1)/2), theta >= 0}
  \item Inventor: Derived from the Student t
  \item Notation 1: \eqn{\theta \sim \mathcal{HT}(\alpha, \nu)}{theta ~
    HT(alpha,nu)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{HT}(\theta | \alpha,
    \nu)}{p(theta) = HT(theta | alpha,nu)}
  \item Parameter 1: scale parameter \eqn{\alpha > 0}{alpha > 0}
  \item Parameter 2: degrees of freedom parameter \eqn{\nu}{nu}
  \item Mean: \eqn{E(\theta)}{E(theta)} = unknown
  \item Variance: \eqn{var(\theta)}{var(theta)} = unknown
  \item Mode: \eqn{mode(\theta) = 0}{mode(theta) = 0}
}

The half-t distribution is derived from the Student t distribution, and
is useful as a weakly informative prior distribution for a scale
parameter. It is more adaptable than the default recommended
half-Cauchy, though it may also be more difficult to estimate due to its
additional degrees of freedom parameter, \eqn{\nu}{nu}. When
\eqn{\nu=1}{nu=1}, the density is proportional to a proper half-Cauchy
distribution. When \eqn{\nu=-1}{nu=-1}, the density becomes an improper,
uniform prior distribution. For more information on propriety, see
\code{is.proper}.

Wand et al. (2011) demonstrated that the half-t distribution may be
represented as a scale mixture of inverse-gamma distributions. This
representation is useful for conjugacy.
}
\value{
  \code{dhalft} gives the density,
  \code{phalft} gives the distribution function,
  \code{qhalft} gives the quantile function, and
  \code{rhalft} generates random deviates.
}
\references{
  Wand, M.P., Ormerod, J.T., Padoan, S.A., and Fruhwirth, R. (2011).
  "Mean Field Variational Bayes for Elaborate Distributions".
  \emph{Bayesian Analysis}, 6: p. 847--900.
}
\seealso{
  \code{\link{dhalfcauchy}},
  \code{\link{dst}},
  \code{\link{dt}},
  \code{\link{dunif}}, and
  \code{\link{is.proper}}.
}
\examples{
library(LaplacesDemon)
x <- dhalft(1,25,1)
x <- phalft(1,25,1)
x <- qhalft(0.5,25,1)
x <- rhalft(10,25,1)

#Plot Probability Functions
x <- seq(from=0.1, to=20, by=0.1)
plot(x, dhalft(x,1,-1), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dhalft(x,1,0.5), type="l", col="green")
lines(x, dhalft(x,1,500), type="l", col="blue")
legend(2, 0.9, expression(paste(alpha==1, ", ", nu==-1),
     paste(alpha==1, ", ", nu==0.5), paste(alpha==1, ", ", nu==500)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{dist.Dirichlet}
\alias{ddirichlet}
\alias{rdirichlet}
\title{Dirichlet Distribution}
\description{
  This is the density function and random generation from the Dirichlet
  distribution.
}
\usage{
ddirichlet(x, alpha, log=FALSE)
rdirichlet(n, alpha)
}
\arguments{
  \item{x}{This is a vector containing a single deviate or matrix
    containing one random deviate per row. Each vector, or matrix row,
    must sum to 1.}
  \item{n}{This is the number of random deviates to generate.}
  \item{alpha}{This is a vector or matrix of shape parameters.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Multivariate
    \item Density: \deqn{p(\theta) = \frac{\gamma(\alpha_1 + \dots +
	\alpha_k)}{\gamma \alpha_1 \dots \gamma \alpha_k}
      \theta^{(\alpha[1]-1)}_1 \dots \theta^{(\alpha[k]-1)}_k, \quad
      \theta_1, \dots, \theta_k > 0, \quad \sum^k_{j=1} \theta_j = 1}{p(theta)
      = (gamma(alpha[1] + ... + alpha[k]) / 
          (gamma(alpha[1]) ... gamma(alpha[k]))) * 
          theta[1]^(alpha[1] - 1) ... theta[k]^(alpha[k] - 1)
          theta[1],...,theta[k] >= 0, 
          (the sum of j=1 to k of) theta[j] = 1}
    \item Inventor: Johann Peter Gustav Lejeune Dirichlet (1805-1859)
    \item Notation 1: \eqn{\theta \sim}{theta ~}
    Dirichlet(\eqn{\alpha_1,\dots,\alpha_k}{alpha[1],..., alpha[k]})
    \item Notation 2: \eqn{p(\theta) =}{p(theta) =} Dirichlet(\eqn{\theta
      | \alpha_1,\dots,\alpha_k}{theta | alpha[1],...,alpha[k]})
    \item Notation 3: \eqn{\theta \sim
      \mathcal{DIR}(\alpha_1,\dots,\alpha_k)}{theta ~ Dir(alpha[1],...,
  alpha[k])}
    \item Notation 4: \eqn{p(\theta) = \mathcal{DIR}(\theta |
      \alpha_1,\dots,\alpha_k)}{p(theta) = Dir(theta |
      alpha[1],...,alpha[k])}
    \item Parameter: 'prior sample sizes' \eqn{\alpha_j > 0, \alpha_0 =
      \sum^k_{j=1} \alpha_j}{alpha[j] > 0, alpha[0] = (the sum from j=1
      to k of) alpha[j]}
    \item Mean: \eqn{E(\theta_j) =
      \frac{\alpha_j}{\alpha_0}}{E(theta[j]) = alpha[j] / alpha[0]}
    \item Variance: \eqn{var(\theta_j) = \frac{\alpha_j (\alpha_0 -
	\alpha_j)}{\alpha^2_0 (\alpha_0 + 1)}}{var(theta[j]) = (alpha[j]
      * (alpha[0] - alpha[j])) / (alpha[0]^2 * (alpha[0]+ 1))}
    \item Covariance: \eqn{cov(\theta_i, \theta_j) = - \frac{\alpha_i
	\alpha_j}{\alpha^2_0 (\alpha_0 + 1)}}{cov(theta[i], theta[j]) =
      - ((alpha[i]*alpha[j]) / (alpha[0]^2 * (alpha[0] + 1)))}
    \item Mode: \eqn{mode(\theta_j) = \frac{\alpha_j - 1}{\alpha_0 -
	k}}{mode(theta[j]) = (alpha[j] - 1) / (alpha[0] - k)}
}

The Dirichlet distribution is the multivariate generalization of the
univariate beta distribution. Its probability density function returns
the belief that the probabilities of \eqn{k} rival events are
\eqn{\theta_j}{theta[j]} given that each event has been observed
\eqn{\alpha_j - 1}{alpha[j] - 1} times.

The Dirichlet distribution is commonly used as a prior distribution in
Bayesian inference. The Dirichlet distribution is the conjugate prior
distribution for the parameters of the categorical and multinomial
distributions.

A very common special case is the symmetric Dirichlet distribution,
where all of the elements in parameter vector \eqn{\alpha}{alpha} have
the same value. Symmetric Dirichlet distributions are often used as 
vague or weakly informative Dirichlet prior distributions, so that one
component is not favored over another. The single value that is entered
into all elements of \eqn{\alpha}{alpha} is called the concentration
parameter.
}
\value{
  \code{ddirichlet} gives the density and
  \code{rdirichlet} generates random deviates.
}
\seealso{
  \code{\link{dbeta}},
  \code{\link{dcat}},
  \code{\link{dmvpolya}},
  \code{\link{dmultinom}}, and
  \code{\link{TransitionMatrix}}.
}
\examples{
library(LaplacesDemon)
x <- ddirichlet(c(.1,.3,.6), c(1,1,1))
x <- rdirichlet(10, c(1,1,1))
}
\keyword{Distribution}

\name{SIR}
\alias{SIR}
\title{Sampling Importance Resampling}
\description{
  The \code{SIR} function performs Sampling Importance Resampling, also
  called Sequential Importance Resampling, and uses a multivariate normal
  proposal density.
}
\usage{
SIR(Model, Data, mu, Sigma, n=1000, CPUs=1, Type="PSOCK")
}
\arguments{
  \item{Model}{This is a model specification function. For more
    information, see \code{\link{LaplaceApproximation}}.}
  \item{Data}{This is a list of data. For more information, see
    \code{\link{LaplaceApproximation}}.}
  \item{mu}{This is a mean vector, \eqn{\mu}{mu}, for a multivariate
    normal distribution, and is usually the posterior means from an
    object of class \code{iterquad} (from
    \code{\link{IterativeQuadrature}}) or class \code{vb} (from
    \code{\link{VariationalBayes}}), or the posterior modes from an
    object of class \code{laplace} (from 
    \code{\link{LaplaceApproximation}}).}
  \item{Sigma}{This is a covariance matrix, \eqn{\Sigma}{Sigma}, for a
    multivariate normal distribution, and is usually the \code{Covar}
    component of an object of class \code{iterquad}, \code{laplace}, or
    \code{vb}.}
  \item{n}{This is the number of samples to be drawn from the posterior
    distribution.}
  \item{CPUs}{This argument accepts an integer that specifies the number
    of central processing units (CPUs) of the multicore computer or
    computer cluster. This argument defaults to \code{CPUs=1}, in which
    parallel processing does not occur.}
  \item{Type}{This argument specifies the type of parallel processing to
    perform, accepting either \code{Type="PSOCK"} or
    \code{Type="MPI"}.}
}
\details{
  Sampling Importance Resampling (SIR) was introduced in Gordon, et
  al. (1993), and is the original particle filtering algorithm (and this
  family of algorithms is also known as Sequential Monte Carlo). A
  distribution is approximated with importance weights, which are
  approximations to the relative posterior densities of the particles,
  and the sum of the weights is one. In this terminology, each sample in
  the distribution is a ``particle''. SIR is a sequential or recursive
  form of importance sampling. As in importance sampling, the
  expectation of a function can be approximated as a weighted
  average. The optimal proposal distribution is the target distribution.

  In the \code{LaplacesDemon} package, the main use of the \code{SIR}
  function is to produce posterior samples for iterative quadrature,
  Laplace Approximation, or Variational Bayes, and \code{SIR} is called
  behind-the-scenes by the \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}}, or \code{\link{VariationalBayes}}
  function.

  Iterative quadrature estimates the posterior mean and the associated
  covariance matrix. Assuming normality, this output characterizes the
  marginal posterior distributions. However, it is often useful to have
  posterior samples, in which case the \code{SIR} function is used to
  draw samples. The number of samples, \code{n}, should increase with
  the number and intercorrelations of the parameters. Otherwise,
  multimodal posterior distributions may occur.

  Laplace Approximation estimates the posterior mode and the associated
  covariance matrix. Assuming normality, this output characterizes the
  marginal posterior distributions. However, it is often useful to have
  posterior samples, in which case the \code{SIR} function is used to
  draw samples. The number of samples, \code{n}, should increase with
  the number and intercorrelations of the parameters. Otherwise,
  multimodal posterior distributions may occur.

  Variational Bayes estimates both the posterior mean and
  variance. Assuming normality, this output characterizes the marginal
  posterior distributions. However, it is often useful to have posterior
  samples, in which case the \code{SIR} function is used to draw
  samples. The number of samples, \code{n}, should increase with the
  number of intercorrelations of the parameters. Otherwise, multimodal
  posterior distributions may occur.

  SIR is also commonly used when considering a mild change in a prior
  distribution. For example, suppose a model was updated in
  \code{\link{LaplacesDemon}}, and it had a least-informative prior
  distribution, but the statistician would like to estimate the impact
  of changing to a weakly-informative prior distribution. The change is
  made in the model specification function, and the posterior means and
  covariance are supplied to the \code{SIR} function. The returned
  samples are estimates of the posterior, given the different prior
  distribution. This is akin to sensitivity analysis (see the
  \code{SensitivityAnalysis} function).

  In other contexts (for which this function is not designed), SIR is
  used with dynamic linear models (DLMs) and state-space models (SSMs)
  for state filtering.

  Parallel processing may be performed when the user specifies
  \code{CPUs} to be greater than one, implying that the specified number
  of CPUs exists and is available. Parallelization may be performed on a
  multicore computer or a computer cluster. Either a Simple Network of
  Workstations (SNOW) or Message Passing Interface (MPI) is used. With
  small data sets and few samples, parallel processing may be slower,
  due to computer network communication. With larger data sets and more
  samples, the user should experience a faster run-time.
  
  This function was adapted from the \code{sir} function in the
  \code{LearnBayes} package.
}
\value{
  The \code{SIR} function returns a matrix of samples drawn from the
  posterior distribution.
}
\references{
  Gordon, N.J., Salmond, D.J., and Smith, A.F.M. (1993). "Novel Approach
  to Nonlinear/Non-Gaussian Bayesian State Estimation". \emph{IEEE
  Proceedings F on Radar and Signal Processing}, 140(2), p. 107--113.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dmvn}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{SensitivityAnalysis}}, and
  \code{\link{VariationalBayes}}.
}
\keyword{High Performance Computing}
\keyword{Sampling Importance Resampling}
\keyword{Sequential Importance Resampling}
\keyword{Utility}

\name{dist.Log.Laplace}
\alias{dllaplace}
\alias{pllaplace}
\alias{qllaplace}
\alias{rllaplace}
\title{Log-Laplace Distribution: Univariate Symmetric}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the univariate, symmetric,
  log-Laplace distribution with location parameter \code{location} and
  scale parameter \code{scale}.
}
\usage{
dllaplace(x, location=0, scale=1, log=FALSE)
pllaplace(q, location=0, scale=1)
qllaplace(p, location=0, scale=1)
rllaplace(n, location=0, scale=1)
}
\arguments{
  \item{x, q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{location}{This is the location parameter \eqn{\mu}{mu}.}
  \item{scale}{This is the scale parameter \eqn{\lambda}{lambda}, which
    must be positive.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density 1: \eqn{p(\theta) =
    \frac{(\sqrt{2}/\lambda)^2}{2(\sqrt{2}/\lambda)}
    \exp(-(\sqrt{2}/\lambda)(\theta - \mu)), \theta \ge
    \exp(\mu)}{p(theta) = ((sqrt(2)/lambda)^2 / (2*(sqrt(2)/lambda))) *
    exp(-(sqrt(2)/lambda)*(theta - mu)), theta >= exp(mu)}
  \item Density 2: \eqn{p(\theta) =
    \frac{(\sqrt{2}/\lambda)^2}{2(\sqrt{2}/\lambda)}
    \exp((\sqrt{2}/\lambda)(\theta - \mu)), \theta < \exp(\mu)}{p(theta)
    = ((sqrt(2)/lambda)^2 / (2*(sqrt(2)/lambda))) *
    exp((sqrt(2)/lambda)*(theta - mu)),theta < exp(mu)}
  \item Inventor: Pierre-Simon Laplace
  \item Notation 1: \eqn{\theta \sim \mathcal{LL}(\mu, \lambda)}{theta ~
    LL(mu, lambda)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{LL}(\theta | \mu,
    \lambda)}{p(theta) = LL(theta | mu, lambda)}
  \item Parameter 1: location parameter \eqn{\mu}{mu}
  \item Parameter 2: scale parameter \eqn{\lambda > 0}{lambda > 0}
  \item Mean: \eqn{E(\theta) = }{E(theta) = }
  \item Variance: \eqn{var(\theta) = }{var(theta) = }
  \item Mode: \eqn{mode(\theta) = }{mode(theta) = }
}

The univariate, symmetric log-Laplace distribution is derived from the
Laplace distribution. Multivariate and asymmetric versions also exist.

These functions are similar to those in the \code{VGAM} package.
}
\value{
  \code{dllaplace} gives the density,
  \code{pllaplace} gives the distribution function,
  \code{qllaplace} gives the quantile function, and
  \code{rllaplace} generates random deviates.
}
\references{
  Kozubowski, T. J. and Podgorski, K. (2003). "Log-Laplace Distributions".
  \emph{International Mathematical Journal}, 3, p. 467--495.
}
\seealso{
  \code{\link{dalaplace}},
  \code{\link{dallaplace}},
  \code{\link{dexp}},
  \code{\link{dlaplace}},
  \code{\link{dlaplacep}},
  \code{\link{dmvl}},
  \code{\link{dnorm}},
  \code{\link{dnormp}}, and
  \code{\link{dnormv}}.
}
\examples{
library(LaplacesDemon)
x <- dllaplace(1,0,1)
x <- pllaplace(1,0,1)
x <- qllaplace(0.5,0,1)
x <- rllaplace(100,0,1)

#Plot Probability Functions
x <- seq(from=0.1, to=20, by=0.1)
plot(x, dllaplace(x,0,0.1), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dllaplace(x,0,0.5), type="l", col="green")
lines(x, dllaplace(x,0,1.5), type="l", col="blue")
legend(2, 0.9, expression(paste(mu==0, ", ", lambda==0.1),
     paste(mu==0, ", ", lambda==0.5), paste(mu==0, ", ", lambda==1.5)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{dist.Skew.Laplace}
\alias{dslaplace}
\alias{pslaplace}
\alias{qslaplace}
\alias{rslaplace}
\title{Skew-Laplace Distribution: Univariate}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the univariate, skew-Laplace
  distribution with location parameter \eqn{\mu}{mu}, and two mixture
  parameters: \eqn{\alpha}{alpha} and \eqn{\beta}{beta}.
}
\usage{
dslaplace(x, mu, alpha, beta, log=FALSE)
pslaplace(q, mu, alpha, beta)
qslaplace(p, mu, alpha, beta)
rslaplace(n, mu, alpha, beta)
}
\arguments{
  \item{x, q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{mu}{This is the location parameter \eqn{\mu}{mu}.}
  \item{alpha}{This is a mixture parameter \eqn{\alpha}{alpha}, which
    must be positive.}
  \item{beta}{This is a mixture parameter \eqn{\beta}{beta}, which
    must be positive.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density 1: \eqn{p(\theta) = \frac{1}{\alpha + \beta}
    \exp(\frac{\theta - \mu}{\alpha}), \theta \le \mu}{p(theta) =
    (1/(alpha+beta)) exp((theta-mu)/alpha), theta <= mu}
  \item Density 2: \eqn{p(\theta) = \frac{1}{\alpha + \beta}
    \exp(\frac{\mu - \theta}{\beta}), \theta > \mu}{p(theta) =
    (1/(alpha+beta)) exp((mu-theta)/beta), theta > mu}
  \item Inventor: Fieller, et al. (1992)
  \item Notation 1: \eqn{\theta \sim \mathcal{SL}(\mu, \alpha, \beta)}{theta ~
    SL(mu, alpha, beta)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{SL}(\theta | \mu,
    \alpha, \beta)}{p(theta) = SL(theta | mu, alpha, beta)}
  \item Parameter 1: location parameter \eqn{\mu}{mu}
  \item Parameter 2: mixture parameter \eqn{\alpha > 0}{alpha > 0}
  \item Parameter 3: mixture parameter \eqn{\beta > 0}{beta > 0}
  \item Mean: \eqn{E(\theta) = \mu + \beta - \alpha}{E(theta) = mu +
    beta - alpha}
  \item Variance: \eqn{var(\theta) = \alpha^2 + \beta^2}{var(theta) =
    alpha^2 + beta^2}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

This is the three-parameter general skew-Laplace distribution, which is
an extension of the two-parameter central skew-Laplace distribution. The
general form allows the mode to be shifted along the real line with
parameter \eqn{\mu}{mu}. In contrast, the central skew-Laplace has mode
zero, and may be reproduced here by setting \eqn{\mu=0}{mu=0}.

The general skew-Laplace distribution is a mixture of a negative
exponential distribution with mean \eqn{\beta}{beta}, and the negative
of an exponential distribution with mean \eqn{\alpha}{alpha}. The
weights of the positive and negative components are proportional to
their means. The distribution is symmetric when
\eqn{\alpha=\beta}{alpha=beta}, in which case the mean is \eqn{\mu}{mu}.

These functions are similar to those in the \code{HyperbolicDist} package.
}
\value{
  \code{dslaplace} gives the density,
  \code{pslaplace} gives the distribution function,
  \code{qslaplace} gives the quantile function, and
  \code{rslaplace} generates random deviates.
}
\references{
  Fieller, N.J., Flenley, E.C., and Olbricht, W. (1992). "Statistics of
  Particle Size Data". \emph{Applied Statistics}, 41, p. 127--146.
}
\seealso{
  \code{\link{dalaplace}},
  \code{\link{dexp}},
  \code{\link{dlaplace}}, 
  \code{\link{dlaplacep}}, and
  \code{\link{dsdlaplace}}.
}
\examples{
library(LaplacesDemon)
x <- dslaplace(1,0,1,1)
x <- pslaplace(1,0,1,1)
x <- qslaplace(0.5,0,1,1)
x <- rslaplace(100,0,1,1)

#Plot Probability Functions
x <- seq(from=0.1, to=3, by=0.01)
plot(x, dslaplace(x,0,1,1), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dslaplace(x,0,0.5,2), type="l", col="green")
lines(x, dslaplace(x,0,2,0.5), type="l", col="blue")
legend(1.5, 0.9, expression(paste(mu==0, ", ", alpha==1, ", ", beta==1),
     paste(mu==0, ", ", alpha==0.5, ", ", beta==2),
     paste(mu==0, ", ", alpha==2, ", ", beta==0.5)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{dist.Normal.Inverse.Wishart}
\alias{dnorminvwishart}
\alias{rnorminvwishart}
\title{Normal-Inverse-Wishart Distribution}
\description{
  These functions provide the density and random number generation for
  the normal-inverse-Wishart distribution.
}
\usage{
dnorminvwishart(mu, mu0, lambda, Sigma, S, nu, log=FALSE) 
rnorminvwishart(n=1, mu0, lambda, S, nu)
}
\arguments{
  \item{mu}{This is data or parameters in the form of a vector of length
    \eqn{k} or a matrix with \eqn{k} columns.}
  \item{mu0}{This is mean vector \eqn{\mu_0}{mu[0]} with length \eqn{k} or
    matrix with \eqn{k} columns.}
  \item{lambda}{This is a positive-only scalar.}
  \item{n}{This is the number of random draws.}
  \item{nu}{This is the scalar degrees of freedom \eqn{\nu}{nu}.}
  \item{Sigma}{This is a \eqn{k \times k}{k x k} covariance matrix
    \eqn{\Sigma}{Sigma}.}
  \item{S}{This is the symmetric, positive-semidefinite, \eqn{k \times
      k}{k x k} scale matrix \eqn{\textbf{S}}{S}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
       density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Multivariate
    \item Density: \eqn{p(\mu, \Sigma) = \mathcal{N}(\mu | \mu_0,
      \frac{1}{\lambda}\Sigma) \mathcal{W}^{-1}(\Sigma | \nu,
      \textbf{S})}{p(mu, Sigma) = N(mu | mu[0], (1/lambda) Sigma)
      W^(-1)(Sigma | nu, S)}
  \item Inventors: Unknown
  \item Notation 1: \eqn{(\mu, \Sigma) \sim \mathcal{NIW}(\mu_0, \lambda,
    \textbf{S}, \nu)}{(mu, Sigmaa) ~ NIW(mu[0], lambda, S, nu)}
  \item Notation 2: \eqn{p(\mu, \Sigma) = \mathcal{NIW}(\mu, \Sigma |
    \mu_0, \lambda, \textbf{S}, \nu)}{p(mu, Sigma) = NIW(mu, Sigma |
    mu[0], lambda, S, nu)}
  \item Parameter 1: location vector \eqn{\mu_0}{mu[0]}
  \item Parameter 2: \eqn{\lambda > 0}{lambda > 0}
  \item Parameter 3: symmetric, positive-semidefinite
    \eqn{k \times k}{k x k} scale matrix \eqn{\textbf{S}}{S}
  \item Parameter 4: degrees of freedom \eqn{\nu \ge k}{nu >= k}
  \item Mean: Unknown
  \item Variance: Unknown
  \item Mode: Unknown
}

The normal-inverse-Wishart distribution, or Gaussian-inverse-Wishart
distribution, is a multivariate four-parameter continuous probability
distribution. It is the conjugate prior of a multivariate normal
distribution with unknown mean and covariance matrix.
}
\value{
  \code{dnorminvwishart} gives the density and 
  \code{rnorminvwishart} generates random deviates and returns a list
    with two components.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dmvn}} and
  \code{\link{dinvwishart}}.
}
\examples{
library(LaplacesDemon)
K <- 3
mu <- rnorm(K)
mu0 <- rnorm(K)
nu <- K + 1
S <- diag(K)
lambda <- runif(1) #Real scalar
Sigma <- as.positive.definite(matrix(rnorm(K^2),K,K))
x <- dnorminvwishart(mu, mu0, lambda, Sigma, S, nu, log=TRUE)
out <- rnorminvwishart(n=10, mu0, lambda, S, nu)
joint.density.plot(out$mu[,1], out$mu[,2], color=TRUE)
}
\keyword{Distribution}

\name{plot.pmc}
\alias{plot.pmc}
\title{Plot samples from the output of PMC}
\description{
  This may be used to plot, or save plots of, samples in an object of
  class \code{pmc}. Plots include a trace plot and density plot for
  parameters, a density plot for deviance and monitored variables, and
  convergence plots.
}
\usage{
\method{plot}{pmc}(x, BurnIn=0, Data, PDF=FALSE, Parms, \dots)
}
\arguments{
  \item{x}{This required argument is an object of class \code{pmc}.}
  \item{BurnIn}{This argument requires zero or a positive integer that
    indicates the number of iterations to discard as burn-in for the
    purposes of plotting.}
  \item{Data}{This required argument must receive the list of data that
    was supplied to \code{\link{PMC}} to create the object of class
    \code{pmc}.}
  \item{PDF}{This logical argument indicates whether or not the user
    wants Laplace's Demon to save the plots as a .pdf file.}
  \item{Parms}{This argument accepts a vector of quoted strings to be
    matched for selecting parameters for plotting. This argument
    defaults to \code{NULL} and selects every parameter for
    plotting. Each quoted string is matched to one or more parameter
    names with the \code{grep} function. For example, if the user
    specifies \code{Parms=c("eta", "tau")}, and if the parameter names
    are beta[1], beta[2], eta[1], eta[2], and tau, then all parameters
    will be selected, because the string \code{eta} is within
    \code{beta}. Since \code{grep} is used, string matching uses
    regular expressions, so beware of meta-characters, though these are
    acceptable: ".", "[", and "]".}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  The plots are arranged in a \eqn{2 \times 2}{2 x 2} matrix. Each row
  represents a parameter, the deviance, or a monitored variable. For
  parameters, the left column displays trace plots and the right column
  displays kernel density plots.

  Trace plots show the history of the distribution of independent
  importance samples. When multiple mixture components are used, each
  mixture component has a different color. These plots are unavailable
  for the deviance and monitored variables.
  
  Kernel density plots depict the marginal posterior distribution.
  Although there is no distributional assumption about this density,
  kernel density estimation uses Gaussian basis functions.

  Following these plots are three plots for convergence. First, ESSN
  (red) and perplexity (black) are plotted by iteration. Convergence
  occurs when both of these seem to stabilize, and higher is
  better. The second plot shows the distribution of the normalized
  importance weights by iteration. The third plot appears only when
  multiple mixture components are used. The third plot displays the
  probabilities of each mixture component by iteration. Although the
  last two plots are not formally convergence plots, they are provided
  so the user can verify the distribution of importance weights and the
  mixture probabilities have become stable.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{ESS}} and
  \code{\link{PMC}}.}
\examples{### See the PMC function for an example.}
\keyword{Plot}

\name{dist.Normal.Precision}
\alias{dnormp}
\alias{pnormp}
\alias{qnormp}
\alias{rnormp}
\title{Normal Distribution: Precision Parameterization}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the univariate normal distribution
  with mean \eqn{\mu}{mu} and precision \eqn{\tau}{tau}.
}
\usage{
dnormp(x, mean=0, prec=1, log=FALSE)
pnormp(q, mean=0, prec=1, lower.tail=TRUE, log.p=FALSE)
qnormp(p, mean=0, prec=1, lower.tail=TRUE, log.p=FALSE)
rnormp(n, mean=0, prec=1)
}
\arguments{
  \item{x, q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{mean}{This is the mean parameter \eqn{\mu}{mu}.}
  \item{prec}{This is the precision parameter \eqn{\tau}{tau}, which
    must be positive.}
  \item{log, log.p}{Logical. If \code{TRUE}, then probabilities
    \eqn{p} are given as \eqn{\log(p)}{log(p)}.}
  \item{lower.tail}{Logical. If \code{TRUE} (default), then probabilities
    are \eqn{Pr[X \le x]}{Pr[X <= x]}, otherwise,
    \eqn{Pr[X > x]}{Pr[X > x]}.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density: \eqn{p(\theta) = \sqrt{\frac{\tau}{2\pi}}
    \exp(-\frac{\tau}{2} (\theta-\mu)^2)}{p(theta) = sqrt(tau/(2*pi)) * exp(-(tau/2)*(x-mu)^2)}
  \item Inventor: Carl Friedrich Gauss or Abraham De Moivre
  \item Notation 1: \eqn{\theta \sim \mathcal{N}(\mu, \tau^{-1})}{theta ~ N(mu, tau^(-1))}
  \item Notation 2: \eqn{p(\theta) = \mathcal{N}(\theta | \mu,
    \tau^{-1})}{p(theta) = N(theta | mu, tau^(-1))}
  \item Parameter 1: mean parameter \eqn{\mu}{mu}
  \item Parameter 2: precision parameter \eqn{\tau > 0}{tau > 0}
  \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}
  \item Variance: \eqn{var(\theta) = \tau^{-1}}{var(theta) = tau^(-1)}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The normal distribution, also called the Gaussian distribution and the
Second Law of Laplace, is usually parameterized with mean and variance,
or in Bayesian inference, with mean and precision, where precision is
the inverse of the variance. In contrast, \code{Base R} parameterizes
the normal distribution with the mean and standard deviation. These
functions provide the precision parameterization for convenience and
familiarity.

Some authors attribute credit for the normal distribution to Abraham
de Moivre in 1738. In 1809, Carl Friedrich Gauss published his
monograph ``Theoria motus corporum coelestium in sectionibus conicis
solem ambientium'', in which he introduced the method of least squares,
method of maximum likelihood, and normal distribution, among many other
innovations.

Gauss, himself, characterized this distribution according to mean and
precision, though his definition of precision differed from the modern
one. The modern Bayesian use of precision \eqn{\tau}{tau} developed
because it was more straightforward to estimate \eqn{\tau}{tau} with a
gamma distribution as a conjugate prior, than to estimate
\eqn{\sigma^2}{sigma^2} with an inverse-gamma distribution as a
conjugate prior.

Although the normal distribution is very common, it often does not fit
data as well as more robust alternatives with fatter tails, such as the
Laplace or Student t distribution.

A flat distribution is obtained in the limit as
\eqn{\tau \rightarrow 0}{tau -> 0}.

For models where the dependent variable, y, is specified to be
normally distributed given the model, the Jarque-Bera test (see
\code{\link{plot.demonoid.ppc}} or \code{\link{plot.laplace.ppc}}) may
be used to test the residuals.

These functions are similar to those in \code{base R}.
}
\value{
  \code{dnormp} gives the density,
  \code{pnormp} gives the distribution function,
  \code{qnormp} gives the quantile function, and
  \code{rnormp} generates random deviates.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dlaplace}},
  \code{\link{dnorm}},
  \code{\link{dnormv}},
  \code{\link{prec2var}},
  \code{\link{dst}},
  \code{\link{dt}},
  \code{\link{plot.demonoid.ppc}}, and
  \code{\link{plot.laplace.ppc}}.
}
\examples{
library(LaplacesDemon)
x <- dnormp(1,0,1)
x <- pnormp(1,0,1)
x <- qnormp(0.5,0,1)
x <- rnormp(100,0,1)

#Plot Probability Functions
x <- seq(from=-5, to=5, by=0.1)
plot(x, dnormp(x,0,0.5), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dnormp(x,0,1), type="l", col="green")
lines(x, dnormp(x,0,5), type="l", col="blue")
legend(2, 0.9, expression(paste(mu==0, ", ", tau==0.5),
     paste(mu==0, ", ", tau==1), paste(mu==0, ", ", tau==5)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{IAT}
\alias{IAT}
\title{Integrated Autocorrelation Time}
\description{
  The \code{IAT} function estimates integrated autocorrelation time,
  which is the computational inefficiency of a continuous chain or MCMC
  sampler. IAT is also called the IACT, ACT, autocorrelation time,
  autocovariance time, correlation time, or inefficiency factor. A lower
  value of \code{IAT} is better. \code{IAT} is a MCMC diagnostic that is
  an estimate of the number of iterations, on average, for an
  independent sample to be drawn, given a continuous chain or Markov
  chain. Put another way, \code{IAT} is the number of correlated samples
  with the same variance-reducing power as one independent sample.

  IAT is a univariate function. A multivariate form is not included.
}
\usage{
IAT(x)
}
\arguments{
  \item{x}{This requried argument is a vector of samples from a chain.}
}
\details{
  \code{IAT} is a MCMC diagnostic that is often used to compare
  continuous chains of MCMC samplers for computational inefficiency,
  where the sampler with the lowest \code{IAT}s is the most efficient
  sampler. Otherwise, chains may be compared within a model, such as
  with the output of \code{\link{LaplacesDemon}} to learn about the
  inefficiency of the continuous chain. For more information on
  comparing MCMC algorithmic inefficiency, see the
  \code{\link{Juxtapose}} function.
  
  \code{IAT} is also estimated in the \code{\link{PosteriorChecks}}
  function. \code{IAT} is usually applied to a stationary, continuous
  chain after discarding burn-in iterations (see \code{\link{burnin}}
  for more information). The \code{IAT} of a continuous chain correlates
  with the variability of the mean of the chain, and relates to
  Effective Sample Size (\code{\link{ESS}}) and Monte Carlo Standard
  Error (\code{\link{MCSE}}).

  \code{IAT} and \code{\link{ESS}} are inversely related, though not
  perfectly, because each is estimated a little differently. Given
  \eqn{N}{N} samples and taking autocorrelation into account,
  \code{\link{ESS}} estimates a reduced number of \eqn{M}{M} samples.
  Conversely, \code{IAT} estimates the number of autocorrelated samples,
  on average, required to produce one independently drawn sample.
  
  The \code{IAT} function is similar to the \code{IAT} function in the
  \code{Rtwalk} package of Christen and Fox (2010), which is currently
  unavailabe on CRAN.
}
\value{
  The \code{IAT} function returns the integrated autocorrelation time of
  a chain.
}
\references{
  Christen, J.A. and Fox, C. (2010). "A General Purpose Sampling
  Algorithm for Continuous Distributions (the t-walk)". \emph{Bayesian
  Analysis}, 5(2), p. 263--282.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{burnin}},
  \code{\link{Compare}},
  \code{\link{ESS}},
  \code{\link{LaplacesDemon}},
  \code{\link{MCSE}}, and
  \code{\link{PosteriorChecks}}.
}
\examples{
library(LaplacesDemon)
theta <- rnorm(100)
IAT(theta)
}
\keyword{Diagnostic}
\keyword{MCMC}

\name{dist.Log.Normal.Precision}
\alias{dlnormp}
\alias{plnormp}
\alias{qlnormp}
\alias{rlnormp}
\title{Log-Normal Distribution: Precision Parameterization}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the univariate log-normal
  distribution with mean \eqn{\mu}{mu} and precision \eqn{\tau}{tau}.
}
\usage{
dlnormp(x, mu, tau=NULL, var=NULL, log=FALSE)
plnormp(q, mu, tau, lower.tail=TRUE, log.p=FALSE)
qlnormp(p, mu, tau, lower.tail=TRUE, log.p=FALSE)
rlnormp(n, mu, tau=NULL, var=NULL)
}
\arguments{
  \item{x, q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{mu}{This is the mean parameter \eqn{\mu}{mu}.}
  \item{tau}{This is the precision parameter \eqn{\tau}{tau}, which
    must be positive. Tau and var cannot be used together}
  \item{var}{This is the variance parameter, which must be positive. Tau and var cannot be used together}
  \item{log, log.p}{Logical. If \code{TRUE}, then probabilities
    \eqn{p} are given as \eqn{\log(p)}{log(p)}.}
  \item{lower.tail}{Logical. If \code{TRUE} (default), then probabilities
    are \eqn{Pr[X \le x]}{Pr[X <= x]}, otherwise,
    \eqn{Pr[X > x]}{Pr[X > x]}.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density: \eqn{p(\theta) = \sqrt{\frac{\tau}{2\pi}}
    \frac{1}{\theta} \exp(-\frac{\tau}{2} (\log(\theta -
    \mu))^2)}{p(theta) = sqrt(tau/(2*pi)) * (1/theta) *
    exp(-(tau/2)*(log(theta-mu))^2)}
  \item Inventor: Carl Friedrich Gauss or Abraham De Moivre
  \item Notation 1: \eqn{\theta \sim \mathrm{Log-}\mathcal{N}(\mu,
    \tau^{-1})}{theta ~ Log-N(mu, tau^(-1))}
  \item Notation 2: \eqn{p(\theta) = \mathrm{Log-}\mathcal{N}(\theta | \mu,
    \tau^{-1})}{p(theta) = Log-N(theta | mu, tau^(-1))}
  \item Parameter 1: mean parameter \eqn{\mu}{mu}
  \item Parameter 2: precision parameter \eqn{\tau > 0}{tau > 0}
  \item Mean: \eqn{E(\theta) = \exp(\mu + \tau^{-1} / 2)}{E(theta) =
    exp(mu + tau^(-1) / 2)}
  \item Variance: \eqn{var(\theta) = (\exp(\tau^{-1}) - 1)\exp(2\mu +
    \tau^{-1})}{var(theta) = exp(tau^(-1) - 1) * exp(2*mu + tau^(-1))}
  \item Mode: \eqn{mode(\theta) = \exp(\mu - \tau^{-1})}{mode(theta) =
    exp(mu - tau^(-1))}
}

The log-normal distribution, also called the Galton distribution, is
applied to a variable whose logarithm is normally-distributed. The
distribution is usually parameterized with mean and variance, or in
Bayesian inference, with mean and precision, where precision is the
inverse of the variance. In contrast, \code{Base R} parameterizes the
log-normal distribution with the mean and standard deviation. These
functions provide the precision parameterization for convenience and
familiarity.

A flat distribution is obtained in the limit as
\eqn{\tau \rightarrow 0}{tau -> 0}.

These functions are similar to those in \code{base R}.
}
\value{
  \code{dlnormp} gives the density,
  \code{plnormp} gives the distribution function,
  \code{qlnormp} gives the quantile function, and
  \code{rlnormp} generates random deviates.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dnorm}},
  \code{\link{dnormp}},
  \code{\link{dnormv}}, and
  \code{\link{prec2var}}.
}
\examples{
library(LaplacesDemon)
x <- dlnormp(1,0,1)
x <- plnormp(1,0,1)
x <- qlnormp(0.5,0,1)
x <- rlnormp(100,0,1)

#Plot Probability Functions
x <- seq(from=0.1, to=3, by=0.01)
plot(x, dlnormp(x,0,0.1), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dlnormp(x,0,1), type="l", col="green")
lines(x, dlnormp(x,0,5), type="l", col="blue")
legend(2, 0.9, expression(paste(mu==0, ", ", tau==0.1),
     paste(mu==0, ", ", tau==1), paste(mu==0, ", ", tau==5)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{dist.Pareto}
\alias{dpareto}
\alias{ppareto}
\alias{qpareto}
\alias{rpareto}
\title{Pareto Distribution}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the pareto distribution.
}
\usage{
dpareto(x, alpha, log=FALSE)
ppareto(q, alpha)
qpareto(p, alpha)
rpareto(n, alpha)
}
\arguments{
  \item{x,q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{alpha}{This is the shape parameter \eqn{\alpha}{alpha}, which
    must be positive.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density or result is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density: \eqn{p(\theta) =
    \frac{\alpha}{\theta^{\alpha+1}}, \theta \ge 1}{p(theta) = alpha /
    theta^(alpha + 1), theta >= 1}
  \item Inventor: Vilfredo Pareto (1848-1923)
  \item Notation 1: \eqn{\theta \sim \mathcal{PA}(\alpha)}{theta ~ PA(alpha)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{PA}(\theta |
    \alpha)}{p(theta) = PA(theta | alpha)}
  \item Parameter 1: shape parameter \eqn{\alpha > 0}{alpha > 0}
  \item Mean: \eqn{E(\theta) = \frac{\alpha}{\alpha - 1}}{E(theta) =
    alpha / (alpha - 1)}
  \item Variance: \eqn{var(\theta) =
    \frac{\alpha}{(\alpha-1)^2(\alpha-2)}, \alpha > 2}{var(theta) =
    alpha / (alpha - 1)^2 (alpha - 2), alpha > 2}
  \item Mode: \eqn{mode(\theta) = 1}{mode(theta) = 1}
}

The Pareto distribution, sometimes called the Bradford distribution, is
related to the exponential distribution. The gamma distribution is the
conjugate prior distribution for the shape parameter \eqn{\alpha}{alpha}
in the Pareto distribution. The Pareto distribution is the conjugate
prior distribution for the range parameters of a uniform
distribution. An extension, elsewhere, is the symmetric Pareto
distribution.
}
\value{
  \code{dpareto} gives the density,
  \code{ppareto} gives the distribution function,
  \code{qpareto} gives the quantile function, and
  \code{rpareto} generates random deviates.
}
\seealso{
  \code{\link{dexp}},
  \code{\link{dlnorm}},
  \code{\link{dlnormp}},
  \code{\link{dnorm}},
  \code{\link{dnormp}},
  \code{\link{dnormv}}.
}
\examples{
library(LaplacesDemon)
x <- dpareto(1,1)
x <- ppareto(0.5,1)
x <- qpareto(0.5,1)
x <- rpareto(10,1)

#Plot Probability Functions
x <- seq(from=1, to=5, by=0.01)
plot(x, dpareto(x,0.1), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dpareto(x,0.5), type="l", col="green")
lines(x, dpareto(x,1), type="l", col="blue")
legend(2, 0.9, expression(alpha==0.1, alpha==0.5, alpha==1),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{dist.HalfCauchy}
\alias{dhalfcauchy}
\alias{phalfcauchy}
\alias{qhalfcauchy}
\alias{rhalfcauchy}
\title{Half-Cauchy Distribution}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the half-Cauchy distribution.
}
\usage{
dhalfcauchy(x, scale=25, log=FALSE)
phalfcauchy(q, scale=25)
qhalfcauchy(p, scale=25)
rhalfcauchy(n, scale=25)
}
\arguments{
  \item{x,q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{scale}{This is the scale parameter \eqn{\alpha}{alpha}, which
    must be positive.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density: \eqn{p(\theta) = \frac{2 \alpha}{\pi(\theta^2 +
      \alpha^2)}, \quad \theta > 0}{p(theta) = 2alpha / pi(theta^2 + alpha^2), theta >= 0}
  \item Inventor: Derived from Cauchy
  \item Notation 1: \eqn{\theta \sim \mathcal{HC}(\alpha)}{theta ~ HC(alpha)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{HC}(\theta | \alpha)}{p(theta) = HC(theta | alpha)}
  \item Parameter 1: scale parameter \eqn{\alpha > 0}{alpha > 0}
  \item Mean: \eqn{E(\theta)}{E(theta)} = does not exist
  \item Variance: \eqn{var(\theta)}{var(theta)} = does not exist
  \item Mode: \eqn{mode(\theta) = 0}{mode(theta) = 0}
}

The half-Cauchy distribution with scale \eqn{\alpha=25}{alpha=25} is a
recommended, default, weakly informative prior distribution for a scale
parameter. Otherwise, the scale, \eqn{\alpha}{alpha}, is recommended to
be set to be just a little larger than the expected standard deviation,
as a weakly informative prior distribution on a standard deviation
parameter.

The Cauchy distribution is known as a pathological distribution because
its mean and variance are undefined, and it does not satisfy the central
limit theorem.
}
\value{
  \code{dhalfcauchy} gives the density,
  \code{phalfcauchy} gives the distribution function,
  \code{qhalfcauchy} gives the quantile function, and
  \code{rhalfcauchy} generates random deviates.
}
\seealso{
  \code{\link{dcauchy}}
}
\examples{
library(LaplacesDemon)
x <- dhalfcauchy(1,25)
x <- phalfcauchy(1,25)
x <- qhalfcauchy(0.5,25)
x <- rhalfcauchy(1,25)

#Plot Probability Functions
x <- seq(from=0, to=20, by=0.1)
plot(x, dhalfcauchy(x,1), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dhalfcauchy(x,5), type="l", col="green")
lines(x, dhalfcauchy(x,10), type="l", col="blue")
legend(2, 0.9, expression(alpha==1, alpha==5, alpha==10),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{Math}
\alias{GaussHermiteQuadRule}
\alias{Hermite}
\alias{logadd}
\alias{partial}
\title{Math Utility Functions}
\description{
  These are utility functions for math.
}
\usage{
GaussHermiteQuadRule(N)
Hermite(x, N, prob=TRUE)
logadd(x, add=TRUE)
partial(Model, parm, Data, Interval=1e-6, Method="simple")
}
\arguments{
  \item{N}{This required argument accepts a positive integer that
    indicates the number of nodes.}
  \item{x}{This is a numeric vector.}
  \item{add}{Logical. This defaults to \code{TRUE}, in which case
    \eqn{\log(x+y)}{log(x+y)} is performed. Otherwise,
    \eqn{\log(x-y)}{log(x-y)} is performed.}
  \item{Model}{This is a model specification function. For more
    information, see \code{\link{LaplacesDemon}}.}
  \item{parm}{This is a vector parameters.}
  \item{prob}{Logical. This defaults to \code{TRUE}, which uses the
    probabilist's kernel for the Hermite polynomial. Otherwise,
    \code{FALSE} uses the physicist's kernel.}
  \item{Data}{This is a list of data. For more information, see
    \code{\link{LaplacesDemon}}.}
  \item{Interval}{This is the interval of numeric differencing.}
  \item{Method}{This accepts a quoted string, and defaults to
    "simple", which is finite-differencing. Alternatively
    \code{Method="Richardson"} uses Richardson extrapolation, which
    is more accurate, but takes longer to calculate. Another method
    called automatic differentiation is currently unsupported, but
    is even more accurate, and takes even longer to calculate.}
}
\details{
  The \code{GaussHermiteQuadRule} function returns nodes and weights for
  univariate Gauss-Hermite quadrature. The nodes and weights are
  obtained from a tridiagonal eigenvalue problem. Weights are calculated
  from the physicist's (rather than the probabilist's) kernel. This has
  been adapted from the GaussHermite function in the pracma package. The
  \code{\link{GaussHermiteCubeRule}} function is a multivariate version.
  This is used in the \code{\link{IterativeQuadrature}} function.

  The \code{Hermite} function evaluates a Hermite polynomial of degree
  \eqn{N} at \eqn{x}, using either the probabilist's (\code{prob=TRUE})
  or physicist's (\code{prob=FALSE}) kernel. This function was adapted
  from the \code{hermite} function in package EQL.
  
  The \code{logadd} function performs addition (or subtraction) when the
  terms are logarithmic. The equations are:

  \deqn{\log(x+y) = \log(x) + \log(1 + \exp(\log(y) - \log(x)))}{log(x+y) =
    log(x) + log(1 + exp(log(y) - log(x)))}
  \deqn{\log(x-y) = \log(x) + \log(1 - \exp(\log(y) - \log(x)))}{log(x-y) =
    log(x) + log(1 - exp(log(y) - log(x)))}
  
  The \code{partial} function estimates partial derivatives of
  parameters in a model specification with data, using either
  forward finite-differencing or Richardson extrapolation. In calculus,
  a partial derivative of a function of several variables is its
  derivative with respect to one of those variables, with the others
  held constant. Related functions include \code{Jacobian} which returns
  a matrix of first-order partial derivatives, and \code{Hessian}, which
  returns a matrix of second-order partial derivatives of the model
  specification function with respect to its parameters. The
  \code{partial} function is not intended to be called by the user, but
  is used by other functions. This is essentially the \code{grad}
  function in the numDeriv package, but defaulting to forward
  finite-differencing with a smaller interval.
}
\value{
  \code{logadd} returns the result of \eqn{\log(x+y)}{log(x+y)} or
  \eqn{\log(x-y)}{log(x-y)}.
  
  \code{partial} returns a vector of partial derivatives.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{GaussHermiteCubeRule}},
  \code{\link{Hessian}},
  \code{\link{IterativeQuadrature}},
  \code{\link{Jacobian}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, and
  \code{\link{VariationalBayes}}.
}
\keyword{Math}

\name{is.model}
\alias{is.model}
\title{Logical Check of a Model}
\description{
  This function provides a logical test of whether or not a \code{Model}
  specification function meets mininum requirements to be considered as
  such.
}
\usage{
is.model(Model, Initial.Values, Data)
}
\arguments{
  \item{Model}{This is a model specification function. For more
    information, see the \code{\link{LaplacesDemon}} function.}
  \item{Initial.Values}{This is a vector of initial values, or current
    parameter values. For more information, see the
    \code{\link{LaplacesDemon}} function.}
  \item{Data}{This is a list of data. For more information, see the
    \code{\link{LaplacesDemon}} function.}
}
\details{
  This function tests for minimum criteria for \code{Model} to be
  considered a model specification function. Specifically, it tests:

  \itemize{
    \item \code{Model} must be a function
    \item \code{Model} must execute without errors
    \item \code{Model} must return a list
    \item \code{Model} must have five components in the list
    \item The first component must be named LP and have length 1
    \item The second component must be named Dev and have length 1
    \item The third component must be named Monitor
    \item The lengths of Monitor and mon.names must be equal
    \item The fourth component must be named yhat
    \item The fifth component must be named parm
    \item The lengths of parm and parm.names must be equal
  }

  This function is not extensive, and checks only for these minimum
  criteria. Additional checks are conducted in
  \code{\link{IterativeQuadrature}}, \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\value{
  The \code{is.model} function returns a logical value of \code{TRUE}
  when \code{Model} meets minimum criteria of a model specification
  function, and \code{FALSE} otherwise.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\keyword{Utility}

\name{Thin}
\alias{Thin}
\title{Thin}
\description{
  This function reduces the number of posterior samples by retaining
  every \eqn{k}th sample.
}
\usage{
Thin(x, By=1)
}
\arguments{
  \item{x}{This is a vector or matrix of posterior samples to be
    thinned.}
  \item{By}{This argument specifies that every \eqn{k}th posterior
    sample will be retained, and \code{By} defaults to 1, meaning that
    thinning will not occur, because every sample will be retained.}
}
\details{
  A thinned matrix of posterior samples is a matrix in which only every
  \eqn{k}th posterior sample (or row) in the original matrix is
  retained. The act of thinning posterior samples has been criticized as
  throwing away information, which is correct. However, it is common
  practice to thin posterior samples, usually associated with MCMC such
  as \code{\link{LaplacesDemon}}, for two reasons. First, Each chain
  (column vector) in a matrix of posterior samples probably has higher
  autocorrelation than desired, which reduces the effective sample size
  (see \code{\link{ESS}} for more information). Therefore, a thinned
  matrix usually contains posterior samples that are closer to
  independent than an un-thinned matrix. The other reason for the
  popularity of thinning is that it a user may not have the
  random-access memory (RAM) to store large, un-thinned matrices of
  posterior samples.

  \code{\link{LaplacesDemon}} and \code{\link{PMC}} automatically thin
  posterior samples, deviance samples, and samples of monitored
  variables, according to its own user-specified argument. The
  \code{Thin} function is made available here, should it be necessary to
  thin posterior samples outside of objects of class \code{demonoid} or
  \code{pmc}.
}
\value{
  The \code{Thin} argument returns a thinned matrix. When \code{x} is a
  vector, the returned object is a matrix with 1 column.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{ESS}},
  \code{\link{LaplacesDemon}}, and
  \code{\link{PMC}}.
}
\examples{
library(LaplacesDemon)
x <- matrix(runif(100), 10, 10)
Thin(x, By=2)
}
\keyword{Diagnostic}
\keyword{MCMC}
\keyword{Monte Carlo}
\keyword{Utility}

\name{LaplacesDemon.RAM}
\alias{LaplacesDemon.RAM}
\title{LaplacesDemon RAM Estimate}
\description{
  This function estimates the random-access memory (RAM) required to
  update a given model and data with the \code{\link{LaplacesDemon}}
  function.

  \emph{Warning:} Unwise use of this function may crash a computer, so
  please read the details below.
}
\usage{
LaplacesDemon.RAM(Model, Data, Iterations, Thinning, Algorithm="RWM")
}
\arguments{
  \item{Model}{This is a model specification function. For more
    information, see \code{\link{LaplacesDemon}}.}
  \item{Data}{This is a list of Data. For more information, see
    \code{\link{LaplacesDemon}}.}
  \item{Iterations}{This is the number of iterations for which
    \code{\link{LaplacesDemon}} would update. For more information,
    see \code{\link{LaplacesDemon}}.}
  \item{Thinning}{This is the amount of thinning applied to the chains
    in \code{\link{LaplacesDemon}}.For more information, see
    \code{\link{LaplacesDemon}}.}
  \item{Algorithm}{This argument accepts the name of the algorithm as a
    string, as entered in \code{\link{LaplacesDemon}}.For more
    information, see \code{\link{LaplacesDemon}}.}
}
\details{
  The \code{LaplacesDemon.RAM} function uses the
  \code{\link{object.size}} function to estimate the size in MB of RAM
  required to update one chain in \code{\link{LaplacesDemon}} for a
  given model and data, and for a number of iterations and specified
  thinning. When RAM is exceeded, the computer will crash. This function
  can be useful when trying to estimate how many iterations to update a
  model without crashing the computer. However, when estimating the
  required RAM, \code{LaplacesDemon.RAM} actually creates several large
  objects, such as \code{post} (see below). If too many iterations are
  given as an argument to \code{LaplacesDemon.RAM}, for example, then it
  will crash the computer while trying to estimate the required RAM.

  The best way to use this function is as follows. First, prepare the
  model specification and list of data. Second, observe how much RAM the
  computer is using at the moment, as well as the maximum available
  RAM. The majority of the difference of these two is the amount of RAM
  the computer may dedicate to updating the model. Next, use this
  function with a small number of iterations (important in some
  algorithms), and with few thinned samples (important in all
  algorithms). Note the estimated RAM. Increase the number of
  iterations and thinned samples, and again note the RAM. Continue to
  increase the number of iterations and thinned samples until, say,
  arbitrarily within 90\% of the above-mentioned difference in RAM.

  The computer operating system uses RAM, as does any other software
  running at the moment. R is currently using RAM, and other functions
  in the \code{LaplacesDemon} package, and any other package that is
  currently activated, are using RAM. There are numerous small objects
  that are not included in the returned list, that use RAM. For example,
  there may be a scalar called \code{alpha} for the acceptance
  probability, etc.

  One potentially larger object that is not included, and depends on
  the algorithm, is a matrix used for estimating \code{\link{LML}}.
  Its use occurs with non-adaptive MCMC algorithms, only with enough
  globally stationary samples, and only when the ratio of parameters to
  samples is not excessive. If used, then the user should create a
  matrix of the appropriate dimensions and use the
  \code{\link{object.size}} function to estimate the RAM.

  If the data is too large for RAM, then consider using either the
  \code{\link{BigData}} function or the SGLD algorithm in
  \code{\link{LaplacesDemon}}.
}
\value{
  \code{LaplacesDemon.RAM} returns a list with several components. Each
  component is an estimate in MB for an object. The list has the
  following components:
  \item{Covar}{This is the estimated size in MB of RAM required for the
    covariance matrix, variance vector, or both (some algorithms store
    both internally, creating one from the other). Blocked covariance
    matrices are not considered at this time.}
  \item{Data}{This is the estimated size in MB of RAM required for the
    list of data.}
  \item{Deviance}{This is the estimated size in MB of RAM required for
    the deviance vector.}
  \item{Initial.Values}{This is the estimated size in MB of RAM required
    for the vector of initial values.}
  \item{Model}{This is the estimated size in MB of RAM required for the
    model specification function.}
  \item{Monitor}{This is the estimated size in MB of RAM required for
    the \eqn{N \times J}{N x J} matrix \code{Monitor}, where \eqn{N} is
    the number of thinned samples and J is the number of monitored
    variables.}
  \item{post}{This is the estimated size in MB of RAM required for a
    matrix of posterior samples. This matrix is used in some algorithms,
    and is not returned by \code{\link{LaplacesDemon}}.}
  \item{Posterior1}{This is the estimated size in MB of RAM required for
    the \eqn{N \times J}{N x J} matrix \code{Posterior1}, where \eqn{N}
    is the number of thinned samples and \eqn{J} is the number of
    initial values or parameters.}
  \item{Posterior2}{This is the estimated size in MB of RAM required for
    the \eqn{N \times J}{N x J} matrix \code{Posterior2}, where \eqn{N}
    is the number of globally stationary thinned samples and \eqn{J} is
    the number of initial values or parameters. Maximum RAM use is
    assumed here, so the same \eqn{N} is used, as in \code{Posterior1}.}
  \item{Summary1}{This is the estimated size in MB of RAM required for
    the summary table of all thinned posterior samples of parameters,
    deviance, and monitored variables.}
  \item{Summary2}{This is the estimated size in MB of RAM required for
    the summary table of all globally stationary thinned posterior
    samples of parameters, deviance, and monitored variables.}
  \item{Total}{This is the estimated size in MB of RAM required in total
    to update one chain in \code{\link{LaplacesDemon}} for a given model
    and data, and for a number of iterations and specified thinning.}
}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{BigData}},
  \code{\link{LaplacesDemon}},
  \code{\link{LML}}, and
  \code{\link{object.size}}.
}
\keyword{Memory}

\name{is.appeased}
\alias{is.appeased}
\title{Appeased}
\description{
  This function returns \code{TRUE} if Laplace's Demon is appeased by the
  object of class \code{demonoid}, and \code{FALSE} otherwise. If
  appeased, then the object passes several tests that indicate potential
  convergence of the Markov chains.
}
\usage{
is.appeased(x)
}
\arguments{
  \item{x}{This is an object of class \code{demonoid}.}
}
\details{
  After updating a model with the \code{\link{LaplacesDemon}} function,
  an output object is created. The output object is of class
  \code{demonoid}. The object may be passed to the \code{\link{Consort}}
  function, which will apply several criteria regarding the potential
  convergence of its Markov chains. If all criteria are met, then
  Laplace's Demon is appeased. Otherwise, Laplace's Demon suggests R
  code to be copy/pasted and executed. The \code{\link{Consort}}
  function prints a large amount of information to the screen. The
  \code{is.appeased} function may be applied as an alternative, though
  it only informs the user as to whether or not Laplace's Demon was
  appeased, as \code{TRUE} or \code{FALSE}.
}
\value{
  The \code{is.appeased} function returns a logical value indicating
  whether or not the supplied object passes several potential Markov
  chain convergence criteria. If the object passes all criteria, then
  Laplace's Demon is appeased, and the logical value returned is
  \code{TRUE}.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{Consort}} and
  \code{\link{LaplacesDemon}}.
}
\keyword{Diagnostic}
\keyword{Utility}

\name{BayesTheorem}
\alias{BayesTheorem}
\title{Bayes' Theorem}
\description{
  Bayes' theorem shows the relation between two conditional
  probabilities that are the reverse of each other. This theorem is
  named after Reverend Thomas Bayes (1702-1761), and is also referred to
  as Bayes' law or Bayes' rule (Bayes and Price, 1763). Bayes' theorem
  expresses the conditional probability, or `posterior probability', of
  an event \eqn{A} after \eqn{B} is observed in terms of the `prior
  probability' of \eqn{A}, prior probability of \eqn{B}, and the
  conditional probability of \eqn{B} given \eqn{A}. Bayes' theorem is
  valid in all common interpretations of probability. This function
  provides one of several forms of calculations that are possible with
  Bayes' theorem.
}
\usage{
BayesTheorem(PrA, PrBA)
}
\arguments{
  \item{PrA}{This required argument is the prior probability of \eqn{A},
    or \eqn{\Pr(A)}{Pr(A)}.}
  \item{PrBA}{This required argument is the conditional probability of
    \eqn{B} given \eqn{A} or \eqn{\Pr(B | A)}{Pr(B | A)}, and is known
    as the data, evidence, or likelihood.}
}
\value{
  The \code{BayesTheorem} function returns the conditional probability
  of \eqn{A} given \eqn{B}, known in Bayesian inference as the
  posterior. The returned object is of class \code{bayestheorem}.
}
\details{
  Bayes' theorem provides an expression for the conditional probability
  of \eqn{A} given \eqn{B}, which is equal to

  \deqn{\Pr(A | B) = \frac{\Pr(B | A)\Pr(A)}{\Pr(B)}}{Pr(A | B) = (Pr(B
    | A)Pr(A)) / Pr(B)}

  For example, suppose one asks the question: what is the probability of
  going to Hell, conditional on consorting (or given that a person
  consorts) with Laplace's Demon. By replacing \eqn{A} with \eqn{Hell}
  and \eqn{B} with \eqn{Consort}, the question becomes

  \deqn{\Pr(\mathrm{Hell} | \mathrm{Consort}) =
  \frac{\Pr(\mathrm{Consort} |
  \mathrm{Hell})\Pr(\mathrm{Hell})}{\Pr(\mathrm{Consort})}}{Pr(Hell |
  Consort) = (Pr(Consort | Hell)Pr(Hell)) / Pr(Consort)}

  Note that a common fallacy is to assume that \eqn{\Pr(A | B) = \Pr(B |
    A)}{Pr(A | B) = Pr(B | A)}, which is called the conditional
  probability fallacy.

  Another way to state Bayes' theorem (and this is the form in the
  provided function) is

  \deqn{\Pr(A_i | B) = \frac{\Pr(B | A_i)\Pr(A_i)}{\Pr(B | A_i)\Pr(A_i)
  +\dots+ \Pr(B | A_n)\Pr(A_n)}}{Pr(A[i] | B) = (Pr(B | A[i])Pr(A[i])) /
  (Pr(B | A[i])Pr(A[i]) +...+ Pr(B | A[n])Pr(A[n]))}

  Let's examine our \emph{burning} question, by replacing
  \eqn{A_i}{A[i]} with Hell or Heaven, and replacing \eqn{B} with
  Consort

  \itemize{
    \item \eqn{\Pr(A_1) = \Pr(\mathrm{Hell})}{Pr(A[1] = Pr(Hell)}
    \item \eqn{\Pr(A_2) = \Pr(\mathrm{Heaven})}{Pr(A[2] = Pr(Heaven)}
    \item \eqn{\Pr(B) = \Pr(\mathrm{Consort})}{Pr(B) = Pr(Consort)}
    \item \eqn{\Pr(A_1 | B) = \Pr(\mathrm{Hell} |
    \mathrm{Consort})}{Pr(A[1] | B) = Pr(Hell | Consort)}
    \item \eqn{\Pr(A_2 | B) = \Pr(\mathrm{Heaven} |
    \mathrm{Consort})}{Pr(A[2] | B) = Pr(Heaven | Consort)}
    \item \eqn{\Pr(B | A_1) = \Pr(\mathrm{Consort} |
    \mathrm{Hell})}{Pr(B | A[1]) = Pr(Consort | Heaven)}
    \item \eqn{\Pr(B | A_2) = \Pr(\mathrm{Consort} |
    \mathrm{Heaven})}{Pr(B | A[2]) = Pr(Consort | Heaven)}
}

  Laplace's Demon was conjured and asked for some data. He was glad to
  oblige.

  \itemize{
    \item 6 people consorted out of 9 who went to Hell.
    \item 5 people consorted out of 7 who went to Heaven.
    \item 75\% of the population goes to Hell.
    \item 25\% of the population goes to Heaven.
  }

  Now, Bayes' theorem is applied to the data. Four pieces are worked out
  as follows

  \itemize{
    \item \eqn{\Pr(\mathrm{Consort} | \mathrm{Hell}) = 6/9 =
    0.666}{Pr(Consort | Hell) = 6/9 = 0.666}
    \item \eqn{\Pr(\mathrm{Consort} | \mathrm{Heaven}) = 5/7 =
    0.714}{Pr(Consort | Heaven) = 5/7 = 0.714}
    \item \eqn{\Pr(\mathrm{Hell}) = 0.75}{Pr(Hell) = 0.75}
    \item \eqn{\Pr(\mathrm{Heaven}) = 0.25}{Pr(Heaven) = 0.25}
  }

  Finally, the desired conditional probability \eqn{\Pr(\mathrm{Hell} |
    \mathrm{Consort})}{Pr(Hell | Consort)} is calculated using Bayes'
  theorem

  \itemize{
    \item \eqn{\Pr(\mathrm{Hell} | \mathrm{Consort}) =
    \frac{0.666(0.75)}{0.666(0.75) + 0.714(0.25)}}{Pr(Hell | Consort) =
    0.666(0.75) / (0.666(0.75) + 0.714(0.25))}
    \item \eqn{\Pr(\mathrm{Hell} | \mathrm{Consort}) = 0.737}{Pr(Hell |
    Consort) = 0.737}
}
    The probability of someone consorting with Laplace's Demon and going
    to Hell is 73.7\%, which is less than the prevalence of 75\% in the
    population. According to these findings, consorting with Laplace's
    Demon does not increase the probability of going to Hell.

    For an introduction to model-based Bayesian inference, see the
    accompanying vignette entitled ``Bayesian Inference'' or
    \url{https://web.archive.org/web/20150206004608/http://www.bayesian-inference.com/bayesian}.
}
\references{
  Bayes, T. and Price, R. (1763). "An Essay Towards Solving a Problem in
  the Doctrine of Chances". By the late Rev. Mr. Bayes, communicated by
  Mr. Price, in a letter to John Canton, M.A. and F.R.S.
  \emph{Philosophical Transactions of the Royal Statistical Society of
  London}, 53, p. 370--418.
}
\author{Statisticat, LLC.}
\seealso{
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
# Pr(Hell|Consort) =
PrA <- c(0.75,0.25)
PrBA <- c(6/9, 5/7)
BayesTheorem(PrA, PrBA)
}
\keyword{Bayesian}
\keyword{Probability}

\name{dist.LASSO}
\alias{dlasso}
\alias{rlasso}
\title{LASSO Distribution}
\description{
  These functions provide the density and random generation for the
  Bayesian LASSO prior distribution.
}
\usage{
dlasso(x, sigma, tau, lambda, a=1, b=1, log=FALSE)
rlasso(n, sigma, tau, lambda, a=1, b=1)
}
\arguments{
  \item{x}{This is a location vector of length \eqn{J} at which to
    evaluate density.}
  \item{n}{This is the number of observations, which must be a
    positive integer that has length 1.}
  \item{sigma}{This is a positive-only scalar hyperparameter
    \eqn{\sigma}{sigma}, which is also the residual standard deviation.}
  \item{tau}{This is a positive-only vector of hyperparameters,
    \eqn{\tau}{tau}, of length \eqn{J} regarding local sparsity.}
  \item{lambda}{This is a positive-only scalar hyperhyperparameter,
    \eqn{\lambda}{lambda}, of global sparsity.}
  \item{a, b}{These are positive-only scalar hyperhyperhyperparameters
    for gamma distributed \eqn{\lambda}{lambda}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Multivariate Scale Mixture
    \item Density: \eqn{p(\theta) \sim \mathcal{N}_k(0, \sigma^2
      diag(\tau^2))(\frac{1}{sigma^2})
      \mathcal{EXP}(\frac{\lambda^2}{2}) \mathcal{G}(a,b)}{p(theta) ~
      N[k](0, sigma^2 diag(tau^2))(1/sigma^2) EXP(lambda^2/2) G(a,b)}    
    \item Inventor: Parks and Casella (2008)
    \item Notation 1: \eqn{\theta \sim \mathcal{LASSO}(\sigma, \tau,
      \lambda, a, b)}{theta ~ LASSO(sigma, tau, lambda, a, b)}
    \item Notation 2: \eqn{p(\theta) = \mathcal{LASSO}(\theta | \sigma,
      \tau, \lambda, a, b)}{p(theta) = LASSO(theta | sigma, tau,
      lambda, a, b)}
    \item Parameter 1: hyperparameter global scale
      \eqn{\sigma > 0}{sigma > 0}
    \item Parameter 2: hyperparameter local scale
      \eqn{\tau > 0}{tau > 0}
    \item Parameter 3: hyperhyperparameter global scale
      \eqn{\lambda > 0}{lambda > 0}
    \item Parameter 4: hyperhyperhyperparameter scale \eqn{a > 0}
    \item Parameter 5: hyperhyperhyperparameter scale \eqn{b > 0}
    \item Mean: \eqn{E(\theta)}{E(theta)}
    \item Variance: 
    \item Mode: 
}

The Bayesian LASSO distribution (Parks and Casella, 2008) is a
heavy-tailed mixture distribution that can be considered a variance
mixture, and it is in the family of multivariate scale mixtures of
normals.

The LASSO distribution was proposed as a prior distribution, as a
Bayesian version of the frequentist LASSO, introduced by Tibshirani
(1996). It is applied as a shrinkage prior in the presence of sparsity
for \eqn{J} regression effects. LASSO priors are most appropriate in
large-dimensional models where dimension reduction is necessary to
avoid overly complex models that predict poorly.

The Bayesian LASSO results in regression effects that are a compromise
between regression effects in the frequentist LASSO and ridge
regression. The Bayesian LASSO applies more shrinkage to weak regression
effects than ridge regression.

The Bayesian LASSO is an alternative to horseshoe regression and ridge
regression.
}
\value{
  \code{dlasso} gives the density and
  \code{rlasso} generates random deviates.
}
\references{
  Park, T. and Casella, G. (2008). "The Bayesian Lasso". \emph{Journal
  of the American Statistical Association}, 103, p. 672--680.

  Tibshirani, R. (1996). "Regression Shrinkage and Selection via the
  Lasso". \emph{Journal of the Royal Statistical Society}, Series B, 58,
  p. 267--288.
}
\seealso{
  \code{\link{dhs}}
}
\examples{
library(LaplacesDemon)
x <- rnorm(100)
sigma <- rhalfcauchy(1, 5)
tau <- rhalfcauchy(100, 5)
lambda <- rhalfcauchy(1, 5)
x <- dlasso(x, sigma, tau, lambda, log=TRUE)
x <- rlasso(length(tau), sigma, tau, lambda)
}
\keyword{Distribution}

\name{print.demonoid}
\alias{print.demonoid}
\title{Print an object of class \code{demonoid} to the screen}
\description{
  This may be used to print the contents of an object of class
  \code{demonoid} to the screen.
}
\usage{\method{print}{demonoid}(x, \dots)}
\arguments{
  \item{x}{An object of class \code{demonoid} is required.}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  If the user has an object of class \code{demonoid.hpc}, then the
  \code{print} function may still be used by specifying the chain as a
  component in a list, such as printing the second chain with
  \code{print(Fit[[2]])} when the \code{demonoid.hpc} object is named
  \code{Fit}, for example.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{Consort}},
  \code{\link{LaplacesDemon}}, and
  \code{\link{LaplacesDemon.hpc}}.
}
\examples{### See the LaplacesDemon function for an example.}
\keyword{print}

\name{cond.plot}
\alias{cond.plot}
\title{Conditional Plots}
\description{
  This function provides several styles of conditional plots with base
  graphics.
}
\usage{
cond.plot(x, y, z, Style="smoothscatter")
}
\arguments{
  \item{x}{This required argument accepts a numeric vector.}
  \item{y}{This argument accepts a numeric vector, and is only used with
    some styles.}
  \item{z}{This required argument accepts a discrete vector.}
  \item{Style}{This argument specifies the style of plot, and accepts
    "boxplot", "densover" (density overlay), "hist", "scatter", or
    "smoothscatter".}
}
\details{
  The \code{cond.plot} function provides simple conditional plots with
  base graphics.  All plot styles are conditional upon \code{z}. Up to
  nine conditional plots are produced in a panel.

  Plots include:

  boxplot: y ~ x | z
  densover: f(x | z)
  hist: x | z
  scatter: x, y | z
  smoothscatter: x, y | z

  The \code{cond.plot} function is not intended to try to compete with
  some of the better graphics packages, but merely to provide simple
  functionality.
}
\value{
  Conditional plots are returned.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{joint.density.plot}} and
  \code{\link{joint.pr.plot}}.
}
\examples{
library(LaplacesDemon)
x <- rnorm(1000)
y <- runif(1000)
z <- rcat(1000, rep(1/4,4))
cond.plot(x, y, z, Style="smoothscatter")
}
\keyword{Plot}

\name{ABB}
\alias{ABB}
\title{Approximate Bayesian Bootstrap}
\description{
  This function performs multiple imputation (MI) with the Approximate
  Bayesian Bootstrap (ABB) of Rubin and Schenker (1986).
}
\usage{
ABB(X, K=1)
}
\arguments{
  \item{X}{This is a vector or matrix of data that must include both
    observed and missing values. When \code{X} is a matrix, missing
    values must occur somewhere in the set, but are not required to
    occur in each variable.}
  \item{K}{This is the number of imputations.}
}
\details{
  The Approximate Bayesian Bootstrap (ABB) is a modified form of the
  \code{\link{BayesianBootstrap}} (Rubin, 1981) that is used for
  multiple imputation (MI). Imputation is a family of statistical
  methods for replacing missing values with estimates. Introduced by
  Rubin and Schenker (1986) and Rubin (1987), MI is a family of
  imputation methods that includes multiple estimates, and therefore
  includes variability of the estimates.

  The data, \eqn{\textbf{X}}{X}, are assumed to be independent and
  identically distributed (IID), contain both observed and missing
  values, and its missing values are assumed to be ignorable (meaning
  enough information is available in the data that the missingness
  mechanism can be ignored, if the information is used properly) and
  Missing Completely At Random (MCAR). When \code{ABB} is used in
  conjunction with a propensity score (described below), missing values
  may be Missing At Random (MAR).

  \code{ABB} does not add auxiliary information, but performs imputation
  with two sampling (with replacement) steps. First,
  \eqn{\textbf{X}^\star_{obs}}{X_star_obs} is sampled from
  \eqn{\textbf{X}_{obs}}{X_obs}. Then,
  \eqn{\textbf{X}^\star_{mis}}{X_star_mis} is sampled from
  \eqn{\textbf{X}^\star_{obs}}{X_star_obs}. The result is a sample of
  the posterior predictive distribution of
  \eqn{(\textbf{X}_{mis}|\textbf{X}_{obs})}{(X_mis|X_obs)}. The first
  sampling step is also known as hotdeck imputation, and the second
  sampling step changes the variance. Since auxiliary information is not
  included, \code{ABB} is appropriate for missing values that are
  ignorable and MCAR.

  Auxiliary information may be included in the process of imputation by
  introducing a propensity score (Rosenbaum and Rubin, 1983; Rosenbaum
  and Rubin, 1984), which is an estimate of the probability of
  missingness. The propensity score is often the result of a binary
  logit model, where missingness is predicted as a function of other
  variables. The propensity scores are discretized into quantile-based
  groups, usually quintiles. Each quintile must have both observed and
  missing values. \code{ABB} is applied to each quintile. This is called
  within-class imputation. It is assumed that the missing mechanism
  depends only on the variables used to estimate the propensity score.

  With \eqn{K=1}, \code{ABB} may be used in MCMC, such as in
  \code{\link{LaplacesDemon}}, more commonly along with a propensity
  score for missingness. MI is performed, despite \eqn{K=1}, because
  imputation occurs at each MCMC iteration. The practical advantage of
  this form of imputation is the ease with which it may be
  implemented. For example, full-likelihood imputation should perform
  better, but requires a chain to be updated for each missing value.
  
  An example of a limitation of \code{ABB} with propensity scores is to
  consider imputing missing values of income from age in a context where
  age and income have a positive relationship, and where the highest
  incomes are missing systematically. \code{ABB} with propensity scores
  should impute these highest missing incomes given the highest observed
  ages, but is unable to infer beyond the observed data.

  ABB has been extended (Parzen et al., 2005) to reduce bias, by
  introducing a correction factor that is applied to the MI variance
  estimate. This correction may be applied to output from \code{ABB}.
}
\value{
  This function returns a list with \eqn{K} components, one for each set
  of imputations. Each component contains a vector of imputations equal
  in length to the number of missing values in the data.
  
  \code{ABB} does not currently return the mean of the imputations, or
  the between-imputation variance or within-imputation variance.
}
\references{
  Parzen, M., Lipsitz, S.R., and Fitzmaurice, G.M. (2005). "A Note on
  Reducing the Bias of the Approximate Bayesian Bootstrap Imputation
  Variance Estimator". \emph{Biometrika}, 92, 4, p. 971--974.

  Rosenbaum, P.R. and Rubin, D.B. (1983). "The Central Role of the
  Propensity Score in Observational Studies for Causal
  Effects". \emph{Biometrika}, 70, p. 41--55.

  Rosenbaum, P.R. and Rubin, D.B. (1984). "Reducing Bias in
  Observational Studies Using Subclassification in the Propensity
  Score". \emph{Journal of the American Statistical Association}, 79,
  p. 516--524.

  Rubin, D.B. (1981). "The Bayesian Bootstrap". \emph{Annals of
  Statistics}, 9, p. 130--134.

  Rubin, D.B. (1987). "Multiple Imputation for Nonresponse in
  Surveys". John Wiley and Sons: New York, NY.

  Rubin, D.B. and Schenker, N. (1986). "Multiple Imputation for Interval
  Estimation from Simple Random Samples with Ignorable
  Nonresponse". \emph{Journal of the American Statistical Association},
  81, p. 366--374.
}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{BayesianBootstrap}},
  \code{\link{LaplacesDemon}}, and
  \code{\link{MISS}}.
}
\examples{
library(LaplacesDemon)

### Create Data
J <- 10 #Number of variables
m <- 20 #Number of missings
N <- 50 #Number of records
mu <- runif(J, 0, 100)
sigma <- runif(J, 0, 100)
X <- matrix(0, N, J)
for (j in 1:J) X[,j] <- rnorm(N, mu[j], sigma[j])

### Create Missing Values
M1 <- rep(0, N*J)
M2 <- sample(N*J, m)
M1[M2] <- 1
M <- matrix(M1, N, J)
X <- ifelse(M == 1, NA, X)

### Approximate Bayesian Bootstrap
imp <- ABB(X, K=1)

### Replace Missing Values in X (when K=1)
X.imp <- X
X.imp[which(is.na(X.imp))] <- unlist(imp)
X.imp
}
\keyword{Imputation}

\name{plot.demonoid}
\alias{plot.demonoid}
\alias{plot.demonoid.hpc}
\title{Plot samples from the output of Laplace's Demon}
\description{
  This may be used to plot, or save plots of, samples in an object of
  class \code{demonoid} or \code{demonoid.hpc}. Plots include a trace
  plot, density plot, autocorrelation or ACF plot, and if an adaptive
  algorithm was used, the absolute difference in the proposal variance,
  or the value of epsilon, across adaptations.
}
\usage{
\method{plot}{demonoid}(x, BurnIn=0, Data, PDF=FALSE, Parms, FileName, \dots)
\method{plot}{demonoid.hpc}(x, BurnIn=0, Data, PDF=FALSE, Parms, FileName, \dots)
}
\arguments{
  \item{x}{
    This required argument is an object of class \code{demonoid} or
    \code{demonoid.hpc}.}
  \item{BurnIn}{
    This argument requires zero or a positive integer that indicates the
    number of thinned samples to discard as burn-in for the purposes of
    plotting. For more information on burn-in, see \code{\link{burnin}}.}
  \item{Data}{
    This required argument must receive the list of data that was
    supplied to \code{\link{LaplacesDemon}} to create the object of
    class \code{demonoid}.}
  \item{PDF}{
    This logical argument indicates whether or not the user wants
    Laplace's Demon to save the plots as a .pdf file.}
  \item{Parms}{
    This argument accepts a vector of quoted strings to be matched for
    selecting parameters for plotting. This argument defaults to
    \code{NULL} and selects every parameter for plotting. Each quoted
    string is matched to one or more parameter names with the
    \code{grep} function. For example, if the user specifies
    \code{Parms=c("eta", "tau")}, and if the parameter names
    are beta[1], beta[2], eta[1], eta[2], and tau, then all parameters
    will be selected, because the string \code{eta} is within
    \code{beta}. Since \code{grep} is used, string matching uses
    regular expressions, so beware of meta-characters, though these are
    acceptable: ".", "[", and "]".}
  \item{FileName}{
    This argument accepts a string and save the plot under the specified name. If \code{PDF=FALSE} this argument in unused. By default, \code{FileName = paste0("laplacesDemon-plot_", format(Sys.time(), "yyyy-mm-dd_h:m:s"), ".pdf")}
  }
  \item{\dots}{Additional arguments are unused.}
}
\details{
  The plots are arranged in a \eqn{3 \times 3}{3 x 3} matrix. Each row
  represents a parameter, the deviance, or a monitored variable. The
  left column displays trace plots, the middle column displays kernel
  density plots, and the right column displays autocorrelation (ACF)
  plots.

  Trace plots show the thinned history of the chain or Markov chain,
  with its value in the y-axis moving by thinned sample across the x-axis.
  A chain or Markov chain with good properties does not suggest a trend
  upward or downward as it progresses across the x-axis (it should
  appear stationary), and it should mix well, meaning it should appear
  as though random samples are being taken each time from the same
  target distribution. Visual inspection of a trace plot cannot
  verify convergence, but apparent non-stationarity or poor mixing can
  certainly suggest non-convergence. A red, smoothed line also appears
  to aid visual inspection.

  Kernel density plots depict the marginal posterior distribution.
  Although there is no distributional assumption about this density,
  kernel density estimation uses Gaussian basis functions.

  Autocorrelation plots show the autocorrelation or serial correlation
  between values of thinned samples at nearby thinned samples. Samples with
  autocorrelation do not violate any assumption, but are inefficient
  because they reduce the effective sample size (\code{\link{ESS}}), and
  indicate that the chain is not mixing well, since each value is
  influenced by values that are previous and nearby. The x-axis
  indicates lags with respect to thinned samples, and the y-axis
  represents autocorrelation. The ideal autocorrelation plot shows
  perfect correlation at zero lag, and quickly falls to zero
  autocorrelation for all other lags.

  If an adaptive algorithm was used, then the distribution of absolute
  differences in the proposal variances, or the value of epsilon, is
  plotted across adaptations. The proposal variance, or epsilon, should
  change less as the adaptive algorithm approaches the target
  distributions. The absolute differences in the proposal variance plot
  should approach zero. This is called the condition of diminishing
  adaptation. If it is not approaching zero, then consider using a
  different adaptive MCMC algorithm. The following quantiles are plotted
  for absolute changes proposal variance: 0.025, 0.500, and 0.975.
}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{burnin}},
  \code{\link{ESS}},
  \code{\link{LaplacesDemon}}, and
  \code{\link{LaplacesDemon.hpc}}.}
\examples{### See the LaplacesDemon function for an example.}
\keyword{Plot}

\name{Precision}
\alias{Cov2Prec}
\alias{Prec2Cov}
\alias{prec2sd}
\alias{prec2var}
\alias{sd2prec}
\alias{sd2var}
\alias{var2prec}
\alias{var2sd}
\title{Precision}
\description{
  Bayesians often use precision rather than variance. These are elementary
  utility functions to facilitate conversions between precision,
  standard deviation, and variance regarding scalars, vectors, and
  matrices, and these functions are designed for those who are new to
  Bayesian inference. The names of these functions consist of two
  different scale parameters, separated by a `2', and capital letters
  refer to matrices while lower case letters refer to scalars and
  vectors. For example, the \code{Prec2Cov} function converts a
  precision matrix to a covariance matrix, while the \code{prec2sd}
  function converts a scalar or vector of precision parameters to
  standard deviation parameters.

  The modern Bayesian use of precision developed because it was more
  straightforward in a normal distribution to estimate precision
  \eqn{\tau}{tau} with a gamma distribution as a conjugate prior, than
  to estimate \eqn{\sigma^2}{sigma^2} with an inverse-gamma distribution
  as a conjugate prior. Today, conjugacy is usually considered to be
  merely a convenience, and in this example, a non-conjugate half-Cauchy
  prior distribution is recommended as a weakly informative prior
  distribution for scale parameters.
}
\usage{
Cov2Prec(Cov)
Prec2Cov(Prec)
prec2sd(prec=1)
prec2var(prec=1)
sd2prec(sd=1)
sd2var(sd=1)
var2prec(var=1)
var2sd(var=1)
}
\arguments{
  \item{Cov}{This is a covariance matrix, usually represented as
    \eqn{\Sigma}{Sigma}.}
  \item{Prec}{This is a precision matrix, usually represented as
    \eqn{\Omega}{Omega}.}
  \item{prec}{This is a precision scalar or vector, usually represented as
    \eqn{\tau}{tau}.}
  \item{sd}{This is a standard deviation scalar or vector, usually
    represented as \eqn{\sigma}{sigma}.}
  \item{var}{This is a variance scalar or vector, usually
    represented as \eqn{\sigma^2}{sigma^2}.}
}
\details{Bayesians often use precision rather than variance, where
  precision is the inverse of the variance. For example, a linear
  regression may be represented equivalently as \eqn{\textbf{y} \sim
    \mathcal{N}(\mu, \sigma^2)}{y ~ N(mu, sigma^2)}, or \eqn{\textbf{y}
    \sim \mathcal{N}(\mu, \tau^{-1})}{y ~ N(mu, tau^(-1))}, where
  \eqn{\sigma^2}{sigma^2} is the variance, and \eqn{\tau}{tau} is the
  precision, which is the inverse of the variance.}
\value{
  \item{Cov2Prec}{
    This returns a precision matrix, \eqn{\Omega}{Omega}, from a
    covariance matrix, \eqn{\Sigma}{Sigma}, where \eqn{\Omega =
    \Sigma^{-1}}{Omega = Sigma^(-1)}.}
  \item{Prec2Cov}{
    This returns a covariance matrix, \eqn{\Sigma}{Sigma}, from a
    precision matrix, \eqn{\Omega}{Omega}, where \eqn{\Sigma =
    \Omega^{-1}}{Sigma = Omega^(-1)}.}
  \item{prec2sd}{
    This returns a standard deviation, \eqn{\sigma}{sigma}, from a
    precision, \eqn{\tau}{tau}, where \eqn{\sigma =
    \sqrt{\tau^{-1}}}{sigma = sqrt(tau^(-1))}.}
  \item{prec2var}{
    This returns a variance, \eqn{\sigma^2}{sigma^2}, from a precision,
    \eqn{\tau}{tau}, where \eqn{\sigma^2 = \tau^{-1}}{sigma^2 =
    tau^(-1)}.}
  \item{sd2prec}{
    This returns a precision, \eqn{\tau}{tau}, from a standard
    deviation, \eqn{\sigma}{sigma}, where \eqn{\tau = \sigma^{-2}}{tau =
    sigma^(-2)}.}
  \item{sd2var}{
    This returns a variance, \eqn{\sigma^2}{sigma^2}, from a standard
    deviation, \eqn{\sigma}{sigma}, where \eqn{\sigma^2 = \sigma
    \sigma}{sigma^2 = sigma x sigma}.}
  \item{var2prec}{
    This returns a precision, \eqn{\tau}{tau}, from a variance,
    \eqn{\sigma^2}{sigma^2}, where \eqn{\tau = \frac{1}{\sigma^2}}{tau = 1
    / sigma^2}.}
  \item{var2sd}{
    This returns a standard deviation, \eqn{\sigma}{sigma}, from a
    variance, \eqn{\sigma^2}{sigma^2}, where \eqn{\sigma =
    \sqrt{\sigma^2}}{sigma = sqrt(sigma^2)}.}
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{\code{\link{Cov2Cor}}}
\examples{
library(LaplacesDemon)
Cov2Prec(matrix(c(1,0.1,0.1,1),2,2))
Prec2Cov(matrix(c(1,0.1,0.1,1),2,2))
prec2sd(0.5)
prec2var(0.5)
sd2prec(1.4142)
sd2var(01.4142)
var2prec(2)
var2sd(2)
}
\keyword{Utility}

\name{Matrices}
\alias{as.indicator.matrix}
\alias{as.inverse}
\alias{as.parm.matrix}
\alias{as.positive.definite}
\alias{as.positive.semidefinite}
\alias{as.symmetric.matrix}
\alias{Cov2Cor}
\alias{CovEstim}
\alias{GaussHermiteCubeRule}
\alias{Hessian}
\alias{is.positive.definite}
\alias{is.positive.semidefinite}
\alias{is.square.matrix}
\alias{is.symmetric.matrix}
\alias{Jacobian}
\alias{logdet}
\alias{lower.triangle}
\alias{read.matrix}
\alias{SparseGrid}
\alias{TransitionMatrix}
\alias{tr}
\alias{upper.triangle}
\title{Matrix Utility Functions}
\description{
  These are utility functions for working with matrices.
}
\usage{
as.indicator.matrix(x)
as.inverse(x)
as.parm.matrix(x, k, parm, Data, a=-Inf, b=Inf, restrict=FALSE, chol=FALSE)
as.positive.definite(x)
as.positive.semidefinite(x)
as.symmetric.matrix(x, k=NULL)
is.positive.definite(x)
is.positive.semidefinite(x)
is.square.matrix(x)
is.symmetric.matrix(x)
Cov2Cor(Sigma)
CovEstim(Model, parm, Data, Method="Hessian")
GaussHermiteCubeRule(N, dims, rule)
Hessian(Model, parm, Data, Interval=1e-6, Method="Richardson")
Jacobian(Model, parm, Data, Interval=1e-6, Method="simple")
logdet(x)
lower.triangle(x, diag=FALSE)
read.matrix(file, header=FALSE, sep=",", nrow=0, samples=0, size=0, na.rm=FALSE)
SparseGrid(J, K)
TransitionMatrix(theta.y=NULL, y.theta=NULL, p.theta=NULL)
tr(x)
upper.triangle(x, diag=FALSE)
}
\arguments{
  \item{N}{This required argument accepts a positive integer that
    indicates the number of nodes.}
  \item{x}{This is a matrix (though \code{as.symmetric.matrix} also
    accepts vectors).}
  \item{J}{This required argument indicates the dimension of the
    integral and accepts a positive integer.}
  \item{k}{For \code{as.parm.matrix}, this is a required argument,
    indicating the dimension of the matrix. For
    \code{as.symmetric.matrix}, this is an optional argument that
    specifies the dimension of the symmetric matrix. This applies
    only when \code{x} is a vector. It defaults to \code{NULL},
    in which case it calculates \code{k <- (-1 + sqrt(1 + 8 *
      length(x)))/ 2}.}
  \item{K}{This required argument indicates the accuracy and accepts a
    positive integer. Larger values result in many more integration
    nodes.}
  \item{diag}{Logical. If \code{TRUE}, then the elements in the main
    diagonal are also returned.}
  \item{dims}{This required argument indicates the dimension of the
    integral and accepts a positive integer.}
  \item{Sigma}{This is a covariance matrix, \eqn{\Sigma}{Sigma}, and may
    be entered either as a matrix or vector.}
  \item{Model}{This is a model specification function. For more
    information, see \code{\link{LaplacesDemon}}.}
  \item{parm}{This is a vector of parameters passed to the model
    specification.}
  \item{Data}{This is the list of data passed to the model
    specification. For more information, see \code{\link{LaplacesDemon}}.}
  \item{a,b}{These optional arguments allow the elements of \code{x} to
    be bound to the interval \eqn{[a,b]}. For example, elements of a
    correlation matrix are in the interval \eqn{[-1,1]}.}
  \item{restrict}{Logical. If \code{TRUE}, then \code{x[1,1]} is
    restricted to 1. This is useful in multinomial probit, for
    example. The variable, \code{LaplacesDemonMatrix}, is
    created in a new environment, \code{LDEnv} so \code{as.parm.matrix}
    can keep track of changes from iteration to iteration.}
  \item{rule}{This is an optional argument that accepts a univariate
    Gauss-Hermite quadrature rule. Usually, this argument is left empty.
    A rule may be supplied that differs from the traditional rule, such
    as when constraints have been observed, and one or more nodes and
    weights were adjusted.}
  \item{chol}{Logical. If \code{TRUE}, then x is an upper-triangular
    matrix.}
  \item{file}{This is the name of the file from which the numeric data
    matrix will be imported or read.}
  \item{header}{Logical. When \code{TRUE}, the first row of the file
    must contain names of the columns, and will be converted to the
    column names of the numeric matrix. When \code{FALSE}, the first row
    of the file contains data, not column names.}
  \item{Interval}{This accepts a small scalar number for precision.}
  \item{Method}{This accepts a quoted string. For \code{Hessian}, it
    defaults to \code{Method="Richardson"}, which uses Richardson
    extrapolation. For \code{Jacobian}, it defaults to
    \code{Method="simple"}, which uses finite-differencing. Richardson
    Richardson extrapolation is more accurate, but slower to calculate.
    Since error due to finite-differencing propagates through to higher
    derivatives, finite-differencing should not be used when
    approximating a Hessian matrix. Another method called automatic
    differentiation is not currently available here, but should be more
    accurate, though even slower to calculate. Another popular
    alternative is to use the \code{\link{BayesianBootstrap}} on the
    data. For \code{CovEstim}, this accepts \code{Method="Hessian"},
    \code{Method="Identity"} (which simply assigns an identity matrix),
    \code{Method="OPG"} (which calculates the sum of outer products of
    record-level gradients), or \code{Method="Sandwich"}, which is the
    sandwich estimator and combines the Hessian and OPG estimates.}
  \item{nrow}{This is the number of rows of the numeric matrix, and
    defaults to \code{nrow=0}. If the number is known, the function will
    perform noticeably faster when it does not have to check.}
  \item{p.theta}{This accepts a matrix of prior probabilities for a
    transition matrix, and defaults to \code{NULL}. If used, each row
    must sum to 1.}
  \item{samples}{This is the number of samples to take from the numeric
    matrix. When \code{samples=0}, sampling is not performed and the
    entire matrix is returned.}
  \item{sep}{This argument indicates a character with which it will
    separate fields when creating column vectors. For example, a
    read a comma-separated file (.csv), use \code{sep=","}.}
  \item{size}{This is the batch size to be used only when reading a
    numeric matrix that is larger than the available computer memory
    (RAM), and only when \code{samples} is greater than zero. Sampling
    of a big data matrix is performed by first determining the records
    to keep, and then reading batches, one by one, and keeping the
    matching records.}
  \item{theta.y}{This accepts a vector of posterior samples of a
    discrete Markov chain, and defaults to \code{NULL}. If used, the
    order of the samples affects the transition probability.}
  \item{na.rm}{Logical. When \code{TRUE}, rows with missing values are
    removed from the matrix after it is read. Rather than removing
    missing values, the user may consider imputing missing values inside
    the model, or before the model with the \code{\link{MISS}}
    function. Examples of within-model imputation may be found in the
    accompanying "Examples" vignette.}
  \item{y.theta}{This accepts a vector of data that are samples of a
    discrete distribution, and defaults to \code{NULL}. If used, the
    order of the samples affects the transition probability.}
}
\details{
  The \code{as.indicator.matrix} function creates an indicator matrix
  from a vector. This function is useful for converting a discrete
  vector into a matrix in which each column represents one of the
  discrete values, and each occurence of that value in the related
  column is indicated by a one, and is otherwise filled with
  zeroes. This function is similar to the \code{class.ind} function in
  the nnet package.

  The \code{as.inverse} function returns the matrix inverse of
  \code{x}. The \code{solve} function in base R also returns the matrix
  inverse, but \code{solve} can return a matrix that is not symmetric,
  and can fail due to singularities. The \code{as.inverse} function
  tries to use the \code{solve} function to return a matrix inverse, and
  when it fails due to a singularity, \code{as.inverse} uses eigenvalue
  decomposition (in which eigenvalues below a tolerance are replaced
  with the tolerance), and coerces the result to a symmetric
  matrix. This is similar to the \code{solvcov} function in the fpc
  package.

  The \code{as.parm.matrix} function prepares a correlation, covariance,
  or precision matrix in two important ways. First,
  \code{as.parm.matrix} obtains the parameters for the matrix specified
  in the \code{x} argument by matching the name of the matrix in the
  \code{x} argument with any parameters in \code{parm}, given the
  parameter names in the \code{Data} listed in \code{parm.names}. These
  obtained parameters are organized into a matrix as the elements of the
  upper-triangular, including the diagonal. A copy is made, without the
  diagonal, and the lower-triangular is filled in, completing the
  matrix. Second, \code{as.parm.matrix} checks for
  positive-definiteness. If matrix \code{x} is positive-definite, then
  the matrix is stored as a variable called \code{LaplacesDemonMatrix}
  in a new environment called \code{LDEnv}. If matrix \code{x} is not
  positive-definite, then \code{LaplacesDemonMatrix} in \code{LDEnv} is
  sought as a replacement. If this variable exists, then it is used to
  replace the matrix. If not, then the matrix is replaced with an
  identity matrix. Back in the model specification, after using
  \code{as.parm.matrix}, it is recommended that the user also pass the
  resulting matrix back into the \code{parm} vector, so the sampler or
  algorithm knows that the elements of the matrix have changed.

  The \code{as.positive.definite} function returns the nearest
  positive-definite matrix for a matrix that is square and symmetric
  (Higham, 2002). This version is intended only for covariance and
  precision matrices, and has been optimized for speed. A more
  extensible function is \code{nearPD} in the matrixcalc package, which
  is also able to work with correlation matrices, and matrices that are
  asymmetric.

  The \code{as.positive.semidefinite} function iteratively seeks to
  return a square, symmetric matrix that is at least
  positive-semidefinite, by replacing each negative eigenvalue and
  calculating its projection. This is intended only for covariance and
  precision matrices. A similar function is \code{makePsd} in the RTAQ
  package, though it is not iterative, and returns matrices that fail a
  logical check with \code{is.positive.semidefinite}.

  The \code{as.symmetric.matrix} function accepts either a vector or
  matrix, and returns a symmetric matrix. In the case of a vector, it
  can be either all elements of the matrix, or the lower triangular. In
  the case of a \code{x} being entered as a matrix, this function
  tolerates non-finite values in one triangle (say, the lower), as long
  as the corresponding element is finite in the other (say, the upper)
  triangle.

  The \code{Cov2Cor} function converts a covariance matrix into a
  correlation matrix, and accepts the covariance matrix either in matrix
  or vector form. This function may be useful inside a model
  specification and also with converting posterior draws of the elements
  of a covariance matrix to a correlation matrix. \code{Cov2Cor} is an
  expanded form of the \code{cov2cor} function in the \code{stats}
  package, where \code{Cov2Cor} is also able to accept and return a
  vectorized matrix.

  The \code{CovEstim} function estimates a covariance matrix with one of
  several methods. This is mainly used by
  \code{\link{LaplaceApproximation}}, where the \code{parm} argument
  receives the posterior modes. See the \code{CovEst} argument for
  more details.

  The \code{GaussHermiteCubeRule} function returns a matrix of nodes and
  a vector of weights for a \code{dims}-dimensional integral given
  \eqn{N} univariate nodes. The number of multivariate nodes will differ
  from the number of univariate nodes. This function is for use with
  multivariate quadrature, often called cubature. This has been adapted
  from the \code{multiquad} function in the NominalLogisticBiplot
  package. The \code{\link{GaussHermiteQuadRule}} function is a
  univariate version. A customized univariate \code{rule} may be
  supplied when constraints necessitate that one or more nodes and
  weights had to be altered.

  The \code{Hessian} returns a symmetric, Hessian matrix, which is a
  matrix of second partial derivatives. The estimation of the Hessian
  matrix is approximated numerically using Richardson extrapolation by
  default. This is a slow function. This function is not intended to be
  called by the user, but is made available here. This is essentially
  the \code{hessian} function from the numDeriv package, adapted to
  Laplace's Demon.
  
  The \code{is.positive.definite} function is a logical test of whether
  or not a matrix is positive-definite. A \eqn{k \times k}{k x k}
  symmetric matrix \eqn{\textbf{X}}{X} is positive-definite if all of
  its eigenvalues are positive (\eqn{\lambda_i > 0, i \in k}{lambda[i] >
  0, i in k}). All main-diagonal elements must be positive. The
  determinant of a positive-definite matrix is always positive, so a
  positive-definite matrix is always nonsingular. Non-symmetric,
  positive-definite matrices exist, but are not considered here.

  The \code{is.positive.semidefinite} function is a logical test of
  whether or not a matrix is positive-semidefinite. A \eqn{k x k}
  symmetric matrix \eqn{\textbf{X}}{X} is positive-semidefinite if all
  of its eigenvalues are non-negative (\eqn{\lambda_i \ge 0, i \in
  k}{lambda[i] >= 0, i ink}).

  The \code{is.square.matrix} function is a logical test of whether or
  not a matrix is square. A square matrix is a matrix with the same
  number of rows and columns, and is usually represented as a \eqn{k
  \times k}{k x k} matrix \eqn{\textbf{X}}{X}.

  The \code{is.symmetric.matrix} function is a logical test of whether
  or not a matrix is symmetric. A symmetric matrix is a square matrix
  that is equal to its transpose, \eqn{\textbf{X} = \textbf{X}^T}{X =
  t(X)}. For example, where \eqn{i} indexes rows and \eqn{j} indexes
  columns, \eqn{\textbf{X}_{i,j} = \textbf{X}_{j,i}}{X[i,j] =
  X[j,i]}. This differs from the \code{isSymmetric} function in base R
  that is inexact, using \code{all.equal}.

  The \code{Jacobian} function estimates the Jacobian matrix, which is
  a matrix of all first-order partial derivatives of the \code{Model}.
  The Jacobian matrix is estimated by default with forward
  finite-differencing, or optionally with Richardson extrapolation. This
  function is not intended to be called by the user, but is made
  available here. This is essentially the \code{jacobian} function from
  the numDeriv package, adapted to LaplacesDemon.

  The \code{logdet} function returns the logarithm of the determinant of
  a positive-definite matrix via the Cholesky decomposition. The
  determinant is a value associated with a square matrix, and was used
  historically to \emph{determine} if a system of linear equations has a
  unique solution. The term \emph{determinant} was introduced by Gauss,
  where Laplace referred to it as the resultant. When the determinant is
  zero, the matrix is singular and non-invertible; there are either no
  solutions or many solutions. A unique solution exists when the
  determinant is non-zero. The \code{det} function in base R works well
  for small matrices, but can return erroneously return zero in larger
  matrices. It is better to work with the log-determinant.

  The \code{lower.triangle} function returns a vector of the lower
  triangular elements of a matrix, and the diagonal is included when
  \code{diag=TRUE}.

  The \code{read.matrix} function is provided here as one of many
  convenient ways to read a numeric matrix into R. The most common
  method of storing data in R is the data frame, because it is
  versatile. For example, a data frame may contain character, factor,
  and numeric variables together. For iterative estimation, common in
  Bayesian inference, the data frame is much slower than the numeric
  matrix. For this reason, the LaplacesDemon package does not use data
  frames, and has not traditionally accepted character or factor
  data. The \code{read.matrix} function returns either an entire numeric
  matrix, or row-wise samples from a numeric matrix. Samples may be
  taken from a matrix that is too large for available computer memory
  (RAM), such as with big data.

  The \code{SparseGrid} function returns a sparse grid for a
  \eqn{J}-dimensional integral with accuracy \eqn{K}, given
  Gauss-Hermite quadrature rules. A grid of order \eqn{K} provides an
  exact result for a polynomial of total order of \eqn{2K - 1} or less.
  \code{SparseGrid} returns a matrix of nodes and a vector of weights.
  A sparse grid is more efficient than the full grid in the
  \code{GaussHermiteCubeRule} function. This has been adapted from the
  SparseGrid package.

  The \code{TransitionMatrix} function has several uses. A user may
  supply a vector of marginal posterior samples of a discrete Markov
  chain as \code{theta.y}, and an observed posterior transition matrix
  is returned. Otherwise, a user may supply data (\code{y.theta}) and/or
  a prior (\code{p.theta}), in which case a posterior transition matrix
  is returned. A common row-wise prior is the dirichlet distribution.
  Transition probabilities are from row element to column element.

  The \code{tr} function returns the trace of a matrix. The trace of a
  matrix is the sum of the elements in the main diagonal of a square
  matrix. For example, the trace of a \eqn{k \times k}{k x k} matrix
  \eqn{\textbf{X}}{X}, is \eqn{\sum_{k=1} \textbf{X}_{k,k}}{sum(k=1)
    X[k,k]}.

  The \code{upper.triangle} function returns a vector of the lower
  triangular elements of a matrix, and the diagonal is included when
  \code{diag=TRUE}.
}
\references{
  Higham, N.J. (2002). "Computing the Nearest Correlation Matrix - a
  Problem from Finance". \emph{IMA Journal of Numerical Analysis}, 22,
  p. 329--343.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{BayesianBootstrap}},
  \code{\link{Cov2Prec}},
  \code{\link{cov2cor}},
  \code{\link{ddirichlet}},
  \code{\link{GaussHermiteQuadRule}},
  \code{\link{isSymmetric}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{lower.tri}},
  \code{\link{MISS}},
  \code{\link{Prec2Cov}},
  \code{\link{solve}}, and
  \code{\link{upper.tri}}.
}
\keyword{Array}
\keyword{High Performance Computing}
\keyword{Math}
\keyword{Matrix}
\keyword{Utility}

\name{joint.density.plot}
\alias{joint.density.plot}
\title{Joint Density Plot}
\description{
  This function plots the joint kernel density from samples of two
  marginal posterior distributions.
}
\usage{
joint.density.plot(x, y, Title=NULL, contour=TRUE, color=FALSE, Trace=NULL)
}
\arguments{
  \item{x,y}{These are vectors consisting of samples from two marginal
    posterior distributions, such as those output by
    \code{\link{LaplacesDemon}} in components \code{Posterior1} (all
    samples) or \code{Posterior2} (stationary samples).}
  \item{Title}{This is the title of the joint posterior density plot.}
  \item{contour}{This logical argument indicates whether or not contour
    lines will be added to the plot. \code{contour} defaults to
    \code{TRUE}.}
  \item{color}{This logical argument indicates whether or not color will
    be added to the plot. \code{color} defaults to \code{FALSE}.}
  \item{Trace}{This argument defaults to \code{NULL}, in which case it
    does not trace the exploration of the joint density. To trace the
    exploration of the joint density, specify \code{Trace} with the
    beginning and ending iteration or sample. For example, to view the
    trace of the first ten iterations or samples, specify
    \code{Trace=c(1,10)}.}
}
\details{
  This function produces either a bivariate scatterplot that may have
  kernel density contour lines added, or a bivariate plot with kernel 
  density-influenced colors, which may also have kernel density contour
  lines added. A joint density plot may be more informative than two
  univariate density plots.

  The \code{Trace} argument allows the user to view the exploration of
  the joint density, such as from MCMC chain output. An efficient
  algorithm jumps to random points of the joint density, and an
  inefficient algorithm explores more slowly. The initial point of the
  trace (which is the first element passed to \code{Trace}) is plotted
  with a green dot. The user should consider plotting the joint density of
  the two marginal posterior distributions with the highest
  \code{\link{IAT}}, as identified with the
  \code{\link{PosteriorChecks}} function, since these are the two least
  efficient MCMC chains. Different sequences of iterations may be
  plotted. This `joint trace plot' may show behavior of the MCMC
  algorithm to the user.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{\code{\link{IAT}},
  \code{\link{LaplacesDemon}}, and
  \code{\link{PosteriorChecks}}
}
\examples{
library(LaplacesDemon)
X <- rmvn(1000, runif(2), diag(2))
joint.density.plot(X[,1], X[,2], Title="Joint Density Plot",
     contour=TRUE, color=FALSE)
joint.density.plot(X[,1], X[,2], Title="Joint Density Plot",
     contour=FALSE, color=TRUE)
joint.density.plot(X[,1], X[,2], Title="Joint Density Plot",
     contour=TRUE, color=TRUE)
joint.density.plot(X[,1], X[,2], Title="Joint Trace Plot",
     contour=FALSE, color=TRUE, Trace=c(1,10))
}
\keyword{Plot}

\name{IterativeQuadrature}
\alias{IterativeQuadrature}
\title{Iterative Quadrature}
\description{
  The \code{IterativeQuadrature} function iteratively approximates the
  first two moments of marginal posterior distributions of a Bayesian
  model with deterministic integration.
}
\usage{
IterativeQuadrature(Model, parm, Data, Covar=NULL, Iterations=100,
     Algorithm="CAGH", Specs=NULL, Samples=1000, sir=TRUE,
     Stop.Tolerance=c(1e-5,1e-15), CPUs=1, Type="PSOCK")
}
\arguments{
  \item{Model}{This required argument receives the model from a
    user-defined function. The user-defined function is where the model
    is specified. \code{IterativeQuadrature} passes two arguments to
    the model function, \code{parms} and \code{Data}. For more
    information, see the \code{\link{LaplacesDemon}} function and
    ``LaplacesDemon Tutorial'' vignette.}
  \item{parm}{This argument requires a vector of initial values equal in
    length to the number of parameters. \code{IterativeQuadrature} will
    attempt to approximate these initial values for the parameters as
    means (or posterior modes) of normal integrals. The
    \code{\link{GIV}} function may be used to randomly generate initial
    values. Parameters must be continuous.}
  \item{Data}{This required argument accepts a list of data. The list of
    data must include \code{mon.names} which contains monitored variable
    names, and \code{parm.names} which contains parameter
    names.}
  \item{Covar}{This argument accepts a \eqn{J \times J}{J x J}
    covariance matrix for \eqn{J} initial values. When a covariance
    matrix is not supplied, a scaled identity matrix is used.}
  \item{Iterations}{This argument accepts an integer that determines the
    number of iterations that \code{IterativeQuadrature} will attempt
    to approximate the posterior with normal
    integrals. \code{Iterations} defaults to 100.
    \code{IterativeQuadrature} will stop before this number of
    iterations if the tolerance is less than or equal to the
    \code{Stop.Tolerance} criterion. The required amount of computer
    memory increases with \code{Iterations}. If computer memory is
    exceeded, then all will be lost.}
  \item{Algorithm}{This optional argument accepts a quoted string that
    specifies the iterative quadrature algorithm. The default
    method is \code{Method="CAGH"}. Options include \code{"AGHSG"} for
    Adaptive Gauss-Hermite Sparse Grid, and \code{"CAGH"} for
    Componentwise Adaptive Gaussian-Hermite.}
  \item{Specs}{This argument accepts a list of specifications for an
    algorithm.}
  \item{Samples}{This argument indicates the number of posterior samples
    to be taken with sampling importance resampling via the
    \code{\link{SIR}} function, which occurs only when
    \code{sir=TRUE}. Note that the number of samples should increase
    with the number and intercorrelations of the parameters.}
  \item{sir}{This logical argument indicates whether or not Sampling
    Importance Resampling (SIR) is conducted via the \code{\link{SIR}}
    function to draw independent posterior samples. This argument
    defaults to \code{TRUE}. Even when \code{TRUE}, posterior samples
    are drawn only when \code{IterativeQuadrature} has
    converged. Posterior samples are required for many other functions,
    including \code{plot.iterquad} and \code{predict.iterquad}. Less
    time can be spent on sampling by increasing \code{CPUs}, if
    available, which parallelizes the sampling.}
  \item{Stop.Tolerance}{This argument accepts a vector of two positive
    numbers, and defaults to \code{1e-5,1e-15}. Tolerance is calculated
    each iteration, and the criteria varies by algorithm. The algorithm
    is considered to have converged to the user-specified
    \code{Stop.Tolerance} when the tolerance is less than or equal to
    the value of \code{Stop.Tolerance}, and the algorithm terminates at
    the end of the current iteration. Unless stated otherwise, the
    first element is the stop tolerance for the change in \eqn{\mu},
    the second element is the stop tolerance for the change in mean
    integration error, and the first tolerance must be met before the
    second tolerance is considered.}
  \item{CPUs}{This argument accepts an integer that specifies the number
    of central processing units (CPUs) of the multicore computer or
    computer cluster. This argument defaults to \code{CPUs=1}, in which
    parallel processing does not occur. When multiple CPUs are
    specified, model function evaluations are parallelized across the
    nodes, and sampling with \code{\link{SIR}} is parallelized when
    \code{sir=TRUE}.}
  \item{Type}{This argument specifies the type of parallel processing to
    perform, accepting either \code{Type="PSOCK"} or
    \code{Type="MPI"}.}
}
\details{
  Quadrature is a historical term in mathematics that means determining
  area. Mathematicians of ancient Greece, according to the Pythagorean
  doctrine, understood determination of area of a figure as the process
  of geometrically constructing a square having the same area
  (squaring). Thus the name quadrature for this process.

  In medieval Europe, quadrature meant the calculation of area by any
  method. With the invention of integral calculus, quadrature has been
  applied to the computation of a univariate definite
  integral. Numerical integration is a broad family of algorithms for
  calculating the numerical value of a definite integral. Numerical
  quadrature is a synonym for quadrature applied to one-dimensional
  integrals. Multivariate quadrature, also called cubature, is the
  application of quadrature to multidimensional integrals.

  A quadrature rule is an approximation of the definite integral of a
  function, usually stated as a weighted sum of function values at
  specified points within the domain of integration. The specified
  points are referred to as abscissae, abscissas, integration points, or
  nodes, and have associated weights. The calculation of the nodes and
  weights of the quadrature rule differs by the type of
  quadrature. There are numerous types of quadrature
  algorithms. Bayesian forms of quadrature usually use Gauss-Hermite
  quadrature (Naylor and Smith, 1982), and placing a Gaussian Process on
  the function is a common extension (O'Hagan, 1991; Rasmussen and
  Ghahramani, 2003) that is called `Bayesian Quadrature'. Often, these
  and other forms of quadrature are also referred to as model-based
  integration.

  Gauss-Hermite quadrature uses Hermite polynomials to calculate the
  rule. However, there are two versions of Hermite polynomials, which
  result in different kernels in different fields. In physics, the
  kernel is \code{exp(-x^2)}, while in probability the kernel is
  \code{exp(-x^2/2)}. The weights are a normal density. If the
  parameters of the normal distribution, \eqn{\mu} and
  \eqn{\sigma^2}{sigma^2}, are estimated from data, then it is referred
  to as adaptive Gauss-Hermite quadrature, and the parameters are the
  conditional mean and conditional variance. Outside of Gauss-Hermite
  quadrature, adaptive quadrature implies that a difficult range in the
  integrand is subdivided with more points until it is
  well-approximated. Gauss-Hermite quadrature performs well when the
  integrand is smooth, and assumes normality or multivariate normality.
  Adaptive Gauss-Hermite quadrature has been demonstrated to outperform
  Gauss-Hermite quadrature in speed and accuracy.

  A goal in quadrature is to minimize integration error, which is the
  error between the evaluations and the weights of the rule. Therefore,
  a goal in Bayesian Gauss-Hermite quadrature is to minimize integration
  error while approximating a marginal posterior distribution that is
  assumed to be smooth and normally-distributed. This minimization often
  occurs by increasing the number of nodes until a change in mean
  integration error is below a tolerance, rather than minimizing
  integration error itself, since the target may be only approximately
  normally distributed, or minimizing the sum of integration error,
  which would change with the number of nodes.

  To approximate integrals in multiple dimensions, one approach applies
  \eqn{N} nodes of a univariate quadrature rule to multiple dimensions
  (using the \code{\link{GaussHermiteCubeRule}} function for example)
  via the product rule, which results in many more multivariate nodes.
  This requires the number of function evaluations to grow exponentially
  as dimension increases. Multidimensional quadrature is usually limited
  to less than ten dimensions, both due to the number of nodes required,
  and because the accuracy of multidimensional quadrature algorithms
  decreases as the dimension increases. Three methods may overcome this
  curse of dimensionality in varying degrees: componentwise quadrature,
  sparse grids, and Monte Carlo.

  Componentwise quadrature is the iterative application of univariate
  quadrature to each parameter. It is applicable with high-dimensional
  models, but sacrifices the ability to calculate the conditional
  covariance matrix, and calculates only the variance of each parameter.

  Sparse grids were originally developed by Smolyak for
  multidimensional quadrature. A sparse grid is based on a
  one-dimensional quadrature rule. Only a subset of the nodes from the
  product rule is included, and the weights are appropriately rescaled.
  Although a sparse grid is more efficient because it reduces the
  number of nodes to achieve the same accuracy, the user must contend
  with increasing the accuracy of the grid, and it remains inapplicable
  to high-dimensional integrals.

  Monte Carlo is a large family of sampling-based algorithms. O'Hagan
  (1987) asserts that Monte Carlo is frequentist, inefficient, regards
  irrelevant information, and disregards relevant information.
  Quadrature, he maintains (O'Hagan, 1992), is the most Bayesian
  approach, and also the most efficient. In high dimensions, he
  concedes, a popular subset of Monte Carlo algorithms is currently the
  best for cheap model function evaluations. These algorithms are called
  Markov chain Monte Carlo (MCMC). High-dimensional models with
  expensive model evaluation functions, however, are not well-suited to
  MCMC. A large number of MCMC algorithms is available in the
  \code{\link{LaplacesDemon}} function.

  Following are some reasons to consider iterative quadrature rather
  than MCMC. Once an MCMC sampler finds equilibrium, it must then draw
  enough samples to represent all targets. Iterative quadrature does not
  need to continue drawing samples. Multivariate quadrature is
  consistently reported as more efficient than MCMC when its assumptions
  hold, though multivariate quadrature is limited to small dimensions.
  High-dimensional models therefore default to MCMC, between the two.
  Componentwise quadrature algorithms like CAGH, however, may also be
  more efficient with cloc-time than MCMC in high dimensions, especially
  against componentwise MCMC algorithms. Another reason to consider
  iterative quadrature are that assessing convergence in MCMC is a
  difficult topic, but not for iterative quadrature. A user of
  iterative quadrature does not have to contend with effective sample
  size and autocorrelation, assessing stationarity, acceptance rates,
  diminishing adaptation, etc. Stochastic sampling in MCMC is less
  efficient when samples occur in close proximity (such as when highly
  autocorrelated), whereas in quadrature the nodes are spread out by
  design.

  In general, the conditional means and conditional variances progress
  smoothly to the target in multidimensional quadrature. For
  componentwise quadrature, movement to the target is not smooth, and
  often resembles a Markov chain or optimization algorithm.

  Iterative quadrature is often applied after
  \code{\link{LaplaceApproximation}} to obtain a more reliable
  estimate of parameter variance or covariance than the negative inverse
  of the \code{\link{Hessian}} matrix of second derivatives, which is
  suitable only when the contours of the logarithm of the unnormalized
  joint posterior density are approximately ellipsoidal (Naylor and
  Smith, 1982, p. 224).

  When \code{Algorithm="AGH"}, the Naylor and Smith (1982) algorithm
  is used. The AGH algorithm uses multivariate quadrature with the
  physicist's (not the probabilist's) kernel.

  There are four algorithm specifications: \code{N} is the number of
  univariate nodes, \code{Nmax} is the maximum number of univariate
  nodes, \code{Packages} accepts any package required for the model
  function when parallelized, and \code{Dyn.libs} accepts dynamic
  libraries for parallelization, if required. The number of univariate
  nodes begins at \eqn{N} and increases by one each iteration. The
  number of multivariate nodes grows quickly with \eqn{N}. Naylor and
  Smith (1982) recommend beginning with as few nodes as \eqn{N=3}. Any
  of the following events will cause \eqn{N} to increase by 1 when
  \eqn{N} is less than \code{Nmax}:

  \itemize{
    \item All LP weights are zero (and non-finite weights are set to zero)
    \item \eqn{\mu} does not result in an increase in LP
    \item All elements in \eqn{\Sigma} are not finite
    \item The square root of the sum of the squared changes in \eqn{\mu}
    is less than or equal to the \code{Stop.Tolerance}
  }

  Tolerance includes two metrics: change in mean integration error and
  change in parameters. Including the change in parameters for tolerance
  was not mentioned in Naylor and Smith (1982).

  Naylor and Smith (1982) consider a transformation due to correlation.
  This is not included here.

  The AGH algorithm does not currently handle constrained parameters,
  such as with the \code{\link{interval}} function. If a parameter is
  constrained and changes during a model evaluation, this changes the
  node and the multivariate weight. This is currently not corrected.

  An advantage of AGH over componentwise adaptive quadrature is that
  AGH estimates covariance, where a componentwise algorithm ignores
  it. A disadvantage of AGH over a componentwise algorithm is that
  the number of nodes increases so quickly with dimension, that AGH is
  limited to small-dimensional models.

  When \code{Algorithm="AGHSG"}, the Naylor and Smith (1982) algorithm
  is applied to a sparse grid, rather than a traditional multivariate
  quadrature rule. This is identical to the AGH algorithm above, except
  that a sparse grid replaces the multivariate quadrature rule.

  The sparse grid reduces the number of nodes. The cost of reducing the
  number of nodes is that the user must consider the accuracy, \eqn{K}.

  There are four algorithm specifications: \code{K} is the accuracy (as a
  positive integer), \code{Kmax} is the maximum accuracy,
  \code{Packages} accepts any package required for the model function
  when parallelized, and \code{Dyn.libs} accepts dynamic libraries for
  parallelization, if required. These arguments represent accuracy
  rather than the number of univariate nodes, but otherwise are similar
  to the AGH algorithm.

  When \code{Algorithm="CAGH"}, a componentwise version of the adaptive
  Gauss-Hermite quadrature of Naylor and Smith (1982) is used. Each
  iteration, each marginal posterior distribution is approximated
  sequentially, in a random order, with univariate quadrature. The
  conditional mean and conditional variance are also approximated each
  iteration, making it an adaptive algorithm.

  There are four algorithm specifications: \code{N} is the number of
  nodes, \code{Nmax} is the maximum number of nodes, \code{Packages}
  accepts any package required for the model function when parallelized,
  and \code{Dyn.libs} accepts dynamic libraries for parallelization, if
  required. The number of nodes begins at \eqn{N}. All parameters have
  the same number of nodes. Any of the following events will cause
  \eqn{N} to increase by 1 when \eqn{N} is less than \code{Nmax}, and
  these conditions refer to all parameters (not individually):

  \itemize{
    \item Any LP weights are not finite
    \item All LP weights are zero
    \item \eqn{\mu} does not result in an increase in LP
    \item The square root of the sum of the squared changes in \eqn{\mu}
    is less than or equal to the \code{Stop.Tolerance}
  }

  It is recommended to begin with \code{N=3} and set \code{Nmax} between
  10 and 100. As long as CAGH does not experience problematic weights,
  and as long as CAGH is improving LP with \eqn{\mu}, the number of nodes
  does not increase. When CAGH becomes either universally problematic or
  universally stable, then \eqn{N} slowly increases until the sum of
  both the mean integration error and the sum of the squared changes in
  \eqn{\mu} is less than the \code{Stop.Tolerance} for two consecutive
  iterations.

  If the highest LP occurs at the lowest or highest node, then the value
  at that node becomes the conditional mean, rather than calculating it
  from all weighted samples; this facilitates movement when the current
  integral is poorly centered toward a well-centered integral. If all
  weights are zero, then a random proposal is generated with a small
  variance.

  Tolerance includes two metrics: change in mean integration error and
  change in parameters, as the square root of the sum of the squared
  differences.

  When a parameter constraint is encountered, the node and weight of the
  quadrature rule is recalculated.

  An advantage of CAGH over multidimensional adaptive quadrature is that
  CAGH may be applied in large dimensions. Disadvantages of CAGH are
  that only variance, not covariance, is estimated, and ignoring
  covariance may be problematic.
}
\value{
  \code{IterativeQuadrature} returns an object of class \code{iterquad}
  that is a list with the following components:
  \item{Algorithm}{This is the name of the iterative quadrature
    algorithm.}
  \item{Call}{This is the matched call of \code{IterativeQuadrature}.}
  \item{Converged}{This is a logical indicator of whether or not
    \code{IterativeQuadrature} converged within the specified
    \code{Iterations} according to the supplied \code{Stop.Tolerance}
    criterion. Convergence does not indicate that the global maximum has
    been found, but only that the tolerance was less than or equal to
    the \code{Stop.Tolerance} criteria.}
  \item{Covar}{This is the estimated covariance matrix. The \code{Covar}
    matrix may be scaled and input into the \code{Covar} argument of the
    \code{\link{LaplacesDemon}} or \code{\link{PMC}} function for
    further estimation. To scale this matrix for use with Laplace's
    Demon or PMC, multiply it by \eqn{2.38^2/d}, where \eqn{d} is the
    number of initial values.}
  \item{Deviance}{This is a vector of the iterative history of the
    deviance in the \code{IterativeQuadrature} function, as it sought
    convergence.}
  \item{History}{This is a matrix of the iterative history of the
    parameters in the \code{IterativeQuadrature} function, as it sought
    convergence.}
  \item{Initial.Values}{This is the vector of initial values that was
    originally given to \code{IterativeQuadrature} in the \code{parm}
    argument.}
  \item{LML}{This is an approximation of the logarithm of the marginal
    likelihood of the data (see the \code{\link{LML}} function for more
    information). When the model has converged and \code{sir=TRUE}, the
    NSIS method is used. When the model has converged and
    \code{sir=FALSE}, the LME method is used. This is the
    logarithmic form of equation 4 in Lewis and Raftery (1997). As a
    rough estimate of Kass and Raftery (1995), the LME-based LML is
    worrisome when the sample size of the data is less than five times
    the number of parameters, and \code{LML} should be adequate in most
    problems when the sample size of the data exceeds twenty times the
    number of parameters (p. 778). The LME is inappropriate with
    hierarchical models. However \code{LML} is estimated, it is useful
    for comparing multiple models with the \code{BayesFactor} function.}
  \item{LP.Final}{This reports the final scalar value for the logarithm
    of the unnormalized joint posterior density.}
  \item{LP.Initial}{This reports the initial scalar value for the
    logarithm of the unnormalized joint posterior density.}
  \item{LPw}{This is the latest matrix of the logarithm of the
    unnormalized joint posterior density. It is weighted and normalized
    so that each column sums to one.}
  \item{M}{This is the final \eqn{N \times J}{N x J} matrix of
    quadrature weights that have been corrected for non-standard normal
    distributions, where \eqn{N} is the number of nodes and \eqn{J}
    is the number of parameters.}
  \item{Minutes}{This is the number of minutes that
    \code{IterativeQuadrature} was running, and this includes the
    initial checks as well as drawing posterior samples and creating
    summaries.}
  \item{Monitor}{When \code{sir=TRUE}, a number of independent
    posterior samples equal to \code{Samples} is taken, and the draws
    are stored here as a matrix. The rows of the matrix are the samples,
    and the columns are the monitored variables.}
  \item{N}{This is the final number of nodes.}
  \item{Posterior}{When \code{sir=TRUE}, a number of independent
    posterior samples equal to \code{Samples} is taken, and the draws
    are stored here as a matrix. The rows of the matrix are the samples,
    and the columns are the parameters.}
  \item{Summary1}{This is a summary matrix that summarizes the
    point-estimated posterior means. Uncertainty around the posterior
    means is estimated from the covariance matrix. Rows are parameters.
    The following columns are included: Mean, SD (Standard Deviation),
    LB (Lower Bound), and UB (Upper Bound). The bounds constitute a 95\%
    probability interval.}
  \item{Summary2}{This is a summary matrix that summarizes the
    posterior samples drawn with sampling importance resampling
    (\code{\link{SIR}}) when \code{sir=TRUE}, given the point-estimated
    posterior modes and the covariance matrix. Rows are parameters. The
    following columns are included: Mean, SD (Standard Deviation),
    LB (Lower Bound), and UB (Upper Bound). The bounds constitute a 95\%
    probability interval.}
  \item{Tolerance.Final}{This is the last \code{Tolerance} of the
    \code{LaplaceApproxiation} algorithm.}
  \item{Tolerance.Stop}{This is the \code{Stop.Tolerance} criteria.}
  \item{Z}{This is the final \eqn{N \times J}{N x J} matrix of the
    conditional mean, where \eqn{N} is the number of nodes and \eqn{J}
    is the number of parameters.}
}
\references{
  Naylor, J.C. and Smith, A.F.M. (1982). "Applications of a Method for
  the Efficient Computation of Posterior Distributions". \emph{Applied
  Statistics}, 31(3), p. 214--225.

  O'Hagan, A. (1987). "Monte Carlo is Fundamentally Unsound". \emph{The
    Statistician}, 36, p. 247--249.

  O'Hagan, A. (1991). "Bayes-Hermite Quadrature". \emph{Journal of
    Statistical Planning and Inference}, 29, p. 245--260.

  O'Hagan, A. (1992). "Some Bayesian Numerical Analysis". In Bernardo,
  J.M., Berger, J.O., David, A.P., and Smith, A.F.M., editors,
  \emph{Bayesian Statistics}, 4, p. 356--363, Oxford University Press.

  Rasmussen, C.E. and Ghahramani, Z. (2003). "Bayesian Monte Carlo". In
  Becker, S. and Obermayer, K., editors, \emph{Advances in Neural
  Information Processing Systems}, 15, MIT Press, Cambridge, MA.
}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{GaussHermiteCubeRule}},
  \code{\link{GaussHermiteQuadRule}},
  \code{\link{GIV}},
  \code{\link{Hermite}},
  \code{\link{Hessian}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{LML}},
  \code{\link{PMC}},
  \code{\link{SIR}}, and
  \code{\link{SparseGrid}}.
}
\examples{
# The accompanying Examples vignette is a compendium of examples.
####################  Load the LaplacesDemon Library  #####################
library(LaplacesDemon)

##############################  Demon Data  ###############################
data(demonsnacks)
y <- log(demonsnacks$Calories)
X <- cbind(1, as.matrix(log(demonsnacks[,10]+1)))
J <- ncol(X)
for (j in 2:J) X[,j] <- CenterScale(X[,j])

#########################  Data List Preparation  #########################
mon.names <- "mu[1]"
parm.names <- as.parm.names(list(beta=rep(0,J), sigma=0))
pos.beta <- grep("beta", parm.names)
pos.sigma <- grep("sigma", parm.names)
PGF <- function(Data) {
     beta <- rnorm(Data$J)
     sigma <- runif(1)
     return(c(beta, sigma))
     }
MyData <- list(J=J, PGF=PGF, X=X, mon.names=mon.names,
     parm.names=parm.names, pos.beta=pos.beta, pos.sigma=pos.sigma, y=y)

##########################  Model Specification  ##########################
Model <- function(parm, Data)
     {
     ### Parameters
     beta <- parm[Data$pos.beta]
     sigma <- interval(parm[Data$pos.sigma], 1e-100, Inf)
     parm[Data$pos.sigma] <- sigma
     ### Log-Prior
     beta.prior <- sum(dnormv(beta, 0, 1000, log=TRUE))
     sigma.prior <- dhalfcauchy(sigma, 25, log=TRUE)
     ### Log-Likelihood
     mu <- tcrossprod(Data$X, t(beta))
     LL <- sum(dnorm(Data$y, mu, sigma, log=TRUE))
     ### Log-Posterior
     LP <- LL + beta.prior + sigma.prior
     Modelout <- list(LP=LP, Dev=-2*LL, Monitor=mu[1],
          yhat=rnorm(length(mu), mu, sigma), parm=parm)
     return(Modelout)
     }

############################  Initial Values  #############################
#Initial.Values <- GIV(Model, MyData, PGF=TRUE)
Initial.Values <- rep(0,J+1)

#########################  Adaptive Gauss-Hermite  ########################
#Fit <- IterativeQuadrature(Model, Initial.Values, MyData, Covar=NULL,
#     Iterations=100, Algorithm="AGH",
#     Specs=list(N=5, Nmax=7, Packages=NULL, Dyn.libs=NULL), CPUs=1)

##################  Adaptive Gauss-Hermite Sparse Grid  ###################
#Fit <- IterativeQuadrature(Model, Initial.Values, MyData, Covar=NULL,
#     Iterations=100, Algorithm="AGHSG",
#     Specs=list(K=5, Kmax=7, Packages=NULL, Dyn.libs=NULL), CPUs=1)

#################  Componentwise Adaptive Gauss-Hermite  ##################
#Fit <- IterativeQuadrature(Model, Initial.Values, MyData, Covar=NULL,
#     Iterations=100, Algorithm="CAGH",
#     Specs=list(N=3, Nmax=10, Packages=NULL, Dyn.libs=NULL), CPUs=1)

#Fit
#print(Fit)
#PosteriorChecks(Fit)
#caterpillar.plot(Fit, Parms="beta")
#plot(Fit, MyData, PDF=FALSE)
#Pred <- predict(Fit, Model, MyData, CPUs=1)
#summary(Pred, Discrep="Chi-Square")
#plot(Pred, Style="Covariates", Data=MyData)
#plot(Pred, Style="Density", Rows=1:9)
#plot(Pred, Style="Fitted")
#plot(Pred, Style="Jarque-Bera")
#plot(Pred, Style="Predictive Quantiles")
#plot(Pred, Style="Residual Density")
#plot(Pred, Style="Residuals")
#Levene.Test(Pred)
#Importance(Fit, Model, MyData, Discrep="Chi-Square")

#End
}
\keyword{Adaptive}
\keyword{Bayesian Inference}
\keyword{Cubature}
\keyword{Gauss-Hermite}
\keyword{High Performance Computing}
\keyword{Initial Values}
\keyword{Integration}
\keyword{Numerical Integration}
\keyword{Quadrature}
\keyword{Sparse Grid}

\name{ESS}
\alias{ESS}
\title{Effective Sample Size due to Autocorrelation}
\description{
  This function may be used to estimate the effective sample size (ESS)
  (not to be confused with Elliptical Slice Sampling) of a continuous
  target distribution, where the sample size is reduced by
  autocorrelation. ESS is a measure of how well each continuous chain is
  mixing.

  ESS is a univariate function that is often applied to each continuous,
  marginal posterior distribution. A multivariate form is not
  included. By chance alone due to multiple independent tests, 5\% of
  the continuous parameters may indicate that ESS is below a user
  threshold of acceptability, such as 100, even when above the
  threshold. Assessing convergence is difficult.
}
\usage{ESS(x)}
\arguments{
  \item{x}{This required argument is a vector or matrix of posterior
    samples.}
}
\details{
  Effective Sample Size (ESS) was recommended by Radford Neal in the
  panel discussion of Kass et al. (1998). When a continuous, marginal
  posterior distribution is sampled with a Markov chain Monte Carlo
  (MCMC) algorithm, there is usually autocorrelation present in the
  samples. More autocorrelation is associated with less posterior
  sampled information, because the information in the samples is
  autocorrelated, or put another way, successive samples are not
  independent from earlier samples. This reduces the effective sample
  size of, and precision in representing, the continuous, marginal
  posterior distribution. \code{ESS} is one of the criteria in the
  \code{\link{Consort}} function, where stopping the MCMC updates is
  not recommended until \code{ESS} \eqn{\ge 100}. Although the need
  for precision of each modeler differs with each model, it is often
  a good goal to obtain \code{ESS} \eqn{= 1000}.

  \code{ESS} is related to the integrated autocorrelation time (see
  \code{\link{IAT}} for more information).

  ESS is usually defined as

  \deqn{\mathrm{ESS}(\theta) = \frac{S}{1 + 2 \sum^{\infty}_{k=1} \rho_k
    (\theta)},}{ESS(theta) = S / (1 + 2 sum[k] rho[k] (theta)),}

  where \eqn{S}{S} is the number of posterior samples,
  \eqn{\rho_k}{rho[k]} is the autocorrelation at lag \eqn{k}{k}, and
  \eqn{\theta}{theta} is the vector of marginal posterior samples. The
  infinite sum is often truncated at lag \eqn{k}{k} when
  \eqn{\rho_k (\theta) < 0.05}{rho[k](theta) < 0.05}. Just as with the
  \code{effectiveSize} function in the \code{coda} package, the
  \code{AIC} argument in the \code{ar} function is used to estimate the
  order.

  ESS is a measure of how well each continuous chain is mixing, and a
  continuous chain mixes better when in the target distribution. This
  does not imply that a poorly mixing chain still searching for its
  target distribution will suddenly mix well after finding it, though
  mixing should improve. A poorly mixing continuous chain does not
  necessarily indicate problems. A smaller ESS is often due to
  correlated parameters, and is commonly found with scale
  parameters. Posterior correlation may be obtained from the
  \code{\link{PosteriorChecks}} function, and plotted with the 
  \code{\link{plotMatrix}} function. Common remedies for poor mixing
  include re-parameterizing the model or trying a different MCMC
  algorithm that better handles correlated parameters. Slow mixing is
  indicative of an inefficiency in which a continuous chain takes longer
  to find its target distribution, and once found, takes longer to
  explore it. Therefore, slow mixing results in a longer required
  run-time to find and adequately represent the continuous target
  distribution, and increases the chance that the user may make
  inferences from a less than adequate representation of the continuous
  target distribution.

  There are many methods of re-parameterization to improve mixing. It
  is helpful when predictors are centered and scaled, such as with the
  \code{\link{CenterScale}} function. Parameters for predictors are
  often assigned prior distributions that are independent per parameter,
  in which case an exchangeable prior distribution or a multivariate
  prior distribution may help. If a parameter with poor mixing is
  bounded with the \code{\link{interval}} function, then
  transforming it to the real line (such as with a log transformation
  for a scale parameter) is often helpful, since constraining a
  parameter to an interval often reduces ESS. Another method is to
  re-parameterize so that one or more latent variables represent the
  process that results in slow mixing. Such re-parameterization uses
  data augmentation.

  This is numerically the same as the \code{effectiveSize} function in
  the \code{coda} package, but programmed to accept a simple vector or
  matrix so it does not require an \code{mcmc} or \code{mcmc.list}
  object, and the result is bound to be less than or equal to the
  original number of samples.
}
\value{
  A vector is returned, and each element is the effective sample size
  (ESS) for a corresponding column of \code{x}, after autocorrelation has
  been taken into account.
}
\references{
  Kass, R.E., Carlin, B.P., Gelman, A., and Neal, R. (1998). "Markov
  Chain Monte Carlo in Practice: A Roundtable Discussion". \emph{The
  American Statistician}, 52, p. 93--100. 
}
\seealso{
  \code{\link{CenterScale}},
  \code{\link{Consort}},
  \code{\link{IAT}},
  \code{\link{interval}},
  \code{\link{LaplacesDemon}},
  \code{\link{plotMatrix}}, and
  \code{\link{PosteriorChecks}}.
}
\keyword{Diagnostic}
\keyword{MCMC}

\name{dist.Scaled.Inverse.Wishart}
\alias{dsiw}
\alias{rsiw}
\title{Scaled Inverse Wishart Distribution}
\description{
  These functions provide the density and random number generation
     for the scaled inverse Wishart distribution.
}
\usage{
   dsiw(Q, nu, S, zeta, mu, delta, log=FALSE)
   rsiw(nu, S, mu, delta)
}
\arguments{
  \item{Q}{This is the symmetric, positive-definite
    \eqn{k \times k}{k x k} matrix \eqn{\textbf{Q}}.}
  \item{nu}{This is the scalar degrees of freedom, \eqn{\nu}{nu}
    regarding \eqn{\textbf{Q}}. The default recommendation is
    \code{nu=k+1}.}
  \item{S}{This is the symmetric, positive-semidefinite
    \eqn{k \times k}{k x k} scale matrix \eqn{\textbf{S}}{S} regarding
    \eqn{\textbf{Q}}. The default recommendation is \code{S=diag(k)}.}
  \item{zeta}{This is a positive-only vector of length \eqn{k} of
    auxiliary scale parameters \eqn{\zeta}{zeta}.}
  \item{mu}{This is a vector of length \eqn{k} of location
    hyperparameters \eqn{\mu}{mu} regarding \eqn{\zeta}{zeta}.}
  \item{delta}{This is a positive-only vector of length \eqn{k} of
    scale hyperparameters \eqn{\delta}{delta} regarding
    \eqn{\zeta}{zeta}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Multivariate
    \item Density: (see below)
    \item Inventor: O'Malley and Zaslavsky (2005)
    \item Notation 1: \eqn{p(\Sigma) \sim \mathcal{SIW}(\textbf{Q},
      \nu, \textbf{S}, \zeta, \mu, \delta)}{p(Sigma) ~ SIW(Q, nu,
      S, zeta, mu, delta)}
    \item Notation 2: \eqn{p(\Sigma) = \mathcal{SIW}(\Sigma |
      \textbf{Q}, \nu, \textbf{S}, \zeta, \mu, \delta}{p(Sigma) =
      SIW(Sigma | Q, nu, S, zeta, mu, delta)}
    \item Parameter 1: symmetric, positive-definite
      \eqn{k \times k}{k x k} matrix \eqn{\textbf{Q}}{Q}
    \item Parameter 2: degrees of freedom \eqn{\nu}{nu}
    \item Parameter 3: symmetric, positive-semidefinite
      \eqn{k \times k}{k x k} scale matrix \eqn{\textbf{S}}{S}
    \item Parameter 4: Auxiliary scale parameter vector
      \eqn{\zeta}{zeta}
    \item Parameter 5: Hyperparameter location vector
      \eqn{\mu}{mu}
    \item Parameter 6: Hyperparameter scale vector
      \eqn{\delta}{delta}
    \item Mean: 
    \item Variance: 
    \item Mode: 
}

The scaled inverse Wishart (SIW) distribution is a prior probability
distribution for a covariance matrix, and is an alternative to the
inverse Wishart distribution.

While the inverse Wishart distribution is applied directly to covariance
matrix \eqn{\Sigma}{Sigma}, the SIW distribution is applied to a
decomposed matrix \eqn{\textbf{Q}} and diagonal scale matrix
\eqn{\zeta}{zeta}. For information on how to apply it to
\eqn{\textbf{Q}}, see the example below.

SIW is more flexible than the inverse Wishart distribution because it
has additional, and some say somewhat redundant, scale parameters. This
makes up for one limitation of the inverse Wishart, namely that all
uncertainty about posterior variances is represented in one parameter.
The SIW prior may somewhat alleviate the dependency in the inverse
Wishart between variances and correlations, though the SIW prior still
retains some of this relationship.

The Huang-Wand (\code{\link{dhuangwand}}) prior is a hierarchical
alternative.
}
\value{
  \code{dsiw} gives the density and
  \code{rsiw} generates random deviates.
}
\references{
  O'Malley, A.J. and Zaslavsky, A.M. (2005), "Domain-Level Covariance
  Analysis for Survey Data with Structured Nonresponse".
}
\seealso{
  \code{\link{dhuangwand}},
  \code{\link{dinvwishartc}},
  \code{\link{dmvn}}, and
  \code{\link{dwishart}}.
}
\examples{
library(LaplacesDemon)
### In the model specification function, input U and zeta, then:
# Q <- t(U) %*% U
# Zeta <- diag(zeta)
# Sigma <- Zeta %*% Q %*% Zeta
# Sigma.prior <- dsiw(Q, nu=Data$K+1, S=diag(Data$K), zeta, mu=0, delta=1)
### Examples
x <- dsiw(diag(3), 4, diag(3), runif(3), rep(0,3), rep(1,3), log=TRUE)
x <- rsiw(4, diag(3), rep(0,3), rep(1,3))
}
\keyword{Distribution}

\name{dist.HuangWand}
\alias{dhuangwand}
\alias{dhuangwandc}
\alias{rhuangwand}
\alias{rhuangwandc}
\title{Huang-Wand Distribution}
\description{
  These are the density and random generation functions for the
  Huang-Wand prior distribution for a covariance matrix.
}
\usage{
dhuangwand(x, nu=2, a, A, log=FALSE)
dhuangwandc(x, nu=2, a, A, log=FALSE)
rhuangwand(nu=2, a, A)
rhuangwandc(nu=2, a, A)
}
\arguments{
  \item{x}{This is a \eqn{k \times k}{k x k} positive-definite
    covariance matrix \eqn{\Sigma}{Sigma} for \code{dhuangwand}, or the
    Cholesky factor \eqn{\textbf{U}}{U} of the covariance matrix for
    \code{dhuangwandc}.}
  \item{nu}{This is a scalar degrees of freedom parameter
    \eqn{\nu}{nu}. The default is \code{nu=2}, which is an
    uninformative prior, resulting in marginal uniform distributions
    on the correlation matrix.}
  \item{a}{This is a positive-only vector of scale parameters
    \eqn{a} of length \eqn{k}.}
  \item{A}{This is a positive-only vector of of scale hyperparameters
    \eqn{A} of length \eqn{k}. Larger values result in a more
    uninformative prior. A default, uninformative prior is
    \code{A=rep(1e6,k)}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Multivariate
    \item Density: \eqn{p(\theta) =
      \mathcal{W}^{-1}_{\nu+k-1}(2 \nu diag(1/a))
      \mathcal{G}^{-1}(1/2, 1/A^2)}{p(theta) =
      W^(-1)[nu+k-1](2*nu*diag(1/a)) G^(-1)(1/2, 1/A^2)}
    \item Inventor: Huang and Wand (2013)
    \item Notation 1: \eqn{\theta \sim \mathcal{HW}_\nu(\textbf{a},
      \textbf{A})}{theta ~ HW[nu](a, A)}
    \item Notation 2: \eqn{p(\theta) \sim \mathcal{HW}_\nu(\theta |
      \textbf{a}, \textbf{A})}{p(theta) ~ HW[nu](theta | a, A)}
    \item Parameter 1: degrees of freedom \eqn{\nu}{nu}
    \item Parameter 2: scale \eqn{a > 0}
    \item Parameter 3: scale \eqn{A > 0}
    \item Mean: 
    \item Variance: 
    \item Mode: 
}

Huang and Wand (2013) proposed a prior distribution for a covariance
matrix that uses a hierarchical inverse Wishart. This is a more flexible
alternative to the inverse Wishart distribution, and the Huang-Wand
prior retains conjugacy. The Cholesky parameterization is also provided
here.

The Huang-Wand prior distribution alleviates two main limitations of an
inverse Wishart distribution. First, the uncertainty in the diagonal
variances of a covariance matrix that is inverse Wishart distributed is
represented with only one degrees of freedom parameter, which may be too
restrictive. The Huang-Wand prior overcomes this limitation. Second, the
inverse Wishart distribution imposes a dependency between variance and
correlation. The Huang-Wand prior lessens, but does not fully remove,
this dependency.

The standard deviations of a Huang-Wand distributed covariance matrix
are half-t distributed, as \eqn{\mathcal{HT}(\nu, \textbf{A})}{HT(nu,
  A)}. This is in accord with modern assumptions about distributions of
scale parameters, and is also useful for sparse covariance matrices.

The \code{rhuangwand} function allows either \code{a} or \code{A} to be
missing. When \code{a} is missing, the covariance matrix is generated
from the hyperparameters. When \code{A} is missing, the covariance
matrix is generated from the parameters.
}
\value{
  \code{dhuangwand} and \code{dhuangwandc} give the density, and
  \code{rhuangwand} and \code{rhuangwandc} generate random deviates.
}
\references{
  Huang, A., Wand, M., et al. (2013), "Simple Marginally Noninformative
  Prior Distributions for Covariance Matrices". \emph{Bayesian
  Analysis}, 8, p. 439--452.
}
\seealso{
  \code{\link{dhalft}} and
  \code{\link{dinvwishart}}
}
\examples{
library(LaplacesDemon)
dhuangwand(diag(3), nu=2, a=runif(3), A=rep(1e6,3), log=TRUE)
rhuangwand(nu=2, A=rep(1e6, 3)) #Missing a
rhuangwand(nu=2, a=runif(3)) #Missing A
}
\keyword{Distribution}

\name{dist.Inverse.Gaussian}
\alias{dinvgaussian}
\alias{rinvgaussian}
\title{Inverse Gaussian Distribution}
\description{
  This is the density function and random generation from the inverse
  gaussian distribution.
}
\usage{
dinvgaussian(x, mu, lambda, log=FALSE)
rinvgaussian(n, mu, lambda)
}
\arguments{
  \item{n}{This is the number of draws from the distribution.}
  \item{x}{This is the scalar location to evaluate density.}
  \item{mu}{This is the mean parameter, \eqn{\mu}{mu}.}
  \item{lambda}{This is the inverse-variance parameter,
    \eqn{\lambda}{lambda}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Univariate
    \item Density: \eqn{p(\theta) = \frac{\lambda}{(2 \pi
	\theta^3)^{1/2}} \exp(-\frac{\lambda (\theta - \mu)^2}{2 \mu^2
	\theta}), \theta > 0}{p(theta) = (lambda / (2*pi*theta^3))^(1/2) *
        exp(-((lambda*(theta-mu)^2) / (2*mu^2*theta))), theta > 0}
    \item Inventor: Schrodinger (1915)
    \item Notation 1: \eqn{\theta \sim \mathcal{N}^{-1}(\mu,
      \lambda)}{theta ~ N^-1(mu, lambda)}
    \item Notation 2: \eqn{p(\theta) = \mathcal{N}^{-1}(\theta | \mu,
      \lambda)}{p(theta) = N^-1(theta | mu, lambda)}
    \item Parameter 1: shape \eqn{\mu > 0}{mu > 0}
    \item Parameter 2: scale \eqn{\lambda > 0}{lambda > 0}
    \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}
    \item Variance: \eqn{var(\theta) = \frac{\mu^3}{\lambda}}{var(theta)
      = mu^3/lambda}
    \item Mode: \eqn{mode(\theta) = \mu((1 + \frac{9 \mu^2}{4
      \lambda^2})^{1/2} - \frac{3 \mu}{2 \lambda})}{mode(theta) = mu*((1
    + ((9*mu^2)/(4*lambda^2)))^(1/2) - \frac{3*mu}{2*lambda})}
}

The inverse-Gaussian distribution, also called the Wald distribution, is
used when modeling dependent variables that are positive and
continuous. When
\eqn{\lambda \rightarrow \infty}{lambda tends to infinity} (or variance
to zero), the inverse-Gaussian distribution becomes similar to a normal
(Gaussian) distribution. The name, inverse-Gaussian, is misleading,
because it is not the inverse of a Gaussian distribution, which is
obvious from the fact that \eqn{\theta}{theta} must be positive.
}
\value{
  \code{dinvgaussian} gives the density and
  \code{rinvgaussian} generates random deviates.
}
\references{
  Schrodinger E. (1915). "Zur Theorie der Fall-und Steigversuche an
  Teilchenn mit Brownscher Bewegung". \emph{Physikalische Zeitschrift},
  16, p. 289--295.
}
\seealso{
  \code{\link{dnorm}},
  \code{\link{dnormp}}, and
  \code{\link{dnormv}}.
}
\examples{
library(LaplacesDemon)
x <- dinvgaussian(2, 1, 1)
x <- rinvgaussian(10, 1, 1)

#Plot Probability Functions
x <- seq(from=1, to=20, by=0.1)
plot(x, dinvgaussian(x,1,0.5), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dinvgaussian(x,1,1), type="l", col="green")
lines(x, dinvgaussian(x,1,5), type="l", col="blue")
legend(2, 0.9, expression(paste(mu==1, ", ", sigma==0.5),
     paste(mu==1, ", ", sigma==1), paste(mu==1, ", ", sigma==5)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{dist.Generalized.Pareto}
\alias{dgpd}
\alias{rgpd}
\title{Generalized Pareto Distribution}
\description{
  These are the density and random generation functions for the
  generalized Pareto distribution.
}
\usage{
dgpd(x, mu, sigma, xi, log=FALSE)
rgpd(n, mu, sigma, xi)
}
\arguments{
  \item{x}{This is a vector of data.}
  \item{n}{This is a positive scalar integer, and is the number of
    observations to generate randomly.}
  \item{mu}{This is a scalar or vector location parameter
    \eqn{\mu}{mu}. When \eqn{\xi}{xi} is non-negative,
    \eqn{\mu}{mu} must not be greater than \eqn{\textbf{x}}{x}. When
    \eqn{\xi}{xi} is negative, \eqn{\mu}{mu} must be less than
    \eqn{\textbf{x} + \sigma / \xi}{x + sigma/xi}.}
  \item{sigma}{This is a positive-only scalar or vector of scale
    parameters \eqn{\sigma}{sigma}.}
  \item{xi}{This is a scalar or vector of shape parameters
    \eqn{\xi}{xi}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Univariate
    \item Density: \eqn{p(\theta) = \frac{1}{\sigma}(1 +
      \xi\textbf{z})^(-1/\xi + 1)}{1/sigma (1 + xi z)^(-1/xi +
      1)} where \eqn{\textbf{z} = \frac{\theta - \mu}{\sigma}}{z =
      (theta - mu)/sigma}
    \item Inventor: Pickands (1975)
    \item Notation 1: \eqn{\theta \sim \mathcal{GPD}(\mu, \sigma,
      \xi)}{theta ~ GPD(mu, sigma, xi)}
    \item Notation 2: \eqn{p(\theta) \sim \mathcal{GPD}(\theta |
      \mu, \sigma, \xi)}{p(theta) ~ GPD(theta | mu, sigma, xi)}
    \item Parameter 1: location \eqn{\mu}{mu}, where \eqn{\mu \le
      \theta}{mu <= theta} when \eqn{\xi \ge 0}{xi >= 0}, and
      \eqn{\mu \ge \theta + \sigma / \xi}{mu >= theta + sigma/xi}
      when \eqn{\xi < 0}{xi < 0}
    \item Parameter 2: scale \eqn{\sigma > 0}{sigma > 0}
    \item Parameter 3: shape \eqn{\xi}{xi}
    \item Mean: \eqn{\mu + \frac{\sigma}{1 - \xi}}{mu + sigma / (1 -
      xi)} when \eqn{\xi < 1}{xi < 1}
    \item Variance: \eqn{\frac{\sigma^2}{(1 - \xi)^2 (1 -
      2\xi)}}{sigma^2 / [(1 - xi)^2 (1 - 2 xi)]} when \eqn{\xi
      < 0.5}{xi < 0.5}
    \item Mode: 
}

The generalized Pareto distribution (GPD) is a more flexible extension
of the Pareto (\code{\link{dpareto}}) distribution. It is equivalent to
the exponential distribution when both \eqn{\mu = 0}{mu = 0} and
\eqn{\xi = 0}{xi = 0}, and it is equivalent to the Pareto
distribution when \eqn{\mu = \sigma / \xi}{mu = sigma / xi} and
\eqn{\xi > 0}{xi > 0}.

The GPD is often used to model the tails of another distribution, and
the shape parameter \eqn{\xi}{xi} relates to
tail-behavior. Distributions with tails that decrease exponentially are
modeled with shape \eqn{\xi = 0}{xi = 0}. Distributions with tails that
decrease as a polynomial are modeled with a positive shape
parameter. Distributions with finite tails are modeled with a negative
shape parameter.
}
\value{
  \code{dgpd} gives the density, and
  \code{rgpd} generates random deviates.
}
\references{
  Pickands J. (1975). "Statistical Inference Using Extreme Order
  Statistics". \emph{The Annals of Statistics}, 3, p. 119--131.
}
\seealso{
  \code{\link{dpareto}}
}
\examples{
library(LaplacesDemon)
x <- dgpd(0,0,1,0,log=TRUE)
x <- rgpd(10,0,1,0)
}
\keyword{Distribution}

\name{Blocks}
\alias{Blocks}
\title{Blocks}
\description{
  The \code{Blocks} function returns a list of \eqn{N} blocks of
  parameters, for use with some MCMC algorithms in the
  \code{\link{LaplacesDemon}} function. Blocks may be created either
  sequentially, or from a hierarchical clustering of the posterior
  correlation matrix.
}
\usage{
Blocks(Initial.Values, N, PostCor=NULL)
}
\arguments{
  \item{Initial.Values}{This required argument is a vector of initial
    values.}
  \item{N}{This optional argument indicates the desired number of
    blocks. If omitted, then the truncated square root of the number of
    initial values is used. If a posterior correlation matrix is
    supplied to \code{PostCor}, then \code{N} may be a scalar, or
    have length two. If \code{N} has length two, then the first element
    indicates the minimum number of blocks, and the second element
    indicates the maximum number of blocks, and the number of blocks
    is the maximum of the mean silhouette width for each hierarchical
    cluster solution.}
  \item{PostCor}{This optional argument defaults to \code{NULL}, in
    which case sequential blocking is performed. If a posterior
    correlation matrix is supplied, then blocks are created based on
    hierarchical clustering.}
}
\details{
  Usually, there is more than one target distribution in MCMC, in which
  case it must be determined whether it is best to sample from target
  distributions individually, in groups, or all at once. Blockwise
  sampling (also called block updating) refers to splitting a
  multivariate vector into groups called blocks, and each block is
  sampled separately. A block may contain one or more parameters.

  Parameters are usually grouped into blocks such that parameters within
  a block are as correlated as possible, and parameters between blocks
  are as independent as possible. This strategy retains as much of the
  parameter correlation as possible for blockwise sampling, as opposed
  to componentwise sampling where parameter correlation is ignored.
  The \code{\link{PosteriorChecks}} function can be used on the output
  of previous runs to find highly correlated parameters. See examples
  below.

  Advantages of blockwise sampling are that a different MCMC algorithm
  may be used for each block (or parameter, for that matter), creating a
  more specialized approach (though different algorithms by block are
  not supported here), the acceptance of a newly proposed state is
  likely to be higher than sampling from all target distributions at
  once in high dimensions, and large proposal covariance matrices can
  be reduced in size, which is most helpful again in high dimensions.

  Disadvantages of blockwise sampling are that correlations probably
  exist between parameters between blocks, and each block is updated
  while holding the other blocks constant, ignoring these correlations
  of parameters between blocks. Without simultaneously taking
  everything into account, the algorithm may converge slowly or never
  arrive at the proper solution. However, there are instances when it
  may be best when everything is not taken into account at once, such
  as in state-space models. Also, as the number of blocks increases,
  more computation is required, which slows the algorithm. In general,
  blockwise sampling allows a more specialized approach at the expense
  of accuracy, generalization, and speed. Blockwise sampling is offered
  in the following algorithms: Adaptive-Mixture Metropolis (AMM),
  Adaptive Metropolis-within-Gibbs (AMWG), Automated Factor Slice
  Sampler (AFSS), Elliptical Slice Sampler (ESS), Hit-And-Run Metropolis
  (HARM), Metropolis-within-Gibbs (MWG), Random-Walk Metropolis (RWM),
  Robust Adaptive Metropolis (RAM), Slice Sampler (Slice), and the
  Univariate Eigenvector Slice Sampler (UESS).

  Large-dimensional models often require blockwise sampling. For
  example, with thousands of parameters, a componentwise algorithm
  must evaluate the model specification function once per parameter per
  iteration, resulting in an algorithm that may take longer than is
  acceptable to produce samples. Algorithms that require derivatives,
  such as the family of Hamiltonian Monte Carlo (HMC), require even more
  evaluations of the model specification function per iteration, and
  quickly become too costly in large dimensions. Finally, algorithms
  with multivariate proposals often have difficulty producing an
  accepted proposal in large-dimensional models. The most practical
  solution is to group parameters into \eqn{N} blocks, and each
  iteration the algorithm evaluates the model specification function
  \eqn{N} times, each with a reduced set of parameters.

  The \code{Blocks} function performs either a sequential assignment of
  parameters to blocks when posterior correlation is not supplied, or
  uses hierarchical clustering to create blocks based on posterior
  correlation. If posterior correlation is supplied, then the user may
  specify a range of the number of blocks to consider, and the optimal
  number of blocks is considered to be the maximum of the mean
  silhouette width of each hierarchical clustering. Silhouette width
  is calculated as per the \code{cluster} package. Hierarchical
  clustering is performed on the distance matrix calculated from the
  dissimilarity matrix (1 - abs(PostCor)) of the posterior correlation
  matrix. With sequential assignment, the number of parameters per
  block is approximately equal. With hierarchical clustering, the
  number of parameters per block may vary widely. Creating blocks from
  hierarchical clustering performs well in practice, though there are
  many alternative methods the user may consider outside of this
  function, such as using factor analysis, model-based clustering, or
  other methods.

  Aside from sequentially-assigned blocks, or blocks based on posterior
  correlation, it is also common to group parameters with similar uses,
  such as putting regression effects parameters into one block, and
  autocorrelation parameters into another block. Another popular way to
  group parameters into blocks is by time-period for some time-series
  models. These alternative blocking strategies are unsupported in the
  \code{Blocks} function, and best left to user discretion.

  Some MCMC algorithms that accept blocked parameters also require
  blocked variance-covariance matrices. The \code{Blocks} function
  does not return these matrices, because it may not be necessary,
  or when it is, the user may prefer identity matrices, scaled identity
  matrices, or matrices with explicitly-defined elements.

  If the user is looking for a place to begin with blockwise sampling,
  then the recommended, default approach (when blocked parameters by
  time-period are not desired in a time-series) is to begin with a
  trial run of the adaptive, unblocked HARM algorithm (since covariance
  matrices are not required) for the purposes of obtaining a posterior
  correlation matrix. Next, create blocks with the \code{Blocks}
  function based on the posterior correlation matrix obtained from the
  trial run. Finally, run the desired, blocked algorithm with the newly
  created blocks (and possibly user-specified covariance matrices),
  beginning where the trial run ended.

  If hierarchical clustering is used, then it is important to note that
  hierarchical clustering has no idea that the user intends to perform
  blockwise sampling in MCMC. If hierarchical clustering returns
  numerous small blocks, then the user may consider combining some or
  all of those blocks. For example, if several 1-parameter blocks are
  returned, then blockwise sampling will equal componentwise sampling
  for those blocks, which will iterate slower. Conversely, if
  hierarchical clustering returns one or more big blocks, each with
  enough parameters that multivariate sampling will have difficulty
  getting an accepted proposal, or an accepted proposal that moves
  more than a small amount, then the user may consider subdividing
  these big blocks into smaller, more manageable blocks, though with
  the understanding that more posterior correlation is unaccounted for.
}
\value{
  The \code{Blocks} function returns an object of class \code{blocks},
  which is a list. Each component of the list is a block of parameters,
  and parameters are indicated by their position in the initial values
  vector.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{LaplacesDemon}} and
  \code{\link{PosteriorChecks}}.
}
\examples{
library(LaplacesDemon)

### Create the default number of sequentially assigned blocks:
Initial.Values <- rep(0,1000)
MyBlocks <- Blocks(Initial.Values)
MyBlocks

### Or, a pre-specified number of sequentially assigned blocks:
#Initial.Values <- rep(0,1000)
#MyBlocks <- Blocks(Initial.Values, N=20)

### If scaled diagonal covariance matrices are desired:
#VarCov <- list()
#for (i in 1:length(MyBlocks))
#  VarCov[[i]] <- diag(length(MyBlocks[[i]]))*2.38^2/length(MyBlocks[[i]])

### Or, determine the number of blocks in the range of 2 to 50 from
### hierarchical clustering on the posterior correlation matrix of an
### object, say called Fit, output from LaplacesDemon:
#MyBlocks <- Blocks(Initial.Values, N=c(2,50),
#  PostCor=cor(Fit$Posterior1))
#lapply(MyBlocks, length) #See the number of parameters per block

### Or, create a pre-specified number of blocks from hierarchical
### clustering on the posterior correlation matrix of an object,
### say called Fit, output from LaplacesDemon:
#MyBlocks <- Blocks(Initial.Values, N=20, PostCor=cor(Fit$Posterior1))

### Posterior correlation from a previous trial run could be obtained
### with either method below (though cor() will be fastest because
### additional checks are not calculated for the parameters):
#rho <- cor(Fit$Posterior1)
#rho <- PosteriorChecks(Fit)$Posterior.Correlation
}
\keyword{MCMC}
\keyword{Multicollinearity}
\keyword{Posterior Correlation}
\keyword{Utility}

\name{dist.Horseshoe}
\alias{dhs}
\alias{rhs}
\title{Horseshoe Distribution}
\description{
  This is the density function and random generation from the horseshoe
  distribution.
}
\usage{
dhs(x, lambda, tau, log=FALSE)
rhs(n, lambda, tau)
}
\arguments{
  \item{n}{This is the number of draws from the distribution.}
  \item{x}{This is a location vector at which to evaluate density.}
  \item{lambda}{This vector is a positive-only local parameter
    \eqn{\lambda}{lambda}.}
  \item{tau}{This scalar is a positive-only global parameter
    \eqn{\tau}{tau}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Multivariate Scale Mixture
    \item Density: (see below)
    \item Inventor: Carvalho et al. (2008)
    \item Notation 1: \eqn{\theta \sim \mathcal{HS}(\lambda,
      \tau)}{theta ~ HS(lambda, tau)}
    \item Notation 2: \eqn{p(\theta) = \mathcal{HS}(\theta | \lambda,
      \tau)}{p(theta) = HS(theta | lambda, tau)}
    \item Parameter 1: local scale \eqn{\lambda > 0}{lambda > 0}
    \item Parameter 2: global scale \eqn{\tau > 0}{tau > 0}
    \item Mean: \eqn{E(\theta)}{E(theta)}
    \item Variance: \eqn{var(\theta)}{var(theta)}
    \item Mode: \eqn{mode(\theta)}{mode(theta)}
}

The horseshoe distribution (Carvalho et al., 2008) is a heavy-tailed
mixture distribution that can be considered a variance mixture,
and it is in the family of multivariate scale mixtures of normals.

The horseshoe distribution was proposed as a prior distribution, and
recommended as a default choice for shrinkage priors in the presence of
sparsity. Horseshoe priors are most appropriate in large-p models where
dimension reduction is necessary to avoid overly complex models that
predict poorly, and also perform well in estimating a sparse covariance
matrix via Cholesky decomposition (Carvalho et al., 2009).

When the number of parameters in variable selection is assumed to be
sparse, meaning that most elements are zero or nearly zero, a horseshoe
prior is a desirable alternative to the Laplace-distributed parameters
in the LASSO, or the parameterization in ridge regression. When the true
value is far from zero, the horseshoe prior leaves the parameter
unshrunk. Yet, the horseshoe prior is accurate in shrinking parameters
that are truly zero or near-zero. Parameters near zero are shrunk more
than parameters far from zero. Therefore, parameters far from zero
experience less shrinkage and are closer to their true values. The
horseshoe prior is valuable in discriminating signal from noise.

By replacing the Laplace-distributed parameters in LASSO with
horseshoe-distributed parameters and including a global scale, the
result is called horseshoe regression.
}
\value{
  \code{dhs} gives the density and
  \code{rhs} generates random deviates.
}
\references{
  Carvalho, C.M., Polson, N.G., and Scott, J.G. (2008). "The Horseshoe
  Estimator for Sparse Signals". \emph{Discussion Paper 2008-31}. Duke
  University Department of Statistical Science.

  Carvalho, C.M., Polson, N.G., and Scott, J.G. (2009). "Handling
  Sparsity via the Horseshoe". \emph{Journal of Machine Learning
  Research}, 5, p. 73--80.
}
\seealso{
  \code{\link{dlaplace}}
}
\examples{
library(LaplacesDemon)
x <- rnorm(100)
lambda <- rhalfcauchy(100, 5)
tau <- 5
x <- dhs(x, lambda, tau, log=TRUE)
x <- rhs(100, lambda=lambda, tau=tau)
plot(density(x))
}
\keyword{Distribution}

\name{BayesianBootstrap}
\alias{BayesianBootstrap}
\title{The Bayesian Bootstrap}
\description{
  This function performs the Bayesian bootstrap of Rubin (1981),
  returning either bootstrapped weights or statistics.
}
\usage{
BayesianBootstrap(X, n=1000, Method="weights", Status=NULL)
}
\arguments{
  \item{X}{This is a vector or matrix of data. When a matrix is
    supplied, sampling is based on the first column.}
  \item{n}{This is the number of bootstrapped replications.}
  \item{Method}{When \code{Method="weights"} (which is the default),
    a matrix of row weights is returned. Otherwise, a function is
    accepted. The function specifies the statistic to be
    bootstrapped. The first argument of the function should be a matrix
    of data, and the second argument should be a vector of weights.}
  \item{Status}{This determines the periodicity of status messages. When
    \code{Status=100}, for example, a status message is displayed every
    100 replications. Otherwise, \code{Status} defaults to \code{NULL},
    and status messages are not displayed.}
}
\details{
  The term, `bootstrap', comes from the German novel \emph{Adventures of
  Baron Munchausen} by Rudolph Raspe, in which the hero saves himself
  from drowning by pulling on his own bootstraps. The idea of the
  statistical bootstrap is to evaluate properties of an estimator
  through the empirical, rather than theoretical, CDF.
  
  Rubin (1981) introduced the Bayesian bootstrap. In contrast to the
  frequentist bootstrap which simulates the sampling distribution of a
  statistic estimating a parameter, the Bayesian bootstrap simulates the
  posterior distribution.

  The data, \eqn{\textbf{X}}{X}, are assumed to be independent and
  identically distributed (IID), and to be a representative sample of
  the larger (bootstrapped) population. Given that the data has \eqn{N}
  rows in one bootstrap replication, the row weights are sampled from a
  Dirichlet distribution with all \eqn{N} concentration parameters equal
  to \eqn{1} (a uniform distribution over an open standard \eqn{N-1}
  simplex). The distributions of a parameter inferred from considering
  many samples of weights are interpretable as posterior distributions
  on that parameter.

  The Bayesian bootstrap is useful for estimating marginal posterior
  covariance and standard deviations for the posterior modes of
  \code{\link{LaplaceApproximation}}, especially when the model
  dimension (the number of parameters) is large enough that estimating
  the \code{\link{Hessian}} matrix of second partial derivatives is too
  computationally demanding.

  Just as with the frequentist bootstrap, inappropriate use of the
  Bayesian bootstrap can lead to inappropriate inferences. The Bayesian
  bootstrap violates the likelihood principle, because the evaluation of
  a statistic of interest depends on data sets other than the observed
  data set. For more information on the likelihood principle, see
  \url{https://web.archive.org/web/20150213002158/http://www.bayesian-inference.com/likelihood#likelihoodprinciple}.

  The \code{BayesianBootstrap} function has many uses, including
  creating test statistics on the population data given the observed
  data (supported here), imputation (with this variation:
  \code{\link{ABB}}), validation, and more.
}
\value{
  When \code{Method="weights"}, this function returns a
  \eqn{N \times n}{S x J} matrix of weights, where the number of rows
  \eqn{N} is equal to the number of rows in \code{X}.

  For statistics, a matrix or array is returned, depending on the number
  of dimensions. The replicates are indexed by row in a matrix or in
  the first dimension of the array.
}
\references{
  Rubin, D.B. (1981). "The Bayesian Bootstrap". \emph{The Annals of
  Statistics}, 9(1), p. 130--134.
}
\author{Bogumil Kaminski, \email{bkamins@sgh.waw.pl} and
  Statisticat, LLC.}
\seealso{
  \code{\link{ABB}},
  \code{\link{Hessian}},
  \code{\link{LaplaceApproximation}}, and
  \code{\link{LaplacesDemon}}.
}
\examples{
library(LaplacesDemon)

#Example 1: Samples
x <- 1:2
BB <- BayesianBootstrap(X=x, n=100, Method="weights"); BB

#Example 2: Mean, Univariate
x <- 1:2
BB <- BayesianBootstrap(X=x, n=100, Method=weighted.mean); BB

#Example 3: Mean, Multivariate
data(demonsnacks)
BB <- BayesianBootstrap(X=demonsnacks, n=100,
     Method=function(x,w) apply(x, 2, weighted.mean, w=w)); BB

#Example 4: Correlation
dye <- c(1.15, 1.70, 1.42, 1.38, 2.80, 4.70, 4.80, 1.41, 3.90)
efp <- c(1.38, 1.72, 1.59, 1.47, 1.66, 3.45, 3.87, 1.31, 3.75)
X <- matrix(c(dye,efp), length(dye), 2)
colnames(X) <- c("dye","efp")
BB <- BayesianBootstrap(X=X, n=100,
     Method=function(x,w) cov.wt(x, w, cor=TRUE)$cor); BB

#Example 5: Marginal Posterior Covariance
#The following example is commented out due to package build time.
#To run the following example, use the code from the examples in
#the LaplaceApproximation function for the data, model specification
#function, and initial values. Then perform the Laplace
#Approximation as below (with CovEst="Identity" and sir=FALSE) until
#convergence, set the latest initial values, then use the Bayesian
#bootstrap on the data, run the Laplace Approximation again to
#convergence, save the posterior modes, and repeat until S samples
#of the posterior modes are collected. Finally, calculate the
#parameter covariance or standard deviation.

#Fit <- LaplaceApproximation(Model, Initial.Values, Data=MyData,
#     Iterations=1000, Method="SPG",  CovEst="Identity", sir=FALSE)
#Initial.Values <- as.initial.values(Fit)
#S <- 100 #Number of bootstrapped sets of posterior modes (parameters)
#Z <- rbind(Fit$Summary1[,1]) #Bootstrapped parameters collected here
#N <- nrow(MyData$X) #Number of records
#MyData.B <- MyData
#for (s in 1:S) {
#     cat("\nIter:", s, "\n")
#     BB <- BayesianBootstrap(MyData$y, n=N)
#     z <- apply(BB, 2, function(x) sample.int(N, size=1, prob=x))
#     MyData.B$y <- MyData$y[z]
#     MyData.B$X <- MyData$X[z,]
#     Fit <- LaplaceApproximation(Model, Initial.Values, Data=MyData.B,
#          Iterations=1000, Method="SPG", CovEst="Identity", sir=FALSE)
#     Z <- rbind(Z, Fit$Summary1[,1])}
#cov(Z) #Bootstrapped marginal posterior covariance
#sqrt(diag(cov(Z))) #Bootstrapped marginal posterior standard deviations
}
\keyword{Utility}

\name{data.demonfx}
\alias{demonfx}
\title{Demon FX Data Set}
\usage{data(demonfx)}
\description{
  This data set consists of daily currency pair prices from 2010 through
  2014. Each currency pair has a close, high, and low price.
}
\format{
  This data frame contains 1,301 rows as time-periods (with row names) and
  39 columns of currency pair prices. The following data dictionary describes
  each time-series or column.
  \describe{
    \item{\code{EURUSD.Close}}{This is the currency pair closing price.}
    \item{\code{EURUSD.High}}{This is the currency pair high price.}
    \item{\code{EURUSD.Low}}{This is the currency pair low price.}
    \item{\code{USDJPY.Close}}{This is the currency pair closing price.}
    \item{\code{USDJPY.High}}{This is the currency pair high price.}
    \item{\code{USDJPY.Low}}{This is the currency pair low price.}
    \item{\code{USDCHF.Close}}{This is the currency pair closing price.}
    \item{\code{USDCHF.High}}{This is the currency pair high price.}
    \item{\code{USDCHF.Low}}{This is the currency pair low price.}
    \item{\code{GBPUSD.Close}}{This is the currency pair closing price.}
    \item{\code{GBPUSD.High}}{This is the currency pair high price.}
    \item{\code{GBPUSD.Low}}{This is the currency pair low price.}
    \item{\code{USDCAD.Close}}{This is the currency pair closing price.}
    \item{\code{USDCAD.High}}{This is the currency pair high price.}
    \item{\code{USDCAD.Low}}{This is the currency pair low price.}
    \item{\code{EURGBP.Close}}{This is the currency pair closing price.}
    \item{\code{EURGBP.High}}{This is the currency pair high price.}
    \item{\code{EURGBP.Low}}{This is the currency pair low price.}
    \item{\code{EURJPY.Close}}{This is the currency pair closing price.}
    \item{\code{EURJPY.High}}{This is the currency pair high price.}
    \item{\code{EURJPY.Low}}{This is the currency pair low price.}
    \item{\code{EURCHF.Close}}{This is the currency pair closing price.}
    \item{\code{EURCHF.High}}{This is the currency pair high price.}
    \item{\code{EURCHF.Low}}{This is the currency pair low price.}
    \item{\code{AUDUSD.Close}}{This is the currency pair closing price.}
    \item{\code{AUDUSD.High}}{This is the currency pair high price.}
    \item{\code{AUDUSD.Low}}{This is the currency pair low price.}
    \item{\code{GBPJPY.Close}}{This is the currency pair closing price.}
    \item{\code{GBPJPY.High}}{This is the currency pair high price.}
    \item{\code{GBPJPY.Low}}{This is the currency pair low price.}
    \item{\code{CHFJPY.Close}}{This is the currency pair closing price.}
    \item{\code{CHFJPY.High}}{This is the currency pair high price.}
    \item{\code{CHFJPY.Low}}{This is the currency pair low price.}
    \item{\code{GBPCHF.Close}}{This is the currency pair closing price.}
    \item{\code{GBPCHF.High}}{This is the currency pair high price.}
    \item{\code{GBPCHF.Low}}{This is the currency pair low price.}
    \item{\code{NZDUSD.Close}}{This is the currency pair closing price.}
    \item{\code{NZDUSD.High}}{This is the currency pair high price.}
    \item{\code{NZDUSD.Low}}{This is the currency pair low price.}
    }
}
\source{Originally downloaded from a database that is not available anymore. See: \url{https://web.archive.org/web/20150222115543/https://www.global-view.com/forex-trading-tools/forex-history/index.html}}
\keyword{datasets}

\name{Hangartner.Diagnostic}
\alias{Hangartner.Diagnostic}
\title{Hangartner's Convergence Diagnostic}
\description{
  Hangartner et al. (2011) proposed a convergence diagnostic for
  discrete Markov chains. A simple Pearson's Chi-squared test for
  two or more non-overlapping periods of a discrete Markov chain
  is a reliable diagnostic of convergence. It does not rely upon the
  estimation of spectral density, on suspect normality assumptions, or
  determining overdispersion within a small number of outcomes, all of
  which can be problematic with discrete measures. A discrete Markov
  chain is split into two or more non-overlapping windows. Two windows
  are recommended, and results may be sensitive to the number of
  selected windows, as well as sample size. As such, a user may try
  several window configurations before concluding there is no evidence
  of non-convergence.

  As the number of discrete events in the sample space increases, this
  diagnostic becomes less appropriate and standard diagnostics become
  more appropriate.
}
\usage{
Hangartner.Diagnostic(x, J=2)
}
\arguments{
  \item{x}{This required argument is a vector of marginal posterior
    samples of a discrete Markov chain, such as selected from the output
    of \code{\link{LaplacesDemon}}.}
  \item{J}{This argument specifies the number \eqn{J} of windows to be
    used, and defaults to \eqn{J=2}.}
}
\value{
  The \code{Hangartner.Diagnostic} returns an object of class
  \code{hangartner}, including the output from a Pearson's Chi-squared
  test. A frequentist p-value less than or equal to 0.05 is usually
  considered to be indicative of non-convergence.
}
\references{
  Hangartner, D., Gill, J., and Cranmer, S., (2011). "An MCMC Diagnostic
  for Purely Discrete Parameters". Paper presented at the annual meeting
  of the Southern Political Science Association, Hotel InterContinental,
  New Orleans, Louisiana Online.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{LaplacesDemon}} and
  \code{\link{TransitionMatrix}}.
}
\examples{
library(LaplacesDemon)
N <- 1000
K <- 3
x <- rcat(N, rep(1/K,K))
hd <- Hangartner.Diagnostic(x, J=2)
hd
}
\keyword{Diagnostic}
\keyword{MCMC}

\name{LossMatrix}
\alias{LossMatrix}
\title{Loss Matrix}
\description{
  A loss matrix is useful in Bayesian decision theory for selecting the
  Bayes action, the optimal Bayesian decision, when there are a discrete
  set of possible choices (actions) and a discrete set of possible
  outcomes (states of the world). The Bayes action is the action that
  minimizes expected loss, which is equivalent to maximizing expected
  utility.
}
\usage{
LossMatrix(L, p.theta)
}
\arguments{
  \item{L}{This required argument accepts a \eqn{S \times A}{S x A}
    matrix or \eqn{S \times A \times N}{S x A x N} array of losses,
    where \eqn{S} is the number of states of the world, \eqn{A} is the
    number of actions, and \eqn{N} is the number of samples. These
    losses have already been estimated, given a personal loss
    function. One or more personal losses has already been estimated
    for each combination of possible actions
    \eqn{a=1,\dots,A}{a=1,...,A} and possible states
    \eqn{s=1,\dots,S}{s=1,...,S}.}
  \item{p.theta}{This required argument accepts a
    \eqn{S \times A}{S x A} matrix or
    \eqn{S \times A \times N}{S x A x N} array of state prior
    probabilities, where \eqn{S} is the number of states of the world,
    \eqn{A} is the number of actions, and \eqn{N} is the number of
    samples. The sum of each column must equal one.}
}
\details{
  Bayesian inference is often tied to decision theory (Bernardo and
  Smith, 2000), and decision theory has long been considered the
  foundations of statistics (Savage, 1954).

  Before using the \code{LossMatrix} function, the user should have
  already considered all possible actions (choices), states of the world
  (outcomes unknown at the time of decision-making), chosen a loss
  function \eqn{L(\theta, \alpha)}{L(theta, alpha)}, estimated loss, and
  elicited prior probabilities \eqn{p(\theta | x)}{p(theta | x)}.
  
  Although possible actions (choices) for the decision-maker and
  possible states (outcomes) may be continuous or discrete, the loss
  matrix is used for discrete actions and states. An example of a
  continuous action may be that a decision-maker has already decided to
  invest, and the remaining, current decision is how much to invest.
  Likewise, an example of continuous states of the world (outcomes) may
  be how much profit or loss may occur after a given continuous unit of
  time.

  The coded example provided below is taken from Berger (1985, p. 6-7)
  and described here. The set of possible actions for a decision-maker
  is to invest in bond ZZZ or alternatively in bond XXX, as it is called
  here. A real-world decision should include a mutually exhaustive list
  of actions, such as investing in neither, but perhaps the
  decision-maker has already decided to invest and narrowed the options
  down to these two bonds.

  The possible states of the world (outcomes unknown at the time of
  decision-making) are considered to be two states: either the chosen
  bond will not default or it will default. Here, the loss function is
  a negative linear identity of money, and hence a loss in element
  \code{L[1,1]} of -500 is a profit of 500, while a loss in
  \code{L[2,1]} of 1,000 is a loss of 1,000.

  The decision-maker's dilemma is that bond ZZZ may return a higher
  profit than bond XXX, however there is an estimated 10\% chance, the
  prior probability, that bond ZZZ will default and return a substantial
  loss. In contrast, bond XXX is considered to be a sure-thing and 
  return a steady but smaller profit. The Bayes action is to choose the
  first action and invest in bond ZZZ, because it minimizes expected
  loss, even though there is a chance of default.

  A more realistic application of a loss matrix may be to replace the
  point-estimates of loss with samples given uncertainty around the
  estimated loss, and replace the point-estimates of the prior
  probability of each state with samples given the uncertainty of the
  probability of each state. The loss function used in the example is
  intuitive, but a more popular monetary loss function may be
  \eqn{-\log(E(W | R))}{-log(E(W | R))}, the negative log of the
  expectation of wealth, given the return. There are many alternative
  loss functions.

  Although isolated decision-theoretic problems exist such as the
  provided example, decision theory may also be applied to the results
  of a probability model (such as from
  \code{\link{IterativeQuadrature}}, \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, \code{\link{PMC}}), or
  \code{\link{VariationalBayes}}, contingent on how
  a decision-maker is considering to use the information from the
  model. The statistician may pass the results of a model to a client,
  who then considers choosing possible actions, given this
  information. The statistician should further assist the client with
  considering actions, states of the world, then loss functions, and
  finally eliciting the client's prior probabilities (such as with the
  \code{\link{elicit}} function).

  When the outcome is finally observed, the information from this
  outcome may be used to refine the priors of the next such decision. In
  this way, Bayesian learning occurs.
}
\value{
  The \code{LossMatrix} function returns a list with two components:
  \item{BayesAction}{This is a numeric scalar that indicates the action
    that minimizes expected loss.}
  \item{E.Loss}{This is a vector of expected losses, one for each
    action.}
}
\references{
  Berger, J.O. (1985). "Statistical Decision Theory and Bayesian
  Analysis", Second Edition. Springer: New York, NY.

  Bernardo, J.M. and Smith, A.F.M. (2000). "Bayesian Theory". John
  Wiley & Sons: West Sussex, England.

  Savage, L.J. (1954). "The Foundations of Statistics". John Wiley &
  Sons: West Sussex, England.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{elicit}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
library(LaplacesDemon)
### Point-estimated loss and state probabilities
L <- matrix(c(-500,1000,-300,-300), 2, 2)
rownames(L) <- c("s[1]: !Defaults","s[2]: Defaults")
colnames(L) <- c("a[1]: Buy ZZZ", "a[2]: Buy XXX")
L
p.theta <- matrix(c(0.9, 0.1, 1, 0), 2, 2)
Fit <- LossMatrix(L, p.theta)

### Point-estimated loss and samples of state probabilities
L <- matrix(c(-500,1000,-300,-300), 2, 2)
rownames(L) <- c("s[1]: Defaults","s[2]: !Defaults")
colnames(L) <- c("a[1]: Buy ZZZ", "a[2]: Buy XXX")
L
p.theta <- array(runif(4000), dim=c(2,2,1000)) #Random probabilities,
#just for a quick example. And, since they must sum to one:
for (i in 1:1000) {
     p.theta[,,i] <- p.theta[,,i] / matrix(colSums(p.theta[,,i]),
          dim(p.theta)[1], dim(p.theta)[2], byrow=TRUE)}
Fit <- LossMatrix(L, p.theta)
Fit

### Point-estimates of loss may be replaced with samples as well.
}
\keyword{Decision Theory}
\keyword{Utility}

\name{summary.miss}
\alias{summary.miss}
\title{MISS Summary}
\description{
  This function summarizes posterior predictive distributions from
  an object of class \code{miss}.
}
\usage{\method{summary}{miss}(object, \dots)}
\arguments{
  \item{object}{An object of class \code{miss} is required.}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  This function summarizes the posterior predictive distributions from
  an object of class \code{miss}.
}
\value{
  This function returns a \eqn{M \times 7}{M x 7} matrix, in which each
  row is the posterior predictive distribution of one of \eqn{M} missing
  values. Columns are Mean, SD, MCSE, ESS, LB, Median, and UB.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{MISS}}.
}
\examples{### See the MISS function for an example.}
\keyword{Imputation}
\keyword{summary}

\name{dist.Inverse.Beta}
\alias{dinvbeta}
\alias{rinvbeta}
\title{Inverse Beta Distribution}
\description{
  This is the density function and random generation from the inverse
  beta distribution.
}
\usage{
dinvbeta(x, a, b, log=FALSE)
rinvbeta(n, a, b)
}
\arguments{
  \item{n}{This is the number of draws from the distribution.}
  \item{x}{This is a location vector at which to evaluate density.}
  \item{a}{This is the scalar shape parameter \eqn{\alpha}{alpha}.}
  \item{b}{This is the scalar shape parameter \eqn{\beta}{beta}}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Univariate
    \item Density: \eqn{p(\theta) = \frac{\theta^{\alpha - 1} (1 +
	\theta)^{-\alpha - \beta}}{\beta(\alpha, \beta)}}{(theta^(alpha -
      1) * (1 + theta)^(-alpha - beta)) / beta(alpha, beta)}
    \item Inventor: Dubey (1970)
    \item Notation 1: \eqn{\theta \sim \mathcal{B}^{-1}(\alpha, \beta)}{theta ~ B^-1(alpha, beta)}
    \item Notation 2: \eqn{p(\theta) = \mathcal{B}^{-1}(\theta | \alpha,
    \beta)}{p(theta) = B^-1(theta | alpha, beta)}
    \item Parameter 1: shape \eqn{\alpha > 0}{alpha > 0}
    \item Parameter 2: shape \eqn{\beta > 0}{beta > 0}
    \item Mean: \eqn{E(\theta) = \frac{\alpha}{\beta - 1}}{E(theta) =
      alpha / (beta - 1)}, for \eqn{\beta > 1}{beta > 1}
    \item Variance: \eqn{var(\theta) = \frac{\alpha(\alpha + \beta -
	1)}{(\beta - 1)^2 (\beta - 2)}}{var(theta) = (alpha * (alpha + beta
      - 1)) / ((beta - 1)^2 * (beta - 2))}
    \item Mode: \eqn{mode(\theta) = \frac{\alpha - 1}{\beta +
	1}}{mode(theta) = (alpha - 1) / (beta + 1)}
}

The inverse-beta, also called the beta prime distribution, applies to
variables that are continuous and positive. The inverse beta is the
conjugate prior distribution of a parameter of a Bernoulli distribution
expressed in odds.

The inverse-beta distribution has also been extended to the generalized
beta prime distribution, though it is not (yet) included here.

}
\value{
  \code{dinvbeta} gives the density and
  \code{rinvbeta} generates random deviates.
}
\references{
  Dubey, S.D. (1970). "Compound Gamma, Beta and F Distributions".
  \emph{Metrika}, 16, p. 27--31.
}
\seealso{
  \code{\link{dbeta}}
}
\examples{
library(LaplacesDemon)
x <- dinvbeta(5:10, 2, 3)
x <- rinvbeta(10, 2, 3)

#Plot Probability Functions
x <- seq(from=0.1, to=20, by=0.1)
plot(x, dinvbeta(x,2,2), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dinvbeta(x,2,3), type="l", col="green")
lines(x, dinvbeta(x,3,2), type="l", col="blue")
legend(2, 0.9, expression(paste(alpha==2, ", ", beta==2),
     paste(alpha==2, ", ", beta==3), paste(alpha==3, ", ", beta==2)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{Model.Specification.Time}
\alias{Model.Spec.Time}
\title{Model Specification Time}
\description{
  The \code{Model.Spec.Time} function returns the time in minutes to
  evaluate a model specification a number of times, as well as
  the evaluations per minute, and componentwise iterations per minute.
}
\usage{
Model.Spec.Time(Model, Initial.Values, Data, n=1000)
}
\arguments{
  \item{Model}{This requried argument is a model specification
    function. For more information, see \code{\link{LaplacesDemon}}.}
  \item{Initial.Values}{This required argument is a vector of initial
    values for the parameters.}
  \item{Data}{This required argument is a list of data. For more
    information, see \code{\link{LaplacesDemon}}.}
  \item{n}{This is the number of evaluations of the model specification,
    and accuracy increases with \code{n}.}
}
\details{
  The largest single factor to affect the run-time of an algorithm --
  whether it is with \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}}, \code{\link{LaplacesDemon}},
  \code{\link{PMC}}, or \code{\link{VariationalBayes}} -- is the time
  that it takes to evaluate the model specification. This has also been
  observed in Rosenthal (2007). It is highly recommended that a user of
  the \code{LaplacesDemon} package should attempt to reduce the run-time
  of the model specification, usually by testing alternate forms of code
  for speed. This is especially true with big data, such as with the
  \code{\link{BigData}} function.

  Every function in the LaplacesDemon package is byte-compiled, which is
  a recent option in R. This reduces run-time, thanks to Tierney's
  compiler package in base R. The model specification, however, is
  specified by the user, and should be byte-compiled. The reduction in
  run-time may range from mild to dramatic, depending on the model. It
  is highly recommended that users concerned with run-time activate the
  compiler package and use the \code{cmpfun} function, as per the
  example below.
  
  A model specification function that is optimized for speed and
  involves many records may result in a model update run-time that is
  considerably less than other popular MCMC-based software algorithms
  that loop through records, even when those algorithms are coded in
  \code{C} or other fast languages. For a comparison, see the
  ``Laplace's Demon Tutorial'' vignette.

  However, if a model specification function in the LaplacesDemon
  package is not fully vectorized (contains \code{for} loops and
  \code{apply} functions), then run-time will typically be slower than
  other popular MCMC-based software algorithms.

  The speed of calculating the model specification function is
  affected by parameter constraints, such as with the
  \code{\link{interval}} function. Parameter constraints may add
  considerable variability to the speed of this calculation, and usually
  more variation occurs with initial values that are far from the target
  distributions.

  Distributions in the \code{LaplacesDemon} package usually have logical
  checks to ensure correctness. These checks may slow the 
  calculation of the density, for example. If the user is confident
  these checks are unnecessary for their model, then the user may
  copy the code to a new function name and comment-out the checks to
  improve speed.

  When speed is of paramount importance, a user is encouraged to
  experiment with writing the model specification function in another
  language such as in \code{C++} with the \code{Rcpp} package, and
  calling drop-in replacement functions from within the \code{Model}
  function, or re-writing the model function entirely in \code{C++}.
  For an introduction to including \code{C++} in \pkg{LaplacesDemon},
  see \url{https://web.archive.org/web/20150227225556/http://www.bayesian-inference.com/softwarearticlescppsugar}.

  When a model specification function is computationally expensive,
  another approach to reduce run-time may be for the user to write
  parallelized code within the model, splitting up difficult tasks and
  assigning each to a separate CPU.

  Another use for \code{Model.Spec.Time} is to allow the user to make an
  informed decision about which MCMC algorithm to select, given the
  speed of their model specification. For example, the Adaptive
  Metropolis-within-Gibbs (AMWG) of Roberts and Rosenthal (2009) is
  currently the adaptive MCMC algorithm of choice in general in many
  cases, but this choice is conditional on run-time. While other
  MCMC algorithms in \code{LaplacesDemon} evaluate the model
  specification function once per iteration, componentwise algorithms
  such as in the MWG family evaluate it once per parameter per
  iteration, significantly increasing run-time per iteration in large
  models. The \code{Model.Spec.Time} function may forewarn the user of
  the associated run-time, and if it should be decided to go with an
  alternate MCMC algorithm, such as AMM, in which each element of its
  covariance matrix must stabilize for the chains to become
  stationary. AMM, for example, will require many more iterations of 
  burn-in (for more information, see the \code{\link{burnin}} function),
  but with numerous iterations, allows more thinning. A general
  recommendation may be to use AMWG when
  \code{Componentwise.Iters.per.Minute} >= 1000, but this is subjective
  and best determined by each user for each model. A better decision may
  be made by comparing MCMC algorithms with the \code{\link{Juxtapose}}
  function for a particular model.
  
  Following are a few common suggestions for increasing the speed of
  \code{R} code in the model specification function. There are often
  exceptions to these suggestions, so case-by-case experimentation is
  also suggested.

  \itemize{
    \item Replace exponents with multiplied terms, such as \code{x^2}
    with \code{x*x}.
    \item Replace \code{mean(x)} with \code{sum(x)/length(x)}.
    \item Replace parentheses (when possible) with curly brackets, such
    as \code{x*(a+b)} with \code{x*\{a+b\}}.
    \item Replace \code{tcrossprod(Data$X, t(beta))} with
    \code{Data$X \%*\% beta} when there are few predictors, and avoid
    \code{tcrossprod(beta, Data$X)}, which is always slowest.
    \item Vectorize functions and eliminate \code{apply} and \code{for}
    functions. There are often specialized functions. For example,
    \code{max.col(X)} is faster than \code{apply(X, 1, which.max)}.
  }

  When seeking speed, things to consider beyond the LaplacesDemon
  package are the Basic Linear Algebra System (BLAS) and hardware. The
  ATLAS (Automatically Tuned Linear Algebra System) should be worth
  installing (and there are several alternatives), but this discussion
  is beyond the scope of this documentation. Lastly, the speed at which
  the computer can process iterations is limited by its hardware, and
  more should be considered than merely the CPU (Central Processing
  Unit). Again, though, this is beyond the scope of this documentation.
}
\value{
  The \code{Model.Spec.Time} function returns a list with three
  components:

  \item{Time}{This is the time in minutes to evaluate the model
    specification \code{n} times.}
  \item{Evals.per.Minute}{This is the number of evaluations of the
    model specification per minute: \code{n} / \code{Time}. This is also
    the number of iterations per minute in an algorithm that is not
    componentwise, where one evaluation occurs per iteration.}
  \item{Componentwise.Iters.per.Minute}{This is the number of iterations
    per minute in a componentwise MCMC algorithm, such as AMWG or
    MWG. It is the evaluations per minute divided by the number of
    parameters, since an evaluation must occur for each parameter, for
    each iteration.}
}
\references{
  Roberts, G.O. and Rosenthal, J.S. (2009). "Examples of Adaptive
  MCMC". \emph{Computational Statistics and Data Analysis}, 18,
  p. 349--367.
}
\author{Statisticat, LLC.}
\seealso{
  \code{\link{.C}},
  \code{\link{.Fortran}},

  \code{\link{apply}},
  \code{\link{BigData}},
  \code{\link{interval}},
  \code{\link{IterativeQuadrature}},
  \code{\link{Juxtapose}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{max.col}},
  \code{\link{PMC}},
  \code{\link{system.time}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
# The accompanying Examples vignette is a compendium of examples.
####################  Load the LaplacesDemon Library  #####################
library(LaplacesDemon)

##############################  Demon Data  ###############################
data(demonsnacks)
y <- log(demonsnacks$Calories)
X <- cbind(1, as.matrix(log(demonsnacks[,c(1,4,10)]+1)))
J <- ncol(X)
for (j in 2:J) {X[,j] <- CenterScale(X[,j])}

#########################  Data List Preparation  #########################
mon.names <- "LP"
parm.names <- as.parm.names(list(beta=rep(0,J), sigma=0))
pos.beta <- grep("beta", parm.names)
pos.sigma <- grep("sigma", parm.names)
PGF <- function(Data) return(c(rnormv(Data$J,0,10), rhalfcauchy(1,5)))
MyData <- list(J=J, PGF=PGF, X=X, mon.names=mon.names,
     parm.names=parm.names, pos.beta=pos.beta, pos.sigma=pos.sigma, y=y)

##########################  Model Specification  ##########################
Model <- function(parm, Data)
     {
     ### Parameters
     beta <- parm[Data$pos.beta]
     sigma <- interval(parm[Data$pos.sigma], 1e-100, Inf)
     parm[Data$pos.sigma] <- sigma
     ### Log of Prior Densities
     beta.prior <- sum(dnormv(beta, 0, 1000, log=TRUE))
     sigma.prior <- dhalfcauchy(sigma, 25, log=TRUE)
     ### Log-Likelihood
     mu <- tcrossprod(Data$X, t(beta))
     LL <- sum(dnorm(Data$y, mu, sigma, log=TRUE))
     ### Log-Posterior
     LP <- LL + beta.prior + sigma.prior
     Modelout <- list(LP=LP, Dev=-2*LL, Monitor=LP,
          yhat=rnorm(length(mu), mu, sigma), parm=parm)
     return(Modelout)
     }

set.seed(666)

############################  Initial Values  #############################
Initial.Values <- GIV(Model, MyData, PGF=TRUE)

############################  Model.Spec.Time  ############################
### Not byte-compiled
Model.Spec.Time(Model, Initial.Values, MyData)
### Byte-compiled
library(compiler)
Model <- cmpfun(Model)
Model.Spec.Time(Model, Initial.Values, MyData)
}
\keyword{Utility}

\name{dist.Inverse.Gamma}
\alias{dinvgamma}
\alias{rinvgamma}
\title{Inverse Gamma Distribution}
\description{
  This is the density function and random generation from the inverse
  gamma distribution.
}
\usage{
dinvgamma(x, shape=1, scale=1, log=FALSE)
rinvgamma(n, shape=1, scale=1)
}
\arguments{
  \item{n}{This is the number of draws from the distribution.}
  \item{x}{This is the scalar location to evaluate density.}
  \item{shape}{This is the scalar shape parameter \eqn{\alpha}{alpha},
    which defaults to one.}
  \item{scale}{This is the scalar scale parameter \eqn{\beta}{beta},
    which defaults to one.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Univariate
    \item Density: \eqn{p(\theta) = \frac{\beta^\alpha}{\Gamma(\alpha)}
      \theta^{-(\alpha + 1)} \exp(-\frac{\beta}{\theta}), \quad \theta > 0}{p(theta) = (beta^alpha / Gamma(alpha)) *
      theta^(-(alpha + 1)) * exp(-beta / theta), theta > 0}
    \item Inventor: Unknown (to me, anyway)
    \item Notation 1: \eqn{\theta \sim \mathcal{G}^{-1}(\alpha, \beta)}{theta ~ G^-1(alpha, beta)}
    \item Notation 2: \eqn{p(\theta) = \mathcal{G}^{-1}(\theta | \alpha,
    \beta)}{p(theta) = G^-1(theta | alpha, beta)}
    \item Parameter 1: shape \eqn{\alpha > 0}{alpha > 0}
    \item Parameter 2: scale \eqn{\beta > 0}{beta > 0}
    \item Mean: \eqn{E(\theta) = \frac{\beta}{\alpha - 1}}{E(theta) =
      beta / (alpha - 1)}, for \eqn{\alpha > 1}{alpha > 1}
    \item Variance: \eqn{var(\theta) = \frac{\beta^2}{(\alpha - 1)^2
	(\alpha - 2)}, \alpha > 2}{var(theta) = beta^2 / ((alpha - 1)^2 * (alpha - 2)), alpha > 2}
    \item Mode: \eqn{mode(\theta) = \frac{\beta}{\alpha + 1}}{mode(theta) = beta / (alpha + 1)}
}

The inverse-gamma is the conjugate prior distribution for the normal
or Gaussian variance, and has been traditionally specified as a vague
prior in that application. The density is always finite; its integral is
finite if \eqn{\alpha > 0}{alpha > 0}. Prior information decreases as
\eqn{\alpha, \beta \rightarrow 0}{alpha, beta -> 0}.

These functions are similar to those in the \code{MCMCpack} package.
}
\value{
  \code{dinvgamma} gives the density and
  \code{rinvgamma} generates random deviates. The parameterization
  is consistent with the Gamma Distribution in the stats package.
}
\seealso{
  \code{\link{dgamma}},
  \code{\link{dnorm}},
  \code{\link{dnormp}}, and
  \code{\link{dnormv}}.
}
\examples{
library(LaplacesDemon)
x <- dinvgamma(4.3, 1.1)
x <- rinvgamma(10, 3.3)

#Plot Probability Functions
x <- seq(from=0.1, to=20, by=0.1)
plot(x, dinvgamma(x,1,1), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dinvgamma(x,1,0.6), type="l", col="green")
lines(x, dinvgamma(x,0.6,1), type="l", col="blue")
legend(2, 0.9, expression(paste(alpha==1, ", ", beta==1),
     paste(alpha==1, ", ", beta==0.6), paste(alpha==0.6, ", ", beta==1)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{summary.vb.ppc}
\alias{summary.vb.ppc}
\title{Posterior Predictive Check Summary}
\description{
  This may be used to summarize either new, unobserved instances of
  \eqn{\textbf{y}}{y} (called \eqn{\textbf{y}^{new}}{y[new]}) or
  replicates of \eqn{\textbf{y}}{y} (called
  \eqn{\textbf{y}^{rep}}{y[rep]}). Either \eqn{\textbf{y}^{new}}{y[new]} or
  \eqn{\textbf{y}^{rep}}{y[rep]} is summarized, depending on
  \code{\link{predict.vb}}.
}
\usage{\method{summary}{vb.ppc}(object, Categorical, Rows, Discrep,
     d, Quiet, \dots)}
\arguments{
  \item{object}{An object of class \code{vb.ppc} is required.}
  \item{Categorical}{Logical. If \code{TRUE}, then \code{y} and
    \code{yhat} are considered to be categorical (such as y=0 or y=1),
    rather than continuous.}
  \item{Rows}{An optional vector of row numbers, for example
    \code{c(1:10)}. All rows will be estimated, but only these rows will
    appear in the summary.}
  \item{Discrep}{A character string indicating a discrepancy
    test. \code{Discrep} defaults to \code{NULL}. Valid character
    strings when \code{y} is continuous are: \code{"Chi-Square"},
    \code{"Chi-Square2"}, \code{"Kurtosis"}, \code{"L-criterion"},
    \code{"MASE"}, \code{"MSE"}, \code{"PPL"}, \code{"Quadratic Loss"},
    \code{"Quadratic Utility"}, \code{"RMSE"}, \code{"Skewness"},
    \code{"max(yhat[i,]) > max(y)"}, \code{"mean(yhat[i,]) > mean(y)"},
    \code{"mean(yhat[i,] > d)"}, \code{"mean(yhat[i,] > mean(y))"},
    \code{"min(yhat[i,]) < min(y)"}, \code{"round(yhat[i,]) = d"}, and
    \code{"sd(yhat[i,]) > sd(y)"}. Valid character strings when \code{y}
    is categorical are: \code{"p(yhat[i,] != y[i])"}. Kurtosis and
    skewness are not discrepancies, but are included here for convenience.}
  \item{d}{This is an optional integer to be used with the
    \code{Discrep} argument above, and it defaults to \code{d=0}.}
  \item{Quiet}{This logical argument defaults to \code{FALSE} and will
    print results to the console. When \code{TRUE}, results are not
    printed.}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  This function summarizes an object of class \code{vb.ppc}, which
  consists of posterior predictive checks on either
  \eqn{\textbf{y}^{new}}{y[new]} or \eqn{\textbf{y}^{rep}}{y[rep]},
  depending respectively on whether unobserved instances of
  \eqn{\textbf{y}}{y} or the model sample of \eqn{\textbf{y}}{y} was
  used in the \code{\link{predict.vb}} function. The deviance and
  monitored variables are also summarized.
  
  The purpose of a posterior predictive check is to assess how well (or
  poorly) the model fits the data, or to assess discrepancies between
  the model and the data. For more information on posterior predictive
  checks, see
  \url{https://web.archive.org/web/20150215050702/http://www.bayesian-inference.com/posteriorpredictivechecks}.

  When \eqn{\textbf{y}}{y} is continuous and known, this function
  estimates the predictive concordance between \eqn{\textbf{y}}{y} and
  \eqn{\textbf{y}^{rep}}{y[rep]} as per Gelfand (1996), and the
  predictive quantile (PQ), which is for record-level outlier detection
  used to calculate Gelfand's predictive concordance.

  When \eqn{\textbf{y}}{y} is categorical and known, this function
  estimates the record-level lift, which is
  \code{p(yhat[i,] = y[i]) / [p(y = j) / n]}, or
  the number of correctly predicted samples over the rate of that
  category of \eqn{\textbf{y}}{y} in vector \eqn{\textbf{y}}{y}.

  A discrepancy measure is an approach to studying discrepancies between
  the model and data (Gelman et al., 1996). Below is a list of
  discrepancy measures, followed by a brief introduction to discrepancy
  analysis:

  \itemize{
    \item The \code{"Chi-Square"} discrepancy measure is the chi-square
  goodness-of-fit test that is recommended by Gelman. For each record
  i=1:N, this returns (y[i] - E(y[i]))^2 / var(yhat[i,]).
    \item The \code{"Chi-Square2"} discrepancy measure returns the
  following for each record: Pr(chisq.rep[i,] > chisq.obs[i,]), where
  chisq.obs[i,] <- (y[i] - E(y[i]))^2 / E(y[i]), and chisq.rep[i,] <-
  (yhat[i,] - E(yhat[i,]))^2 / E(yhat[i,]), and the overall
  discrepancy is the percent of records that were outside of the 95\%
  quantile-based probability interval (see \code{\link{p.interval}}).
    \item The \code{"Kurtosis"} discrepancy measure returns the kurtosis
  of \eqn{\textbf{y}^{rep}}{y[rep]} for each record, and the discrepancy
  statistic is the mean for all records. This does not measure
  discrepancies between the model and data, and is useful for finding
  kurtotic replicate distributions.
    \item The \code{"L-criterion"} discrepancy measure of Laud and Ibrahim
  (1995) provides the record-level combination of two components (see
  below), and the discrepancy statistic is the sum, \code{L}, as well as
  a calibration number, \code{S.L}. For more information on the
  L-criterion, see the accompanying vignette entitled "Bayesian
  Inference".
    \item The \code{"MASE"} (Mean Absolute Scaled Error) is a
  discrepancy measure for the accuracy of time-series forecasts,
  estimated as \code{(|y - yhat|) / mean(abs(diff(y)))}. The discrepancy
  statistic is the mean of the record-level values.
    \item The \code{"MSE"} (Mean Squared Error) discrepancy measure
  provides the MSE for each record across all replicates, and the
  discrepancy statistic is the mean of the record-level MSEs. MSE and
  quadratic loss are identical.
    \item The \code{"PPL"} (Posterior Predictive Loss) discrepancy
  measure of Gelfand and Ghosh (1998) provides the record-level
  combination of two components: one involves the predictive variance
  and the other includes the accuracy of the means of the predictive
  distribution. The \code{d=0} argument applies the following weight to
  the accuracy component, which is then added to the variance component:
  \eqn{d/(d+1)}. For \eqn{\textbf{y}^{new}}{y[new]}, use \eqn{d=0}. For
  \eqn{\textbf{y}^{rep}}{y[rep]} and model comparison, \eqn{d} is
  commonly set to 1, 10, or 100000. Larger values of \eqn{d} put more
  stress on fit and downgrade the precision of the estimates.
    \item The \code{"Quadratic Loss"} discrepancy measure provides the
  mean quadratic loss for each record across all replicates, and the
  discrepancy statistic is the mean of the record-level mean quadratic
  losses. Quadratic loss and MSE are identical, and quadratic loss is
  the negative of quadratic utility.
    \item The \code{"Quadratic Utility"} discrepancy measure provides
  the mean quadratic utility for each record across all replicates, and
  the discrepancy statistic is the mean of the record-level mean
  quadratic utilities. Quadratic utility is the negative of quadratic
  loss.
    \item The \code{"RMSE"} (Root Mean Squared Error) discrepancy
  measure provides the RMSE for each record across all replicates, and
  the discrepancy statistic is the mean of the record-level RMSEs.
    \item The \code{"Skewness"} discrepancy measure returns the skewness
  of \eqn{\textbf{y}^{rep}}{y[rep]} for each record, and the discrepancy
  statistic is the mean for all records. This does not measure
  discrepancies between the model and data, and is useful for finding
  skewed replicate distributions.
    \item The \code{"max(yhat[i,]) > max(y)"} discrepancy measure
  returns a record-level indicator when a record's maximum
  \eqn{\textbf{y}^{rep}_i}{y[rep][i]} exceeds the maximum of
  \eqn{\textbf{y}}{y}. The discrepancy statistic is the mean of the
  record-level indicators, reporting the proportion of records with
  replications that exceed the maximum of \eqn{\textbf{y}}{y}.
    \item The \code{"mean(yhat[i,]) > mean(y)"} discrepancy measure
    returns a record-level indicator when the mean of a record's
  \eqn{\textbf{y}^{rep}_i}{y[rep][i]} is greater than the mean of
  \eqn{\textbf{y}}{y}. The discrepancy statistic is the mean of the
  record-level indicators, reporting the proportion of records with
  mean replications that exceed the mean of \eqn{\textbf{y}}{y}.
    \item The \code{"mean(yhat[i,] > d)"} discrepancy measure returns a
  record-level proportion of \eqn{\textbf{y}^{rep}_i}{y[rep][i]} that
  exceeds a specified value, \code{d}. The discrepancy statistic is the
  mean of the record-level proportions.
    \item The \code{"mean(yhat[i,] > mean(y))"} discrepancy measure
  returns a record-level proportion of
  \eqn{\textbf{y}^{rep}_i}{y[rep][i]} that exceeds the mean of
  \eqn{\textbf{y}}{y}. The discrepancy statistic is the mean of the
  record-level proportions.
    \item The \code{"min(yhat[i,]) < min(y)"} discrepancy measure
  returns a record-level indicator when a record's minimum
  \eqn{\textbf{y}^{rep}_i}{y[rep][i]} is less than the minimum of
  \eqn{\textbf{y}}{y}. The discrepancy statistic is the mean of the
  record-level indicators, reporting the proportion of records with
  replications less than the minimum of \eqn{\textbf{y}}{y}.
    \item The \code{"round(yhat[i,]) = d"} discrepancy measure returns a
  record-level proportion of \eqn{\textbf{y}^{rep}_i}{y[rep][i]} that,
  when rounded, is equal to a specified discrete value, \code{d}. The
  discrepancy statistic is the mean of the record-level proportions.
    \item The \code{"sd(yhat[i,]) > sd(y)"} discrepancy measure returns a
  record-level indicator when the standard deviation of replicates is
  larger than the standard deviation of all of \eqn{\textbf{y}}{y}. The
  discrepancy statistic is the mean of the record-level indicators,
  reporting the proportion of records with larger standard deviations
  than \eqn{\textbf{y}}{y}.
    \item The \code{"p(yhat[i,] != y[i])"} discrepancy measure returns
  the record-level probability that \eqn{\textbf{y}^{rep}_i}{y[rep][i]}
  is not equal to \eqn{\textbf{y}}{y}. This is valid when
  \eqn{\textbf{y}}{y} is categorical and \code{yhat} is the predicted
  category. The probability is the proportion of replicates.
}
  After observing a discrepancy statistic, the user attempts to improve
  the model by revising the model to account for discrepancies between
  data and the current model. This approach to model revision relies on
  an analysis of the discrepancy statistic. Given a discrepancy measure
  that is based on model fit, such as the L-criterion, the user may
  correlate the record-level discrepancy statistics with the dependent
  variable, independent variables, and interactions of independent
  variables. The discrepancy statistic should not correlate with the
  dependent and independent variables. Interaction variables may be
  useful for exploring new relationships that are not in the current
  model. Alternatively, a decision tree may be applied to the
  record-level discrepancy statistics, given the independent variables,
  in an effort to find relationships in the data that may be helpful
  in the model. Model revision may involve the addition of a finite
  mixture component to account for outliers in discrepancy, or
  specifying the model with a distribution that is more robust to
  outliers. There are too many suggestions to include here, and
  discrepancy analysis varies by model.
}
\value{
  This function returns a list with the following components:
  \item{BPIC}{The Bayesian Predictive Information Criterion (BPIC) was
    introduced by Ando (2007). BPIC is a variation of the Deviance
    Information Criterion (DIC) that has been modified for predictive
    distributions. For more information on DIC (Spiegelhalter
    et al., 2002), see the accompanying vignette entitled "Bayesian
    Inference". \eqn{BPIC = Dbar + 2pD}. The goal is to minimize BPIC.}
  \item{Concordance}{This is the percentage of the records of y that are
    within the 95\% quantile-based probability interval (see
    \code{\link{p.interval}}) of \eqn{\textbf{y}^{rep}}{y[rep]}.
    Gelfand's suggested goal is to achieve 95\% predictive concordance.
    Lower percentages indicate too many outliers and a poor fit of the
    model to the data, and higher percentages may suggest overfitting.
    Concordance occurs only when \eqn{\textbf{y}}{y} is continuous.}
  \item{Mean Lift}{This is the mean of the record-level lifts, and
    occurs only when \eqn{\textbf{y}}{y} is specified as categorical
    with \code{Categorical=TRUE}.}
  \item{Discrepancy.Statistic}{This is only reported if the
    \code{Discrep} argument receives a valid discrepancy measure as
    listed above. The \code{Discrep} applies to each record of
    \eqn{\textbf{y}}{y}, and the \code{Discrepancy.Statistic} reports
    the results of the discrepancy measure on the entire data set. For
    example, if \code{Discrep="min(yhat[i,]) < min(y)"}, then the
    overall result is the proportion of records in which the minimum
    sample of yhat was less than the overall minimum
    \eqn{\textbf{y}}{y}. This is \code{Pr(min(yhat[i,]) < min(y) | y,
      Theta)}, where \code{Theta} is the parameter set.}
  \item{L-criterion}{The L-criterion (Laud and Ibrahim, 1995) was
    developed for model and variable selection. It is a sum of two
    components: one involves the predictive variance and the other
    includes the accuracy of the means of the predictive
    distribution. The L-criterion measures model performance with a
    combination of how close its predictions are to the observed data
    and variability of the predictions. Better models have smaller
    values of \code{L}. \code{L} is measured in the same units as
    the response variable, and measures how close the data vector
    \eqn{\textbf{y}}{y} is to the predictive distribution. In addition
    to the value of \code{L}, there is a value for \code{S.L}, which is
    the calibration number of \code{L}, and is useful in determining how
    much of a decrease is necessary between models to be noteworthy.}
  \item{Monitor}{This is a \eqn{N \times 5}{N x 5} matrix, where \eqn{N}
    is the number of monitored variables and there are 5 columns, as
    follows: Mean, SD, LB (the 2.5\% quantile), Median, and UB (the
    97.5\% quantile).}
  \item{Summary}{When \eqn{\textbf{y}}{y} is continuous, this is a
    \eqn{N \times 8}{N x 8} matrix, where \eqn{N} is the number of
    records of \eqn{\textbf{y}}{y} and there are 8 columns, as follows:
    y, Mean, SD, LB (the 2.5\% quantile), Median, UB (the 97.5\%
    quantile), PQ (the predictive quantile, which is
    \eqn{Pr(\textbf{y}^{rep} \ge \textbf{y})}{Pr(y[rep] >= y)}), and
    Test, which shows the record-level result of a test, if
    specified. When \eqn{\textbf{y}}{y} is categorical, this matrix has
    a number of columns equal to the number of categories of
    \eqn{\textbf{y}}{y} plus 3, also including \code{y}, \code{Lift},
    and \code{Discrep}.}
}
\references{
  Ando, T. (2007). "Bayesian Predictive Information Criterion for
  the Evaluation of Hierarchical Bayesian and Empirical Bayes Models".
  \emph{Biometrika}, 94(2), p. 443--458.
  
  Gelfand, A. (1996). "Model Determination Using Sampling Based
  Methods". In Gilks, W., Richardson, S., Spiegehalter, D., Chapter 9 in
  Markov Chain Monte Carlo in Practice. Chapman and Hall: Boca Raton, FL.

  Gelfand, A. and Ghosh, S. (1998). "Model Choice: A Minimum Posterior
  Predictive Loss Approach". \emph{Biometrika}, 85, p. 1--11.

  Gelman, A., Meng, X.L., and Stern H. (1996). "Posterior Predictive
  Assessment of Model Fitness via Realized Discrepancies".
  \emph{Statistica Sinica}, 6, p. 733--807.

  Laud, P.W. and Ibrahim, J.G. (1995). "Predictive Model Selection".
  \emph{Journal of the Royal Statistical Society}, B 57, p. 247--262.

  Spiegelhalter, D.J., Best, N.G., Carlin, B.P., and van der Linde, A.
  (2002). "Bayesian Measures of Model Complexity and Fit (with
  Discussion)". \emph{Journal of the Royal Statistical Society}, B 64,
  p. 583--639.
}
\author{Statisticat, LLC.}
\seealso{
  \code{\link{predict.vb}},
  \code{\link{p.interval}}, and
  \code{\link{VariationalBayes}}.
}
\examples{### See the VariationalBayes function for an example.}
\keyword{BPIC}
\keyword{Chi-Square}
\keyword{Kurtosis}
\keyword{L-criterion}
\keyword{MASE}
\keyword{MSE}
\keyword{Posterior Predictive Checks}
\keyword{Posterior Predictive Loss}
\keyword{Quadratic Loss}
\keyword{Quadratic Utility}
\keyword{RMSE}
\keyword{Skewness}
\keyword{summary}

\name{KS.Diagnostic}
\alias{KS.Diagnostic}
\title{Kolmogorov-Smirnov Convergence Diagnostic}
\description{
  The Kolmogorov-Smirnov test is a nonparametric test of stationarity
  that has been applied as an MCMC diagnostic (Brooks et al, 2003), such
  as to the posterior samples from the \code{\link{LaplacesDemon}}
  function. The first and last halves of the chain are compared. This
  test assumes IID, which is violated in the presence of
  autocorrelation.

  The \code{KS.Diagnostic} is a univariate diagnostic that is usually
  applied to each marginal posterior distribution. A multivariate form
  is not included. By chance alone due to multiple independent tests,
  5\% of the marginal posterior distributions should appear
  non-stationary when stationarity exists. Assessing multivariate 
  convergence is difficult.
}
\usage{
KS.Diagnostic(x)
}
\arguments{
  \item{x}{This is a vector of posterior samples for which a
    Kolmogorov-Smirnov test will be applied that compares the first and
    last halves for stationarity.}
}
\details{
  There are two main approaches to using the Kolmogorov-Smirnov test as
  an MCMC diagnostic. There is a version of the test that has
  been adapted to account for autocorrelation (and is not included
  here). Otherwise, the chain is thinned enough that autocorrelation is
  not present or is minimized, in which case the two-sample
  Kolmogorov-Smirnov test is applied. The CDFs of both samples are
  compared. The \code{ks.test} function in base R is used.

  The advantage of the Kolmogorov-Smirnov test is that it is easier and
  faster to calculate. The disadvantages are that autocorrelation biases
  results, and the test is generally biased on the conservative side
  (indicating stationarity when it should not).
}
\value{
  The \code{KS.Diagnostic} function returns a frequentist p-value, and
  stationarity is indicated when p > 0.05.
}
\references{
  Brooks, S.P., Giudici, P., and Philippe, A. (2003). "Nonparametric
  Convergence Assessment for MCMC Model Selection". \emph{Journal of
  Computational and Graphical Statistics}. 12(1), p. 1--22.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{is.stationary}},
  \code{\link{ks.test}}, and
  \code{\link{LaplacesDemon}}.
}
\examples{
library(LaplacesDemon)
x <- rnorm(1000)
KS.Diagnostic(x)
}
\keyword{Diagnostic}
\keyword{MCMC}

\name{plot.importance}
\alias{plot.importance}
\title{Plot Variable Importance}
\description{
  This may be used to plot variable importance with BPIC, predictive
  concordance, a discrepancy statistic, or the L-criterion regarding an
  object of class \code{importance}.
}
\usage{\method{plot}{importance}(x, Style="BPIC", \dots)}
\arguments{
  \item{x}{This required argument is an object of class
    \code{importance}.}
  \item{Style}{When \code{Style="BPIC"}, BPIC is shown, and \code{BPIC}
    is the default. Otherwise, predictive concordance is plotted when
    \code{Style="Concordance"}, a discrepancy statistic is plotted when
    \code{Style="Discrep"}, or the L-criterion is plotted when
    \code{Style="L-criterion"}.}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  The x-axis is either BPIC (Ando, 2007), predictive concordance
  (Gelfand, 1996), a discrepancy statistic (Gelman et al., 1996), or the
  L-criterion (Laud and Ibrahim, 1995) of the \code{\link{Importance}}
  function (depending on the \code{Style} argument), and variables are
  on the y-axis. A more important variable is associated with a dot that
  is plotted farther to the right. For more information on variable
  importance, see the \code{\link{Importance}} function.
}
\references{
  Ando, T. (2007). "Bayesian Predictive Information Criterion for
  the Evaluation of Hierarchical Bayesian and Empirical Bayes Models".
  \emph{Biometrika}, 94(2), p. 443--458.

  Gelfand, A. (1996). "Model Determination Using Sampling Based
  Methods". In Gilks, W., Richardson, S., Spiegehalter, D., Chapter 9 in
  Markov Chain Monte Carlo in Practice. Chapman and Hall: Boca Raton, FL.

  Gelman, A., Meng, X.L., and Stern H. (1996). "Posterior Predictive
  Assessment of Model Fitness via Realized Discrepancies".
  \emph{Statistica Sinica}, 6, p. 733--807.
  
  Laud, P.W. and Ibrahim, J.G. (1995). "Predictive Model
  Selection". \emph{Journal of the Royal Statistical Society}, B 57,
  p. 247--262.
}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{Importance}}}
\keyword{Plot}

\name{hpc_server}
\alias{server_Listening}
\title{Server Listening}
\description{
  This function is not intended to be called directly by the user. It is
  an internal-only function to prevent cluster problems while using the
  \code{INCA} algorithm in the \code{LaplacesDemon.hpc} function.
}
\usage{
server_Listening(n=2, port=19009)
}
\arguments{
  \item{n}{This is the number of CPUs. For more information, see
    \code{\link{LaplacesDemon.hpc}}.}
  \item{port}{This is a port for server listening, and defaults to
    port \code{19009}.}
}
\details{
  For the \code{INCA} algorithm, a server has been built into the
  \code{LaplacesDemon.hpc} function. The server exchanges information
  between processes, and has been designed to be portable. The
  \code{server_Listening} function is run as a separate process via the
  \code{system} function, when \code{INCA} is selected in
  \code{LaplacesDemon.hpc}.

  Socket connections and the \code{serialize} function are used as per
  the \pkg{Snow} package to update a single proposal covariance matrix
  given all parallel chains. The sockets are opened/closed in each
  process with a small random sleep time to avoid collisions during
  connections to the internal server of
  \code{LaplacesDemon.hpc}. Blocking sockets are used to synchronize
  processes.
}
\author{Silvere Vialet-Chabrand \email{silvere@vialet-chabrand.com}}
\seealso{
  \code{\link{LaplacesDemon}} and
  \code{\link{LaplacesDemon.hpc}}.
}
\keyword{High Performance Computing}
\keyword{Parallel Chains}

\name{dist.Multivariate.Laplace.Cholesky}
\alias{dmvlc}
\alias{rmvlc}
\title{Multivariate Laplace Distribution: Cholesky Parameterization}
\description{
  These functions provide the density and random number generation
  for the multivariate Laplace distribution, given the Cholesky
  parameterization.
}
\usage{
dmvlc(x, mu, U, log=FALSE)
rmvlc(n, mu, U)
}
\arguments{
  \item{x}{This is data or parameters in the form of a vector of length
       \eqn{k} or a matrix with \eqn{k} columns.}
  \item{n}{This is the number of random draws.}
  \item{mu}{This is mean vector \eqn{\mu}{mu} with length \eqn{k} or
    matrix with \eqn{k} columns.}
  \item{U}{This is the \eqn{k \times k}{k x k} upper-triangular matrix
    that is Cholesky factor \eqn{\textbf{U}}{U} of covariance matrix
    \eqn{\Sigma}{Sigma}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
       density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Multivariate
  \item Density: \deqn{p(\theta) = \frac{2}{(2\pi)^{k/2}
      |\Sigma|^{1/2}} \frac{(\pi/(2\sqrt{2(\theta - \mu)^T \Sigma^{-1}
      (\theta - \mu)}))^{1/2} \exp(-\sqrt{2(\theta - \mu)^T \Sigma^{-1}
      (\theta - \mu)})}{\sqrt{((\theta - \mu)^T \Sigma^{-1} (\theta -
      \mu) / 2)}^{k/2-1}}}{p(theta) = (2 / ((2*pi)^(k/2) *
    |Sigma|^(1/2)))
    ((sqrt(pi/(2*sqrt(2*(theta-mu)^TSigma^(-1)(theta-mu)))) *
    exp(-sqrt(2*(theta-mu)^TSigma^(-1)(theta-mu)))) /
    sqrt((theta-mu)^TSigma^(-1)(theta-mu)/2)^(k/2-1))}
  \item Inventor: Fang et al. (1990)
  \item Notation 1: \eqn{\theta \sim \mathcal{MVL}(\mu, \Sigma)}{theta ~
    MVL(mu, Sigma)}
  \item Notation 2: \eqn{\theta \sim \mathcal{L}_k(\mu, \Sigma)}{theta ~
    L[k](mu, Sigma)}
  \item Notation 3: \eqn{p(\theta) = \mathcal{MVL}(\theta | \mu,
    \Sigma)}{p(theta) = MVL(theta | mu, Sigma)}
  \item Notation 4: \eqn{p(\theta) = \mathcal{L}_k(\theta | \mu,
    \Sigma)}{p(theta) = L[k](theta | mu, Sigma)}
  \item Parameter 1: location vector \eqn{\mu}{mu}
  \item Parameter 2: positive-definite \eqn{k \times k}{k x k}
  covariance matrix \eqn{\Sigma}{Sigma}
  \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}
  \item Variance: \eqn{var(\theta) = \Sigma}{var(theta) = Sigma}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The multivariate Laplace distribution is a multidimensional extension of
the one-dimensional or univariate symmetric Laplace distribution. There
are multiple forms of the multivariate Laplace distribution.

The bivariate case was introduced by Ulrich and Chen (1987), and the
first form in larger dimensions may have been Fang et al. (1990), which
requires a Bessel function. Alternatively, multivariate Laplace was soon
introduced as a special case of a multivariate Linnik distribution
(Anderson, 1992), and later as a special case of the multivariate power
exponential distribution (Fernandez et al., 1995; Ernst, 1998). Bayesian
considerations appear in Haro-Lopez and Smith (1999). Wainwright and
Simoncelli (2000) presented multivariate Laplace as a Gaussian scale
mixture. Kotz et al. (2001) present the distribution formally. Here, the
density is calculated with the asymptotic formula for the Bessel
function as presented in Wang et al. (2008).

The multivariate Laplace distribution is an attractive alternative to
the multivariate normal distribution due to its wider tails, and remains
a two-parameter distribution (though alternative three-parameter forms
have been introduced as well), unlike the three-parameter multivariate t
distribution, which is often used as a robust alternative to the
multivariate normal distribution.

In practice, \eqn{\textbf{U}}{U} is fully unconstrained for proposals
when its diagonal is log-transformed. The diagonal is exponentiated
after a proposal and before other calculations. Overall, the Cholesky
parameterization is faster than the traditional parameterization.
Compared with \code{dmvl}, \code{dmvlc} must additionally
matrix-multiply the Cholesky back to the covariance matrix, but it
does not have to check for or correct the covariance matrix to
positive-definiteness, which overall is slower. Compared with
\code{rmvl}, \code{rmvlc} is faster because the Cholesky decomposition
has already been performed.
}
\value{
  \code{dmvlc} gives the density, and
  \code{rmvlc} generates random deviates.
}
\references{
  Anderson, D.N. (1992). "A Multivariate Linnik Distribution".
  \emph{Statistical Probability Letters}, 14, p. 333--336.

  Eltoft, T., Kim, T., and Lee, T. (2006). "On the Multivariate Laplace
  Distribution". \emph{IEEE Signal Processing Letters}, 13(5),
  p. 300--303.

  Ernst, M. D. (1998). "A Multivariate Generalized Laplace
  Distribution". \emph{Computational Statistics}, 13, p. 227--232.

  Fang, K.T., Kotz, S., and Ng, K.W. (1990). "Symmetric Multivariate and
  Related Distributions". Monographs on Statistics and Probability, 36,
  Chapman-Hall, London.

  Fernandez, C., Osiewalski, J. and Steel, M.F.J. (1995). "Modeling and
  Inference with v-spherical Distributions". \emph{Journal of the
  American Statistical Association}, 90, p. 1331--1340.
  
  Gomez, E., Gomez-Villegas, M.A., and Marin, J.M. (1998). "A
  Multivariate Generalization of the Power Exponential Family of
  Distributions". \emph{Communications in Statistics-Theory and
  Methods}, 27(3), p. 589--600.

  Haro-Lopez, R.A. and Smith, A.F.M. (1999). "On Robust Bayesian
  Analysis for Location and Scale Parameters". \emph{Journal of
  Multivariate Analysis}, 70, p. 30--56.

  Kotz., S., Kozubowski, T.J., and Podgorski, K. (2001). "The Laplace
  Distribution and Generalizations: A Revisit with Applications to
  Communications, Economics, Engineering, and Finance". Birkhauser:
  Boston, MA.

  Ulrich, G. and Chen, C.C. (1987). "A Bivariate Double Exponential
  Distribution and its Generalization". \emph{ASA Proceedings on
  Statistical Computing}, p. 127--129.

  Wang, D., Zhang, C., and Zhao, X. (2008). "Multivariate Laplace
  Filter: A Heavy-Tailed Model for Target Tracking". \emph{Proceedings
  of the 19th International Conference on Pattern Recognition}: FL.

  Wainwright, M.J. and Simoncelli, E.P. (2000). "Scale Mixtures of
  Gaussians and the Statistics of Natural Images". \emph{Advances in
  Neural Information Processing Systems}, 12, p. 855--861.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{chol}},
  \code{\link{daml}},
  \code{\link{dlaplace}},
  \code{\link{dmvnc}},
  \code{\link{dmvnpc}},
  \code{\link{dmvpec}},
  \code{\link{dmvtc}},
  \code{\link{dnorm}},
  \code{\link{dnormp}}, and
  \code{\link{dnormv}}.
}
\examples{
library(LaplacesDemon)
Sigma <- diag(3)
U <- chol(Sigma)
x <- dmvlc(c(1,2,3), c(0,1,2), U)
X <- rmvlc(1000, c(0,1,2), U)
joint.density.plot(X[,1], X[,2], color=TRUE)
}
\keyword{Distribution}

\name{dist.Multivariate.t.Precision}
\alias{dmvtp}
\alias{rmvtp}
\title{Multivariate t Distribution: Precision Parameterization}
\description{
  These functions provide the density and random number generation for
  the multivariate t distribution, otherwise called the multivariate
  Student distribution. These functions use the precision
  parameterization.
}
\usage{
dmvtp(x, mu, Omega, nu=Inf, log=FALSE)
rmvtp(n=1, mu, Omega, nu=Inf)
}
\arguments{
  \item{x}{This is either a vector of length \eqn{k} or a matrix with 
       a number of columns, \eqn{k}, equal to the number of columns in
       precision matrix \eqn{\Omega}{Omega}.}
  \item{n}{This is the number of random draws.}
  \item{mu}{This is a numeric vector representing the location parameter,
       \eqn{\mu}{mu} (the mean vector), of the multivariate distribution
       (equal to the expected value when \code{df > 1}, otherwise
       represented as \eqn{\nu > 1}{nu > 1}). It must be of length
       \eqn{k}, as defined above.}
  \item{Omega}{This is a \eqn{k \times k}{k x k} positive-definite
       precision matrix \eqn{\Omega}{Omega}.}
  \item{nu}{This is the degrees of freedom \eqn{\nu}{nu}, which must be
       positive.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
       density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Multivariate
  \item Density: \deqn{p(\theta) =
    \frac{\Gamma((\nu+k)/2)}{\Gamma(\nu/2)\nu^{k/2}\pi^{k/2}}
    |\Omega|^{1/2} (1 + \frac{1}{\nu} (\theta-\mu)^T \Omega
    (\theta-\mu))^{-(\nu+k)/2}}{p(theta) = (Gamma((nu+k)/2) /
    (Gamma(nu/2)*nu^(k/2)*pi^(k/2))) * |Omega|^(1/2) *
    (1 + (1/nu) (theta-mu)^T Omega (theta-mu))^(-(nu+k)/2)}
  \item Inventor: Unknown (to me, anyway)
  \item Notation 1: \eqn{\theta \sim \mathrm{t}_k(\mu,
    \Omega^{-1}, \nu)}{theta ~ t[k](mu, Omega^(-1), nu)}
  \item Notation 2: \eqn{p(\theta) = \mathrm{t}_k(\theta | \mu,
    \Omega^{-1}, \nu)}{p(theta) = t[k](theta | mu, Omega^(-1), \nu)}
  \item Parameter 1: location vector \eqn{\mu}{mu}
  \item Parameter 2: positive-definite \eqn{k \times k}{k x k} precision
  matrix \eqn{\Omega}{Omega}
  \item Parameter 3: degrees of freedom \eqn{\nu > 0}{nu > 0}
  \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}, for \eqn{\nu > 1}{nu
    > 1}, otherwise undefined
  \item Variance: \eqn{var(\theta) = \frac{\nu}{\nu - 2}
    \Omega^{-1}}{var(theta) = (nu / (nu - 2))*Omega^(-1)}, for \eqn{\nu
    > 2}{nu> 2}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The multivariate t distribution, also called the multivariate Student or
multivariate Student t distribution, is a multidimensional extension of the
one-dimensional or univariate Student t distribution. A random vector is
considered to be multivariate t-distributed if every linear
combination of its components has a univariate Student t-distribution.

It is usually parameterized with mean and a covariance matrix, or in
Bayesian inference, with mean and a precision matrix, where the
precision matrix is the matrix inverse of the covariance matrix. These
functions provide the precision parameterization for convenience and
familiarity. It is easier to calculate a multivariate t density
with the precision parameterization, because a matrix inversion can be
avoided.

This distribution has a mean parameter vector \eqn{\mu}{mu} of length
\eqn{k}, and a \eqn{k \times k}{k x k} precision matrix
\eqn{\Omega}{Omega}, which must be positive-definite. When degrees of
freedom \eqn{\nu=1}{nu=1}, this is the multivariate Cauchy distribution.
}
\value{
  \code{dmvtp} gives the density and 
  \code{rmvtp} generates random deviates.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dwishart}},
  \code{\link{dmvc}},
  \code{\link{dmvcp}},
  \code{\link{dmvt}},
  \code{\link{dst}},
  \code{\link{dstp}}, and
  \code{\link{dt}}.
}
\examples{
library(LaplacesDemon)
x <- seq(-2,4,length=21)
y <- 2*x+10
z <- x+cos(y) 
mu <- c(1,12,2)
Omega <- matrix(c(1,2,0,2,5,0.5,0,0.5,3), 3, 3)
nu <- 4
f <- dmvtp(cbind(x,y,z), mu, Omega, nu)
X <- rmvtp(1000, c(0,1,2), diag(3), 5)
joint.density.plot(X[,1], X[,2], color=TRUE)
}
\keyword{Distribution}

\name{dist.Inverse.Matrix.Gamma}
\alias{dinvmatrixgamma}
\title{Inverse Matrix Gamma Distribution}
\description{
  This function provides the density for the inverse matrix gamma
  distribution.
}
\usage{
dinvmatrixgamma(X, alpha, beta, Psi, log=FALSE)
}
\arguments{
  \item{X}{This is a \eqn{k \times k}{k x k} positive-definite
    covariance matrix.}
  \item{alpha}{This is a scalar shape parameter (the degrees of freedom),
    \eqn{\alpha}{alpha}.}
  \item{beta}{This is a scalar, positive-only scale parameter,
    \eqn{\beta}{beta}.}
  \item{Psi}{This is a \eqn{k \times k}{k x k} positive-definite scale
    matrix.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
       density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Multivariate Matrix
    \item Density: \eqn{p(\theta) = \frac{|\Psi|^\alpha}{\beta^{k
	  \alpha} \Gamma_k(\alpha)}
      |\theta|^{-\alpha-(k+1)/2}\exp(tr(-\frac{1}{\beta}\Psi\theta^{-1}))}{p(theta) = {|Psi|^alpha / [beta^(k alpha) Gamma[k](alpha)]} |theta|^[-alpha-(k+1)/2] exp(tr(-(1/beta)Psi theta^(-1)))}
  \item Inventors: Unknown
  \item Notation 1: \eqn{\theta \sim \mathcal{IMG}_k(\alpha, \beta,
    \Psi)}{theta ~ IMG[k](alpha, beta, Psi)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{IMG}_k(\theta | \alpha,
    \beta, \Psi)}{p(theta) = IMG[k](theta | alpha, beta, Psi)}
  \item Parameter 1: shape \eqn{\alpha > 2}{alpha > 2}
  \item Parameter 2: scale \eqn{\beta > 0}{beta > 0}
  \item Parameter 3: positive-definite \eqn{k \times k}{k x k} scale
    matrix \eqn{\Psi}{Psi}
  \item Mean: 
  \item Variance: 
  \item Mode: 
}

The inverse matrix gamma (IMG), also called the inverse matrix-variate
gamma, distribution is a generalization of the inverse gamma
distribution to positive-definite matrices. It is a more general and
flexible version of the inverse Wishart distribution
(\code{\link{dinvwishart}}), and is a conjugate prior of the covariance
matrix of a multivariate normal distribution (\code{\link{dmvn}}) and
matrix normal distribution (\code{\link{dmatrixnorm}}).

The compound distribution resulting from compounding a matrix normal
with an inverse matrix gamma prior over the covariance matrix is a
generalized matrix t-distribution.

The inverse matrix gamma distribution is identical to the inverse
Wishart distribution when \eqn{\alpha = \nu / 2}{alpha = nu / 2} and
\eqn{\beta = 2}{beta = 2}.
}
\value{
  \code{dinvmatrixgamma} gives the density.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dinvgamma}}
  \code{\link{dmatrixnorm}},
  \code{\link{dmvn}}, and
  \code{\link{dinvwishart}}
}
\examples{
library(LaplacesDemon)
k <- 10
dinvmatrixgamma(X=diag(k), alpha=(k+1)/2, beta=2, Psi=diag(k), log=TRUE)
dinvwishart(Sigma=diag(k), nu=k+1, S=diag(k), log=TRUE)
}
\keyword{Distribution}

\name{dist.Inverse.ChiSquare}
\alias{dinvchisq}
\alias{rinvchisq}
\title{(Scaled) Inverse Chi-Squared Distribution}
\description{
  This is the density function and random generation for the (scaled)
  inverse chi-squared distribution.
}
\usage{
dinvchisq(x, df, scale, log=FALSE)
rinvchisq(n, df, scale=1/df)
}
\arguments{
  \item{x}{This is a vector of quantiles.}
  \item{n}{This is the number of observations. If \code{length(n) > 1},
    then the length is taken to be the number required.}
  \item{df}{This is the degrees of freedom parameter, usually
    represented as \eqn{\nu}{nu}.}
  \item{scale}{This is the scale parameter, usually represented as
    \eqn{\lambda}{lambda}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density: \deqn{p(\theta) = \frac{{\nu/2}^{\nu/2}}{\Gamma(\nu/2)}
    \lambda^\nu \frac{1}{\theta}^{\nu/2+1} \exp(-\frac{\nu
      \lambda^2}{2\theta}), \theta \ge 0}{p(theta) =
    ((nu/2)^(nu/2))/(\Gamma(nu/2)) lambda^nu (1/theta)^((nu/2)+1)
    exp(-(nu lambda^2)/(2*theta)), theta >= 0}
  \item Inventor: Derived from the chi-squared distribution
  \item Notation 1: \eqn{\theta \sim \chi^{-2}(\nu, \lambda)}{theta ~
    chi^(-2)(nu, lambda)}
  \item Notation 2: \eqn{p(\theta) = \chi^{-2}(\theta | \nu,
    \lambda)}{p(theta) = chi^(-2)(theta | nu, lambda)}
  \item Parameter 1: degrees of freedom parameter \eqn{\nu > 0}{nu > 0}
  \item Parameter 2: scale parameter \eqn{\lambda}{lambda}
  \item Mean: \eqn{E(\theta)}{E(theta)} = unknown
  \item Variance: \eqn{var(\theta)}{var(theta)} = unknown
  \item Mode: \eqn{mode(\theta) = }{mode(theta) = }
}

The inverse chi-squared distribution, also called the
inverted chi-square distribution, is the multiplicate inverse of the
chi-squared distribution. If \eqn{x} has the chi-squared distribution
with \eqn{\nu}{nu} degrees of freedom, then \eqn{1 / x} has the
inverse chi-squared distribution with \eqn{\nu}{nu} degrees of freedom,
and \eqn{\nu / x}{nu / x} has the inverse chi-squared distribution with
\eqn{\nu}{nu} degrees of freedom.

These functions are similar to those in the GeoR package.
}
\value{
  \code{dinvchisq} gives the density and
  \code{rinvchisq} generates random deviates.
}
\seealso{
  \code{\link{dchisq}}
}
\examples{
library(LaplacesDemon)
x <- dinvchisq(1,1,1)
x <- rinvchisq(10,1)

#Plot Probability Functions
x <- seq(from=0.1, to=5, by=0.01)
plot(x, dinvchisq(x,0.5,1), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dinvchisq(x,1,1), type="l", col="green")
lines(x, dinvchisq(x,5,1), type="l", col="blue")
legend(3, 0.9, expression(paste(nu==0.5, ", ", lambda==1),
     paste(nu==1, ", ", lambda==1), paste(nu==5, ", ", lambda==1)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{dist.Stick}
\alias{dStick}
\alias{rStick}
\title{Truncated Stick-Breaking Prior Distribution}
\description{
  These functions provide the density and random number generation of
  the original, truncated stick-breaking (TSB) prior distribution given
  \eqn{\theta}{theta} and \eqn{\gamma}{gamma}, as per Ishwaran and James
  (2001).
}
\usage{
dStick(theta, gamma, log=FALSE)
rStick(M, gamma)
}
\arguments{
  \item{M}{This accepts an integer that is equal to one less than the
    number of truncated number of possible mixture components
    (\eqn{M=1}). Unlike most random deviate functions, this is not the
    number of random deviates to return.}
  \item{theta}{This is \eqn{\theta}{theta}, a vector of length
    \eqn{M-1}, where \eqn{M} is the truncated number of possible mixture
    components.}
  \item{gamma}{This is \eqn{\gamma}{gamma}, a scalar, and is usually
    gamma-distributed.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
  \item Application: Discrete Multivariate
  \item Density: \eqn{p(\pi) = \frac{(1-\theta)^{\beta-1}}{\mathrm{B}(1,\beta)}}{p(pi) = ((1-theta)^(beta-1))/(B(1,\beta))}
  \item Inventor: Sethuraman, J. (1994)
  \item Notation 1: \eqn{\pi \sim
    \mathrm{Stick}(\theta,\gamma)}{pi ~ Stick(theta, gamma)}
  \item Notation 2: \eqn{\pi \sim
    \mathrm{GEM}(\theta,\gamma)}{pi ~ Stick(theta, gamma)}
  \item Notation 3: \eqn{p(\pi) = \mathrm{Stick}(\pi | \theta,
    \gamma)}{p(pi) = Stick(pi | theta, gamma)}
  \item Notation 4: \eqn{p(\pi) = \mathrm{GEM}(\pi | \theta,
    \gamma)}{p(pi) = GEM(pi | theta, gamma)}
  \item Parameter 1: shape parameter \eqn{\theta \in (0,1)}{theta in (0,1)}
  \item Parameter 2: shape parameter \eqn{\gamma > 0}{gamma > 0}
  \item Mean: \eqn{E(\pi) = \frac{1}{1+\gamma}}{E(pi) = 1/(1+gamma)}
  \item Variance: \eqn{var(\pi) = \frac{\gamma}{(1+\gamma)^2
      (\gamma+2)}}{var(pi) = gamma / ((1+gamma)^2 (gamma+2))}
  \item Mode: \eqn{mode(\pi) = 0}{mode(pi) = 0}
}
The original truncated stick-breaking (TSB) prior distribution assigns
each \eqn{\theta}{theta} to be beta-distributed with parameters
\eqn{\alpha=1}{alpha=1} and \eqn{\beta=\gamma}{beta=gamma} (Ishwaran and
James, 2001). This distribution is commonly used in truncated Dirichlet
processes (TDPs).
}
\value{
  \code{dStick} gives the density and
  \code{rStick} generates a random deviate vector of length \eqn{M}.
}
\references{
  Ishwaran, H. and James, L. (2001). "Gibbs Sampling Methods for Stick
  Breaking Priors". \emph{Journal of the American Statistical
  Association}, 96(453), p. 161--173.
  
  Sethuraman, J. (1994). "A Constructive Definition of Dirichlet
  Priors". \emph{Statistica Sinica}, 4, p. 639--650.
}
\seealso{
  \code{\link{ddirichlet}},
  \code{\link{dmvpolya}}, and
  \code{\link{Stick}}.
}
\examples{
library(LaplacesDemon)
dStick(runif(4), 0.1)
rStick(4, 0.1)
}
\keyword{Distribution}

\name{dist.Zellner}
\alias{dhyperg}
\alias{dzellner}
\alias{rzellner}
\title{Hyperprior-g Prior and Zellner's g-Prior}
\description{
  These functions provide the density of the hyper-g prior (Liang et
  al., 2008), and both the density and random generation of Zellner's
  g-prior (Zellner, 1986).
}
\usage{
dhyperg(g, alpha=3, log=FALSE)
dzellner(beta, g, sigma, X, log=FALSE)
rzellner(n, g, sigma, X)
}
\arguments{
  \item{alpha}{This is a positive scale hyperhyperparameter that is
    proper when \eqn{\alpha > 2}{alpha > 2}. The default is
    \code{alpha=3}.}
  \item{beta}{This is regression effects \eqn{\beta}{beta}, a vector of
    length \eqn{J}.}
  \item{g}{This is hyperparameter \eqn{g}, a positive scalar.}
  \item{n}{This is the number of random deviates to generate.}
  \item{sigma}{This is the residual standard deviation
    \eqn{\sigma}{sigma}, a positive scalar.}
  \item{X}{This is a full-rank \eqn{N \times J}{N x J} design matrix
    \eqn{\textbf{X}}{X} for \eqn{N} records and \eqn{J} predictors,
    where \eqn{J+1 < N}. Zellner's g-prior has been extended (elsewhere)
    via singular value decomposition (SVD) to the case where
    \eqn{J > N}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Multivariate
  \item Density: \eqn{p(\theta) = \frac{1}{(2\pi)^{J/2}|(g
      \sigma^2(\textbf{X}^T \textbf{X})^{-1})^{-1}|^{1/2}}
    \exp(-\frac{1}{2}(\theta - \mu)'(g \sigma^2(\textbf{X}^T
    \textbf{X})^{-1})^{-1}(\theta - \mu))}{p(theta) =
    (1/((2*pi)^(J/2)*|Sigma|^(1/2))) * 
    exp(-(1/2)*(theta-mu)'*Sigma^(-1)*(theta-mu))}
  \item Inventor: Zellner, A. (1986)
  \item Notation 1: \eqn{\theta \sim
    \mathrm{N}_J(0, g \sigma^2(\textbf{X}^T \textbf{X})^{-1})}{theta ~
    N[J](0, g sigma^2 (X^T X)^(-1))}
  \item Notation 2: \eqn{p(\theta) = \mathrm{N}_J(\theta | g, \sigma^2,
    \textbf{X})}{p(theta) = N[J](theta, g, sigma^2, X)}
  \item Parameter 1: location parameter \eqn{\beta}{beta}
  \item Parameter 2: scale parameter \eqn{g > 0}
  \item Parameter 3: scale parameter \eqn{\sigma^2 > 0}{sigma^2 > 0}
  \item Mean:
  \item Variance:
  \item Mode:
}
Zellner's g-prior is a popular, data-dependent, elliptical, improper,
least-informative prior distribution on regression effects
\eqn{\beta}{beta} in a Gaussian regression model. It is a particular
form in the conjugate Normal-Gamma family. Zellner's g-prior is also
used for estimating Bayes factors (for hypothesis testing) with a
simpler form, as well as in model selection and variable selection. The
marginal posterior distribution of regression effects \eqn{\beta}{beta}
is multivariate t.

One of many nice properties of Zellner's g-prior is that it adapts
automatically to near-collinearity between different
predictors. Zellner's g-prior puts most of its prior mass in the
direction that causes the regression coefficients of correlated
predictors to be smoothed away from each other. When coupled with model
selection, Zellner's g-prior discourages highly collinear predictors
from entering the models simultaneously by inducing a negative
correlation between the coefficients. However, when it is desirable for
collinear predictors to enter simultaneously, a modification has been
proposed (though not included here) in which
\eqn{(\textbf{X}^T \textbf{X})^{-1}}{(X^T X)^(-1)} is replaced with
\eqn{(\textbf{X}^T \textbf{X})^\lambda}{(X^T X)^lambda}. For more
information, see Krishna et al. (2009).

For variable selection, large values of \eqn{g}, with a prior mean of
zero for \eqn{\beta}{beta}, encourage models with few, large
coefficients. Conversely, small values of \eqn{g} encourage saturated
models with many, small coefficients.

The design matrix \eqn{\textbf{X}}{X} is converted to Fisher's
information matrix, which is used as a covariance matrix for
\eqn{\beta}{beta}. This is computationally efficient, because each
element of the covariance matrix does not need to be estimated as a
parameter. When \eqn{\textbf{X}}{X} is nearly singular, regression
effects \eqn{\beta}{beta} may be poorly estimated.

Hyperparameter \eqn{g} acts as an inverse relative prior sample size, or
as a dimensionality penalty. Zellner (1986) recommended that a
hyperprior distribution is assigned to \eqn{g} so that it is estimated
from the data, although in practice \eqn{g} has often been fixed, usually
to \eqn{N} when no information is available, since it has the
interpretation of adding prior information equivalent to one
observation. A variety of hyperpriors have been suggested for \eqn{g},
such as in Bove and Held (2011), Liang et al. (2008), and Maruyama and
George (2011). \eqn{g} becomes diffuse as it approaches infinity, and
the Bayes factor approaches zero. The hyper-g prior of Liang et al.
(2008) is proper when \eqn{\alpha > 2}{alpha > 2}, and any value in
the interval \eqn{(2,4]} may be reasonable.
}
\value{
  \code{dhyperg} gives the density of the hyper-g prior of Liang et
  al. (2008), \code{dzellner} gives the density of Zellner's g-prior,
  and \code{rzellner} generates random deviates.
}
\references{
  Bove, D.S. and Held, L. (2011). "Hyper-g Priors for Generalized
  Linear Models". \emph{Bayesian Analysis}, 6(3), p. 387--410.

  Krishna, A., Bondell, H.D., and Ghosh, S.K. (2009). "Bayesian Variable
  Selection Using an Adaptive Powered Correlation Prior". \emph{Journal
  of Statistical Planning Inference}, 139(8), p. 2665-2674..

  Liang, F., Paulo, R., Molina, G., Clyde, M.A., and Berger,
  J.O. (2008). "Mixtures of g Priors for Bayesian Variable
  Selection". \emph{Journal of the American Statistical Association},
  103, p. 410--423.

  Maruyama, Y. and George, E.I. (2011). "Fully Bayes Factors with a
  Generalised g-Prior". \emph{Annals of Statistics}, 39, p. 2740--2765.

  Zellner, A. (1986). "On Assessing Prior Distributions and Bayesian
  Regression Analysis with g-Prior Distributions". In \emph{Bayesian
  Inference and Decision Techniques: Essays in Honor of Bruno de
  Finetti}, p. 233--243. Elsevier: Amsterdam, North Holland.
}
\seealso{
  \code{\link{BayesFactor}} and
  \code{\link{dmvt}}
}
\examples{
library(LaplacesDemon)
set.seed(667)
beta <- rnorm(10)
g <- 100
sigma <- 2
X <- cbind(1,matrix(rnorm(100*9),100,9))
dhyperg(g, alpha=3)
dzellner(beta, g, sigma, X)
rzellner(1, g, sigma, X)
}
\keyword{Distribution}

\name{BayesFactor}
\alias{BayesFactor}
\title{Bayes Factor}
\description{
  This function calculates Bayes factors for two or more fitted objects
  of class \code{demonoid}, \code{iterquad}, \code{laplace}, \code{pmc},
  or \code{vb} that were estimated respectively with the
  \code{\link{LaplacesDemon}}, \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}}, \code{\link{PMC}}, or
  \code{\link{VariationalBayes}} functions, and indicates the strength
  of evidence in favor of the hypothesis (that each model,
  \eqn{\mathcal{M}_i}{M[i]}, is better than another model,
  \eqn{\mathcal{M}_j}{M[j]}).
}
\usage{
BayesFactor(x)
}
\arguments{
  \item{x}{This is a list of two or more fitted objects of class
    \code{demonoid}, \code{iterquad}, \code{laplace}, \code{pmc}, or
    \code{vb}. The components are named in order beginning with model 1,
    \code{M1}, and \eqn{k} models are usually represented as
    \eqn{\mathcal{M}_1,\dots,\mathcal{M}_k}{M[1],...,M[k]}.}
}
\details{
  Introduced by Harold Jeffreys, a 'Bayes factor' is a Bayesian
  alternative to frequentist hypothesis testing that is most often used
  for the comparison of multiple models by hypothesis testing, usually
  to determine which model better fits the data (Jeffreys, 1961). Bayes
  factors are notoriously difficult to compute, and the Bayes factor is
  only defined when the marginal density of \eqn{\textbf{y}}{y} under
  each model is proper (see \code{\link{is.proper}}). However, the Bayes
  factor is easy to approximate with the Laplace-Metropolis estimator
  (Lewis and Raftery, 1997) and other methods of approximating the
  logarithm of the marginal likelihood (for more information, see
  \code{\link{LML}}).

  Hypothesis testing with Bayes factors is more robust than frequentist
  hypothesis testing, since the Bayesian form avoids model selection
  bias, evaluates evidence in favor of the null hypothesis, includes
  model uncertainty, and allows non-nested models to be compared (though
  of course the model must have the same dependent variable). Also,
  frequentist significance tests become biased in favor of rejecting the
  null hypothesis with sufficiently large sample size.

  The Bayes factor for comparing two models may be approximated as the
  ratio of the marginal likelihood of the data in model 1 and model
  2. Formally, the Bayes factor in this case is

\deqn{B =
  \frac{p(\textbf{y}|\mathcal{M}_1)}{p(\textbf{y}|\mathcal{M}_2)} =
  \frac{\int
    p(\textbf{y}|\Theta_1,\mathcal{M}_1)p(\Theta_1|\mathcal{M}_1)d\Theta_1}{\int
    p(\textbf{y}|\Theta_2,\mathcal{M}_2)p(\Theta_2|\mathcal{M}_2)d\Theta_2}}{B
  = (p(y|M_1)) / p(y|M_2) = (integral of p(y|Theta_1,M_1)p(Theta_1|M_1)dTheta_1)
  / (integral of p(y|Theta_2,M_2)p(Theta_2|M_2)dTheta_2)}

  where \eqn{p(\textbf{y}|\mathcal{M}_1)}{p(y|M_1)} is the marginal
  likelihood of the data in model 1.

  The \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}}, \code{\link{LaplacesDemon}},
  \code{\link{PMC}}, and \code{\link{VariationalBayes}} functions each
  return the \code{\link{LML}}, the approximate logarithm of the
  marginal likelihood of the data, in each fitted object of class
  \code{iterquad}, \code{laplace}, \code{demonoid}, \code{pmc}, or
  \code{vb}. The \code{BayesFactor} function calculates matrix \code{B},
  a matrix of Bayes factors, where each element of matrix \code{B} is a
  comparison of two models. Each Bayes factor is calculated as the
  exponentiated difference of \code{LML} of model 1
  (\eqn{\mathcal{M}_1}{M[1]}) and \code{LML} of model 2
  (\eqn{\mathcal{M}_2}{M[2]}), and the hypothesis for each element of
  matrix \code{B} is that the model associated with the row is greater
  than the model associated with the column. For example, element
  \code{B[3,2]} is the Bayes factor that model 3 is greater than model
  2. The 'Strength of Evidence' aids in the interpretation (Jeffreys,
  1961).

  A table for the interpretation of the strength of evidence for Bayes
  factors is available at
  \url{https://web.archive.org/web/20150214194051/http://www.bayesian-inference.com/bayesfactors}.

  Each Bayes factor, \code{B}, is the posterior odds in favor of the
  hypothesis divided by the prior odds in favor of the hypothesis, where
  the hypothesis is usually
  \eqn{\mathcal{M}_1 > \mathcal{M}_2}{M[1] > M[2]}. For example, when
  \code{B[3,2]=2}, the data favor \eqn{\mathcal{M}_3}{M[3]} over
  \eqn{\mathcal{M}_2}{M[2]} with 2:1 odds.

  It is also popular to consider the natural logarithm of the Bayes
  factor. The scale of the logged Bayes factor is the same above and
  below one, which is more appropriate for visual comparisons. For
  example, when comparing two Bayes factors at 0.5 and 2, the logarithm
  of these Bayes factors is -0.69 and 0.69.

  Gelman finds Bayes factors generally to be irrelevant, because they
  compute the relative probabilities of the models conditional on one
  of them being true. Gelman prefers approaches that measure the
  distance of the data to each of the approximate models (Gelman et al.,
  2004, p. 180), such as with posterior predictive checks (see the
  \code{\link{predict.iterquad}} function regarding iterative
  quadrature, \code{\link{predict.laplace}} function in the context of
  Laplace Approximation, \code{\link{predict.demonoid}} function in the
  context of MCMC, \code{\link{predict.pmc}} function in the context
  of PMC, or \code{\link{predict.vb}} function in the context of
  Variational Bayes). Kass et al. (1995) asserts this can be done
  without assuming one model is the true model.
}
\value{
  \code{BayesFactor} returns an object of class \code{bayesfactor} that
  is a list with the following components:
  \item{B}{This is a matrix of Bayes factors.}
  \item{Hypothesis}{
    This is the hypothesis, and is stated as 'row > column', indicating
    that the model associated with the row of an element in matrix
    \code{B} is greater than the model associated with the column of
    that element.}
  \item{Strength.of.Evidence}{
    This is the strength of evidence in favor of the hypothesis.}
  \item{Posterior.Probability}{
    This is a vector of the posterior probability of each model, given
    flat priors.}
}
\references{
  Gelman, A., Carlin, J., Stern, H., and Rubin, D. (2004). "Bayesian
  Data Analysis, Texts in Statistical Science, 2nd ed.". Chapman and
  Hall, London.

  Jeffreys, H. (1961). "Theory of Probability, Third Edition". Oxford
  University Press: Oxford, England.

  Kass, R.E. and Raftery, A.E. (1995). "Bayes Factors". \emph{Journal
  of the American Statistical Association}, 90(430), p. 773--795.

  Lewis, S.M. and Raftery, A.E. (1997). "Estimating Bayes Factors via
  Posterior Simulation with the Laplace-Metropolis Estimator".
  \emph{Journal of the American Statistical Association}, 92,
  p. 648--655.
}
\author{Statisticat, LLC.}
\seealso{
  \code{\link{is.bayesfactor}},
  \code{\link{is.proper}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{LML}},
  \code{\link{PMC}},
  \code{\link{predict.demonoid}},
  \code{\link{predict.iterquad}},
  \code{\link{predict.laplace}},
  \code{\link{predict.pmc}},
  \code{\link{predict.vb}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
# The following example fits a model as Fit1, then adds a predictor, and
# fits another model, Fit2. The two models are compared with Bayes
# factors.

library(LaplacesDemon)

##############################  Demon Data  ###############################
data(demonsnacks)
J <- 2
y <- log(demonsnacks$Calories)
X <- cbind(1, as.matrix(log(demonsnacks[,10]+1)))
X[,2] <- CenterScale(X[,2])

#########################  Data List Preparation  #########################
mon.names <- "LP"
parm.names <- as.parm.names(list(beta=rep(0,J), sigma=0))
pos.beta <- grep("beta", parm.names)
pos.sigma <- grep("sigma", parm.names)
PGF <- function(Data) {
     beta <- rnorm(Data$J)
     sigma <- runif(1)
     return(c(beta, sigma))
     }
MyData <- list(J=J, PGF=PGF, X=X, mon.names=mon.names,
     parm.names=parm.names, pos.beta=pos.beta, pos.sigma=pos.sigma, y=y)

##########################  Model Specification  ##########################
Model <- function(parm, Data)
     {
     ### Parameters
     beta <- parm[Data$pos.beta]
     sigma <- interval(parm[Data$pos.sigma], 1e-100, Inf)
     parm[Data$pos.sigma] <- sigma
     ### Log-Prior
     beta.prior <- sum(dnormv(beta, 0, 1000, log=TRUE))
     sigma.prior <- dhalfcauchy(sigma, 25, log=TRUE)
     ### Log-Likelihood
     mu <- tcrossprod(Data$X, t(beta))
     LL <- sum(dnorm(Data$y, mu, sigma, log=TRUE))
     ### Log-Posterior
     LP <- LL + beta.prior + sigma.prior
     Modelout <- list(LP=LP, Dev=-2*LL, Monitor=LP,
          yhat=rnorm(length(mu), mu, sigma), parm=parm)
     return(Modelout)
     }

############################  Initial Values  #############################
Initial.Values <- GIV(Model, MyData, PGF=TRUE)

########################  Laplace Approximation  ##########################
Fit1 <- LaplaceApproximation(Model, Initial.Values, Data=MyData,
     Iterations=10000)
Fit1

##############################  Demon Data  ###############################
data(demonsnacks)
J <- 3
y <- log(demonsnacks$Calories)
X <- cbind(1, as.matrix(demonsnacks[,c(7,8)]))
X[,2] <- CenterScale(X[,2])
X[,3] <- CenterScale(X[,3])

#########################  Data List Preparation  #########################
mon.names <- c("sigma","mu[1]")
parm.names <- as.parm.names(list(beta=rep(0,J), sigma=0))
pos.beta <- grep("beta", parm.names)
pos.sigma <- grep("sigma", parm.names)
PGF <- function(Data) return(c(rnormv(Data$J,0,10), rhalfcauchy(1,5)))
MyData <- list(J=J, PGF=PGF, X=X, mon.names=mon.names,
     parm.names=parm.names, pos.beta=pos.beta, pos.sigma=pos.sigma, y=y)

############################  Initial Values  #############################
Initial.Values <- GIV(Model, MyData, PGF=TRUE)

########################  Laplace Approximation  ##########################
Fit2 <- LaplaceApproximation(Model, Initial.Values, Data=MyData,
     Iterations=10000)
Fit2

#############################  Bayes Factor  ##############################
Model.list <- list(M1=Fit1, M2=Fit2)
BayesFactor(Model.list)
}
\keyword{Bayes Factor}
\keyword{Hypothesis Testing}
\keyword{Model Selection}

\name{AcceptanceRate}
\alias{AcceptanceRate}
\title{Acceptance Rate}
\description{
  The \code{Acceptance.Rate} function calculates the acceptance rate per
  chain from a matrix of posterior MCMC samples.
}
\usage{
AcceptanceRate(x)
}
\arguments{
  \item{x}{This required argument accepts a \eqn{S \times J}{S x J}
    numeric matrix of \eqn{S} posterior samples for \eqn{J} variables,
    such as \code{Posterior1} or \code{Posterior2} from an object of
    class \code{demonoid}.}
}
\details{
  The acceptance rate of an MCMC algorithm is the percentage of
  iterations in which the proposals were accepted.

  Optimal Acceptance Rates\cr
  The optimal acceptance rate varies with the number of parameters and
  by algorithm. Algorithms with componentwise Gaussian proposals have an
  optimal acceptance rate of 0.44, regardless of the number of
  parameters. Algorithms that update with multivariate Gaussian
  proposals tend to have an optimal acceptance rate that ranges from
  0.44 for one parameter (one IID Gaussian target distribution) to 0.234
  for an infinite number of parameters (IID Gaussian target
  distributions), and 0.234 is approached quickly as the number of
  parameters increases. The AHMC, HMC, and THMC algorithms have an
  optimal acceptance rate of 0.67, except with the algorithm
  specification \code{L=1}, where the optimal acceptance rate is
  0.574. The target acceptance rate is specified in HMCDA and NUTS, and
  the recommended rate is 0.65 and 0.60 respectively. Some algorithms
  have an acceptance rate of 1, such as AGG, ESS, GG, GS (MISS only),
  SGLD, or Slice.

  Global and Local Acceptance Rates\cr
  \code{\link{LaplacesDemon}} reports the global acceptance rate for the
  un-thinned chains. However, componentwise algorithms make a proposal
  per parameter, and therefore have a local acceptance rate for each
  parameter. Since only the global acceptance rate is reported, the
  \code{AcceptanceRate} function may be used to calculate the local
  acceptance rates from a matrix of un-thinned posterior samples.

  Thinning\cr
  Thinned samples tend to have higher local acceptance rates than
  un-thinned samples. With enough samples and enough thinning, local
  acceptance rates approach 1. Local acceptance rates do not need to
  approach the optimal acceptance rates above. Conversely, local
  acceptance rates do not need to approach 1, because too much
  information may possibly be discarded by thinning. For more
  information on thinning, see the \code{\link{Thin}} function.

  Diagnostics\cr
  The \code{AcceptanceRate} function may be used to calculate local
  acceptance rates on a matrix of thinned or un-thinned samples. Any
  chain with a local acceptance rate that is an outlier may be studied
  for reasons that may cause the outlier. A local acceptance rate
  outlier does not violate theory and is often acceptable, but may
  indicate a potential problem. Only some of the many potential problems
  include: identifiability, model misspecification, multicollinearity,
  multimodality, choice of prior distributions, or becoming trapped in a
  low-probability space. The solution to local acceptance rate outliers
  tends to be either changing the MCMC algorithm or re-specifying the
  model or priors. For example, an MCMC algorithm that makes
  multivariate Gaussian proposals for a large number of parameters may
  have low global and local acceptance rates when far from the target
  distributions.
}
\value{
  The \code{AcceptanceRate} function returns a vector of acceptance
  rates, one for each chain.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{LaplacesDemon}},
  \code{\link{MISS}},
  \code{\link{PosteriorChecks}}, and
  \code{\link{Thin}}.
}
\examples{
library(LaplacesDemon)
AcceptanceRate(matrix(rnorm(5000),1000,5))
}
\keyword{Diagnostic}
\keyword{MCMC}
\keyword{Utility}

\name{dist.Multivariate.Cauchy}
\alias{dmvc}
\alias{rmvc}
\title{Multivariate Cauchy Distribution}
\description{
  These functions provide the density and random number generation for
  the multivariate Cauchy distribution.
}
\usage{
dmvc(x, mu, S, log=FALSE)
rmvc(n=1, mu, S)
}
\arguments{
  \item{x}{This is either a vector of length \eqn{k} or a matrix with 
       a number of columns, \eqn{k}, equal to the number of columns in
       scale matrix \eqn{\textbf{S}}{S}.}
  \item{n}{This is the number of random draws.}
  \item{mu}{This is a numeric vector representing the location parameter,
       \eqn{\mu}{mu} (the mean vector), of the multivariate distribution
       It must be of length \eqn{k}, as defined above.}
  \item{S}{This is a \eqn{k \times k}{k x k} positive-definite scale
       matrix \eqn{\textbf{S}}{S}.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
       density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Multivariate
  \item Density: \deqn{p(\theta) =
    \frac{\Gamma[(1+k)/2]}{\Gamma(1/2)1^{k/2}\pi^{k/2}|\Sigma|^{1/2}[1+(\theta-\mu)^{\mathrm{T}}\Sigma^{-1}(\theta-\mu)]^{(1+k)/2}}}{p(theta)
    = Gamma[(1+k)/2] /
    {Gamma(1/2)1^(k/2)pi^(k/2)|Sigma|^(1/2)[1+(theta-mu)^T*Sigma^(-1)(theta-mu)]^[(1+k)/2]}}
  \item Inventor: Unknown (to me, anyway)
  \item Notation 1: \eqn{\theta \sim \mathcal{MC}_k(\mu, \Sigma)}{theta ~
    MC[k](mu, Sigma)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{MC}_k(\theta | \mu,
    \Sigma)}{p(theta) = MC[k](theta | mu, Sigma)}
  \item Parameter 1: location vector \eqn{\mu}{mu}
  \item Parameter 2: positive-definite \eqn{k \times k}{k x k} scale
  matrix \eqn{\Sigma}{Sigma}
  \item Mean: \eqn{E(\theta) = \mu}{E(theta) = mu}
  \item Variance: \eqn{var(\theta) = undefined}{var(theta) = undefined}
  \item Mode: \eqn{mode(\theta) = \mu}{mode(theta) = mu}
}

The multivariate Cauchy distribution is a multidimensional extension of the
one-dimensional or univariate Cauchy distribution. The multivariate
Cauchy distribution is equivalent to a multivariate t distribution with
1 degree of freedom. A random vector is considered to be multivariate
Cauchy-distributed if every linear combination of its components has a
univariate Cauchy distribution.

The Cauchy distribution is known as a pathological distribution because
its mean and variance are undefined, and it does not satisfy the central
limit theorem.
}
\value{
  \code{dmvc} gives the density and 
  \code{rmvc} generates random deviates.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dcauchy}},
  \code{\link{dinvwishart}},
  \code{\link{dmvcp}},
  \code{\link{dmvt}}, and
  \code{\link{dmvtp}}.
}
\examples{
library(LaplacesDemon)
x <- seq(-2,4,length=21)
y <- 2*x+10
z <- x+cos(y) 
mu <- c(1,12,2)
Sigma <- matrix(c(1,2,0,2,5,0.5,0,0.5,3), 3, 3)
f <- dmvc(cbind(x,y,z), mu, Sigma)

X <- rmvc(1000, rep(0,2), diag(2))
X <- X[rowSums((X >= quantile(X, probs=0.025)) &
     (X <= quantile(X, probs=0.975)))==2,]
joint.density.plot(X[,1], X[,2], color=TRUE)
}
\keyword{Distribution}

\name{BMK.Diagnostic}
\alias{BMK.Diagnostic}
\title{BMK Convergence Diagnostic}
\description{
  Given a matrix of posterior samples from MCMC, the
  \code{BMK.Diagnostic} function calculates Hellinger distances between
  consecutive batches for each chain. This is useful for monitoring
  convergence of MCMC chains.
}
\usage{
BMK.Diagnostic(X, batches=10)
}
\arguments{
  \item{X}{This required argument accepts a matrix of posterior
    samples or an object of class \code{demonoid}, in which case it uses
    the posterior samples in \code{X$Posterior1}.}
  \item{batches}{This is the number of batches on which the convergence
    diagnostic will be calculated. The \code{batches} argument defaults
    to 10.}
}
\details{
  Hellinger distance is used to quantify dissimilarity between two
  probability distributions. It is based on the Hellinger integral,
  introduced by Hellinger (1909). Traditionally, Hellinger distance is
  bound to the interval [0,1], though another popular form occurs in
  the interval [0,\eqn{\sqrt{2}}{sqrt(2)}]. A higher value of Hellinger
  distance is associated with more dissimilarity between the
  distributions.

  Convergence is assumed when Hellinger distances are below a threshold,
  indicating that posterior samples are similar between consecutive
  batches. If all Hellinger distances beyond a given batch of samples is
  below the threshold, then \code{burnin} is suggested to occur
  immediately before the first batch of satisfactory Hellinger
  distances.

  As an aid to interpretation, consider a matrix of 1,000 posterior
  samples from three chains: \code{beta[1]}, \code{beta[2]}, and
  \code{beta[3]}. With 10 batches, the column names are: 100, 200,
  \dots, 900. A Hellinger distance for the chain \code{beta[1]} at 100
  is the Hellinger distance between two batches: samples 1-100, and
  samples 101:200.

  A benefit to using \code{BMK.Diagnostic} is that the resulting
  Hellinger distances may easily be plotted with the \code{plotMatrix}
  function, allowing the user to see quickly which consecutive batches
  of which chains were dissimilar. This makes it easier to find
  problematic chains.

  The \code{BMK.Diagnostic} is calculated automatically in the
  \code{\link{LaplacesDemon}} function, and is one of the criteria in
  the \code{\link{Consort}} function regarding the recommendation of
  when to stop updating the Markov chain Monte Carlo (MCMC) sampler in
  \code{\link{LaplacesDemon}}.

  For more information on the related topics of burn-in and
  stationarity, see the \code{burnin} and \code{\link{is.stationary}}
  functions, and the accompanying vignettes.
}
\value{
  The \code{BMK.Diagnostic} function returns an object of class
  \code{bmk} that is a \eqn{J \times B}{J x B} matrix of Hellinger
  distances between consecutive batches for \eqn{J} parameters of
  posterior samples. The number of columns, \eqn{B} is equal to the
  number of batches minus one.

  The \code{BMK.Diagnostic} function is similar to the
  \code{bmkconverge} function in package BMK.
}
\references{
  Boone, E.L., Merrick, J.R. and Krachey, M.J. (2013). "A Hellinger
  Distance Approach to MCMC Diagnostics". \emph{Journal of Statistical
  Computation and Simulation}, in press.

  Hellinger, E. (1909). "Neue Begrundung der Theorie quadratischer
  Formen von unendlichvielen Veranderlichen" (in German). \emph{Journal
  fur die reine und angewandte Mathematik}, 136, p. 210--271.
}
\seealso{
  \code{\link{burnin}},
  \code{\link{Consort}},
  \code{\link{is.stationary}}, and 
  \code{\link{LaplacesDemon}}.
}
\examples{
library(LaplacesDemon)
N <- 1000 #Number of posterior samples
J <- 10 #Number of parameters
Theta <- matrix(runif(N*J),N,J)
colnames(Theta) <- paste("beta[", 1:J, "]", sep="")
for (i in 2:N) {Theta[i,1] <- Theta[i-1,1] + rnorm(1)}
HD <- BMK.Diagnostic(Theta, batches=10)
plot(HD, title="Hellinger distance between batches")
}
\keyword{Diagnostic}
\keyword{MCMC}
\keyword{Stationarity}
\keyword{Utility}

\name{dist.Normal.Laplace}
\alias{dnormlaplace}
\alias{rnormlaplace}
\title{Normal-Laplace Distribution: Univariate Asymmetric}
\description{
  These functions provide the density and random generation for the
  univariate, asymmetric, normal-Laplace distribution with location
  parameter \eqn{\mu}{mu}, scale parameter \eqn{\sigma}{\sigma}, and
  tail-behavior parameters \eqn{\alpha}{alpha} and \eqn{\beta}{beta}.
}
\usage{
dnormlaplace(x, mu=0, sigma=1, alpha=1, beta=1, log=FALSE)
rnormlaplace(n, mu=0, sigma=1, alpha=1, beta=1)
}
\arguments{
  \item{x}{This is a vector of data.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{mu}{This is the location parameter \eqn{\mu}{mu}.}
  \item{sigma}{This is the scale parameter \eqn{\sigma}{sigma}, which
    must be positive.}
  \item{alpha}{This is shape parameter \eqn{\alpha}{alpha} for
    left-tail behavior.}
  \item{beta}{This is shape parameter \eqn{\beta}{beta} for
    right-tail behavior.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
    \item Application: Continuous Univariate
    \item Density: \eqn{p(\theta) = \frac{\alpha\beta}{\alpha +
	\beta}\phi\frac{\theta - \mu}{\sigma} [R(\alpha\sigma -
      \frac{\theta - \mu}{\sigma}) + R(\beta\sigma + \frac{\theta -
	\mu}{\sigma})]}{p(theta) = [(alpha*beta)/(alpha + beta)] phi
      [(theta - mu) / sigma] [R(alpha*sigma - [(theta - mu) / sigma]) +
      R(beta*sigma + [(theta - mu) / sigma])]}
  \item Inventor: Reed (2006)
  \item Notation 1: \eqn{\theta \sim
    \mathrm{NL}(\mu,\sigma,\alpha,\beta)}{theta ~ NL(mu, sigma, alpha,
    beta)}
  \item Notation 2: \eqn{p(\theta) = \mathrm{NL}(\theta | \mu,
    \sigma, \alpha, \beta)}{p(theta) = NL(theta | mu, sigma, alpha,
    beta)}
  \item Parameter 1: location parameter \eqn{\mu}{mu}
  \item Parameter 2: scale parameter \eqn{\sigma > 0}{sigma > 0}
  \item Parameter 3: shape parameter \eqn{\alpha > 0}{alpha > 0}
  \item Parameter 4: shape parameter \eqn{\beta > 0}{beta > 0}
  \item Mean:
  \item Variance:
  \item Mode:
}

The normal-Laplace (NL) distribution is the convolution of a normal
distribution and a skew-Laplace distribution. When the NL distribution
is symmetric (when \eqn{\alpha = \beta}{alpha = beta}), it behaves
somewhat like the normal distribution in the middle of its range,
somewhat like the Laplace distribution in its tails, and functions
generally between the normal and Laplace distributions. Skewness is
parameterized by including a skew-Laplace component. It may be applied,
for example, to the logarithmic price of a financial instrument.

Parameters \eqn{\alpha}{alpha} and \eqn{\beta}{beta} determine the
behavior in the left and right tails, respectively. A small value
corresponds to heaviness in the corresponding tail. As
\eqn{\sigma}{sigma} approaches zero, the NL distribution approaches a
skew-Laplace distribution. As \eqn{\beta}{beta} approaches infinity,
the NL distribution approaches a normal distribution, though it never
quite reaches it.
}
\value{
  \code{dnormlaplace} gives the density, and
  \code{rnormlaplace} generates random deviates.
}
\references{
  Reed, W.J. (2006). "The Normal-Laplace Distribution and Its Relatives".
  In \emph{Advances in Distribution Theory, Order Statistics and
  Inference}, p. 61--74, Birkhauser, Boston.
}
\seealso{
  \code{\link{dalaplace}},
  \code{\link{dallaplace}},
  \code{\link{daml}},
  \code{\link{dlaplace}}, and
  \code{\link{dnorm}}
}
\examples{
library(LaplacesDemon)
x <- dnormlaplace(1,0,1,0.5,2)
x <- rnormlaplace(100,0,1,0.5,2)

#Plot Probability Functions
x <- seq(from=-5, to=5, by=0.1)
plot(x, dlaplace(x,0,0.5), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dlaplace(x,0,1), type="l", col="green")
lines(x, dlaplace(x,0,2), type="l", col="blue")
legend(2, 0.9, expression(paste(mu==0, ", ", lambda==0.5),
     paste(mu==0, ", ", lambda==1), paste(mu==0, ", ", lambda==2)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{joint.pr.plot}
\alias{joint.pr.plot}
\title{Joint Probability Region Plot}
\description{
  Given two vectors, the \code{joint.pr.plot} function creates a
  scatterplot with ellipses of probability regions.
}
\usage{
joint.pr.plot(x, y, quantiles=c(0.25,0.50,0.75,0.95))
}
\arguments{
  \item{x}{This required argument is a vector.}
  \item{y}{This required argument is a vector.}
  \item{quantiles}{These are the quantiles for which probability regions
    are estimated with ellipses. The center of the ellipse is plotted by
    default. The 0.95 quantile creates a probability region that
    contains approximately 95\% of the data or samples of \code{x} and
    \code{y}. By default, four quantiles are included.}
}
\details{
  A probability region is also commonly called a credible region. For
  more information on probability regions, see \code{\link{p.interval}}.

  Joint probability regions are plotted only for two variables, and the
  regions are estimated with functions modified from the \code{car}
  package. The internal ellipse functions assume bivariate normality.

  This function is often used to plot posterior distributions of
  samples, such as from the \code{\link{LaplacesDemon}} function.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{LaplacesDemon}} and
  \code{\link{p.interval}}
}
\examples{
library(LaplacesDemon)
x <- rnorm(100)
y <- rnorm(100)
joint.pr.plot(x, y)
}
\keyword{Plot}

\name{MISS}
\alias{MISS}
\title{Multiple Imputation Sequential Sampling}
\description{
  This function performs multiple imputation (MI) on a numeric matrix
  by sequentially sampling variables with missing values, given all
  other variables in the data set.
}
\usage{
MISS(X, Iterations=100, Algorithm="GS", Fit=NULL, verbose=TRUE)
}
\arguments{
  \item{X}{This required argument accepts a numeric matrix of data that
    contains both observed and missing values. Data set
    \eqn{\textbf{X}}{X} must not have any rows or columns that are
    completely missing. \eqn{\textbf{X}}{X} must not have any
    constants. The user must apply any data transformations appropriate
    for these models. Missing values are assumed to be Missing At Random
    (MAR).}
  \item{Iterations}{This is the number of iterations to perform
    sequential sampling via MCMC algorithms.}
  \item{Algorithm}{The MCMC algorithm defaults to the Gibbs Sampler (GS).}
  \item{Fit}{This optional argument accepts an object of class
    \code{miss}. When supplied, \code{MISS} will continue where it left
    off, provided the user does not change the algorithm (different
    methods are used with different algortihms, so model parameters will
    not match). In short, changing algorithms requires starting from
    scratch.}
  \item{verbose}{Logical. When \code{FALSE}, only the iteration prints
    to the console. When \code{TRUE}, which is the default, both the
    iteration and which variable is being imputed are printed to the
    console.}
}
\details{
  Imputation is a family of statistical methods for replacing missing
  values with estimates. Introduced by Rubin and Schenker (1986) and
  Rubin (1987), Multiple Imputation (MI) is a family of imputation
  methods that includes multiple estimates, and therefore includes
  variability of the estimates.
  
  The Multiple Imputation Sequential Sampler (MISS) function performs
  MI by determining the type of variable and therefore the sampler for
  each variable, and then sequentially progresses through each variable
  in the data set that has missing values, updating its prediction of
  those missing values given all other variables in the data set each
  iteration.

  MI is best performed within a model, where it is called
  full-likelihood imputation. Examples may be found in the "Examples"
  vignette. However, sometimes it is impractical to impute within a
  model when there are numerous missing values and a large number of
  parameters are therefore added. As an alternative, MI may be
  performed on the data set before the data is passed to the model,
  such as in the \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}}, \code{\link{LaplacesDemon}}, or
  \code{\link{VariationalBayes}} function. This is less desirable, but
  MISS is available for MCMC-based MI in this case.

  Missing values are initially set to column means for continuous
  variables, and are set to one for discrete variables.

  MISS uses the following methods and MCMC algorithms:

  Missing values of continuous variables are estimated with a
  ridge-stabilized linear regression Gibbs sampler.

  Missing values of binary variables that have only 0 or 1 for values
  are estimated either with a binary robit (t-link logistic
  regression model) Gibbs sampler of Albert and Chib (1993).

  Missing values of discrete variables with 3 or more (ordered or
  unordered) discrete values are considered continuous.

  In the presence of big data, it is suggested that the user
  sequentially read in batches of data that are small enough to be
  manageable, and then apply the MISS function to each batch. Each batch
  should be representative of the whole, of course.

  It is common for multiple imputation functions to handle variable
  transformations. MISS does not transform variables, but imputes what
  it gets. For example, if a user has a variable that should be positive
  only, then it is recommended here that the user log-transform the
  variable, pass the data set to MISS, and when finished, exponentiate
  both the observed and imputed values of that variable.

  The \code{CenterScale} function should also be considered to speed up
  convergence.

  It is hoped that MISS is helpful, though it is not without limitation
  and there are numerous alternatives outside of the
  \code{LaplacesDemon} package. If MISS does not fulfill the needs of
  the user, then the following packages are recommended: Amelia, mi, or
  mice. MISS emphasizes MCMC more than these alternatives, though MISS is
  not as extensive. When a data set does not have a simple structure,
  such as merely continuous or binary or unordered discrete, the
  \code{\link{LaplacesDemon}} function should be considered, where a
  user can easily specify complicated structures such as multilevel,
  spatial or temporal dependence, and more.

  Matrix inversions are required in the Gibbs sampler. Matrix inversions
  become more cumbersome as the number \eqn{J} of variables increases.
  
  If a large number of iterations is used, then the user may consider
  studying the imputations for approximate convergence with the
  \code{\link{BMK.Diagnostic}} function, by supplying the transpose of
  \code{Fit$Imp}. In the presence of numerous missing values, say more
  than 100, the user may consider iterating through the study of the
  imputations of 100 missing values at a time.
}
\value{
  This function returns an object of class \code{miss} that is a list
  with five components:
  \item{Algorithm}{This indicates which algorithm was selected.}
  \item{Imp}{This is a \eqn{M \times T}{M x T} matrix of \eqn{M} missing
    values and \eqn{T} iterations that contains imputations.}
  \item{parm}{This is a list of length \eqn{J} for \eqn{J} variables,
    and each component of the list contains parameters associated with
    the prediction of missing values for that variable.}
  \item{PostMode}{This is a vector of posterior modes. If the user
    intends to replace missing values in a data set with only one
    estimate per missing values (single, not multiple imputation), then
    this vector contains these values.}
  \item{Type}{This is a vector of length \eqn{J} for \eqn{J} variables
    that indicates the type of each variable, as MISS will consider it.
    When \code{Type=1}, the variable is considered to be continuous.
    When \code{Type=2}, only two discrete values (0 and 1) were found.}
}
\references{
  Albert, J.H. and Chib, S. (1993). "Bayesian Analysis of Binary and
  Polychotomous Response Data". \emph{Journal of the American
  Statistical Association}, 88(422), p. 669--679.

  Rubin, D.B. (1987). "Multiple Imputation for Nonresponse in
  Surveys". John Wiley and Sons: New York, NY.

  Rubin, D.B. and Schenker, N. (1986). "Multiple Imputation for Interval
  Estimation from Simple Random Samples with Ignorable
  Nonresponse". \emph{Journal of the American Statistical Association},
  81, p. 366--374.
}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{ABB}},
  \code{\link{BMK.Diagnostic}},
  \code{\link{CenterScale}},
  \code{\link{IterativeQuadrature}}
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
#library(LaplacesDemon)
### Create Data
#N <- 20 #Number of Simulated Records
#J <- 5 #Number of Simulated Variables
#pM <- 0.25 #Percent Missing
#Sigma <- as.positive.definite(matrix(runif(J*J),J,J))
#X <- rmvn(N, rep(0,J), Sigma)
#m <- sample.int(N*J, round(pM*N*J))
#X[m] <- NA
#head(X)

### Begin Multiple Imputation
#Fit <- MISS(X, Iterations=100, Algorithm="GS", verbose=TRUE)
#Fit
#summary(Fit)
#plot(Fit)
#plot(BMK.Diagnostic(t(Fit$Imp)))

### Continue Updating if Necessary
#Fit <- MISS(X, Iterations=100, Algorithm="GS", Fit, verbose=TRUE)
#summary(Fit)
#plot(Fit)
#plot(BMK.Diagnostic(t(Fit$Imp)))

### Replace Missing Values in Data Set with Posterior Modes
#Ximp <- X
#Ximp[which(is.na(X))] <- Fit$PostMode

### Original and Imputed Data Sets
#head(X)
#head(Ximp)
#summary(X)
#summary(Ximp)

### or Multiple Data Sets, say 3
#Ximp <- array(X, dim=c(nrow(X), ncol(X), 3))
#for (i in 1:3) {
#     Xi <- X
#     Xi[which(is.na(X))] <- Fit$Imp[,sample.int(ncol(Fit$Imp), 1)]
#     Ximp[,,i] <- Xi}
#head(X)
#head(Ximp[,,1])
#head(Ximp[,,2])
#head(Ximp[,,3])

#End
}
\keyword{Gibbs Sampler}
\keyword{Imputation}
\keyword{Utility}

\name{as.ppc}
\alias{as.ppc}
\title{As Posterior Predictive Check}
\description{
  This function converts an object of class \code{demonoid.val} to an
  object of class \code{demonoid.ppc}.
}
\usage{
as.ppc(x, set=3)
}
\arguments{
  \item{x}{This is an object of class \code{demonoid.val}.}
  \item{set}{This is an integer that indicates which list component is
    to be used. When \code{set=1}, the modeled data set is used. When
    \code{set=2}, the validation data set is used. When \code{set=3},
    both data sets are used.}
}
\details{
  After using the \code{\link{Validate}} function for holdout
  validation, it is often suggested to perform posterior predictive
  checks. The \code{as.ppc} function converts the output object of
  \code{\link{Validate}}, which is an object of class
  \code{demonoid.val}, to an object of class \code{demonoid.ppc}. The
  returned object is the same as if it were created with the
  \code{\link{predict.demonoid}} function, rather than the
  \code{\link{Validate}} function.

  After this conversion, the user may use posterior predictive checks,
  as usual, with the \code{\link{summary.demonoid.ppc}} function.
}
\value{
  The returned object is an object of class \code{demonoid.ppc}.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{predict.demonoid}},
  \code{\link{summary.demonoid.ppc}}, and
  \code{\link{Validate}}.
}
\keyword{Initial Values}

\name{MinnesotaPrior}
\alias{MinnesotaPrior}
\title{Minnesota Prior}
\description{
  The Minnesota prior, also called the Litterman prior, is a shrinkage
  prior for autoregressive parameters in vector autoregressive (VAR)
  models. There are many variations of the Minnesota prior. This
  Minnesota prior is calculated as presented in Lutkepohl (2005,
  p. 225), and returns one or more prior covariance matrices in an
  array.
}
\usage{
MinnesotaPrior(J, lags=c(1,2), lambda=1, theta=0.5, sigma)
}
\arguments{
  \item{J}{This is the scalar number of time-series in the VAR.}
  \item{lags}{This accepts an integer vector of lags of the
    autoregressive parameters. The lags are not required to be
    successive.}
  \item{lambda}{This accepts a scalar, positive-only hyperparameter that
    controls how tightly the parameter of the first lag is concentrated
    around zero. A smaller value results in smaller diagonal variance.
    When equal to zero, the posterior equals the prior and data is not
    influential. When equal to infinity, no shrinkage occurs and
    posterior expectations are closest to estimates from ordinary least
    squares (OLS). It has been asserted that as the number, \eqn{J}, of
    time-series increases, this hyperparameter should decrease.}
  \item{theta}{This accepts a scalar hyperparameter in the interval
    [0,1]. When one, off-diagonal elements have variance similar or
    equal to diagonal elements. When zero, off-diagonal elements have
    zero variance. A smaller value is associated with less off-diagonal
    variance.}
  \item{sigma}{This accepts a vector of length \eqn{J} of residual
    standard deviations of the dependent variables given the
    expectations.}
}
\details{
  The Minnesota prior was introduced in Doan, Litterman, and Sims
  (1984) as a shrinkage prior for autoregressive parameters in vector
  autoregressive (VAR) models. The Minnesota prior was reviewed in
  Litterman (1986), and numerous variations have been presented since.
  This is the version of the Minnesota prior as described in Lutkepohl
  (2005, p. 225) for stationary time-series.

  Given one or more \eqn{J \times J} matrices of autoregressive
  parameters in a VAR model, the user specifies two tuning
  hyperparameters for the Minnesota prior: \code{lambda} and
  \code{theta}. Each iteration of the numerical approximation algorithm,
  the latest vector of residual standard deviation parameters is
  supplied to the \code{MinnesotaPrior} function, which then returns an
  array that contains one or more prior covariance matrices for the
  autoregressive parameters. Multiple prior covariance matrices are
  returned when multiple lags are specified. The tuning hyperparameters,
  \code{lambda} and \code{theta}, can be estimated from the data via
  hierarchical Bayes.

  It is important to note that the Minnesota prior does not technically
  return a covariance matrix, because the matrix is not symmetric, and
  therefore not positive-definite. For this reason, a Minnesota prior
  covariance matrix should not be supplied as a covariance matrix to a
  multivariate normal distribution, such as with the \code{\link{dmvn}}
  function, though it would be accepted and then (incorrectly)
  converted to a symmetric matrix. Instead, \code{\link{dnormv}} should
  be used for element-wise evaluation.

  While the Minnesota prior is used to specify the prior covariance for
  VAR autoregressive parameters, prior means are often all set to zero,
  or sometimes the first lag is set to an identity matrix.

  An example is provided in the Examples vignette.
}
\value{
  This function returns a \eqn{J \times J \times L}{J x J x L} array
  for \eqn{J} time-series and \eqn{L} lags.
}
\references{
  Doan, T., Litterman, R.B. and Sims, C.A. (1984). "Forecasting and
  Conditional Projection using Realistic Prior Distributions".
  \emph{Econometric Reviews}, 3, p. 1--144.

  Litterman, R.B. (1986). "Forecasting with Bayesian Vector
  Autoregressions - Five Years of Experience". \emph{Journal of
  Business & Economic Statistics}, 4, p. 25--38.

  Lutkepohl, H. (2005). "New Introduction to Multiple Time Series
  Analysis". Springer, Germany.
}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{dmvn}},
  \code{\link{dnormv}}, and
  \code{\link{LaplacesDemon}}.
}
\keyword{Litterman Prior}
\keyword{Utility}

\name{PosteriorChecks}
\alias{PosteriorChecks}
\title{Posterior Checks}
\description{
  Not to be confused with posterior predictive checks, this function
  provides additional information about the marginal posterior
  distributions of continuous parameters, such as the probability that
  each posterior coefficient of the parameters (referred to generically
  as \eqn{\theta}{theta}), is greater than zero
  [\eqn{p(\theta > 0)}{p(theta > 0)}], the estimated number of modes,
  the kurtosis and skewness of the posterior distributions, the burn-in
  of each chain (for MCMC only), integrated autocorrelation time,
  independent samples per minute, and acceptance rate. A posterior
  correlation matrix is provided only for objects of class
  \code{demonoid} or \code{pmc}.

  For discrete parameters, see the \code{\link{Hangartner.Diagnostic}}.
}
\usage{
PosteriorChecks(x, Parms)
}
\arguments{
  \item{x}{This required argument accepts an object of class
    \code{demonoid}, \code{iterquad}, \code{laplace}, \code{pmc}, or
    \code{vb}.}
  \item{Parms}{This argument accepts a vector of quoted strings to be
    matched for selecting parameters. This argument defaults to
    \code{NULL} and selects every parameter. Each quoted string is
    matched to one or more parameter names with the \code{grep}
    function. For example, if the user specifies \code{Parms=c("eta",
      "tau")}, and if the parameter names are beta[1], beta[2], eta[1],
    eta[2], and tau, then all parameters will be selected, because the
    string \code{eta} is within \code{beta}. Since \code{grep} is used,
    string matching uses regular expressions, so beware of
    meta-characters, though these are acceptable: ".", "[", and "]".}
}
\details{\code{PosteriorChecks} is a supplemental function that returns
  a list with two components. Following is a summary of popular uses of
  the \code{PosteriorChecks} function.

  First (and only for MCMC users), the user may be considering the
  current MCMC algorithm versus others. In this case, the
  \code{PosteriorChecks} function is often used to find the two MCMC
  chains with the highest \code{\link{IAT}}, and these chains are
  studied for non-randomness with a joint trace plot, via the
  \code{\link{joint.density.plot}} function. The best algorithm has the
  chains with the highest independent samples per minute (ISM).

  Posterior correlation may be studied between model updates as well as
  after a model seems to have converged. While frequentists consider
  multicollinear predictor variables, Bayesians tend to consider
  posterior correlation of the parameters. Models with multicollinear
  parameters take more iterations to converge. Hierarchical models often
  have high posterior correlations. Posterior correlation often
  contributes to a lower effective sample size (\code{\link{ESS}}).
  Common remedies include transforming the predictors,
  re-parameterization to reduce posterior correlation, using WIPs
  (Weakly-Informative Priors), or selecting a different numerical
  approximation algorithm. An example of re-parameterization is to
  constrain related parameters to sum to zero. Another approach is to
  specify the parameters according to a multivariate distribution that
  is assisted by estimating a covariance matrix. Some algorithms are
  more robust to posterior correlation than others. For example,
  posterior correlation should generally be less problematic for twalk
  than AMWG in \code{\link{LaplacesDemon}}. Posterior correlation may be
  plotted with the \code{\link{plotMatrix}} function, and may be useful
  for blocking parameters. For more information on blockwise sampling,
  see the \code{\link{Blocks}} function.

  After a user is convinced of the applicability of the current MCMC
  algorithm, and that the chains have converged, \code{PosteriorChecks}
  is often used to identify multimodal marginal posterior distributions
  for further study or model re-specification.

  Although many marginal posterior distributions appear normally
  distributed, there is no such assumption. Nonetheless, a marginal
  posterior distribution tends to be distributed the same as its prior
  distribution. If a parameter has a prior specified with a Laplace
  distribution, then the marginal posterior distribution tends also to
  be Laplace-distributed. In the common case of normality, kurtosis
  and skewness may be used to identify discrepancies between the prior
  and posterior, and perhaps this should be called a `prior-posterior
  check'.

  Lastly, parameter importance may be considered, in which case it is
  recommended to be considered simultaneously with variable importance
  from the \code{\link{Importance}} function.
}
\value{
  \code{PosteriorChecks} returns an object of class
  \code{posteriorchecks} that is a list with the following components:
  \item{Posterior.Correlation}{
    This is a correlation matrix of the parameters selected with the
    \code{Parms} argument. This component is returned as \code{NA} for
    objects of classes \code{"laplace"} or \code{"vb"}.}
  \item{Posterior.Summary}{This is a matrix in which each row is a
    parameter and there are eight columns: p(theta > 0), N.Modes,
    Kurtosis, Skewness, Burn-In, IAT, ISM, and AR. The first column,
    p(theta > 0), indicates parameter importance by reporting how much
    of the distribution is greater than zero. An important parameter
    distribution will have a result at least as extreme as 0.025 or
    0.975, and an unimportant parameter distribution is centered at
    0.5. This is not the importance of the associated variable relative
    to how well the model fits the data. For variable importance, see
    the \code{\link{Importance}} function. The second column, N.Modes,
    is the number of modes, estimated with the \code{\link{Modes}}
    function. Kurtosis and skewness are useful posterior checks that may
    suggest that a posterior distribution is non-normal or does not fit
    well with a distributional assumption, assuming a distributional
    assumption exists, which it may not. The burn-in is estimated for
    each chain (only for objects of class \code{demonoid} with the
    \code{\link{burnin}} function. The integrated autocorrelation
    time is estimated with \code{\link{IAT}}. The number of independent
    samples per minute (ISM) is calculated for objects of class
    \code{"demonoid"} as \code{\link{ESS}} divided by minutes. Lastly,
    the local acceptance rate of each MCMC chain is calculated with the
    \code{\link{AcceptanceRate}} function, and is set to 1 for objects
    of class \code{iterquad}, \code{laplace}, \code{pmc}, or \code{vb}.}
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{AcceptanceRate}},
  \code{\link{Blocks}},
  \code{\link{burnin}},
  \code{\link{ESS}},
  \code{\link{Hangartner.Diagnostic}},
  \code{\link{joint.density.plot}},
  \code{\link{IAT}},
  \code{\link{Importance}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{Modes}},
  \code{\link{plotMatrix}},
  \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\examples{### See the LaplacesDemon function for an example.}
\keyword{Diagnostic}
\keyword{MCMC}
\keyword{Monte Carlo}
\keyword{Multicollinearity}
\keyword{Posterior Correlation}
\keyword{Utility}

\name{plot.juxtapose}
\alias{plot.juxtapose}
\title{Plot MCMC Juxtaposition}
\description{
  This may be used to plot a juxtaposition of MCMC algorithms according
  either to \code{\link{IAT}} or ISM (Independent Samples per Minute).
}
\usage{\method{plot}{juxtapose}(x, Style="ISM", \dots)}
\arguments{
  \item{x}{This required argument is an object of class
    \code{juxtapose}.}
  \item{Style}{This argument accepts either \code{IAT} or \code{ISM},
    and defaults to \code{ISM}.}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  When \code{Style="IAT"}, the medians and 95\% probability intervals of
  the integrated autocorrelation times (IATs) of MCMC algorithms are
  displayed in a caterpillar plot. The best, or least inefficient, MCMC
  algorithm is the algorithm with the lowest IAT.

  When \code{Style="ISM"}, the medians and 95\% probability intervals of
  the numbers of independent samples per minute (ISM) of MCMC algorithms
  are displayed in a caterpillar plot. The best, or least inefficient,
  MCMC algorithm is the algorithm with the highest ISM.

  For more information, see the \code{\link{Juxtapose}} function.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{Juxtapose}}}
\keyword{Plot}

\name{PMC}
\alias{PMC}
\title{Population Monte Carlo}
\description{
  The \code{PMC} function updates a model with Population Monte Carlo.
  Given a model specification, data, and initial values, \code{PMC}
  maximizes the logarithm of the unnormalized joint
  posterior density and provides samples of the marginal
  posterior distributions, deviance, and other monitored variables.
}
\usage{
PMC(Model, Data, Initial.Values, Covar=NULL, Iterations=10, Thinning=1,
alpha=NULL, M=1, N=1000, nu=9, CPUs=1, Type="PSOCK")
}
\arguments{
  \item{Model}{This is a model specification function. For more
    information, see \code{\link{LaplacesDemon}}.}
  \item{Initial.Values}{This is either a vector initial values, one for
    each of \eqn{K} parameters, or in the case of a mixture of \eqn{M}
    components, this is a \eqn{M \times K}{M x K} matrix of initial
    values. If all initial values are zero in this vector, or in the
    first row of a matrix, then \code{\link{LaplaceApproximation}} is
    used to optimize initial values, in which case all mixture
    components receive the same initial values and covariance matrix
    from the object of class \code{laplace}. Parameters must be
    continuous.}
  \item{Data}{This is a list of data. For more information, see
    \code{\link{LaplacesDemon}}.}
  \item{Covar}{This is a \eqn{K \times K}{K x K} covariance matrix for
    \eqn{K} parameters, or for multiple mixture components, this is a
    \eqn{K \times K \times M}{K x K x M} array of \eqn{M} covariance
    matrices, where \eqn{M} is the number of mixture components.
    \code{Covar} defaults to \code{NULL}, in which case a scaled
    identity matrix (with the same scale as in
    \code{\link{LaplacesDemon}}) is applied to all mixture components.}
  \item{Iterations}{This is the number of iterations during which PMC
    will update the model. Updating the model for only one iteration is
    the same as applying non-adaptive importance sampling.}
  \item{Thinning}{This is the number by which the posterior is
    thinned. To have 1,000 posterior samples with \code{M=3} mixture
    components and \code{N=10000} samples each, \code{Thinning=30}. For
    more information, see the \code{\link{Thin}} function.}
  \item{alpha}{This is a vector of length \eqn{M}, the number of mixture
    components. \eqn{\alpha}{alpha} is the probability of each mixture
    component. The default value is \code{NULL}, which assigns an equal
    probability to each component.}
  \item{M}{This is the number \eqn{M} of multivariate t distribution
    mixture components.}
  \item{N}{This is the number \eqn{N} of samples per mixture
    component. The required number of samples increases with the number
    \eqn{K} of parameters. These samples are also called walkers or
    particles.}
  \item{nu}{This is the degrees of freedom parameter \eqn{\nu}{nu} for
    the multivariate t distribution for each mixture component. If a
    multivariate normal distribution is preferred, then set
    \eqn{\nu > 1e4}{nu > 1e4}.}
  \item{CPUs}{This argument is required for parallel processing, and
    and indicates the number of central processing units (CPUs) of the
    computer or cluster. For example, when a user has a quad-core
    computer, \code{CPUs=4}.}
  \item{Type}{This argument defaults to \code{"PSOCK"} and uses the
    Simple Network of Workstations (SNOW) for parallelization.
    Alternatively, \code{Type="MPI"} may be specified to use Message
    Passing Interface (MPI) for parallelization.}
}
\details{
  The \code{PMC} function uses the adaptive importance sampling
  algorithm of Wraith et al. (2009), also called Mixture PMC or M-PMC
  (Cappe et al., 2008). Iterative adaptive importance sampling was
  introduced in the 1980s. Modern PMC was introduced (Cappe et al.,
  2004), and extended to multivariate Gaussian or t-distributed mixtures
  (Cappe et al., 2008). This version uses a multivariate t distribution
  for each mixture component, and also allows a multivariate normal
  distribution when the degrees of freedom, \eqn{\nu > 1e4}{nu >
    1e4}. At each iteration, a mixture distribution is sampled with
  importance sampling, and the samples (or populations) are adapted to
  improve the importance sampling. Adaptation is a variant of EM
  (Expectation-Maximization). The sample is self-normalized, and is an
  example of self-normalized importance sampling (SNIS), or
  self-importance sampling. The vector \eqn{\alpha}{alpha} contains the
  probability of each mixture component. These, as well as multivariate
  t distribution mixture parameters (except \eqn{\nu}{nu}), are adapted
  each iteration.

  Advantages of PMC over MCMC include:
  \itemize{
    \item It is difficult to assess convergence of MCMC chains, and this
  is not necessary in PMC (Wraith et al., 2009).  
    \item MCMC chains have autocorrelation that effectively reduces
  posterior samples. PMC produces independent samples that are not
  reduced with autocorrelation.
    \item PMC has been reported to produce samples with less variance
  than MCMC.
    \item It is difficult to parallelize MCMC. Posterior samples from
  parallel chains can be pooled when all chains have converged, but
  until this occurs, parallelization is unhelpful. PMC, on the other
  hand, can parallelize the independent, Monte Carlo samples during each
  iteration and reduce run-time as the number of processors
  increases. Currently, PMC is not parallelized here.
    \item The multivariate mixture in PMC can represent a multimodal
  posterior, where MCMC with parallel chains may be used to identify a
  multimodal posterior, but probably will not yield combined samples
  that proportionally represent it.
  }

  Disadvantages of PMC, compared to MCMC, include:
  \itemize{
    \item In PMC, the required number of samples at each iteration
  increases quickly with respect to an increase in parameters. MCMC is
  more suitable for models with large numbers of parameters, and
  therefore, MCMC is more generalizable.
    \item PMC is more sensitive to initial values than MCMC, especially
  as the number of parameters increases.
    \item PMC is more sensitive to the initial covariance matrix (or
  matrices for mixture components) than adaptive MCMC. PMC requires more
  information about the target distributions before updating. The
  covariance matrix from a converged iterative quadrature algorithm,
  Laplace Approximation, or Variational Bayes may be required (see
  \code{\link{IterativeQuadrature}}, \code{\link{LaplaceApproximation}},
  or \code{\link{VariationalBayes}} for more information).
  }

  Since PMC requires better initial information than iterative
  quadrature, Laplace Approximation, MCMC, and Variational Bayes, it is
  not recommended to begin updating a model that has little prior
  information with PMC, especially when the model has more than a few
  parameters. Instead, iterative quadrature, Laplace Approximation,
  MCMC, or Variational Bayes should be used. However, once convergence
  is found or assumed, it is recommended to attempt to update the model
  with PMC, given the latest parameters and convariance matrix from
  iterative quadrature, Laplace Approximation, MCMC, or Variational
  Bayes. Used in this way, PMC may improve the model fit obtained with
  MCMC and should reduce the variance of the marginal posterior
  distributions,  which is desirable for predictive modeling.
  
  Convergence is assessed by observing two outputs: normalized effective
  sample size (\code{ESSN}) and normalized perplexity
  (\code{Perplexity}). These are described below. PMC is considered to
  have converged when these diagnostics stabilize (Wraith et al., 2009),
  or when the normalized perplexity becomes sufficiently close to 1
  (Cappe et al., 2008). If they do not stabilize, then it is suggested
  to begin PMC again with a larger number \code{N} of samples, and
  possibly with different initial values and covariance matrix or
  matrices. \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}}, or \code{\link{VariationalBayes}}
  may be helpful to provide better starting values for \code{PMC}.

  If a message appears that warns about `bad weights', then \code{PMC}
  is attempting to work with an iteration in which importance weights
  are problematic. If this occurs in the first iteration, then all
  importance weights are set to \eqn{1/N}. If this occurs in other
  iterations, then the information from the previous iteration is used
  instead and different draws are made from that importance
  distribution. This may allow \code{PMC} to eventually adapt
  successfully to the target. If not, the user is advised to begin again
  with a larger number \eqn{N} of samples, and possibly different
  initial values and covariance matrix or matrices, as above. PMC can
  experience difficulty when it begins with poor initial conditions.

  The user may combine samples from previous iterations with samples
  from the latest iteration for inference, if the algorithm converged
  before the last iteration. Currently, a function is not provided for
  combining previous samples.
}
\value{
  The returned object is an object of class \code{pmc} with the
  following components:
  \item{alpha}{This is a \eqn{M \times T}{M x T} matrix of the
    probabilities of mixture components, where \eqn{M} is the number of
    mixture components and \eqn{T} is the number of iterations.}
  \item{Call}{This is the matched call of \code{PMC}.}
  \item{Covar}{This stores the \eqn{K \times K \times T \times M}{K x K
    x T x M} proposal covariance matrix in an array, where \eqn{K} is
    the dimension or number of parameters or initial values, \eqn{T} is
    the number of iterations, and \eqn{M} is the number of mixture
    components. If the model is updated in the future, then the latest
    covariance matrix for each mixture component can be extracted and
    used to start the next update where the last update left off.}
  \item{Deviance}{This is a vector of the deviance of the model, with a
    length equal to the number of thinned samples that were retained.
    Deviance is useful for considering model fit, and is equal to the
    sum of the log-likelihood for all rows in the data set, which is
    then multiplied by negative two.}
  \item{DIC}{This is a vector of three values: Dbar, pD, and DIC. Dbar
    is the mean deviance, pD is a measure of model complexity indicating
    the effective number of parameters, and DIC is the Deviance
    Information Criterion, which is a model fit statistic that is the
    sum of Dbar and pD. \code{DIC} is calculated over the thinned
    samples. Note that pD is calculated as \code{var(Deviance)/2} as in
    Gelman et al. (2004).}
  \item{ESSN}{This is a vector of length \eqn{T} that contains the
    normalized effective sample size (ESSN) per iteration across \eqn{T}
    iterations. ESSN is used as a convergence diagnostic. ESSN is
    normalized between zero and one, and can be interpreted as the
    proportion of samples with non-zero weight. Higher is better.}
  \item{Initial.Values}{This is the vector or matrix of
    \code{Initial.Values}.}
  \item{Iterations}{This reports the number of \code{Iterations} for
    updating.}
  \item{LML}{This is an approximation of the logarithm of the marginal
    likelihood of the data (see the \code{\link{LML}} function for more
    information). \code{LML} is estimated with nonparametric
    self-normalized importance sampling (NSIS), given LL and the
    marginal posterior samples of the parameters. \code{LML} is useful
    for comparing multiple models with the \code{\link{BayesFactor}}
    function.}
  \item{M}{This reports the number of mixture components.}
  \item{Minutes}{This indicates the number of minutes that \code{PMC}
    was running, and this includes the initial checks as well as time it
    took to perform final sampling and create summaries.}
  \item{Model}{This contains the model specification \code{Model}.}
  \item{N}{This is the number of un-thinned samples per mixture
    component.}
  \item{nu}{This is the degrees of freedom parameter \eqn{\nu}{nu} for
  each multivariate t distribution in each mixture component.}
  \item{Mu}{This is a \eqn{T \times K \times M}{T x K x M} array of
    means for the importance sampling distribution across \eqn{T}
    iterations, \eqn{K} parameters, and \eqn{M} mixture components.}
  \item{Monitor}{This is a \eqn{S \times J}{S x J} matrix of
    thinned samples of monitored variables, where \eqn{S} is the number
    of thinned samples and \eqn{J} is the number of monitored
    variables.}
  \item{Parameters}{This reports the number \eqn{K} of parameters.}
  \item{Perplexity}{This is a vector of length \eqn{T} that contains the
    normalized perplexity per iteration across \eqn{T} iterations, and
    is used as a convergence diagnostic. Perplexity is an approximation
    of the negative of the Kullback-Leibler divergence (see
    \code{\link{KLD}}) between the target and the importance function.
    Perplexity is normalized between zero and one, and a higher
    normalized perplexity relates to less divergence, so higher is
    better. A normalized perplexity that is close to one indicates good
    agreement between the target density and the importance
    function. This is based on the Shannon entropy of the normalized
    importance weights, which is used frequently to measure the quality
    of importance samples.}
  \item{Posterior1}{This is an \eqn{N \times K \times T \times M}{N x K
    x T x M} array of un-thinned posterior samples across \eqn{N}
    samples, \eqn{K} parameters, \eqn{T} iterations, and \eqn{M} mixture
    components.}
  \item{Posterior2}{This is a \eqn{S \times K}{S x K} matrix of
    thinned posterior samples, where \eqn{S} is the number of thinned
    samples and \eqn{K} is the number of parameters.}
  \item{Summary}{This is a matrix that summarizes the marginal
    posterior distributions of the parameters, deviance, and monitored
    variables from thinned samples. The following summary statistics are
    included: mean, standard deviation, MCSE (Monte Carlo Standard
    Error), ESS is the effective sample size due to autocorrelation, and
    finally the 2.5\%, 50\%, and 97.5\% quantiles are reported. MCSE is
    essentially a standard deviation around the marginal posterior mean
    that is due to uncertainty associated with using Monte Carlo
    sampling. The acceptable size of the MCSE depends on the acceptable
    uncertainty associated around the marginal posterior mean. The
    default \code{IMPS} method is used. Next, the desired precision of
    ESS depends on the user's goal.}
  \item{Thinned.Samples}{This is the number of thinned samples in
    \code{Posterior2}.}
  \item{Thinning}{This is the amount of thinning requested by the user.}
  \item{W}{This is a \eqn{N \times T}{N x T} matrix of normalized
    importance weights, where \eqn{N} is the number of un-thinned
    samples per mixture component and \eqn{T} is the number of
    iterations. Computationally, the algorithm uses the logarithm of the
    weights.}
}
\references{
  Cappe, O., Douc, R., Guillin, A., Marin, J.M., and Robert, C. (2008).
  "Adaptive Importance Sampling in General Mixture Classes".
  \emph{Statistics and Computing}, 18, p. 587--600.

  Cappe, O., Guillin, A., Marin, J.M., and Robert, C. (2004). "Population 
  Monte Carlo". \emph{Journal of Computational and Graphical
  Statistics}, 13, p. 907--929.

  Gelman, A., Carlin, J., Stern, H., and Rubin, D. (2004). "Bayesian
  Data Analysis, Texts in Statistical Science, 2nd ed.". Chapman and
  Hall, London.
  
  Wraith, D., Kilbinger, M., Benabed, K., Cappe, O., Cardoso, J.F.,
  Fort, G., Prunet, S., and Robert, C.P. (2009). "Estimation of
  Cosmological Parameters Using Adaptive Importance Sampling".
  \emph{Physical Review D}, 80(2), p. 023507.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{BayesFactor}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LML}},
  \code{\link{PMC.RAM}},
  \code{\link{Thin}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
# The accompanying Examples vignette is a compendium of examples.
####################  Load the LaplacesDemon Library  #####################
library(LaplacesDemon)

##############################  Demon Data  ###############################
data(demonsnacks)
y <- log(demonsnacks$Calories)
X <- cbind(1, as.matrix(log(demonsnacks[,c(1,4,10)]+1)))
J <- ncol(X)
for (j in 2:J) X[,j] <- CenterScale(X[,j])

#########################  Data List Preparation  #########################
mon.names <- "LP"
parm.names <- as.parm.names(list(beta=rep(0,J), sigma=0))
pos.beta <- grep("beta", parm.names)
pos.sigma <- grep("sigma", parm.names)
PGF <- function(Data) {
     beta <- rnorm(Data$J)
     sigma <- runif(1)
     return(c(beta, sigma))
     }
MyData <- list(J=J, PGF=PGF, X=X, mon.names=mon.names,
     parm.names=parm.names, pos.beta=pos.beta, pos.sigma=pos.sigma, y=y)

##########################  Model Specification  ##########################
Model <- function(parm, Data)
     {
     ### Parameters
     beta <- parm[Data$pos.beta]
     sigma <- interval(parm[Data$pos.sigma], 1e-100, Inf)
     parm[Data$pos.sigma] <- sigma
     ### Log-Prior
     beta.prior <- sum(dnormv(beta, 0, 1000, log=TRUE))
     sigma.prior <- dhalfcauchy(sigma, 25, log=TRUE)
     ### Log-Likelihood
     mu <- tcrossprod(Data$X, t(beta))
     LL <- sum(dnorm(Data$y, mu, sigma, log=TRUE))
     ### Log-Posterior
     LP <- LL + beta.prior + sigma.prior
     Modelout <- list(LP=LP, Dev=-2*LL, Monitor=LP,
          yhat=rnorm(length(mu), mu, sigma), parm=parm)
     return(Modelout)
     }

set.seed(666)

############################  Initial Values  #############################
Initial.Values <- GIV(Model, MyData, PGF=TRUE)

########################  Population Monte Carlo  #########################
Fit <- PMC(Model, MyData, Initial.Values, Covar=NULL, Iterations=5,
     Thinning=1, alpha=NULL, M=1, N=100, CPUs=1)
Fit
print(Fit)
PosteriorChecks(Fit)
caterpillar.plot(Fit, Parms="beta")
plot(Fit, BurnIn=0, MyData, PDF=FALSE)
Pred <- predict(Fit, Model, MyData, CPUs=1)
summary(Pred, Discrep="Chi-Square")
plot(Pred, Style="Covariates", Data=MyData)
plot(Pred, Style="Density", Rows=1:9)
plot(Pred, Style="ECDF")
plot(Pred, Style="Fitted")
plot(Pred, Style="Jarque-Bera")
plot(Pred, Style="Predictive Quantiles")
plot(Pred, Style="Residual Density")
plot(Pred, Style="Residuals")
Levene.Test(Pred)
Importance(Fit, Model, MyData, Discrep="Chi-Square")

#End
}
\keyword{High Performance Computing}
\keyword{Importance Sampling}
\keyword{Monte Carlo}

\name{is.stationary}
\alias{is.stationary}
\title{Logical Check of Stationarity}
\description{
  This function returns \code{TRUE} if the object is stationary
  according to the \code{\link{Geweke.Diagnostic}} function, and
  \code{FALSE} otherwise.
}
\usage{
is.stationary(x)
}
\arguments{
    \item{x}{This is a vector, matrix, or object of class
      \code{demonoid}.}
}
\details{
  Stationarity, here, refers to the limiting distribution in a Markov
  chain. A series of samples from a Markov chain, in which each sample
  is the result of an iteration of a Markov chain Monte Carlo (MCMC)
  algorithm, is analyzed for stationarity, meaning whether or not the
  samples trend or its moments change across iterations. A stationary
  posterior distribution is an equilibrium distribution, and assessing
  stationarity is an important diagnostic toward inferring Markov chain
  convergence.

  In the cases of a matrix or an object of class \code{demonoid}, all
  Markov chains (as column vectors) must be stationary for
  \code{is.stationary} to return \code{TRUE}.

  Alternative ways to assess stationarity of chains are to use the
  \code{\link{BMK.Diagnostic}} or \code{\link{Heidelberger.Diagnostic}}
  functions.
}
\value{
  \code{is.stationary} returns a logical value indicating whether or not
  the supplied object is stationary according to the
  \code{\link{Geweke.Diagnostic}} function.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{BMK.Diagnostic}},
  \code{\link{Geweke.Diagnostic}},
  \code{\link{Heidelberger.Diagnostic}}, and
  \code{\link{LaplacesDemon}}.
}
\examples{
library(LaplacesDemon)
is.stationary(rnorm(100))
is.stationary(matrix(rnorm(100),10,10))
}
\keyword{Diagnostic}
\keyword{Stationarity}
\keyword{Utility}

\name{dist.Asymmetric.Log.Laplace}
\alias{dallaplace}
\alias{pallaplace}
\alias{qallaplace}
\alias{rallaplace}
\title{Asymmetric Log-Laplace Distribution}
\description{
  These functions provide the density, distribution function, quantile
  function, and random generation for the univariate, asymmetric,
  log-Laplace distribution with location parameter \eqn{\mu}{mu},
  scale parameter \eqn{\lambda}{\lambda}, and asymmetry or skewness
  parameter \eqn{\kappa}{kappa}.
}
\usage{
dallaplace(x, location=0, scale=1, kappa=1, log=FALSE)
pallaplace(q, location=0, scale=1, kappa=1)
qallaplace(p, location=0, scale=1, kappa=1)
rallaplace(n, location=0, scale=1, kappa=1)
}
\arguments{
  \item{x, q}{These are each a vector of quantiles.}
  \item{p}{This is a vector of probabilities.}
  \item{n}{This is the number of observations, which must be a positive
    integer that has length 1.}
  \item{location}{This is the location parameter \eqn{\mu}{mu}.}
  \item{scale}{This is the scale parameter \eqn{\lambda}{lambda}, which
    must be positive.}
  \item{kappa}{This is the asymmetry or skewness parameter
    \eqn{\kappa}{kappa}, which must be positive.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
}
\details{
  \itemize{
  \item Application: Continuous Univariate
  \item Density 1: \eqn{p(\theta) = \exp(-\mu)\frac{(\sqrt(2)\kappa / \lambda)(\sqrt(2) / \lambda\kappa)}{(\sqrt(2)\kappa / \lambda)+(\sqrt(2) / (\lambda\kappa))} \exp(-(\frac{\sqrt(2)\kappa}{\lambda})+1), \quad \theta \ge \exp(\mu)}{p(theta) = exp(-mu) * (sqrt(2)*kappa/lambda) * (sqrt(2)/(lambda*kappa)) / ((sqrt(2)*kappa/lambda)+(sqrt(2)/(lambda*kappa))) * exp(-((sqrt(2)*kappa/lambda)+1)), theta >= exp(mu)}
  \item Density 2: \eqn{p(\theta) = \exp(-\mu) \frac{(\sqrt(2)\kappa / \lambda) (\sqrt(2) / (\lambda\kappa))}{(\sqrt(2)\kappa / \lambda) + (\sqrt(2) / (\lambda\kappa))} \exp(\frac{\sqrt(2)(\log(\theta)-\mu)}{\lambda\kappa} - (\log(\theta)-\mu)), \quad \theta < \exp(\mu)}{p(theta) = exp(-mu) * (sqrt(2)*kappa/lambda) * (sqrt(2)/(lambda*kappa)) / ((sqrt(2)*kappa/lambda)+(sqrt(2)/(lambda*kappa))) * exp(((sqrt(2)*(log(theta)-mu)) / (lambda*kappa)) - (log(theta)-mu)), theta < exp(mu)}
  \item Inventor: Pierre-Simon Laplace
  \item Notation 1: \eqn{\theta \sim \mathcal{ALL}(\mu, \lambda,
    \kappa)}{theta ~ ALL(mu, lambda, kappa)}
  \item Notation 2: \eqn{p(\theta) = \mathcal{ALL}(\theta | \mu,
    \lambda, \kappa)}{p(theta) = ALL(theta | mu, lambda, kappa)}
  \item Parameter 1: location parameter \eqn{\mu}{mu}
  \item Parameter 2: scale parameter \eqn{\lambda > 0}{lambda > 0}
  \item Mean: \eqn{E(\theta) = }{E(theta) = }
  \item Variance: \eqn{var(\theta) = }{var(theta) = }
  \item Mode: \eqn{mode(\theta) = }{mode(theta) = }
}

The univariate, asymmetric log-Laplace distribution is derived from the
Laplace distribution. Multivariate and symmetric versions also exist.

These functions are similar to those in the \code{VGAM} package.
}
\value{
  \code{dallaplace} gives the density,
  \code{pallaplace} gives the distribution function,
  \code{qallaplace} gives the quantile function, and
  \code{rallaplace} generates random deviates.
}
\references{
  Kozubowski, T. J. and Podgorski, K. (2003). "Log-Laplace Distributions".
  \emph{International Mathematical Journal}, 3, p. 467--495.
}
\seealso{
  \code{\link{dalaplace}},
  \code{\link{dexp}},
  \code{\link{dlaplace}},
  \code{\link{dlaplacep}},
  \code{\link{dllaplace}},
  \code{\link{dmvl}},
  \code{\link{dnorm}},
  \code{\link{dnormp}},
  \code{\link{dnormv}}.
}
\examples{
library(LaplacesDemon)
x <- dallaplace(1,0,1,1)
x <- pallaplace(1,0,1,1)
x <- qallaplace(0.5,0,1,1)
x <- rallaplace(100,0,1,1)

#Plot Probability Functions
x <- seq(from=0.1, to=10, by=0.1)
plot(x, dallaplace(x,0,1,0.5), ylim=c(0,1), type="l", main="Probability Function",
     ylab="density", col="red")
lines(x, dallaplace(x,0,1,1), type="l", col="green")
lines(x, dallaplace(x,0,1,5), type="l", col="blue")
legend(5, 0.9, expression(paste(mu==0, ", ", lambda==1, ", ", kappa==0.5),
     paste(mu==0, ", ", lambda==1, ", ", kappa==1),
     paste(mu==0, ", ", lambda==1, ", ", kappa==5)),
     lty=c(1,1,1), col=c("red","green","blue"))
}
\keyword{Distribution}

\name{BigData}
\alias{BigData}
\title{Big Data}
\description{
  This function enables Bayesian inference with data that is too large
  for computer memory (RAM) with the simplest method: reading in batches
  of data (where each batch is a section of rows), applying a function
  to the batch, and combining the results.
}
\usage{
BigData(file, nrow, ncol, size=1, Method="add", CPUs=1, Type="PSOCK",
FUN, ...)
}
\arguments{
  \item{file}{This required argument accepts a path and filename that
    must refer to a .csv file, and that must contain only a numeric
    matrix without a header, row names, or column names.}
  \item{nrow}{This required argument accepts a scalar integer that
    indicates the number of rows in the big data matrix.}
  \item{ncol}{This required argument accepts a scalar integer that
    indicates the number of columns in the big data matrix.}
  \item{size}{This argument accepts a scalar integer that specifies the
    number of rows of each batch. The last batch is not required to have
    the same number of rows as the other batches. The largest possible
    size, and therefore the fewest number of batches, should be
    preferred.}
  \item{Method}{This argument accepts a scalar string, defaults to
    "add", and alternatively accepts "rbind". When
    \code{Method="rbind"}, the user-specified function \code{FUN} is
    applied to each batch, and results are combined together by rows.
    For example, if calculating \eqn{\mu = \textbf{X}\beta}{mu = XB} in,
    say, 10 batches, then the output column vector \eqn{\mu}{mu} is equal
    to the number of rows of the big data set.}
  \item{CPUs}{This argument accepts an integer that specifies the number
    of central processing units (CPUs) of the multicore computer or
    computer cluster. This argument defaults to \code{CPUs=1}, in which
    parallel processing does not occur.}
  \item{Type}{This argument specifies the type of parallel processing to
    perform, accepting either \code{Type="PSOCK"} or
    \code{Type="MPI"}.}
  \item{FUN}{This required argument accepts a user-specified function
    that will be performed on each batch. The first argument in the
    function must be the data.}
  \item{...}{Additional arguments are used within the user-specified
    function. Additional arguments often refer to parameters.}
}
\details{
  Big data is defined loosely here as data that is too large for
  computer memory (RAM). The \code{BigData} function uses the
  split-apply-combine strategy with a big data set. The unmanageable
  big data set is split into smaller, manageable pieces (batches),
  a function is applied to each batch, and results are combined.

  Each iteration, the \code{BigData} function opens a connection to a
  big data set and keeps the connection open while the \code{scan}
  function reads in each batch of data (elsewhere, batches are often
  referred to chunks). A user-specified function is applied to each
  batch of data, the results are combined together, the connection is
  closed, and the results are returned.

  As an introductory example, suppose a statistician updates a linear
  regression model, but the design matrix \eqn{\textbf{X}}{X} is too
  large for computer memory. Suppose the design matrix has 100 million
  rows, and the statistician specifies \code{size=1e6}. The statistician
  combines dependent variable \eqn{\textbf{y}} with design matrix
  \eqn{\textbf{X}}. Each iteration in \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}}, \code{\link{LaplacesDemon}},
  \code{\link{PMC}}, or \code{\link{VariationalBayes}}, the
  \code{BigData} function sequentially reads in one million rows of the
  combined data \eqn{\textbf{X}}, calculates expectation vector
  \eqn{\mu}, and finally returns the sum of the log-likelihood. The sum
  of the log-likelihood is added together for all batches, and returned.
  
  There are many limitations with this function.

  This function is not fast, in the sense that the entire big data set
  is processed in batches, each iteration. With iterative methods, this
  may perform well, albeit slowly.

  There are many functions that cannot be performed on batches, though
  most models in the Examples vignette may easily be updated with big
  data.

  Large matrices of samples are unaddressed, only the data.

  Although many (but not all) models may be estimated, many additional
  functions in this package will not work when applied after the model
  has updated. Instead, a batch or random sample of data (see the
  \code{\link{read.matrix}} function for sampling from big data) should
  be used in the usual way, in the \code{Data} argument, and the
  \code{Model} function coded in the usual way without the
  \code{BigData} function.

  Parallel processing may be performed when the user specifies
  \code{CPUs} to be greater than one, implying that the specified number
  of CPUs exists and is available. Parallelization may be performed on a
  multicore computer or a computer cluster. Either a Simple Network of
  Workstations (SNOW) or Message Passing Interface (MPI) is used. Each
  call to \code{BigData} establishes and closes the parallelization,
  which is costly, and unfortunately results in copious output to the
  console. With small data sets, parallel processing may be slower, due
  to computer network communication. With larger data sets, the user
  should experience a faster run-time.

  There have been several alternative approaches suggested for big data.

  Huang and Gelman (2005) propose that the user creates batches by
  sampling from big data, updating a separate Bayesian model on each
  batch, and combining the results into a consensus posterior. This
  many-mini-model approach may be faster when feasible, because multiple
  models may be updated in parallel, say one per CPU. Such results will
  work with all functions in this package. With the many-mini-model
  approach, several methods are proposed for combining posterior samples
  from batch-level models, such as by using a normal approximation,
  updating from prior to posterior sequentially (the posterior from the
  last batch becomes the prior of the next batch), sample from the full
  posterior via importance sampling from the batched posteriors, and
  more.

  Scott et al. (2013) propose a method that they call Consensus Monte
  Carlo, which consists of breaking the data down into chunks, calling
  each chunk a shard, and use a many-mini-model approach as well, but
  propose their own method of weighting the posteriors back together.
  
  Balakrishnan and Madigan (2006) introduced a Sequential Monte Carlo
  (SMC) sampler, a refinement of an earlier proposal, that was designed
  for big data. It makes one pass through the massive data set, after an
  initial MCMC estimation on a small sample. Each particle is updated
  for each record, resulting in numerous evaluations per record.

  Welling and Teh (2011) proposed a new class of MCMC sampler in which
  only a random sample of big data is used each iteration. The
  stochastic gradient Langevin dynamics (SGLD) algorithm is available
  in the \code{\link{LaplacesDemon}} function.

  An important alternative to consider is using the \code{ff} package,
  where "ff" stands for fast access file. The \code{ff} package has been
  tested successfully with updating a model in \code{LaplacesDemon}.
  Once the big data set, say \eqn{\textbf{X}}{X}, is an object of
  class \code{ff_matrix}, simply include it in the list of data as
  usual, and modify the \code{Model} specification function
  appropriately. For example, change \code{mu <- tcrossprod(X, t(beta))}
  to \code{mu <- tcrossprod(X[], t(beta))}. The \code{ff} package is
  not included as a dependency in the \code{LaplacesDemon} package, so
  it must be installed and activated.
}
\value{
  The \code{BigData} function returns output that is the result of
  performing a user-specified function on batches of big data. Output is
  a matrix, and may have one or more column vectors.
}
\references{
  Balakrishnan, S. and Madigan, D. (2006). "A One-Pass Sequential Monte
  Carlo Method for Bayesian Analysis of Massive Datasets".
  \emph{Bayesian Analysis}, 1(2), p. 345--362.

  Huang, Z. and Gelman, A. (2005) "Sampling for Bayesian Computation
  with Large Datasets". \emph{SSRN eLibrary}.

  Scott, S.L., Blocker, A.W. and Bonassi, F.V. (2013). "Bayes and Big
  Data: The Consensus Monte Carlo Algorithm". In \emph{Bayes 250}.

  Welling, M. and Teh, Y.W. (2011). "Bayesian Learning via Stochastic
  Gradient Langevin Dynamics". \emph{Proceedings of the 28th
  International Conference on Machine Learning (ICML)}, p. 681--688.
}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{LaplacesDemon.RAM}},
  \code{\link{PMC}},
  \code{\link{PMC.RAM}},
  \code{\link{read.matrix}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
### Below is an example of a linear regression model specification
### function in which BigData reads in a batch of 1,000 records of
### Data$N records from a data set that is too large to fully open
### in memory. The example simulates on 10,000 records, which is
### not big data; it's just a toy example. The data set is file X.csv,
### and the first column of matrix X is the dependent variable y. The
### user supplies a function to BigData along with parameters beta and
### sigma. When each batch of 1,000 records is read in,
### mu = XB is calculated, and then the LL is calculated as
### y ~ N(mu, sigma^2). These results are added together from all
### batches, and returned as LL.

library(LaplacesDemon)
N <- 10000
J <- 10 #Number of predictors, including the intercept
X <- matrix(1,N,J)
for (j in 2:J) {X[,j] <- rnorm(N,runif(1,-3,3),runif(1,0.1,1))}
beta.orig <- runif(J,-3,3)
e <- rnorm(N,0,0.1)
y <- as.vector(tcrossprod(beta.orig, X) + e)
mon.names <- c("LP","sigma")
parm.names <- as.parm.names(list(beta=rep(0,J), log.sigma=0))
PGF <- function(Data) return(c(rnormv(Data$J,0,0.01),
     log(rhalfcauchy(1,1))))
MyData <- list(J=J, PGF=PGF, N=N, mon.names=mon.names,
     parm.names=parm.names) #Notice that X and y are not included here
filename <- tempfile("X.csv")  
write.table(cbind(y,X), filename, sep=",", row.names=FALSE,
  col.names=FALSE)

Model <- function(parm, Data)
     {
     ### Parameters
     beta <- parm[1:Data$J]
     sigma <- exp(parm[Data$J+1])
     ### Log(Prior Densities)
     beta.prior <- sum(dnormv(beta, 0, 1000, log=TRUE))
     sigma.prior <- dhalfcauchy(sigma, 25, log=TRUE)
     ### Log-Likelihood
     LL <- BigData(file=filename, nrow=Data$N, ncol=Data$J+1, size=1000,
          Method="add", CPUs=1, Type="PSOCK",
          FUN=function(x, beta, sigma) sum(dnorm(x[,1], tcrossprod(x[,-1],
               t(beta)), sigma, log=TRUE)), beta, sigma)
     ### Log-Posterior
     LP <- LL + beta.prior + sigma.prior
     Modelout <- list(LP=LP, Dev=-2*LL, Monitor=c(LP,sigma),
          yhat=0,#rnorm(length(mu), mu, sigma),
          parm=parm)
     return(Modelout)
     }

### From here, the user may update the model as usual.
}
\keyword{High Performance Computing}
\keyword{Utility}

\name{data.demonchoice}
\alias{demonchoice}
\title{Demon Choice Data Set}
\usage{data(demonchoice)}
\description{
  This data set is for discrete choice models and consists of the choice
  of commuting route to school: arterial, two-lane, or freeway. There
  were 151 Pennsylvania commuters who started from a residential complex
  in State College, PA, and commute to downtown State College.
}
\format{
  This data frame contains 151 rows of individual choices and 9
  columns. The following data dictionary describes each variable or
  column.
  \describe{
    \item{\code{Choice}}{This is the route choice: four-lane arterial
      (35 MPH speed limit), two-lane highway (35 MPH speed limit, with
      one lane in each direction), or a limited-access four-lane freeway
      (55 MPH speed liimit.)}
    \item{\code{HH.Income}}{This is an ordinal variable of annual
      household income of the commuter in USD. There are four
      categories: 1 is less than 20,000 USD, 2 is 20,000-29,999 USD, 3
      is 30,000-39,999 USD, and 4 is 40,000 USD or greater.}
    \item{\code{Vehicle.Age}}{This is the age in years of the vehicle of
      the commuter.}
    \item{\code{Stop.Signs.Arterial}}{This is the number of stop signs
      along the arterial route.}
    \item{\code{Stop.Signs.Two.Lane}}{This is the number of stop signs
      along the two-lane route.}
    \item{\code{Stop.Signs.Freeway}}{This is the number of stop signs
      along the freeway route.}
    \item{\code{Distance.Arterial}}{This is distance in miles of the
      arterial route.}
    \item{\code{Distance.Two.Lane}}{This is the distance in miles of
      the two-lane route.}
    \item{\code{Distance.Freeway}}{This is the distance in miles of
      the freeway route.}
    }
}
\source{Washington, S., Congdon, P., Karlaftis, M., and Mannering,
  F. (2009). "Bayesian Multinomial Logit: Theory and Route Choice
  Example". Transportation Research Record, 2136, p. 28--36.}
\keyword{datasets}

\name{is.constant}
\alias{is.constant}
\title{Logical Check of a Constant}
\description{
  This function provides a logical test of whether or not a vector is a
  constant.
}
\usage{
is.constant(x)
}
\arguments{
  \item{x}{This is a vector.}
}
\details{
  As opposed to a variable, a constant is a vector in which the elements
  contain less than or equal to one unique value.
}
\value{
  The \code{is.constant} function returns a logical result, reporting
  \code{TRUE} when a vector is a constant, or \code{FALSE} otherwise.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{unique}}
}
\examples{
library(LaplacesDemon)
is.constant(rep(1,10)) #TRUE
is.constant(1:10) #FALSE
}
\keyword{Utility}

\name{as.parm.names}
\alias{as.parm.names}
\title{Parameter Names}
\description{
  This function creates a vector of parameter names from a list of
  parameters, and the list may contain any combination of scalars,
  vectors, matrices, upper-triangular matrices, and arrays.
}
\usage{
as.parm.names(x, uppertri=NULL)
}
\arguments{
  \item{x}{This required argument is a list of named parameters. The
    list may contain scalars, vectors, matrices, and arrays. The value
    of the named parameters does not matter here, though they are
    usually set to zero. However, if a missing value occurs, then the
    associated element is omitted in the output.}
  \item{uppertri}{This optional argument must be a vector with a length
    equal to the number of named parameters. Each element in
    \code{uppertri} must be either a 0 or 1, where a 1 indicates that an
    upper triangular matrix will be used for the associated element in
    the vector of named parameters. Each element of \code{uppertri} is
    associated with a named parameter. The \code{uppertri} argument does
    not function with arrays.}
}
\details{
  Each \code{model} function for \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}}, \code{\link{LaplacesDemon}},
  \code{\link{PMC}}, or \code{\link{VariationalBayes}} requires a vector
  of parameters (specified at first as \code{Initial.Values}) and a list
  of data. One component in the list of data must be named
  \code{parm.names}. Each element of \code{parm.names} is a name
  associated with the corresponding parameter in \code{Initial.Values}.

  The \code{parm.names} vector is easy to program explicitly for a simple
  model, but can require considerably more programming effort for more
  complicated models. The \code{as.parm.names} function is a utility
  function designed to minimize programming by the user.

  For example, a simple model may only require \code{parm.names <-
  c("alpha", "beta[1]", "beta[2]", "sigma")}. A more complicated model
  may contain hundreds of parameters that are a combination of scalars,
  vectors, matrices, upper-triangular matrices, and arrays, and is the
  reason for the \code{as.parm.names} function. The code for the above
  is \code{as.parm.names(list(alpha=0, beta=rep(0,2), sigma=0))}.

  In the case of an upper-triangular matrix, simply pass the full matrix
  to \code{as.parm.names} and indicate that only the upper-triangular
  will be used via the \code{uppertri} argument. For example,
  \code{as.parm.names(list(beta=rep(0,J),U=diag(K)), uppertri=c(0,1))}
  creates parameter names for a vector of \eqn{\beta} parameters of
  length \eqn{J} and an upper-triangular matrix \eqn{\textbf{U}} of
  dimension \eqn{K}.

  Numerous examples may be found in the accompanying ``Examples''
  vignette.
}
\value{
  This function returns a vector of parameter names.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{IterativeQuadrature}}
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
library(LaplacesDemon)
N <- 100
J <- 5
y <- rnorm(N,0,1)
X <- matrix(runif(N*J,-2,2),N,J)
S <- diag(J)
T <- diag(2)
mon.names <- c("LP","sigma")
parm.names <- as.parm.names(list(log.sigma=0, beta=rep(0,J), S=diag(J),
     T=diag(2)), uppertri=c(0,0,0,1))
MyData <- list(J=J, N=N, S=S, T=T, X=X, mon.names=mon.names,
     parm.names=parm.names, y=y)
MyData
}
\keyword{Parameter Names}
\keyword{Utility}

\name{dist.Truncated}
\alias{dtrunc}
\alias{extrunc}
\alias{ptrunc}
\alias{qtrunc}
\alias{rtrunc}
\alias{vartrunc}
\title{Truncated Distributions}
\description{
  Density, distribution function, quantile function and random
  generation for truncated distributions.
}
\usage{
dtrunc(x, spec, a=-Inf, b=Inf, log=FALSE, ...)
extrunc(spec, a=-Inf, b=Inf, ...)
ptrunc(x, spec, a=-Inf, b=Inf, ...)
qtrunc(p, spec, a=-Inf, b=Inf, ...)
rtrunc(n, spec, a=-Inf, b=Inf, ...)
vartrunc(spec, a=-Inf, b=Inf, ...)
}
\arguments{
  \item{n}{This is a the number of random draws for \code{rtrunc}.}
  \item{p}{This is a vector of probabilities.}
  \item{x}{This is a vector to be evaluated.}
  \item{spec}{The base name of a probability distribution is
    specified here. For example, to estimate the density of a
    truncated normal distribution, enter \code{norm}.}
  \item{a}{This is the lower bound of truncation, which defaults
    to negative infinity.}
  \item{b}{This is the upper bound of truncation, which defaults
    to infinity.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
  \item{\dots}{Additional arguments pertain to the probability
    distribution specified in the \code{spec} argument.}
}
\details{
  A truncated distribution is a conditional distribution that results
  from a priori restricting the domain of some other probability
  distribution. More than merely preventing values outside of truncated
  bounds, a proper truncated distribution integrates to one within the
  truncated bounds. For more information on propriety, see
  \code{\link{is.proper}}. In contrast to a truncated distribution, a
  censored distribution occurs when the probability distribution is
  still allowed outside of a pre-specified range. Here, distributions
  are truncated to the interval \eqn{[a,b]}, such as \eqn{p(\theta) \in
  [a,b]}{p(theta) in [a,b]}.

  The \code{dtrunc} function is often used in conjunction with the
  \code{\link{interval}} function to truncate prior probability
  distributions in the model specification function for use with these
  numerical approximation functions: \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, and \code{\link{PMC}}.

  The R code of Nadarajah and Kotz (2006) has been modified to work with
  log-densities.
}
\value{
  \code{dtrunc} gives the density,
  \code{extrunc} gives the expectation,
  \code{ptrunc} gives the distribution function,
  \code{qtrunc} gives the quantile function,
  \code{rtrunc} generates random deviates, and
  \code{vartrunc} gives the variance of the truncated distribution.
}
\references{
  Nadarajah, S. and Kotz, S. (2006). "R Programs for Computing Truncated
  Distributions". \emph{Journal of Statistical Software}, 16,
  Code Snippet 2, p. 1--8.
}
\seealso{
  \code{\link{interval}},
  \code{\link{is.proper}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, and
  \code{\link{PMC}}.
}
\examples{
library(LaplacesDemon)
x <- seq(-0.5, 0.5, by = 0.1)
y <- dtrunc(x, "norm", a=-0.5, b=0.5, mean=0, sd=2)
}
\keyword{Distribution}

\name{plot.vb}
\alias{plot.vb}
\title{Plot the output of \code{\link{VariationalBayes}}}
\description{
  This may be used to plot, or save plots of, the iterated history of
  the parameters and variances, and if posterior samples were taken,
  density plots of parameters and monitors in an object of class
  \code{vb}.
}
\usage{\method{plot}{vb}(x, Data, PDF=FALSE, Parms, \dots)}
\arguments{
  \item{x}{
    This required argument is an object of class \code{vb}.}
  \item{Data}{
    This required argument must receive the list of data that was
    supplied to \code{\link{VariationalBayes}} to create the object
    of class \code{vb}.}
  \item{PDF}{
    This logical argument indicates whether or not the user wants
    Laplace's Demon to save the plots as a .pdf file.}
  \item{Parms}{
    This argument accepts a vector of quoted strings to be matched for
    selecting parameters for plotting. This argument defaults to
    \code{NULL} and selects every parameter for plotting. Each quoted
    string is matched to one or more parameter names with the
    \code{grep} function. For example, if the user specifies
    \code{Parms=c("eta", "tau")}, and if the parameter names
    are beta[1], beta[2], eta[1], eta[2], and tau, then all parameters
    will be selected, because the string \code{eta} is within
    \code{beta}. Since \code{grep} is used, string matching uses
    regular expressions, so beware of meta-characters, though these are
    acceptable: ".", "[", and "]".}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  The plots are arranged in a \eqn{3 \times 3}{3 x 3} matrix. The
  purpose of the iterated history plots is to show how the value of each
  parameter, variance, and the deviance changed by iteration as the
  \code{\link{VariationalBayes}} attempted to maximize the logarithm
  of the unnormalized joint posterior density. If the algorithm
  converged, and if \code{sir=TRUE} in
  \code{\link{VariationalBayes}}, then plots are produced of
  selected parameters and all monitored variables.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{\code{\link{VariationalBayes}}}
\examples{### See the VariationalBayes function for an example.}
\keyword{Plot}

\name{data.demontexas}
\alias{demontexas}
\title{Demon Space-Time Data Set}
\usage{data(demontexas)}
\description{
  This data set is for space-time models that require latitude and
  longitude, or coordinates. This data set consists of the minimum,
  mean, and maximum temperatures in Texas for 13 months.
}
\format{
  This data frame contains 369 rows of sites in Texas and 43
  columns. The following data dictionary describes each variable or
  column.
  \describe{
    \item{\code{Elevation}}{This is the elevation of the site.}
    \item{\code{Latitude}}{This is the latitude of the site.}
    \item{\code{Longitude}}{This is the longitude of the site.}
    \item{\code{Gulf}}{This is a gulf indicator of the site.}
    \item{\code{Max1}}{This is the maximum temperature in month 1.}
    \item{\code{Max2}}{This is the maximum temperature in month 2.}
    \item{\code{Max3}}{This is the maximum temperature in month 3.}
    \item{\code{Max4}}{This is the maximum temperature in month 4.}
    \item{\code{Max5}}{This is the maximum temperature in month 5.}
    \item{\code{Max6}}{This is the maximum temperature in month 6.}
    \item{\code{Max7}}{This is the maximum temperature in month 7.}
    \item{\code{Max8}}{This is the maximum temperature in month 8.}
    \item{\code{Max9}}{This is the maximum temperature in month 9.}
    \item{\code{Max10}}{This is the maximum temperature in month 10.}
    \item{\code{Max11}}{This is the maximum temperature in month 11.}
    \item{\code{Max12}}{This is the maximum temperature in month 12.}
    \item{\code{Max13}}{This is the maximum temperature in month 13.}
    \item{\code{Mean1}}{This is the mean temperature in month 1.}
    \item{\code{Mean2}}{This is the mean temperature in month 2.}
    \item{\code{Mean3}}{This is the mean temperature in month 3.}
    \item{\code{Mean4}}{This is the mean temperature in month 4.}
    \item{\code{Mean5}}{This is the mean temperature in month 5.}
    \item{\code{Mean6}}{This is the mean temperature in month 6.}
    \item{\code{Mean7}}{This is the mean temperature in month 7.}
    \item{\code{Mean8}}{This is the mean temperature in month 8.}
    \item{\code{Mean9}}{This is the mean temperature in month 9.}
    \item{\code{Mean10}}{This is the mean temperature in month 10.}
    \item{\code{Mean11}}{This is the mean temperature in month 11.}
    \item{\code{Mean12}}{This is the mean temperature in month 12.}
    \item{\code{Mean13}}{This is the mean temperature in month 13.}
    \item{\code{Min1}}{This is the minimum temperature in month 1.}
    \item{\code{Min2}}{This is the minimum temperature in month 2.}
    \item{\code{Min3}}{This is the minimum temperature in month 3.}
    \item{\code{Min4}}{This is the minimum temperature in month 4.}
    \item{\code{Min5}}{This is the minimum temperature in month 5.}
    \item{\code{Min6}}{This is the minimum temperature in month 6.}
    \item{\code{Min7}}{This is the minimum temperature in month 7.}
    \item{\code{Min8}}{This is the minimum temperature in month 8.}
    \item{\code{Min9}}{This is the minimum temperature in month 9.}
    \item{\code{Min10}}{This is the minimum temperature in month 10.}
    \item{\code{Min11}}{This is the minimum temperature in month 11.}
    \item{\code{Min12}}{This is the minimum temperature in month 12.}
    \item{\code{Min13}}{This is the minimum temperature in month 13.}
    }
}
\source{\url{https://users.stat.ufl.edu/~winner/datasets.html}}
\keyword{datasets}

\name{Stick}
\alias{Stick}
\title{Truncated Stick-Breaking}
\description{
  The \code{Stick} function provides the utility of truncated
  stick-breaking regarding the vector
  \eqn{\theta}{theta}. Stick-breaking is commonly referred to as a
  stick-breaking process, and is used often in a Dirichlet
  process (Sethuraman, 1994). It is commonly associated with
  infinite-dimensional mixtures, but in practice, the `infinite' number
  is truncated to a finite number, since it is impossible to estimate an
  infinite number of parameters (Ishwaran and James, 2001).
}
\usage{
Stick(theta)
}
\arguments{
  \item{theta}{This required argument, \eqn{\theta}{theta} is a vector
    of length \eqn{(M-1)}{M-1} regarding \eqn{M}{M} mixture components.}
}
\details{
  The Dirichlet process (DP) is a stochastic process used in Bayesian
  nonparametric modeling, most commonly in DP mixture models, otherwise
  known as infinite mixture models. A DP is a distribution over
  distributions. Each draw from a DP is itself a discrete
  distribution. A DP is an infinite-dimensional generalization of
  Dirichlet distributions. It is called a DP because it has
  Dirichlet-distributed, finite-dimensional, marginal distributions,
  just as the Gaussian process has Gaussian-distributed,
  finite-dimensional, marginal distributions. Distributions drawn from a
  DP cannot be described using a finite number of parameters, thus the
  classification as a nonparametric model. The truncated stick-breaking
  (TSB) process is associated with a truncated Dirichlet process (TDP).

  An example of a TSB process is cluster analysis, where the number of
  clusters is unknown and treated as mixture components. In such a
  model, the TSB process calculates probability vector \eqn{\pi}{pi}
  from \eqn{\theta}{theta}, given a user-specified maximum number of
  clusters to explore as \eqn{C}{C}, where \eqn{C}{C} is the length of
  \eqn{\theta + 1}{theta + 1}. Vector \eqn{\pi}{pi} is assigned a TSB
  prior distribution (for more information, see \code{\link{dStick}}).

  Elsewhere, each element of \eqn{\theta}{theta} is constrained to the
  interval (0,1), and the original TSB form is beta-distributed with the
  \eqn{\alpha}{alpha} parameter of the beta distribution constrained
  to 1 (Ishwaran and James, 2001). The \eqn{\beta}{beta} hyperparameter
  in the beta distribution is usually gamma-distributed.

  A larger value for a given \eqn{\theta_m}{theta[m]} is associated
  with a higher probability of the associated mixture component,
  however, the proportion changes according to the position of the
  element in the \eqn{\theta}{theta} vector.

  A variety of stick-breaking processes exist. For example, rather than
  each \eqn{\theta}{theta} being beta-distributed, there have been other
  forms introduced such as logistic and probit, among others.
}
\value{
  The \code{Stick} function returns a probability vector wherein each
  element relates to a mixture component.
}
\references{
  Ishwaran, H. and James, L. (2001). "Gibbs Sampling Methods for Stick
  Breaking Priors". \emph{Journal of the American Statistical
  Association}, 96(453), p. 161--173.
  
  Sethuraman, J. (1994). "A Constructive Definition of Dirichlet
  Priors". \emph{Statistica Sinica}, 4, p. 639--650.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{ddirichlet}},
  \code{\link{dmvpolya}}, and 
  \code{\link{dStick}}.
}
\keyword{Stick-Breaking Process}
\keyword{Truncated Stick-Breaking Process}
\keyword{Utility}

\name{predict.vb}
\alias{predict.vb}
\title{Posterior Predictive Checks}
\description{
  This may be used to predict either new, unobserved instances of
  \eqn{\textbf{y}}{y} (called \eqn{\textbf{y}^{new}}{y[new]}) or
  replicates of \eqn{\textbf{y}}{y} (called
  \eqn{\textbf{y}^{rep}}{y[rep]}), and then perform posterior
  predictive checks. Either \eqn{\textbf{y}^{new}}{y[new]} or
  \eqn{\textbf{y}^{rep}}{y[rep]} is predicted given an object of
  class \code{vb}, the model specification, and data. This function
  requires that posterior samples were produced with
  \code{\link{VariationalBayes}}.
}
\usage{\method{predict}{vb}(object, Model, Data, CPUs=1, Type="PSOCK", \dots)}
\arguments{
  \item{object}{An object of class \code{vb} is required.}
  \item{Model}{The model specification function is required.}
  \item{Data}{A data set in a list is required. The dependent
    variable is required to be named either \code{y} or \code{Y}.}
  \item{CPUs}{This argument accepts an integer that specifies the number
    of central processing units (CPUs) of the multicore computer or
    computer cluster. This argument defaults to \code{CPUs=1}, in which
    parallel processing does not occur.}
  \item{Type}{This argument specifies the type of parallel processing to
    perform, accepting either \code{Type="PSOCK"} or
    \code{Type="MPI"}.}
  \item{\dots}{Additional arguments are unused.}
}
\details{
  Since Variational Bayes characterizes marginal posterior
  distributions with modes and variances, and posterior predictive
  checks involve samples, the \code{predict.vb} function requires
  the use of independent samples of the marginal posterior
  distributions, provided by \code{\link{VariationalBayes}} when
  \code{sir=TRUE}.

  The samples of the marginal posterior distributions of the target
  distributions (the parameters) are passed along with the data to the
  \code{Model} specification and used to draw samples from the deviance
  and monitored variables. At the same time, the fourth component in the
  returned list, which is labeled \code{yhat}, is a vector of
  expectations of \eqn{\textbf{y}}{y}, given the samples, model
  specification, and data. To predict \eqn{\textbf{y}^{rep}}{y[rep]},
  simply supply the data set used to estimate the model. To predict
  \eqn{\textbf{y}^{new}}{y[new]}, supply a new data set instead (though
  for some model specifications, this cannot be done, and
  \eqn{\textbf{y}_{new}}{y[new]} must be specified in the \code{Model}
  function). If the new data set does not have \eqn{\textbf{y}}{y}, then
  create \code{y} in the list and set it equal to something sensible,
  such as \code{mean(y)} from the original data set.

  The variable \code{y} must be a vector. If instead it is matrix
  \code{Y}, then it will be converted to vector \code{y}. The vectorized
  length of \code{y} or \code{Y} must be equal to the vectorized length
  of \code{yhat}, the fourth component of the returned list of the
  \code{Model} function.

  Parallel processing may be performed when the user specifies
  \code{CPUs} to be greater than one, implying that the specified number
  of CPUs exists and is available. Parallelization may be performed on a
  multicore computer or a computer cluster. Either a Simple Network of
  Workstations (SNOW) or Message Passing Interface is used (MPI). With
  small data sets and few samples, parallel processing may be slower,
  due to computer network communication. With larger data sets and more
  samples, the user should experience a faster run-time.

  For more information on posterior predictive checks, see
  \url{https://web.archive.org/web/20150215050702/http://www.bayesian-inference.com/posteriorpredictivechecks}.
}
\value{
  This function returns an object of class \code{vb.ppc} (where
  ``ppc'' stands for posterior predictive checks). The returned object
  is a list with the following components:
  \item{y}{
    This stores \eqn{\textbf{y}}{y}, the dependent variable.}
  \item{yhat}{
    This is a \eqn{N \times S}{N x S} matrix, where \eqn{N} is the
    number of records of \eqn{\textbf{y}}{y} and \eqn{S} is the number of 
    posterior samples.}
  \item{Deviance}{
    This is a vector of length \eqn{S}, where \eqn{S} is the number of
    independent posterior samples. Samples are obtained with the
    sampling importance resampling algorithm, \code{\link{SIR}}.}
  \item{monitor}{
    This is a \eqn{N \times S}{N x S} matrix, where \eqn{N} is the
    number of monitored variables and \eqn{S} is the number of independent
    posterior samples. Samples are obtained with the sampling importance
    resampling algorithm, \code{\link{SIR}}.}
}
\author{Statisticat, LLC.}
\seealso{
  \code{\link{SIR}} and
  \code{\link{VariationalBayes}}.
}
\keyword{High Performance Computing}
\keyword{Posterior Predictive Checks}
\keyword{Predict}

\name{Elicitation}
\alias{delicit}
\alias{elicit}
\title{Prior Elicitation}
\description{
  Prior elicitation is the act of inducing personal opinion to be
  expressed by the probabilities the person associates with an event
  (Savage, 1971). The \code{elicit} function elicits personal opinion
  and the \code{delicit} function estimates probability density to be
  used with model specification in the
  \code{\link{IterativeQuadrature}}, \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}}, \code{\link{LaplacesDemon.hpc}},
  \code{\link{PMC}}, or \code{\link{VariationalBayes}} functions.
}
\usage{
delicit(theta, x, a=-Inf, b=Inf, log=FALSE)
elicit(n, cats, cat.names, show.plot=FALSE)
}
\arguments{
  \item{theta}{This is a scalar or vector of parameters for which the
    density is estimated with respect to the kernel density estimate of
    \code{x}.}
  \item{x}{This is the elicited vector.}
  \item{a}{This is an optional lower bound for support.}
  \item{b}{This is an optional upper bound for support.}
  \item{log}{Logical. If \code{log=TRUE}, then the logarithm of the
    density is returned.}
  \item{n}{This is the number of chips.}
  \item{cats}{This is a vector of \eqn{k} categories, bins, or
    intervals. When the variable is continuous, the mid-point of each
    category is used. For example, if the continuous interval [0,1] has
    5 equal-sized categories, then \code{cats=c(0.1,0.3,0.5,0.7,0.9)}.}
  \item{cat.names}{This is a vector of category names. For example, if
    the continuous interval [0,1] has 5 equal-sized categories, then one
    way or naming the categories may be \code{cat.names=c("0:<.2",
      ".2:<.4", ".4:<.6", ".6:<.8", ".8:1")}.}
  \item{show.plot}{Logical. If \code{show.plot=TRUE}, then a barplot is
    shown after each allocation of chips.}
}
\details{
  The \code{elicit} function elicits a univariate, discrete,
  non-conjugate, informative, prior probability distribution by
  offering a number of chips (specified as \code{n} by the statistician)
  for the user to allocate into categories specified by the
  statistician. The results of multiple elicitations (meaning, with
  multiple people), each the output of \code{elicit}, may be combined
  with the \code{c} function in base R.

  This discrete distribution is included with the data for
  a model and supplied to a model specification function, where in turn
  it is supplied to the \code{delicit} function, which estimates the
  density at the current value of the prior distribution,
  \eqn{p(\theta)}{p(theta)}. The prior distribution may be either
  continuous or discrete, will be proper, and may have bounded support
  (constrained to an interval).

  For a minimal example, a statistician elicits the prior probability
  distribution for a regression effect, \eqn{\beta}. Non-statisticians
  would not be asked about expected parameters, but could be asked about
  how much \eqn{\textbf{y}}{y} would be expected to change given a
  one-unit change in \eqn{\textbf{x}}{x}. After consulting with others
  who have prior knowledge, the support does not need to be bounded,
  and their guesses at the range result in the statistician creating
  5 catgories from the interval [-1,4], where each interval has a width
  of one. The statistician schedules time with 3 people, and each person
  participates when the statistician runs the following R code:

  \code{x <- elicit(n=10, cats=c(-0.5, 0.5, 1.5, 2.5, 3.5),
    cat.names=c("-1:<0", "0:<1", "1:<2", "2:<3", "3:4"), show.plot=TRUE)}

  Each of the 3 participants receives 10 chips to allocate among the
  5 categories according to personal beliefs in the probability of the
  regression effect. When the statistician and each participant accept
  their elicited distribution, all 3 vectors are combined into one
  vector. In the model form, the prior is expressed as

  \deqn{p(\beta) \sim \mathcal{EL}}{p(beta) ~ EL}

  and the code for the model specification is

  \code{elicit.prior <- delicit(beta, x, log=TRUE)}

  This method is easily extended to priors that are multivariate,
  correlated, or conditional.

  As an alternative, Hahn (2006) also used a categorical approach,
  eliciting judgements about the relative likelihood of each category,
  and then minimizes the KLD (for more information on KLD, see the
  \code{\link{KLD}} function).
}
\references{
  Hahn, E.D. (2006). "Re-examining Informative Prior Elicitation Through
  the Lens of Markov chain Monte Carlo Methods". \emph{Journal of the
  Royal Statistical Society}, A 169 (1), p. 37--48.
  
  Savage, L.J. (1971). "Elicitation of Personal Probabilities and
  Expectations". \emph{Journal of the American Statistical Association},
  66(336), p. 783--801.
}
\author{Statisticat, LLC. \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{de.Finetti.Game}},
  \code{\link{KLD}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{LaplacesDemon.hpc}},
  \code{\link{PMC}}, and
  \code{\link{VariationalBayes}}.
}
\examples{
library(LaplacesDemon)
x <- c(1,2,2,3,3,3,4,7,8,8,9,10) #Elicited with elicit function
theta <- seq(from=-5,to=15,by=.1)
plot(theta, delicit(theta,x), type="l", xlab=expression(theta),
     ylab=expression("p(" * theta * ")"))
}
\keyword{Elicitation}
\keyword{Personal Probability}
\keyword{Prior}
\keyword{Subjective Probability}
\keyword{Utility}

\name{Raftery.Diagnostic}
\alias{Raftery.Diagnostic}
\title{Raftery and Lewis's diagnostic}
\description{
  Raftery and Lewis (1992) introduced an MCMC diagnostic that estimates
  the number of iterations needed for a given level of precision in
  posterior samples, as well as estimating burn-in, when quantiles are
  the posterior summaries of interest.
}
\usage{
Raftery.Diagnostic(x, q=0.025, r=0.005, s=0.95, eps=0.001)
}
\arguments{
  \item{x}{This required argument accepts an object of class
    \code{demonoid}. It attempts to use \code{Posterior2}, but when this
    is missing it uses \code{Posterior1}.}
  \item{q}{This is the quantile to be estimated.}
  \item{r}{This is the desired margin of error of the estimate, also
    called the accuracy.}
  \item{s}{This is the probability of obtaining an estimate in the
    interval (q-r, q+r).}
  \item{eps}{This is the precision required for the estimate of time to
    convergence.}
}
\details{
  In this MCMC diagnostic, a posterior quantile \eqn{q} of interest is
  specified. Next, an acceptable tolerance \eqn{r} is specified for
  \eqn{q}, which means that it is desired to measure \eqn{q} with an
  accuracy of +/- \eqn{r}. Finally, the user selects a probability
  \eqn{s}, which is the probability of being within the interval
  \eqn{(q-r, q+r)}. The \code{Raftery.Diagnostic} then estimates the
  number \eqn{N} of iterations and the number \eqn{M} of burn-in
  iterations that are necessary to satisfy the specified conditions
  regarding quantile \eqn{q}.

  The diagnostic was designed to test a short, initial update, in which
  the chains were called pilot chains, and the application was later
  suggested for iterative use after any update as a general method for
  pursuing convergence (Raftery and Lewis, 1996).

  Results of the \code{Raftery.Diagnostic} differ depending on the
  chosen quantile \eqn{q}. Estimates are conservative, so more
  iterations are suggested than necessary.
}
\note{
  The \code{Raftery.Diagnostic} function was adapted from the
  \code{raftery.diag} function in the coda package, which was adapted
  from the FORTRAN program `gibbsit', written by Steven Lewis.
}
\value{
  The \code{Raftery.Diagnostic} function returns an object of class
  \code{raftery} that is list. A print method is available for objects
  of this class. The list has the following components:
  \item{tspar}{These are the time-series parameters of the posterior
    samples in \code{x}.}
  \item{params}{This is a vector containing the parameters \code{q},
    \code{r}, and \code{s}.}
  \item{Niters}{This is the number of iterations in the posterior
    samples in \code{x}.}
  \item{resmatrix}{This is a 3-dimensional array containing the
    results: \eqn{M} is the suggested burn-in, \eqn{N} is the suggested
    number of iterations, \eqn{Nmin} is the suggested number of
    iterations based on zero autocorrelation, and
    \eqn{I = (M+N)/Nmin} is the "dependence factor". The dependence
    factor is interpreted as the proportional increase in the number of
    iterations attributable to autocorrelation. Highly autocorrelated
    chains (> 5) are worrisome, and may be due to influential initial
    values, parameter correlations, or poor mixing.}
}
\references{
  Raftery, A.E. and Lewis, S.M. (1992). "How Many Iterations in the
  Gibbs Sampler?" In \emph{Bayesian Statistics}, 4 (J.M. Bernardo, J.O.
  Berger, A.P. Dawid and A.F.M. Smith, eds.). Oxford, U.K.: Oxford
  University Press, p. 763--773.

  Raftery, A.E. and Lewis, S.M. (1992). "One Long Run with
  Diagnostics: Implementation Strategies for Markov chain Monte Carlo".
  \emph{Statistical Science}, 7, p. 493--497.

  Raftery, A.E. and Lewis, S.M. (1996). "Implementing MCMC".
  \emph{In} Practical Markov Chain Monte Carlo (W.R. Gilks,
  D.J. Spiegelhalter and S. Richardson, eds.). Chapman and Hall:
  Baton Rouge, FL.
}
 \seealso{
  \code{\link{burnin}},
  \code{\link{LaplacesDemon}},
  \code{\link{print.raftery}}, and
  \code{\link{Thin}}.
}
\examples{
#library(LaplacesDemon)
###After updating with LaplacesDemon, do:
#rd <- Raftery.Diagnostic(Fit)
#print(rd)
}
\keyword{Diagnostic}
\keyword{Gibbsit}
\keyword{MCMC}

\name{VariationalBayes}
\alias{VariationalBayes}
\title{Variational Bayes}
\description{
  The \code{VariationalBayes} function is a numerical approximation
  method for deterministically estimating the marginal posterior
  distributions, target distributions, in a Bayesian model with
  approximated distributions by minimizing the Kullback-Leibler
  Divergence (\code{\link{KLD}}) between the target and its
  approximation.
}
\usage{
VariationalBayes(Model, parm, Data, Covar=NULL, Interval=1.0E-6,
     Iterations=1000, Method="Salimans2", Samples=1000, sir=TRUE,
     Stop.Tolerance=1.0E-5, CPUs=1, Type="PSOCK")
}
\arguments{
  \item{Model}{This required argument receives the model from a
    user-defined function. The user-defined function is where the model
    is specified. \code{VariationalBayes} passes two arguments to
    the model function, \code{parms} and \code{Data}. For more
    information, see the \code{\link{LaplacesDemon}} function and
    ``LaplacesDemon Tutorial'' vignette.}
  \item{parm}{This argument requires a vector of initial values equal in
    length to the number of parameters. \code{VariationalBayes} will
    attempt to optimize these initial values for the parameters, where
    the optimized values are the posterior means, for later use with the
    \code{\link{IterativeQuadrature}}, \code{\link{LaplacesDemon}}, or
    \code{\link{PMC}} function. The \code{\link{GIV}} function may be
    used to randomly generate initial values. Parameters must be
    continuous.}
  \item{Data}{This required argument accepts a list of data. The list of
    data must include \code{mon.names} which contains monitored variable
    names, and \code{parm.names} which contains parameter
    names. \code{VariationalBayes} must be able to determine the
    sample size of the data, and will look for a scalar sample size
    variable \code{n} or \code{N}. If not found, it will look for
    variable \code{y} or \code{Y}, and attempt to take its number of
    rows as sample size. \code{VariationalBayes} needs to determine
    sample size due to the asymptotic nature of this method. Sample size
    should be at least \eqn{\sqrt{J}}{sqrt(J)} with \eqn{J} exchangeable
    parameters.}
  \item{Covar}{This argument defaults to \code{NULL}, but may otherwise
    accept a \eqn{K \times K}{K x K} covariance matrix (where \eqn{K}
    is the number of dimensions or parameters) of the parameters. When
    the model is updated for the first time and prior variance or
    covariance is unknown, then \code{Covar=NULL} should be used. Once
    \code{VariationalBayes} has finished updating, it may be desired
    to continue updating where it left off, in which case the covariance
    matrix from the last run can be input into the next run.}
  \item{Interval}{This argument receives an interval for estimating
    approximate gradients. The logarithm of the unnormalized joint
    posterior density of the Bayesian model is evaluated at the current
    parameter value, and again at the current parameter value plus this
    interval.}
  \item{Iterations}{This argument accepts an integer that determines the
    number of iterations that \code{VariationalBayes} will attempt
    to maximize the logarithm of the unnormalized joint posterior
    density. \code{Iterations} defaults to 1000.
    \code{VariationalBayes} will stop before this number of
    iterations if the tolerance is less than or equal to the
    \code{Stop.Tolerance} criterion. The required amount of computer
    memory increases with \code{Iterations}. If computer memory is
    exceeded, then all will be lost.}
  \item{Method}{This optional argument currently accepts only
    \code{Salimans2}, which is the second algorithm in Salimans and
    Knowles (2013).}
  \item{Samples}{This argument indicates the number of posterior samples
    to be taken with sampling importance resampling via the
    \code{\link{SIR}} function, which occurs only when
    \code{sir=TRUE}. Note that the number of samples should increase
    with the number and intercorrelations of the parameters.}
  \item{sir}{This logical argument indicates whether or not Sampling
    Importance Resampling (SIR) is conducted via the \code{\link{SIR}}
    function to draw independent posterior samples. This argument
    defaults to \code{TRUE}. Even when \code{TRUE}, posterior samples
    are drawn only when \code{VariationalBayes} has
    converged. Posterior samples are required for many other functions,
    including \code{plot.vb} and \code{predict.vb}. The only
    time that it is advantageous for \code{sir=FALSE} is when
    \code{VariationalBayes} is used to help the initial values for
    \code{\link{IterativeQuadrature}}, \code{\link{LaplacesDemon}}, or
    \code{\link{PMC}}, and it is unnecessary for time to be spent on
    sampling. Less time can be spent on sampling by increasing
    \code{CPUs}, which parallelizes the sampling.}
  \item{Stop.Tolerance}{This argument accepts any positive number and
    defaults to 1.0E-3. Tolerance is calculated each iteration, and the
    criteria varies by algorithm. The algorithm is considered to have
    converged to the user-specified \code{Stop.Tolerance} when the
    tolerance is less than or equal to the value of
    \code{Stop.Tolerance}, and the algorithm terminates at the end of
    the current iteration. Often, multiple criteria are used, in
    which case the maximum of all criteria becomes the tolerance. For
    example, when partial derivatives are taken, it is commonly required
    that the Euclidean norm of the partial derivatives is a criterion,
    and another common criterion is the Euclidean norm of the
    differences between the current and previous parameter
    values. Several algorithms have other, specific tolerances.}
  \item{CPUs}{This argument accepts an integer that specifies the number
    of central processing units (CPUs) of the multicore computer or
    computer cluster. This argument defaults to \code{CPUs=1}, in which
    parallel processing does not occur. Parallelization occurs only for
    sampling with \code{\link{SIR}} when \code{sir=TRUE}.}
  \item{Type}{This argument specifies the type of parallel processing to
    perform, accepting either \code{Type="PSOCK"} or
    \code{Type="MPI"}.}
}
\details{
  Variational Bayes (VB) is a family of numerical approximation
  algorithms that is a subset of variational inference algorithms, or
  variational methods. Some examples of variational methods include the
  mean-field approximation, loopy belief propagation, tree-reweighted
  belief propagation, and expectation propagation (EP).

  Variational inference for probabilistic models was introduced in the
  field of machine learning, influenced by statistical physics
  literature (Saul et al., 1996; Saul and Jordan, 1996; Jaakkola, 1997).
  The mean-field methods in Neal and Hinton (1999) led to variational
  algorithms.
  
  Variational inference algorithms were later generalized for conjugate
  exponential-family models (Attias, 1999, 2000; Wiegerinck, 2000;
  Ghahramani and Beal, 2001; Xing et al., 2003). These algorithms still
  require different designs for different model forms. Salimans and
  Knowles (2013) introduced general-purpose VB algorithms for Gaussian
  posteriors.
  
  A VB algorithm deterministically estimates the marginal posterior
  distributions (target distributions) in a Bayesian model with
  approximated distributions by minimizing the Kullback-Leibler
  Divergence (\code{\link{KLD}}) between the target and its
  approximation. The complicated posterior distribution is approximated
  with a simpler distribution. The simpler, approximated distribution is
  called the variational approximation, or approximation distribution,
  of the posterior. The term variational is derived from the calculus of
  variations, and regards optimization algorithms that select the best
  function (which is a distribution in VB), rather than merely selecting
  the best parameters.

  VB algorithms often use Gaussian distributions as approximating
  distributions. In this case, both the mean and variance of the
  parameters are estimated.
  
  Usually, a VB algorithm is slower to convergence than a Laplace
  Approximation algorithm, and faster to convergence than a Monte Carlo
  algorithm such as Markov chain Monte Carlo (MCMC). VB often provides
  solutions with comparable accuracy to MCMC in less time. Though Monte
  Carlo algorithms provide a numerical approximation to the exact
  posterior using a set of samples, VB provides a locally-optimal,
  exact analytical solution to an approximation of the posterior. VB is
  often more applicable than MCMC to big data or large-dimensional
  models.

  Since VB is deterministic, it is asymptotic and subject to the same
  limitations with respect to sample size as Laplace Approximation.
  However, VB estimates more parameters than Laplace Approximation,
  such as when Laplace Approximation optimizes the posterior mode of a
  Gaussian distribution, while VB optimizes both the Gaussian mean and
  variance.

  Traditionally, VB algorithms required customized equations. The
  \code{VariationalBayes} function uses general-purpose algorithms. A
  general-purpose VB algorithm is less efficient than an algorithm
  custom designed for the model form. However, a general-purpose
  algorithm is applied consistently and easily to numerous model forms.

  When \code{Method="Salimans2"}, the second algorithm of Salimans and
  Knowles (2013) is used. This requires the gradient and Hessian, which
  is more efficient with a small number of parameters as long as the
  posterior is twice differentiable. The step size is constant. This
  algorithm is suitable for marginal posterior distributions that are
  Gaussian and unimodal. A stochastic approximation algorithm is used
  in the context of fixed-form VB, inspired by considering fixed-form VB
  to be equivalent to performing a linear regression with the sufficient
  statistics of the approximation as independent variables and the
  unnormalized logarithm of the joint posterior density as the dependent
  variable. The number of requested iterations should be large, since the
  step-size decreases for larger requested iterations, and a small
  step-size will eventually converge. A large number of requested
  iterations results in a smaller step-size and better convergence
  properties, so hope for early convergence. However convergence is
  checked only in the last half of the iterations after the algorithm
  begins to average the mean and variance from the samples of the
  stochastic approximation. The history of stochastic samples is
  returned.
}
\value{
  \code{VariationalBayes} returns an object of class \code{vb}
  that is a list with the following components:
  \item{Call}{This is the matched call of \code{VariationalBayes}.}
  \item{Converged}{This is a logical indicator of whether or not
    \code{VariationalBayes} converged within the specified
    \code{Iterations} according to the supplied \code{Stop.Tolerance}
    criterion. Convergence does not indicate that the global maximum has
    been found, but only that the tolerance was less than or equal to
    the \code{Stop.Tolerance} criterion.}
  \item{Covar}{This is the estimated covariance matrix. The
    \code{Covar} matrix may be scaled and input into the \code{Covar}
    argument of the \code{\link{LaplacesDemon}} or \code{\link{PMC}}
    function for further estimation, or the diagonal of this matrix may
    be used to represent the posterior variance of the parameters,
    provided the algorithm converged and matrix inversion was
    successful. To scale this matrix for use with Laplace's Demon or
    PMC, multiply it by \eqn{2.38^2/d}, where \eqn{d} is the number
    of initial values.}
  \item{Deviance}{This is a vector of the iterative history of the
    deviance in the \code{VariationalBayes} function, as it sought
    convergence.}
  \item{History}{This is an array of the iterative history of the
    parameters in the \code{VariationalBayes} function, as it sought
    convergence. The first matrix is for means and the second matrix is
    for variances.}
  \item{Initial.Values}{This is the vector of initial values that was
    originally given to \code{VariationalBayes} in the \code{parm}
    argument.}
  \item{LML}{This is an approximation of the logarithm of the marginal
    likelihood of the data (see the \code{\link{LML}} function for more
    information). When the model has converged and \code{sir=TRUE}, the
    NSIS method is used. When the model has converged and
    \code{sir=FALSE}, the LME method is used. This is the
    logarithmic form of equation 4 in Lewis and Raftery (1997). As a
    rough estimate of Kass and Raftery (1995), the LME-based LML is
    worrisome when the sample size of the data is less than five times
    the number of parameters, and \code{LML} should be adequate in most
    problems when the sample size of the data exceeds twenty times the
    number of parameters (p. 778). The LME is inappropriate with
    hierarchical models. However \code{LML} is estimated, it is useful
    for comparing multiple models with the \code{BayesFactor} function.}
  \item{LP.Final}{This reports the final scalar value for the logarithm
    of the unnormalized joint posterior density.}
  \item{LP.Initial}{This reports the initial scalar value for the
    logarithm of the unnormalized joint posterior density.}
  \item{Minutes}{This is the number of minutes that
    \code{VariationalBayes} was running, and this includes the
    initial checks as well as drawing posterior samples and creating
    summaries.}
  \item{Monitor}{When \code{sir=TRUE}, a number of independent
    posterior samples equal to \code{Samples} is taken, and the draws
    are stored here as a matrix. The rows of the matrix are the samples,
    and the columns are the monitored variables.}
  \item{Posterior}{When \code{sir=TRUE}, a number of independent
    posterior samples equal to \code{Samples} is taken, and the draws
    are stored here as a matrix. The rows of the matrix are the samples,
    and the columns are the parameters.}
  \item{Step.Size.Final}{This is the final, scalar \code{Step.Size}
    value at the end of the \code{VariationalBayes} algorithm.}
  \item{Step.Size.Initial}{This is the initial, scalar \code{Step.Size}.}
  \item{Summary1}{This is a summary matrix that summarizes the
    point-estimated posterior means and variances. Uncertainty around
    the posterior means is estimated from the estimated covariance
    matrix. Rows are parameters. The following columns are included:
    Mean, SD (Standard Deviation), LB (Lower Bound), and UB (Upper
    Bound). The bounds constitute a 95\% probability interval.}
  \item{Summary2}{This is a summary matrix that summarizes the
    posterior samples drawn with sampling importance resampling
    (\code{\link{SIR}}) when \code{sir=TRUE}, given the point-estimated
    posterior means and covariance matrix. Rows are parameters. The
    following columns are included: Mean, SD (Standard Deviation),
    LB (Lower Bound), and UB (Upper Bound). The bounds constitute a 95\%
    probability interval.}
  \item{Tolerance.Final}{This is the last \code{Tolerance} of the
    \code{VariationalBayes} algorithm.}
  \item{Tolerance.Stop}{This is the \code{Stop.Tolerance} criterion.}
}
\references{
  Attias, H. (1999). "Inferring Parameters and Structure of Latent
  Variable Models by Variational Bayes". In \emph{Uncertainty in
  Artificial Intelligence}.
  
  Attias, H. (2000). "A Variational Bayesian Framework for Graphical
  Models". In \emph{Neural Information Processing Systems}.

  Ghahramani, Z. and Beal, M. (2001). "Propagation Algorithms for
  Variational Bayesian Learning". In \emph{Neural Information Processing
  Systems}, p. 507--513.

  Jaakkola, T. (1997). "Variational Methods for Inference and Estimation
  in Graphical Models". PhD thesis, Massachusetts Institute of
  Technology.

  Salimans, T. and Knowles, D.A. (2013). "Fixed-Form Variational
  Posterior Approximation through Stochastic Linear Regression".
  \emph{Bayesian Analysis}, 8(4), p. 837--882.

  Neal, R. and Hinton, G. (1999). "A View of the EM Algorithm that
  Justifies Incremental, Sparse, and Other Variants". In Learning in
  Graphical Models, p. 355--368. MIT Press, 1999.

  Saul, L. and Jordan, M. (1996). "Exploiting Tractable Substructures in
  Intractable Networks". \emph{Neural Information Processing Systems}.

  Saul, L., Jaakkola, T., and Jordan, M. (1996). "Mean Field Theory for
  Sigmoid Belief Networks". \emph{Journal of Artificial Intelligence
  Research}, 4, p. 61--76.

  Wiegerinck, W. (2000). "Variational Approximations Between Mean Field
  Theory and the Junction Tree Algorithm". In \emph{Uncertainty in
  Artificial Intelligence}.

  Xing, E., Jordan, M., and Russell, S. (2003). "A Generalized Mean
  Field Algorithm for Variational Inference in Exponential Families". In
  \emph{Uncertainty in Artificial Intelligence}.
}
\author{Statisticat, LLC \email{software@bayesian-inference.com}}
\seealso{
  \code{\link{BayesFactor}},
  \code{\link{IterativeQuadrature}},
  \code{\link{LaplaceApproximation}},
  \code{\link{LaplacesDemon}},
  \code{\link{GIV}},
  \code{\link{LML}},
  \code{\link{PMC}}, and
  \code{\link{SIR}}.
}
\examples{
# The accompanying Examples vignette is a compendium of examples.
####################  Load the LaplacesDemon Library  #####################
library(LaplacesDemon)

##############################  Demon Data  ###############################
data(demonsnacks)
y <- log(demonsnacks$Calories)
X <- cbind(1, as.matrix(log(demonsnacks[,10]+1)))
J <- ncol(X)
for (j in 2:J) X[,j] <- CenterScale(X[,j])

#########################  Data List Preparation  #########################
mon.names <- "mu[1]"
parm.names <- as.parm.names(list(beta=rep(0,J), sigma=0))
pos.beta <- grep("beta", parm.names)
pos.sigma <- grep("sigma", parm.names)
PGF <- function(Data) {
     beta <- rnorm(Data$J)
     sigma <- runif(1)
     return(c(beta, sigma))
     }
MyData <- list(J=J, PGF=PGF, X=X, mon.names=mon.names,
     parm.names=parm.names, pos.beta=pos.beta, pos.sigma=pos.sigma, y=y)

##########################  Model Specification  ##########################
Model <- function(parm, Data)
     {
     ### Parameters
     beta <- parm[Data$pos.beta]
     sigma <- interval(parm[Data$pos.sigma], 1e-100, Inf)
     parm[Data$pos.sigma] <- sigma
     ### Log-Prior
     beta.prior <- sum(dnormv(beta, 0, 1000, log=TRUE))
     sigma.prior <- dhalfcauchy(sigma, 25, log=TRUE)
     ### Log-Likelihood
     mu <- tcrossprod(Data$X, t(beta))
     LL <- sum(dnorm(Data$y, mu, sigma, log=TRUE))
     ### Log-Posterior
     LP <- LL + beta.prior + sigma.prior
     Modelout <- list(LP=LP, Dev=-2*LL, Monitor=mu[1],
          yhat=rnorm(length(mu), mu, sigma), parm=parm)
     return(Modelout)
     }

############################  Initial Values  #############################
#Initial.Values <- GIV(Model, MyData, PGF=TRUE)
Initial.Values <- rep(0,J+1)

#Fit <- VariationalBayes(Model, Initial.Values, Data=MyData, Covar=NULL,
#     Iterations=1000, Method="Salimans2", Stop.Tolerance=1e-3, CPUs=1)
#Fit
#print(Fit)
#PosteriorChecks(Fit)
#caterpillar.plot(Fit, Parms="beta")
#plot(Fit, MyData, PDF=FALSE)
#Pred <- predict(Fit, Model, MyData, CPUs=1)
#summary(Pred, Discrep="Chi-Square")
#plot(Pred, Style="Covariates", Data=MyData)
#plot(Pred, Style="Density", Rows=1:9)
#plot(Pred, Style="Fitted")
#plot(Pred, Style="Jarque-Bera")
#plot(Pred, Style="Predictive Quantiles")
#plot(Pred, Style="Residual Density")
#plot(Pred, Style="Residuals")
#Levene.Test(Pred)
#Importance(Fit, Model, MyData, Discrep="Chi-Square")

#Fit$Covar is scaled (2.38^2/d) and submitted to LaplacesDemon as Covar.
#Fit$Summary[,1] is submitted to LaplacesDemon as Initial.Values.
#End
}
\keyword{Adaptive}
\keyword{Optimization}
